#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

dynInit -fast

if {[flagLoc $ARGV -help]} \
   {
    puts "DC: Dipolar Coupling Calculations."
    puts " -inD   dcInName  \[dObsA.tab\]   DC Input Table."
    puts " -pdb   pdbList   \[ref.pdb\]     List of PDB Inputs."
    puts " -outD  dcOutName \[dCalcA.tab\]  DC Output Table."
    puts " -fixDD                         Add or Adjust DD and W Values as Needed."
    puts "Arguments for TCL Script Use:"
    dynSimulate -dc 0 -help
    exit 1 
   }

set pdbList     ref.pdb
set dcInName    dObsA.tab
set dcOutName   dCalcA.tab
set vArg        0
set ignoreChain 1
set ignoreSeg   1
set fixDDFlag   0

set dcArgs      ""

getListArgD $ARGV -pdb  pdbList
getArgD     $ARGV -inD  dcInName
getArgD     $ARGV -outD dcOutName

set pdbTitle   $pdbList
set dcInTitle  $dcInName
set dcOutTitle $dcInName

getListArgD $ARGV -pdbTitle  pdbTitle
getArgD     $ARGV -inDTitle  dcInTitle
getArgD     $ARGV -outDTitle dcOutTitle

if {[flagLoc $ARGV -fixDD]}    {set fixDDFlag 1}
if {[flagLoc $ARGV -nofixDD]}  {set fixDDFlag 0}

set loc [flagLoc $ARGV -verb]

if {$loc} \
   {
    set nextFlag [nextFlag $ARGV $loc]
    set vArg 1

    if {$nextFlag == $loc + 2} \
       {
        getArgD $ARGV -verb vArg
       }
   }

if {[flagLoc $ARGV -noverb]} {set vArg 0}

if {[flagLoc $ARGV -ignore]} \
   {
    set ignoreChain 1
    set ignoreSeg   1
   }

if {[flagLoc $ARGV -noignore]} \
   {
    set ignoreChain 0
    set ignoreSeg   0
   }

if {[flagLoc $ARGV -ignoreChain]}   {set ignoreChain 1}
if {[flagLoc $ARGV -noignoreChain]} {set ignoreChain 0}

if {[flagLoc $ARGV -ignoreSeg]}     {set ignoreSeg 1}
if {[flagLoc $ARGV -noignoreSeg]}   {set ignoreSeg 0}

#
#-------------------------------------------------------------------------

proc showPDBInfo { src } \
{
   set pInfo [dynGetInfo -src $src]
   set r1        [lindex $pInfo 0]
   set rN        [lindex $pInfo 1]
   set atomCount [lindex $pInfo 2] 
   set resCount  [lindex $pInfo 3] 
   set chCount   [lindex $pInfo 4] 
   set segCount  [lindex $pInfo 5] 

   set str "Atoms: $atomCount Residues: $resCount Chains: $chCount"
}

set thisTitle [lindex $pdbTitle 0]
set nPDB      [llength $pdbList]
set i 0

foreach pdbName $pdbList {lappend pdbTitle $thisTitle}

foreach pdbName $pdbList \
   {
    set thisTitle [lindex $pdbTitle $i]

    if {![file exists $pdbName]} \
       {
        puts stderr "DC Error Finding Input PDB $thisTitle"
        exit 1
       }

    incr i
   }

set pdbTitle [lrange $pdbTitle 0 [expr $nPDB - 1]]

if {![file exists $dcInName]} \
   {
    puts stderr "DC Error Finding Input DC File $dcInName"
    exit 1
   }

if {$fixDDFlag} \
   {
    if {[file exists $fixName]} {catch "file delete -force $fixName"}

    catch {exec "scaleDI.tcl -pdb $thisPDBName -in $dcInNameOrig -out $fixName -calcDI -calcDD -calcW -keepDI -keepDD -keepW -noverb"}

    if {![file exists $fixName]} \
       {
        puts stderr "DC Error Adjusting DD and W Values in $thisPDBTitle"
       }
   }

if {$ignoreChain} {dynSetChainMode -ignore}
if {$ignoreSeg}   {dynSetSegMode   -ignore}

for {set i 3} {$i < $ARGC} {incr i} \
   {
    set thisArg [lindex $ARGV $i]

    if {![strcasecmp $thisArg -noverb]}        continue
    if {![strcasecmp $thisArg -verb]}          continue

    if {![strcasecmp $thisArg -ignore]}        continue
    if {![strcasecmp $thisArg -ignoreChain]}   continue
    if {![strcasecmp $thisArg -ignoreSeg]}     continue

    if {![strcasecmp $thisArg -noignore]}      continue
    if {![strcasecmp $thisArg -noignoreChain]} continue
    if {![strcasecmp $thisArg -noignoreSeg]}   continue
 
    if {![strcasecmp $thisArg -inD] || ![strcasecmp $thisArg -inDTitle]} \
       {
        incr i
        continue
       }

    if {![strcasecmp $thisArg -outD] || ![strcasecmp $thisArg -outDTitle]} \
       {
        incr i
        continue
       }

    if {![strcasecmp $thisArg -pdb] || ![strcasecmp $thisArg -pdbTitle]} \
       {
        set i [nextFlag $ARGV $i]
        if {$i >= $ARGC} break

        incr i -1
        continue
       }

    set dcArgs [concat $dcArgs $thisArg]
   }

if {![llength $dcArgs]} {set dcArgs "-svd"}

set pdbSrc ""

foreach pdbName $pdbList \
   { 
    set thisSrc [dynRead -pdb -in $pdbName]
    lappend pdbSrc $thisSrc
   }

set thisSrc   [lindex $pdbSrc 0]
set thisTitle [lindex $pdbTitle 0]
set dcSrc     [dynRead -dc  -in $dcInName]
set nD        [dynGetInfo -dc $dcSrc -brief]

puts "DC Input: $dcInTitle N: $nD Options: $dcArgs -verb $vArg"
puts "PDB $thisTitle Models: $nPDB [showPDBInfo $thisSrc]"

set dInfo [eval "dynSimulate -src $pdbSrc -dc $dcSrc $dcArgs -verb"]

if {[llength $dInfo] < 5} \
   {
    puts stderr "DC Error Calculating Dipolar Couplings."
    exit 1
   }

eval {dynWrite -dc $dcSrc -out $dcOutName -verb $vArg}

set dRMS   [lindex $dInfo 0]
set Da     [lindex $dInfo 3]
set Dr     [lindex $dInfo 4]
set dcN    [lindex $dInfo end]

if {$Da == 0.0} \
   {
    set rh 0.0
   } \
else \
   {
    set rh [expr $Dr/$Da]
   }

puts [format "DC RMS: %6.3f N: %4d Da: % .3e Dr: % .3e Rh: %.3f" $dRMS $dcN $Da $Dr $rh]

exit 0 
