#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]


if {[flagLoc $ARGV -help]} \
   {
    puts "Combine Two NMRPipe Stream Files:"
    puts " -in1      inName1  \[A.dat\]  Input Spectrum 1."
    puts " -in2      inName2  \[B.dat\]  Input Spectrum 2." 
    puts " -out      outName  \[C.dat\]  Output Spectrum." 
    puts " -nThread  tCount   \[1\]      Number of Threads."
    puts "Actions:"
    puts " -add     Add." 
    puts " -sub     Subtract." 
    puts " -mult    Multiply." 
    puts " -div     Divide."
    puts " -cmult   Complex Multiply." 
    puts " -hmult   Hypercomplex Multiply." 
    puts " -mask    Mask." 
    puts " -imask   Inverse Mask." 
    puts "Notes:"
    puts " 1. Option -div sets any divide by zero to zero."
    puts " 2. Option -cmult requires complex or hypercomplex data."
    puts " 3. Option -hmult requires hypercomplex data."
    puts " 4. Option -mask returns the values from -in1 with the result"
    puts "    set to zero whereever -in2 is zero."
    puts " 5. Option -imask returns the values from -in1 with the result"
    puts "    set to zero whereever -in2 is not zero."
    exit 0
   }

set inName1 A.dat 
set inName2 B.dat 
set outName C.dat 
set fnName  vvAdd

set threadCount 1
set sizeAdj     1

getArgD $ARGV -in1 inName1
getArgD $ARGV -in2 inName2
getArgD $ARGV -out outName

getArgD $ARGV -nThread threadCount

if {[flagLoc $ARGV -mult]}   {set fnName vvMultMT}
if {[flagLoc $ARGV -mul]}    {set fnName vvMultMT}

if {[flagLoc $ARGV -add]}    {set fnName vvAddMT}
if {[flagLoc $ARGV -sum]}    {set fnName vvAddMT}
if {[flagLoc $ARGV -sub]}    {set fnName vvSubMT}
if {[flagLoc $ARGV -dif]}    {set fnName vvSubMT}
if {[flagLoc $ARGV -div]}    {set fnName vvDivMT}
if {[flagLoc $ARGV -mask]}   {set fnName vvMaskMT}
if {[flagLoc $ARGV -imask]}  {set fnName vvIMaskMT}

if {[flagLoc $ARGV -cmult]} \
   {
    set fnName  vvCMult
    set sizeAdj 2
   }

if {[flagLoc $ARGV -hmult]} \
   {
    set fnName  vvHMult
    set sizeAdj 4
   }

if {![file exists $inName1] || ![file exists $inName2]} \
   {
    puts "addPipe.tcl Error Finding Input File $inName1/$inName2"
    exit 1
   }

set error1 [catch {set inUnit1 [sysOpen $inName1 r]}]
set error2 [catch {set inUnit2 [sysOpen $inName2 r]}]

if {$error1 || $error2} \
   {
    puts "addPipe.tcl Error Opening File $inName1/$inName2"
    exit 1
   }

set error [catch {set outUnit [sysOpen $outName w]}]

if {$error} \
   {
    puts "addPipe.tcl Error Opening Output File $outName"
    exit 1
   }

set error1 [catch {set fdata1 [sysMalloc $FDATASIZE float]}]
set error2 [catch {set fdata2 [sysMalloc $FDATASIZE float]}]

if {$error1 || $error2} \
   {
    puts "addPipe.tcl Error Allocating Header."
    exit 1
   }

set error1 [catch {sysRead $inUnit1 $fdata1 $FDATASIZE}]
set error2 [catch {sysRead $inUnit2 $fdata2 $FDATASIZE}]

if {$error1 || $error2} \
   {
    puts "addPipe.tcl Error Reading Header."
    exit 1
   }

set dimCount [getParmI $fdata1 FDDIMCOUNT NULL_DIM]
set pipeFlag [getParmI $fdata1 FDPIPEFLAG NULL_DIM]
set cubeFlag [getParmI $fdata1 FDCUBEFLAG NULL_DIM]
set fmtCount [getFmtCount $inName1]

set qSize    [getQuadSize [getParmI $fdata1 FDQUADFLAG NULL_DIM]]

set xSize [getParmI $fdata1 NDSIZE CUR_XDIM]
set ySize [getParmI $fdata1 NDSIZE CUR_YDIM]
set zSize [getParmI $fdata1 NDSIZE CUR_ZDIM]
set aSize [getParmI $fdata1 NDSIZE CUR_ADIM]

if {$fmtCount == 0} \
   {
    if {!$pipeFlag && !$cubeFlag} \
       {
        if {$dimCount > 2} {set dimCount 2}
       }
   } 

if {$dimCount < 4} {set aSize 1}
if {$dimCount < 3} {set zSize 1}
if {$dimCount < 2} {set ySize 1}

set planeSize  [expr $xSize*$ySize*$qSize]
set planeCount [expr $zSize*$aSize]

set error1 [catch {set buff1 [sysMalloc $planeSize float]}]
set error2 [catch {set buff2 [sysMalloc $planeSize float]}]

if {$error1 || $error2} \
   {
    puts "addPipe.tcl Error Allocating Memory."
    exit 1
   }

set error [catch {sysWrite $outUnit $fdata1 $FDATASIZE}]

if {$error} \
   {
    puts "addPipe.tcl Error Writing Header to $outName."
    exit 1
   }

for {set i 1} {$i <= $planeCount} {incr i} \
   {
    set error1 [catch {sysRead $inUnit1 $buff1 $planeSize}]
    set error2 [catch {sysRead $inUnit2 $buff2 $planeSize}]

    if {$error1 || $error2} \
       {
        puts "addPipe.tcl Error Reading Plane $i"
        exit 1
       }

    if {$sizeAdj == 1} \
       {
        $fnName $buff1 $buff2 $planeSize $threadCount 
       } \
    else \
       {
        set thisSize  [expr $xSize*$sizeAdj]
        set thisCount [expr 2*$ySize/$sizeAdj]

        set off 0

        for {set iy 0} {$iy < $thisCount} {incr iy} \
           {
            $fnName [incrPtr $buff1 $off] [incrPtr $buff2 $off] $thisSize $threadCount 
            incr off $thisSize
           }
       }

    set error [catch {sysWrite $outUnit $buff1 $planeSize}]

    if {$error} \
       {
        puts "addPipe.tcl Error Writing Plane $i"
        exit 1
       }

   }

catch "sysClose $inUnit1"
catch "sysClose $inUnit2"
catch "sysClose $outUnit"

exit 0
