#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2020.052.18.49"

if {[flagLoc $ARGV -help]} \
   {
    puts "Find Optimal Alignment Between Two 1D Spectra (Version $thisVersion):"
    puts " -in     inName   \[test.ft1\]  Input Spectrum to Shift." 
    puts " -ref    refName  \[ref.ft1\]   Input Spectrum to Match." 
    puts " -mask   maskName \[None\]      Optional Mask Spectrum Input."
    puts " -out    outName  \[align.ft1\] Shifted Spectrum Output, or Keyword None." 
    puts " -masked adjName  \[None\]      Shifted, Masked Output Spectrum, or Keyword None." 
    puts " -dxMin  dxMin    \[-0.10ppm\]  Minimum Allowed Shift (Hz pts ppm %)."
    puts " -dxMax  dxMax    \[0.10ppm\]   Maximum Allowed Shift (Hz pts ppm %)."
    puts " -dxStep dxStep   \[0.01ppm\]   Initial Step Size (Hz pts ppm %)."
    puts " -iter   iter     \[4\]         Maximum Number of Iterations."
    puts " -frac   f        \[0.5\]       Factor for Reducing Step Size at Each Iteration."
    puts " -hdr                         Update Header of Output (Default)."
    puts " -nohdr                       Do Not Update Header of Output."
    exit 0
   }

set refName    ref.ft1
set inName     test.ft1
set outName    align.ft1
set maskName   None
set maskedName None

set dxMin    -0.1ppm
set dxMax     0.1ppm
set dxStep    0.005ppm
set frac      0.5
set iterMax   4

set verbose   1
set briefFlag 0
set hdrFlag   1

set regList  "0% 100%"

getArgD $ARGV -in     inName
getArgD $ARGV -out    outName
getArgD $ARGV -ref    refName
getArgD $ARGV -mask   maskName
getArgD $ARGV -masked maskedName

getArgD $ARGV -dxMin  dxMin
getArgD $ARGV -dxMax  dxMax
getArgD $ARGV -dxStep dxStep

getArgD $ARGV -iter   iterMax
getArgD $ARGV -frac   frac

getListArgD $ARGV -reg regList

if {[flagLoc $ARGV -hdr]}     {set hdr 1}
if {[flagLoc $ARGV -nohdr]}   {set hdr 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {[flagLoc $ARGV -brief]}   {set briefFlag 1}
if {[flagLoc $ARGV -nobrief]} {set briefFlag 0}

#
#----------------------------------------------------------------------------------------

proc allocWork { n } \
{
   set error [catch {set ptr [sysMalloc $n float]}]
   
   if {$error || ![info exists ptr]} \
      {
       puts stderr "align1D. Error Allocating Memory."
       exit 1
      }

   return $ptr
}

set id(ref)       1
set id(spec)      2
set id(mask)      3
set id(refMasked) 4
set id(work)      5

if {![file exists $refName]} \
   {
    puts stderr "align1D. Error Finding Reference Spectrum $refName"
    exit 1
   }

if {![file exists $inName]} \
   {
    puts stderr "align1D. Error Finding Input Spectrum $inName"
    exit 1
   }

readROI -in $refName -ndim 1 -roi $id(ref)  -x X_AXIS 0% 100%
readROI -in $inName  -ndim 1 -roi $id(spec) -x X_AXIS 0% 100%
readROI -in $inName  -ndim 1 -roi $id(work) -x X_AXIS 0% 100%

if {![strcasecmp $maskName None]} \
   {
    readROI -in $inName   -ndim 1 -roi $id(mask) -x X_AXIS 0% 100%
   } \
else \
   {
    if {![file exists $maskName]} \
       {
       puts stderr "align1D. Error Finding Mask Spectrum $maskName"
       exit 1
      }

    readROI -in $maskName -ndim 1 -roi $id(mask) -x X_AXIS 0% 100%
   }

foreach type {ref spec mask work} \
   {
    set roiInfo($type) [getROIInfo -roi $id($type)]
    set rdata($type)   [lindex $roiInfo($type) 0]
    set fdata($type)   [lindex $roiInfo($type) 1]
    set size($type)    [lindex [lindex $roiInfo($type) 2] 0]

    set xSW($type)     [getParm $fdata($type) NDSW   CUR_XDIM]
    set xOBS($type)    [getParm $fdata($type) NDOBS  CUR_XDIM]
    set xCAR($type)    [getParm $fdata($type) NDCAR  CUR_XDIM]
    set xORIG($type)   [getParm $fdata($type) NDORIG CUR_XDIM]

    set xSWPPM($type)  [expr $xSW($type)/$xOBS($type)]
   }

if {$size(ref) != $size(spec) || $size(ref) != $size(mask)} \
   {
    puts stderr "align1D. Error: Size Mastmatch ref $size(ref) spec $size(spec) mask $size(mask)"
    exit 1
   }

if {![strcasecmp $maskName None]} {vsSet $rdata(mask) 1.0 $size(mask)}

set n $size(spec)

foreach type {fid ht shift refMasked} \
   {
    set rdata($type) [allocWork $n]
    set idata($type) [allocWork $n]
   }

set xAdjPPM 0.0
set xAdjPts 0.0

vvMove $rdata(ht) $rdata(spec) $n
vsSet  $idata(ht) 0.0 $n

vHT $rdata(ht) $idata(ht) $n

vvMove $rdata(fid) $rdata(ht) $n
vvMove $idata(fid) $idata(ht) $n

vFT $rdata(fid) $idata(fid) $n -inv

set dx1  [spec2pntW $fdata(spec) X_AXIS $dxMin]
set dxn  [spec2pntW $fdata(spec) X_AXIS $dxMax]
set step [spec2pntW $fdata(spec) X_AXIS $dxStep]

if {int(3 + abs($dxn) + abs($dx1)) > $n} \
   {
    puts stderr "align1D. Error: Spectrum Size ($n) to Small for Shift ($dx1 $dxn)."
    exit 1
   }

set nLeft  [expr 1 + int(abs($dx1) + 0.5)]
set nRight [expr 1 + int(abs($dxn) + 0.5)]

vsSet $rdata(mask) 0.0 $nLeft
vsSet [incrPtr $rdata(mask) [expr $size(spec) - $nRight]] 0.0 $nRight

vvMult $rdata(refMasked) $rdata(ref) $rdata(mask) $n

set bestDX    $dx1
set bestScore 1.0e+20

set k 0

for {set iter 1} {$iter <= $iterMax} {incr iter} \
   {
    set thisUnit [getUnitsLabel $dxMin]
    set dxMin    [format "% 7.4f%s" [pnt2specW $fdata(spec) X_AXIS $dx1 $thisUnit] $thisUnit]

    set thisUnit [getUnitsLabel $dxMax]
    set dxMax    [format "% 7.4f%s" [pnt2specW $fdata(spec) X_AXIS $dxn $thisUnit] $thisUnit]

    set thisUnit [getUnitsLabel $dxStep]
    set dxStep   [format "% 7.4f%s" [pnt2specW $fdata(spec) X_AXIS $step $thisUnit] $thisUnit]

    if {$verbose} \
       {
        set fmt "align1D. Iteration: %d dxMin: %s %9.3f pts dxMax: %s %9.3f pts dxStep: %s %8.3f pts"

        puts stderr [format $fmt $iter $dxMin $dx1 $dxMax $dxn $dxStep $step]
       }

    for {set dx $dx1} {$dx <= $dxn} {set dx [expr $dx + $step]} \
       {
        vvMove $rdata(shift) $rdata(fid) $n
        vvMove $idata(shift) $idata(fid) $n

        vPS $rdata(shift) $idata(shift) $n 0.0 [expr -360.0*$dx]
        vFT $rdata(shift) $idata(shift) $n

        vvMult $rdata(shift) $rdata(mask) $n

        set s  [vvGetScale $rdata(ref) $rdata(shift) $n]

        vsMult $rdata(shift) [expr -1.0*$s] $n 
        vvAdd  $rdata(shift) $rdata(ref) $n

        set thisScore [vRMS $rdata(shift) $n]

        if {$k == 0 || $thisScore < $bestScore} \
           {
            set bestScore $thisScore
            set bestDX    $dx
           }

        incr k

        if {$verbose} \
           {
            set thisUnit [getUnitsLabel $dxMin]
            set dxS      [pnt2specW $fdata(spec) X_AXIS $dx     $thisUnit]
            set bestDXS  [pnt2specW $fdata(spec) X_AXIS $bestDX $thisUnit]

            set fmt " %5d. DX: %7.4f%s %9.3f pts Score: %.4e Best: %.4e %7.4f%s %9.3f pts"

            puts [format $fmt $k $dxS $thisUnit $dx $thisScore $bestScore $bestDXS $thisUnit $bestDX]
           }
       }

    set range [expr $frac*($dxn - $dx1)]
    set step  [expr $frac*$step]

    set dx1   [expr $bestDX - 0.5*$range]
    set dxn   [expr $bestDX + 0.5*$range]
   }

if {[strcasecmp $outName None] || [strcasecmp $maskedName None]} \
   {
    vvMove $rdata(shift) $rdata(fid) $n
    vvMove $idata(shift) $idata(fid) $n

    vPS $rdata(shift) $idata(shift) $n 0.0 [expr -360.0*$bestDX]
    vFT $rdata(shift) $idata(shift) $n

    vvMove $rdata(work) $rdata(shift) $n

    if {$hdrFlag} \
       {

        setParm $fdata(work) NDCAR  $xCAR(ref)  CUR_XDIM 
        setParm $fdata(work) NDORIG $xORIG(ref) CUR_XDIM 
       }

    if {[strcasecmp $outName None]} \
       {
        writeROI -roi $id(work) -out $outName -noverb -ov
       }

    if {[strcasecmp $maskedName None]} \
       {
        vvMult $rdata(work) $rdata(mask) $n
        writeROI -roi $id(work) -out $maskedName -noverb -ov
       }
   }

set dxCarPPM [expr $xCAR(ref) - $xCAR(spec)]
set dxCarPts [expr $size(spec)*$dxCarPPM/$xSWPPM(spec)]

set xAdjPPM [format %.4f [pnt2specW $fdata(spec) X_AXIS $bestDX $thisUnit]]
set xAdjPts [format %.3f $bestDX]

puts "xAdjPPM $xAdjPPM xAdjPts $xAdjPts dxCarPPM $dxCarPPM dxCarPts $dxCarPts size $size(ref)"

exit 0
