#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

#
# Automatic Creation of nlinLS Fitting Script.

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set thisVersion  2023.235.13.36

#
# Initialize variables.

proc initFitVars {} \
{
   global specName inTabName auxTabName outTabName simName difName comName localName localFitName fitArgs multipletType
   global noiseRMS thresh dimCount zLoc aLoc simAllFlag simAllDir simAllName simAllScript timeFlag tsFlag templateFlag
   global execFlag xtermFlag covarFlag sumFlag localFlag localArgs sleepTime 
   global dirName seriesDim fitFlag seriesFlag specCount
   global simSize specSize dRegSize aRegSize maxMove minLW maxLW model
   global tol maxf iter adVarName verbArg

   set specName     test.ft2
   set inTabName    test.tab
   set auxTabName   axt.tab
   set outTabName   nlin.tab
   set simName      None 
   set difName      None
   set localName    local.tab
   set localFitName nlin_local.tab
   set comName      ./autoFit.com
   set adVarName    VOL
   set dirName      .
  
   set fitArgs     None 
   set localArgs   None 
   set verbArg     verb 
   set dimCount    0
   set sleepTime   0
   set seriesFlag  0
   set specCount   0
   set execFlag    1
   set xtermFlag   1
   set covarFlag   0
   set localFlag   0
   set seriesDim   None 

   set multipletType PEAK

   set maxf          750
   set iter          750
   set tol           1.0e-8

   foreach dim {X Y Z A} \
      {
       set specSize($dim)    1
       set aRegSize($dim)    Auto
       set dRegSize($dim)    Auto
       set simSize($dim)     Auto
       set maxMove($dim)     Auto
       set minLW($dim)       0.0 
       set maxLW($dim)       Auto
       set model($dim)       GAUSS1D
      }

   set noiseRMS    Auto
   set thresh      FromFile

   set simAllFlag   0
   set simAllDir    simAll
   set simAllName   Auto
   set simAllScript simAll.com

   set fitFlag     1
   set timeFlag    0
   set sumFlag     0
   set tsFlag      0
   set zLoc        0
   set aLoc        0
}

#
# Extract command line argumemts.

proc initFitArgs {} \
{
   global specName inTabName auxTabName outTabName 
   global simName difName comName errTabName adVarName localName localFitName fitArgs multipletType
   global tol maxf iter dirName seriesDim seriesFlag fitFlag execFlag xtermFlag covarFlag localFlag localArgs
   global noiseRMS thresh dimCount timeFlag tsFlag sumFlag simAllFlag simAllDir simAllName simAllScript
   global templateFlag simSize specSize aRegSize dRegSize maxMove minLW maxLW model sleepTime
   global argv0 ARGC ARGV verbArg thisVersion

   if {[flagLoc $ARGV -help]} \
      {
       puts "$argv0: Automatic Creation of nlinLS Fitting Script ($thisVersion):"
       puts { -ndim      [0]         Dimension Count, Zero for Auto.}
       puts { -specName  [test.ft2]  Input Spectrum Name or Template.}
       puts { -simName   [sim.ft2]   Output Simulation Spectrum.}
       puts { -difName   [dif.ft2]   Output Difference Spectrum.}
       puts { -inTab     [test.tab]  Corresponding Input Peak Table.}
       puts { -auxTab    [axt.tab]   Temporary Peak Table Name, If Needed.}
       puts { -outTab    [nlin.tab]  Output Peak Table.}
       puts { -noise     [Auto]      RMS of Spectral Noise.}
       puts { -thresh    [FromFile]  Detection Threshold (or FromFile or Auto).}
       puts { -maxf      [750]       Maximum Function Evaluations Per Fit.}
       puts { -iter      [750]       Maximum Iterations Per Fit.}
       puts { -tol       [1.0e-8]    Relative Fit Tolerance.}
       puts { -sum                   Compute Integrals by Summation.}
       puts { -nosum                 Compute Integrals from Model Function (Default).}
       puts { -series                Last Dimension is Pseudo-ND.}
       puts { -sleep     [tSec]      Optional Sleep Time.}
       puts { -dir       [.]         Working Directory.}
       puts { -fit                   Perform Fitting (Default).}
       puts { -nofit                 No Fitting.}
       puts { -covar                 Report Covariance Matrix After Fit.}
       puts { -exec                  Execute Fit Script (Default). }
       puts { -noexec                Create Fit Script, Do Not Execute. }
       puts { -xterm                 Display Output in a New Terminal Window (Default). }
       puts { -noxterm               Do Not Create a Terminal Window. }
       puts "Arguments for nlinLS (Should be Last on Command Line):"
       puts { -fitArgs      [None]        Arguments for nlinLS (See nlinLS -help).}
       puts "Arguments for Simulating All Peaks as Individual Spectra:"
       puts { -simAll                     Simulate Individual Peaks (Or for 1D: -sim1D).}
       puts { -simAllDir    [simAll]      Output Directory for Individual Simulations.}
       puts { -simAllName   [Auto]        Output Template Name for Simulations.}
       puts { -simAllScript [simAll.com]  Script Name for Generating Simulations.}
       puts "Local Fitting:"
       puts { -local                      Use Local Fitting Mode.}
       puts { -localArgs    [None]        Comma-Separated Options for localFit.tcl.}
       puts "Time-Domain-Based Models:"
       puts { -time                       Use Time-Domain Model.}
       puts { -ts                         Scale by Integral of Decay.} 
       puts { -nots                       No Time-Domain Scaling (Default).} 
       puts "Lineshape Model Constraints and Types (use X Y Z A):"
       puts { -minXW        [0.0]         Min Full X-Width at Half Height, Pts.}
       puts { -maxXW        [Auto]        Max Full X-Width at Half Height, Pts.}
       puts { -regX         [Auto]        Peak Region X-Width, +/- Pts.}
       puts { -simX         [Auto]        Simulation Region X-Width, +/- Pts.}
       puts { -dX           [Auto]        Max Allowable X-Position Change, Pts.}
       puts { -modX         [GAUSS1D]     Model Function for X-Axis.}
       puts { -mult         [PEAK]        Multiplet Type (PEAK JX JY JXY AX).}
       puts "Intermediate Series Results (seriesTab Arguments):"
       puts { -adVar        [VOL]         Variable for Analysis Output.}
       puts { -adx          [Auto]        Region Analysis X-Width, +/- Pts.}
       puts { -ady          [Auto]        Region Analysis Y-Width, +/- Pts.}
       puts { -adz          [Auto]        Region Analysis Z-Width, +/- Pts.}
       exit 0
      }

   set loc [flagLoc $ARGV -fitArgs]

   if {$loc > 0} \
      {
       set fitArgs [lrange $ARGV [expr $loc + 1] end]
       set ARGV    [lrange $ARGV 0 [expr $loc - 1]]
      }

   if {[flagLoc $ARGV -verb]}    {set verbArg    verb}
   if {[flagLoc $ARGV -noverb]}  {set verbArg    noverb}

   if {[flagLoc $ARGV -local]}   {set localFlag 1}
   if {[flagLoc $ARGV -nolocal]} {set localFlag 0}

   if {[flagLoc $ARGV -ts]}      {set tsFlag 1}
   if {[flagLoc $ARGV -nots]}    {set tsFlag 0}

   if {[flagLoc $ARGV -time]}     {set timeFlag   1}

   if {[flagLoc $ARGV -sim1D]}    {set simAllFlag 1}
   if {[flagLoc $ARGV -sim2D]}    {set simAllFlag 1}

   if {[flagLoc $ARGV -simAll]}   {set simAllFlag 1}
   if {[flagLoc $ARGV -nosimAll]} {set simAllFlag 0}

   if {[flagLoc $ARGV -series]}  {set seriesFlag 1}
  
   if {[flagLoc $ARGV -fit]}     {set fitFlag    1}
   if {[flagLoc $ARGV -nofit]}   {set fitFlag    0}

   if {[flagLoc $ARGV -sum]}    {set sumFlag    1}
   if {[flagLoc $ARGV -nosum]}  {set sumFlag    0}

   if {[flagLoc $ARGV -exec]}   {set execFlag   1}
   if {[flagLoc $ARGV -noexec]} {set execFlag   0}

   if {[flagLoc $ARGV -xterm]}   {set xtermFlag 1}
   if {[flagLoc $ARGV -noxterm]} {set xtermFlag 0}

   if {[flagLoc $ARGV -covar]}   {set covarFlag 1}
   if {[flagLoc $ARGV -nocovar]} {set covarFlag 0}

   getArgD $ARGV -ndim  dimCount
   getArgD $ARGV -dir   dirName
   
   getArgD $ARGV -simAllDir    simAllDir
   getArgD $ARGV -simAllName   simAllName
   getArgD $ARGV -simAllScript simAllScript
   getArgD $ARGV -localArgs    localArgs
 
   if {[strcasecmp $dirName .]} {cd $dirName}

   if {$dimCount == 1} {set specName test.ft1}
 
   getArgD $ARGV -specName specName

   if {[string first "%" $specName] < 0} \
      {
       set templateFlag 0
      } \
   else \
      {
       set templateFlag 1
      }

   getSpecSizes

   if {$dimCount == 1} \
      {
       if {$seriesFlag} \
          {
           set simName   sim/test%03d.ft$dimCount
           set difName   dif/test%03d.ft$dimCount
           set seriesDim Y
          } \
       else \
          {
           set simName sim.ft$dimCount
           set difName dif.ft$dimCount
          }
      } \
   elseif {$dimCount == 2} \
      {
       if {$seriesFlag} \
          {
           set simName   sim/test%03d.ft$dimCount
           set difName   dif/test%03d.ft$dimCount
           set seriesDim Z
          } \
       else \
          {
           set simName sim.ft$dimCount
           set difName dif.ft$dimCount
          }
      } \
   elseif {$dimCount == 3} \
      {
       if {$seriesFlag} \
          {
           set simName sim/test%03d.ft2
           set difName dif/test%03d.ft2
           set seriesDim Z
          } \
       else \
          {
           set simName sim/test%03d.ft3
           set difName dif/test%03d.ft3
          }
      }

   getArgD $ARGV -simName  simName
   getArgD $ARGV -difName  difName
   getArgD $ARGV -adVar    adVarName

   getArgD $ARGV -localName    localName
   getArgD $ARGV -localFitName localFitName

   getArgD $ARGV -inTab    inTabName
   getArgD $ARGV -auxTab   auxTabName
   getArgD $ARGV -outTab   outTabName
   
   getArgD $ARGV -ndim     dimCount
   getArgD $ARGV -zLoc     zLoc
   getArgD $ARGV -aLoc     aLoc 
   getArgD $ARGV -noise    noiseRMS 
   getArgD $ARGV -thresh   thresh
   getArgD $ARGV -sleep    sleepTime
 
   getArgD $ARGV -mult     multipletType
   getArgD $ARGV -maxf     maxf
   getArgD $ARGV -iter     iter
   getArgD $ARGV -tol      tol
 
   foreach dim {X Y Z A} \
      { 
       if {$timeFlag}                 {set model($dim) GTIME1D}
       if {![strcmp $dim $seriesDim]} {set model($dim) SCALE1D}
   
       getArgD $ARGV -d${dim}      maxMove($dim)
       getArgD $ARGV -areg${dim}   aRegSize($dim)
       getArgD $ARGV -reg${dim}    dRegSize($dim)
       getArgD $ARGV -sim${dim}    simSize($dim)
       getArgD $ARGV -mod${dim}    model($dim)
       getArgD $ARGV -min${dim}W   minLW($dim)
       getArgD $ARGV -max${dim}W   maxLW($dim)
      }

   foreach dim {x y z a} \
     { 
      set uDim [string toupper $dim]
      getArgD $ARGV -ad${dim}  aRegSize($uDim)
     }

   set errTabName err.$outTabName

   writeSpecLists
}

#
# Extract spectral sizes and dimension count.

proc getSpecSizes {} \
{
    global templateFlag specName simName specSize specCount
    global zLoc aLoc dimCount seriesFlag

    set thisName [format $specName 1 1]
    set error    [rdFDATA $thisName fdata]

    if {$error} \
      {
       puts stderr "Error reading header in $thisName"
       exit 1
      }

    if {$templateFlag} \
       {
        for {set i 1} {$i <= 65536} {incr i} \
           {
            if {![file exists [format $specName $i]]} break
            set specCount $i 
           }
       } \
    else \
       {
        set specCount 1 
       }

    if {!$dimCount} {set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]}

    foreach dim {X Y Z A} \
       {
        set specSize($dim) [getParmI $fdata NDSIZE CUR_${dim}DIM]
       }

    if {$dimCount == 2 && $specSize(Y) == 1} \
       {
        set dimCount 1
       }

    if {!$templateFlag} \
       {
        if {$dimCount > 2} \
           {
            set dimCount    2
            set specSize(Z) 1
            set specSize(A) 1
           }
       }

    if {$seriesFlag && $dimCount == 1} {set specSize(Y) $specCount}
    if {$seriesFlag && $dimCount == 2} {set specSize(Z) $specCount}

    if {!$zLoc} {set zLoc [expr 1 + int(0.33*$specSize(Z))]}
    if {!$aLoc} {set aLoc [expr 1 + int(0.33*$specSize(A))]}

    sysFree $fdata 
}

#
# Create files that list ND input and output spectra, if any.

proc writeSpecLists {} \
{
    global dimCount specSize specName simName specCount seriesFlag

    set zCount $specSize(Z)
    set aCount $specSize(A)

    if {$seriesFlag} \
       {
        if {$dimCount < 3} {set zCount $specCount}
       } 

    if {$seriesFlag || $dimCount > 2} \
       {
        foreach i {0 1} \
           {
            set listName [lindex "nlin.spec.list sim.spec.list" $i] 
            set thisName [lindex "$specName $simName" $i]

            set error [catch {set outUnit [open $listName w]}]

            if {$error} \
               {
                puts stderr "Error opening output list $listName"
                exit 1
               }

             for {set ia 1} {$ia <= $aCount} {incr ia} \
                {
                 for {set iz 1} {$iz <= $zCount} {incr iz} \
                    {
                     if {$dimCount == 4} \
                        {
                         puts $outUnit [format $thisName $ia $iz]
                        } \
                     else \
                        {
                         puts $outUnit [format $thisName $iz]
                        }
                    }
                }

            close $outUnit
           }
       }
}

#
# Sets noise estimate of the current spectrum.

proc getSpecNoise {} \
{
   global specName specSize zLoc aLoc dimCount noiseRMS

   if {![strcasecmp $noiseRMS Auto]} {set noiseRMS [estSpecNoiseND $specName]}

   return

   set thisName [format $specName $zLoc $aLoc]

   switch $dimCount \
      {
       1  {readROI -roi 1 -ndim 1 -in $thisName -x X_AXIS}

       2  {readROI -roi 1 -ndim 2 -in $thisName -x X_AXIS -y Y_AXIS}

       3  {
           readROI -roi 1 -ndim 2 -in $thisName -x X_AXIS -y Y_AXIS -dz Z_AXIS $zLoc 0
          }

       4  {
           readROI -roi 1 -ndim 2 -in $thisName -x X_AXIS -y Y_AXIS -dz Z_AXIS $zLoc 0 -da A_AXIS $aLoc 0
          }

       default {
                puts "Error: bad dimension count."
                exit 1
               }
      }

   set roiInfo  [getROIInfo -roi 1]
   set rdata    [lindex $roiInfo 0]
   set sizeList [lindex $roiInfo 2]
   set n        1

   foreach size $sizeList {set n [expr $n*$size]}

   if {![strcasecmp $noiseRMS Auto]} {set noiseRMS [vEstNoise $rdata $n]}
}

#
# Estimate or extract detection threshold.

proc getSpecThresh {} \
{
    global thresh noiseRMS inTab simAllFlag

    if {![strcasecmp $thresh FromFile]} \
       {
        set tPtr [gdbGetInfo $inTab -parms text]

        while {$tPtr} \
           {
            set str [gdbGetInfo $tPtr -parms value]

            if {"[lrange $str 1 2]" == "Detection Levels:"} \
               {
                set thresh [lindex $str end]
                return $thresh
               }

            set tPtr [gdbNext $tPtr]
           } 
       }

    if {![strcasecmp $thresh Auto]} {set thresh [expr 6.0*$noiseRMS]}

   return $thresh
}

#
# Estimate various parameters required for nlinLS.

proc getFitParams {} \
{
   global specSize simSize aRegSize dRegSize maxMove lw minLW maxLW noiseRMS 
   global specName inTabName inTab seriesDim seriesFlag dimCount

#
# Read peak table.
# Find average linewidth in pts (xw yw).
# Estimate other fit parameters from these widths.

   set inTab   [gdbRead table -in $inTabName -name "Original Peak Data"]
   set dimList {X Y Z A}

   set nDim $dimCount

   if {$seriesFlag && $dimCount == 1} {set nDim 2} 
   if {$seriesFlag && $dimCount == 2} {set nDim 3} 
   if {$seriesFlag && $dimCount == 3} {set nDim 4} 

   for {set i 1} {$i <= $nDim} {incr i} \
      {
       set dim [lindex $dimList [expr $i-1]]

       if {![strcmp $dim $seriesDim]} \
          {
           set lw($dim) $specSize($dim)
          } \
       else \
          {
           set lw($dim) [getTabAvg $inTab ${dim}W]
          }

       if {![strcasecmp $dRegSize($dim) Auto]} \
          {
           if {![strcmp $dim $seriesDim]} \
              {
               set dRegSize($dim) $specSize($dim)
              } \
           else \
              {
               set dRegSize($dim) [expr 1 + int(0.5 + $lw($dim))]
              }
          }

       if {![strcasecmp $maxMove($dim) Auto]} \
          {
           set maxMove($dim) [format %.2f [expr 0.25*$lw($dim)]]
           if {$maxMove($dim) < 1.0} {set maxMove($dim) 1.0}
          }

       if {![strcasecmp $minLW($dim) Auto]} {set minLW($dim) 0.0}

       if {![strcasecmp $maxLW($dim) Auto]} \
          {
           set maxLW($dim) [format %.2f [expr 2.0*$lw($dim)]]
           if {$maxLW($dim) < 1.0} {set maxLW($dim) 1.0}
          }

       if {![strcasecmp $simSize($dim) Auto]} \
          {
           set simSize($dim) $specSize($dim)
          }
      }

   foreach dim $dimList \
      {
       if {![strcasecmp $dRegSize($dim) Auto]} \
          {
           set dRegSize($dim) 1
          }

      if {![strcasecmp $aRegSize($dim) Auto]} \
          {
           set aRegSize($dim) $dRegSize($dim)
          }
      }
}

#
# Create the optimization script.

proc writeFitScript {} \
{
   global tsFlag timeFlag simAllFlag simAllDir simAllName simAllScript covarFlag localFlag sleepTime seriesDim
   global seriesFlag sumFlag dimCount specCount specSize simSize aRegSize dRegSize maxMove lw minLW maxLW model
   global fitArgs multipletType specName inTabName outTabName auxTabName errTabName localName localFitName localArgs noiseRMS 
   global tol maxf iter simName  difName comName fitFlag adVarName verbArg

   set error [catch {set outUnit [open $comName w 0744]}]

   if {$error} \
      {
       puts stderr "Error opening script $comName"
       exit 1
      }

   if {$timeFlag} \
      {
       set apodInput apod.list
      } \
   else \
      {
       set apodInput None
      }

   set dimList   {X Y Z A}
   set deltaList ""
   set modList   ""
   set limList   ""
   set wList     ""
   set wSimList  ""

   if {$dimCount > 2 || $seriesFlag} \
      {
       set inArg  -list
       set inVal  nlin.spec.list
       set simVal sim.spec.list
      } \
   else \
      {
       set inArg  -data
       set inVal  \$specName
       set simVal \$simName
      }

   set simDir None
   set difDir None

   if {[strcasecmp $simName None]} \
      {
       set simDir [file dirname [format $simName 1 1]]
      } 

   if {[strcasecmp $difName None]} \
      {
       set difDir [file dirname [format $difName 1 1]]
      }

   puts $outUnit "#!/bin/csh -f"
   puts $outUnit ""

   puts $outUnit "set specName     = $specName"
   puts $outUnit "set simName      = $simName"
   puts $outUnit "set simDir       = $simDir"
   puts $outUnit "set difName      = $difName"
   puts $outUnit "set difDir       = $difDir"
   puts $outUnit "set inTabName    = $inTabName"
   puts $outUnit "set auxTabName   = $auxTabName"
   puts $outUnit "set outTabName   = $outTabName"
   puts $outUnit "set errTabName   = $errTabName"

   if {$localFlag} \
      {
       puts $outUnit "set localName    = $localName"
       puts $outUnit "set localFitName = $localFitName"
      }

   puts $outUnit "set noiseRMS     = $noiseRMS"
   puts $outUnit "set specCount    = $specCount"
   puts $outUnit ""

   set nDim $dimCount

   if {$seriesFlag && $dimCount == 1} {set nDim 2} 
   if {$seriesFlag && $dimCount == 2} {set nDim 3} 

   for {set i 1} {$i <= $nDim} {incr i} \
      {
       set dim [lindex $dimList [expr $i - 1]]

       puts $outUnit "set aRegSize${dim}  = $aRegSize($dim)" 
       puts $outUnit "set dRegSize${dim}  = $dRegSize($dim)" 
       puts $outUnit "set maxD${dim}      = $maxMove($dim)"
       puts $outUnit "set min${dim}W      = $minLW($dim)"
       puts $outUnit "set max${dim}W      = $maxLW($dim)"
       puts $outUnit "set simSize${dim}   = $simSize($dim)"
       puts $outUnit ""

       append modList   " $model($dim) "
       append wList     " \$dRegSize${dim} "
       append wSimList  " \$simSize${dim} "

       if {[strcmp $dim $seriesDim]} \
          {
           append deltaList " ${dim}_AXIS \$maxD${dim} "

           if {[strcasecmp $model($dim) CTIME1D]} \
              {
               append limList   " ${dim}W \$min${dim}W \$max${dim}W"
              }
          }
      }

   if {$timeFlag} \
      {
       puts $outUnit "apodAuto.com \$specName"
      }

   if {$seriesFlag} \
      {
       set xZF 64
       set yZF 64
       set zZF 64

       if {$xZF < 4*$dRegSize(X)} {set xZF [expr 4*$dRegSize(X)]}
       if {$yZF < 4*$dRegSize(Y)} {set yZF [expr 4*$dRegSize(Y)]}
       if {$zZF < 4*$dRegSize(Z)} {set zZF [expr 4*$dRegSize(Z)]}

       if {$dimCount == 1} \
          {
           puts $outUnit ""
           puts $outUnit "seriesTab -in  \$inTabName $inArg $inVal -ndim 1 \\"
           puts $outUnit " -dx  \$dRegSizeX \\"
           puts $outUnit " -adx \$aRegSizeX \\"
           puts $outUnit " -xzf $xZF -max \\"
           puts $outUnit " -out \$auxTabName -adVar $adVarName -$verbArg"
           puts $outUnit ""
          } \
       elseif {$dimCount == 3} \
          {
           puts $outUnit ""
           puts $outUnit "seriesTab -in  \$inTabName $inArg $inVal \\"
           puts $outUnit " -dx  \$dRegSizeX -dy  \$dRegSizeY \\"
           puts $outUnit " -adx \$aRegSizeX -ady \$aRegSizeY \\"
           puts $outUnit " -xzf $xZF -yzf $yZF -max \\"
           puts $outUnit " -out \$auxTabName -adVar $adVarName -$verbArg"
           puts $outUnit ""
          } \
       else \
          {
          puts $outUnit ""
          puts $outUnit "seriesTab -in  \$inTabName $inArg $inVal \\"
          puts $outUnit " -dx  \$dRegSizeX -dy  \$dRegSizeY -dz  \$dRegSizeZ \\"
          puts $outUnit " -adx \$aRegSizeX -ady \$aRegSizeY -adz \$aRegSizeZ \\"
          puts $outUnit " -xzf $xZF -yzf $yZF -zzf $zZF -max \\"
          puts $outUnit " -out \$auxTabName -adVar $adVarName -$verbArg"
          puts $outUnit ""
          }
      } \
   else \
      {
       puts $outUnit ""
       puts $outUnit "cp \$inTabName \$auxTabName"
       puts $outUnit ""
      }

   if {$timeFlag} \
      {
       puts $outUnit ""
       puts $outUnit "resetLW.tcl -in \$auxTabName -out \$auxTabName"
       puts $outUnit ""
      }

   if {$fitFlag} \
      {
       puts $outUnit ""

       if {$localFlag} \
          {
           if {![strcasecmp $localArgs None]} \
              {
               puts $outUnit "localFit.tcl -in \$auxTabName -ft \$specName -out \$localName"
              } \
           else \
              {
               puts $outUnit "localFit.tcl -in \$auxTabName -ft \$specName -out \$localName [expandArg $localArgs]"
              }

           puts $outUnit ""
          }

       if {![strcasecmp $fitArgs None] || ![llength $fitArgs]} \
          {
           if {![strcasecmp $multipletType PEAK]} \
              {
               puts $outUnit "nlinLS \\"
              } \
           else \
              {
               puts $outUnit "nlinLS -mult $multipletType \\"
              }
          } \
       else \
          {
           if {![strcasecmp $multipletType PEAK]} \
              {
               puts $outUnit "nlinLS $fitArgs \\"
              } \
           else \
              {
               puts $outUnit "nlinLS $fitArgs -mult $multipletType \\"
              }
          }

       puts $outUnit " -tol $tol -maxf $maxf -iter $iter \\"

       if {$localFlag} \
          {
           puts $outUnit " -in \$localName -out \$localFitName $inArg $inVal \\"
          } \
       else \
          {
           puts $outUnit " -in \$auxTabName -out \$outTabName $inArg $inVal \\"
          }

       puts $outUnit " -apod  $apodInput \\"
       puts $outUnit " -noise \$noiseRMS \\"
       puts $outUnit " -mod   $modList \\"
       puts $outUnit " -delta $deltaList \\"

       if {[llength $limList]} \
          {
           puts $outUnit " -limit $limList \\"
          }

       puts $outUnit " -w     $wList \\"

       if {!$tsFlag}   {puts -nonewline $outUnit " -nots"} 
       if {$sumFlag}   {puts -nonewline $outUnit " -sum"} 
       if {$localFlag} {puts -nonewline $outUnit " -nowarn"} 

       if {$covarFlag} \
          {
           puts $outUnit " -norm -ppm -covar"
          } \
       else \
          {
           puts $outUnit " -norm -ppm"
          }

       if {$localFlag} \
          {
           puts $outUnit ""
           puts $outUnit "localUpdate.tcl -in \$auxTabName -local \$localFitName -ft \$specName -out \$outTabName"
           puts $outUnit ""
          }
      } \
   else \
      {
       puts $outUnit ""
       puts $outUnit "cp \$auxTabName \$outTabName"
       puts $outUnit ""
      }

   if {[strcasecmp $simName None]} \
      {
       puts $outUnit ""
       puts $outUnit "if (!(-e \$simDir)) then"
       puts $outUnit "   mkdir \$simDir"
       puts $outUnit "endif"

       if {$dimCount < 3} \
          {
           if {$seriesFlag} \
              {
               puts $outUnit ""
               puts $outUnit "set ii = 1"
               puts $outUnit ""
               puts $outUnit "while (\$ii <= \$specCount)"
               puts $outUnit " set thisInName  = (`nmrPrintf $specName \$ii`)"
               puts $outUnit " set thisOutName = (`nmrPrintf $simName \$ii`)"
               puts $outUnit ""
               puts $outUnit " nmrPipe -in \$thisInName -out \$thisOutName -fn SET -r 0.0 -ov"
               puts $outUnit ""
               puts $outUnit " @ ii++"
               puts $outUnit "end"
               puts $outUnit ""
              } \
           else \
              {
               puts $outUnit ""
               puts $outUnit "nmrPipe -in \$specName -out \$simName -fn SET -r 0.0 -$verbArg -ov"
               puts $outUnit ""
              }
          } \
       else \
          {
           puts $outUnit ""
           puts $outUnit "xyz2pipe -in \$specName -$verbArg \\"
           puts $outUnit "| nmrPipe -fn SET -r 0.0 \\"
           puts $outUnit "| pipe2xyz -out \$simName -ov"
           puts $outUnit ""
          }

       if {![strcasecmp $multipletType PEAK]} \
          {
           puts $outUnit "simSpecND -in \$outTabName $inArg $simVal \\"
          } \
       else \
          {
           puts $outUnit "simSpecND -in \$outTabName $inArg $simVal -mult $multipletType \\"
          }

       puts $outUnit "          -mod  $modList \\"
       puts $outUnit "          -w    $wSimList \\"

       if {$tsFlag} \
          {
           puts $outUnit "          -apod $apodInput -$verbArg"
          } \
       else \
          {
           puts $outUnit "          -apod $apodInput -nots -$verbArg"
          }

       puts $outUnit ""
       puts $outUnit "appendTitle.com $simName SIM"
       puts $outUnit ""
      }

   if {[strcasecmp $simName None] && [strcasecmp $difName None]} \
      {
       puts $outUnit ""
       puts $outUnit "if (!(-e \$difDir)) then"
       puts $outUnit "   mkdir \$difDir"
       puts $outUnit "endif"
       puts $outUnit ""

       if {$dimCount < 3 && $seriesFlag} \
          {
           puts $outUnit ""
           puts $outUnit "set ii = 1"
           puts $outUnit ""
           puts $outUnit "while (\$ii <= \$specCount)"
           puts $outUnit " set thisInName1 = (`nmrPrintf $specName \$ii`)"
           puts $outUnit " set thisInName2 = (`nmrPrintf $simName \$ii`)"
           puts $outUnit " set thisOutName = (`nmrPrintf $difName \$ii`)"
           puts $outUnit ""
           puts $outUnit " addNMR -in1 \$thisInName1 -in2 \$thisInName2 -out \$thisOutName -sub"
           puts $outUnit ""
           puts $outUnit " @ ii++"
           puts $outUnit "end"
           puts $outUnit ""
          } \
       else \
          {
           puts $outUnit "addNMR -in1 \$specName -in2 \$simName -out \$difName -sub"
          }

       puts $outUnit ""
       puts $outUnit "appendTitle.com $difName DIF"
       puts $outUnit ""
      }

   if {$simAllFlag} \
      {
       if {$dimCount == 1} \
          {
           puts $outUnit ""
           puts $outUnit "sim1D.tcl $specName $outTabName \\"
           puts $outUnit "          -mod $modList -w $wSimList -apod $apodInput"
           puts $outUnit ""
          } \
       else \
          {
           puts $outUnit ""
           puts $outUnit "simAllPeaks.tcl -noexec -ndim $dimCount -ft $specName -tab $outTabName \\"
           puts $outUnit "                -simAllDir $simAllDir -simAllName $simAllName -simAllScript $simAllScript \\"
           puts $outUnit "                -simArgs -mod $modList -w $wSimList -apod $apodInput"
           puts $outUnit ""
           puts $outUnit "$simAllScript"
          }
      }

   if {$sleepTime} \
      {
       puts $outUnit ""
       puts $outUnit "sleep $sleepTime"
      }

   close $outUnit
}

#
# Execute the existing nlinLS script.

proc execFitScript {} \
{
   global comName execFlag xtermFlag

   if {!$execFlag} return

   if {$xtermFlag} \
      {
       set g 90x12+15+15
       set error [catch {exec xterm -bg cyan -fg black -geom $g -e $comName}]
      } \
   else \
      {
       set logName nlin[pid].log

       catch "file delete $logName"
       catch "exec $comName >& $logName"

       if {[file exists $logName]} {printFile stdout $logName}

       catch "file delete $logName"
      }
}

#
# Aux function to sort peak residuals

proc pkSort { r1 r2 } \
{
    set d1 [lindex $r1 0]
    set d2 [lindex $r2 0]

    if {$d1 < $d2} \
       {
        return 1
       } \
    elseif {$d1 > $d2} \
       {
        return -1
       }

    return 0
}

#
# Test the residual for points over the threshold.

proc testResidual {} \
{
   global inTab difName thresh dimCount execFlag

   if {!$execFlag} return
   if {$dimCount != 2} return
   if {![strcasecmp $difName None]} return
 
   set entry [gdbFirst $inTab]
   set rList ""

   while {$entry} \
      {
       set id    [gdbGet $entry INDEX]
       set clust [gdbGet $entry CLUSTID]

       set ix1 [gdbGet $entry X1] 
       set ixN [gdbGet $entry X3] 
       set iy1 [gdbGet $entry Y1]
       set iyN [gdbGet $entry Y3]

       readROI -roi 2 -ndim 2 -in $difName \
               -x X_AXIS $ix1 $ixN -y Y_AXIS $iy1 $iyN 

       set roiInfo [getROIInfo -roi 2]
    
       set rdata       [lindex $roiInfo 0]
       set thisXSize   [lindex [lindex $roiInfo 2] 0]
       set thisYSize   [lindex [lindex $roiInfo 2] 1]
       set n           [expr $thisXSize*$thisYSize]

       vAbs $rdata $n

       set maxVal      [vMax    $rdata $n]

       if {$maxVal > $thresh} \
          {
           set maxLoc [expr int([vMaxLoc $rdata $n])]
           set iy0    [expr int($maxLoc/$thisXSize)]
           set ix0    [expr int($maxLoc - $iy0*$thisXSize)]

           set ix0 [expr $ix0 + $ix1]
           set iy0 [expr $iy0 + $iy1]

           lappend rList "$maxVal $id $clust $ix0 $iy0"
          }

       set entry [gdbNext $entry]
      }

   set sList [lsort -command pkSort $rList]

   set fmt "Peak: %4d   Cluster: %4d  Max: %f at %d,%d."

   foreach s $sList \
      {
       set maxVal [lindex $s 0]
       set id     [lindex $s 1]
       set clust  [lindex $s 2]
       set ix0    [lindex $s 3]
       set iy0    [lindex $s 4]

       puts [format $fmt $id $clust $maxVal $ix0 $iy0]
      }
}

gdbInit 
gdbCreate dBase

initFitVars
initFitArgs

getSpecNoise
getFitParams
getSpecThresh
writeFitScript
execFitScript
testResidual

exit
