#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Extraction and Simple Alignment of a Selected Region in 1D Data:"
   echo " -in        inName    [test.dat]  1D Input to be Aligned." 
   echo " -ref       refName   [ref.dat]   1D Data Reference."
   echo " -out       outName   [shift.dat] Aligned Result."
   echo " -shiftName sName     [shift.txt] Alignment Parameter Output."
   echo " -del                             Delete Temporary Files."
   echo " -nodel                           Save Temporary Files (Default)."
   echo " -verb                            Verbose Mode ON (Default)."
   echo " -noverb                          Verbose Mode OFF."
   echo "Spectral Region, ppm Hz pts %:"
   echo " -x1        x1        [0%]"
   echo " -xn        xn        [100%]"
   echo "Shift Limits and Step Sizes, Integer Points:"
   echo " -sMin      sMinPts   [-64]"
   echo " -sMax      sMaxPts   [64]"
   echo " -sStep     sStepPts  [1]"
   echo "Notes:"
   echo " 1. Input Data will be aligned to reference."
   echo " 2. Input Data and Reference should have the same digital resolution."
   exit 0
endif

set refName = (`getArgD $argv -ref  ref.dat`)
set inName  = (`getArgD $argv -in   test.dat`)
set outName = (`getArgD $argv -out  shift.dat`)

set refTmp    = (`getArgD $argv -refTmp    reftmp.dat`)
set regTmp    = (`getArgD $argv -regTmp    regtmp.dat`)
set shiftName = (`getArgD $argv -shiftName shift.txt`)

set x1      = (`getArgD $argv -x1       0%`)
set xn      = (`getArgD $argv -xn     100%`)
set sMin    = (`getArgD $argv -sMin   -64`)
set sMax    = (`getArgD $argv -sMax    64`)
set sStep   = (`getArgD $argv -sStep    1`)

set delFlag  = 0
set verbFlag = 1

if (`flagLoc $argv -del`) then
   set delFlag = 1
endif

if (`flagLoc $argv -nodel`) then
   set delFlag = 0
endif

if (`flagLoc $argv -verb`) then
   set verbFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set verbFlag = 0
endif

foreach i ($refTmp $regTmp $shiftName)
   if (-e $i) then
      /bin/rm -f $i
   endif
end

if (!(-e $refName)) then
   nmrPrintf -stderr "Error Finding Alignment Reference:\n"
   nmrPrintf -stderr "$refName\n"
   exit 1
endif

if (!(-e $inName)) then
   nmrPrintf -stderr "Error Finding Alignment Input:\n"
   nmrPrintf -stderr "$inName\n"
   exit 1
endif

readROI -ndim 1 -x X_AXIS $x1 $xn -out $refTmp -in $refName -noverb -silent -ov
readROI -ndim 1 -x X_AXIS $x1 $xn -out $regTmp -in $inName  -noverb -silent -ov

set totalScale = 1.0

if (`flagLoc $argv -rescale`) then
   set refScale = (`scale2D -brief $refTmp`)
   set regScale = (`scale2D -brief $regTmp`)

   nmrPipe -in $refTmp -out $refTmp -inPlace -fn MULT -c $refScale -inv
   nmrPipe -in $regTmp -out $regTmp -inPlace -fn MULT -c $regScale -inv

   set totalScale = (`MATH "div( $refScale, $regScale )"`)
endif

modelXY -noverb -silent \
  -data $refTmp -aData $regTmp \
  -macro $NMRTXT/vvScaleShift1DQ.M -p 1.0 -gc 1 \
  -var sMin $sMin sMax $sMax sStep $sStep >& $shiftName

set shiftInfo = (`tail -1 $shiftName`)

if (`flagLoc $argv -applyScale`) then
   set totalScale = (`MATH "$totalScale*$shiftInfo[4]"`) 
else
   set totalScale = 1.0
endif

set scaleInfo = "refScale $refScale regScale $regScale totalScale $totalScale" 
echo $scaleInfo >> $shiftName

if (`string csame $outName None`) then
   set totalScale = 1.0
else
   if (`string same $inName $outName`) then
      nmrPipe -in $inName -fn MULT -c $totalScale | nmrPipe -fn CS -ls $shiftInfo[2] -out $outName -ov -inPlace
   else
      nmrPipe -in $inName -fn MULT -c $totalScale | nmrPipe -fn CS -ls $shiftInfo[2] -out $outName -ov
   endif
endif

if ($verbFlag == 0) then
   echo $shiftInfo[2]
else
   echo "refName $refName alignName $inName $shiftInfo $scaleInfo"
endif


if ($delFlag != 0) then
   foreach i ($refTmp $regTmp)
      if (-e $i) then
         /bin/rm -f $i
      endif
   end
endif
