#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Apply X-Axis Baseline Correction to Axial Edges:"
   echo " -in   inName  [test.ft2]   Input Spectrum."
   echo " -out  outName [base.ft2]   Output Spectrum."
   echo " -ord  ord     [4]          Baseline Correction Polynomial Order."
   echo " -frac f       [0.05]       Baseline Region Width Fraction."
   echo " -verb                      Verbose Mode ON."
   echo "Notes:"
   echo " 1. If the Y-Axis size is ySize, then at each edge, a region"
   echo "    of f*ySize will be baseline corrected, and the following"
   echo "    region of f*ySize will be baseline corrected with a " 
   echo "    cosine-squared roll-off."
   echo " 2. No baseline correction will be applied for -f 0.0 or"
   echo "    for -ord None." 
   echo " 3. For in-place processing, inName and outName must be"
   echo "    specified identically."
   exit 0
endif

set inName  = (`getArgD $argv -in   test.ft2`)
set outName = (`getArgD $argv -out  base.ft2`)
set ord     = (`getArgD $argv -ord  4`)
set f       = (`getArgD $argv -frac 0.05`)

set vFlag   = (`getLogArgD $argv -noverb -verb 0 1`)
 
if (!(-e $inName)) then
   echo "Baseline Correction Error Finding Input: $inName"
   exit 0
endif

@ baseFlag = 1

if (`string same $inName $outName`) then
   @ inPlaceFlag = 1
   set inPlaceArg = -inPlace
else
   @ inPlaceFlag = 0
   set inPlaceArg = "" 

endif

if (`IMATH "$f <= 0.0"`) then
   set ord = "None"
endif

if (`string csame $ord None`) then
   if ($inPlaceFlag == 0) then
      /bin/cp $inName $outName
   endif

   echo "X-Axis Baseline Correction: Exit Without Correction."
   exit 0
endif

if (`IMATH "$f >= 0.5"`) then
   nmrPrintf "X-Axis Baseline Correction. Full Correction. Order: %s\n" $ord
   nmrPipe -in $inName -fn POLY -auto -ord $ord -out $outName -ov $inPlaceArg
   exit 0
endif

set size = (`getParm -in $inName -parm NDSIZE -dim CUR_YDIM -fmt %.0f`)
set n    = (`IMATH "$f*$size"`)
set ix1  = (`IMATH "$n + 1"`)
set ixn  = (`IMATH "1 + $size - 2*$n"`)
set jx1  = (`IMATH "2*$n + 1"`)
set jxn  = (`IMATH "$ixn - 1"`)

set jx1m  = (`IMATH "2*$n"`)
set ixnp  = (`IMATH "$ixn + $n - 1"`)
set ixnpp = (`IMATH "$ixnp + 1"`)

if ($vFlag) then
   echo ""
   nmrPrintf "X-Axis Baseline Correction. Y-Axis Fraction: %s Order: %s\n" $f $ord
   nmrPrintf " Leading Edge:  %4d %4d\n" 1      $n
   nmrPrintf " Roll-Off:      %4d %4d\n" $ix1   $jx1m
   nmrPrintf " No Correction: %4d %4d\n" $jx1   $jxn
   nmrPrintf " Roll-Off:      %4d %4d\n" $ixn   $ixnp
   nmrPrintf " Trailing Edge: %4d %4d\n" $ixnpp $size
   echo ""
endif

set tmpName = base_tmp$$.dat
set difName = base_dif$$.dat

nmrPipe -in $inName -fn POLY -auto -ord $ord -out $tmpName -ov $inPlaceArg

addNMR -in1 $inName -in2 $tmpName -out $difName -sub

nmrPipe -in $difName \
| nmrPipe -fn TP \
| nmrPipe -fn SP -off 0.5 -end 1.0 -pow 2 -size $n -start $ix1 -one \
| nmrPipe -fn SP -off 0.0 -end 0.5 -pow 2 -size $n -start $ixn -one \
| nmrPipe -fn SET -c 0.0 -x1 $jx1 -xn $jxn \
| nmrPipe -fn TP \
  -out $difName -ov -inPlace

addNMR -in1 $inName -in2 $difName -out $outName -sub

/bin/rm -f $tmpName $difName
