#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "basicAutoPhase.com Version 2023.047.16.19"
   echo "Perform X-Axis Auto-Phase for Basic Processing Scripts:"
   echo " -in       in    [Auto]    Time-Domain Input."
   echo " -apMode   mode  [sumNeg]  Phase Metric: sumNeg sum min max rms balance."
   echo " -pseudo3D                 Input is Pseudo-3D."
   echo " -apxy                     Process XY Plane (2D-4D, Default)."
   echo " -apxz                     Process XZ Plane (3D or 4D)."
   echo " -apxa                     Process XA Plane (4D Only)."
   echo " -verb                     Verbose Mode ON."
   echo "Analysis Region (Pts, Hz, ppm, %):"
   echo " -apx1     x1    [15%]     X-Axis Analysis Region Start (Non-HN Default: 55%)."
   echo " -apxn     xn    [45%]     X-Axis Analysis Region End (Non-HN Default: 95%)."
   echo " -apy1     y1    [33%]     Y-Axis Analysis Region Start."
   echo " -apyn     yn    [66%]     Y-Axis Analysis Region End."
   echo " -apBDX    bdx   [0.1ppm]  Width of Edge for -apMode balance (Pts, Hz, ppm, %)."
   echo "Interim Processing for Autophase:"
   echo " -apxELB   elb   [0.0]     X-Axis Exponential Broadening, Hz."
   echo " -apxGLB   glb   [0.0]     X-Axis Gaussian Broadening, Hz."
   echo " -apyELB   elb   [0.0]     Y-Axis Exponential Broadening, Hz."
   echo " -apyGLB   glb   [0.0]     Y-Axis Gaussian Broadening, Hz."
   echo " -apyFTARG arg   [None]    FT Fourier Transform (FT) Options (None neg alt)."
   echo " -apxP1    xP1   [0.0]     X-Axis First Order Phase Correction."
   echo " -apyP0    yP0   [0.0]     Y-Axis Zero Order Phase Correction."
   echo " -apyP1    yP1   [0.0]     Y-Axis First Order Phase Correction."
   echo "Temporary Baseline Correction:"
   echo " -apOrd    ord   [0]       Baseline Correction Polynomial Order, -1 for None."
   echo " -apWindow w     [8]       Auto-Correction Window (Pts Hz ppm %)."
   echo " -apbx1    bx1   [None]    Non-Auto Correction Signal-Free Region Start."
   echo " -apbxn    bxn   [None]    Non-Auto Correction Signal-Free Region End."
   echo "Solvent Band (Pts, Hz, ppm, %):"
   echo " -apsx1    sx1   [None]    X-Axis Coord of Solvent Band Start."
   echo " -apsxn    sxn   [None]    X-Axis Coord of Solvent Band End."
   echo "Output/Temporary Files:"
   echo ' -prefix    p    [tmp_ps$$]   Prefix for Temporary Files ($$ is Process ID).'
   echo " -store     name [None]       Option Text Output File with Auto Phase Value."
   echo ' -noclean                     Do Not Delete Temporary Files After Autophase.'
   echo "Phase Limits, Degrees:"
   echo " -apxP0First xP0First [-200]  Start of Phase Value Range."
   echo " -apxP0Last  xP0First [200]   End of Phase Value Range."
   echo " -apxP0Step  xP0Step  [1]     Phase Value Step."
   echo "Options for 2D Planes from 3D and 4D Data:"
   echo " -apyPlane   yPlane   [1]     Y-Axis Coord of Plane (3D-4D -apxz or -apxa)."
   echo " -apzPlane   zPlane   [1]     Z-Axis Coord of Plane (3D-4D -apxy or -apxa)."
   echo " -apaPlane   aPlane   [1]     A-Axis Coord of Plane (4D Only -apxy or -apxz)."
#  echo "Options for basicFT1.com Processing Step; Should be Last on Command-Line:"
#  echo " -apProcArgs pArgs    [None]  Options for basicFT1.com etc."
   echo "Notes:"
   echo " 1. If -apOrd is 0 or greater, a temporary baseline correction will be"
   echo "    applied at each iteration of phase optimization."
   echo " 2. If -bx1 and -bxn are specified to identify a signal free region, the"
   echo "    baseline correction will be a simple subtraction of the average of"
   echo "    this region. Otherwise, the baseline correction will be automated, and"
   echo "    the -apOrd value will determine the polynomial order of the correction."
   exit 0
endif

set loc      = (`flagLoc $argv -apProcArgs`)
set thisArgv = ($argv)
set procArgs = ()

@ vFlag = 0

if (`flagLoc $argv -verb`) then
   @ vFlag = 1
endif

if ($loc >= 1) then
   @ locP = $loc - 1
   @ locN = $loc + 1

   set procArgs = ($argv[$locN-])
   set thisArgv = ($argv[1-$locP])
endif

set inName = (`getArgD $thisArgv -in  Auto`)

if (`string csame $inName Auto`) then
   set fList  = (fid/test%03d.fid data/test%03d.fid fid/spec%03d.fid data/spec%03d.fid fid/test%04d.fid data/test%04d.fid test.fid spec.fid)
   set inName = test.fid

   foreach i ($fList)
      set thisName = (`nmrPrintf $i 1 1`)

      if (-e $thisName) then
         set inName = $i
         break
      endif
   end
endif

if (`string csame $inName Auto`) then
   echo "BasicAutoPhase Error Finding Auto Input File."
   exit 1
endif

@ pseudo3DFlag = 0

if (`flagLoc $thisArgv -pseudo3D`) then
   @ pseudo3DFlag = 1
endif

set thisName = (`nmrPrintf $inName 1 1`)

set xLAB = (`getParm -in $inName -txt -fmt %s -parm NDLABEL -dim CUR_XDIM`)

if (`string match "HN*" $xLAB`) then
   set x1     = (`getArgD $thisArgv -apx1     15%`)
   set xn     = (`getArgD $thisArgv -apxn     45%`)
   set y1     = (`getArgD $thisArgv -apy1     33%`)
   set yn     = (`getArgD $thisArgv -apyn     66%`)
else
   set x1     = (`getArgD $thisArgv -apx1     55%`)
   set xn     = (`getArgD $thisArgv -apxn     95%`)
   set y1     = (`getArgD $thisArgv -apy1     33%`)
   set yn     = (`getArgD $thisArgv -apyn     66%`)
endif

set bx1     = (`getArgD $thisArgv -apbx1      None`)
set bxn     = (`getArgD $thisArgv -apbxn      None`)

set solx1   = (`getArgD $thisArgv -apsx1      None`)
set solxn   = (`getArgD $thisArgv -apsxn      None`)

set ord     = (`getArgD $thisArgv -apOrd     0`)
set window  = (`getArgD $thisArgv -apWindow  8`)
set mode    = (`getArgD $thisArgv -apMode    sumNeg`)
set bdx     = (`getArgD $thisArgv -apBDX     0.1ppm`)
set yFT     = (`getArgD $thisArgv -apyFTARG  None`)

set xELB    = (`getArgD $thisArgv -apxELB      0.0`)
set xGLB    = (`getArgD $thisArgv -apxGLB      0.0`)

set yELB    = (`getArgD $thisArgv -apyELB      0.0`)
set yGLB    = (`getArgD $thisArgv -apyGLB      0.0`)

set xP1     = (`getArgD $thisArgv -apxP1       0.0`)
set yP0     = (`getArgD $thisArgv -apyP0       0.0`)
set yP1     = (`getArgD $thisArgv -apyP1       0.0`)

set xP0First  = (`getArgD $thisArgv -apxP0First -200`)
set xP0Last   = (`getArgD $thisArgv -apxP0Last   200`)
set xP0Step   = (`getArgD $thisArgv -apxP0Step   1`)

set yPlane    = (`getArgD $thisArgv -apyPlane  1`)
set zPlane    = (`getArgD $thisArgv -apzPlane  1`)
set aPlane    = (`getArgD $thisArgv -apaPlane  1`)

set prefix    = (`getArgD $thisArgv -prefix    tmp_ps$$`)
set storeName = (`getArgD $thisArgv -store     None`)

set xZeroList = (`getListArgD $thisArgv -xZero -- None`)
set yZeroList = (`getListArgD $thisArgv -yZero -- None`)

@ xyMode = 1

if (`flagLoc $thisArgv -apxy`) then
   @ xyMode = 1
endif

if (`flagLoc $thisArgv -apxz`) then
   @ xyMode = 2
endif

if (`flagLoc $thisArgv -apxa`) then
   @ xyMode = 3
endif

#
#---------------------------------------------------------------------------------------------------------------------------------------------

@ fmtCount   = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   nmrPrintf -stderr "BasicAutoPhase Error Finding Input File $thisName\n"
   exit 1
endif

set logName = ${prefix}.log
set comName = ${prefix}.com

set tmpName = ${prefix}_ft.dat
set htName  = ${prefix}_ht.dat
set psName  = ${prefix}_ps.dat

set dimCount = (`getParm -in $inName -parm FDDIMCOUNT -dim NULL_DIM -fmt %.0f`)

if ($pseudo3DFlag) then
   set xyArg = ""
else if ($dimCount == 3 || $dimCount == 4) then
   if ($xyMode == 3) then
      set xyArg = "-xa"
   else if ($xyMode == 2) then
      set xyArg = "-xz"
   else
      set xyArg = "-xy"
   endif
else if ($dimCount == 2) then
   set xyArg = ""
else
   set xyArg = ""
   set y1    = 1  
   set yn    = 1  
endif

set bandArgs = ()

if (`string notcsame $solx1 None`) then
   set bandArgs = (-xPOSTFT SET,c=0.0,x1=$solx1,xn=$solxn)
endif
   
if ($dimCount == 1) then
   set cmnd = (basicFT1.com $procArgs -in $thisName -out $tmpName -xELB $xELB -xGLB $xGLB \
                            -xEXTX1 0% -xEXTXN 100% -xP1 $xP1 $bandArgs -nodi -script $comName -noverb)

   if ($vFlag) then
      echo $cmnd
   endif

   $cmnd >& $logName 

   /bin/cp $tmpName $htName
else
   set cmnd = (basicFT2.com $procArgs $bandArgs $xyArg -in $inName -out $tmpName \
                            -yPlane $yPlane -zPlane $zPlane -aPlane $aPlane -xELB $xELB -xGLB $xGLB -yELB $yELB -yGLB $yGLB \
                            -xEXTX1 0% -xEXTXN 100% -xP1 $xP1 -yP0 $yP0 -yP1 $yP1 -yFTARG $yFT \
                            $bandArgs -script $comName -noverb)

   if ($vFlag) then
      echo $cmnd
   endif

   $cmnd >&  $logName 

   set cmnd = (nmrPipe -in $tmpName -fn HT -nowarn -out $htName -ov)

   if ($vFlag) then
      echo $cmnd
   endif

   $cmnd >>& $logName 
endif

set cmnd = (phaseNMR -in $htName -out $psName -ix1 $x1 -ixn $xn -bx1 $bx1 -bxn $bxn \
                     -iy1 $y1 -iyn $yn -xP0First $xP0First -xP0Last $xP0Last -xP0Step $xP0Step \
                     -ord $ord -window $window -$mode -bdx $bdx)

if ($vFlag) then
   echo $cmnd
endif

set psInfo = (`$cmnd` 0.0)

if (`flagLoc $thisArgv -noclean`) then
else
   /bin/rm $tmpName $htName $psName $logName $comName
endif

echo $psInfo[1]

if (`string csame $storeName None` == 0) then
   echo $psInfo[1] > $storeName
endif

