#!/bin/csh

set thisVersion = "2020.027.11.15"
set progName    = "basicSierra.com"

if (`flagLoc $argv -help`) then
   echo "$progName (Using basicFT2.com) $thisVersion"
   echo "https://www.ibbr.umd.edu/sierra/home"
   echo "Perform 2D Processing and SIERRA Residual Excipient Signal Subtraction:"
   echo " -in        inName   [test.fid]       Input 2D Time-Domain Data."
   echo " -out       outName  [sierra.ft2]     Signal-subtracted Output Spectrum."
   echo " -ft        ftName   [orig.ft2]       Conventionally Processed Output Spectrum."
   echo " -method    mName    [basicFT2.com]   General Processing Script to Use."
#  echo " -script    sName    [nmr_sierra.com] Output SIERRA Processing Script."
   echo " -modelArgs mArgs    [None]           Extra Arguments for SMILE Modeling."
   echo " -baseArgs  bArgs    [None]           Extra Arguments for tdBaseline.com script."
   echo " -prefix    prefix   [sierra]         Prefix for Intermediate Files."
   echo " -noclean                             Do Not Delete Intermediate Files."
#  echo " -noexec                              Create SIERRA Script, Do Not Process Data."
   echo "Excipient Signal Regions in PPM (Groups of Four Values x1 xn y1 yn):" 
   echo " -reg       regList  [2.12 2.07 27.0 25.0 ...]"
   echo "Options for Non-Uniformly Sampled (NUS) Data:"
   echo " -sample    sName    [Auto]           Sample Schedule Input Name."
   echo " -mask      mName    [mask.fid]       NMRPipe-Format NUS Mask Input."
   echo "Excipient Region Y-Axis Baseline Correction by Time-Domain Replacement:"
   echo " -tdN       tdNList  [0 ...]          Number of Time-Domain Points to Replace."
   echo " -exPPM     exList   [0.025 ...]      Baseline Edge Width, PPM."
   echo " -dxPPM     dxList   [0.050 ...]      Baseline Roll-off Width, PPMz."
   echo " -baseMask  bName    [None]           Optional Baseline Mask Output."
   echo " -baseDif   dName    [None]           Optional Baseline Difference Output."
   echo "Processing Method Options (Must be Last on the Command Line):"
   echo " -procArgs  procArgs [None]           Processing Options for basicFT2.com"
   echo ""
   echo "Notes:"
   echo " 1. SIERRA is a measurement approach for selective suppression of excipient"
   echo "    signals in 2D NMR 1H/13C methyl spectra of proteins. This script is intended"
   echo "    to subtract any residual excipient signals that still remain. SIERRA is"
   echo "    described here:"
   echo ""
   echo "       LW Arbogast, F Delaglio, J Tolman, and JP Marino: Selective suppression"
   echo "       of excipient signals in 2D 1H–13C methyl spectra of biopharmaceutical"
   echo "       products. J. Biomol NMR (2018) https://doi.org/10.1007/s10858-018-0214-1"
   echo ""
   echo "       https://www.ibbr.umd.edu/sierra/home"
   echo ""
   echo " 2. This script requires the SMILE reconstruction plugin, which should also be"
   echo "    cited when using this script:"
   echo ""
   echo "       J Ying, F Delaglio, DA Torchia, and A Bax: Sparse multidimensional"
   echo "       iterative lineshape-enhanced (SMILE) reconstruction of both non-uniformly"
   echo "       sampled and conventional NMR data. J. Biomol. NMR (2016)"
   echo "       https://doi.org/10.1007/s10858-016-0072-7"
   echo ""
   echo "       https://spin.niddk.nih.gov/bax/software/smile"
   echo ""
   echo " 3. This script processes time-domain data, uses SMILE to build a peak"
   echo "    table that models the spectrum, and extracts only those peaks that"
   echo "    lie in the excipient regions. The extracted peak table is used to"
   echo "    generate synthetic time-domain data that is processed in the same"
   echo "    way as the original spectrum, to give a synthetic spectrum containing"
   echo "    only the signals in the exciptient regions. This result is subtracted"
   echo "    from the original spectrum to generate an excipient-subtracted spectrum."
   echo " 4. Option -method specifies the general-purpose processing script to use"
   echo "    for processing the original and synthetic data. It is usually set to"
   echo "    basicFT2.com for Uniformly Sampled (US) data, and to ist2D.com or"
   echo "    smile2D.com for NUS data. If a NUS sampling schedule is found, ist2D.com"
   echo "    will be the default method."
   echo " 5. In some cases, residual excipient signals have depressions that need"
   echo "    additional baseline correction before signal modeling and subtraction."
   echo "    This script can apply a baseline correction that works by replacing a"
   echo "    small number (~1-10) of the initial points in each time-domain vector,"
   echo "    which can work well for this purpose. However, it is better not to apply"
   echo "    this correction to the entire spectrum, so instead the correction is"
   echo "    localized to the bands containing the excipient signals."
   echo " 6. The localized baseline correction will be applied when option -tdN is set"
   echo "    to an integer greater than zero."
   echo " 7. The regions used for localized baseline correction correspond to the X-Axis"
   echo "    bounds of the excipient regions, expanded on either side by the values of"
   echo "    the -exPPM and -dxPPM options. For this reason, the edges of the spectrum"
   echo "    must be far enough away from the excipient regions to account for the extra"
   echo "    region width specified by the -exPPM and -dxPPM options."
   echo " 8. If more than one region is specified via the -reg option, then options -tdN,"
   echo "    -exPPM, and -dxPPM can have a list of values, one for each region. This"
   echo "    allows each region to be baseline corrected independently, or to have a"
   echo "    correction applied to some regions and not to others. If only single values"
   echo "    are supplied, the same correction will be applied to all excipient regions."
   echo ""
   echo " Example: Process data, model and subtract the excipient signal in the region"
   echo "          1H 2.12ppm 2.07ppm, 13C 27.0ppm 25.0ppm, using localized baseline"
   echo "          correction of the excipient band:"
   echo ""
   echo "          basicSierra.com -in test.fid -ft orig.ft2 -out sierra.ft2 \"
   echo "                          -reg 2.12 2.07 27.0 25.0 -tdN 6 \"
   echo "                          -procArgs -xP0 Auto -xEXTX1 3.5ppm -xEXTXN -1.5ppm \"
   echo "                          -xBASEARG POLY,auto,ord=4 -yP0 -90"
   echo ""
   exit 0
endif

set thisArgv = ($argv)
set procArgs = None 

set loc = (`flagLoc $argv -procArgs`)
set n   = $#argv

if ($loc) then
   @ locPrev = $loc - 1
   @ locNext = $loc + 1

   set thisArgv = ($argv[1-$locPrev])
   @ loc++
   set procArgs = ($argv[$locNext-$n])
endif

set defMethod    = basicFT2.com
set defSchedName = None

set fList = (nuslist sampling.sch `find ./ -maxdepth 1 -name schedule\[0-9\]\* -print` `find ./ -maxdepth 1 -name \*.hdr_3 -print` fake.nuslist)

foreach f ($fList)
   if (-e $f) then
      set defSchedName = $f
      set defMethod    = ist2D.com
      break
   endif
end

set schedName    = (`getArgD $thisArgv -sample    $defSchedName`)

set prefix       = (`getArgD $thisArgv -prefix    sierra`)
set fidName      = (`getArgD $thisArgv -in        test.fid`)
set ftName       = (`getArgD $thisArgv -ft        orig.ft2`)
set sierraName   = (`getArgD $thisArgv -out       sierra.ft2`)
set scriptName   = (`getArgD $thisArgv -script    nmr_sierra.com`)
set method       = (`getArgD $thisArgv -method    $defMethod`)
set baseArgs     = (`getArgD $thisArgv -baseArgs  None`)
set smileArgs    = (`getArgD $thisArgv -modelArgs None`)
set maskName     = (`getArgD $thisArgv -mask      mask.fid`)
set baseMaskName = (`getArgD $thisArgv -baseMask  None`)
set baseDifName  = (`getArgD $thisArgv -baseDif   None`)

set defReg      = (2.12 2.07 27.0 25.0)
set regList     = (`getListArgD $thisArgv -reg   -- $defReg`)
set tdNListOrig = (`getListArgD $thisArgv -tdN   -- 0`)
set dxListOrig  = (`getListArgD $thisArgv -dx    -- 0.050`)
set exListOrig  = (`getListArgD $thisArgv -ex    -- 0.025`)

@ execFlag      = 1
@ cleanFlag     = 1
@ titleFlag     = 0
@ nusFlag       = 0

if (`flagLoc $thisArgv -exec`) then
   @ execFlag = 1
endif

if (`flagLoc $thisArgv -noexec`) then
   @ execFlag = 0
endif

if (`flagLoc $thisArgv -clean`) then
   @ cleanFlag = 1
endif

if (`flagLoc $thisArgv -noclean`) then
   @ cleanFlag = 0
endif

if (`flagLoc $thisArgv -updateTitle`) then
   @ titleFlag = 1
endif

if (`flagLoc $thisArgv -noupdateTitle`) then
   @ titleFlag = 0
endif

if (`string csame $procArgs None`) then
   set procArgs = ""
endif

if (`string csame $smileArgs None`) then
   set smileArgs = ""
endif

if (`string csame $schedName None` == 0) then
   if (!(-e $schedName)) then
      echo "basicSIERRA Error Finding NUS Schedule $schedName"
      exit 1
   endif

   @ nusFlag = 1
endif

if (!(-e $fidName)) then
   echo "basicSIERRA Error Finding Input Time-Domain Data $fidName"
   exit 1
endif

@ itemCount = $#regList
@ regCount  = (`IMATH "$itemCount/4"`)

if (`IMATH "$itemCount % 4"` != 0) then
   echo "basicSIERRA Error: Number of -reg Values Must Be a Multiple of Four."
   exit 1
endif

#
#-------------------------------------------------------------------------------------------------

set tmpName    = ${prefix}_tmp.ft2
set baseName   = ${prefix}_base.ft2
set smileName  = ${prefix}_smile.ft2
set sumName    = ${prefix}_sum.ft2
set logName    = ${prefix}_smile.log
set smileTab   = ${prefix}_smile.tab
set tmpTab     = ${prefix}_tmp.tab

if ($regCount == 1) then
   set simFIDName = ${prefix}_sim.fid
   set simFTName  = ${prefix}_sim.ft2
   set filtTab    = ${prefix}_filt.tab
else
   set simFIDName = ${prefix}_sim%d.fid
   set simFTName  = ${prefix}_sim%d.ft2
   set filtTab    = ${prefix}_filt%d.tab
endif

set tdNList = (`extendList.com -n $regCount -list $tdNListOrig`)
set dxList  = (`extendList.com -n $regCount -list $dxListOrig`)
set exList  = (`extendList.com -n $regCount -list $exListOrig`)

set x1BaseList = ""
set xnBaseList = ""
set dxBaseList = ""
set x1List     = ""
set xnList     = ""
set y1List     = ""
set ynList     = ""

@ i = 1
@ k = 1

while ($i <= $regCount)
   set thisX1 = $regList[$k]
   @ k++

   set thisXN = $regList[$k]
   @ k++

   set thisY1 = $regList[$k]
   @ k++

   set thisYN = $regList[$k]
   @ k++

   if (`IMATH "$thisX1 < $thisXN"`) then
      set tmp    = $thisX1
      set thisX1 = $thisXN
      set thisXN = $tmp 
   endif

   if (`IMATH "$thisY1 < $thisYN"`) then
      set tmp    = $thisY1
      set thisY1 = $thisYN
      set thisYN = $tmp
   endif

   set x1Base = (`MATH "$thisX1 + $exList[$i]"`)
   set xnBase = (`MATH "$thisXN - $exList[$i]"`)

   set x1BaseList = ($x1BaseList ${x1Base}ppm)
   set xnBaseList = ($xnBaseList ${xnBase}ppm)
   set dxBaseList = ($dxBaseList $dxList[$i]ppm)

   set x1List = ($x1List $thisX1)
   set xnList = ($xnList $thisXN)
   set y1List = ($y1List $thisY1)
   set ynList = ($ynList $thisYN)
   @ i++
end

if ($cleanFlag) then
   set cleanList = ($tmpName $baseName $sumName $smileName $logName $smileTab $tmpTab)
else
   set cleanList = ($tmpName $tmpTab)
endif

echo ""
echo "basicSierra. 2D Processing and SIERRA Excipient Signal Subtraction."
echo "basicSierra. https://www.ibbr.umd.edu/sierra/home"
echo ""
echo "basicSierra. Version:            $thisVersion"
echo "basicSierra. Time-Domain Input:  $fidName"

if ($nusFlag) then
   echo "basicSierra. NUS Schedule:       $schedName"
   echo "basicSierra. NUS Mask:           $maskName"
endif

echo "basicSierra. Freq-Domain Output: $ftName"
echo "basicSierra. SIERRA Output:      $sierraName"
echo ""

#
# Perform basic processing.
# Extract Y-Axis processing parameters of result to use for SMILE:

if ($nusFlag) then
   prettyPrintCmnd.tcl -nocomment $method -in $fidName -out $ftName $procArgs -mask $maskName -sample $schedName
   $method -in $fidName -out $ftName $procArgs -mask $maskName -sample $schedName
else
   prettyPrintCmnd.tcl -nocomment $method -in $fidName -out $ftName $procArgs
   $method -in $fidName -out $ftName $procArgs
endif

echo ""

set yApodName = (`getParm -in $ftName -parm NDAPODCODE -dim CUR_YDIM -fmt %s -text`)
set yQ1       = (`getParm -in $ftName -parm NDAPODQ1   -dim CUR_YDIM -fmt %.3f`)
set yQ2       = (`getParm -in $ftName -parm NDAPODQ2   -dim CUR_YDIM -fmt %.3f`)
set yELB      = (`getParm -in $ftName -parm NDAPODQ3   -dim CUR_YDIM -fmt %.3f`)
set yGLB      = (`getParm -in $ftName -parm NDAPODQ3   -dim CUR_YDIM -fmt %.3f`)

set xT        = (`getParm -in $ftName -parm NDAPOD     -dim CUR_XDIM -fmt %.0f`)
set xN        = (`getParm -in $ftName -parm NDSIZE     -dim CUR_XDIM -fmt %.0f`)
set xP0       = (`getParm -in $ftName -parm NDP0       -dim CUR_XDIM -fmt %.2f`)
set xP1       = (`getParm -in $ftName -parm NDP1       -dim CUR_XDIM -fmt %.2f`)

set yT        = (`getParm -in $ftName -parm NDAPOD     -dim CUR_YDIM -fmt %.0f`)
set yN        = (`getParm -in $ftName -parm NDSIZE     -dim CUR_YDIM -fmt %.0f`)
set yP0       = (`getParm -in $ftName -parm NDP0       -dim CUR_YDIM -fmt %.2f`)
set yP1       = (`getParm -in $ftName -parm NDP1       -dim CUR_YDIM -fmt %.2f`)
set yT2       = (`IMATH "2*$yT"`)

if (`string csame $yApodName SP`) then
   set yQ3 = (`getParm -in $ftName -parm NDAPODQ3 -dim CUR_YDIM -fmt %.0f`)
else
   set yQ3 = (`getParm -in $ftName -parm NDAPODQ3 -dim CUR_YDIM -fmt %.3f`)
endif

#
# Perform time-domain baseline correction on signal regions as needed:

baseRegions.com -in $ftName -out $baseName \
                -dx $dxBaseList -x1 $x1BaseList -xn $xnBaseList \
                -tdN $tdNList -mask $baseMaskName -dif $baseDifName -tdArgs `expandArg $baseArgs`

#
# Inverse transform the indirect dimension.
# Use SMILE to produce a list of signals and a reconstructed spectrum.
# Scale the SMILE reconstruction to match the conventional FT spectrum.
# Convert the SMILE output signal list to an NMRPipe peak table.

set smileInfo = (`expandArg $smileArgs`)

nmrPipe -in $baseName \
| nmrPipe -fn TP \
| nmrPipe -fn HT -auto \
| nmrPipe -fn PS -inv -hdr \
| nmrPipe -fn FT -inv \
| nmrPipe -fn ZF -inv \
| nmrPipe -fn $yApodName -inv -hdr \
| nusPipe -fn SMILE $smileInfo -log $logName -nDim 2 -maxIter 2048 -nSigma 2 -report 2 -sample None \
                    -xApod $yApodName -xQ1 $yQ1 -xQ2 $yQ2 -xQ3 $yQ3 -xELB $yELB -xGLB $yGLB \
                    -xT $yT2 -xP0 $yP0 -xP1 $yP1 -scaling 1.0 \
| nmrPipe -fn ZF -zf 1 -auto \
| nmrPipe -fn FT \
| nmrPipe -fn PS -p0 $yP0 -p1 0.0 -di \
| nmrPipe -fn TP \
  -out $smileName -ov

set maxValFT = (`specStat.com -in $ftName -x1 5% -xn 95% -y1 5% -yn 95% -stat vMaxAbs -brief`)

rescale.com -in $smileName -out $smileName -max $maxValFT -x1 5% -xn 95% -y1 5% -yn 95%

smile2tab.tcl -in $logName -out $smileTab -ref $ftName 

#
# For every excipient region:
#  Extract only the SMILE peaks in that region.
#  Simulate time-domain data for the extracted signals.
#  Process the time-domain data to generate a a spectrum for that excipient signal.
#  Find a scale factor that minimizes the difference between the original data and simulation. 

echo ""
echo "basicSIERRA. SMILE Model Time-Domain Size: $xT $yT"
echo "basicSIERRA. SMILE Model Freq-Domain Size: $xN $yN"
echo "basicSIERRA. SMILE Model Region Count:     $regCount"

set simArgs = (`getSimParms.com -fid $fidName -ft $ftName`)
set tStr    = ""

/bin/cp $baseName $sierraName

if ($titleFlag) then
   set tStr = (`getParm -in $fidName -parm FDTITLE -text -fmt %s`)

   if (`string length $tStr` != 0) then
      set tStr = ${tStr}'-'
   endif

   sethdr $ftName     -title ${tStr}Sierra
   sethdr $sierraName -title ${tStr}Sierra_Subtracted
   sethdr $baseName   -title ${tStr}Baseline_Corrected
   sethdr $smileName  -title ${tStr}SMILE

   if (`string csame $baseMaskName None` == 0) then
      if (-e $baseMaskName) then
         sethdr $baseMaskName -title ${tStr}Baseline_Mask
      endif
   endif

   if (`string csame $baseDifName None` == 0) then
      if (-e $baseDifName) then
         sethdr $baseDifName -title ${tStr}Baseline
      endif
   endif
endif

nmrPipe -in $ftName -out $sumName -fn SET -c 0.0 -ov

if ($titleFlag) then
   sethdr $sumName -title ${tStr}Simulated_Excipient-Sum
endif

@ i = 1

while ($i <= $regCount)
   echo ""
   nmrPrintf "basicSIERRA. Excipient Region %d X:   %6.3fppm %6.3fppm\n" $i $x1List[$i] $xnList[$i]
   nmrPrintf "basicSIERRA. Excipient Region %d Y:   %6.3fppm %6.3fppm\n" $i $y1List[$i] $ynList[$i]
   echo ""

   if ($tdNList[$i] == 0) then
      nmrPrintf "basicSIERRA. Excipient Region %d Baseline Correction:     None\n" $i
   else
      nmrPrintf "basicSIERRA. Excipient Region %d Baseline Correction tdN: %d  dx: %.3fppm  de: %.3fppm\n" $i $tdNList[$i] $dxList[$i] $exList[$i]
   endif

   echo ""

   set cond = "X_PPM <= $x1List[$i] && X_PPM >= $xnList[$i] && Y_PPM <= $y1List[$i] && Y_PPM >= $ynList[$i]"

   set thisFiltTab  = (`nmrPrintf $filtTab    $i`)
   set thisFIDName  = (`nmrPrintf $simFIDName $i`)
   set thisFTName   = (`nmrPrintf $simFTName  $i`)

   selectTab.tcl -in $smileTab -out $thisFiltTab -hdr -cond $cond

   prettyPrintCmnd.tcl -nocomment simTimeND -in $thisFiltTab -out $thisFIDName -tdd -nots $simArgs -verb -ov
   echo ""

   simTimeND -in $thisFiltTab -out $thisFIDName -tdd -nots $simArgs -verb -ov

   echo ""
   prettyPrintCmnd.tcl -nocomment basicFT2.com -in $thisFIDName -out $thisFTName -xP0 $xP0 -xP1 $xP1 $procArgs

   basicFT2.com -in $thisFIDName -out $thisFTName -xP0 $xP0 -xP1 $xP1 $procArgs


   set c = (`findScale.tcl -in1 $sierraName -in2 $thisFTName -roiArg -x X_AXIS $x1List[$i]ppm $xnList[$i]ppm -y Y_AXIS $y1List[$i]ppm $ynList[$i]ppm`)

   nmrPipe -in $thisFTName -out $thisFTName -inPlace -fn MULT -c $c

   addNMR -in1 $sierraName -in2 $thisFTName -out $sierraName -sub
   addNMR -in1 $sumName    -in2 $thisFTName -out $sumName    -add

   if ($titleFlag) then
      if ($regCount == 1) then
         sethdr $thisFIDName  -title ${tStr}Simulated_Excipient
         sethdr $thisFTName   -title ${tStr}Simulated_Excipient
      else
         sethdr $thisFIDName  -title ${tStr}Simulated_Excipient-$i
         sethdr $thisFTName   -title ${tStr}Simulated_Excipient-$i
      endif
   endif

   if ($cleanFlag) then
      set cleanList = ($cleanList $thisFiltTab $thisFIDName $thisFTName)
   endif

   echo ""
   @ i++
end

/bin/rm -f $cleanList
