#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Adjust Cluster Definitions (CLUSTID and MEMCNT) in a Table:" 
    puts " -in   inName  \[test.tab\]  Input Table or PDB File."
    puts " -out  outName \[clust.tab\] Output Table."
    puts " -x    xList   \[X_PPM\]     List of Table Variables or Locations." 
    puts " -c    cList   \[1.0 ...\]   Scaling Values for Variables, or Keyword Auto."
    puts " -a    aList   \[None\]      Variables Which are Angles in Degrees." 
    puts " -dist dList   \[1.0\]       List of Neighbor Distances." 
    puts " -nc   cCount  \[0\]         Desired Final Cluster Count."
    puts "PDB Options:"
    puts " -pdb                      Input is PDB Data (Default for .pdb)."
    puts " -nopdb                    Input is NMRPipe-Format Table."
    puts " -hetatm                   Interpret HETATMs as ATOMs."
    puts "Notes:"
    puts " 1. Default value for -x is X in PDB Mode."
    puts " 2. Output is always in NMRPipe-Table Format."
    puts " 3. If -nc is 1 or greater, hierarchical clustering will be used."
    exit 1
   }

set inName     test.tab
set outName    clust.tab
set xList      X_PPM
set dList      1.0
set cList      1.0
set finalCount 0 
set aList      None 
set pdbMode    nopdb
set hetMode    nohetatm
set linkMode   max
set verbose    1

getArgD $ARGV -in  inName

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

if {[flagLoc $ARGV -verb]}      {set verbose 1}
if {[flagLoc $ARGV -noverb]}    {set verbose 0}

if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

if {![strcasecmp $pdbMode pdb]} {set xList X}

getArgD $ARGV -out outName
getArgD $ARGV -nc  finalCount

getListArgD $ARGV -x     xList
getListArgD $ARGV -c     cList
getListArgD $ARGV -angle aList
getListArgD $ARGV -dist  dList

#
#------------------------------------------------------------------------------

#
# Cluster aux proc: Returns 1 if entries are neighbors.

proc isNear { eI eJ } \
{
   global vList fList cList dList dN

   for {set i 0} {$i < $dN} {incr i} \
      {
       set v     [lindex $vList $i]
       set d     [lindex $dList $i]
       set c     [lindex $cList $i]
       set aFlag [lindex $fList $i]

       set xI  [gdbGet $eI $v]
       set xJ  [gdbGet $eJ $v]
       set dIJ [expr $c*($xI - $xJ)]

       if {$aFlag} \
          {
           while {$dIJ < -180.0} {set dIJ [expr $dIJ + 360.0]}
           while {$dIJ >  180.0} {set dIJ [expr $dIJ - 360.0]}
          }

       if {abs($dIJ) > $d} {return 0}
      }

   return 1
}

#
# Cluster aux proc: Returns distance between two clusters.

proc findClustDist { eListI eListJ } \
{
   global linkMode

   if {![llength $eListI] || ![llength $eListJ]} {return 0.0}

   set i 0

   if {![strcasecmp $linkMode min]} \
      {
       set bestD 1.0e+16

       foreach eJ $eListJ \
          {
           foreach eI $eListI \
              {
               set d [findDistIJ $eI $eJ]

               if {!$i} {set bestD $d}
               if {$d < $bestD} {set bestD $d}
               incr i
              }
          }
      } \
   elseif {![strcasecmp $linkMode max]} \
      {
       set bestD 0.0

       foreach eJ $eListJ \
          {
           foreach eI $eListI \
              {
               set d [findDistIJ $eI $eJ]

               if {!$i} {set bestD $d}
               if {$d > $bestD} {set bestD $d}
               incr i
              }
          }
      }

   return $bestD
}

proc findDistIJ { eI eJ } \
{
   global prevDistInfo vList fList cList dList dN

   if {[info exists prevDistInfo($eI,$eJ)]} {return $prevDistInfo($eI,$eJ)}

   set thisDist 0.0

   for {set i 0} {$i < $dN} {incr i} \
      {
       set v     [lindex $vList $i]
       set d     [lindex $dList $i]
       set c     [lindex $cList $i]
       set aFlag [lindex $fList $i]

       set xI  [gdbGet $eI $v]
       set xJ  [gdbGet $eJ $v]
       set dIJ [expr $c*($xI - $xJ)]

       if {$aFlag} \
          {
           while {$dIJ < -180.0} {set dIJ [expr $dIJ + 360.0]}
           while {$dIJ >  180.0} {set dIJ [expr $dIJ - 360.0]}
          }

       set thisDist [expr $thisDist + sqrt($dIJ*$dIJ)]
      }

   set prevDistInfo($eI,$eJ) $thisDist

   return $thisDist 
}

#
#------------------------------------------------------------------------------

#
# Test arguments, read input table:

if {![file exists $inName]} \
   {
    puts "Error Finding Input Table $inName"
    exit 1
   }

set xN [llength $xList]
set cN [llength $cList]
set dN [llength $dList]

if {$cN == 1} \
   {
    set c [lindex $cList 0]
    for {set i 1} {$i < $xN} {incr i} {lappend cList $c}
   }

if {$dN == 1} \
   {
    set d [lindex $dList 0]
    for {set i 1} {$i < $xN} {incr i} {lappend dList $d}
   }

set cN [llength $cList]
set dN [llength $cList]

if {$cN != $xN} \
   {
    puts "clustTab Error: -c Scale Value/-x Variable Mismatch."
    exit 1
   }

if {$xN != $dN} \
   {
    puts "clustTab Error: -dist Value/-x Variable Mismatch."
    exit 1
   }

if {![strcasecmp $pdbMode pdb]} \
   {
    set cTab [gdbRead table -in $inName -$pdbMode -$hetMode]
   } \
else \
   {
    set cTab [gdbRead table -in $inName]
   }

set vNames [gdbGetInfo $cTab -parms varNames]
set eList  [gdbGetInfo $cTab -parms entries]

set cListNew ""

for {set i 0} {$i < $xN} {incr i} \
   {
    set c [lindex $cList $i]

    if {![strcasecmp $c Auto]} \
       {
        lappend cListNew [safeDiv 1.0 [getTabStat getRange $cTab [lindex $xList $i]]]
       } \
    else \
       {
        lappend cListNew $c
       }
   }

set cList $cListNew

#
# Tabulate entries and variables:

foreach eI $eList {set clustInfo($eI) $eI}

set eN [llength $eList]
set cN $eN

set xNN    [expr $xN - 1]
set vList  ""

foreach i $xList \
   {
    if {[isInt $i]} \
       {
        if {$i < 0 || $i > $xNN} \
           {
            puts "Error: Bad Table Variable Location $i (0 ... $xNN)."
            exit 1
           }

        lappend vList [lindex $vNames $i]
       } \
    else \
       {
        if {[lsearch -exact $vNames $i] < 0} \
           {
            puts "Error: Finding Table Variable $i"
            exit 1
           } 

        lappend vList $i
       } 
   }

#
# Adjust list of angle flags:

if {![strcasecmp $aList None]} {set aList ""}

set fList ""

foreach v $vList \
   {
    if {[lsearch -exact $aList $v] >= 0} \
       {
        lappend fList 1
       } \
    else \
       {
        lappend fList 0
       }
   }

#
# Add cluster info variables to table if needed:

foreach v {CLUSTID MEMCNT} \
   {
    if {[lsearch -exact $vNames $v] < 0} \
       {
        gdbCreate variable -parent $cTab -name $v -varType int -varFmt %4d
       }
   }

#
# Perform cluster analysis:

if {$finalCount} \
   {
    if {$verbose} \
       {
        puts ""
        puts "Hierarchical Cluster Analysis. Variables: $xList Target Clusters: $finalCount"
       }

    set clustIDList [findHClusters clustInfo $eList findClustDist $finalCount $verbose]

    set usedList ""
    set newID 0
    set i     0

    foreach entry $eList \
       {
        set clustID [lindex $clustIDList $i]
        set memCnt  $clustInfo(memCount,$clustID)

        set loc [lsearch -exact $usedList $clustID]

        if {$loc < 0} \
           {
            lappend usedList $clustID
            set newID [llength $usedList]
           } \
        else \
           {
            set newID [expr $loc + 1]
           }

        gdbSet $entry CLUSTID $newID
        gdbSet $entry MEMCNT  $memCnt
        incr i
       }
   } \
else \
   {
    if {$verbose} \
       {
        puts ""
        puts "Iterative Cluster Analysis. Variables: $xList Distances: $dList"
       }
    
    set clustIDList [findClusters clustInfo $eList isNear $verbose]

    set i 0

    foreach entry $eList \
       {
        set clustID [lindex $clustIDList $i]
        set memCnt  $clustInfo(memCount,$clustID)
    
        gdbSet $entry CLUSTID $clustID
        gdbSet $entry MEMCNT  $memCnt
        incr i
       }
   }

if {$finalCount} \
   {
    gdbAddNote $cTab "DATA  CLUSTER_VARS $vList"
   } \
else \
   {
    for {set i 0} {$i < $dN} {incr i} \
       {
        set v [lindex $vList $i]
        set d [lindex $dList $i]
    
        gdbAddNote $cTab "DATA  CLUSTER $v +/- $d"
       }
   }

gdbWrite $cTab $outName

exit
