#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# cmndargs.tcl:  Modules for command line parsing. These modules expect that
#                the first item in a command line is never a flag, but rather
#                the name of a command, so that a location return value of 0
#                means that the flag was not found.
#
#  flagLoc:      returns the index of the given flag in the command line.
#  isFlag:       returns 1 if the given item is a flag, 0 otherwise.
#  nextFlag:     returns the index of the next flag after a given location.
#  getArgD:      sets a variable if a command argument exists.
#  getArgDN:     sets array variables if command argument series exists.
#  getListArgD:  sets a list if a command argument exists.
#  setArgD:      replace a value if its command argument exists.
#  setListArgD:  replace a list if its command argument exists.
#  setFlagD:     replace a flag if it exists.
#  setFlag:      replace a flag if it exists, add it otherwise.
#  arg2array:    convert argument list to array info.
#  getNamedItem: extract the value found after a given name in a list.
#  expandArg:    expand comma-separated items into flags.
#
#  getListArgDT: sets a -- terminated list if a command argument exists;
#                returns a version of the input argument list with the 
#                sought-for elements removed.

proc flagLoc {thisArgv flagName} \
{
   set loc [lsearch -exact $thisArgv $flagName]

   if {$loc == -1} {return 0}

   return $loc
}

proc isFlag { item } \
{
   if {[string compare [string index $item 0] "-"]} {return 0}
   if {[string first   [string index $item 1] ".0123456789"] >= 0} {return 0}

   return 1
}

proc nextFlag { thisArgv prevFlag } \
{
   set thisArgc [llength $thisArgv]

   for {set i [expr $prevFlag + 1]} {$i < $thisArgc} {incr i} \
      {
       if {[isFlag [lindex $thisArgv $i]]} \
          {
           return $i
          }
      }

   return $thisArgc
}

proc nextFlagTerminator { thisArgv prevFlag } \
{
   set thisArgc [llength $thisArgv]

   for {set i [expr $prevFlag + 1]} {$i < $thisArgc} {incr i} \
      {
       if {![strcmp [lindex $thisArgv $i] "--"]} \
          {
           return $i
          }
      }

   return $thisArgc
}

proc arg2array { thisArgv aaPtr } \
{
   upvar $aaPtr aa

   set n [llength $thisArgv]
   set aa(argv0) ""
   set i 0

   while {$i < $n} \
      {
       set item [lindex $thisArgv $i]

       if {[isFlag $item]} \
          {
           set key [string trimleft $item "+-"]
           set loc [nextFlag $thisArgv $i]
        
           if {$loc == $i + 1} \
              {
               set aa($key) 1
              } \
           else \
              {
               set aa($key) [lrange $thisArgv [expr $i + 1] [expr $loc - 1]]
              }

           set i $loc
          } \
       else \
          {
           set aa(argv$i) $item
           incr i
          }
      }
}

proc getArgDN { thisArgv flagName varPtr n } \
{
   upvar $varPtr var

   if {$n < 1} {return 0}

   if {[info exists var(def)]} \
      {
       set def $var(def)
      } \
   else \
      {
       set def 0 
      }

   for {set i 1} {$i <= $n} {incr i} \
      {
       if {![info exists var($i)]} {set var($i) $def}
      }

   set loc1 [flagLoc $thisArgv $flagName]
   if {!$loc1} {return 0}

   set loc2   [nextFlag $thisArgv $loc1]
   set vList  [lrange $thisArgv [expr $loc1 + 1] [expr $loc2 - 1]]
   set vCount [llength $vList]

   if {$vCount > $n} \
      {
       puts stderr "Notice: unused arguments for $flagName option."
      }

   if {$vCount >= $n} \
      {
       for {set i 1} {$i <= $n} {incr i} \
          {
           set var($i) [lindex $vList [expr $i - 1]]
           if {![strcasecmp $var($i) default]} {set var($i) $var(def)}
          }
      } \
   elseif {$vCount == 1} \
      {
       set def [lindex $vList 0]
       if {![strcasecmp $def default]} {set def $var(def)}
       for {set i 1} {$i <= $n} {incr i} {set var($i) $def}
      } \
   elseif {$vCount < $n} \
      {
       puts stderr "Error: expected $n arguments for $flagName option."
       exit 1
      } 

   return $vCount 
}

proc getArgD {thisArgv flagName varPtr} \
{
   upvar $varPtr var

   set loc [flagLoc $thisArgv $flagName]

   if {$loc} \
      {
       set var [lindex $thisArgv [expr $loc + 1]]
       return $loc
      }

   return 0 
}

proc getListArgD {thisArgv flagName varListPtr} \
{
   upvar $varListPtr varList

   set loc1 [flagLoc $thisArgv $flagName]

   if {!$loc1} {return 0}

   set loc2    [nextFlag $thisArgv $loc1]
   set varList [lrange $thisArgv [expr $loc1 + 1] [expr $loc2 - 1]]

   return [llength $varList]
}

proc getListArgDT { thisArgv flagName varListPtr } \
{
   upvar $varListPtr varList

   set loc1 [flagLoc $thisArgv $flagName]

   if {!$loc1} {return $thisArgv}

   set loc2    [nextFlagTerminator $thisArgv $loc1]
   set varList [lrange $thisArgv [expr $loc1 + 1] [expr $loc2 - 1]]

   set newArgv [concat [lrange $thisArgv 0 [expr $loc1 - 1]] [lrange $thisArgv [expr $loc2 + 1] end]]

   return $newArgv
}

proc setArgD {thisArgvPtr flagName val} \
{
   upvar $thisArgvPtr thisArgv
   
   set loc [flagLoc $thisArgv $flagName]

   if {$loc} \
      {
       set newArgv [lrange $thisArgv 0 $loc]

       lappend newArgv $val

       foreach i [lrange $thisArgv [expr $loc + 2] end] \
          {
           lappend newArgv $i
          } 

       set thisArgv $newArgv 

       return $loc
      }

   return 0
}

proc setListArgD {thisArgvPtr flagName vList} \
{
   upvar $thisArgvPtr thisArgv
   
   set loc [flagLoc $thisArgv $flagName]

   if {$loc} \
      {
       set newArgv [lrange $thisArgv 0 $loc]
       set nextLoc [nextFlag $thisArgv $loc]

       foreach val $vList {lappend newArgv $val}

       foreach i [lrange $thisArgv $nextLoc end] \
          {
           lappend newArgv $i
          } 

       set thisArgv $newArgv 

       return $loc
      }

   return 0
}

proc setFlagD {thisArgvPtr oldFlagName newFlagName} \
{
   upvar $thisArgvPtr thisArgv

   set loc [flagLoc $thisArgv $oldFlagName]

   if {$loc} \
      {
       set newArgv [lrange $thisArgv 0 [expr $loc - 1]]

       lappend newArgv $newFlagName 

       foreach i [lrange $thisArgv [expr $loc + 1] end] \
          {
           lappend newArgv $i
          }

       set thisArgv $newArgv

       return $loc
      }

   return 0
}

proc setFlag {thisArgvPtr oldFlagName newFlagName} \
{
   upvar $thisArgvPtr thisArgv

   set loc [setFlagD thisArgv $oldFlagName $newFlagName]
  
   if {!$loc} \
      {
       set n [llength $thisArgv]

       lappend thisArgv $newFlag

       return $n 
      }

   return 0
}

proc getNamedItem { iName iList } \
{
   set val ""
   set loc [lsearch -exact $iList $iName]

   if {$loc < 0} {return $val}

   set val [lindex $iList [expr $loc + 1]]

   return $val
}

proc getNamedItemD { iName iList defVal } \
{
   set val $defVal 
   set loc [lsearch -exact $iList $iName]

   if {$loc < 0} {return $val}

   set val [lindex $iList [expr $loc + 1]]

   return $val
}

proc expandArg args \
{
   set aList ""

   foreach ssss $args \
      {
       foreach sss [string trim $ssss "{}"] \
          {
           foreach ss [split $sss ","] \
              {
               set s [split $ss "="]

               if {[llength $s] == 1} \
                  {
                   set aList "$aList -[lindex $s 0]"
                  } \
               else \
                  {
                   set aList "$aList -[lindex $s 0] [lrange $s 1 end]"
                  }
              }
          }
      }

   return [string trim $aList]
}

proc expandArg0 args \
{
   set aList ""
   set n 0

   foreach ssss $args \
      {
       foreach sss [string trim $ssss "{}"] \
          {
           foreach ss [split $sss ","] \
              {
               set s [split $ss "="]

               if {$n} \
                  {
                   set c "-"
                  } \
               else \
                  {
                   set c ""
                  }

               if {[llength $s] == 1} \
                  {
                   set aList "$aList $c[lindex $s 0]"
                  } \
               else \
                  {
                   set aList "$aList $c[lindex $s 0] [lrange $s 1 end]"
                  }
              }
          }
      }

   return [string trim $aList]
}

