#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert Coherent 2D Spectroscopy Text Data to NMRPipe Format:"
    puts " -in  inName  \[coh.txt\]   2D Text Input."
    puts " -out outName \[test.dat\]  NMRPipe-Format Output."
    puts " -tmp tmpName \[tmp.txt\]   Temporary Text Output."
    puts "Axis Limits:"
    puts " -wx1 wx1     \[None\]      Wavelength of First Point in X-Axis, nm."
    puts " -wxn wxN     \[None\]      Wavelength of Last Point in X-Axis, nm."
    puts " -wy1 wy1     \[None\]      Wavelength of First Point in Y-Axis, nm."
    puts " -wyn wyN     \[None\]      Wavelength of Last Point in Y-Axis, nm."
    puts "Skipping Vectors from Input:"
    puts " -skip sList  \[None\]      List of Vectors Locations to Skip,"
    puts "                            First Vector is 1." 
    puts " -skipName    \[skip/test%03d.dat\]
    puts "Other Options:"
    puts " -lead                    Leading Number is Row Wavelength (Default)."
    puts " -nolead                  No Row Wavelength Values are Present."
    puts " -base                    Baseline Correction ON (Default)."
    puts " -nobase                  Baseline Correction OFF."
    puts " -verb                    Verbose Mode ON (Default)."
    puts " -noverb                  Verbose Mode OFF."
    puts "Notes:"
    puts " 1. Input text file consists of one row per vector."
    puts " 2. Axis limit arguments -yf1 and -yfN are accepted in place"
    puts "    of -wy1 and -wyn back-compatibility.
    exit 1
   }

set inName   coh.txt
set outName  test.dat
set tmpName  tmp.txt

set wx1      None 
set wxn      None 
set wy1      None 
set wyn      None 

set leadN    1
set skipList None 

set baseFlag 1
set verbose  1

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -tmp tmpName

getArgD $ARGV -xf1 wx1 
getArgD $ARGV -xfN wxn 
getArgD $ARGV -yf1 wy1 
getArgD $ARGV -yfN wyn 

getArgD $ARGV -wx1 wx1
getArgD $ARGV -wxn wxn 
getArgD $ARGV -wy1 wy1 
getArgD $ARGV -wyn wyn 

if {[flagLoc $ARGV -lead]}   {set leadN 1}
if {[flagLoc $ARGV -nolead]} {set leadN 0}

if {[flagLoc $ARGV -base]}   {set baseFlag 1}
if {[flagLoc $ARGV -nobase]} {set baseFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

getListArgD $ARGV -skip skipList

if {![strcasecmp $skipList None]} {set skipList ""}

#
#-----------------------------------------------------------------------------

proc execConv {} \
{
   global xSize ySize xSW ySW wx1 wxn wy1 wyn
   global tmpName outName baseVal baseFlag

   set C_NM_SEC 2.99792e+17

   set xHz [expr $C_NM_SEC/$xSW]
   set yHz [expr $C_NM_SEC/$ySW]

   set xOr [expr $C_NM_SEC/$wxn]
   set yOr [expr $C_NM_SEC/$wyn]

   if {$baseFlag} \
      {
       exec txt2bin.tcl < $tmpName | bin2pipe -noswap \
        -xN            $xSize  -yN            $ySize \
        -xT            $xSize  -yT            $ySize \
        -xMODE           Real  -yMODE           Real \
        -xMETH         Direct  -yMETH         Direct \
        -xFTDOM      Spectral  -yFTDOM      Spectral \
        -xFT             Freq  -yFT             Freq \
        -xSW             $xHz  -ySW             $yHz \
        -xORIG           $xOr  -yORIG           $yOr \
        -xOBS            1.00  -yOBS            1.00 \
        -xCAR            0.00  -yCAR            0.00 \
        -xLAB               X  -yLAB               Y \
        -ndim               2  -aq2D          States \
        | nmrPipe -fn TP | nmrPipe -fn POLY -auto | nmrPipe -fn TP \
        | nmrPipe -fn POLY -auto -ord 0 | nmrPipe -fn TP -nohdr    \
          -out $outName -ov 
      } \
   else \
      {
       exec txt2bin.tcl < $tmpName | bin2pipe -noswap \
        -xN            $xSize  -yN            $ySize \
        -xT            $xSize  -yT            $ySize \
        -xMODE           Real  -yMODE           Real \
        -xMETH         Direct  -yMETH         Direct \
        -xFTDOM      Spectral  -yFTDOM      Spectral \
        -xFT             Freq  -yFT             Freq \
        -xSW             $xSW  -ySW             $ySW \
        -xORIG           $wxn  -yORIG           $wyn \
        -xOBS         1000.00  -yOBS         1000.00 \
        -xCAR            0.00  -yCAR            0.00 \
        -xLAB               X  -yLAB               Y \
        -ndim               2  -aq2D          States \
        | nmrPipe -fn TP -nohdr \
          -out $outName -ov
      }
}

#
#-----------------------------------------------------------------------------

if {![strcmp $inName $outName] || \
     ![strcmp $inName $tmpName] ||
      ![strcmp $tmpName $outName]} \
   {
    puts stderr "Error: Input, Output, and Temporary File Names must be Unique."
    exit 1
   }

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Text File $inName"
    exit 1
   }

set error [catch {set inUnit [open $inName r]}]

if {$error} \
   {
    puts stderr "Error Opening Input Text File $inName"
    exit 1
   }

set error [catch {set tmpUnit [open $tmpName w]}]

if {$error} \
   {
    puts stderr "Error Opening Temporary Output Text File $tmpName"
    exit 1
   }

set thisN      0 
set targetN    0
set lineCount  0
set validCount 0
set skipCount  0

set minList  ""
set maxList  ""
set wList    ""

while {[gets $inUnit line] >= 0} \
   {
    incr lineCount 

    foreach val $line \
       {
        if {$val > 2000} {puts stderr "$lineCount. $val"}
       }

    if {[lsearch -exact $skipList $lineCount] >= 0} \
       {
        incr skipCount
        set skipInfo($skipCount) $line
        continue
       }
 
    set thisN [llength $line]

    if {!$targetN} \
       {
        if {$verbose} \
           {
            puts stderr "Input: $inName  Line $lineCount: $thisN items"
           }

        set targetN $thisN
       }

    if {$thisN != $targetN} \
       {
        if {$verbose} {puts stderr "Warning: Line $lineCount, $thisN items"}
        continue
       }

    if {$leadN} \
       {
        set iList     [lrange $line 1 end]
        lappend wList [lindex $line 0]
       } \
    else \
       {
        set iList $line
       }

    lappend minList  [getMin $iList]
    lappend maxList  [getMax $iList]
 
    puts $tmpUnit $iList

    incr validCount
   }

set xSize   [expr $targetN - $leadN]
set ySize   $validCount

set minVal  [getMin $minList]
set maxVal  [getMax $maxList]

if {![strcasecmp $wx1 None] || ![strcasecmp $wx1 Auto]} \
   {
    if {$leadN} \
       {
        set wx1 [getMin $wList]
       } \
    else \
       {
        set wx1 1
       }
   }

if {![strcasecmp $wxn None] || ![strcasecmp $wxn Auto]} \
   {
    if {$leadN} \
       {
        set wxn [getMax $wList]
       } \
    else \
       {
        set wxn $xSize 
       }
   }

if {![strcasecmp $wx1 None] || ![strcasecmp $wx1 Auto]} {set wx1 1} 
if {![strcasecmp $wxn None] || ![strcasecmp $wxn Auto]} {set wxn $xSize}

if {![strcasecmp $wy1 None] || ![strcasecmp $wy1 Auto]} {set wy1 1} 
if {![strcasecmp $wyn None] || ![strcasecmp $wyn Auto]} {set wyn $ySize}

set wxList "$wx1 $wxn"
set wyList "$wy1 $wyn"

set wx1 [getMin $wxList]
set wxn [getMax $wxList]

set wy1 [getMin $wyList]
set wyn [getMax $wyList]
 
if {$baseFlag} \
   {
    set baseVal [getMedian $minList]
   } \
else \
   {
    set baseVal 0.0
   }

set xSW [expr abs($wxn - $wx1)]
set ySW [expr abs($wyn - $wy1)]

if {$verbose} \
   {
    set fmt " %s-Axis: %6d Pts Wavelength Range: %9.3f to %9.3f SW: %9.3f"

    puts stderr [format $fmt X $xSize $wx1 $wxn $xSW]
    puts stderr [format $fmt Y $ySize $wy1 $wyn $ySW]
    puts stderr ""
       
    puts stderr [format " Minimum Value:  %.4e" $minVal]
    puts stderr [format " Maximum Value:  %.4e" $maxVal]

    puts stderr ""
   }

catch {file delete $outName}
catch {close $tmpUnit}
catch {execConv}
catch {file delete $tmpName}

if {![file exists $outName]} \
   {
    puts stderr "Error Creating Output $outName" 
   } \
else \
   {
    if {$verbose} {puts stderr "Created NMRPipe-Format Output $outName"}
   }

exit 0
