#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert Euler Angles to Alternate Framework:"
    puts " -euler    eList \[0 0 0 ...\]   Triplets of Euler Angles to Convert."
    puts " -inFrame  iList \[X Y Z\]       Framework for Input Angles."
    puts " -outFrame oList \[Z Y Z\]       Desired Framework for Output Angles."
    puts " -min      mList \[-180 0 -180\] Desired Minimums for Output Angles."
    puts " -max      mList \[180 180 180\] Desired Maximums for Output Angles."
    puts " -iseed    is    \[54321\]       Random Number Seed."
    puts " -trials   n     \[100\]         Number of Trials."
    puts " -verb                           Verbose Mode ON."
    puts " -rot                            Report Rotation Matrix Values."
    exit 1
   }

set inEulerList "0.0 0.0 0.0"
set minList     "-180.0 -180 -180.0"
set maxList     "180.0 180.0 180.0"
set inFrame     "X Y Z"
set outFrame    "Z Y Z"

set iseed      54321
set gc         100
set verbose    1
set rotFlag    0

getArgD $ARGV -iseed   iseed
getArgD $ARGV -trials  gc

getListArgD $ARGV -euler    inEulerList
getListArgD $ARGV -inFrame  inFrame 
getListArgD $ARGV -outFrame outFrame
getListArgD $ARGV -min      minList
getListArgD $ARGV -max      maxList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -rot]}   {set rotFlag 1}
if {[flagLoc $ARGV -norot]} {set rotFlag 0}

set inFrame  [string toupper $inFrame]
set outFrame [string toupper $outFrame]

#
#---------------------------------------------------------------------------

proc isFrame { f } \
{
   if {[llength $f] != 3} {return 0}

   foreach a $f \
      {
       if {[strcmp $a X] && [strcmp $a Y] && [strcmp $a Z]} {return 0}
      }

   return 1
}

proc inLimits { eulerTriplet } \
{
   global minList maxList

   foreach loc {0 1 2} \
      {
       set omega  [lindex $eulerTriplet $loc]
       set minVal [lindex $minList $loc]
       set maxVal [lindex $maxList $loc]

       if {$omega < $minVal || $omega > $maxVal} {return 0}
      }

   return 1
}

proc getRotMat { eulerTriplets frameTriplet } \
{
   dynTransform -reset

   set n   [llength $eulerTriplets]
   set nT  [expr int($n/3)]
   set k   0

   for {set i 0} {$i < $nT} {incr i} \
      {
       foreach j {0 1 2} \
          {
           set omega [lindex $eulerTriplets $k]
           set f     [lindex $frameTriplet  $j]

           incr k

           switch -exact $f \
              {
               X {dynTransform -rx $omega}
               Y {dynTransform -ry $omega}
               Z {dynTransform -rz $omega}
              }
          }
      }

   set mList [dynTransform -verb] 
   set m "" 

   foreach i {0 1 2 4 5 6 8 9 10} \
      {
       lappend m [lindex $mList $i]
      }

   return $m
}

proc rotDevProc {} \
{
   global outFrame bestScore bestEuler verbose
   global pList qList yModel id reportFlag

   set rx [lindex $pList 0]
   set ry [lindex $pList 1]
   set rz [lindex $pList 2]

   set yModel [getRotMat $pList $outFrame]
   set dev    [getVsStat getRMS $yModel $qList]

   if {$dev < $bestScore} \
      {
       set bestEuler [format "% 8.3f % 8.3f % 8.3f" $rx $ry $rz]
       set bestScore $dev
      }

   if {$reportFlag && $verbose} \
      {
       set fmt " Trial: %3d  Rot: % 8.3f % 8.3f % 8.3f Dev: %8.6f"
       puts [format $fmt $id $rx $ry $rz $dev]
      }
}

proc randEulerTriplet {} \
{
   global minList maxList

   set eList ""

   foreach i {0 1 2} \
      {
       set minVal [lindex $minList $i]
       set maxVal [lindex $maxList $i]
       set r      [expr $maxVal - $minVal]

       lappend eList [expr $maxVal - $r*[rand]]
      }

   return $eList
}

#
#------------------------------------------------------------------------

if {[llength $minList] != 3 || [llength $maxList] != 3} \
   {
    puts stderr "Error in Length of Euler Min/Max Lists."
    exit 1
   }

if {![isFrame $inFrame] || ![isFrame $outFrame]} \
   {
    puts stderr "Error in Euler Frame Lists."
    exit 1
   }

set bestEuler "0.0 0.0 0.0"
set bestScore 1.0e+16
set tol       1.0e-8

srand $iseed

set y [getRotMat $inEulerList $inFrame]

for {set i 1} {$i <= $gc} {incr i} \
    {
     srand [expr $i + $i*11 + $iseed]

     set p [randEulerTriplet]

     set ans \
      [modelXY -y $y -p $p -pLo $minList -pHi $maxList -macro rotDevProc \
               -gc 1 -tol $tol -iter 2000 -maxf 2000 -id $i -noverb]
    }

if {$gc > 0} \
   {
    if {$verbose} \
       {
        set fmt " Best Value. Rot($outFrame): %s Dev: %8.6f"
        
        puts ""
        puts [format $fmt $bestEuler $bestScore]
        puts ""
       } \
    else \
       {
        puts [format "%s %.6f" $bestEuler $bestScore]
       }
   }

if {$rotFlag} \
   {
    if {$verbose} \
       {
        puts "Rotation Matrix:"

        set k 0

        for {set i 0} {$i < 3} {incr i} \
           {
            for {set j 0} {$j < 3} {incr j} \
                {
                 puts -nonewline [format " % .6f" [lindex $y $k]]
                 incr k
                }

            puts ""
           }
       } \
    else \
       {
        puts $y
       }
   }

exit 0
