#
# Return simple ./ version of a file name if it is the same as the full path.

proc fixLocalName { thisName } \
{
   if {![strcasecmp $thisName None]} {return $thisName}

   set tailName  [file tail $thisName]
   set fullName  [file normalize $thisName]
   set localName [file normalize ./$tailName]

   if {![strcmp $localName $fullName]} {set thisName ./$tailName}

   return $thisName
}

#
# Remove ./ from the start of a file name.

proc stripDotSlash { thisName } \
{
   set c1 [string index $thisName 0]
   set c2 [string index $thisName 1]

   if {![strcmp $c1 "."] && ![strcmp $c2 "/"]} {set thisName [string range $thisName 2 end]}

   set c1 [string index $thisName 0]
   set c2 [string index $thisName 1]

   if {![strcmp $c1 "."] && ![strcmp $c2 "/"]} {set thisName [string range $thisName 2 end]}

   return $thisName
}

proc stripWhitespace { s } \
{
   set n    [string length $s]
   set sNew ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $s $i]
       if {[string is space $c]} continue
       append sNew $c
      }

   return $sNew
}

#
# Replace {} [] with () and " with ' to avoid parsing errors.

proc fixTCLString { str } \
{
   set n [string length $str]
   set q 0

   set res ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $str $i]

       if {"$c" == "\{" || "$c" == "\["} \
          {
           append res "("
          } \
       elseif {"$c" == "\}" || "$c" == "\]"} \
          {
           append res ")"
          } \
       elseif {"$c" == "\""} \
          {
           append res "'"
          } \
       else \
          {
           append res $c
          }
      }

   return $res
}

#
# Create a float array from values in rList:

proc listMalloc { rList } \
{
   set n [llength $rList]
   if {$n < 0} {return ""}

   set error [catch {set rPtr [sysMalloc $n float]}]
   if {$error} {return ""}

   set i 0

   foreach r $rList \
      {
       setPtrVal $rPtr $i $r
       incr i
      }
 
   return $rPtr
}

#
# Create a float array from values in rList:

proc tabMalloc { t varList } \
{
   if {![strcmp $varList "*"]} {set varList [gdbGetInfo $t -parms varNames]}

   set eList      [gdbGetInfo $t -parms entries]

   set varCount   [llength $varList]
   set entryCount [llength $eList]

   set n [expr $varCount*$entryCount]

   set error [catch {set mPtr [sysMalloc $n float]}]
   if {$error} {return ""}

   set k 0

   foreach e $eList \
      {
       foreach v $varList \
          {
           setPtrVal $mPtr $k [gdbGet $e $v]
           incr k
          }
      }

   return $mPtr
}

#
# Determine if a command is on the path:

proc cmndExists { cmnd } \
{
   set s ""

   set error [catch {set s [exec which $cmnd]}] 

   if {![info exists s]}  {return 0}
   if {[llength $s] != 1} {return 0}

   if {[file exists [lindex $s 0]]} {return 1}

   return 0
}

#
# Determine if a line begins with the words in prefix.

proc lineBegins { prefix line } \
{
   set i 0

   foreach item $prefix \
      {
       set key [lindex $line $i]
       if {[strcmp $item $key]} {return 0}
       incr i
      }

   return 1
}

#
# Catch error when formatting a single value:

proc safeFormat { fmt val } \
{
   set error [catch "format $fmt $val" result]
   if {$error || ![info exists result]} {return $val}

   return $result
}

proc safeFltFormat { fmt val } \
{
   if {![string is integer $val] && ![string is double $val]} {set val 0.0}

   return [safeFormat $fmt $val]
}

#
# Interpret boolean:

proc valIsYes { val } \
{
   foreach key {1 y yes true} \
      {
       if {![strcasecmp $val $key]} {return 1}
      }

   return 0
}

proc valIsNo { val } \
{
   foreach key {0 n no false} \
      {
       if {![strcasecmp $val $key]} {return 1}
      }

   return 0
}

#
# Replace all oldStr with newStr in tOrig.

proc replaceStr { tOrig oldStr newStr } \
{
   set t     $tOrig
   set m     [string length $oldStr]
   set count [string length $t]

   for {set i 0} {$i <= $count} {incr i} \
      {
       set loc [string first $oldStr $t]
       if {$loc < 0} {return $t}

       set n [string length $t]

       set headStr ""
       set tailStr ""

       if {$loc > 0} {set headStr [string range $t 0 [expr $loc - 1]]}

       incr loc $m

       if {$loc < $n - 1} {set tailStr [string range $t $loc end]}

       set t $headStr$newStr$tailStr
      }

   return $t
}

#
# Replace instances of characters in str.

proc replaceChars { str oldChars newChar } \
{
   set n      [string length $str]
   set newStr ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $str $i]
       set j [string first $c $oldChars]

       if {$j >= 0} \
          {
           append newStr $newChar
          } \
       else \
          {
           append newStr $c
          }
      }

   return $newStr
}

#
# Add text to a debug output.

proc dbgMsg { dbgName msg } \
{
   if {[file exists $dbgName]} \
      {
       set error [catch {set outUnit [open $dbgName a]}]
       catch "seek $outUnit 0 end"
      } \
   else \
      {
       set error [catch {set outUnit [open $dbgName w]}]
      }

   catch "puts $outUnit $msg"

   catch "close $outUnit"
}

#
# Print all lines in a file.

proc printFile { outUnit fName } \
{
   if {![file exists $fName]} {return -1}

   set error [catch {set inUnit [open $fName r]}]
   if {$error} {return -2}

   fconfigure $inUnit -translation lf

   set n 0

   while {[gets $inUnit line] >= 0} \
      {
       puts $outUnit $line
       incr n
      }

   catch {close $inUnit}

   return $n
}

#
# Return items in text file as a flat list.

proc getTextList { fName } \
{
   set iList ""

   if {![file exists $fName]} {return $iList}

   set error [catch {set inUnit [open $fName r]}]
   if {$error} {return $iList}

   while {[gets $inUnit line] >= 0} \
      {
       foreach item $line {lappend iList [string trim $item]} 
      }

   catch {close $inUnit}

   return $iList
}

#
# Reads lines of text into the given array, along with info.

proc getCSVLines { fName fArrayPtr hdrFlag } \
{
   upvar $fArrayPtr fArray

   set fArray(rowCount) 0
   set fArray(colCount) 0
   set fArray(delList)  ""
   set fArray(newList)  ""
   set fArray(hdr)      ""

   if {![file exists $fName]} {return -1}

   set error [catch {set inUnit [open $fName r]}]
   if {$error} {return -2}

   set rowCount 0
   set colCount 0

   while {[gets $inUnit line] >= 0} \
      {
       set str    [string trim $line]
       set nChars [string length $str]
       set key    [getToken 1 $str]
       set c      [string index $key 0]
       set nCol   [cntToken $str ","]

       if {!$nChars || ![strcmp $c "#"]} continue

       if {$colCount == 0} {set colCount $nCol}

       for {set j 0} {$j < $nCol} {incr j} {set fArray($rowCount,$j) [string trim [getToken [expr $j + 1] $str ","]]}

       incr rowCount 
      }

   if {$rowCount} \
      {
       if {$hdrFlag} \
          {
           for {set j 0} {$j < $colCount} {incr j} \
               {
                lappend fArray(hdr) $fArray(0,$j)
                set fArray(hdr,$j)  $fArray(0,$j)
               }
          } \
       else \
          {
           for {set j 0} {$j < $colCount} {incr j} \
               {
                lappend fArray(hdr) VAR$j
                set fArray(hdr,$j)  VAR$j
               }
          }
      }

   for {set i 0} {$i < $rowCount} {incr i} \
      {
       for {set j 0} {$j < $colCount} {incr j} \
          {
           set var $fArray(hdr,$j)
           set fArray($i,$var) $fArray($i,$j) 
          }
      }

   set fArray(rowCount) $rowCount 
   set fArray(colCount) $colCount 
  
   catch "close $inUnit"
 
   return $rowCount 
}

proc putCSVLines { fName fArrayPtr } \
{
   upvar $fArrayPtr fArray

   if {![info exists fArray(rowCount)] || ![info exists fArray(colCount)]} {return -1}

   set error [catch {set outUnit [open $fName w]}]
   if {$error} {return -2}

   for {set j 0} {$j < $fArray(colCount)} {incr j} {set nMax($j) 0}

   for {set i 0} {$i < $fArray(rowCount)} {incr i} \
      {
       for {set j 0} {$j < $fArray(colCount)} {incr j} \
          {
           set n [string length $fArray($i,$j)]

           if {[llength $fArray($i,$j)] > 1} {incr n 2}
           if {$j < $fArray(colCount) - 1}   {incr n 1} 

           if {$n > $nMax($j)} {set nMax($j) $n}
          }
      }

   for {set i 0} {$i < $fArray(rowCount)} {incr i} \
      {
       if {[lsearch -exact $fArray(delList) $i] >= 0} continue

       for {set j 0} {$j < $fArray(colCount)} {incr j} \
          {
           if {$j < $fArray(colCount) - 1} \
              {
               set fmt " %-$nMax($j)s"
               set c   ","
              } \
           else \
              {
               set fmt " %s"
               set c ""
              }

           if {[llength $fArray($i,$j)] > 1} \
              {
               puts -nonewline $outUnit [format $fmt \"$fArray($i,$j)\"$c]
              } \
           else \
              {
               puts -nonewline $outUnit [format " %-$nMax($j)s" $fArray($i,$j)$c]
              }
          }

       puts $outUnit ""
      }

   close $outUnit

   return 0
}

#
# Reads lines of text into the given array, along with info.

proc getTextLines { fName fArrayPtr } \
{
   upvar $fArrayPtr fArray

   set fArray(lineCount) 0
   set fArray(minWords)  0
   set fArray(maxWords)  0
   set fArray(maxLength) 0

   if {![file exists $fName]} {return -1}

   set error [catch {set inUnit [open $fName r]}]
   if {$error} {return -2}

   set i 0

   while {[gets $inUnit line] >= 0} \
      {
       set s [string trim $line]
       set m [string length $s]
       set n [llength $s]

       if {!$i} \
          {
           set fArray(minWords)  $n
           set fArray(maxWords)  $n
           set fArray(maxLength) $m
          }

       if {$n < $fArray(minWords)}  {set fArray(minWords)  $n}
       if {$n > $fArray(maxWords)}  {set fArray(maxWords)  $n}
       if {$m > $fArray(maxLength)} {set fArray(maxLength) $m}

       set fArray($i) $s
       incr i
      }

   catch {close $inUnit}

   set fArray(lineCount) $i

   return $i
}

#
# Count the number of lines in a text file.

proc cntLines { fName } \
{
   if {![file exists $fName]} {return -1}

   set error [catch {set inUnit [open $fName r]}]
   if {$error} {return -2}

   set i 0

   while {[gets $inUnit line] >= 0} {incr i}

   catch {close $inUnit}

   return $i
}

#
# Extract line count etc from text file.

proc getTextFileInfo { fName lineCountPtr minWordPtr maxWordPtr maxLenPtr } \
{
    upvar $lineCountPtr lineCount
    upvar $minWordPtr   minWords
    upvar $maxWordPtr   maxWords
    upvar $maxLenPtr    maxLength

    set lineCount  0 
    set minWords   0
    set maxWords   0
    set wordCount  0
    set maxLength  0

   if {![file exists $fName]} {return -1}

   set error [catch {set inUnit [open $fName r]}]
   if {$error} {return -2}

   set lineCount 0

   while {[gets $inUnit line] >= 0} \
      {
       set n [string length $line]
       if {$n > $maxLength} {set maxLength $n}

       set n [llength $line]
       if {$n > $maxWords} {set maxWords $n}

       if {!$lineCount}    {set minWords $n}
       if {$n < $minWords} {set minWords $n}

       incr wordCount $n
       incr lineCount 
      }

   catch {close $inUnit}

   return $wordCount
}

#
# getNthLine: extract the Nth line from the given file (first line = 1).

proc getNthLine { fName n } \
{
   if {![file exists $fName]} {return ""}

   set error [catch {set inUnit [open $fName r]}]
   if {$error} {return ""}

   set i 0

   while {[gets $inUnit line] >= 0} \
      {
       incr i
       if {$i == $n} {return $line}
      }

   catch {close $inUnit}

   return ""
}

#
# getNthCol: extract the Nth column from the given file (first col = 1).

proc getNthCol { fName n } \
{
   if {![file exists $fName]} {return ""}

   set error [catch {set inUnit [open $fName r]}]
   if {$error} {return ""}

   set iList ""
   incr n -1

   while {[gets $inUnit line] >= 0} \
      {
       lappend iList [lindex $line $n]
      }

   catch {close $inUnit}

   return $iList 
}

#
# parseTags: parse text with <> tags.

proc parseTags { str arrayPtr stripChars } \
{
   upvar $arrayPtr tagInfo

   set tagInfo(tagCount) 0

   set n [string length $str]
   set tagCount 0

   for {set i 0} {$i < $n} {incr i} \
      {
       set ci [string index $str $i]
       if {[string first $ci $stripChars] >= 0} continue

       if {![strcmp $ci "<"]} \
          {
           incr tagCount
           set tagInfo($tagCount) $ci 

           for {set j [expr $i + 1]} {$j < $n} {incr j} \
              {
               set cj [string index $str $j]
               if {[string first $cj $stripChars] >= 0} continue

               append tagInfo($tagCount) $cj 

               if {![strcmp $cj ">"]} break
              }

           set i $j 
          } \
       else \
          {
           incr tagCount

           set tagInfo($tagCount) $ci

           for {set j [expr $i + 1]} {$j < $n} {incr j} \
              {
               set cj [string index $str $j]
               if {[string first $cj $stripChars] >= 0} continue

               if {![strcasecmp $cj "<"]} \
                  {
                   incr j -1
                   break
                  }

               append tagInfo($tagCount) $cj
              }

           set i $j 
          }
      }

   set tagInfo(tagCount) $tagCount

   return $tagCount
}

proc getTagData { tagOrig type } \
{
   set val ""

   set tag [string trim $tagOrig "</>"] 
   set loc [string first [string tolower $type]= [string tolower $tag]]
   if {$loc < 0} {return $val}

   set nTag  [string length $tag]
   set nType [string length $type]
 
   for {set i [expr $loc + $nType + 1]} {$i < $nTag} {incr i} \
      {
       set c [string index $tag $i]
       if {[string is space $c]} break 
       append val $c
      }

   return $val
}

proc prettyPrintList { outUnit prefix vList } \
{
   set nWords  [llength $vList]
   set nChars  0

   foreach v $vList \
      {
       set n [string length $v]
       if {$n > $nChars} {set nChars $n}
      }

   set nCols [expr (78 - [string length $prefix])/($nChars + 2)]
   if {$nCols < 1} {set nCols 1}

   set i 0

   while {$i < $nWords} \
      {
       puts -nonewline $outUnit $prefix

       for {set j 0} {$j < $nCols} {incr j} \
          {
           puts -nonewline $outUnit [format " %-${nChars}s" [lindex $vList $i]]
           if {$i > $nWords - 1} break

           incr i 
          }

       puts $outUnit ""
      }
}

proc prettyPrintCmnd { outUnit prefix cmnd } \
{
   set nWords  [llength $cmnd]
   set nChars  0
   set nBase   0

   for {set i 0} {$i < $nWords} {incr i} \
      {
       set thisItem [lindex $cmnd $i]
       set nextItem [lindex $cmnd [expr $i + 1]]

       if {!$i} \
          {
           set thisItem "$prefix[file tail $thisItem]"
           set nBase    [string length $thisItem]
           if {![string length $prefix]} {incr nBase 2}
          }

       puts -nonewline $outUnit "$thisItem "

       incr nChars [string length "$thisItem "]

       if {[isFlag $thisItem] && ![isFlag $nextItem]} continue

       if {$nChars > 50 && $i < $nWords - 1} \
          {
           puts $outUnit "\\"
           puts -nonewline $outUnit $prefix

           for {set j 0} {$j < $nBase - 1} {incr j} {puts -nonewline $outUnit " "} 

           set nChars 0
          }
      }

   puts $outUnit ""
}
 
proc expand { s } \
{
   set n [string length $s]
   set sList ""

   for {set i 0} {$i < $n} {incr i} \
      {
       lappend sList [string index $s $i]
      }

   return $sList
}

proc isInt { s } \
{
   set n [string length $s]
   if {!$n} {return 0}

   for {set i 0} {$i < $n} {incr i} \
      {
       set c   [string index $s $i]
       set loc [string first $c "-0123456789"]

       if {$loc < 0} {return 0}

       if {!$loc} \
          {
           if {$i || $n < 2} {return 0}
          }
      }
 
   return 1
}

proc intOnly { s } \
{
   set v ""

   set n [string length $s]

   for {set i 0} {$i < $n} {incr i} \
      {
       set c   [string index $s $i]
       set loc [string first $c "0123456789"]

       if {$loc >= 0} {append v $c}
      }

   return $v
}

proc parseNum { val signPtr decPtr dotPtr fracPtr ePtr } \
{
   upvar $signPtr sign
   upvar $decPtr  dec
   upvar $dotPtr  dot
   upvar $fracPtr frac
   upvar $ePtr    e

   set sign ""
   set dec  ""
   set dot  ""
   set frac ""
   set e    ""

   set v [string tolower $val]
   set n [string length $v]

   if {!$n} {return 0} 

   set c [string index $v 0]

   if {![strcmp $c "-"] || ![strcmp $c "+"]} \
      {
       set sign $c
       set v    [string range $v 1 end]
      }

    set vInfo [split $v "."]

    set v1    [lindex $vInfo 0]
    set v2    [lindex $vInfo 1]

    set dec   [intOnly $v1]
    set nDec  [string length $dec]

    if {[llength $vInfo] == 1} {return $nDec}

    set dot   "."

    if {![llength $v2]} {return $nDec}
     
    set eInfo [split $v2 "e"]
    set e1    [lindex $eInfo 0]
    set e2    [lindex $eInfo 1]
    
    set frac  [intOnly $e1]
    set nFrac [string length $frac]

    if {[llength $eInfo] == 1} {return [expr $nDec + $nFrac]}

    scan $e2 %d e

    return [expr $nDec + $nFrac]
}

#
# Removes %

proc removePct { s } \
{
   set n  [string length $s]
   set ss ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $s $i]

       if {![strcmp $c %]} {set c "-"} 

       append ss $c 
      }

   return $ss
}

#
# Removes null items.

proc removeNullElements { w } \
{
   set s ""

   foreach v $w \
      {
       if {![llength $v]} continue
       lappend s $v
      }

   return $s
}

#
# Returns list with nth element removed.

proc removeElement { w n } \
{
   set s ""
   set i 0

   foreach v $w \
      {
       if {$i != $n} {lappend s $v}
       incr i
      }

   return $s
}

#
# Returns a list with n repeats of item. 

proc lrepeat { item n } \
{
   set iList ""

   for {set i 0} {$i < $n} {incr i} {lappend iList $item}

   return $iList 
}

#
# Returns a list of all items in iList that match patterns in pList

proc listMatch { iList pList } \
{
   set mList ""

   foreach p $pList \
      {
       foreach i $iList \
          {
           if {[string match $p $i]} \
              {
               if {[lsearch -exact $mList $i] < 0} {lappend mList $i}
              }
          }
      }

   return $mList
}

#
# Returns 1 if wI and wJ contain the same unique members.

proc sameList { wI wJ } \
{
   foreach i $wI \
     {
      if {[lsearch -exact $wJ $i] < 0} {return 0}
     }

   foreach j $wJ \
     {
      if {[lsearch -exact $wI $j] < 0} {return 0}
     }

   return 1
}

#
# Repeat str n times.

proc repeatStr { str n } \
{
   set s ""

   for {set i 0} {$i < $n} {incr i} {append s $str}

   return $s
}

#
# Returns counts of items in uList found in sList.

proc listCounts { uList sList } \
{
   set nList ""
   
   foreach u $uList \
      {
       set n 0

       foreach s $sList \
          {
           if {![strcmp $u $s]} {incr n}
          }

       lappend nList $n
      }

   return $nList
}

#
# Returns list in reverse order.

proc listReverse { w } \
{
   set n [llength $w]
   set q ""

   for {set i [expr $n - 1]} {$i >= 0} {incr i -1} \
      {
       lappend q [lindex $w $i]
      }

    return $q
}

#
# Returns elements of list in s1 following occurance of s2.

proc listFollowing { s1 s2 } \
{
   set loc [lsearch -exact $s1 $s2]

   if {$loc < 0} {return ""}

   return [lrange $s1 [expr $loc + 1] end]
}

#
# Returns unique list elements of w.

proc listUniq { w } \
{
   set u ""

   foreach i $w \
      {
       if {[lsearch -exact $u $i] < 0} {lappend u $i}
      }

   return $u
}

proc listUniqCount { w } \
{
   set u ""

   foreach i $w \
      {
       if {[lsearch -exact $u $i] < 0} {lappend u $i}
      }

   return [llength $u]
}

#
# Returns union of unique elements in wI and unique elements in wJ.

proc listUnion { wI wJ } \
{
   set wU [listUniq $wI]

   foreach j $wJ \
      {
       if {[lsearch -exact $wU $j] < 0} {lappend wU $j}
      }

   return $wU
}

#
# Returns unique elements common to wI and wJ.

proc listIntersect { wI wJ } \
{
   set uI [listUniq $wI]
   set uJ [listUniq $wJ]
   set x  ""

   foreach i $uI \
      {
       if {[lsearch -exact $uJ $i] >= 0} {lappend x $i}
      }

   return $x
}

#
# Returns unique elements in wI but not in wJ.

proc listDisjoint { wI wJ } \
{
   set dI [listUniq $wI]
   set x  ""

   foreach i $dI \
      {
       if {[lsearch -exact $wJ $i] < 0} {lappend x $i}
      }

   return $x
}

#
# Returns all items as a single list. Cheating text version.

proc listFlatten args \
{
    regsub -all {[{}]} $args {} tmp   
    regsub -all { +} $tmp { }   tmp  

    set fList [string trim $tmp]

    return $fList
}

#
# Form simple permutations, for example of {1 2 3}.

proc simplePermutation { sList } \
{
   set m [llength $sList]

   if {$m < 2} {return $sList} 

   set pList ""
   set i 0

   foreach s $sList \
      {
       set qList ""
       set j 0

       foreach ss $sList \
          {
           if {$i != $j} {lappend qList $ss}
           incr j
          } 

       set rList [simplePermutation $qList]

       foreach r $rList {lappend pList "$s $r"}
       incr i
      }

   return $pList
}

#
# Form simple permutations of length N.

proc simplePermutationN { sList n } \
{
   set m [llength $sList]

   if {$m < 2 || $n < 2} {return $sList} 

   set pList ""
   set i 0

   foreach s $sList \
      {
       set qList ""
       set j 0

       foreach ss $sList \
          {
           if {$i != $j} {lappend qList $ss}
           incr j
          } 

       set rList [simplePermutationN $qList [expr $n - 1]]

       foreach r $rList {lappend pList "$s $r"}
       incr i
      }

   return $pList
}

#
# Form complex permutations, for example of: A|B,C,D|E|F 

proc permList { qStr } \
{
    set qList [split $qStr ","]
    set count [llength $qList]

    if {$count == 1} \
       {
        set hList [split $qList "|"]
        return $hList
       } \
    else \
       {
        set head  [lindex $qList 0]
        set tail  [lrange $qList 1 end]

        set hList [split $head "|"]
        set tList [permList [join $tail ","]]

        foreach i $hList \
           {
            foreach j $tList \
               {
                lappend pList $i,$j
               }
           }
       }

    return $pList
}

#
# Iterate permutations of a set of lists i.e, {{1 2 3} {A B C}}

proc iterPermList { qList } \
{
    set count [llength $qList]
    set pList ""

    if {$count == 1} \
       {
        set hList [lindex $qList 0]
        return $hList
       } \
    else \
       {
        set hList  [lindex $qList 0]
        set tList  [iterPermList [lrange $qList 1 end]]

        foreach i $hList \
           {
            foreach j $tList \
               {
                lappend pList "$i $j"
               }
           }
       }

    return $pList
}
 
proc storeArrayDB { dirName aList } \
{
   if {![file exists dirName]} {file mkdir $dirName}
   puts ""

   foreach a $aList \
      {
       eval "global $a"

       puts -nonewline [format "Storing Array %-10s " $a:]
       flush stdout
       set n [storeArray $dirName/$a.info $a]

       if {$n < 0} {return 1}

       puts [format " %6d Entries." $n]
      }

   return 0
}

proc loadArrayDB { dirName } \
{
   if {![file exists db]} \
      {
       puts stderr "Error Finding Array Database $dirName"
      }

   puts ""

   set aList [lsort -dictionary [glob -nocomplain $dirName/*.info]]

   foreach aInfo $aList \
      {
       set a [file root [file tail $aInfo]]

       eval "global $a"

       puts -nonewline [format "Loading Array %-10s " $a:]
       flush stdout
       set n [loadArray $dirName/$a.info $a]
       puts [format " %6d Entries." $n]
      }
}

proc loadArray { inName aPtr } \
{
   upvar $aPtr a

   set error [catch {set inUnit [open $inName r]}]

   set nn  0
   set nnn 0

   if {$error} \
      {
       puts stderr ""
       puts stderr "Error Opening $aPtr Input Data in $inName"
       puts ""
       return -1
      }

   while {[gets $inUnit line] >= 0} \
      {
       incr nnn

       if {[llength $line] != 2} \
          {
           puts stderr "Error Loading Index Data for $aPtr in $inName:"
           puts stderr " Line $nnn:"
           puts stderr $line
           puts ""
           return -1
          }

       set i [lindex $line 0]
       set n [lindex $line 1]

       set a($i) ""

       for {set j 0} {$j < $n} {incr j} \
          {
           if {[gets $inUnit v] < 0} \
              {
               puts stderr "Error Loading Data Item for $aPtr in $inName:"
               puts stderr " Index: $i" 
               puts stderr " Line $nnn, Index $i"
               puts ""
               return -1 
              }

           incr nnn

           lappend a($i) $v 
          }

       incr nn
      } 

   close $inUnit

   return $nn
}

proc extractArray { aPtr } \
{
   upvar $aPtr a

   set id    [array startsearch a]
   set aInfo ""

   while {[array anymore a $id]} \
      {
       set i [array nextelement a $id]

       lappend aInfo $i
       lappend aInfo $a($i)
      }

   array donesearch a $id

   return $aInfo
}

proc storeArray { outName aPtr } \
{
   upvar $aPtr a

   set error [catch {set outUnit [open $outName w]}]
   set nn 0

   if {$error} \
      {
       puts stderr ""
       puts stderr "Error Saving $aPtr Data in $outName"
       puts ""
       return -1 
      }

   set id [array startsearch a]

   while {[array anymore a $id]} \
      {
       set i [array nextelement a $id]
       set n [llength $a($i)]

       puts $outUnit "$i $n"

       foreach v $a($i) {puts $outUnit $v}

       incr nn
      }

   array donesearch a $id

   close $outUnit
 
   return $nn
}

#
# Star Format Modules ----------------------------------------------------------

proc getStarLoop { inUnit infoPtr } \
{
   upvar $infoPtr starInfo

   set loopCount $starInfo(loopCount)
   set loc 0

   while {[gets $inUnit line] >= 0} \
      {
       incr starInfo(lineCount)

       set n   [string length $line]
       set key ""

       scan $line %s key

       set lc [format %4d $starInfo(lineCount)]

       if {!$n || "$key" == ""} \
          {
           if {$starInfo(verbose)} \
            {
             puts stderr "STAR $lc Process Loop: Blank. $line"
            }

           continue 
          }

       if {![strcasecmp $key "stop_"]} \
          {
           if {$starInfo(verbose)} \
              {
               puts stderr "STAR $lc Terminate Loop: $line"
              }

           break
          }

       if {$starInfo(verbose)} \
          {
           puts stderr \
            "STAR [format %4d $starInfo(lineCount)] Process Loop: $line"
          }

       set j 1

       foreach vName $starInfo($loopCount,vList) \
          {
           set starInfo($loopCount,$loc,$vName) [getTokenS $j $line]
           incr j
          }

       incr loc
      }

    set starInfo($loopCount,lineCount) $loc
}

proc getStarLines { inUnit infoPtr } \
{
   upvar $infoPtr starInfo

   set str ""
   set c   ""
   set lc  0

   while {[gets $inUnit line] >= 0} \
      {
       incr starInfo(lineCount)

       set n   [string length $line]
       set key ""

       scan $line %s key

       if {!$n || "$key" == ""} continue 

       set c [string index $key 0]

       if {![strcmp $c ";"]} break
      }

   if {[strcmp $c ";"]} {return $str} 

   while {[gets $inUnit line] >= 0} \
      {
       incr starInfo(lineCount)

       set key ""
       scan $line %s key

       set c [string index $key 0]

       if {![strcmp $c ";"]} break

       if {$lc} \
          {
           append str " $line"
          } \
       else \
          {
           append str $line
          }

       incr lc
      }

   return $str
}

proc getStarVList { inUnit infoPtr } \
{
   upvar $infoPtr starInfo

   set vList ""

   while {[gets $inUnit line] >= 0} \
      {
       incr starInfo(lineCount)

       set n   [string length $line]
       set key ""

       scan $line %s key

       if {!$n || "$key" == ""} break

       set c [string index $key 0]

       if {![strcmp $c "#"]} continue

       lappend vList $key
      }

   return $vList
}

proc readStar { inUnit infoPtr } \
{
   upvar $infoPtr starInfo 

   if {![info exists starInfo(verbose)]} {set starInfo(verbose) 0}

   set starInfo(lineCount) 0
   set starInfo(loopCount) 0

   set starInfo(default)   ""
   set starInfo(comment)   ""
   set starInfo(vList)     ""

   set curType     None
   set curCatagory None
   set loopCount   0

   while {[gets $inUnit line] >= 0} \
      {
       incr starInfo(lineCount)

       set n   [string length $line]
       set key ""

       scan $line %s key

       if {!$n || "$key" == ""} \
          {
           if {$starInfo(verbose)} \
              {
               puts stderr \
                "STAR [format %4d $starInfo(lineCount)] Blank. $line"
              }

           continue
          }

       set c [string index $key 0]

       if {![strcmp $c "#"]} \
          {
           if {$starInfo(verbose)} \
              {
               puts stderr \
                "STAR [format %4d $starInfo(lineCount)] Comment: $line"

               lappend starInfo(comment) $line
              }

           continue
          }

       switch -glob [string tolower $key] \
          {
           _saveframe_category \
              {
               if {$starInfo(verbose)} \
                  {
                   puts stderr \
                    "STAR [format %4d $starInfo(lineCount)] Saveframe: $line"
                  }

               set curCatagory [lindex $line 1]
              }

           loop_ {
                  if {$starInfo(verbose)} \
                     {
                      puts stderr \
                       "STAR [format %4d $starInfo(lineCount)] Process Loop:"
                     }

                  set starInfo($loopCount,type)      $curType
                  set starInfo($loopCount,catagory)  $curCatagory

                  set starInfo($loopCount,vList) \
                   [getStarVList $inUnit starInfo] 

                  set starInfo($loopCount,lineCount) 0

                  if {$starInfo(verbose)} \
                     {
                      puts -nonewline stderr \
                       "STAR [format %4d $starInfo(lineCount)]"

                      puts stderr " Loop Start: $starInfo($loopCount,vList)"
                     }

                  getStarLoop $inUnit starInfo

                  if {$starInfo(verbose)} \
                     {
                      puts -nonewline stderr \
                       "STAR [format %4d $starInfo(lineCount)]"

                      puts stderr \
                       " Loop End, $starInfo($loopCount,lineCount) lines."
                     }

                  incr loopCount

                  set starInfo(loopCount) $loopCount
                 }

           save_  \
            {
             if {$starInfo(verbose)} \
                {
                 puts stderr \
                  "STAR [format %4d $starInfo(lineCount)] Blank Save: $line"
                }

             set curType None
            }

           save_* \
            {
             if {$starInfo(verbose)} \
                {
                 puts stderr \
                  "STAR [format %4d $starInfo(lineCount)] Generic Save: $line"
                }

             set curType [lindex $line 0]
            }

           _* {
               set vName [lindex $line 0] 

               if {[llength $line] == 1} \
                  {
                   lappend starInfo($vName,val) [getStarLines $inUnit starInfo]
                  } \
               else \
                  {
                   lappend starInfo($vName,val) \
                    [string trim [lrange $line 1 end] \"\']
                  }

                if {$starInfo(verbose)} \
                   {
                    puts -nonewline stderr \
                     "STAR [format %4d $starInfo(lineCount)] Variable $vName:"

                    puts stderr " $starInfo($vName,val)"
                   }

               lappend starInfo(vList) $vName
              }

            default \
             {
              if {$starInfo(verbose)} \
                 {
                  puts stderr \
                   "STAR [format %4d $starInfo(lineCount)] Default: $line"
                 }

              lappend starInfo(default) $line
             }
          }
      }

   return $loopCount
}

#
# Cluster Analysis Modules ... -------------------------------------------------

#
# Returns 1 if there are any neighbors between two clusters.

proc isClusterOverlap { cInfoPtr locI locJ } \
{
    upvar $cInfoPtr clustInfo

    foreach i $clustInfo(locList,$locI) \
       {
        foreach j $clustInfo(locList,$locJ) \
           {
            set nearFlag \
             [eval $clustInfo(nearProc) $clustInfo(val,$i) $clustInfo(val,$j)]

            if {$nearFlag} {return 1}
           }
       }

   return 0
}

#
# Perform one iteration to search for and combine clusters.

proc scanClusters { cInfoPtr } \
{
   upvar $cInfoPtr clustInfo

   set clustInfo(adjustments) 0
   set clustInfo(clusters)    0

   for {set i 0} {$i < $clustInfo(N) - 1} {incr i} \
      {
       if {![llength $clustInfo(locList,$i)]} continue

       for {set j [expr $i + 1]} {$j < $clustInfo(N)} {incr j} \
          {
           if {![llength $clustInfo(locList,$j)]} continue

           if {[isClusterOverlap clustInfo $i $j]} \
              {
               set clustInfo(locList,$i) \
                [listUnion $clustInfo(locList,$i) $clustInfo(locList,$j)]

               set clustInfo(locList,$j) ""

               incr clustInfo(adjustments) 
              }
          }
      }

   for {set i 0} {$i < $clustInfo(N)} {incr i} \
      {
       if {[llength $clustInfo(locList,$i)]} {incr clustInfo(clusters)}
      }

   return $clustInfo(adjustments)
}

#
# Called to find clusters.

proc findHClusters { cInfoPtr valList nearProc targetCount verbose } \
{
   upvar $cInfoPtr clustInfo

   catch "unset clustInfo"

   set clustInfo(N) [llength $valList]

   set clustInfo(nearProc)    $nearProc
   set clustInfo(adjustments) 0
   set clustInfo(clusters)    0
   set clustInfo(clustIDUsed) ""

   set valCount  [llength $valList]
   set clustList ""
   set clustID   1

   set nn 0
   set i  0

   foreach val $valList \
      {
       set clustInfo(val,$i)      $val
       set clustInfo(valList,$i)  $val
       set clustInfo(locList,$i)  $i
       set clustInfo(clustID,$i)  $clustID

       set clustInfo(memCount,$clustID) 1

       incr clustInfo(clusters)

       incr clustID
       incr i
      }

   while {$clustInfo(clusters) > $targetCount} \
      {
       set bestD 1.0e+16
       set bestI 0
       set bestJ 0
       set n     0

       for {set i 0} {$i < $valCount - 1} {incr i} \
          {
           if {![llength $clustInfo(locList,$i)]} continue

           for {set j [expr $i + 1]} {$j < $valCount} {incr j} \
              {
               if {![llength $clustInfo(locList,$j)]} continue

               set d [$nearProc $clustInfo(valList,$i) $clustInfo(valList,$j)]

               if {!$n || $d < $bestD} \
                  {
                   set bestI $i
                   set bestJ $j
                   set bestD $d
                  }
               
               incr n
              }
          }

       if {!$n} break

       incr nn

       set i $bestI
       set j $bestJ

       if {$verbose} \
          {
           puts "$nn. Joining Cluster $clustInfo(clustID,$i) and $clustInfo(clustID,$j)"
           puts " $clustInfo(locList,$i) $clustInfo(locList,$j)"
          }

       set clustInfo(locList,$i) [concat $clustInfo(locList,$i) $clustInfo(locList,$j)]
       set clustInfo(valList,$i) [concat $clustInfo(valList,$i) $clustInfo(valList,$j)]

       set clustInfo(locList,$j) ""
       set clustInfo(valList,$j) ""

       incr clustInfo(clusters) -1 
      }

   for {set i 0} {$i < $valCount} {incr i} \
      {
       set n [llength $clustInfo(locList,$i)]

       if {!$n} continue

       set clustID $clustInfo(clustID,$i)

       set clustInfo(memCount,$clustID) $n 

       foreach j $clustInfo(locList,$i) {set clustInfo(clustID,$j) $clustID}

       if {[lsearch -exact $clustInfo(clustIDUsed) $clustID] < 0} \
          {
           lappend clustInfo(clustIDUsed) $clustID
          }
      }

  for {set i 0} {$i < $valCount} {incr i} {lappend clustList $clustInfo(clustID,$i)} 

  return $clustList
}

proc findClusters { cInfoPtr valList nearProc verbose } \
{
   upvar $cInfoPtr clustInfo

   catch "unset clustInfo"

   set clustInfo(N) [llength $valList]

   set clustInfo(nearProc)    $nearProc
   set clustInfo(adjustments) 0
   set clustInfo(clusters)    0

   set clustID 1
   set i 0

   foreach val $valList \
      {
       set clustInfo(val,$i)      $val
       set clustInfo(locList,$i)  $i
       set clustInfo(clustID,$i)  $clustID 

       set clustInfo(memCount,$clustID) 1

       incr clustInfo(clusters)

       incr clustID
       incr i
      }

   for {set i 1} {$i < 2*$clustInfo(N)} {incr i} \
      {
       set n   [scanClusters clustInfo] 
       set fmt "  Iteration: %2d Adjustments: %3d Clusters: %4d"

       if {$verbose} {puts stderr [format $fmt $i $n $clustInfo(clusters)]}
   
       if {!$n} break
      }

   set clustList ""
   set clustID   1

   for {set i 0} {$i < $clustInfo(N)} {incr i} \
      {
       if {![llength $clustInfo(locList,$i)]} continue

       set clustInfo(memCount,$clustID) [llength $clustInfo(locList,$i)]
       set clustInfo(members,$clustID)  $clustInfo(locList,$i)

       foreach j $clustInfo(locList,$i) \
          {
           set clustInfo(clustID,$j) $clustID
          }

       incr clustID
      }

   set clustInfo(clustIDUsed) ""

   for {set i 0} {$i < $clustInfo(N)} {incr i} \
      {
       set clustID $clustInfo(clustID,$i)

       lappend clustList $clustID

       if {[lsearch -exact $clustInfo(clustIDUsed) $clustID] < 0} \
          {
           lappend clustInfo(clustIDUsed) $clustID 
          }
      }

   return $clustList
}

proc getTokenS { n s } \
{
   set nn [string length $s]

   set wasSpace 1
   set count    0

   for {set i 0} {$i < $nn} {incr i} \
      {
       while {[string is space [string index $s $i]] && $i < $nn} {incr i}

       if {$i > $nn - 1} {return ""}

       set c [string index $s $i]

       incr count

       if {![strcmp $c '] || ![strcmp $c \"]} \
          {
           incr i

           set j  $i
           set qc $c

           while {[strcmp [string index $s $j] $qc] && $j < $nn} {incr j}
          } \
       else \
          {
           set j $i

           while {![string is space [string index $s $j]] && $j < $nn} {incr j}
          }

       if {$count == $n} \
          {
           return [string range $s $i [expr $j - 1]]
          }

       set i $j
      }

   return "" 
}

proc cntTokenS { s } \
{
   set nn [string length $s]

   set wasSpace 1
   set count    0

   for {set i 0} {$i < $nn} {incr i} \
      {
       while {[string is space [string index $s $i]] && $i < $nn} {incr i}

       if {$i > $nn - 1} {return $count}

       set c [string index $s $i]

       incr count

       if {![strcmp $c '] || ![strcmp $c \"]} \
          {
           incr i

           set j  $i
           set qc $c

           while {[strcmp [string index $s $j] $qc] && $j < $nn} {incr j}
          } \
       else \
          {
           set j $i

           while {![string is space [string index $s $j]] && $j < $nn} {incr j}
          }

       set i $j
      }

   return $count 
}

proc getStackTrace {} \
{
   set stack "Stack Trace:\n"

   for {set i 1} {$i < [info level]} {incr i} \
      {
       set lvl   [info level -$i]
       set pname [lindex $lvl 0]

       append stack [string repeat " " $i]$pname

       foreach value [lrange $lvl 1 end] arg [info args $pname] \
          {
           if {$value eq ""} {info default $pname $arg value}
           append stack " $arg='$value'"
          }

       append stack "\n"
      }

    return $stack
}
