#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#--------------------------------------------------------------

set inName  None
set pdbName None
set segName None

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Convert XPLOR Dipolar Coupling Input to DYNAMO Format."
    puts " -in  inName    XPLOR Input (Dipolar Information Only)."
    puts " -pdb pdbName   Corresponding PDB (for Sequence and Atom Names)."
    puts " -seg segName   Default Seg Name."
    exit 0
   }

getArgD $ARGV -in  inName
getArgD $ARGV -pdb pdbName
getArgD $ARGV -seg segName

#
#--------------------------------------------------------------

proc parseDCAss { assID line } \
{
   parseXPLORAss $assID $line 6 assList tail

   set assI  [lindex $assList 4]
   set assJ  [lindex $assList 5]

   set d     [lindex $tail 0]

   set assText $assI,$assJ
   set pList   [permList $assText]
   set groupID 1
 
   foreach p $pList \
      {
       printDCAss $assID $groupID $p $d
       incr groupID
      }
}

proc getDCWeight { resIDI resNameI atomI resIDJ resNameJ atomJ ddPtr wPtr } \
{
   upvar $ddPtr dd
   upvar $wPtr  w 

   set resDif [expr $resIDI - $resIDJ]
   set dd     0.0
   set w      0.0

   switch -exact -- $resDif \
      {
       "-1" {
             switch -glob $atomI,$atomJ \
                {
                 "C,HN" {
                         set dd 0.333 
                         set w  3.000 
                        }

                 "C,N" {
                        set dd 0.125
                        set w  8.000 
                       }

                 "H*,P" -
                 "P,H*" {
                         set dd 1.000
                         set w  1.000
                        }
                }
            }

       "0"  {
             switch -glob $atomI,$atomJ \
                {
                 "HN,N" -
                 "N,HN" {
                         set dd 1.0
                         set w  1.0
                        }

                 "H*,C*" -
                 "C*,H*" {
                          set dd 1.0
                          set w  0.5 
                         }

                 "C*,C*" {
                          set dd 0.2 
                          set w  5.0
                         }
                 "H*,H*" {
                          set dd 0.333
                          set w  3.000 
                         }

                 "H*,P" -
                 "P,H*" {
                         set dd 1.000
                         set w  1.000
                        }
                }
            }

       "1"  {
             switch -glob $atomI,$atomJ \
                {
                 "HN,C" {
                         set dd 0.333 
                         set w  3.000 
                        }

                 "N,C" {
                        set dd 0.125
                        set w  8.000 
                       }

                 "H*,P" -
                 "P,H*" {
                         set dd 1.000
                         set w  1.000
                        }
                }
            }
      } 

   if {$w == 0.0} \
      {
       puts stderr "DC Unknown Coupling Type: $resIDI $atomI, $resIDJ $atomJ"

       set dd 1.0
       set w  1.0
      }
}

proc printDCAss { assID groupID p d } \
{
   global seqInfo

   set str [string toupper [split $p "~,"]]

   set resIDI    [lindex $str 0]
   set atomNameI [lindex $str 1]
   set segNameI  [lindex $str 2]
   set resNameI  $seqInfo($resIDI,$segNameI)
   set atomListI [expandXPLORAtom $resIDI $atomNameI $segNameI]
   set atomListI $atomNameI

   set resIDJ    [lindex $str 3]
   set atomNameJ [lindex $str 4]
   set segNameJ  [lindex $str 5]
   set resNameJ  $seqInfo($resIDJ,$segNameJ)
   set atomListJ [expandXPLORAtom $resIDJ $atomNameJ $segNameJ]
   set atomListJ $atomNameJ

   foreach atomI $atomListI \
      {
   foreach atomJ $atomListJ \
      {
       puts -nonewline [format "%4d %2d" $assID $groupID]
       puts -nonewline [format " %3d %-4s %-4s" $resIDI $resNameI $atomI]
       puts -nonewline [format " %-4s" $segNameI]
       puts -nonewline [format " %3d %-4s %-4s" $resIDJ $resNameJ $atomJ]
       puts -nonewline [format " %-4s" $segNameJ]

       getDCWeight $resIDI $resNameI $atomI $resIDJ $resNameJ $atomJ dd w

       puts [format " % 8.3f % 8.3f % 8.3f" $d $dd $w]
      }
      }
}

proc printDCHdr {} \
{
   puts -nonewline "VARS INDEX GROUP"
   puts -nonewline " RESID_I RESNAME_I ATOMNAME_I SEGNAME_I"
   puts -nonewline " RESID_J RESNAME_J ATOMNAME_J SEGNAME_J"
   puts -nonewline " D DD W"
   puts ""

   puts -nonewline "FORMAT %3d %3d"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %9.3f %9.3f %9.3f"
   puts ""
   puts ""
}

proc parseDC {} \
{
   global inUnit

   set itemCount 0
   set assInfo($itemCount) ""

   while {[gets $inUnit line] >= 0} \
      {
       set keyword [lindex $line 0]

       if {"$keyword" == "assign"} \
          {
           set line [lrange $line 1 end]
           set line [stripXPLORComment $line]

           incr itemCount

           wm title . "Reading $itemCount"
           update
          }

       append assInfo($itemCount) " $line"
      }

   for {set assID 1} {$assID <= $itemCount} {incr assID} \
      {
       wm title . "Interpreting $assID/$itemCount"
       update

       set result ""
       set error  0

       set error [catch {parseDCAss $assID $assInfo($assID)}]

       if {$error} \
          {
           puts stderr "DYNAMO Warning: Error interpreting Restraint $assID:"
           puts stderr "  Error Code: $error $result"
           puts stderr "  Line: $assInfo($assID)"
          }
      }
}

if {![file exists $inName]} \
   {
    puts stderr "Error finding input restraint file $inName"
    exit 1
   }

if {![file exists $pdbName]} \
   {
    puts stderr "Error finding input PDB file $pdbName"
    exit 1
   }

dynInit -fast

showDynLogo .

wm title . "Reading $pdbName"
update

set inUnit [open $inName r]

getXPLORPDBInfo $pdbName

wm title . "Reading $inName"
update

printDCHdr
parseDC

exit
