#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# dcEval: show a PDB structure's agreement with dipolar couplings and shifts.
#-------------------------------------------------------------------------------

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

dynInit

gdbInit
gdbCreate dBase

wm iconify .

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: "
    puts " Show a PDB structure's agreement with dipolar couplings and shifts."
    puts "  -dir   dirName    \[.\]          Default Data Directory."
    puts "  -pdb   pdbName    \[ref.pdb\]    PDB Structure File."
    puts "  -dc    dcNameList \[dObs*.tab\]  Dipolar Coupling Tables."
    puts "  -cs    csName     \[csObs.tab\]  Chemical Shift Table."
    puts "  -out   outName    \[None\]       Optional Output by Residue."
    puts "  -nohdr                         Suppress GDB Header in Output Table."
    exit 0
   }

set pdbName   None
set outName   None
set dNameList None
set csName    None
set dirName   .
set hdrFlag   1

getListArgD $ARGV -dc  dNameList
getArgD     $ARGV -dir dirName
getArgD     $ARGV -pdb pdbName
getArgD     $ARGV -cs  csName
getArgD     $ARGV -out outName

if {[flagLoc $ARGV -hdr]}   {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]} {set hdrFlag 0}

if {"$pdbName" == "None"} \
   {
    set pdbName $dirName/ref.pdb
   }

if {"$csName" == "None"} \
   {
    set csName $dirName/csObs.tab
   }

if {"$dNameList" == "None"} \
   {
    set dNameList [glob -nocomplain $dirName/dObs*.tab]
   }

#
# Find dipolar coupling agreement by residue:

proc sumDC { dTab tensor } \
{
   global dSum minResID maxResID outName

   if {"$outName" == "None"} return

   set entry [gdbFirst $dTab]

   while {$entry} \
      {
       set resIDI [gdbGet $entry RESID_I]
       set resIDJ [gdbGet $entry RESID_J]
       set d      [gdbGet $entry D_DIFF]
       set w      [gdbGet $entry W]

       if {$resIDI < $resIDJ} \
          {
           set resID $resIDJ
          } \
       else \
          {
           set resID $resIDI
          }

       if {$resID < $minResID} {set minResID $resID}
       if {$resID > $maxResID} {set maxResID $resID}

       if {$d < 999.0} \
          {
           lappend dSum($tensor,$resID) [expr $d*$w]
          }
       
       set entry [gdbNext $entry]
      }
}

#
# Find chemical shift agreement by residue:

proc sumCS { csTab } \
{
   global csSum minResID maxResID outName

   if {"$outName" == "None"} return

   set entry [gdbFirst $csTab]

   while {$entry} \
      {
       set resID [gdbGet $entry RESID]
       set cs    [gdbGet $entry CS_DIFF]
       set sigma [gdbGet $entry SIGMA]

       if {$resID < $minResID} {set minResID $resID}
       if {$resID > $maxResID} {set maxResID $resID}

       if {$cs < 999.0} \
          {
           lappend csSum($resID) [expr $cs/$sigma]
          }

       set entry [gdbNext $entry]
      }
}

#
# Report DC and CS agreement by residue.

proc reportResidueSums {} \
{
   global dSum csSum minResID maxResID tensorList hdrFlag outName
   global pdbName dirName dData csData

   if {"$outName" == "None"} return

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       puts stderr "Error opening output file $outName"
       exit 1
      }

   if {$hdrFlag} \
      {
       puts $outUnit "REMARK Dipolar Coupling and Chemical Shift Residuals"
       puts $outUnit "REMARK $pdbName $dirName"
       puts $outUnit ""
       puts $outUnit "DATA DC $dData"
       puts $outUnit "DATA CS [lindex $csData 0]"
       puts $outUnit ""
 
       puts -nonewline $outUnit "VARS   RESID"

       foreach tensor $tensorList \
          {
           puts -nonewline $outUnit " DC_$tensor   "
          }

       puts $outUnit " CS"

       puts -nonewline $outUnit "FORMAT %4d"

       foreach tensor $tensorList \
          {
           puts -nonewline $outUnit "   %8.3f"
          }

       puts $outUnit "   %8.3f"
       puts $outUnit ""
      }

   for {set resID $minResID} {$resID <= $maxResID} {incr resID} \
      {
       puts -nonewline $outUnit [format " %4d"   $resID]

       foreach tensor $tensorList \
          {
           if {[info exists dSum($tensor,$resID)]} \
              {
               set q [getRMS $dSum($tensor,$resID)]
              } \
           else \
              {
               set q 9999.0
              }

           puts -nonewline $outUnit [format " %8.3f" $q]
          }

       if {[info exists csSum($resID)]} \
          {
           set q [getRMS $csSum($resID)]
          } \
       else \
          {
           set q 9999.0
          }

       puts $outUnit [format " %8.3f" $q]
      }

   close $outUnit
}

#
# Perform the simulations for each of one or more alignment tensors:

set minResID  999999
set maxResID -999999

set tensorChoices "A B C D E"
set tensorList    ""
set i 0

set dData  ""
set csData ""

foreach dName $dNameList \
   {
    set tensor [lindex $tensorChoices $i]

    lappend tensorList $tensor

    catch {exec DC -pdb $pdbName -inD $dName -inCS $csName \
                   -outD $dirName/d$tensor.tmp.tab -outCS $dirName/cs.tmp.tab \
                   -euler -verb}

    set csTab [gdbRead table -in $dirName/cs.tmp.tab]
    set dTab  [gdbRead table -in $dirName/d$tensor.tmp.tab]

    sumDC $dTab  $tensor
    sumCS $csTab

    catch {unset dInfo}
    catch {unset csInfo}

    getKeyData $dTab  dInfo
    getKeyData $csTab csInfo
     
    lappend dData  $dInfo(RMS)
    lappend csData [expr $csInfo(Chi2)/$csInfo(N)]

    incr i
   }
 
puts "$dData [lindex $csData 0]"

reportResidueSums

exit
