#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Estimate Effect of Noise on Dipolar Coupling Parameters:"
    puts " $argv0 \[options\] \[-DC dcOptions\]"
    puts " -in     inName  \[dObsA.tab\] Name of Input DC Table."
    puts " -pdb    pdbList \[ref.pdb\]   List of Input PDB Files."
    puts " -trials nT      \[50\]        Number of Trials."
    puts " -iseed  rand    \[54321\]     Initial Random Seed (Zero for Auto)."
    puts " -DC     dcArgs  \[-svd\]      Arguments for DC Calculation."
    puts "Options for Randomizing PDB Coords (Angtroms):" 
    puts " -aSigma aStdDev \[0.1\]       Std Deviation of Coord Noise to Add."
    puts "Options for Randomizing DC Values:"
    puts " -dSigma dStdDev \[1.0\]       Std Deviation of Noise to Add."
    puts " -var    tabVar  \[D\]         Name of Table Variable to Adjust."
    puts " -nvar   nVar    \[DD\]        Table Variable with Std Dev of Noise,"
    puts "                             To Be Scaled by Sigma Above." 
    puts "Temporary Files:"
    puts " -tmpTab tName   \[dcNoise_tmp.tab\]      Temporary Output Table."
    puts " -tmpPDB pName   \[dcNoise%03d_tmp.pdb\]  Temporary Output PDB Template."
    puts "Notes:"
    puts " 1. Use zero values for -dSigma or -aSigma to turn off"
    puts "    randomization of DC values or PDB coords."
    puts " 2. Use -DC arguments to restrain tensor parameters as desired;"
    puts "    try DC -help for a list of options."
    puts " 3. Arguments for -DC must be the last ones on the command line."
    exit 0
   }

set dcName      dObsA.tab 
set dcTitle     dObsA.tab 
set pdbNameList ref.pdb
set pdbTitle    ref.pdb

set tmpDCName  dcNoise_tmp.tab
set tmpPDBName dcNoise%03d_tmp.pdb
set dcArgs     "-svd"
set noiseVar   DD 
set tabVar     D

set dSigma     1.0
set aSigma     0.1 
set iseed      54321
set trials     50 
set verbose    0

getArgD $ARGV -in       dcName
getArgD $ARGV -in       dcTitle
getArgD $ARGV -inTitle  dcTitle

getListArgD $ARGV -pdb  pdbNameList

getArgD $ARGV -pdb      pdbTitle
getArgD $ARGV -pdbTitle pdbTitle

getArgD $ARGV -tmpTab tmpDCName
getArgD $ARGV -tmpPDB tmpPDBName
getArgD $ARGV -var    tabVar
getArgD $ARGV -nvar   noiseVar

getArgD $ARGV -dSigma dSigma 
getArgD $ARGV -aSigma aSigma 
getArgD $ARGV -trials trials
getArgD $ARGV -iseed  iseed

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

set loc [flagLoc $ARGV -DC]

if {$loc} {set dcArgs [lrange $ARGV [expr $loc + 1] end]}

#
#--------------------------------------------------------------------

if {![file exists $dcName]} \
   {
    puts stderr "DC Noise Error Finding DC Input File $dcName"
    exit 1
   }

if {$verbose} {puts stderr "DC Noise Initializing ..."}

catch "file delete $tmpDCName"

dynInit -fast

set pdbSrc ""
set ii 0

foreach pdbName $pdbNameList \
   {
    incr ii

    set thisName [format $tmpPDBName $ii]
    catch "file delete $thisName"

    if {![file exists $pdbName]} \
       {
        puts stderr "DC Noise Error Finding PDB Input File $pdbName"
        exit 1
       }

    if {$verbose} {puts stderr " Reading Original PDB Input: $pdbName ..."}
    lappend pdbSrc [dynRead -pdb -in $pdbName]
   }

if {$verbose} {puts stderr " Reading Original DC Input:  $dcName ..."}

set dcSrc  [dynRead -dc -in $dcName]
set dInfo  [eval "dynSimulate -dc $dcSrc -src $pdbSrc -verb $dcArgs"]

set rxRef  [lindex $dInfo 6]
set ryRef  [lindex $dInfo 7]
set rzRef  [lindex $dInfo 8]

if {$dSigma == 0.0} {file copy -force $dcName $tmpDCName} 

if {$aSigma == 0.0} \
   {
    set ii 0

    foreach pdbName $pdbNameList \
       {
        incr ii
        set thisName [format $tmpPDBName $ii]
        file copy -force $pdbName $thisName
       }
   } 

for {set i 0} {$i < $trials} {incr i} \
   {
    if {$verbose} {puts stderr "DC Noise Trial [expr $i + 1] ..."}

    if {$dSigma != 0.0} \
       {
        exec addTabNoise.tcl -in $dcName -out $tmpDCName -sigma $dSigma -var $tabVar -nvar $noiseVar -iseed $iseed -noverb
        incr iseed 13

        if {$verbose} {puts stderr " Reading Adjusted DC Input:  $tmpDCName ..."}

        dynFree -dc  $dcSrc 
        set dcSrc [dynRead -dc  -in $tmpDCName]
       }

    if {$aSigma != 0.0} \
       {
        dynFree -pdb $pdbSrc 

        set pdbSrc ""
        set ii 0

        foreach pdbName $pdbNameList \
           {
            incr ii
            set thisName [format $tmpPDBName $ii]

            exec addPDBNoise.tcl -in $pdbName -out $thisName -tab None -sigma $aSigma -iseed $iseed -noverb
            incr iseed 13

            if {$verbose} {puts stderr " Reading Adjusted PDB Input: $thisName ..."}
            lappend pdbSrc [dynRead -pdb -in $thisName]
           }
       }

    set dInfo  [eval "dynSimulate -dc $dcSrc -src $pdbSrc -verb $dcArgs"]

    set rms [lindex $dInfo 0]
    set da  [lindex $dInfo 3]
    set dr  [lindex $dInfo 4]

    if {$verbose} {puts stderr " RMS: $rms Da: $da Dr: $dr"}

    if {$da == 0.0} \
       {
        set rh 0.0
       } \
    else \
       { 
        set rh [expr abs($dr/$da)]
       }

    set rx  [lindex $dInfo 6]
    set ry  [lindex $dInfo 7]
    set rz  [lindex $dInfo 8]

    set rotAdj [getDCRotNearest $rxRef $ryRef $rzRef $rx $ry $rz]

    lappend sInfo(DC_RMS) $rms
    lappend sInfo(Da)     $da
    lappend sInfo(Dr)     $dr
    lappend sInfo(Rh)     $rh

    lappend sInfo(ROTATION_X)  [lindex $rotAdj 0]
    lappend sInfo(ROTATION_Y)  [lindex $rotAdj 1] 
    lappend sInfo(ROTATION_Z)  [lindex $rotAdj 2] 
   }

catch "file delete $tmpDCName"

set ii 0

foreach pdbName $pdbNameList \
   {
    incr ii

    set thisName [format $tmpPDBName $ii]
    catch "file delete $thisName"
   }

puts "REMARK Dipolar Coupling Error Analysis for $dcTitle and $pdbTitle"
puts ""
puts "#"
puts "#  INDEX      is the trial number."
puts "#  DC_RMS     is the RMS of W*(observed - calculated) couplings."
puts "#  DA and DR  are the tensor axial and rhombic parameters Da and Dr."
puts "#  RH         is the Rhombicty = Dr/Da."
puts "#  RX RY RZ   are the angles that rotate the PDB onto the alignment frame."
puts ""

if {$dSigma == 0.0} \
   {
    puts "DATA ANALYSIS_MODE Structural Noise: $aSigma Angstroms"
   } \
elseif {$aSigma == 0.0} \
   {
    puts "DATA ANALYSIS_MODE DC Noise: $dSigma times DC Uncertainty"
   } \
else \
   {
    puts "DATA ANALYSIS_MODE DC Noise: $dSigma times DC Uncertainty. Structural Noise: $aSigma Angstroms"
   }

puts "DATA DC_OPTIONS $dcArgs"
puts ""

set fmtList { "%7.3f" "%7.3f" "% 8.3e" "% 8.3e" }
set i 0

set parmFmt "%-6s "

foreach parm {DC_RMS Rh Da Dr} \
   {
    puts -nonewline [format "DATA STAT $parmFmt" $parm]

    set fmt [lindex $fmtList $i]

    foreach stat {Min Max Avg StdDev} \
       {
        set statInfo($parm,$stat) [format $fmt [get$stat $sInfo($parm)]]

        puts -nonewline " [string toupper $stat]"
        puts -nonewline " $statInfo($parm,$stat)"
       }
  
    if {![strcasecmp $parm Rh]} \
       {
        set parmFmt "%-3s"
        puts ""
       }

    puts ""
    incr i
   }

puts ""

set labList {Min Max Avg StdDev}

foreach parm {ROTATION_X ROTATION_Y ROTATION_Z} \
   {
    puts -nonewline "DATA STAT $parm "
    set i 0

    foreach stat {Min Max AvgAngle StdDevAngle} \
       {
        set statInfo($parm,$stat) [format "% 7.2f" [get$stat $sInfo($parm)]]
        set lab [lindex $labList $i]

        puts -nonewline " [string toupper $lab]"
        puts -nonewline " $statInfo($parm,$stat)"
        incr i
       }

    puts ""
   }

puts ""
puts "VARS   INDEX DC_RMS DA   DR   RH   RX    RY    RZ"
puts "FORMAT %4d   %8.3f  %.4e %.4e %.4f %8.2f %8.2f %8.2f"
puts ""

set fmt " %4d %8.3f  % .4e % .4e  % .4f % 8.2f % 8.2f % 8.2f"

for {set i 0} {$i < $trials} {incr i} \
   {
    set rms [lindex $sInfo(DC_RMS) $i]
    set da  [lindex $sInfo(Da)  $i]
    set dr  [lindex $sInfo(Dr)  $i]
    set rh  [lindex $sInfo(Rh)  $i]

    set rx  [lindex $sInfo(ROTATION_X) $i]
    set ry  [lindex $sInfo(ROTATION_Y) $i]
    set rz  [lindex $sInfo(ROTATION_Z) $i]

    puts [format $fmt [expr $i + 1] $rms $da $dr $rh $rx $ry $rz]
   }

exit 0
