#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
puts "Cross-Validation of Dipolar Coupling Parameters:"
puts " $argv0 \[options\] \[-DC dcOptions\]"
puts " -in       inName   \[dObsA.tab\]  Name of Input DC Table."
puts " -pdb      pdbList  \[ref.pdb\]    List of Input PDB Files."
puts " -out      outName  \[stdout\]     Name for Output Result."
puts " -trials   nT       \[50\]         Number of Trials."
puts " -keep     keepFrac \[0.9\]        Approximate Fraction of Data to Keep."
puts " -iseed    rand     \[54321\]      Initial Random Seed (Zero for Auto)."
puts " -DC       dcArgs   \[-svd\]       Arguments for DC Calculation."
puts " -selTab   sName    \[seltmp.tab\] Temporary Selection Output Table."
puts " -delTab   dName    \[deltmp.tab\] Temporary Deletion Output Table."
puts " -selByRes                       Select by Residue ID."
puts " -noselByRes                     Select by Individual Coupling (Default)."
puts " -all                            Use Systematic Selection; Overrides"
puts "                                 Values for -trials and -keep Options."
puts " -noclean                        Do Not Delete Temporary Files."
puts "Notes:"
puts " 1. Use -DC arguments to restrain tensor parameters as desired;"
puts "    try DC -help for a list of options."
puts " 2. Arguments for -DC must be the last ones on the command line."
exit 1
}

set dcName      dObsA.tab 
set dcTitle     dObsA.tab 
set pdbNameList ref.pdb
set pdbTitle    ref.pdb
set outName     stdout
set outUnit     stdout
set tmpSelName  seltmp.tab
set tmpDelName  deltmp.tab
set dcArgs      "-svd"
set iseed       54321
set keepFrac    0.9
set trials      50 
set allFlag     0
set cleanFlag   1
set selByRes    0
set selArg      noselByRes
set modeStr     None

getArgD $ARGV -in       dcName
getArgD $ARGV -in       dcTitle
getArgD $ARGV -inTitle  dcTitle

getListArgD $ARGV -pdb  pdbNameList

getArgD $ARGV -pdb      pdbTitle
getArgD $ARGV -pdbTitle pdbTitle

getArgD $ARGV -out      outName
getArgD $ARGV -selTab   tmpSelName
getArgD $ARGV -delTab   tmpDelName
getArgD $ARGV -trials   trials
getArgD $ARGV -iseed    iseed
getArgD $ARGV -keep     keepFrac

if {[flagLoc $ARGV -all]}   {set allFlag 1}
if {[flagLoc $ARGV -noall]} {set allFlag 0}

if {[flagLoc $ARGV -clean]}   {set cleanFlag 1}
if {[flagLoc $ARGV -noclean]} {set cleanFlag 0}

if {[flagLoc $ARGV -selByRes]}   {set selByRes 1}
if {[flagLoc $ARGV -noselByRes]} {set selByRes 0}

set loc [flagLoc $ARGV -DC]

if {$loc} {set dcArgs [lrange $ARGV [expr $loc + 1] end]}

if {$allFlag} \
   {
    if {$selByRes} \
       {
        set modeStr "Systematic Deletion of Single Residues."
        set selArg  selByRes
       } \
    else \
       {
        set modeStr "Systematic Deletion of Single Values."
        set selArg  noselByRes
       }
   } \
else \
   {
    if {$selByRes} \
       {
        set selArg  selByRes
        set modeStr "Delete Random Fraction of Residues."
       } \
    else \
       {
        set selArg  noselByRes
        set modeStr "Delete Random Fraction of Values."
       }
   }

if {$keepFrac < 0.0} {set keepFrac 0.0}
if {$keepFrac > 1.0} {set keepFrac 1.0}

proc getWSigma { dName } \
{
   if {![file exists $dName]} {return 1.0}

   set dTab  [gdbRead table -in $dName]
   set vList [gdbGetInfo $dTab -parms varNames]

   if {[lsearch -exact $vList DD] < 0} {return 1.0}
   if {[lsearch -exact $vList W] < 0}  {return 1.0}
 
   set ddList [getTabVals $dTab DD]
   set wList  [getTabVals $dTab W]
   set n      [llength $ddList]

   if {!$n} {return 1.0}

   set rms 0.0

    for {set i 0} {$i < $n} {incr i} \
       {
        set rms [expr $rms + [lindex $ddList $i]*[lindex $wList $i]]
       }

    set rms [expr $rms/$n]

   gdbFree $dTab

   return $rms
}

#
#--------------------------------------------------------------------

if {![file exists $dcName]} \
   {
    puts stderr "DC Validate Error Finding DC Input File $dcTitle"
    exit 1
   }

foreach pdbName $pdbNameList \
   {
    if {![file exists $pdbName]} \
       {
        puts stderr "DC Validate Error Finding PDB Input File $pdbTitle"
        exit 1
       }
   }

set pdbCount [llength $pdbNameList]

if {![strcasecmp $outName None] || ![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
elseif {![strcasecmp $outName stderr]} \
   {
    set outUnit stderr
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "DC Validate Error Opening DC Cross-Validation Output File."
        exit 1
       }
   }

dynInit -fast

catch "file delete $tmpSelName"
catch "file delete $tmpDelName"

set fmt " %4d %8.3f  % .4e % .4e  % .4f % 8.2f % 8.2f % 8.2f"

set pdbSrc ""

foreach pdbName $pdbNameList \
   {
    lappend pdbSrc [dynRead -pdb  -in $pdbName]
   }

set dcTab   [gdbRead table -in $dcName]
set eList   [gdbGetInfo $dcTab -parms entries]
set nDC     [llength $eList]
set resList ""

set minRes  [getMin "[getTabMin $dcTab RESID_I] [getTabMin $dcTab RESID_J]"]
set maxRes  [getMax "[getTabMax $dcTab RESID_I] [getTabMax $dcTab RESID_J]"]

set dcSrc     [dynRead -dc -in $dcName]
set dInfoOrig [eval "dynSimulate -dc $dcSrc -src $pdbSrc -verb $dcArgs"]
set k         [expr [llength $dInfoOrig] - 6]
set saupeOrig [lrange $dInfoOrig $k [expr $k + 4]]

set rxRef     [lindex $dInfoOrig 6]
set ryRef     [lindex $dInfoOrig 7]
set rzRef     [lindex $dInfoOrig 8]

foreach e $eList \
   {
    set resID [getMax [gdbGet $e "RESID_I RESID_J"]]
    if {[lsearch -exact $resList $resID] < 0} {lappend resList $resID}
   }

if {$allFlag} \
   {
    if {$selByRes} \
       {
        set trials [expr 1 + $maxRes - $minRes]
       } \
    else \
       {
        set trials $nDC
       }
   }

for {set i 0} {$i < $trials} {incr i} \
   {
    set resID 0

    if {$allFlag} \
       {
        if {$selByRes} \
           {
            set resID [expr $minRes + $i]
            if {[lsearch -exact $resList $resID] < 0} continue

            puts "All: $allFlag SelByRes: $selByRes Command: delTab.tcl -in $dcName -out $tmpSelName -del $tmpDelName -resID $resID -selByRes -noverb"
 
            exec delTab.tcl -in $dcName -out $tmpSelName -del $tmpDelName \
                            -resID $resID -selByRes -noverb 
           } \
        else \
           {
            puts "All: $allFlag SelByRes: $selByRes Command: delTab.tcl -in $dcName -out $tmpSelName -del $tmpDelName -n $i -noselByRes -noverb"

            exec delTab.tcl -in $dcName -out $tmpSelName -del $tmpDelName \
                            -n $i -noselByRes -noverb 
           }
       } \
    else \
       {
        puts "All: $allFlag SelByRes: -$selByRes Command: delTab.tcl -in $dcName -out $tmpSelName -del $tmpDelName -keep $keepFrac -iseed $iseed -$selArg -noverb"

        exec delTab.tcl -in $dcName -out $tmpSelName -del $tmpDelName \
                        -keep $keepFrac -iseed $iseed -$selArg -noverb 
       }

    incr iseed 13

    set selSrc  [dynRead -dc -in $tmpSelName]
    set delSrc  [dynRead -dc -in $tmpDelName]

    set dInfo  [eval "dynSimulate -dc $selSrc -src $pdbSrc -verb $dcArgs"]

    set selRMS [lindex $dInfo 0]
    set selN   [lindex $dInfo end]
    set da     [lindex $dInfo 3]
    set dr     [lindex $dInfo 4]

    if {$da == 0.0} \
       {
        set rh 0.0
       } \
    else \
       { 
        set rh [expr $dr/$da]
       }

    set rx  [lindex $dInfo 6]
    set ry  [lindex $dInfo 7]
    set rz  [lindex $dInfo 8]

    set rotAdj [getDCRotNearest $rxRef $ryRef $rzRef $rx $ry $rz]

    set k     [expr [llength $dInfo] - 6]
    set saupe [lrange $dInfo $k [expr $k + 4]]

    set dInfo  [dynSimulate -dc $delSrc -src $pdbSrc -saupe $saupe -verb]
    set delRMS [lindex $dInfo 0]
    set delN   [lindex $dInfo end]

    lappend sInfo(DEL_RESID)  $resID
    lappend sInfo(SEL_RMS)    $selRMS
    lappend sInfo(DEL_RMS)    $delRMS
    lappend sInfo(SEL_WSIGMA) [getWSigma $tmpSelName]
    lappend sInfo(DEL_WSIGMA) [getWSigma $tmpDelName] 
    lappend sInfo(N_SEL)      $selN
    lappend sInfo(N_DEL)      $delN
    lappend sInfo(Da)         $da
    lappend sInfo(Dr)         $dr
    lappend sInfo(Rh)         $rh

    lappend sInfo(ROTATION_X)  [lindex $rotAdj 0] 
    lappend sInfo(ROTATION_Y)  [lindex $rotAdj 1] 
    lappend sInfo(ROTATION_Z)  [lindex $rotAdj 2] 

    set dInfo  [dynSimulate -dc $selSrc -src $pdbSrc -saupe $saupeOrig -verb]
    set selRMS [lindex $dInfo 0]

    set dInfo  [dynSimulate -dc $delSrc -src $pdbSrc -saupe $saupeOrig -verb]
    set delRMS [lindex $dInfo 0]

    lappend sInfo(SEL_RMS_ORIG) $selRMS
    lappend sInfo(DEL_RMS_ORIG) $delRMS

    set dObsVal   0.0
    set dCalcVal  0.0
    set dWVal     0.0
    set dSigmaVal 0.0

    if {$allFlag && !$selByRes} \
       {
        dynWrite -dc $delSrc -out $tmpDelName -verb

        set delTab [gdbRead table -in $tmpDelName]
        set eList  [gdbGetInfo $delTab -parms entries]

        if {[llength $eList] == 1} \
           {
            set e [lindex $eList 0]

            set dCalcVal  [gdbGet $e D]
            set dObsVal   [gdbGet $e D_OBS]
            set dSigmaVal [gdbGet $e DD]
            set dWVal     [gdbGet $e W]
           }
       
        gdbFree $delTab 
       }

    lappend sInfo(D_WOBS)   [expr $dWVal*$dObsVal]
    lappend sInfo(D_WCALC)  [expr $dWVal*$dCalcVal]
    lappend sInfo(D_WSIGMA) [expr $dWVal*$dSigmaVal]
    lappend sInfo(D_WDIFF)  [expr $dWVal*($dObsVal - $dCalcVal)]

    dynFree -dc $selSrc 
    dynFree -dc $delSrc 
   }

if {$cleanFlag} {catch "file delete $tmpSelName"}
if {$cleanFlag} {catch "file delete $tmpDelName"}

if {$pdbCount == 1} \
   {
    puts $outUnit "REMARK DC Cross-Validation Error Analysis for $dcTitle and $pdbTitle"
   } \
else \
   {
    puts $outUnit "REMARK DC Cross-Validation Error Analysis for $dcTitle and $pdbTitle ..."
   }

puts $outUnit "REMARK Structure Count: $pdbCount"
puts $outUnit "REMARK Validation Mode: $modeStr"
puts $outUnit "REMARK DC Options: $dcArgs"
puts $outUnit ""

set parmFmt "%-12s "
set i 0

if {$allFlag && !$selByRes} \
   {
    set pList   {D_WDIFF  SEL_RMS SEL_RMS_ORIG Rh       Da       Dr}
    set fmtList { "%7.3f" "%7.3f" "%7.3f"      "%10.3f" "% 8.3e" "% 8.3e"}
   } \
else \
   {
    set pList   {SEL_RMS  DEL_RMS SEL_RMS_ORIG DEL_RMS_ORIG Rh       Da       Dr}
    set fmtList { "%7.3f" "%7.3f" "%7.3f"      "%7.3f"      "%10.3f" "% 8.3e" "% 8.3e"}
   }

foreach parm $pList \
   {
    if {![strcasecmp $parm Rh]} \
       {
        set parmFmt "%-3s"
        puts $outUnit ""
       }

    puts -nonewline $outUnit [format "DATA STAT $parmFmt" $parm]

    set fmt [lindex $fmtList $i]

    foreach stat {Min Max Avg StdDev} \
       {
        set statInfo($parm,$stat) [format $fmt [get$stat $sInfo($parm)]]

        puts -nonewline $outUnit " [string toupper $stat]"
        puts -nonewline $outUnit " $statInfo($parm,$stat)"
       }

    puts $outUnit ""
    incr i
   }

puts $outUnit ""

set labList {Min Max Avg StdDev}

foreach parm {ROTATION_X ROTATION_Y ROTATION_Z} \
   {
    puts -nonewline $outUnit "DATA STAT $parm "
    set i 0

    foreach stat {Min Max AvgAngle StdDevAngle} \
       {
        set statInfo($parm,$stat) [format "% 7.2f" [get$stat $sInfo($parm)]]
        set lab [lindex $labList $i]

        puts -nonewline $outUnit " [string toupper $lab]"
        puts -nonewline $outUnit " $statInfo($parm,$stat)"
        incr i
       }

    puts $outUnit ""
   }

puts $outUnit ""
puts $outUnit "# http://spin.niddk.nih.gov/NMRPipe/dc" 
puts $outUnit "#"

if {$allFlag} \
   {
    if {$selByRes} \
       {
        puts $outUnit "# DC Error Analysis by Systematic Deletion of Each Residue."
        puts $outUnit "#"
        puts $outUnit "#  RMS refers to the RMS of W*(observed - calculated) couplings."
        puts $outUnit "#  where W is the weighting factor in the input DC table."
        puts $outUnit "#"
        puts $outUnit "#  INDEX        is the trial number."
        puts $outUnit "#  DEL_RESID    is the RESID of the deleted residue."
        puts $outUnit "#  SEL_RMS      is the RMS for the selected values."
        puts $outUnit "#  DEL_RMS      is the RMS for the deleted values."
        puts $outUnit "#  SEL_RMS_ORIG is the RMS for the selected values in the complete data."
        puts $outUnit "#  DEL_RMS_ORIG is the RMS for the deleted values in the complete data."
        puts $outUnit "#  N_DEL        is the number of deleted values."
        puts $outUnit "#  N_SEL        is the number of selected values."
        puts $outUnit "#  DA and DR    are the tensor axial and rhombic parameters Da and Dr."
        puts $outUnit "#  RH           is the Rhombicty = Dr/Da."
        puts $outUnit "#  RX RY RZ     are the angles that rotate the PDB onto the alignment frame."
        puts $outUnit ""
        puts $outUnit "VARS   INDEX DEL_RESID SEL_RMS N_SEL DEL_RMS N_DEL SEL_RMS_ORIG DEL_RMS_ORIG SEL_WSIGMA DEL_WSIGMA DA   DR   RH   RX    RY    RZ"
        puts $outUnit "FORMAT %4d   %4d       %8.3f   %4d   %8.3f   %4d   %8.3f        %8.3f        %8.3f      %8.3f      %.4e %.4e %.4f %8.2f %8.2f %8.2f"

        set fmt " %4d %4d %8.3f %4d %8.3f %4d %8.3f %8.3f %8.3f %8.3f  % .4e % .4e  % .4f  % 8.2f % 8.2f % 8.2f"
       } \
    else \
       {
        puts $outUnit "# DC Error Analysis by Systematic Deletion of Each Value."
        puts $outUnit "#"
        puts $outUnit "#  RMS refers to the RMS of W*(observed - calculated) couplings."
        puts $outUnit "#  where W is the weighting factor in the input DC table."
        puts $outUnit "#"
        puts $outUnit "#  INDEX     is the trial number."
        puts $outUnit "#  SEL_RMS   is the RMS for the selected values."
        puts $outUnit "#  DEL_RMS   is the RMS for the deleted value."
        puts $outUnit "#  D_WOBS    is W*obs for the deleted value."
        puts $outUnit "#  D_WCALC   is W*calc for the deleted value."
        puts $outUnit "#  D_WDIFF   W*(observed - calculated) for the deleted value."
        puts $outUnit "#  D_WSIGMA  W*DD for the deleted value (DD is uncertainty in observed value)."
        puts $outUnit "#  N_DEL     is the number of deleted values."
        puts $outUnit "#  N_SEL     is the number of selected values."
        puts $outUnit "#  DA and DR are the tensor axial and rhombic parameters Da and Dr."
        puts $outUnit "#  RH        is the Rhombicty = Dr/Da."
        puts $outUnit "#  RX RY RZ  are the angles that rotate the PDB onto the alignment frame."
        puts $outUnit ""
        puts $outUnit "VARS   INDEX SEL_RMS N_SEL D_WOBS D_WCALC D_WDIFF  D_WSIGMA N_DEL DA    DR   RH   RX    RY    RZ"
        puts $outUnit "FORMAT %4d   %8.3f   %4d   %8.3f  %8.3f   %8.3f    %8.3f   %4d   %.4e  %.4e %.4f %8.2f %8.2f %8.2f"

        set fmt " %4d  %8.3f %4d  % 8.3f % 8.3f % 8.3f %8.3f  %4d  %.4e %.4e %.4f  %8.2f %8.2f %8.2f"
      }
   } \
else \
   {
    set pct [format %.1f [expr 100.0*(1.0 - $keepFrac)]]

    if {$selByRes} \
       {
        puts $outUnit "# DC Error Analysis by Random Deletion of ${pct}% of the Residues."
       } \
    else \
       {
        puts $outUnit "# DC Error Analysis by Random Deletion of ${pct}% of the Values."
       }

    puts $outUnit "#"
    puts $outUnit "#  RMS refers to the RMS of W*(observed - calculated) couplings."
    puts $outUnit "#  where W is the weighting factor in the input DC table."
    puts $outUnit "#"
    puts $outUnit "#  INDEX        is the trial number."
    puts $outUnit "#  SEL_RMS      is the RMS for the selected values."
    puts $outUnit "#  DEL_RMS      is the RMS for the deleted values."
    puts $outUnit "#  SEL_RMS_ORIG is the RMS for the selected values in the complete data."
    puts $outUnit "#  DEL_RMS_ORIG is the RMS for the deleted values in the complete data."
    puts $outUnit "#  N_DEL        is the number of deleted values."
    puts $outUnit "#  N_SEL        is the number of selected values."
    puts $outUnit "#  DA and DR    are the tensor axial and rhombic parameters Da and Dr."
    puts $outUnit "#  RH           is the Rhombicty = Dr/Da."
    puts $outUnit "#  RX RY RZ     are the angles that rotate the PDB onto the alignment frame."
    puts $outUnit ""
    puts $outUnit "VARS   INDEX SEL_RMS N_SEL DEL_RMS N_DEL SEL_RMS_ORIG DEL_RMS_ORIG SEL_WSIGMA DEL_WSIGMA DA   DR   RH   RX    RY    RZ"
    puts $outUnit "FORMAT %4d   %8.3f   %4d   %8.3f   %4d   %8.3f        %8.3f        %8.3f      %8.3f      %.4e %.4e %.4f %8.2f %8.2f %8.2f"

    set fmt " %4d %8.3f %4d %8.3f %4d %8.3f %8.3f %8.3f %8.3f  % .4e % .4e  % .4f  % 8.2f % 8.2f % 8.2f"
   }

puts $outUnit ""

for {set i 0} {$i < $trials} {incr i} \
   {
    set delResID   [lindex $sInfo(DEL_RESID) $i]
    set selRMS     [lindex $sInfo(SEL_RMS) $i]
    set selRMSOrig [lindex $sInfo(SEL_RMS_ORIG) $i]
    set selWSigma  [lindex $sInfo(SEL_WSIGMA) $i]
    set nSel       [lindex $sInfo(N_SEL) $i]

    set delRMS     [lindex $sInfo(DEL_RMS) $i]
    set delRMSOrig [lindex $sInfo(DEL_RMS_ORIG) $i]
    set delWSigma  [lindex $sInfo(DEL_WSIGMA) $i]
    set nDel       [lindex $sInfo(N_DEL) $i]

    set dWObs      [lindex $sInfo(D_WOBS)   $i]
    set dWCalc     [lindex $sInfo(D_WCALC)  $i]
    set dWSigma    [lindex $sInfo(D_WSIGMA) $i]
    set dWDiff     [lindex $sInfo(D_WDIFF)  $i]

    set da  [lindex $sInfo(Da)  $i]
    set dr  [lindex $sInfo(Dr)  $i]
    set rh  [lindex $sInfo(Rh)  $i]

    set rx  [lindex $sInfo(ROTATION_X) $i]
    set ry  [lindex $sInfo(ROTATION_Y) $i]
    set rz  [lindex $sInfo(ROTATION_Z) $i]

    set k   [expr $i + 1]

    if {$allFlag} \
       {
        if {$selByRes} \
           {
            puts $outUnit [format $fmt $k $delResID $selRMS $nSel $delRMS $nDel $selRMSOrig $delRMSOrig $selWSigma $delWSigma $da $dr $rh $rx $ry $rz]
           } \
        else \
           {
            puts $outUnit [format $fmt $k $selRMS $nSel $dWObs $dWCalc $dWDiff $dWSigma $nDel $da $dr $rh $rx $ry $rz]
           }
       } \
    else \
       {
        puts $outUnit [format $fmt $k $selRMS $nSel $delRMS $nDel $selRMSOrig $delRMSOrig $selWSigma $delWSigma $da $dr $rh $rx $ry $rz]
       }
   }

if {[strcasecmp $outUnit stdout] && [strcasecmp $outUnit stderr]} \
   {
    catch "close $outUnit"
   }

exit 0
