#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# deco.tcl: decompose a 1D dataset as a linear combination of others.

#
# Command line:

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: decompose a 1D dataset as a linear combination of others."
    puts " -in     inName     \[brain.ft1\] Input Name of Data to Decompose."
    puts " -out    outName    \[deco.ft1\]  Output Name of Combination."
    puts " -tab    outTabName \[deco.tab\]  Output Name of Coefficient Table."
    puts " -dif    difName    \[dif.ft1\]   Output Name of Difference."
    puts " -inList inNameList \[ft/*.ft1\]  List of Basis Names."
    puts " -x1     x1         \[0%\]        Start of Analysis Region."
    puts " -xn     xn         \[100%\]      End of Analysis Region."
    puts " -sigma  stdDev     \[0.0\]       Std Dev of Noise (Use 0 for Auto)."
    puts " -verb                          Verbose Mode ON (Default)."
    puts " -noverb                        Verbose Mode OFF."
    exit 0
   }

set inName    brain.ft1
set outName   deco.ft1
set tabName   deco.tab
set difName   dif.ft1
set x1        0%
set xn        100%
set sigma     0.0
set inList    ""
set verbose   1

set inList [lsort -dictionary [glob -nocomplain ft/*.ft1]]

getArgD     $ARGV -in      inName
getArgD     $ARGV -out     outName
getArgD     $ARGV -tab     tabName
getArgD     $ARGV -dif     difName
getArgD     $ARGV -x1      x1
getArgD     $ARGV -xn      xn
getArgD     $ARGV -sigma   sigma
getListArgD $ARGV -inList  inList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
# Report utility.

proc report { msg } \
{
   global outUnit verbose

   puts $outUnit $msg

   if {$verbose} {puts $msg}
}

#
# Prepare output file:

set outUnit stdout

catch {set outUnit [open $tabName w]}

#
# Read the spectrum to analyze:

readROI -roi 1 -ndim 1 -in $inName 

set roiInfo [getROIInfo -roi 1]

set rdata   [lindex $roiInfo 0]
set eCount  [lindex $roiInfo 2]
set cCount  [llength $inList]
set vCount  1

if {$sigma == 0.0} {set sigma [vEstNoise $rdata $eCount]}

#
# Allocate workspaces:

set xPtr [sysMalloc [expr $eCount*$cCount] float]
set yPtr [sysMalloc [expr $vCount*$eCount] float]
set aPtr [sysMalloc [expr $vCount*$cCount] float]
set ePtr [sysMalloc [expr $vCount*$cCount] float]

#
# Copy the data to decompose.
# Read the basis set.

vvMove $yPtr $rdata $eCount

set rPtr   $xPtr

set bList  ""
set bCount 0

foreach bName $inList \
   {
    readROI -roi 1 -ndim 1 -in $bName 

    set roiInfo [getROIInfo -roi 1]
    set rdata   [lindex $roiInfo 0]
    set size    [lindex $roiInfo 2] 

    if {$verbose} {puts stderr "$bName $size"}

    if {$size != $eCount} \
       {
        puts stderr "Size mismatch for $bName"
       } \
    else \
       {
        vvMove $rPtr $rdata $eCount
     
        set rPtr [incrPtr $rPtr $eCount] 

        lappend bList $bName
        incr    bCount  
       }
   }

#
# Do the decomposition.
# Extract the results.

vLinSol -xPtr $xPtr -yPtr $yPtr -aPtr $aPtr -ePtr $ePtr \
        -eCount $eCount -vCount $vCount -cCount $bCount -eJump $vCount -vJump 1

set fList ""
set eList ""
set cList ""	
set i     0

foreach bName $bList \
   {
    set c [getPtrVal $aPtr $i]
    set e [getPtrVal $ePtr $i]

    lappend fList $bName
    lappend eList $e
    lappend cList $c

    incr i
   }

#
# Form a synthetic spectrum and difference.
# Analyze the difference:

eval exec synth.tcl -inList $fList -cList $cList -out $outName
eval exec addNMR -in1 $inName -in2 $outName -out $difName -sub

readROI -ndim 1 -roi 1 -in $difName 

set rdata   [lindex $roiInfo 0]

vsDiv  $rdata $sigma $eCount
vvMult $rdata $rdata $eCount

set sum  [vSum $rdata $eCount]
set chi2 [expr $sum/$eCount] 

#
# Report the results:

report [format "DATA SIGMA %.2f" $sigma]
report [format "DATA CHI2  %.2f" $chi2]
report ""
report "VARS   INDEX C     C_ERR B_NAME"
report "FORMAT %4d   %9.4f %9.4f %-24s"
report ""

set i 0

foreach bName $bList \
   {
    set bName [lindex $fList $i]
    set c     [lindex $cList $i]
    set e     [lindex $eList $i]

    incr i

    report [format " %4d % 9.4f %9.4f %-24s" $i $c $e $bName]
   }

if {!$verbose} {puts $chi2}

catch {close $outUnit}

exit
