#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# deco2D.tcl: decompose a 2D dataset as a linear combination of others.

#
# Command line:

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: decompose a 2D dataset as a linear combination of others."
    puts " -in     inName     \[test.ft2\]    Input Name of Data to Decompose."
    puts " -basis  basisList  \[peaks/*.ft2\] List of Basis Input Names."
    puts " -out    outName    \[deco.ft2\]    Output Name of Combination, or Keyword None."
    puts " -tab    outTabName \[deco.tab\]    Output Name of Coefficient Table, or Keyword None."
    puts " -x1     x1         \[0%\]          Start of Analysis Region."
    puts " -xn     xn         \[100%\]        End of Analysis Region."
    puts " -y1     y1         \[0%\]          Start of Analysis Region."
    puts " -yn     yn         \[100%\]        End of Analysis Region."
    puts " -verb                            Verbose Mode ON (Default)."
    puts " -noverb                          Verbose Mode OFF."
    puts " -brief                           Verbose Mode OFF, List Coefficients."
    exit 0
   }

set inName    test.ft2
set outName   deco.ft2
set tabName   deco.tab
set x1        0%
set xn        100%
set y1        0%
set yn        100%
set basisList ""
set verbose   1
set briefFlag 0

set basisList [lsort -dictionary [glob -nocomplain peaks/*.ft2]]

getArgD     $ARGV -in      inName
getArgD     $ARGV -out     outName
getArgD     $ARGV -tab     tabName
getArgD     $ARGV -x1      x1
getArgD     $ARGV -xn      xn
getArgD     $ARGV -y1      y1
getArgD     $ARGV -yn      yn
getListArgD $ARGV -basis   basisList

if {[flagLoc $ARGV -brief]} \
   {
    set verbose   0
    set briefFlag 1
   }

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
# Read the spectrum to analyze:

set error [rdFDATA $inName fdataOrig]

readROI -roi 1 -ndim 2 -in $inName -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn

set roiInfo [getROIInfo -roi 1]

set rdata   [lindex $roiInfo 0]
set fdata   [lindex $roiInfo 1]
set xSize   [lindex [lindex $roiInfo 2] 0]
set ySize   [lindex [lindex $roiInfo 2] 1]

set planeSize [expr $xSize*$ySize]
set coefCount [llength $basisList]

#
# Allocate workspaces:

set xPtr [sysMalloc [expr $planeSize*$coefCount] float]
set yPtr [sysMalloc [expr $planeSize] float]

#
# Copy the data to decompose.
# Read the basis set.

vvMove $yPtr $rdata $planeSize

set destPtr $xPtr

if {$verbose} {puts stderr "Reading $coefCount basis inputs ..."}

foreach basisName $basisList \
   {
    readROI -roi 2 -ndim 2 -in $basisName -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn

    set roiInfo   [getROIInfo -roi 2]
    set rdata     [lindex $roiInfo 0]
    set thisXSize [lindex [lindex $roiInfo 2] 0]
    set thisYSize [lindex [lindex $roiInfo 2] 1]

    if {$xSize != $thisXSize || $ySize != $thisYSize} \
       {
        puts stderr "Size mismatch for $basisName (Expected: $xSize $ySize Actual: $thisXSize $thisYSize)."
        exit 1
       }

    vvMove2 $destPtr $coefCount 1 $rdata 1 $planeSize $planeSize 1
    
    set destPtr [incrPtr $destPtr 1]
   }

#
# Do the decomposition.
# Extract the results.

if {$verbose} {puts stderr "Decomposing $inName using $coefCount basis terms ..."}

set tInit  [lindex [sysTime] 0]

set cList  [svdLinSol -xPtr $xPtr -yPtr $yPtr -n $planeSize -m $coefCount]

set tFinal [lindex [sysTime] 0]

if {[llength $cList] != $coefCount} \
   {
    puts stderr "Error forming decomposition."
   }

if {$briefFlag} {puts $cList}

if {$verbose} {puts stderr "Decomposition finished in [expr $tFinal - $tInit] seconds."}

if {[strcasecmp $tabName None]} \
   {
    set error [catch {set outUnit [open $tabName w]}]

    puts $outUnit ""
    puts $outUnit "#"
    puts $outUnit "# Decomposition of $inName"
    puts $outUnit ""

    puts $outUnit "VARS    INDEX C       BASIS_NAME"
    puts $outUnit "FORMAT  %4d   %14.6e  %s"
    puts $outUnit ""

    for {set i 0} {$i < $coefCount} {incr i} \
       {
        puts $outUnit [format " %4d % .6e %s" [expr $i + 1] [lindex $cList $i] [lindex $basisList $i]]
       }
 
    catch "close $outUnit"

    if {$verbose} {puts stderr "Created coefficient table $tabName"}
   }

if {[strcasecmp $outName None]} \
   {
    readROI -roi 1 -ndim 2 -in $inName -x X_AXIS 0% 100% -y Y_AXIS 0% 100%

    set roiInfo [getROIInfo -roi 1]
    set rdata   [lindex $roiInfo 0]
    set fdata   [lindex $roiInfo 1]
    set xSize   [lindex [lindex $roiInfo 2] 0]
    set ySize   [lindex [lindex $roiInfo 2] 1]

    set planeSize [expr $xSize*$ySize]

    vsSet $rdata 0.0 $planeSize

    for {set i 0} {$i < $coefCount} {incr i} \
       {
        set basisName [lindex $basisList $i]

        readROI -roi 2 -ndim 2 -in $basisName -x X_AXIS 0% 100% -y Y_AXIS 0% 100%

        set roiInfo   [getROIInfo -roi 2]
        set bdata     [lindex $roiInfo 0]
        set thisXSize [lindex [lindex $roiInfo 2] 0]
        set thisYSize [lindex [lindex $roiInfo 2] 1]

        if {$xSize != $thisXSize || $ySize != $thisYSize} \
           {
            puts stderr "Size mismatch for $basisName (Expected: $xSize $ySize Actual: $thisXSize $thisYSize)."
            exit 1
           } 

        set c [lindex $cList $i]

        vsMult $bdata $c $planeSize
        vvAdd  $rdata $bdata $planeSize
       }

    writeROI -roi 1 -out $outName -ov
    wrFDATA $outName $fdataOrig

    if {$verbose} {puts stderr "Created linear combination output $outName"}
   }

exit 0
