#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2021.028.16.40"

set inName    mc.fid
set outName   exp.fid
set skipSize  0 
set maxIter   4
set briefFlag 0

set noise    Auto
set nx1      95%
set nxn      99%

set aFrac    0.1
set aMin     Auto
set aMax     Auto
set dA       0.1
set nA       128

set bFrac    0.1
set bMin     Auto
set bMax     Auto
set dB       0.1
set nB       128

if {[flagLoc $ARGV -help]} \
   {
    puts "Extract 1D Exponential Decay by Grid Search (Version $thisVersion):"
    puts " -in    inName   Input Time-Domain Data.           \[$inName\]"
    puts " -skip  skip     Input Points to Skip.             \[$skipSize\]"
    puts " -aFrac aFrac    Decay Fraction for Auto Estimate. \[$aFrac\]"
    puts " -aMin  aMin     Minimum Decay for Grid Search.    \[$aMin\]"
    puts " -aMax  aMax     Maximum Decay for Grid Search.    \[$aMax\]"
    puts " -nA    nA       Number of Steps for Grid Search.  \[$nA\]"
    puts " -dA    dA       Initial +/- Width of Grid Search. \[$dA\]"
    puts " -iter  maxIter  Iteration Count.                  \[$maxIter\]"
    puts " -noise noise    Noise Estimate.                   \[$noise\]"
    puts " -nx1   nx1      Region Start for -noise Auto.     \[$nx1\]"
    puts " -nxn   nxn      Region End for -noise Auto.       \[$nxn\]"
    puts " -brief          Brief Report Mode ON."
    puts "Notes:"
    puts " 1. The final output will be in this form:"
    puts ""
    puts "       a0 0.8478 pA 8.4711e+3 pB -1.4365e+4 pC 6.09269e+3 aFit 0.8478 aStdDev 0.0025"
    puts ""
    puts "    where a0 is the best decay found by grid search, pA pB pC are the quadratic"
    puts "    coefficients that fit the RMS vs decay curve, aFit is the best decay as estimated"
    puts "    by the fit, and aStdDev is the estimated standard deviation of the error in the decay."
    exit 0
   }

if {[flagLoc $ARGV -brief]}   {set briefFlag 1}
if {[flagLoc $ARGV -nobrief]} {set briefFlag 0}
 
getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -skip  skipSize
getArgD $ARGV -iter  maxIter

getArgD $ARGV -noise noise
getArgD $ARGV -nx1   nx1
getArgD $ARGV -nxn   nxn

getArgD $ARGV -aFrac aFrac
getArgD $ARGV -aMin  aMin
getArgD $ARGV -aMax  aMax
getArgD $ARGV -dA    dA
getArgD $ARGV -nA    nA

getArgD $ARGV -bFrac bFrac
getArgD $ARGV -bMin  bMin
getArgD $ARGV -bMax  bMax
getArgD $ARGV -dB    dB
getArgD $ARGV -nB    nB

if {![file exists $inName]} \
   {
    puts stderr "expGrid. Error finding input $inName"
    exit 1
   }

if {![strcasecmp $noise Auto]} \
   {
    readROI -roi 1 -in $inName -ndim 1 -x X_AXIS $nx1 $nxn
    set noise [getROIStat 1 vRMS]
   }

if {$noise == 0.0} {set noise 1.0}

readROI -roi 1 -in $inName -ndim 1 -x X_AXIS 0% 100% 

set rInfo [getROIInfo -roi 1]

set rdata [lindex $rInfo 0]
set fdata [lindex $rInfo 1]

set size  [lindex [lindex $rInfo 2] 0]
set xSW   [getParm $fdata NDSW CUR_XDIM]
set dT    [expr 1.0/$xSW]

if {$skipSize > $size - 2} \
   {
    puts stderr "expGrid. Error: skip size is too large ($skipSize $outSize)."
    exit 1
   }

vsMult $rdata [expr 1.0/$noise] $size]

set xdata [sysMalloc $size float]
set ydata [sysMalloc $size float]
set ddata [sysMalloc $size float]
set qdata [sysMalloc $size float]

set n      [expr $size - $skipSize]
set maxVal [vMax $rdata $size]

set tFrac 0.0
set t     0.0

for {set i 0} {$i < $size} {incr i} \
   {
    if {$tFrac == 0.0 && [getPtrVal $rdata $i] < $aFrac*$maxVal} {set tFrac $t} 

    setPtrVal $xdata $i $t

    set t [expr $t + $dT]
   }

set a0 [expr -1.0*log(0.1)/$tFrac]

if {![strcasecmp $aMin Auto]} {set aMin [expr $a0 - $dA]}
if {![strcasecmp $aMax Auto]} {set aMax [expr $a0 + $dA]}

for {set iter 1} {$iter <= $maxIter} {incr iter} \
   {
    set aStep [expr ($aMax - $aMin)/($nA - 1)]

    set rmsBest [expr 10.0*$maxVal*$maxVal*$size]
    set aBest   $aMin

    set xList($iter) ""
    set yList($iter) ""
 
    for {set i 0} {$i < $nA} {incr i} \
       {
        set a  [expr $aMin + $i*$aStep]

        vsMult $ydata $xdata [expr -1.0*$a] $size
        vExp   $ydata $size 

        set s [vvGetScale [incrPtr $rdata $skipSize] [incrPtr $ydata $skipSize] $n]

        vsMult $ydata $s $size
        vvSub  $ddata $ydata $rdata $size

        set r [vRMS [incrPtr $ddata $skipSize] $n]

        lappend xList($iter) $a
        lappend yList($iter) $r

        if {$i == 0 || $r < $rmsBest} \
           {
            vvMove $qdata $ydata $n

            set rmsBest $r
            set aBest   $a
           }

        if {!$briefFlag && ($iter == $maxIter)} {puts [format "a %e s %e r %e a0 %e n %d" $a $s $r $a0 $n]}
       }

    set a0 $aBest
    set dA [expr 0.5*$dA]

    set aMin [expr $a0 - $dA]
    set aMax [expr $a0 + $dA]

    if {!$briefFlag} {puts stderr [format "iter %d best %e limits %e %e rms %e noise %e" $iter $a0 $aMin $aMax $rmsBest $noise]}
   }

if {[strcasecmp $outName None]} \
   {
    set tInfo [split [getParmStr $fdata FDTITLE NULL_DIM] "-"]

    setParmStr $fdata FDTITLE [join [lrange $tInfo 0 end-1] "-"]-exp1 NULL_DIM

    vvMove $rdata $qdata $size
    vsMult $rdata $noise $size

    writeROI -roi 1 -out $outName -ov
   }

#
# Fit the final RMS vs decay value curve to a parabola.
# The location of the parabola minimum vertex is the estimated best decay value.
# If the parabola is lowered so that the vertex is below the X-Axis by the noise, 
# the location of the zero crossings gives the error estimate.

set cList [polySolve $xList($maxIter) $yList($maxIter) 2]

set pA    [lindex $cList 0]
set pB    [lindex $cList 1]
set pC    [lindex $cList 2]

if {$pA == 0.0} {set pA 1.0}

set aFit [expr -1.0*$pB/(2.0*$pA)]
set yFit [expr $pA*$aFit*$aFit + $pB*$aFit + $pC]
set pCM  [expr $pC - ($yFit + $noise)]
set aSD  [expr (-1.0*$pB + sqrt($pB*$pB - 4.0*$pA*$pCM))/(2.0*$pA) - $aFit]

if {$briefFlag} \
   {
    puts -nonewline [format "a0 %.6f pA %e pB %e pC %e noise %e aFit %.6f aStdDev %.6f" $a0 $pA $pB $pC $noise $aFit $aSD]
   } \
else \
   {
    puts [format "# a0 %.6f pA %e pB %e pC %e noise %e aFit %.6f aStdDev %.6f" $a0 $pA $pB $pC $noise $aFit $aSD]
   }

exit 0
