#!/bin/csh

if (`flagLoc $argv -help`) then
   nmrPrintf -stderr "Create a Sampling Schedule for Uniformly-Sampled NMR Data.\n"
   nmrPrintf -stderr " -in    inName  [Auto]   NMRPipe-Format Input.\n"
   nmrPrintf -stderr " -verb                   Verbose Mode ON (Default).\n"
   nmrPrintf -stderr " -noverb                 Verbose Mode OFF.\n"
   nmrPrintf -stderr "Dimension Count and Sizes (Use -y... -z.. -a...):\n"
   nmrPrintf -stderr " -ndim          [Auto]   Dimension Count.\n"
   nmrPrintf -stderr " -yT            [Auto]   Number of Complex Increments in Fully-Sampled Y-Axis.\n"
   nmrPrintf -stderr "Generation of Non-Uniform Sampling (NUS) Schedule:\n"
   nmrPrintf -stderr " -nus   nusFrac [1.0]    Fraction of Samples to Keep.\n"
   nmrPrintf -stderr " -iseed iseed   [767537] Random Number Seed, Zero for Auto.\n"
   nmrPrintf -stderr "Notes:\n"
   nmrPrintf -stderr " 1. The input data is used to determine sizes for the schedule.\n"
   nmrPrintf -stderr " 2. The Non-Uniform Schedules generated have uniform random\n"
   nmrPrintf -stderr "    density, and always include the first and last increments.\n"
   nmrPrintf -stderr " 3. Using -nus 1.0 will generate a Uniformly-Sampled Schedule.\n"
   nmrPrintf -stderr " 4. Results are written to standard output, for example:\n"
   nmrPrintf -stderr "\n"
   nmrPrintf -stderr "       fakeNUS.com -in test.fid > fake.nuslist\n"
   nmrPrintf -stderr "\n"
   exit 0
endif

set inName   = (`getArgD $argv -in    Auto`)
set iseed    = (`getArgD $argv -iseed 767537`)
set nusFrac  = (`getArgD $argv -nus   1.0`)

set dimCount = (`getArgD $argv -ndim  Auto`)
set xT       = (`getArgD $argv -xT    Auto`)
set yT       = (`getArgD $argv -yT    Auto`)
set zT       = (`getArgD $argv -zT    Auto`)
set aT       = (`getArgD $argv -aT    Auto`)

@ vFlag = 1
@ pseudo3DFlag = 0

if (`flagLoc $argv -verb`) then
   @ vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   @ vFlag = 0
endif

if (`flagLoc $argv -pseudo3D`) then
   @ pseudo3DFlag = 1
endif

if (`string csame $inName Auto`) then
   set fList  = (fid/test%03d.fid data/test%03d.fid fid/spec%03d.fid data/spec%03d.fid fid/test%04d.fid data/test%04d.fid test.fid spec.fid)

   foreach i ($fList)
      set thisName = (`nmrPrintf $i 1`)

      if (-e $thisName) then
         set inName = $i
         break
      endif
   end
endif

if (`string csame $inName Auto`) then
   set thisName = None
else
   set thisName = (`nmrPrintf $inName 1 1`)
endif

if (`string csame $dimCount Auto`) then
   if (`string csame $inName Auto`) then
      nmrPrintf -stderr "fakeNUS. Error Finding Auto-Mode Input for Setting -ndim Option.\n"
      exit 1 
   endif

   if (!(-e $thisName)) then
      nmrPrintf -stderr "fakeNUS. Error Finding Input %s for Setting -ndim Option.\n" $inName
      exit 1 
   endif
   
   set dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -dim NULL_DIM -fmt %.0f`)
endif

if (`string csame $xT Auto`) then
   if (`string csame $thisName None`) then
      set xT = Unknown
   else
      set xT = (`getParm -in $thisName -parm NDAPOD -dim CUR_XDIM -fmt %.0f`)
   endif
endif

if ($dimCount >= 2 && `string csame $yT Auto`) then
   if (`string csame $thisName None`) then
      nmrPrintf -stderr "fakeNUS. Error: Auto Parameter Settings Require the -in Option.\n"
      exit 1
   endif

   set yT = (`getParm -in $thisName -parm NDAPOD -dim CUR_YDIM -fmt %.0f`)
endif

if ($pseudo3DFlag && $dimCount != 3) then
   nmrPrintf -stderr "fakeNUS. Error: -pseudo3D Mode Requires 3D Data."
   exit 1
endif

if ($dimCount >= 3 && `string csame $zT Auto`) then
   if (`string csame $thisName None`) then
      nmrPrintf -stderr "fakeNUS. Error: Auto Parameter Settings Require the -in Option.\n"
      exit 1
   endif

   set zT = (`getParm -in $thisName -parm NDAPOD -dim CUR_ZDIM -fmt %.0f`)
endif

if ($dimCount >= 4 && `string csame $aT Auto`) then
   if (`string csame $thisName None`) then
      nmrPrintf -stderr "fakeNUS. Error: Auto Parameter Settings Require the -in Option.\n"
      exit 1
   endif

   set aT = (`getParm -in $thisName -parm NDAPOD -dim CUR_ADIM -fmt %.0f`)
endif

@ sampleCount = 1

if ($dimCount >= 2) then
   @ sampleCount = (`IMATH "$yT*$sampleCount"`)
endif

if ($dimCount >= 3) then
   @ sampleCount = (`IMATH "$zT*$sampleCount"`)
endif

if ($dimCount >= 4) then
   @ sampleCount = (`IMATH "$aT*$sampleCount"`)
endif

if (`IMATH "$nusFrac == 1.0"`) then
   @ nusFlag = 0
else
   @ nusFlag = 1
endif

if ($nusFlag) then
   if ($pseudo3DFlag) then
      set iList = ()

      @ iz = 0

      while ($iz < $zT)
         set iList = ($iList `randList.tcl -n $yT -uniform -bool -xMax $nusFrac -iseed $iseed -first -last`)
         set iseed = (`IMATH "$iseed + 222"`)
         @ iz++
      end
   else
      set iList = (`randList.tcl -n $sampleCount -uniform -bool -xMax $nusFrac -iseed $iseed -first -last`)
   endif
else
   set iList = ""
   @ i = 0

   while ($i < $sampleCount)
      set iList = ($iList 1)
      @ i++
   end
endif

@ n  = 0
@ nn = 0

if ($vFlag) then
   nmrPrintf -stderr "\n"
   if ($pseudo3DFlag) then
      nmrPrintf -stderr "Creating Pseudo-3D Schedule with Uniform Random Density:\n" $dimCount 
   else
      nmrPrintf -stderr "Creating %sD Schedule with Uniform Random Density:\n" $dimCount 
   endif
endif

if ($dimCount == 2) then
   if ($vFlag) then
      nmrPrintf -stderr " Input:            %s\n" $inName 
      nmrPrintf -stderr " Sizes:            xT: %s yT: %s\n" $xT $yT
   endif

   @ iy = 0

   while ($iy < $yT)
      @ n++

      if ($iList[$n]) then
         nmrPrintf " %4d\n" $iy
         @ nn++
      endif

      @ iy++
   end

else if ($dimCount == 3) then
   if ($vFlag) then
      nmrPrintf -stderr " Input:            %s\n" $inName 
      nmrPrintf -stderr " Sizes:            xT: %s yT: %s zT: %s\n" $xT $yT $zT
   endif

   @ iz = 0

   while ($iz < $zT)
      @ iy = 0

      while ($iy < $yT)
         @ n++

         if ($iList[$n]) then
            nmrPrintf " %4d %4d\n" $iy $iz
            @ nn++
         endif

         @ iy++
      end

      @ iz++
   end

else if ($dimCount == 4) then
   if ($vFlag) then
      nmrPrintf -stderr " Input:            %s\n" $inName 
      nmrPrintf -stderr " Sizes:            xT: %s yT: %s zT: %s aT: %s\n" $xT $yT $zT $aT
   endif

   @ ia = 0

   while ($ia < $aT)
      @ iz = 0

      while ($iz < $zT)
         @ iy = 0
   
         while ($iy < $yT)
            @ n++

            if ($iList[$n]) then
               nmrPrintf " %4d %4d %4d\n" $iy $iz $ia
               @ nn++
            endif

            @ iy++
         end

         @ iz++
      end

      @ ia++
   end
endif

if ($vFlag) then
      nmrPrintf -stderr " Possible Samples: %d\n" $sampleCount 
      nmrPrintf -stderr " Samples Kept:     %d %.1f%%\n" $nn `MATH "100.0*$nn/$sampleCount"`
endif

exit 0
