
#
# Allocate and read a file header:

proc rdFDATA { inName fdataPtr } \
{
   upvar $fdataPtr fdata

   set fdata 0

   set thisName [format  $inName 1 1]
   set inUnit   [sysOpen $thisName r]

   if {$inUnit < 0} \
      {
       puts "Error opening input data file $thisName"
       return 1
      }

   set error [rdFDATAU $inUnit fdata]

   sysClose $inUnit

   return $error
}

proc rdFDATAU { inUnit fdataPtr } \
{
   upvar $fdataPtr fdata
   global FDATASIZE

   set fdata 0

   set swapInfo [sysByteSwap]
   set swapFlag [lindex $swapInfo 0]
   set autoFlag [lindex $swapInfo 1]

   set error [catch {set fdata [sysMalloc $FDATASIZE float]}]
    
   if {$error} \
      {
       puts stderr "Error allocating FDATA header array."
       return 1
      }

   if {$inUnit} \
      {
       set error [catch {sysSeek   $inUnit 0}]
       set error [catch {sysReadNS $inUnit $fdata $FDATASIZE}]
      } \
   else \
      {
       set error [catch {sysReadBNS $inUnit $fdata $FDATASIZE}]
      }
    
   if {$error} \
      {
       puts stderr "Error reading FDATA header array."
      }

   set status [testHdr $fdata]

   if {![strcmp $status swapped]} \
      {
       swapHdr $fdata

       if {$autoFlag} {sysByteSwap 1 $autoFlag}

       if {$autoFlag > 1} \
          {
           puts stderr "TCL Byte Swap Header: $FDATASIZE"
          }
      }

   fixHdr $fdata

   return 0
}

proc wrFDATA { outName fdata } \
{
   global FDATASIZE

   set thisName [format  $outName 1 1]
   set outUnit  [sysOpen $thisName rw]
                                                                                
   if {$outUnit < 0} \
      {
       puts "Error opening output data file $thisName"
       return 1
      }

   set error [catch {sysSeek  $outUnit 0}]
   set error [catch {sysWrite $outUnit $fdata $FDATASIZE}]
                                                                                
   sysClose $outUnit
                                                                                
   return $error
}

#
# Return byte-swap info relative to current system.

proc getDataSwapState { inName } \
{
   global FDATASIZE FDFLTORDER

   set thisName [format  $inName 1 1]
   set inUnit   [sysOpen $thisName r]

   if {$inUnit < 0} \
      {
       puts "Error opening input data file $thisName"
       return -1
      }

   set error [catch {set thisFDATA [sysMalloc $FDATASIZE float]}]

   if {$error} \
      {
       puts "Error allocating FDATA header array."
       return -1
      }

   set error [catch {sysSeek   $inUnit 0}]
   set error [catch {sysReadNS $inUnit $thisFDATA $FDATASIZE}]

   if {$error} \
      {
       puts "Error reading input data file $thisName"
       return -1
      }

   set val [getPtrVal $thisFDATA $FDFLTORDER]

   sysClose $inUnit
   sysFree  $thisFDATA

   if {abs($val - 2.345) < 0.001} {return 0}

   return 1 
}

#
# Return total number of points in individual NMRPipe file according to header.

proc getDataSize { fdata } \
{
    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
    set pipeFlag [getParmI $fdata FDPIPEFLAG NULL_DIM]
    set cubeFlag [getParmI $fdata FDCUBEFLAG NULL_DIM]
    set qSize    [getQuadSize [getParmI $fdata FDQUADFLAG NULL_DIM]]

    set xSize    1
    set ySize    1
    set zSize    1
    set aSize    1

    if {$dimCount >= 1} {set xSize [getParmI $fdata NDSIZE CUR_XDIM]}
    if {$dimCount >= 2} {set ySize [getParmI $fdata NDSIZE CUR_YDIM]}
    if {$dimCount >= 3} {set zSize [getParmI $fdata NDSIZE CUR_ZDIM]}
    if {$dimCount >= 4} {set aSize [getParmI $fdata NDSIZE CUR_ADIM]}

    if {!$pipeFlag} \
       {
        if {$cubeFlag} \
           {
            set aSize 1
           } \
        else \
           {
            set zSize 1
            set aSize 1
           }
       }

    if {$ySize == 1 && $zSize == 1 && $aSize == 1} {set dimCount 1} 

    set n [expr $xSize*$ySize*$zSize*$aSize*$qSize]

    return "$n $dimCount $qSize $xSize $ySize $zSize $aSize"
}

proc getSpecSizeInfo { specName } \
{
   set thisName [format $specName 1 1]
   set fdata    0

   if {![file exists $thisName]} {return ""}

   set error [rdFDATA $thisName fdata]

    if {$error} \
       {
        sysFree $fdata
        return  "" 
       }

    set inUnit   None

    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
    set pipeFlag [getParmI $fdata FDPIPEFLAG NULL_DIM]
    set cubeFlag [getParmI $fdata FDCUBEFLAG NULL_DIM]
    set fmtCount [getFmtCount $specName]

    set xSize    1
    set ySize    1
    set zSize    1
    set aSize    1
    set qSize    1

    if {$fmtCount == 0} \
       {
        if {!$pipeFlag && !$cubeFlag} \
           {
            if {$dimCount > 2} {set dimCount 2}
           }

        if {$cubeFlag && $dimCount > 3} {set dimCount 3}
       }

    if {$dimCount >= 1} {set xSize [getParmI $fdata NDSIZE CUR_XDIM]}
    if {$dimCount >= 2} {set ySize [getParmI $fdata NDSIZE CUR_YDIM]}
    if {$dimCount >= 3} {set zSize [getParmI $fdata NDSIZE CUR_ZDIM]}
    if {$dimCount >= 4} {set aSize [getParmI $fdata NDSIZE CUR_ADIM]}

    if {$ySize < 1} {set ySize 1}
    if {$zSize < 1} {set zSize 1}
    if {$aSize < 1} {set aSize 1}

    set qSize [getQuadSize [getParmI $fdata FDQUADFLAG NULL_DIM]]

    sysFree $fdata

    set fdata 0

    set n [expr $qSize*$xSize*$ySize*$zSize*$aSize]
  
    return "$n $dimCount $qSize $xSize $ySize $zSize $aSize"
}

#
# Return size list of the given spectrum.

proc getSpecSizeList { specName } \
{
    set fdata 0

    set thisName [format $specName 1 1]
    if {![file exists $thisName]} {return ""}

    set error [rdFDATA $thisName fdata]
    if {$error} {return ""} 

    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
    set sizeList ""

    for {set i 0} {$i < $dimCount} {incr i} \
       {
        set dim [lindex "X Y Z A B C" $i]
        lappend sizeList [getParmI $fdata NDSIZE CUR_${dim}DIM]
       }

    sysFree $fdata 

    return $sizeList
}

#
# Return Quad Size according to quad state.

proc getQuadSize { quadState } \
{
   if {int($quadState) == 0} {return 2}

   return 1
}

#
# Return tau value for the given spectrum.

proc getTau { specName } \
{
   if {![file exists $specName]} {return -1.0}

   set error [rdFDATA $specName fdata]

   if {$error} {return -1.0} 

   set tau [getParm $fdata FDTAU 0]

   sysFree $fdata
 
   return $tau
}

#
# Update the NDORIG value of the given dimension.

proc updateOrig { fdata fSize dim } \
{
    set sw  [getParm $fdata NDSW   $dim]
    set obs [getParm $fdata NDOBS  $dim]
    set car [getParm $fdata NDCAR  $dim]

    if {$sw  == 0.0} {set sw   1.0}
    if {$obs == 0.0} {set obs  1.0}
    if {$fSize == 0} {set size 1}

    set mid  [expr 1 + int($fSize/2)]
    set orig [expr $obs*$car - $sw*($fSize - $mid)/$fSize]

    setParm $fdata NDCENTER $mid  $dim
    setParm $fdata NDORIG   $orig $dim
}

proc getParmI { fdata parm dim } \
{
   set v [expr int([getParm $fdata $parm $dim])]

   return $v
}

proc getParmF { fdata parm dim } \
{
   set v [expr double([getParm $fdata $parm $dim])]

   return $v
}

proc setROITitle args \
{
   set thisARGV "setROITitle $args"

   set roiID      1
   set title      nmr
   set appendFlag 0

   getArgD $thisARGV -roi   roiID
   getArgD $thisARGV -title title

   if {[flagLoc $thisARGV -append]}   {set appendFlag 1}
   if {[flagLoc $thisARGV -noappend]} {set appendFlag 0}

   set fdata [lindex [getROIInfo -roi $roiID] 1]

   if {$appendFlag} \
      {
       set tStr [getParmStr $fdata FDTITLE NULL_DIM]

       if {[llength $tStr]} \
          {
           set tStr ${tStr}-$title
          } \
       else \
          {
           set tStr $title
          }
      } \
   else \
      {
       set tStr $title
      }

   setParmStr $fdata FDTITLE $tStr NULL_DIM
}

proc getROIParm { roiID parm dim } \
{
   set v [getParm [lindex [getROIInfo -roi $roiID] 1] $parm $dim]
   return $v
}

proc getROIParmI { roiID parm dim } \
{
   set v [getParmI [lindex [getROIInfo -roi $roiID] 1] $parm $dim]
   return $v
}

proc getROIParmF { roiID parm dim } \
{
   set v [getParmF [lindex [getROIInfo -roi $roiID] 1] $parm $dim]
   return $v
}

proc getROIInfoVals args \
{
   set thisArgc [llength $args]

   if {$thisArgc < 1} {return 0}
  
   set roiID [lindex $args 0]
   set rInfo [getROIInfo -roi $roiID]

   set sizeList [lindex $rInfo 2]
   set dimCount [llength $sizeList]

   lappend sizeList 1 1 1 1

   set n 1

   if {$thisArgc > 1} \
      {
       set rdataPtr [lindex $args 1]
       upvar $rdataPtr rdata

       set rdata    [lindex $rInfo 0]
      }

   if {$thisArgc > 2} \
      {
       set fdataPtr [lindex $args 2]
       upvar $fdataPtr fdata
       
       set fdata    [lindex $rInfo 1]
      }

   if {$thisArgc > 3} \
      {
       set xSizePtr [lindex $args 3]
       upvar $xSizePtr xSize

       set xSize    [lindex $sizeList 0]
       set n        [expr $n*$xSize]
      }

   if {$thisArgc > 4} \
      {
       set ySizePtr [lindex $args 4]

       upvar $ySizePtr ySize

       set ySize    [lindex $sizeList 1]
       set n        [expr $n*$ySize]
      }

   if {$thisArgc > 5} \
      {
       set zSizePtr [lindex $args 5]
       upvar $zSizePtr zSize

       set zSize    [lindex $sizeList 2]
       set n        [expr $n*$zSize]
      }

   if {$thisArgc > 6} \
      {
       set aSizePtr [lindex $args 6]
       upvar $aSizePtr aSize

       set aSize    [lindex $sizeList 3]
       set n        [expr $n*$aSize]
      }

   return $n
}

#
# Apply 1D functions to rows and columns of a matrix.
# Strings {$ptr} {$xSize} {$ySize} will be substituted. Example:
#  vApply2D $mat 512 128 {vCShift $ptr -3 $ySize} {vCShift $ptr 5 $ySize}

proc vApply2D { src xSize ySize xSizeFinal ySizeFinal vxFn vyFn } \
{
   set maxSize [getMax "$xSize $xSizeFinal $ySize $ySizeFinal"]

   set tmpBuff [sysMalloc [expr $xSizeFinal*$ySize] float]
   set ptr     [sysMalloc $maxSize float]

   set srcPtr  $src
   set destPtr $tmpBuff

   for {set iy 0} {$iy < $ySize} {incr iy} \
      {
       if {$xSizeFinal > $xSize} {vsSet $ptr 0.0 $xSizeFinal}

       vvMove $ptr $srcPtr $xSize

       eval [subst $vxFn]

       vvMove $destPtr $ptr $xSizeFinal

       set srcPtr  [incrPtr $srcPtr  $xSize]
       set destPtr [incrPtr $destPtr $xSizeFinal]
      }

   set srcPtr  $tmpBuff
   set destPtr $src

   for {set ix 0} {$ix < $xSizeFinal} {incr ix} \
      {
       if {$ySizeFinal > $ySize} {vsSet $ptr 0.0 $ySizeFinal}

       vvMove2 $ptr 1 0 $srcPtr $xSizeFinal 0 $ySize 1

       eval [subst $vyFn]

       vvMove2 $destPtr $xSizeFinal 0 $ptr 1 0 $ySizeFinal 1 

       set srcPtr  [incrPtr $srcPtr  1]
       set destPtr [incrPtr $destPtr 1]
      }

   sysFree $tmpBuff
   sysFree $ptr
}

#
# Extract subregion from an ND matrix.

proc getThisRegion { dest src sizeList regList } \
{
    set n 0

    set dimCount [llength $sizeList]

    if {$dimCount < 1 || $dimCount > 4 || 2*$dimCount != [llength $regList]} {return $n}

    set xSize 0
    set ySize 0
    set zSize 0
    set aSize 0

    set xSizeNew 0
    set ySizeNew 0
    set zSizeNew 0
    set aSizeNew 0

    if {$dimCount >= 1} \
       {
        set xSize [lindex $sizeList 0]
        set ix1   [lindex $regList 0]
        set ixn   [lindex $regList 1]

        if {$ix1 < 1}      {set ix1 1}
        if {$ix1 > $xSize} {set ix1 $xSize}

        if {$ixn < 1}      {set ixn 1}
        if {$ixn > $xSize} {set ixn $xSize}

        if {$ix1 > $ixn} \
           {
            set itmp $ix1
            set ix1  $ixn
            set ixn  $itmp
           }

        set xSizeNew [expr 1 + $ixn - $ix1]
       }

    if {$dimCount >= 2} \
       {
        set ySize [lindex $sizeList 1]
        set iy1   [lindex $regList 2]
        set iyn   [lindex $regList 3]

        if {$iy1 < 1}      {set iy1 1}
        if {$iy1 > $ySize} {set iy1 $ySize}

        if {$iyn < 1}      {set iyn 1}
        if {$iyn > $ySize} {set iyn $ySize}

        if {$iy1 > $iyn} \
           {
            set itmp $iy1
            set iy1  $iyn
            set iyn  $itmp
           }

        set ySizeNew [expr 1 + $iyn - $iy1]
       }

    if {$dimCount >= 3} \
       {
        set zSize [lindex $sizeList 2]
        set iz1   [lindex $regList 4]
        set izn   [lindex $regList 5]

        if {$iz1 < 1}      {set iz1 1}
        if {$iz1 > $zSize} {set iz1 $zSize}

        if {$izn < 1}      {set izn 1}
        if {$izn > $zSize} {set izn $zSize}

        if {$iz1 > $izn} \
           {
            set itmp $iz1
            set iz1  $izn
            set izn  $itmp
           }

        set zSizeNew [expr 1 + $izn - $iz1]
       }

    if {$dimCount >= 4} \
       {
        set aSize [lindex $sizeList 3]
        set ia1   [lindex $regList 6]
        set ian   [lindex $regList 7]

        if {$ia1 < 1}      {set ia1 1}
        if {$ia1 > $aSize} {set ia1 $aSize}

        if {$ian < 1}      {set ian 1}
        if {$ian > $aSize} {set ian $aSize}

        if {$ia1 > $ian} \
           {
            set itmp $ia1
            set ia1  $ian
            set ian  $itmp
           }

        set aSizeNew [expr 1 + $ian - $ia1]
       }

    if {$dimCount == 1} \
       {
        set n $xSizeNew

        vvMove $dest [incrPtr $src [expr $ix1 - 1]] $xSizeNew
       } \
    elseif {$dimCount == 2} \
       {
        set n [expr $xSizeNew*$ySizeNew]

        vvMove2 $dest 1 $xSizeNew \
               [incrPtr $src [expr $ix1 + ($iy1 - 1)*$xSize - 1]] 1 $xSize \
               $xSizeNew $ySizeNew
       } \
    elseif {$dimCount == 3} \
       {
        set n [expr $xSizeNew*$ySizeNew*$zSizeNew]

        for {set iz $iz1} {$iz <= $izn} {incr iz} \
           {
            vvMove2 [incrPtr $dest [expr ($iz - 1)*$xSizeNew*$ySizeNew]] 1 $xSizeNew \
                    [incrPtr $src  [expr ($iz - 1)*$xSize*$ySize + $ix1 + ($iy1 - 1)*$xSize - 1]] 1 $xSize \
                    $xSizeNew $ySizeNew
           }
       } \
    elseif {$dimCount == 4} \
       {
        set n [expr $xSizeNew*$ySizeNew*$zSizeNew*$aSizeNew]

        for {set ia $iz1} {$ia <= $ian} {incr ia} \
           {
            for {set iz $iz1} {$iz <= $izn} {incr iz} \
               {
                vvMove2 [incrPtr $dest [expr ($ia - 1)*$xSizeNew*$ySizeNew*$zSizeNew + ($iz - 1)*$xSizeNew*$ySizeNew]] 1 $xSizeNew \
                        [incrPtr $src  [expr ($ia - 1)*$xSize*$ySize*$zSize + ($iz - 1)*$xSize*$ySize + $ix1 + ($iy1 - 1)*$xSize - 1]] 1 $xSize \
                        $xSizeNew $ySizeNew
               }
           }
       }

    return $n
}

#
# Insert subregion into an ND matrix.

proc putThisRegion { src dest sizeList regList } \
{
    set n 0

    set dimCount [llength $sizeList]

    if {$dimCount < 1 || $dimCount > 4 || 2*$dimCount != [llength $regList]} {return $n}

    set xSize 0
    set ySize 0
    set zSize 0
    set aSize 0

    set xSizeNew 0
    set ySizeNew 0
    set zSizeNew 0
    set aSizeNew 0

    if {$dimCount >= 1} \
       {
        set xSize [lindex $sizeList 0]
        set ix1   [lindex $regList 0]
        set ixn   [lindex $regList 1]

        if {$ix1 < 1}      {set ix1 1}
        if {$ix1 > $xSize} {set ix1 $xSize}

        if {$ixn < 1}      {set ixn 1}
        if {$ixn > $xSize} {set ixn $xSize}

        if {$ix1 > $ixn} \
           {
            set itmp $ix1
            set ix1  $ixn
            set ixn  $itmp
           }

        set xSizeNew [expr 1 + $ixn - $ix1]
       }

    if {$dimCount >= 2} \
       {
        set ySize [lindex $sizeList 1]
        set iy1   [lindex $regList 2]
        set iyn   [lindex $regList 3]

        if {$iy1 < 1}      {set iy1 1}
        if {$iy1 > $ySize} {set iy1 $ySize}

        if {$iyn < 1}      {set iyn 1}
        if {$iyn > $ySize} {set iyn $ySize}

        if {$iy1 > $iyn} \
           {
            set itmp $iy1
            set iy1  $iyn
            set iyn  $itmp
           }

        set ySizeNew [expr 1 + $iyn - $iy1]
       }

    if {$dimCount >= 3} \
       {
        set zSize [lindex $sizeList 2]
        set iz1   [lindex $regList 4]
        set izn   [lindex $regList 5]

        if {$iz1 < 1}      {set iz1 1}
        if {$iz1 > $zSize} {set iz1 $zSize}

        if {$izn < 1}      {set izn 1}
        if {$izn > $zSize} {set izn $zSize}

        if {$iz1 > $izn} \
           {
            set itmp $iz1
            set iz1  $izn
            set izn  $itmp
           }

        set zSizeNew [expr 1 + $izn - $iz1]
       }

    if {$dimCount >= 4} \
       {
        set aSize [lindex $sizeList 3]
        set ia1   [lindex $regList 6]
        set ian   [lindex $regList 7]

        if {$ia1 < 1}      {set ia1 1}
        if {$ia1 > $aSize} {set ia1 $aSize}

        if {$ian < 1}      {set ian 1}
        if {$ian > $aSize} {set ian $aSize}

        if {$ia1 > $ian} \
           {
            set itmp $ia1
            set ia1  $ian
            set ian  $itmp
           }

        set aSizeNew [expr 1 + $ian - $ia1]
       }

    if {$dimCount == 1} \
       {
        set n $xSizeNew

        vvMove [incrPtr $dest [expr $ix1 - 1]] $src $xSizeNew
       } \
    elseif {$dimCount == 2} \
       {
        set n [expr $xSizeNew*$ySizeNew]

        vvMove2 [incrPtr $dest [expr $ix1 + ($iy1 - 1)*$xSize - 1]] 1 $xSize \
                $src 1 $xSizeNew \
                $xSizeNew $ySizeNew
       } \
    elseif {$dimCount == 3} \
       {
        set n [expr $xSizeNew*$ySizeNew*$zSizeNew]

        for {set iz $iz1} {$iz <= $izn} {incr iz} \
           {
            vvMove2 [incrPtr $dest [expr ($iz - 1)*$xSize*$ySize + $ix1 + ($iy1 - 1)*$xSize - 1]] 1 $xSize \
                    [incrPtr $src  [expr ($iz - 1)*$xSizeNew*$ySizeNew]] 1 $xSizeNew \
                    $xSizeNew $ySizeNew
           }
       } \
    elseif {$dimCount == 4} \
       {
        set n [expr $xSizeNew*$ySizeNew*$zSizeNew*$aSizeNew]

        for {set ia $iz1} {$ia <= $ian} {incr ia} \
           {
            for {set iz $iz1} {$iz <= $izn} {incr iz} \
               {
                vvMove2 [incrPtr $dest [expr ($ia - 1)*$xSize*$ySize*$zSize + ($iz - 1)*$xSize*$ySize + $ix1 + ($iy1 - 1)*$xSize - 1]] 1 $xSize \
                        [incrPtr $src  [expr ($ia - 1)*$xSizeNew*$ySizeNew*$zSizeNew + ($iz - 1)*$xSizeNew*$ySizeNew]] 1 $xSizeNew \
                        $xSizeNew $ySizeNew
               }
           }
       }

    return $n
}

#
# Get point coords for specified region.

proc getPointCoords { fdata specCoords } \
{
   set n     [llength $specCoords]
   set iList ""

   set dimList  "CUR_XDIM CUR_YDIM CUR_ZDIM CUR_ADIM"
   set dimCount 0 

   for {set i 0} {$i < $n} {incr i 2} \
      {
       set sx1 [lindex $specCoords $i]
       set sxn [lindex $specCoords [expr $i + 1]]

       if {![llength $sx1] || ![llength $sxn]} break

       incr dimCount

       if {$dimCount > 4} break

       set thisDim [lindex $dimList [expr $dimCount - 1]]

       set ix1 [expr int([spec2pnt $fdata $thisDim $sx1])]
       set ixn [expr int([spec2pnt $fdata $thisDim $sxn])]

       if {$ix1 > $ixn} \
          {
           set itmp $ix1
           set ix1  $ixn
           set ixn  $itmp
          }

       lappend iList $ix1 $ixn
      }

   return $iList
}

#
# Get dim keyword associated with a dimension name.

proc getDimKeyword { dimName fdata } \
{
    set dList {CUR_XDIM CUR_YDIM CUR_ZDIM CUR_ADIM}

    foreach dim $dList \
       {
        set thisName [getParmStr $fdata NDLABEL $dim]
        if {![strcmp $thisName $dimName]} {return $dim}
       }

    foreach dim $dList \
       {
        if {![strcmp $dim $dimName]} {return $dim}
       }

    set i 0

    foreach dim {X_AXIS Y_AXIS Z_AXIS A_AXIS} \
       {
        if {![strcmp $dim $dimName]} {return [lindex $dList $i]}
        incr i
       }

   if {![strcasecmp $dimName H_AXIS]} \
      {
       set tpFlag [getParmI $fdata FDTRANSPOSED NULL_DIM]

       if {$tpFlag == 0} \
          {
           return CUR_XDIM
          } \
       else \
          {
           return CUR_YDIM
          }
      }

   if {![strcasecmp $dimName V_AXIS]} \
      {
       set tpFlag [getParmI $fdata FDTRANSPOSED NULL_DIM]

       if {$tpFlag == 0} \
          {
           return CUR_YDIM
          } \
       else \
          {
           return CUR_XDIM
          }
      }

   return 0
}

#
# Get dim number associated with a dimension name.

proc getDim { dimName fdata } \
{
    foreach dim {1 2 3 4} \
       {
        set thisName [getParmStr $fdata NDLABEL $dim]

        if {![strcmp $thisName $dimName]} {return $dim}
       }

   return 0
}

#
# Get dim letter associated with a dimension name.

proc getDimLetter { dimName fdata } \
{
    set dimList {X Y Z A}

    set c [string toupper [string index $dimName 0]]
    set n [llength $dimList]

    set loc [lsearch -exact $dimList $c]
    if {$loc >= 0} {return [lindex $dimList $loc]}

    foreach dim $dimList \
       {
        if {![strcasecmp $dimName CUR${dim}_DIM]} {return $dim}
       }

    for {set dimID 1} {$dimID <= $n} {incr $dimID} \
       {
        if {![strcasecmp $dimName $dimID]} \
           {
            return [lindex $dimList [expr $dimID - 1]]
           }
       }

    set tpFlag [getParmI $fdata FDTRANSPOSED NULL_DIM]

    if {![strcasecmp $dimName CUR_HDIM] || ![strcasecmp $dimName H_AXIS]} \
       {
        if {$tpFlag} \
           {
            return Y
           } \
        else \
           {
            return X
           }
       }

    if {![strcasecmp $dimName CUR_VDIM] || ![strcasecmp $dimName V_AXIS]} \
       {
        if {$tpFlag} \
           {
            return X
           } \
        else \
           {
            return Y
           }
       }

    foreach dim $dimList \
       {
        set thisName [getParmStr $fdata NDLABEL CUR${dim}_DIM]
        if {![strcasecmp $dimName $thisName]} {return $dim}
       }

}
  

#
# Get the name of the directly-acquired dimension.

proc getDirectDim { fdata } \
{
   set tpFlag [getParmI $fdata FDTRANSPOSED NULL_DIM]

   if {$tpFlag} \
      {
       set dimName [getParmStr $fdata NDLABEL CUR_YDIM]
      } \
   else \
      {
       set dimName [getParmStr $fdata NDLABEL CUR_XDIM]
      }
 
   foreach dim {1 2 3 4} \
      {
       set dimCode [getParmI $fdata FDDIMORDER$dim NULL_DIM]

       if {$dimCode == 2} \
          {
           set dimName [getParmStr $fdata NDLABEL $dim]
           return $dimName
          }
      }

    return $dimName
}

proc getSecondDim { } \
{
   set tpFlag [getParmI $fdata FDTRANSPOSED NULL_DIM]

   if {$tpFlag} \
      {
       set dimName [getParmStr $fdata NDLABEL CUR_YDIM]
      } \
   else \
      {
       set dimName [getParmStr $fdata NDLABEL CUR_XDIM]
      }

   foreach dim {1 2 3 4} \
      {
       set dimCode [getParmI $fdata FDDIMORDER$dim NULL_DIM]

       if {$dimCode == 1} \
          {
           set dimName [getParmStr $fdata NDLABEL $dim]
           return $dimName
          }
      }

    return $dimName
}

#
# Get the spectrum type, HN, HC, NMR 

proc getSpecType args \
{
   set fdata [lindex $args 0]

   set nN 0
   set nC 0

   foreach dim {1 2 3 4} \
      {
       set dimName [string toupper [getParmStr $fdata NDLABEL $dim]]

       if {[string first 15N $dimName] >= 0} {incr nN}
       if {[string first N15 $dimName] >= 0} {incr nN}
       if {[string first N   $dimName] == 0} {incr nN}

       if {[string first 13C $dimName] >= 0} {incr nC}
       if {[string first C13 $dimName] >= 0} {incr nC}
       if {[string first C   $dimName] == 0} {incr nC}
      }

   if {$nN} {return HN}
   if {$nC} {return HC}

   return NMR 
}

#
# Get the name of the most likely 13C dimension.

proc get13CDim args \
{
   set fdata [lindex $args 0]

   if {[llength $args] > 1} \
      {
       set valPtr [lindex $args 1]
       upvar $valPtr bestVal 
      }

   if {[llength $args] > 2} \
      {
       set dimPtr [lindex $args 2]
       upvar $dimPtr bestDim
      }

   set bestDim 3
   set bestVal 0.0

   foreach dim {1 2 3 4} \
      {
       set size    [getParmI $fdata NDSIZE $dim]
       set dimName [getParmStr $fdata NDLABEL $dim]

       set x1 [pnt2spec $fdata $dim $size ppm]
       set xN [pnt2spec $fdata $dim 1     ppm]

       if {$x1 == $xN} continue

       set q1 $x1
       set qN $xN

       if {$q1 <  20.0} {set q1  20.0}
       if {$q1 > 100.0} {set q1 100.0}
       if {$qN <  20.0} {set qN  95.0}
       if {$qN > 100.0} {set qN 100.0}

       set val 0.0

       catch {set val [expr ($qN - $q1)/($xN - $x1)]}

       if {$val > $bestVal} \
          {
           set bestVal $val
           set bestDim $dim
          }

       set q1 $x1
       set qN $xN

       if {$q1 < 160.0} {set q1 160.0}
       if {$q1 > 190.0} {set q1 190.0}
       if {$qN < 160.0} {set qN 160.0}
       if {$qN > 190.0} {set qN 190.0}

       set val 0.0

       catch {set val [expr ($qN - $q1)/($xN - $x1)]}

       if {$val > $bestVal} \
          {
           set bestVal $val
           set bestDim $dim
          }

       foreach nName {C CO CA CACB 13C C13 13Cy 13Cx C13y C13z} \
          {
           if {![strcasecmp $nName $dimName]} \
              {
               set bestVal $val
               set bestDim $dim

               return $dimName
              }
          }
      } 

   set dimName [getParmStr $fdata NDLABEL $bestDim]

   return $dimName
}

#
# Get the name of the most likely 15N dimension.

proc get15NDim args \
{
   set fdata [lindex $args 0]

   if {[llength $args] > 1} \
      {
       set valPtr [lindex $args 1]
       upvar $valPtr bestVal 
      }

   if {[llength $args] > 2} \
      {
       set dimPtr [lindex $args 2]
       upvar $dimPtr bestDim
      }

   set bestDim 3
   set bestVal 0.0

   foreach dim {1 2 3 4} \
      {
       set size    [getParmI $fdata NDSIZE $dim]
       set dimName [getParmStr $fdata NDLABEL $dim]

       set x1 [pnt2spec $fdata $dim $size ppm]
       set xN [pnt2spec $fdata $dim 1     ppm]

       if {$x1 == $xN} continue

       set q1 $x1
       set qN $xN

       if {$q1 <  95.0} {set q1  95.0}
       if {$q1 > 145.0} {set q1 145.0}
       if {$qN <  95.0} {set qN  95.0}
       if {$qN > 145.0} {set qN 145.0}

       set val 0.0

       catch {set val [expr ($qN - $q1)/($xN - $x1)]}

       if {$val > $bestVal} \
          {
           set bestVal $val
           set bestDim $dim
          }

       foreach nName {N 15N N15 Ny Nz Na 15Ny 15Nz 15Na N15y N15z N15a} \
          {
           if {![strcasecmp $nName $dimName]} \
              {
               set bestVal $val
               set bestDim $dim

               return $dimName
              }
          }
      } 

   set dimName [getParmStr $fdata NDLABEL $bestDim]

   return $dimName
}

#
# Get the non-HN/15N dimension name if possible.

proc getIndirectDim args \
{
   set fdata    [lindex $args 0]

   set specType [getSpecType $fdata]
   set xName    [getDirectDim $fdata]

   if {![strcasecmp $specType HC]} \
      {
       set yName [get13CDim $fdata]
      } \
   else \
      {
       set yName [get15NDim $fdata]
      }

   foreach dim {1 2 3 4} \
      {
       set dimName [getParmStr $fdata NDLABEL    $dim]
       set dimCode [getParmI $fdata FDDIMORDER$dim NULL_DIM]
      }

   foreach dim {1 2 3 4} \
      {
       set dimName [getParmStr $fdata NDLABEL $dim]

       if {![strcmp $dimName $xName]} continue
       if {![strcmp $dimName $yName]} continue

       return $dimName
      }

   return Z_AXIS 
}

proc getFourthDim { fdata } \
{
   set hnName  [getDirectDim   $fdata]
   set nName   [get15NDim      $fdata]
   set fName   [getIndirectDim $fdata]
 
   foreach dim {1 2 3 4} \
      {
       set dimName [getParmStr $fdata NDLABEL    $dim]
       set dimCode [getParmI $fdata FDDIMORDER$dim NULL_DIM]
      }

   foreach dim {1 2 3 4} \
      {
       set dimName [getParmStr $fdata NDLABEL $dim]

       if {![strcmp $dimName $hnName]} continue
       if {![strcmp $dimName $nName]}  continue
       if {![strcmp $dimName $fName]}  continue

       return $dimName
      }

   return A_AXIS
}

#
# Find reasonable PPM position tolerance for the given dimension.

proc getPPMTol { dimCode fdata } \
{
   set obs [getParm $fdata NDOBS $dimCode]

   if {$obs > 400.0} \
      {
       set tol 0.04
      } \
   elseif {$obs > 95.0} \
      {
       set tol 0.2
      } \
   elseif {$obs > 40.0} \
      {
       set tol 0.15
      } \
   else \
      {
       set tol 0.1
      }

   return $tol
}

#
# Find likely peak table variable corresponding to HN ppm.

proc getHNTabVar { tab } \
{
   if {!$tab} {return None}

   set vList   [gdbGetInfo $tab -parms varNames]
   set bestVar None 
   set bestVal 0.0
 
   foreach vName {X_PPM Y_PPM Z_PPM A_PPM HN HN_PPM 1H H1 H} \
      {
       set val($vName) 0.0 

       if {[lsearch -exact $vList $vName] < 0} continue

       set vID    [gdbFindByName $tab $vName]
       set vType  [gdbGetInfo $vID -parms varType]

       if {[strcasecmp $vType float]} continue

       set eList  [gdbGetInfo $tab -parms entries]
       set eCount 0
       set n      0

       foreach entry $eList \
          {
           set p [gdbGet $entry $vName]

           if {$p > 999.0 || $p == 0.0} continue
           if {$p > 5.0 && $p < 12.0} {incr n}
 
           incr eCount
          }

       if {$eCount} {set val($vName) [expr double($n)/$eCount]}

       if {$val($vName) > $bestVal && $val($vName) > 0.9} \
          {
           set bestVar $vName
           set bestVal $val($vName)
          }
      }

   return $bestVar
}

#
# Find likely peak table variable corresponding to 15N ppm.

proc get15NTabVar { tab } \
{
   if {!$tab} {return None}

   set vList   [gdbGetInfo $tab -parms varNames]
   set bestVar None
   set bestVal 0.0   

   foreach vName {X_PPM Y_PPM Z_PPM A_PPM 15N N15 N N_PPM} \
      {
       set val($vName) 0.0 

       if {[lsearch -exact $vList $vName] < 0} continue

       set vID    [gdbFindByName $tab $vName]
       set vType  [gdbGetInfo $vID -parms varType]

       if {[strcasecmp $vType float]} continue

       set eList  [gdbGetInfo $tab -parms entries]
       set eCount 0
       set n      0

       foreach entry $eList \
          {
           set p [gdbGet $entry $vName] 

           if {$p > 999.0 || $p == 0.0} continue
           if {$p > 95.0 && $p < 145.0} {incr n} 

           incr eCount
          }

       if {$eCount} {set val($vName) [expr double($n)/$eCount]}

       if {$val($vName) > $bestVal && $val($vName) > 0.9} \
          {
           set bestVar $vName
           set bestVal $val($vName)
          }
      }

   return $bestVar
}

#
# Find likely peak table variable corresponding to indirect dimension ppm.

proc getIndirectTabVar { tab } \
{
   if {!$tab} {return None}

   set vList  [gdbGetInfo $tab -parms varNames]
   set hnName [getHNTabVar  $tab]
   set nName  [get15NTabVar $tab]

   set qList {X_PPM Y_PPM Z_PPM A_PPM  \
              CO CA CB CACB HA HB HAHB \
              CO_PPM CA_PPM CB_PPM CACB_PPM HA_PPM HB_PPM HAHB_PPM}

   foreach vName $qList \
      {
       if {![strcmp $vName $hnName]} continue
       if {![strcmp $vName $nName]}  continue

       set val($vName) 0.0

       if {[lsearch -exact $vList $vName] < 0} continue

       set vID    [gdbFindByName $tab $vName]
       set vType  [gdbGetInfo $vID -parms varType]

       if {[strcasecmp $vType float]} continue

       return $vName
      }

    return None
}

#
# Extract the units label from a spectral location or width.

proc getUnitsLabel { sOrig } \
{
   set s [string tolower $sOrig]
   set n [string length  $s]
 
   foreach thisUnit {pts hz ppm wn cm mm mic nm wl-nm in c1 c2 pix hi sec user % @} \
      {
       set loc [string last $thisUnit $s]

       if {$loc >= 0 && $loc == ($n - [string length $thisUnit])} {return $thisUnit}
      }

   return pts
}

#
# Count number of % fmts in fName

proc getFmtCount { fName } \
{
   set fmtCount 0

   set n [expr [string length $fName] - 1]

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $fName $i]
       if {[strcmp $c "%"]} continue

       set c [string index $fName [expr $i + 1]]

       if {![strcmp $c "%"]} \
          {
           incr i
           continue
          }

       incr fmtCount
      }

   return $fmtCount
}

#
# Convert a file name to its 3D format if possible.

proc name2fmt { fName } \
{
   for {set iz 1} {$iz <= 999} {incr iz} \
      {
       set infix [format %03d $iz 1]
       set loc   [string last $infix $fName]
      
       if {$loc >= 0} \
          {
           set    fFmt [string range $fName 0 [expr $loc - 1]]
           append fFmt %03d
           append fFmt [string range $fName [expr $loc + 3] end] 

           return $fFmt
          } 
      } 

    return $fName
}

#
# Extract value and trailing label (% or @) from contour height.

proc getHiVal { hi } \
{
   set val [string trim $hi " @%"]
   return $val
}

proc getHiLab { hi } \
{
   set c [string index [string trim $hi] end]

   if {![strcmp $c "%"] || ![strcmp $c "@"]} {return $c}

   return ""
}          

proc incrHiVal { hi f } \
{
   set thisHi  [getHiVal $hi]
   set thisLab [getHiLab $hi]
 
   if {![string is double $thisHi]} {return $hi}
 
   if {![strcmp $thisLab "%"] || ![strcmp $thisLab "@"]} \
      {
       set fmt %.4f
      } \
   else \
      {
       set fmt %.4e
      }
 
   return [format ${fmt}%s [expr $thisHi*$f] $thisLab]
}

proc decrHiVal { hi f } \
{
   if {$f == 0.0} {return 0.0}
   return [incrHiVal $hi [expr 1.0/$f]]
}

proc getSpecContHi { inName hi pLev nLev mult } \
{
   set msg "Contour Height"
   
   if {![strcasecmp $mult Auto] || ![strcasecmp $mult Default]} {set mult 7.0}
   if {![strcasecmp $pLev Auto] || ![strcasecmp $pLev Default]} {set pLev 10}
   if {![strcasecmp $nLev Auto] || ![strcasecmp $nLev Default]} {set nLev 10}

   if {![strcasecmp $hi Auto]} \
      {
       set noise [estSpecNoise $inName]
       set hi    [expr $noise*$mult]
       if {$hi == 0.0} {set hi 1.0e+6}

       set hi  [format %.3e $hi]
       set msg [format "Auto Mode Contour Height (Noise %.3e)" $noise]
      } \
   elseif {![strcasecmp $hi Full]} \
      {
       set range [getSpecRange $inName minVal maxVal]
       set hi    [getFirstLevel $pLev $nLev $minVal $maxVal 1.3]
       if {$hi == 0.0} {set hi 1.0e+6}

       set fmt "Full Mode Contour Height (%.2e to %.2e)"
       set hi  [format %.3e $hi]
       set msg [format $fmt $minVal $maxVal]
      } \
   elseif {[string last % $hi] > 0} \
      {
       set range [getSpecRange $inName minVal maxVal]
       set fmt   "Percent Mode Contour Height (%s of %.2e to %.2e)"
       set msg   [format $fmt $hi $minVal $maxVal]

       set v  [getMaxAbs "$minVal $maxVal"]
       set p  [string trim $hi " %"]
       set hi [expr $p*$v/100.0]
       if {$hi == 0.0} {set hi 1.0e+6}

       set hi [format %.3e $hi]
      } \
   elseif  {[string last @ $hi] > 0} \
      {
       set p     [string trim $hi " @"]
       set error [getSpecSelect $inName $p hi]
       set fmt   "Fraction of Data Contour Height %s %.3e"
       set msg   [format $fmt $p% $hi]
       if {$hi == 0.0} {set hi 1.0e+6}

       set hi [format %.3e $hi]
      } \
   else \
      {
       set msg "Specified Contour Height"
      }

   return "$hi $msg"
}

proc estSpecNoiseND { specName } \
{
    set noise     0.0
    set noiseList ""

    set xSize  1
    set ySize  1
    set zSize  1
    set aSize  1

    set inUnit None

    set thisName [format $specName 1 1]

    if {![strcasecmp $specName None]} {return $noise}
    if {![file exists $thisName]}     {return $noise} 

    set inUnit [sysOpen $thisName "r"]

    if {!$inUnit} \
       {
        puts   "estSpecNoiseND Error Opening File $thisName"
        return -1.0
       }

    set error [rdFDATAU $inUnit fdata]

    if {$error} \
       {
        puts stderr "estSpecNoiseND Error Reading Header from $thisName"
        sysFree $fdata
        return -1.0
       }

    sysClose $inUnit

    set inUnit   None

    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
    set pipeFlag [getParmI $fdata FDPIPEFLAG NULL_DIM]
    set cubeFlag [getParmI $fdata FDCUBEFLAG NULL_DIM]
    set fmtCount [getFmtCount $specName]

    set xSize    1
    set ySize    1
    set zSize    1
    set aSize    1
    set qSize    1

    if {$fmtCount == 0} \
       {
        if {!$pipeFlag && !$cubeFlag} \
           {
            if {$dimCount > 2} {set dimCount 2}
           }

        if {$cubeFlag && $dimCount == 4} {set dimCount 3}
       }

    if {$dimCount >= 1} {set xSize [getParmI $fdata NDSIZE CUR_XDIM]}
    if {$dimCount >= 2} {set ySize [getParmI $fdata NDSIZE CUR_YDIM]}
    if {$dimCount >= 3} {set zSize [getParmI $fdata NDSIZE CUR_ZDIM]}
    if {$dimCount >= 4} {set aSize [getParmI $fdata NDSIZE CUR_ADIM]}

    if {$ySize < 1} {set ySize 1}
    if {$zSize < 1} {set zSize 1}
    if {$aSize < 1} {set aSize 1}

    set qSize [getQuadSize [getParmI $fdata FDQUADFLAG NULL_DIM]]
    set roiID [getROIInfo -free]

    if {$dimCount == 1} \
       {
        readROI -roi $roiID -in $specName -ndim 1 -x X_AXIS 0% 100% 
        set noise [getROIStat $roiID vEstNoise]
       } \
    elseif {$dimCount == 2} \
       {
        set dx $xSize 
        set dy 128

        for {set iy1 1} {$iy1 <= $ySize} {incr iy1 $dy} \
           {
            set iyn [expr $iy1 + $dy - 1]
            if {$iyn > $ySize} {set iyn $ySize}

            for {set ix1 1} {$ix1 <= $xSize} {incr ix1 $dx} \
               {
                set ixn [expr $ix1 + $dx - 1]
                if {$ixn > $xSize} {set ixn $xSize}

                readROI -roi $roiID -in $specName -ndim 2 -x X_AXIS $ix1 $ixn -y Y_AXIS $iy1 $iyn
                lappend noiseList [getROIStat $roiID vEstNoise]
               }
           }

        set noise [getMedian $noiseList]
       } \
    elseif {$dimCount == 3} \
       {
        set dx $xSize 
        set dy 128
        set dz 128

        for {set iz1 1} {$iz1 <= $zSize} {incr iz1 $dz} \
           {
            set izn [expr $iz1 + $dz - 1]
            if {$izn > $zSize} {set izn $zSize}

            for {set iy1 1} {$iy1 <= $ySize} {incr iy1 $dy} \
               {
                set iyn [expr $iy1 + $dy - 1]
                if {$iyn > $ySize} {set iyn $ySize}

                for {set ix1 1} {$ix1 <= $xSize} {incr ix1 $dx} \
                   {
                    set ixn [expr $ix1 + $dx - 1]
                    if {$ixn > $xSize} {set ixn $xSize}
                    readROI -roi $roiID -in $specName -ndim 3 -x X_AXIS $ix1 $ixn -y Y_AXIS $iy1 $iyn -z Z_AXIS $iz1 $izn
                    lappend noiseList [getROIStat $roiID vEstNoise]
                   }
               }
           }

        set noise [getMedian $noiseList]
       } \
    else \
       {
        set dx $xSize
        set dy 64 
        set dz 32 
        set da 16 

        for {set ia1 1} {$ia1 <= $aSize} {incr ia1 $da} \
           {
            set ian [expr $ia1 + $da - 1]
            if {$ian > $aSize} {set ian $aSize}

            for {set iz1 1} {$iz1 <= $zSize} {incr iz1 $dz} \
               {
                set izn [expr $iz1 + $dz - 1]
                if {$izn > $zSize} {set izn $zSize}
   
                for {set iy1 1} {$iy1 <= $ySize} {incr iy1 $dy} \
                    {
                     set iyn [expr $iy1 + $dy - 1]
                     if {$iyn > $ySize} {set iyn $ySize}

                     for {set ix1 1} {$ix1 <= $xSize} {incr ix1 $dx} \
                        {
                         set ixn [expr $ix1 + $dx - 1]
                         if {$ixn > $xSize} {set ixn $xSize}

                         readROI -roi $roiID -in $specName -ndim 4 -x X_AXIS $ix1 $ixn -y Y_AXIS $iy1 $iyn -z Z_AXIS $iz1 $izn -a A_AXIS $ia1 $ian
                         lappend noiseList [getROIStat $roiID vEstNoise]
                        }
                    }
               }
           }

        set noise [getMedian $noiseList]
       }

    freeROI -roi $roiID

    return $noise
}

proc getSpecSelect { specName pct hiPtr } \
{
    upvar $hiPtr hi

    set fdata 0
    set hi    0.0

    if {![strcasecmp $specName None]} {return 0}

    set thisName [format $specName 1 1]
    if {![file exist $thisName]} {return 0.0}

    set sizeInfo [getSpecSizeInfo $specName]
    set dimCount [lindex $sizeInfo 1]
    set id       [getFreeROIID]

    if {$dimCount == 1} \
       {
        readROI -in $specName -roi $id -ndim 1 -x X_AXIS 0% 100% 
       } \
    elseif {$dimCount == 2} \
       {
        readROI -in $specName -roi $id -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%
       } \
    elseif {$dimCount == 3} \
       {
        readROI -in $specName -roi $id -ndim 3 -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -z Z_AXIS 0% 100%
       } \
    elseif {$dimCount == 4} \
       {
        readROI -in $specName -roi $id -ndim 4 -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -z Z_AXIS 0% 100% -a A_AXIS 0% 100%
       } \
    else \
       {
        return 0.0
       }

    set n [getROISize $id]

    if {$n < 1} \
       {
        freeROI -roi $id
        return 0.0
       }

    set rdata [lindex [getROIInfo -roi $id] 0]
    
    if {$n == 1} \
       {
        set hi [getPtrVal $rdata]
        freeROI -roi $id
        return $hi
       }

    set loc [expr int(0.5 + 0.01*(100.0 - $pct)*($n - 1))]

    rClip loc 0 [expr $n - 1]

    vAbs  $rdata $n
    vSort $rdata $n

    set hi [getPtrVal $rdata $loc]

    freeROI -roi $id

    return $hi
}

proc getSpecRange { specName minPtr maxPtr } \
{
    upvar $minPtr minVal
    upvar $maxPtr maxVal

    global FDATASIZE

    set minVal 0.0
    set maxVal 0.0

    set xSize  1
    set ySize  1
    set zSize  1
    set aSize  1

    set inUnit None

    if {![strcasecmp $specName None]} {return 0.0}

    set thisName [format $specName 1 1]
    if {![file exists $thisName]} {return 0.0} 

    set inUnit [sysOpen $thisName "r"]

    if {!$inUnit} \
       {
        puts   "GetSpecRange Error Opening File $thisName"
        return -1.0
       }

    set error [rdFDATAU $inUnit fdata]

    if {$error} \
       {
        puts stderr "GetSpecRange Error Reading Header from $thisName"
        sysFree $fdata
        return -1.0
       }

    sysClose $inUnit

    set inUnit   None

    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
    set pipeFlag [getParmI $fdata FDPIPEFLAG NULL_DIM]
    set cubeFlag [getParmI $fdata FDCUBEFLAG NULL_DIM]
    set fmtCount [getFmtCount $specName]

    set xSize    1
    set ySize    1
    set zSize    1
    set aSize    1
    set qSize    1

    if {$fmtCount == 0} \
       {
        if {!$pipeFlag && !$cubeFlag} \
           {
            if {$dimCount > 2} {set dimCount 2}
           }

        if {$cubeFlag && $dimCount == 4} {set dimCount 3}
       }
    
    if {$dimCount >= 1} {set xSize [getParmI $fdata NDSIZE CUR_XDIM]}
    if {$dimCount >= 2} {set ySize [getParmI $fdata NDSIZE CUR_YDIM]}
    if {$dimCount >= 3} {set zSize [getParmI $fdata NDSIZE CUR_ZDIM]}
    if {$dimCount >= 4} {set aSize [getParmI $fdata NDSIZE CUR_ADIM]}

    if {$ySize < 1} {set ySize 1}
    if {$zSize < 1} {set zSize 1}
    if {$aSize < 1} {set aSize 1}

    set qSize [getQuadSize [getParmI $fdata FDQUADFLAG NULL_DIM]]

    set n [expr $xSize*$ySize*$qSize]

    set error [catch {set rdata [sysMalloc $n float]}]

    if {$error} \
       {
        puts stderr "GetSpecRange Error Allocating Data."
        sysFree $fdata
        return -1.0
       }

    set rPtr $rdata

    for {set ia 1} {$ia <= $aSize} {incr ia} \
       {
        for {set iz 1} {$iz <= $zSize} {incr iz} \
            {
             set openFlag  0
             set closeFlag 0

             if {$fmtCount == 0} \
                {
                 set thisName $specName

                 if {$iz == 1 && $ia == 1} {set openFlag 1}
                 if {$iz == $zSize && $ia == $aSize} {set closeFlag 1}
                } \
             elseif {$fmtCount == 1 && $dimCount == 4} \
                {
                 set thisName [format $specName $ia]

                 if {$iz == 1} {set openFlag 1}
                 if {$iz == $zSize} {set closeFlag 1}
                } \
             elseif {$fmtCount == 1 && $dimCount == 3} \
                {
                 set thisName  [format $specName $iz]
                 set openFlag  1
                 set closeFlag 1
                } \
             else \
                {
                 set thisName  [format $specName $ia $iz]
                 set openFlag  1
                 set closeFlag 1
                }

             if {$openFlag} \
                {                
                 set inUnit [sysOpen $thisName "r"]
                 sysRead  $inUnit $fdata $FDATASIZE
                }

             sysRead $inUnit $rdata $n

             if {$closeFlag} \
                {
                 sysClose $inUnit
                 set inUnit None
                }

             set thisMin [vMin $rdata $n]
             set thisMax [vMax $rdata $n]

             if {$ia == 1 && $iz == 1} \
                {
                 set minVal $thisMin
                 set maxVal $thisMax
                } \
             else \
                {
                 if {$thisMin < $minVal} {set minVal $thisMin}
                 if {$thisMax > $maxVal} {set maxVal $thisMax}
                }
            }
       }

    sysFree $fdata
    sysFree $rdata

    if {[strcasecmp $inUnit None]} {catch "close $inUnit"}

    set range [expr $maxVal - $minVal]

    return $range
}

proc getFirstLevel { pLev nLev minVal maxVal factor } \
{
   if {abs($minVal) > abs($maxVal)} \
      {
       set hi [expr abs($minVal)]
      } \
   else \
      {
       set hi [expr abs($maxVal)]
      }

   if {$pLev > $nLev} \
      {
       set n $pLev
      } \
   else \
      {
       set n $nLev
      }

   for {set i 0} {$i <= $n} {incr i} \
      {
       set hi [expr $hi/double($factor)]
      }

   return $hi
}

proc getFreeROIID {} \
{
   for {set id 1} {$id < 1000000} {incr id} \
      {
       set rInfo ""

       catch {set rInfo [getROIInfo -roi $id]}
       if {![info exists rInfo]} {set rInfo ""}

       if {![llength $rInfo]} {return $id}
      }

   return $id
}

proc getROIrdata { roiID } \
{
   set roiInfo [getROIInfo -roi $roiID]

   if {[llength $roiInfo]} {return [lindex $roiInfo 0]}
   return 0
}

proc getROIfdata { roiID } \
{
   set roiInfo [getROIInfo -roi $roiID]

   if {[llength $roiInfo]} {return [lindex $roiInfo 1]}
   return 0
}

proc getROISize { roiID } \
{
   set roiInfo  [getROIInfo -roi $roiID]
   set sizeInfo [lindex $roiInfo 2]
   set n        1

   foreach size $sizeInfo {set n [expr $n*$size]}

   return $n
}

proc getROIStat { roiID procName } \
{
   set roiInfo  [getROIInfo -roi $roiID]
   set rdata    [lindex $roiInfo 0]
   set sizeInfo [lindex $roiInfo 2]
   set n        1

   foreach size $sizeInfo {set n [expr $n*$size]}

   set s [$procName $rdata $n]

   return $s
}

proc getROIVals { roiID } \
{
   set roiInfo  [getROIInfo -roi $roiID]
   set rdata    [lindex $roiInfo 0]
   set sizeInfo [lindex $roiInfo 2]
   set n        1

   foreach size $sizeInfo {set n [expr $n*$size]}

   set vList ""

   for {set i 0} {$i < $n} {incr i} \
      {
       lappend vList [getPtrVal $rdata $i]
      }

   return $vList
}

proc getROINthVal { roiID nLoc } \
{
   set roiInfo  [getROIInfo -roi $roiID]
   set rdata    [lindex $roiInfo 0]
   set sizeInfo [lindex $roiInfo 2]
   set n        1

   foreach size $sizeInfo {set n [expr $n*$size]}

   if {$nLoc < 1 || $nLoc > $n} {return 0.0}

   set val [getPtrVal $rdata [expr $nLoc - 1]]

   return $val
}

proc findNMRInput { dirList ext } \
{
   foreach d $dirList \
      {
       set fList "$d/test%05d.$ext $d/spec%05d.$ext $d/test%04d.$ext $d/spec%04d.ext $d/test%03d.$ext $d/spec%03d.ext $d/test%03d%03d.$ext $d/spec%03d%03d.ext $d/test%02d%03d.$ext $d/spec%02d%03d.ext test.$ext spec.$ext"

       foreach f $fList \
          {
           if {[file exists [format $f 1 1]]} {return $f}
          }
      }

   return None
}

proc findNUSInput args \
{
   set dirName [lindex $args 0]

   set fList ""

   foreach f [glob -nocomplain $dirName/nuslist $dirName/sampling.sch $dirName/schedule[file tail [pwd]]] {lappend fList [stripDotSlash $f]}

   foreach f [lsort [glob -nocomplain $dirName/schedule*]] {lappend fList [stripDotSlash $f]}

   foreach f [glob -nocomplain $dirName/*.hdr_3 $dirName/*.sched $dirName/vclist.* $dirName/vclist]  {lappend fList [stripDotSlash $f]}

   return $fList
}

#
# Bottom.
