#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Summarize Peak Region Sizes and Heights from Peak Table:"
    puts " -in     inName   \[test.tab\]        Input Peak Table."
    puts " -parent specName \[test.ft2\]        Corresponding Spectrum."
    puts " -units  units    \[PPM\]             Units for Reporting (% Hz PPM)."
    puts " -lwMult lwMult   \[2.0\]             Multiplier for Values for Linewidth."
    puts " -hiMult hiMult   \[1.0\]             Multiplier for Values for Height."
    puts " -int                               Report Point Values as Integers."
    puts " -brief                             Brief Mode (Report Name/Value Pairs)."
    exit 0
   }

set inName    test.tab 
set specName  test.ft2
set lwMult    2.0
set hiMult    1.0
set specUnits PPM 
set intFlag   0
set verbose   1

getArgD $ARGV -in     inName
getArgD $ARGV -parent specName
getArgD $ARGV -lwMult lwMult
getArgD $ARGV -hiMult hiMult
getArgD $ARGV -units  specUnits

if {[flagLoc $ARGV -noverb]} {set verbose 0}
if {[flagLoc $ARGV -brief]}  {set verbose 0}

if {[flagLoc $ARGV -int]}    {set intFlag 1}
if {[flagLoc $ARGV -noint]}  {set intFlag 0}

set thisName [format $specName 1 1]

if {![file exists $inName]} \
   {
    puts stderr "findRegSize.tcl Error Finding Input Table $inName"
    exit 1
   }

if {![file exist $thisName]} \
   {
    puts stderr "findRegSize.tcl Error Finding Input Spectrum $specName"
    exit 1
   }

set error [rdFDATA $thisName fdata]

if {$error} \
   {
    puts stderr "findRegSize.tcl Reading Header in Spectrum $specName"
    exit 1
   }

set t     [gdbRead table -in $inName]
set vList [gdbGetInfo $t -parms varNames]
set n     [gdbGetInfo $t -parms entryCount]

if {$verbose} {puts "Peaks: $n"}

foreach dim {X Y Z A} \
   {
    if {[lsearch -exact $vList ${dim}1] < 0} break

    if {$verbose} {puts ""}

    set ldim   [string tolower $dim]
    set x1List [getTabVals $t ${dim}1]
    set x3List [getTabVals $t ${dim}3]
    set xwList [getTabVals $t ${dim}W]
    set dList  ""

    for {set i 0} {$i < $n} {incr i} {lappend dList [expr 1 + [lindex $x3List $i] - [lindex $x1List $i]]}

    set thisList $dList
    set thisMult 1.0

    foreach parm {Reg w} \
       {
        set minVal [expr $thisMult*[getMin $thisList]]
        set maxVal [expr $thisMult*[getMax $thisList]]
        set avgVal [expr $thisMult*[getAvg $thisList]]
        set medVal [expr $thisMult*[getSimpleMedian $thisList]]

        set minW   [pnt2specW $fdata ${dim}_AXIS $minVal $specUnits]
        set maxW   [pnt2specW $fdata ${dim}_AXIS $maxVal $specUnits]
        set avgW   [pnt2specW $fdata ${dim}_AXIS $avgVal $specUnits]
        set medW   [pnt2specW $fdata ${dim}_AXIS $medVal $specUnits]

        if {$intFlag} \
           {
            set minVal [expr int(0.5 + $minVal)]
            set maxVal [expr int(0.5 + $maxVal)]
            set avgVal [expr int(0.5 + $avgVal)]
            set medVal [expr int(0.5 + $medVal)]
           }

        if {$verbose} \
           { 
            if {$intFlag} \
               {
                set f "%3d"
               } \
            else \
               {
                set f "%6.3f"
               }

            if {![strcasecmp $parm Reg]} \
               {
                set fmt "%s-Axis Peak Region Size.   Min: ${f}pts %6.4f$specUnits Max: ${f}pts %6.4f$specUnits Avg: ${f}pts %6.4f$specUnits Median: ${f}pts %6.4f$specUnits"
               } \
            else \
               {
                set fmt "%s-Axis Peak Width (x $lwMult). Min: ${f}pts %6.4f$specUnits Max: ${f}pts %6.4f$specUnits Avg: ${f}pts %6.4f$specUnits Median: ${f}pts %6.4f$specUnits"
               }

            puts [format $fmt $dim $minVal $minW $maxVal $maxW $avgVal $avgW $medVal $medW]
           } \
        else \
           {
            if {![strcasecmp $parm W] || [strcasecmp $dim X]} {puts -nonewline " "}

            if {$intFlag} \
               {
                set f "%d"
               } \
            else \
               {
                set f "%.3f"
               }

            set p "%.4f"

            if {![strcmp $specUnits "%"]} \
               {
                set uu Pct
               } \
            else \
               {
                set uu $specUnits
               }

            puts -nonewline [format "${ldim}${parm}MinPts $f ${ldim}${parm}MaxPts $f ${ldim}${parm}AvgPts $f ${ldim}${parm}MedianPts $f " $minVal $maxVal $avgVal $medVal]
            puts -nonewline [format "${ldim}${parm}Min$uu $p ${ldim}${parm}Max$uu $p ${ldim}${parm}Avg$uu $p ${ldim}${parm}Median$uu $p"  $minW   $maxW   $avgW   $medW]
           }

        set thisList $xwList
        set thisMult $lwMult
       }
   }

set thisList [getTabVals $t HEIGHT]
set thisMult $hiMult

set minVal [expr $thisMult*[getMin $thisList]]
set maxVal [expr $thisMult*[getMax $thisList]]
set avgVal [expr $thisMult*[getAvg $thisList]]
set medVal [expr $thisMult*[getSimpleMedian $thisList]]

if {$verbose} \
   {
    puts ""
    puts [format "Peak Height.               Min: % .4e  Max: % .4e  Avg: % .4e  Median: % .4e" $minVal $maxVal $avgVal $medVal]
   } \
else \
   {
    puts -nonewline [format " hiMin %.4e hiMax %.4e hiAvg %.4e hiMedian %.4e" $minVal $maxVal $avgVal $medVal]
   }

set absList  ""

foreach hi $thisList {lappend absList [expr abs($hi)]}

set minVal [expr $thisMult*[getMin $absList]]
set maxVal [expr $thisMult*[getMax $absList]]
set avgVal [expr $thisMult*[getAvg $absList]]
set medVal [expr $thisMult*[getSimpleMedian $absList]]

if {$verbose} \
   {
    puts [format "Absolute Peak Height.      Min: % .4e  Max: % .4e  Avg: % .4e  Median: % .4e" $minVal $maxVal $avgVal $medVal]
   } \
else \
   {
    puts -nonewline [format " hiAbsMin %.4e hiAbsMax %.4e hiAbsAvg %.4e hiAbsMedian %.4e" $minVal $maxVal $avgVal $medVal]
   }

if {!$verbose} {puts " lwMult $lwMult hiMult $hiMult N $n"}

exit 0
