#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Find Optimal Shift to Align 2D Spectra:"
    puts " -in     inName  \[test.ft2\]  Spectrum to Align."
    puts " -ref    refName \[ref.ft2\]   Spectrum Target for Alignment."
    puts " -out    outName \[None\]      Shifted Spectrum Output, or Keyword None."
    puts " -x1     x1      \[0%\]        X-Axis Input Region Start."
    puts " -xn     xn      \[100%\]      X-Axis Input Region End."
    puts " -y1     y1      \[0%\]        Y-Axis Input Region Start."
    puts " -yn     yn      \[100%\]      Y-Axis Input Region End."
    puts " -sxMin  sxMin   \[-10\]       Initial X-Axis Minimum Shift, Points."
    puts " -sxMax  sxMax   \[10\]        Initial X-Axis Maximum Shift, Points."
    puts " -sxStep sxStep  \[1\]         Initial X-Axis Shift Step, Points."
    puts " -syMin  sxMin   \[-10\]       Initial Y-Axis Minimum Shift, Points."
    puts " -syMax  sxMax   \[10\]        Initial Y-Axis Maximum Shift, Points."
    puts " -syStep sxStep  \[1\]         Initial X-Axis Shift Step, Points."
    puts " -iter   nIter   \[3\]         Iteration Count."
    puts " -verb                       Verbose Mode ON (Default)."
    puts " -noverb                     Verbose Mode OFF."
    exit 0
   }

set inName  test.ft2
set refName ref.ft2
set outName None

set extName tmp_ext.ft2
set zfName  tmp_zf.ft2

set x1 0%
set xn 100%
set y1 0% 
set yn 100%

set sxMin -10 
set sxMax  10 
set sxStep  1

set syMin -10 
set syMax  10 
set syStep  1

set iterCount 3

set verbose   1

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -ref refName
getArgD $ARGV -tmp tmpName

getArgD $ARGV -x1  x1
getArgD $ARGV -xn  xn
getArgD $ARGV -y1  y1
getArgD $ARGV -yn  yn

getArgD $ARGV -sxMin  sxMin
getArgD $ARGV -sxMax  sxMax
getArgD $ARGV -sxStep sxStep

getArgD $ARGV -syMin  syMin
getArgD $ARGV -syMax  syMax
getArgD $ARGV -syStep syStep

getArgD $ARGV -iter   iterCount

if {[flagLoc $ARGV -verb]}   {set verbose 1} 
if {[flagLoc $ARGV -noverb]} {set verbose 0} 

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Spectrum $inName"
    exit 1
   }

if {![file exists $refName]} \
   {
    puts stderr "Error Finding Reference Spectrum $refName"
    exit 1
   }

set error [rdFDATA $refName fdata]

if {$error} \
   {
    puts stderr "Error Reading Header from Input Spectrum $inName"
    exit 1
   }

set kx1 [expr int( 0.5 + [spec2pnt $fdata CUR_XDIM $x1] )]
set kxn [expr int( 0.5 + [spec2pnt $fdata CUR_XDIM $xn] )]
set ky1 [expr int( 0.5 + [spec2pnt $fdata CUR_YDIM $y1] )]
set kyn [expr int( 0.5 + [spec2pnt $fdata CUR_YDIM $yn] )]

readROI -roi 1 -ndim 2 -in $refName -x X_AXIS $kx1 $kxn -y Y_AXIS $ky1 $kyn
readROI -roi 2 -ndim 2 -in $inName  -x X_AXIS $kx1 $kxn -y Y_AXIS $ky1 $kyn

set refInfo  [getROIInfo -roi 1]
set refPtr   [lindex $refInfo 0]
set refXSize [lindex [lindex $refInfo 2] 0]
set refYSize [lindex [lindex $refInfo 2] 1]

set inInfo  [getROIInfo -roi 2]
set inPtr   [lindex $inInfo 0]
set xSize   [lindex [lindex $inInfo 2] 0]
set ySize   [lindex [lindex $inInfo 2] 1]

if {$xSize != $refXSize || $ySize != $refYSize} \
   {
    puts stderr "Error: Size Mismatch. Input X: $xSize Y: $ySize Reference X: $refXSize Y: $refYSize"
    exit 1
   }

set bestDX  $sxMin
set bestDY  $syMin
set bestRMS 1.0e+20

set thisX1    $sxMin
set thisXN    $sxMax
set thisXStep $sxStep

set thisY1    $syMin
set thisYN    $syMax
set thisYStep $syStep

set n       [expr $xSize*$ySize]
set workPtr [sysMalloc $n float]

for {set i 1} {$i <= $iterCount} {incr i} \
   {
    for {set dy $thisY1} {$dy <= $thisYN} {set dy [expr $dy + $thisYStep]} \
       {
        for {set dx $thisX1} {$dx <= $thisXN} {set dx [expr $dx + $thisXStep]} \
           {
            vvMove $workPtr $inPtr $n
            vInterpShift $workPtr $xSize $ySize $dx $dy
            vvSub  $workPtr $refPtr $n

            set thisRMS [vRMS $workPtr $n]

            if {$thisRMS < $bestRMS} \
               {
                set bestRMS $thisRMS
                set bestDX  $dx
                set bestDY  $dy
               }
           }
       }

    if {$verbose} \
       {
        set fmt "Iteration: %d  DX: % 8.3f DY: % 8.3f RMS: % .4e Limits X: % 8.3f %8.3f Y: % 8.3f % 8.3f Step X: %5.3f Y: %5.3f"
        puts [format $fmt $i $bestDX $bestDY $bestRMS $thisX1 $thisXN $thisY1 $thisYN $thisXStep $thisYStep]
       }

    set xRange [expr 0.25*(1.0 + $thisXN - $thisX1)]
    set yRange [expr 0.25*(1.0 + $thisYN - $thisY1)]

    set thisX1 [expr $bestDX - $xRange]
    set thisXN [expr $bestDX + $xRange]

    set thisY1 [expr $bestDY - $yRange]
    set thisYN [expr $bestDY + $yRange]

    set thisXStep [expr 0.5*$thisXStep]
    set thisYStep [expr 0.5*$thisYStep]
   }

if {[strcasecmp $outName None]} \
   {
    vInterpShift $inPtr $xSize $ySize $bestDX $bestDY
    writeROI -roi 2 -out $outName -ov
   }

if {!$verbose} {puts -nonewline "$bestDX $bestDY"}

exit 0
