#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Report Earth Mover Distance Between NMR Spectra:"
    puts " -in1    inName1 \[ref.ft2\]   Input 1 (NMRPipe Format 1D or 2D)."
    puts " -in2    inName2 \[test.ft2\]  Input 2 (NMRPipe Format 1D or 2D)."
    puts " -x1     x1      \[0%\]        X-Axis Region Start."
    puts " -xn     xn      \[100%\]      X-Axis Region End"
    puts " -y1     y1      \[0%\]        Y-Axis Region Start (2D Only)."
    puts " -yn     yn      \[100%\]      Y-Axis Region End (2D Only)."
    puts " -thresh t       \[None\]      Intensity Threshold."
    puts " -normFn fn      \[vSum\]      Normalization Function, or Keyword None."
    puts " -hiMult hiMult  \[1.0\]       Scale Factor for Height After Normalization."
    puts " -xMult  xMult   \[1.0\]       Scale Factor for X-Axis Chemical Shift."
    puts " -yMult  yMult   \[Auto\]      Scale Factor for Y-Axis Chemical Shift."
    puts " -prefix p       \[tmpEMD\$\$]  Prefix for Temporary Files (\$\$ is the Process ID)."
    puts " -noemd                      Skip EMD Calculation, Report L1, L2, etc."
    puts "Handling of Signed Data (Use Only One):"
    puts " -posNeg                       Separate Positive and Negative Channels (Default)."
    puts " -pos                          Use Only Positive Intensities."
    puts " -abs                          Use Absolute Value Intensities."
    puts "Notes:"
    puts " 1. Input regions for spectra inName1 and inName2 must have the same size"
    puts "    and number of dimensions."
    puts ""
    puts " 2. If a threshold is given via the -thresh option, intensities"
    puts "    below the threshold are set to zero."
    puts ""
    puts " 3. Normalization is applied after thresholding. Normalization Functions:"
    puts ""
    puts "       vSum vAvg    vSumSq vSumAbs   vMinAbs vMaxAbs   vMin"
    puts "       vMax vStdDev vRMS   vVariance vMedian vEstNoise"
    exit 0
   }

set inName1  ref.ft2 
set inName2  test.ft2

set thresh   None
set normFn   vSum
set signMode posNeg
set emdFlag  1

set hiMult   1.0
set xMult    1.0
set yMult    Auto

set x1   0%
set xn 100%
set y1   0%
set yn 100%

set prefix tmpEMD[pid]

getArgD $ARGV -in1 inName1
getArgD $ARGV -in2 inName2

getArgD $ARGV -thresh thresh
getArgD $ARGV -mode   normMode

getArgD $ARGV -hiMult hiMult
getArgD $ARGV -xMult  xMult
getArgD $ARGV -yMult  yMult

getArgD $ARGV -x1 x1 
getArgD $ARGV -xn xn 
getArgD $ARGV -y1 y1 
getArgD $ARGV -yn yn 

getArgD $ARGV -prefix prefix

if {[flagLoc $ARGV -posNeg]} {set signMode posNeg}
if {[flagLoc $ARGV -pos]}    {set signMode pos}
if {[flagLoc $ARGV -abs]}    {set signMode abs}

if {[flagLoc $ARGV -noemd]}  {set emdFlag 0}

#
#----------------------------------------------------------------------------------

proc getL0Dist { v1 v2 t n } \
{
   global workPtr1 workPtr2

   vvMove $workPtr1 $v1 $n
   vvMove $workPtr2 $v2 $n

   vBinarize $workPtr1 $t $n
   vBinarize $workPtr2 $t $n

   vAbs $workPtr1 $n
   vAbs $workPtr2 $n

   vvSub $workPtr1 $workPtr2 $n
   vAbs  $workPtr1 $n

   set d [vSum $workPtr1 $n]

   return $d
}

proc getL1Dist { v1 v2 n } \
{
   global workPtr1 workPtr2

   vvSub $workPtr1 $v1 $v2 $n
   vAbs  $workPtr1 $n

   set d [vSum $workPtr1 $n]

   return $d
}

proc getL2Dist { v1 v2 n } \
{
   global workPtr1 workPtr2

   vvSub $workPtr1 $v1 $v2 $n

   set d [vRMS $workPtr1 $n]

   return $d
}

proc getChi2Dist { v1 v2 n } \
{
   global workPtr1 workPtr2

   vAbs  $workPtr1 $v1 $n
   vAbs  $workPtr2 $v2 $n

   vvAdd $workPtr2 $workPtr1 $n
   vvSub $workPtr1 $v1 $v2 $n
   vvDiv $workPtr1 $workPtr2 $n

   set d [vRMS $workPtr1 $n]

   return $d
}

proc getLInfDist { v1 v2 n } \
{
   global workPtr1 workPtr2

   vvSub $workPtr1 $v1 $v2 $n
   vAbs  $workPtr1 $n

   set d [vMax $workPtr1 $n]

   return $d
}

if {![file exists $inName1]} \
   {
    puts stderr "EMD Error Finding Input 1: $inName1"
    exit 1
   }

if {![file exists $inName2]} \
   {
    puts stderr "EMD Error Finding Input 2: $inName2"
    exit 1
   }

set error [rdFDATA $inName1 fdata1]

if {$error} \
   {
    puts stderr "EMD Error Reading Header from Input 1: $inName1"
    exit 1
   }

set error [rdFDATA $inName2 fdata2]

if {$error} \
   {
    puts stderr "EMD Error Reading Header from Input 2: $inName2"
    exit 1
   }

set xOBS  [getParm $fdata1 NDOBS CUR_XDIM]
set yOBS  [getParm $fdata1 NDOBS CUR_YDIM]

if {$xOBS == 0.0} {set xOBS 1.0}
if {$yOBS == 0.0} {set yOBS 1.0}

set nDim1 [getParmI $fdata1 FDDIMCOUNT NULL_DIM]
set nDim2 [getParmI $fdata2 FDDIMCOUNT NULL_DIM]

if {$nDim1 > 2} {set nDim1 2}
if {$nDim2 > 2} {set nDim2 2}

if {$nDim1 != $nDim2} \
   {
    puts stderr "EMD Error: Dimension Count Mismatch ($inName1 $nDim1 $inName2 $nDim2)."
    exit 1
   }

set dimCount $nDim1 

if {$dimCount == 1} \
   {
    readROI -roi 1 -ndim 1 -in $inName1 -x X_AXIS $x1 $xn
    readROI -roi 2 -ndim 1 -in $inName2 -x X_AXIS $x1 $xn

    readROI -roi 3 -ndim 1 -in $inName1 -x X_AXIS $x1 $xn
    readROI -roi 4 -ndim 1 -in $inName2 -x X_AXIS $x1 $xn

    readROI -roi 5 -ndim 1 -in $inName1 -x X_AXIS $x1 $xn
    readROI -roi 6 -ndim 1 -in $inName2 -x X_AXIS $x1 $xn
   } \
else \
   {
    readROI -roi 1 -ndim 2 -in $inName1 -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn
    readROI -roi 2 -ndim 2 -in $inName2 -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn

    readROI -roi 3 -ndim 2 -in $inName1 -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn
    readROI -roi 4 -ndim 2 -in $inName2 -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn

    readROI -roi 5 -ndim 2 -in $inName1 -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn
    readROI -roi 6 -ndim 2 -in $inName2 -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn
   }

set rInfo1 [getROIInfo -roi 1]
set rInfo2 [getROIInfo -roi 2]
set rInfo3 [getROIInfo -roi 3]
set rInfo4 [getROIInfo -roi 4]
set rInfo5 [getROIInfo -roi 5]
set rInfo6 [getROIInfo -roi 6]

set posPtr1  [lindex $rInfo1 0]
set posPtr2  [lindex $rInfo2 0]

set negPtr1  [lindex $rInfo3 0]
set negPtr2  [lindex $rInfo4 0]

set workPtr1 [lindex $rInfo5 0]
set workPtr2 [lindex $rInfo6 0]

set xSize1 [lindex [lindex $rInfo1 2] 0]
set xSize2 [lindex [lindex $rInfo1 2] 0]

if {$dimCount == 1} \
   {
    set ySize1 1
    set ySize2 1
   } \
else \
   {
    set ySize1 [lindex [lindex $rInfo1 2] 1]
    set ySize2 [lindex [lindex $rInfo1 2] 1]
   }

if {$xSize1 != $xSize2 || $ySize1 != $ySize2} \
   {
    puts stderr "EMD Error: Dimension Size Mismatch ($inName1 $xSize1 $ySize1 $inName2 $xSize2 $ySize2)."
    exit 1
   }

set xSize $xSize1
set ySize $ySize1

set n     [expr $xSize*$ySize]

set dEMD   0.0
set dL0    0.0
set dL1    0.0
set dL2    0.0
set dLInf  0.0
set dChi2  0.0

set emdPos 0.0
set emdNeg 0.0

set sumPos 0.0
set sumNeg 0.0

set cleanList ""

if {![strcasecmp $xMult Auto]} {set xMult 1.0}
if {![strcasecmp $yMult Auto]} {set yMult [expr $yOBS/$xOBS]}

if {![strcasecmp $signMode posNeg]} \
   {
    vClipPos $posPtr1 $n
    vClipPos $posPtr2 $n

    vClipNeg $negPtr1 $n
    vClipNeg $negPtr2 $n

    if {[strcasecmp $thresh None]} \
       {
        vsClipBelow $posPtr1 $thresh $n
        vsClipBelow $posPtr2 $thresh $n

        vsClipBelow $negPtr1 $thresh $n
        vsClipBelow $negPtr2 $thresh $n
       }

    set sumPos [expr [vSum $posPtr1 $n] + [vSum $posPtr2 $n]]
    set sumNeg [expr [vSum $negPtr1 $n] + [vSum $negPtr2 $n]]

    if {[strcasecmp $normFn None]} \
       {
        set val1Pos [$normFn $posPtr1 $n]
        set val2Pos [$normFn $posPtr2 $n]

        set val1Neg [$normFn $negPtr1 $n]
        set val2Neg [$normFn $negPtr2 $n]

        set sum1Pos [vSum $posPtr1 $n]
        set sum2Pos [vSum $posPtr2 $n]

        set sum1Neg [vSum $negPtr1 $n]
        set sum2Neg [vSum $negPtr2 $n]

        vsDiv $posPtr1 $val1Pos $n
        vsDiv $posPtr2 $val2Pos $n

        vsDiv $negPtr1 $val1Neg $n
        vsDiv $negPtr2 $val2Neg $n
       }
   } \
else \
   {
    if {![strcasecmp $signMode pos]} \
       {
        vClipNeg $posPtr1 $n
        vClipNeg $posPtr2 $n
       } \
    elseif {![strcasecmp $signMode abs]} \
       {
        vAbs $posPtr1 $n
        vAbs $posPtr2 $n
       }

    if {[strcasecmp $thresh None]} \
       {
        vsClipBelow $posPtr1 $thresh $n
        vsClipBelow $posPtr2 $thresh $n
       }

    set dL0    [getL0Dist   $posPtr1 $posPtr2 $thresh $n]
    set sumPos [expr [vSum $posPtr1 $n] + [vSum $posPtr2 $n]]
    set sumNeg 0.0

    if {[strcasecmp $normFn None]} \
       {
        set val1 [$normFn $posPtr1 $n] 
        set val2 [$normFn $posPtr2 $n] 

        vsDiv $posPtr1 [expr $val1] $n
        vsDiv $posPtr2 [expr $val2] $n
       }

    set dL1   [getL1Dist   $posPtr1 $posPtr2 $n]
    set dL2   [getL2Dist   $posPtr1 $posPtr2 $n]
    set dChi2 [getChi2Dist $posPtr1 $posPtr2 $n]
    set dLInf [getLInfDist $posPtr1 $posPtr2 $n]

    vsMult  $posPtr1 $hiMult $n
    vsMult  $posPtr2 $hiMult $n

    set outName1 ${prefix}_pos1.dat
    set outName2 ${prefix}_pos2.dat

    set txtName1 ${prefix}_nmr_pos1.txt
    set txtName2 ${prefix}_nmr_pos2.txt

    set emdName1 ${prefix}_emd_pos1.txt
    set emdName2 ${prefix}_emd_pos2.txt

    catch "file delete $outName1 $outName2 $txtName1 $txtName2 $emdName1 $emdName2"

    writeROI -roi 1 -out $outName1 -ov
    writeROI -roi 2 -out $outName2 -ov

    exec pipe2txt.tcl -index ppm -fmt %.4e -xy $outName1 > $txtName1 
    exec pipe2txt.tcl -index ppm -fmt %.4e -xy $outName2 > $txtName2 

    exec nmr2emd -x $xMult -y $yMult $txtName1 > $emdName1
    exec nmr2emd -x $xMult -y $yMult $txtName2 > $emdName2

    if {$emdFlag} \
       {
        set emdInfo [exec emdkws $emdName1 $emdName2]
        set dEMD    [lindex $emdInfo 0]
        set emdPos  $dEMD
       } \
    else \
       {
        set dEMD    0.0
        set emdPos  0.0
       }

    vvSub $workPtr1 $posPtr1 $posPtr2 $n

    catch "file delete $outName1 $outName2 $txtName1 $txtName2 $emdName1 $emdName2"
   }

puts -nonewline "dEMD [format %.4e $dEMD] dL0 [format %.4e $dL0] dL1 [format %.4e $dL1] dL2 [format %.4e $dL2] dLInf [format %.4e $dLInf] dChi2 [format %.4e $dChi2]  emdPos [format %.4e $emdPos] emdNeg [format %.4e $emdPos] sumPos [format %.4e $sumPos] sumNeg [format %.4e $sumNeg] normFn $normFn thresh $thresh signMode $signMode inName1 $inName1 inName2 $inName2 dimCount $dimCount xSize $xSize ySize $ySize N $n"

exit 0

