#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# gif2roi.tcl: crude conversion from GIF image to NMRPipe format;
#              very slow, as conversion is performed via script,
#              one pixel at a time.
#
#              Creates an NMRPipe format ".dat" file for every ".gif"
#              file input in the command line.

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert GIF image to NMRPipe Format:"
    puts " $argv0 \[-ext .dat] gifNameList"
    puts "By defaut, output files will have .dat extension."
    exit 1
   }

set outExt .dat

getArgD $ARGV -ext outExt

if {![strcasecmp $outExt None]} {set outExt ""}

#
# Creates a graphical interface to show current row being processed.

proc initWin {} \
{
   label  .label -text  "GIF to NMRPipe - Rows Left"
   button .quit  -text  Quit -command exit
   scale  .sc    -orient horizontal -length 256
   pack   .sc

   pack   .label .sc .quit -in . -side top -expand 1 -fill x

   update
}

#
# Converts given GIF input to an NMRWish ROI.

proc image2ROI { gifName roiID } \
{
   global FDATASIZE

#
# Initialize:
#  Read the GIF image using the usual TK commands.
#  Extract the image sizes.
#  Update the graphic title and state accordingly.

   set error 0

   set imageName roiImage

   catch {image delete $imageName}
   image create photo $imageName -file $gifName -palette 256

   set xSize     [image width  $imageName]
   set ySize     [image height $imageName]
   set n         [expr $xSize*$ySize]

   wm title . "$gifName ${xSize}x${ySize}"
   
   .sc configure -from 0 -to $ySize

#
# Allocate memory for the NMRPipe format header and data.
# (Uses custom command sysMalloc)

   set error [catch {set fdata [sysMalloc $FDATASIZE float]}]

   if {$error} \
      {
       showError ImageView "Error allocating image header."
       return $error
      }

   set error [catch {set rdata [sysMalloc $n float]}]

   if {$error} \
      {
       showError ImageView "Error allocating image data."
       return $error
      }

#
# Build the ROI header:
# (Uses custom command buildHdr)

   buildHdr $fdata \
     -xN      $xSize  -yN      $ySize  \
     -xT      $xSize  -yT      $ySize  \
     -xMODE   Real    -yMODE   Real    \
     -xSW     $xSize  -ySW     $ySize  \
     -xOBS    1.0000  -yOBS    1.0000  \
     -xCAR    0.0000  -yCAR    0.0000  \
     -xFT     Freq    -yFT     Freq    \
     -xLAB    XPixel  -yLAB    YPixel  \
     -ndim    2       -aq2D    Image

#
# Build the image matrix:
# (Uses custom commands setPtrVal and incrPtr)

   set rPtr    $rdata
   set rgbList ""

   for {set iy [expr $ySize - 1]} {$iy >= 0} {incr iy -1} \
      {
       if {!($iy % 10)} \
          {
           .sc set $iy
           update
          }

       for {set ix 0} {$ix < $xSize} {incr ix} \
           {
            set cInfo [$imageName get $ix $iy]

            set ir [lindex $cInfo 0]
            set ig [lindex $cInfo 1]
            set ib [lindex $cInfo 2]

            if {[lsearch -exact $rgbList $ir,$ig,$ib] < 0} {lappend rgbList $ir,$ig,$ib}

            setPtrVal *$rPtr [expr 0.3333*($ir + $ig + $ib)] 

            set rPtr  [incrPtr $rPtr]
           } 
      }

   puts "[llength $rgbList] Colors Used: [lsort $rgbList]"

#
# Create the ROI.
# Deallocate work buffers.
# (Uses custom commands readROI and sysFree)

   readROI -roi $roiID -inPtr $rdata -inHdr $fdata -x XPixel -y YPixel

   sysFree $fdata $FDATASIZE
   sysFree $rdata $n
   
   return $error
}

#
# Main Program:
#  Initialize the graphics interface.
#
#  For each input GIF image on the command line:
#    Convert the GIF image to an ROI in memory.
#    Write the ROI as an NMRPipe format file.
#
#  Exit.

initWin

set n [llength $argv]

for {set i 0} {$i < $n} {incr i} \
   {
    set thisItem [lindex $argv $i]

    if {[isFlag $thisItem]} \
       {
        incr i
        continue
       }

    set gifName $thisItem
    set outName [file root $gifName]$outExt

    puts "GIF Input: $gifName NMRPipe Output: $outName"

    set error [image2ROI $gifName 1]

    writeROI -roi 1 -out $outName -ov
   }

exit

#
# Bottom.
