#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

#
# Creates NMRPipe-format spectra from list of related images.
# frank.delaglio@nist.gov 6/22/2022 w

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

set inList       test.tiff
set outName      Auto
set outDir       spec
set hdrName      None
set tmpName      tmpim[pid].pfm

set cRed      0.2126
set cGreen    0.7152
set cBlue     0.0722

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert Images to NMRPipe-format (Requires Image Utility mogrify):"
    puts " -in      inList  \[test.tiff\] List of Input Images."
    puts " -outDir  outDir  \[spec\]      Output Directory."
    puts " -outName outName \[Auto\]      Output Spectrum."
    puts " -hdr     inName  \[None\]      Optional Input Spectrum for NMRPipe-format Header."
    puts "RGB to Spectral Intensity:"
    puts " -cRed    cRed    \[0.2126\]    Red Scale Factor."
    puts " -cGreen  cGreen  \[0.7152\]    Green Scale Factor."
    puts " -cBlue   cBlue   \[0.0722\]    Blue Scale Factor."
    puts "Temporary File (Default Name Includes PID):"
    puts " -tmp     tmpName \[$tmpName\]"
    exit 0
   }

getListArgD $ARGV -in inList

getArgD $ARGV -hdr     hdrName
getArgD $ARGV -tmp     tmpName
getArgD $ARGV -outDir  outDir
getArgD $ARGV -outName outName

getArgD $ARGV -cRed    cRed
getArgD $ARGV -cGreen  cGreen
getArgD $ARGV -cBlue   cBlue

if {![strcasecmp $outDir None]} {set outDir "."}

#
#-------------------------------------------------------------------------------------------

proc rdPFM { thisName } \
{
   global xSize ySize rgbData cRed cGreen cBlue 

   set error [catch {set inUnit [open $thisName r]}]

   set line1 ""
   set line2 ""
   set line3 ""

   set n1 [gets $inUnit line1]
   set n2 [gets $inUnit line2]
   set n3 [gets $inUnit line3]

   catch "close $inUnit"

#  puts "$thisName"
#  puts " $line1 ($n1 [string length $line1])"
#  puts " $line2 ($n2 [string length $line2])"
#  puts " $line3 ($n3 [string length $line3])"

   if {[llength $line2] != 2 || [llength $line3] != 1} \
      {
       puts "image2spec. Error interpreting header in $thisName"
       return 1
      }

   set xSize     [lindex $line2 0]
   set ySize     [lindex $line2 1]
   set planeSize [expr $xSize*$ySize]
   set n         [expr 3*$xSize*$ySize]

   set rgbData   [sysMalloc $n         float]
   set redData   [sysMalloc $planeSize float]
   set greenData [sysMalloc $planeSize float]
   set blueData  [sysMalloc $planeSize float]

   set error [catch {set inUnit [sysOpen $thisName r]}]

   sysSeek $inUnit [expr $n1 + $n2 + $n3 + 3]

   set error [sysRead $inUnit $rgbData $n]

#
#  vvMove2 destPtr pJumpDest vJumpDest srcPtr  pJumpSrc vJumpSrc pntCount vecCount

   vvMove2 $redData   1 $xSize [incrPtr $rgbData 0] 3 [expr 3*$xSize] $xSize $ySize
   vvMove2 $greenData 1 $xSize [incrPtr $rgbData 1] 3 [expr 3*$xSize] $xSize $ySize
   vvMove2 $blueData  1 $xSize [incrPtr $rgbData 2] 3 [expr 3*$xSize] $xSize $ySize

   vByteSwap $redData   $planeSize
   vByteSwap $blueData  $planeSize
   vByteSwap $greenData $planeSize

   vsSet $rgbData 0.0 $planeSize

   vsMult $redData   $cRed   $planeSize
   vsMult $greenData $cGreen $planeSize
   vsMult $blueData  $cBlue  $planeSize

   vvAdd $rgbData $redData   $planeSize
   vvAdd $rgbData $greenData $planeSize
   vvAdd $rgbData $blueData  $planeSize

   sysFree $redData   $planeSize
   sysFree $greenData $planeSize
   sysFree $blueData  $planeSize

   return 0
}

if {![file isdirectory $outDir]} {catch "file mkdir $outDir"}

set xSize 0
set ySize 0

set rgbData   0
set redData   0
set blueData  0
set greenData 0
set fdata     0
           
if {![strcasecmp $hdrName None]} \
   {
    set fdata [sysMalloc $FDATASIZE float]

    buildHdr $fdata \
     -xN      256     -yN      265     \
     -xT      256     -yT      256     \
     -xMODE   Real    -yMODE   Real    \
     -xSW     256     -ySW     256     \
     -xOBS    1.0000  -yOBS    1.0000  \
     -xCAR    0.0000  -yCAR    0.0000  \
     -xFT     Freq    -yFT     Freq    \
     -xLAB    XPixel  -yLAB    YPixel  \
     -ndim    2       -aq2D    Image
   } \
else \
   {
    if {![file exists $hdrName]} \
       {
        puts "image2spec. Error finding header input $hdrName"
        exit 1
       }

    set error [rdFDATA $inName fdata]

    if {$error} \
       {
        puts "image2spec. Error reading header in $hdrName"
        exit 1
       }
   }

foreach inName $inList \
   {
    if {![file exists $inName]} \
       {
        puts stderr "image2spec. Error finding image input $inName"
        exit
       }

    set rootName [file root [file tail $inName]]

    if {![strcasecmp $outName Auto]} \
       {
        set thisOutName $outDir/${rootName}.ft2
       } \
    else \
       {
        set thisOutName $outDir/$outName
       }
           
    set rgbData 0
    set xSize   0
    set ySize   0

    if {![file exists $inName]} \
       {
        puts "image2spec. Error finding input $inName"
        exit 1
       }

    catch "file delete $tmpName"
    catch "exec mogrify -format PFM -write $tmpName $inName"

    set error [rdPFM $tmpName]

    if {$error} \
       {
        puts "image2spec. Error reading converted image $tmpName"
        exit 1
       }

    puts "$inName $thisOutName $xSize $ySize"

    setParm $fdata NDSIZE $xSize CUR_XDIM
    setParm $fdata NDAPOD $xSize CUR_XDIM
    setParm $fdata NDSW   $xSize CUR_XDIM

    setParm $fdata NDSIZE $ySize CUR_YDIM
    setParm $fdata NDAPOD $ySize CUR_YDIM
    setParm $fdata NDSW   $ySize CUR_YDIM

    set error [catch {set outUnit [sysOpen $thisOutName w]}]

    if {$error} \
       {
        puts "image2spec. Error Opening Output File $thisOutName"
        return $error
       }

    sysWrite $outUnit $fdata   $FDATASIZE
    sysWrite $outUnit $rgbData [expr $xSize*$ySize] 

    catch "sysClose $outUnit"

    sysFree $rgbData [expr 3*$xSize*$ySize]

    catch "file delete $tmpName"
   }

exit 0
