#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

# imageView.tcl
#
#  3D Spectral Image Viewer (C) Frank Delaglio 1998-2019
#----------------------------------------------------------------

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

#
# Some Global Constants:

set Def1DPixmap 1
set Def2DPixmap 2
set DefHVPixmap 3 

set Def1DROI    1
set Def2DROI    2
set Def3DROI    3
set DefHROI     4
set DefVROI     5
set DefZROI     6

#
#-----------------------------------------------------------

proc new1D { xPix yPix } \
{
   global objInfo Def1DROI

   if {!$objInfo(data,roi3D)} return

   set objInfo(data,roi1D) 0

   getDataXY $xPix $yPix objInfo(data,ix) objInfo(data,iy)

   set error \
       [catch {readROI -roi $Def1DROI -ndim 1 -inROI $objInfo(data,roi3D) \
                       -dy $objInfo(data,xAxis) $objInfo(data,ix) 0       \
                       -dz $objInfo(data,yAxis) $objInfo(data,iy) 0       \
                       -x  $objInfo(data,zAxis)}]

   if {$error} \
      {
       showError ImageView "Error reading 1D ROI from $objInfo(data,name)"
       return
      }

   set roiInfo               [getROIInfo -roi $Def1DROI]
   set objInfo(data,rdata1D) [lindex $roiInfo 0]

   set objInfo(data,roi1D)   $Def1DROI 
   set objInfo(win2D,mouseX) $xPix
   set objInfo(win2D,mouseY) $yPix

   untrack1D
   track2D $xPix $yPix
   draw1D
}

proc new2D args \
{
   global objInfo Def2DROI

   if {!$objInfo(data,roi3D)} return

   set objInfo(data,roi2D) 0

   set error \
       [catch {readROI -ndim 2 -roi $Def2DROI -inROI $objInfo(data,roi3D) \
               -x  $objInfo(data,xAxis) $objInfo(data,x1) $objInfo(data,xn) \
               -y  $objInfo(data,yAxis) $objInfo(data,y1) $objInfo(data,yn) \
               -dz $objInfo(data,zAxis) $objInfo(data,iz) 0}]


   set error 0

   if {$error} \
      {
       showError ImageView "Error reading 2D ROI from $objInfo(data,name)"
       return
      }

   set objInfo(data,roi2D) $Def2DROI 

   set roiInfo               [getROIInfo -roi $objInfo(data,roi2D)]
   set objInfo(data,rdata2D) [lindex $roiInfo 0]
   set objInfo(data,zPos)    [getDataZPos $objInfo(data,iz)]

   untrack2D

   draw2D
   draw1D
}

proc new3D {} \
{
   global objInfo Def3DROI spiffFlag

   if {$objInfo(data,roi3D)} {freeROI -roi $objInfo(data,roi3D)}

   set objInfo(data,roi3D) 0

   if {"$objInfo(data,name)" == "None"} return

   set thisExt              [file extension $objInfo(data,name)]
   set objInfo(data,format) NMRPipe

   foreach ext {.spc .SPC .seq .SEQ .abs .ABS} \
      {
       if {"$ext" == "$thisExt"} \
          {
           set objInfo(data,format) SPIFF
           break
          }
      }

   wm title . "Working ..."
   update

   if {$objInfo(data,format) == "SPIFF"} \
      {
       newSpiff $objInfo(data,name) fdata rdata

       set error \
           [catch {readROI -ndim 3 -roi $Def3DROI -inPtr $rdata -inHdr $fdata \
                   -x X_AXIS -y Y_AXIS -z Z_AXIS}]

       sysFree $fdata
       sysFree $rdata
      } \
   else \
      {
       set error [newNMR $objInfo(data,name)]
      }

   if {$error} \
      {
       showError ImageView "Error $error reading 3D ROI in $objInfo(data,name)"
       return
      }

   set roiInfo [getROIInfo -roi $Def3DROI]

   set objInfo(data,roi3D)   $Def3DROI 
   set objInfo(data,rdata3D) [lindex $roiInfo 0]
   set objInfo(data,fdata)   [lindex $roiInfo 1]
}

proc find2D { xPix yPix } \
{
   global objInfo

   set dx [expr double($objInfo(win1D,xn) - $objInfo(win1D,x1))]

   if {!$dx} {set dx 1}

   set iz [expr int($objInfo(data,zSize)*($xPix - $objInfo(win1D,x1))/$dx)]

   if {$iz < 1} {set iz 1}
   if {$iz > $objInfo(data,zSize)} {set iz $objInfo(data,zSize)}

   set objInfo(data,iz) $iz

   new2D
}

proc getDataXY { xPix yPix ixPtr iyPtr } \
{
   upvar $ixPtr ix
   upvar $iyPtr iy

   global objInfo

   set xw [expr double(1 + $objInfo(win2D,xn) - $objInfo(win2D,x1))]
   set yw [expr double(1 + $objInfo(win2D,yn) - $objInfo(win2D,y1))]

   set dx [expr 1 + $objInfo(data,xn) - $objInfo(data,x1)]
   set dy [expr 1 + $objInfo(data,yn) - $objInfo(data,y1)]

   if {!$xw} {set xw 1}
   if {!$yw} {set yw 1}

   set ix [expr int($objInfo(data,x1)+$dx*($xPix - $objInfo(win2D,x1))/$xw)]
   set iy [expr int($objInfo(data,y1)+$dy*($objInfo(win2D,yn) - $yPix)/$yw)]

   if {$ix < $objInfo(data,x1)} {set ix $objInfo(data,x1)}
   if {$ix > $objInfo(data,xn)} {set ix $objInfo(data,xn)}

   if {$iy < $objInfo(data,y1)} {set iy $objInfo(data,y1)}
   if {$iy > $objInfo(data,yn)} {set iy $objInfo(data,yn)}
}

proc getDataZPos { iz } \
{
   global objInfo

   set fdata $objInfo(data,fdata)
   set zVal  [pnt2spec $fdata CUR_ZDIM $iz $objInfo(data,zUnits)]
   set zPos  [format %.3f $zVal]

   return $zPos
}

proc usePlane { mode } \
{
   global objInfo DefHROI DefVROI DefZROI

   set thisName [format $objInfo(data,name) $objInfo(data,iz)]

   switch $mode \
      {
       h {
          set objInfo(data,hName) $thisName
          openHV hName roiH $DefHROI
         }

       v {
          set objInfo(data,vName) $thisName
          openHV vName roiV $DefVROI
         }

       z {
          set objInfo(data,zName) $thisName
          openHV zName roiZ $DefZROI
         }
      }

   clearWin winHV
   drawHV
}

proc zoomON { xPix yPix } \
{
   global objInfo

   if {$xPix < $objInfo(win2D,x1) || $xPix > $objInfo(win2D,xn)} return
   if {$yPix < $objInfo(win2D,y1) || $yPix > $objInfo(win2D,yn)} return 

   set objInfo(zoom2D,x1Pix) $xPix
   set objInfo(zoom2D,y1Pix) $yPix
   set objInfo(zoom2D,xnPix) $xPix
   set objInfo(zoom2D,ynPix) $yPix

   $objInfo(win2D,canvas) create rectangle \
      $objInfo(zoom2D,x1Pix) $objInfo(zoom2D,y1Pix) \
      $objInfo(zoom2D,xnPix) $objInfo(zoom2D,ynPix) \
      -outline $objInfo(color,win2Dfg) -tags zoom2D 
}

proc trackZoom { xPix yPix } \
{
   global objInfo

   if {!$objInfo(data,roi2D)} return

   if {$xPix < $objInfo(win2D,x1) || $xPix > $objInfo(win2D,xn)} return
   if {$yPix < $objInfo(win2D,y1) || $yPix > $objInfo(win2D,yn)} return 

   set objInfo(zoom2D,xnPix) $xPix
   set objInfo(zoom2D,ynPix) $yPix

   $objInfo(win2D,canvas) coords zoom2D \
      $objInfo(zoom2D,x1Pix) $objInfo(zoom2D,y1Pix) \
      $objInfo(zoom2D,xnPix) $objInfo(zoom2D,ynPix)

   track2D $xPix $yPix
}

proc zoom2D { xPix yPix } \
{
   global objInfo

   if {!$objInfo(data,roi2D)} return

   set objInfo(data,roi2D) 0

   $objInfo(win2D,canvas) delete zoom2D 

   if {$objInfo(zoom2D,x1Pix) > $objInfo(zoom2D,xnPix)} \
      {
       set itemp                  $objInfo(zoom2D,x1Pix)
       set $objInfo(zoom2D,x1Pix) $objInfo(zoom2D,xnPix)
       set $objInfo(zoom2D,xnPix) $itemp
      }

   if {$objInfo(zoom2D,y1Pix) > $objInfo(zoom2D,ynPix)} \
      {
       set itemp                  $objInfo(zoom2D,y1Pix)
       set $objInfo(zoom2D,y1Pix) $objInfo(zoom2D,ynPix)
       set $objInfo(zoom2D,ynPix) $itemp
      }

   getDataXY $objInfo(zoom2D,x1Pix) $objInfo(zoom2D,ynPix) \
             objInfo(data,x1)       objInfo(data,y1)

   getDataXY $objInfo(zoom2D,xnPix) $objInfo(zoom2D,y1Pix) \
             objInfo(data,xn)       objInfo(data,yn)

   clearWin win2D

   new2D 
}

proc untrack1D {} \
{
   global objInfo

   if {!$objInfo(data,roi1D)} return

   set str ""

   append str "X=$objInfo(data,ix), Y=$objInfo(data,iy)"
   append str " \[Scale\] \[Offset\] \[Select Plane\]"
   
   wm title $objInfo(win1D,window) $str

   update
}

proc track1D { xPix yPix } \
{
   global objInfo

   if {!$objInfo(data,roi1D)} return

   set dx [expr double($objInfo(win1D,xn) - $objInfo(win1D,x1))]

   if {!$dx} {set dx 1}

   set iz [expr int($objInfo(data,zSize)*($xPix - $objInfo(win1D,x1))/$dx)]

   if {$iz < 1} {set iz 1}
   if {$iz > $objInfo(data,zSize)} {set iz $objInfo(data,zSize)}

   set hi   [format %.3f [getPtrVal $objInfo(data,rdata1D) [expr $iz-1]]]
   set zPos [getDataZPos $iz]

   set    str "X=$objInfo(data,ix), Y=$objInfo(data,iy) Z=$iz"
   append str " $zPos $objInfo(data,zUnits) I=$hi"

   wm title $objInfo(win1D,window) $str 

   update
}

proc untrack2D {} \
{
   global objInfo

   if {!$objInfo(data,roi2D)} return

   set str ""
 
   append str "$objInfo(data,name) Z=$objInfo(data,iz) $objInfo(data,zPos) $objInfo(data,zUnits)"
   append str " \[Right: Zoom\]"

   wm title $objInfo(win2D,window) $str 

   unshowHV
   update
}

proc track2D { xPix yPix } \
{
   global objInfo

   if {!$objInfo(data,roi2D)} return

   if {$xPix < $objInfo(win2D,x1) || $xPix > $objInfo(win2D,xn)} \
      {
       untrack2D
       return
      }

   if {$yPix < $objInfo(win2D,y1) || $yPix > $objInfo(win2D,yn)} \
      {
       untrack2D
       return
      }

   getDataXY $xPix $yPix ix iy

   set dx     [expr 1 + $objInfo(data,xn) - $objInfo(data,x1)]
   set offset [expr ($ix - $objInfo(data,x1)) + ($iy - $objInfo(data,y1))*$dx]
   set hi     [format %.3f [getPtrVal $objInfo(data,rdata2D) $offset]]
 
   set    str "X=$ix Y=$iy Z=$objInfo(data,iz)"
   append str " $objInfo(data,zPos) $objInfo(data,zUnits)"
   append str " I=$hi"

   wm title $objInfo(win2D,window) $str

   showHV $offset
   update
}

proc unshowHV {} \
{
   global objInfo

   if {!$objInfo(data,roiH) || !$objInfo(data,roiV)} return

   $objInfo(winHV,canvas) coords box 3 3 5 5
}

proc showHV { offset } \
{
   global objInfo

   if {!$objInfo(data,roiH) || !$objInfo(data,roiV)} return

   set hiH [getPtrVal $objInfo(data,rdataroiH) $offset]
   set hiV [getPtrVal $objInfo(data,rdataroiV) $offset]

   set dx  [expr double($objInfo(winHV,xn) - $objInfo(winHV,x1))]
   set dy  [expr double($objInfo(winHV,yn) - $objInfo(winHV,y1))]

   set dH  [expr $objInfo(data,maxValroiH) - $objInfo(data,minValroiH)]
   set dV  [expr $objInfo(data,maxValroiV) - $objInfo(data,minValroiV)]

   if {$dH == 0.0} {set dH 1.0}
   if {$dV == 0.0} {set dV 1.0}

   set xPix \
    [expr int($objInfo(winHV,x1) + $dx*($hiH - $objInfo(data,minValroiH))/$dH)] 

   set yPix \
    [expr int($objInfo(winHV,yn) - $dy*($hiV - $objInfo(data,minValroiV))/$dV)] 

   set x1 [expr $xPix - 1]
   set y1 [expr $yPix - 1]
   set xn [expr $xPix + 1]
   set yn [expr $yPix + 1]

   $objInfo(winHV,canvas) coords box $x1 $y1 $xn $yn 
}

proc untrackHV {} \
{
   global objInfo

   wm title $objInfo(winHV,window) "Scatter Plot"

   update
}

proc trackHV { xPix yPix } \
{
   global objInfo

   if {$xPix < $objInfo(winHV,x1) || $xPix > $objInfo(winHV,xn)} \
      {
       untrackHV
       return
      }

   if {$yPix < $objInfo(winHV,y1) || $yPix > $objInfo(winHV,yn)} \
      {
       untrackHV
       return
      }

   wm title $objInfo(winHV,window) "Pixel $xPix,$yPix"

   update
}

proc drawHV {} \
{
   global objInfo

   if {!$objInfo(data,roiH)} return
   if {!$objInfo(data,roiV)} return

   if {!$objInfo(data,roiZ)} \
      {
       drawROI -roi1 $objInfo(data,roiH) -roi2 $objInfo(data,roiV) \
               -cell $objInfo(winHV,cell) -scat \
               -color1 blue -color2 red 
   
      } \
   else \
      {
       drawROI -roi1 $objInfo(data,roiH) -roi2 $objInfo(data,roiV)  \
               -roi3 $objInfo(data,roiZ) -cell $objInfo(winHV,cell) \
               -scat
      }


   update
}

proc clearWin { obj } \
{
    global objInfo

    update

    newScreen -win $objInfo($obj,pixmap)

    drawRec -cell $objInfo($obj,cell) -x 0% 100% -y 0% 100% -fg black
    drawBox -cell $objInfo($obj,cell) -x 0% 100% -y 0% 100% -fg yellow

    update
}

proc setMinPct { mPct } \
{
   global objInfo

   if {$objInfo(data,minPct) > $objInfo(data,maxPct)} \
      {
       set objInfo(data,maxPct) $objInfo(data,minPct)
      }

   draw2D
}

proc setMaxPct { mPct } \
{
   global objInfo

   if {$objInfo(data,maxPct) < $objInfo(data,minPct)} \
      {
       set objInfo(data,minPct) $objInfo(data,maxPct)
      }

   draw2D
}

proc openHV { nameType roiType defROI } \
{
   global objInfo

   set objInfo(data,roi$nameType) 0

   set error \
      [catch {readROI -ndim 2 -roi $defROI          \
                      -in $objInfo(data,$nameType)  \
                      -x  X_AXIS -y Y_AXIS}]

   if {$error} \
      {
       showError ImageView "Error reading 2D ROI from $objInfo(data,$nameType)"
       return
      }

   set objInfo(data,$roiType) $defROI

   set roiInfo [getROIInfo -roi $objInfo(data,$roiType)]
   set rdata   [lindex $roiInfo 0]
   set dx      [expr 1 + $objInfo(data,xn) - $objInfo(data,x1)]
   set dy      [expr 1 + $objInfo(data,yn) - $objInfo(data,y1)]

   set objInfo(data,rdata$roiType)  $rdata
   set objInfo(data,minVal$roiType) [vMin $rdata [expr $dx*$dy]]
   set objInfo(data,maxVal$roiType) [vMax $rdata [expr $dx*$dy]]
}

proc draw1D {} \
{
   global objInfo

   if {!$objInfo(data,roi1D)} return
   if {!$objInfo(data,zSize)} return

   set obj win1D

   set x1 [expr 100.0*($objInfo(data,iz) - 1)/double($objInfo(data,zSize))]
   set xn [expr 100.0*($objInfo(data,iz) - 0)/double($objInfo(data,zSize))]

   drawRec -win $objInfo($obj,pixmap) -x 0 $objInfo($obj,width) -y 0 $objInfo($obj,height) -fg gray10 -noupdate

   drawRec -fg $objInfo(color,win1Dbg) -cell $objInfo(win1D,cell) -x 0% 100% -y 0% 100% -noupdate 
   drawRec -fg dimGray -cell $objInfo(win1D,cell) -x $x1% $xn% -y 0% 100% -noupdate

   drawROI -roi $objInfo(data,roi1D) -cell $objInfo(win1D,cell)  \
     -zero -fhi $objInfo(data,minVal) -lhi $objInfo(data,maxVal) \
     -axis Bottom,units=$objInfo(data,zUnits) -fg $objInfo(color,win1Dfg) \
     -update

   update
}

proc draw2D {} \
{
   global objInfo

   if {!$objInfo(data,roi2D)} return

   set obj win2D

   drawRec -win $objInfo($obj,pixmap) -x 0 $objInfo($obj,width) -y 0 $objInfo($obj,height) -fg gray10 

   drawROI -roi $objInfo(data,roi2D) -cell $objInfo(win2D,cell) \
     -axis Bottom,units=Pts Right,units=Pts \
     -pix -mult 1 -pcc 64 -plev 64  \
     -pHue1 0.54 -pHueN 0.54 -pSat1 0.35 -pSatN 0.35 -pVal1 0.0 -pValN 1.0 \
     -fhi $objInfo(data,minPct)% -lhi $objInfo(data,maxPct)%

   update
}

proc full2D {} \
{
   global objInfo

   set objInfo(data,x1) 1
   set objInfo(data,xn) $objInfo(data,xSize)
   set objInfo(data,y1) 1
   set objInfo(data,yn) $objInfo(data,ySize)

   clearWin win2D

   new2D
}

proc drawHistogram {} \
{
}

proc next2D {} \
{
   global objInfo

   incr objInfo(data,iz)

   if {$objInfo(data,iz) > $objInfo(data,zSize)} {set objInfo(data,iz) 1}

   new2D
}

proc prev2D {} \
{
   global objInfo

   incr objInfo(data,iz) -1

   if {$objInfo(data,iz) < 1} {set objInfo(data,iz) $objInfo(data,zSize)}

   new2D
}

proc scale1D { ix iy } \
{
   global objInfo

   if {$iy > $objInfo(win1D,mouseY)} \
      {
       set f 1.11111
      } \
   else \
      {
       set f 0.90000
      }

   set objInfo(data,minVal) [expr $objInfo(data,minVal)*$f]
   set objInfo(data,maxVal) [expr $objInfo(data,maxVal)*$f]

   set objInfo(win1D,mouseX) $ix
   set objInfo(win1D,mouseY) $iy

   draw1D
}

proc move1D { ix iy } \
{
   global objInfo

   if {$iy > $objInfo(win1D,mouseY)} \
      {
       set f [expr  0.05*($objInfo(data,maxVal) - $objInfo(data,minVal))]
      } \
   else \
      {
       set f [expr -0.05*($objInfo(data,maxVal) - $objInfo(data,minVal))]
      }

   set objInfo(data,minVal) [expr $objInfo(data,minVal) + $f]
   set objInfo(data,maxVal) [expr $objInfo(data,maxVal) + $f]

   set objInfo(win1D,mouseX) $ix
   set objInfo(win1D,mouseY) $iy

   draw1D
}

proc resizeCanvas { obj width height type rProc } \
{
   global objInfo

   bind $objInfo($obj,canvas) <Configure> ""

   $objInfo($obj,canvas) delete pixmap -tags $obj

   set dx [expr $objInfo($obj,leftMargin) + $objInfo($obj,rightMargin)]
   set dy [expr $objInfo($obj,topMargin)  + $objInfo($obj,bottomMargin)]

   set objInfo($obj,xSize) [expr $width  - $dx]
   set objInfo($obj,ySize) [expr $height - $dy]

   setCanvasSize $obj

   $rProc

   bind $objInfo($obj,canvas) <Configure> "resizeCanvas $obj %w %h %T $rProc"
}

proc getHVName { nameType roiType roiDef } \
{
   global objInfo

   set types \
      {
       { {All Files}  *      }
       { {spc}        {.spc} }
       { {abs}        {.abs} }
       { {seq}        {.seq} }
       { {fid}        {.fid} }
       { {DAT}        {.dat} }
       { {dat}        {.dat} }
       { {ft}         {.ft*} }
      }

   set thisName \
      [tk_getOpenFile -title "Image Data Input" -filetypes $types]

   if {"$thisName" == ""} return

   set objInfo(data,$nameType) $thisName
   
   $objInfo(winF,window).$nameType xview end

   openHV $nameType $roiType $roiDef
}

proc openViewer {} \
{
   global exeExt pcFlag

   set types \
      {
       { {All Files}  *      }
       { {spc}        {.spc} }
       { {abs}        {.abs} }
       { {seq}        {.seq} }
       { {fid}        {.fid} }
       { {DAT}        {.DAT} }
       { {dat}        {.dat} }
       { {ft}         {.ft*} }
      }

   set thisName \
      [tk_getOpenFile -title "Image Data Input" -filetypes $types]

   if {"$thisName" == ""} return

   if {$pcFlag} \
      {
       system -nowin nmrWish$exeExt imageView.tcl -- -in $thisName
      } \
   else \
      {
       exec imageView.tcl -in $thisName
      }

   exit
}

proc printViewer {} \
{
}

proc quitViewer {} \
{
   okExit exit
}

proc helpViewer {} \
{
}

proc aboutViewer {} \
{
}

proc showSettings {} \
{
}

proc showColorTable {} \
{
}

proc pkDetect {} \
{
}

proc filter1D {} \
{
}

proc filter2D {} \
{
}

proc pca3D {} \
{
   global shExt exeExt objInfo DefHROI DefVROI DefZROI pcFlag SYS_BYTESWAP

   global  Nx     Ny     Nz     \
           Tx     Ty     Tz     \
           MODEx  MODEy  MODEz  \
           SWx    SWy    SWz    \
           OBSx   OBSy   OBSz   \
           CARx   CARy   CARz

   if {$pcFlag} \
      {
       set scriptName pca.ksh
      } \
   else \
      {
       set scriptName pca.com
      }

   set error [catch "set outUnit [open $scriptName w 0744]"]

   if {$error} \
      {
       showError ImageView "Error building PCA command file."
       return
      }

   if {$SYS_BYTESWAP} \
      {
       set swapFlag "-noswap"
      } \
   else \
      {
       set swapFlag "-swap"
      }

   if {"$objInfo(data,format)" == "SPIFF"} \
      {
       set thisName  [format $objInfo(data,name) 1 1]
       set dirName   [file root [file tail $thisName]]_pca
       set tifName   [file root [file tail $objInfo(data,name)]].tif

       set objInfo(data,hName) $dirName/ir001.dat
       set objInfo(data,vName) $dirName/ir002.dat
       set objInfo(data,zName) $dirName/ir003.dat

       puts $outUnit "bin2pipe $swapFlag \\"
       puts $outUnit " -in $objInfo(data,name) -bo $objInfo(data,hdrSize) \\"
       puts $outUnit " -xN      $Nx     -yN      $Ny     -zN      $Nz     \\"
       puts $outUnit " -xT      $Tx     -yT      $Ty     -zT      $Tz     \\"
       puts $outUnit " -xMODE   $MODEx  -yMODE   $MODEy  -zMODE   $MODEz  \\"
       puts $outUnit " -xSW     $SWx    -ySW     $SWy    -zSW     $SWz    \\"
       puts $outUnit " -xOBS    $OBSx   -yOBS    $OBSy   -zOBS    $OBSz   \\"
       puts $outUnit " -xCAR    $CARx   -yCAR    $CARy   -zCAR    $CARz   \\"
       puts $outUnit " -xFT     Freq    -yFT     Freq    -zFT     Freq    \\"
       puts $outUnit " -xLAB    XPixel  -yLAB    YPixel  -zLAB    IR      \\"
       puts $outUnit " -ndim    3       -aq2D    Image                    \\"
       puts $outUnit " -out pca/pca%03d.dat -ov"

       puts $outUnit "pcaNMR -in pca/pca%03d.dat -pca $dirName/ir%03d.dat -nc 5"
       puts $outUnit "rm -rf pca"
      } \
   else \
      {
       set dirName   pca
       set tifName   pca.tif

       set objInfo(data,hName) $dirName/pca001.dat
       set objInfo(data,vName) $dirName/pca002.dat
       set objInfo(data,zName) $dirName/pca003.dat

       puts $outUnit \
        "pcaNMR -in $objInfo(data,name) -pca $dirName/pca%03d.dat -nc 5"
      }

   close $outUnit

   if {![file isdirectory $dirName]} {file mkdir $dirName}

   if {$pcFlag} \
      {
       system -wait sh scriptName 
      } \
   else \
      {
       set g "=80x10+20-10"
       catch {[exec xterm -geom $g -bg cyan -fg black -e $scriptName]}
      }

   rgbImage $tifName
}

proc rgbImage { tifName } \
{
   global exeExt objInfo DefHROI DefVROI DefZROI pcFlag

   openHV hName roiH $DefHROI
   openHV vName roiV $DefVROI
   openHV zName roiZ $DefZROI

   writeImage -tiff -out $tifName \
              -roiB $objInfo(data,roiH) \
              -roiR $objInfo(data,roiV) \
              -roiG $objInfo(data,roiZ) -ov

   clearWin winHV
   drawHV

   if {$pcFlag} \
      {
       system -nowin xv -expand $objInfo(data,expand) $tifName
      } \
   else \
      {
       exec xv -expand $objInfo(data,expand) $tifName &
      }
}

proc setCanvasSize { obj } \
{
   global objInfo

   set objInfo($obj,x1) [expr $objInfo($obj,leftMargin) + 1]
   set objInfo($obj,y1) [expr $objInfo($obj,topMargin)  + 1]

   set objInfo($obj,xn) [expr $objInfo($obj,x1) + $objInfo($obj,xSize) - 1]
   set objInfo($obj,yn) [expr $objInfo($obj,y1) + $objInfo($obj,ySize) - 1]

   set objInfo($obj,width)  [expr $objInfo($obj,xn)+$objInfo($obj,rightMargin)]
   set objInfo($obj,height) [expr $objInfo($obj,yn)+$objInfo($obj,bottomMargin)]

   set objInfo($obj,mouseX) [expr 0.5*($objInfo($obj,x1) + $objInfo($obj,xn))]
   set objInfo($obj,mouseY) [expr 0.5*($objInfo($obj,y1) + $objInfo($obj,yn))]

   $objInfo($obj,canvas) configure \
      -width $objInfo($obj,width) -height $objInfo($obj,height)

   $objInfo($obj,canvas) create pixmap 0 0 \
      -width $objInfo($obj,width) -height $objInfo($obj,height) \
      -win $objInfo($obj,pixmap) -tags $obj

   defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
      -x $objInfo($obj,x1) $objInfo($obj,xn) \
      -y $objInfo($obj,y1) $objInfo($obj,yn)

   clearWin $obj
}

proc initVars { newName } \
{
   global objInfo

   setOSType

   set objInfo(data,name)       $newName

   set objInfo(win1D,xAxisFlag) 1
   set objInfo(win1D,yAxisFlag) 1

   set objInfo(win2D,xAxisFlag) 1
   set objInfo(win2D,yAxisFlag) 1

   set objInfo(winHV,xAxisFlag) 0
   set objInfo(winHV,yAxisFlag) 0

   set objInfo(color,win1Dbg)     black
   set objInfo(color,win1Dfg)     yellow
   set objInfo(color,win1Dshadow) black

   set objInfo(color,win2Dbg)     black
   set objInfo(color,win2Dfg)     yellow
   set objInfo(color,win2Dshadow) black

   set objInfo(data,hName) None
   set objInfo(data,vName) None
   set objInfo(data,zName) None
}

proc initArgs {} \
{
   global ARGC ARGV objInfo

 if {[flagLoc $ARGV -help]} \
    {
     puts "imageView.tcl: Display and Multivariate Analysis of Spectral Images."
     puts " -in     inName  Input Template for Spectral Image."
     puts " -hName  hName   2D Plane for Horizontal Axis of Scatterplot."
     puts " -vName  vName   2D Plane for Vertical Axis of Scatterplot."
     puts " -zName  zName   2D Plane for Colorizing Scatterplot."
     exit 0
    }

   getArgD $ARGV -in    objInfo(data,name) 
   getArgD $ARGV -hName objInfo(data,hName) 
   getArgD $ARGV -vName objInfo(data,vName) 
   getArgD $ARGV -zName objInfo(data,zName) 
}

proc init1DWindow {} \
{
   global objInfo pcFlag Def1DPixmap

   set obj win1D

   if {$objInfo($obj,xAxisFlag)} \
      {
       if {$pcFlag} \
          {
           set objInfo($obj,leftMargin)   12 
           set objInfo($obj,rightMargin)  12 
          } \
       else \
          {
           set objInfo($obj,leftMargin)   20 
           set objInfo($obj,rightMargin)  20 
          }
      } \
   else \
      {
       set objInfo($obj,leftMargin)   8
       set objInfo($obj,rightMargin)  8 
      }

   if {$objInfo($obj,yAxisFlag)} \
      {
       if {$pcFlag} \
          {
           set objInfo($obj,topMargin)     15 
           set objInfo($obj,bottomMargin)  60 
          } \
       else \
          {
           set objInfo($obj,topMargin)    20
           set objInfo($obj,bottomMargin) 70
          }
      } \
   else \
      {
       set objInfo($obj,topMargin)     8
       set objInfo($obj,bottomMargin)  8 
      }

   set objInfo($obj,xSize)        480
   set objInfo($obj,ySize)        220

   set objInfo($obj,cell)         $Def1DPixmap
   set objInfo($obj,pixmap)       $Def1DPixmap 
   set objInfo($obj,window)       .t
   set objInfo($obj,canvas)       .t.c

   if {![winfo exists $objInfo($obj,window)]} \
      {
       toplevel $objInfo($obj,window)
       canvas   $objInfo($obj,canvas) -border 0

       pack .t.c    -in .t -side top -expand 1 -fill both

       bind .t.c <B1-Motion>        "scale1D  %x %y"
       bind .t.c <B2-Motion>        "move1D   %x %y"
       bind .t.c <ButtonRelease-3>  "find2D   %x %y"
       bind .t.c <Motion>           "track1D  %x %y"
       bind .t.c <Leave>            "untrack1D"
      }

   wm title $objInfo($obj,window) "1D Spectrum"

   setCanvasSize $obj
}

proc init2DWindow {} \
{
   global objInfo pcFlag Def2DPixmap

   set obj win2D

   if {$objInfo($obj,xAxisFlag)} \
      {
       if {$pcFlag} \
          {
           set objInfo($obj,leftMargin)   15 
           set objInfo($obj,rightMargin)  75 
          } \
       else \
          {
           set objInfo($obj,leftMargin)   20 
           set objInfo($obj,rightMargin)  80 
          }
      } \
   else \
      {
       set objInfo($obj,leftMargin)   8
       set objInfo($obj,rightMargin)  8 
      }

   if {$objInfo($obj,yAxisFlag)} \
      {
       if {$pcFlag} \
          {
           set objInfo($obj,topMargin)     15 
           set objInfo($obj,bottomMargin)  60 
          } \
       else \
          {
           set objInfo($obj,topMargin)    20
           set objInfo($obj,bottomMargin) 70
          }
      } \
   else \
      {
       set objInfo($obj,topMargin)     8
       set objInfo($obj,bottomMargin)  8 
      }

#
# General window parameters:


   set objInfo($obj,cell)         $Def2DPixmap 
   set objInfo($obj,pixmap)       $Def2DPixmap 
   set objInfo($obj,window)       .
   set objInfo($obj,canvas)       .c

#
# Command panal and menus:

   if {![winfo exists .cmndFrame]} \
      {
       frame  .cmndFrame

      menubutton .bFile -relief flat -menu .bFile.m -text "File"    -underline 0
      menubutton .bDraw -relief flat -menu .bDraw.m -text "Draw"    -underline 0
      menubutton .bProc -relief flat -menu .bProc.m -text "Process" -underline 0
      menubutton .bHelp -relief flat -menu .bHelp.m -text "Help"    -underline 0

       menu       .bFile.m
       menu       .bDraw.m
       menu       .bProc.m
       menu       .bHelp.m

#      .bFile.m add command -label "Open ..."          -command openViewer
#      .bFile.m add command -label "Print ..."         -command printViewer
       .bFile.m add command -label "Exit"              -command quitViewer

       .bDraw.m add command -label "Next Image"        -command next2D
       .bDraw.m add command -label "Previous Image"    -command prev2D
       .bDraw.m add command -label "Redraw Image"      -command draw2D
       .bDraw.m add command -label "Full View"         -command full2D
       .bDraw.m add command -label "Draw Scatterplot"  -command drawHV
       .bDraw.m add command -label "Clear Scatterplot" -command "clearWin winHV"
#      .bDraw.m add command -label "Settings ..."      -command showSettings 
#      .bDraw.m add command -label "Color Table ..."   -command showColorTable 
    
#      .bProc.m   add command -label "Peak Detect ..." -command pkDetect
#      .bProc.m   add command -label "Filter 1D ..."   -command filter1D
#      .bProc.m   add command -label "Filter 2D ..."   -command filter1D
       .bProc.m   add command -label "PCA"             -command pca3D

       .bHelp.m   add command -label "Help ..."            -command helpViewer
       .bHelp.m   add command -label "About ImageView ..." -command aboutViewer

       pack    .bFile .bDraw .bProc -in .cmndFrame -side left
       pack    .bHelp -in .cmndFrame -side right
    
       pack    .cmndFrame   -side top -fill x
      }

#
# 2D Drawing canvas:

   if {![winfo exists $objInfo($obj,canvas)]} \
      {
       canvas  $objInfo($obj,canvas) -border 0
       pack    .c         -side top -fill both -expand 1

       bind    .c  <ButtonPress-1>    "new1D     %x %y"
       bind    .c  <B1-Motion>        "new1D     %x %y"
       bind    .c  <ButtonPress-3>    "zoomON    %x %y"
       bind    .c  <ButtonRelease-3>  "zoom2D    %x %y"
       bind    .c  <B3-Motion>        "trackZoom %x %y"
       bind    .c  <Motion>           "track2D   %x %y"
       bind    .c  <Leave>            "untrack2D"

       frame .buttonFrame

       button .uh -text "Scatterplot H"     -command "usePlane h"
       button .uv -text "Scatterplot V"     -command "usePlane v"
       button .uz -text "Scatterplot Color" -command "usePlane z"

       pack  .uh .uv .uz -in .buttonFrame -side left -expand 1 -fill x
       pack  .buttonFrame -side bottom -expand 1 -fill x -before .c

       label .verLabel -text "ImageView (C) F. Delaglio 2019.067.17.12"
       pack  .verLabel -side bottom -before .c 
      }

#
# 2D Drawing Area:

   set xSize $objInfo(data,xSize)
   set ySize $objInfo(data,ySize)

   if {$xSize > $ySize} \
      {
       set maxSize $xSize
      } \
   else \
      {
       set maxSize $ySize
      }

   if {$maxSize < 300} \
      {
       set f [expr 300.0/$maxSize]
      } \
   else \
      {
       set f 1.0
      }

   set objInfo(data,expand) $f
   set objInfo($obj,xSize)  [expr int($f*$xSize)]
   set objInfo($obj,ySize)  [expr int($f*$ySize)]

   wm title $objInfo($obj,window) "2D Image Plane"

   setCanvasSize $obj
}

proc initHVDrawWindow {} \
{
   global objInfo DefHVPixmap

#
# General window parameters:

   set obj winHV

   set objInfo($obj,cell)         $DefHVPixmap
   set objInfo($obj,pixmap)       $DefHVPixmap 
   set objInfo($obj,window)       .s
   set objInfo($obj,canvas)       .s.c

   set objInfo($obj,leftMargin)    8 
   set objInfo($obj,topMargin)     8 
   set objInfo($obj,rightMargin)   8 
   set objInfo($obj,bottomMargin)  8 

   set objInfo($obj,xSize)       220
   set objInfo($obj,ySize)       220

   if {![winfo exists $objInfo($obj,window)]} \
      {
       toplevel $objInfo($obj,window)

       canvas  $objInfo($obj,canvas) -border 0
       pack    .s.c -in .s -side top -fill both -expand 1

       bind    .s.c  <Motion>           "trackHV  %x %y"
       bind    .s.c  <Leave>            "untrackHV"
   
       .s.c delete box
      }
 
   wm title $objInfo($obj,window) Scatterplot

   setCanvasSize $obj

   .s.c create rectangle 3 3 5 5 -outline white -tags box
}

proc initHVFileWindow {} \
{
   global objInfo env DefHROI DefVROI DefZROI

   set obj                  winF
   set objInfo($obj,window) .f

   if {![winfo exists $objInfo($obj,window)]} \
      {
       toplevel $objInfo($obj,window)

       frame    .f.hFrame
       frame    .f.vFrame
       frame    .f.zFrame

       label .f.hLabel  -text "1. (Horizontal):" -anchor w -width 16 
       label .f.vLabel  -text "2. (Vertical):  " -anchor w -width 16 
       label .f.zLabel  -text "3. (Colorize):  " -anchor w -width 16 

       button .f.hMenu -command "getHVName hName roiH $DefHROI" \
          -relief raised -bitmap @$env(NMRTXT)/caret.map -fg green

       button .f.vMenu -command "getHVName vName roiV $DefVROI" \
          -relief raised -bitmap @$env(NMRTXT)/caret.map -fg green

       button .f.zMenu -command "getHVName zName roiZ $DefZROI" \
          -relief raised -bitmap @$env(NMRTXT)/caret.map -fg green

       entry .f.hName -relief sunken -width 20 -textvariable objInfo(data,hName)
       entry .f.vName -relief sunken -width 20 -textvariable objInfo(data,vName)
       entry .f.zName -relief sunken -width 20 -textvariable objInfo(data,zName)

       pack  .f.hLabel .f.hName .f.hMenu -in .f.hFrame -side left
       pack  .f.vLabel .f.vName .f.vMenu -in .f.vFrame -side left
       pack  .f.zLabel .f.zName .f.zMenu -in .f.zFrame -side left

       pack  .f.hFrame .f.vFrame .f.zFrame -in .f -side top

       bind   .f.hName <Return> {+ openHV hName roiH $DefHROI}
       bind   .f.vName <Return> {+ openHV vName roiV $DefVROI}
       bind   .f.zName <Return> {+ openHV zName roiZ $DefZROI}
      }
 
   wm title $objInfo($obj,window) "Scatterplot Images"
}

proc initMinMaxWindow {} \
{
   global objInfo 

   set obj                  winMM
   set objInfo($obj,window) .w

   if {"$objInfo(data,sign)" == "PlusOnly"} \
      {
       set first 0.0
       set last  100.0
      } \
   else \
      {
       set first -100.0
       set last   100.0
      }

   if {![winfo exists $objInfo($obj,window)]} \
      {
       toplevel $objInfo($obj,window)

       frame    .w.top
       frame    .w.bot

       scale .w.sMin -repeatdelay 2000 \
        -resolution 0.1 -from $first -to $last -showvalue 0 \
        -length 256 -orient horiz -command setMinPct        \
        -variable objInfo(data,minPct)

       scale .w.sMax -repeatdelay 2000 \
        -resolution 0.1 -from $first -to $last -showvalue 0 \
        -length 256 -orient horiz -command setMaxPct        \
        -variable objInfo(data,maxPct)

       entry .w.eMin -textvariable objInfo(data,minPct) -width 6
       entry .w.eMax -textvariable objInfo(data,maxPct) -width 6

       bind  .w.eMin <Return> {+ setMinPct $objInfo(data,minPct)}
       bind  .w.eMax <Return> {+ setMaxPct $objInfo(data,maxPct)}

       pack  .w.eMin .w.sMin -in .w.top -side left
       pack  .w.eMax .w.sMax -in .w.bot -side left

       pack  .w.top .w.bot -side top
      }

   wm title $objInfo($obj,window) "Display Max and Min in Percent"
}

proc initIZWindow {} \
{
   global objInfo

   set obj                  winIZ
   set objInfo($obj,window) .z

   if {![winfo exists $objInfo($obj,window)]} \
      {
       toplevel $objInfo($obj,window)

       scale .z.sIZ -repeatdelay 2000 \
        -from 1 -to $objInfo(data,zSize) -showvalue 0 \
        -length 256 -orient horiz -command new2D -variable objInfo(data,iz)

       entry .z.eIZ -textvariable objInfo(data,iz) -width 6
    
       pack  .z.eIZ .z.sIZ -side left
       bind  .z.eIZ <Return> "+ new2D"
      }

   wm title $objInfo($obj,window) "Plane Number"
}

proc initSpectralData {} \
{
   global objInfo

   set obj data 

   set objInfo($obj,roi1D)   0
   set objInfo($obj,roi2D)   0
   set objInfo($obj,roi3D)   0
   set objInfo($obj,roiH)    0
   set objInfo($obj,roiV)    0
   set objInfo($obj,roiZ)    0
   set objInfo($obj,rdata1D) 0
   set objInfo($obj,rdata2D) 0
   set objInfo($obj,rdata3D) 0
   set objInfo($obj,fdata)   0

   set objInfo($obj,xSize)   300
   set objInfo($obj,ySize)   300
   set objInfo($obj,zSize)   64
   set objInfo($obj,sign)    PlusOnly

#
# Read data cube.
# Extract size parameters:

   new3D

   if {!$objInfo(data,roi3D)} return

   set fdata $objInfo(data,fdata)

   set objInfo(data,tpFlag) [getParm $fdata FDTRANSPOSED NULL_DIM]
   
   if {$objInfo(data,tpFlag)} \
      {
       set hDim CUR_YDIM
       set vDim CUR_XDIM
      } \
   else \
      {
       set hDim CUR_XDIM
       set vDim CUR_YDIM
      }

   set objInfo($obj,xAxis) [getParmStr $fdata NDLABEL $hDim] 
   set objInfo($obj,yAxis) [getParmStr $fdata NDLABEL $vDim] 
   set objInfo($obj,zAxis) [getParmStr $fdata NDLABEL CUR_ZDIM]

   set objInfo($obj,xSize) [expr int([getParm $fdata NDSIZE $hDim])]
   set objInfo($obj,ySize) [expr int([getParm $fdata NDSIZE $vDim])]
   set objInfo($obj,zSize) [expr int([getParm $fdata NDSIZE CUR_ZDIM])]

   set objInfo($obj,ix)    [expr int($objInfo($obj,xSize)/2)]
   set objInfo($obj,iy)    [expr int($objInfo($obj,ySize)/2)]
   set objInfo($obj,iz)    [expr int($objInfo($obj,zSize)/2)]

   set objInfo($obj,x1)    1
   set objInfo($obj,xn)    $objInfo($obj,xSize)
   set objInfo($obj,y1)    1
   set objInfo($obj,yn)    $objInfo($obj,ySize)

   set totalSize \
    [expr $objInfo(data,xSize)*$objInfo(data,ySize)*$objInfo(data,zSize)]

   set objInfo($obj,totalSize) $totalSize

#
# Extract misc parameters:

   set objInfo($obj,xUnits) Pts
   set objInfo($obj,yUnits) Pts

   set objInfo(data,minVal) [vMin $objInfo(data,rdata3D) $totalSize]
   set objInfo(data,maxVal) [vMax $objInfo(data,rdata3D) $totalSize]

   set range [expr $objInfo(data,maxVal) - $objInfo(data,minVal)]

   set objInfo(data,minVal) [expr $objInfo(data,minVal) - 0.1*$range]
   set objInfo(data,maxVal) [expr $objInfo(data,maxVal) + 0.1*$range]

   if {$objInfo(data,minVal) < 0.0} \
      {
       set objInfo(data,sign)     PlusMinus
       set objInfo(data,maxPct)   100.0

       if {$objInfo(data,maxVal) == 0.0} \
          {
           set objInfo(data,minPct) -100.0
          } \
       else \
          {
           set objInfo(data,minPct) \
               [expr 100.0*$objInfo(data,minVal)/$objInfo(data,maxVal)]

           set objInfo(data,minPct) 0.0 
          }
      } \
   else \
      {
       set objInfo(data,sign)     PlusOnly
       set objInfo(data,maxPct)   100.0
       set objInfo(data,minPct)   0.0
      }

#
# Adjust display parameters according to data type:

   switch $objInfo($obj,zAxis) \
      {
       Fluor   {set objInfo($obj,zUnits) WL-nm}

       IR      {set objInfo($obj,zUnits) WN}

       PCA     {
                set objInfo($obj,zUnits)      Pts 
                set objInfo(data,iz)            1
               }

       default {set objInfo($obj,zUnits) Pts}
      }

   set objInfo(data,zPos) [getDataZPos $objInfo(data,iz)]
}

proc initResize {} \
{
   global objInfo

   update

   bind $objInfo(win2D,canvas) <Configure> "resizeCanvas win2D %w %h %T draw2D"
   bind $objInfo(win1D,canvas) <Configure> "resizeCanvas win1D %w %h %T draw1D"
   bind $objInfo(winHV,canvas) <Configure> "resizeCanvas winHV %w %h %T drawHV"
}

proc initPositions {} \
{
   global objInfo

   update

   set dx 15
   set dy 15

   set ix $dx 
   set iy $dy 

   wm geometry $objInfo(win2D,window) +$ix+$iy

   incr ix $dx
   incr iy [expr 2*$dy]
   incr iy [winfo height $objInfo(win2D,window)]

   wm geometry $objInfo(winMM,window) +$ix+$iy

   incr iy [expr 2*$dy]
   incr iy [winfo height $objInfo(winMM,window)]

   wm geometry $objInfo(winIZ,window) +$ix+$iy

   set  iy $dy 
   incr ix $dx
   incr ix [winfo width $objInfo(win2D,window)]

   wm geometry $objInfo(win1D,window) +$ix+$iy

   incr iy [expr 3*$dy]
   incr iy [winfo height $objInfo(win1D,window)]

   wm geometry $objInfo(winHV,window) +$ix+$iy

   incr ix $dx
   incr ix [winfo width $objInfo(winHV,window)]
   incr iy $dy

   wm geometry $objInfo(winF,window) +$ix+$iy

   update
}

proc newNMR { nmrName } \
{
   global objInfo Def3DROI 

   global  Nx     Ny     Nz     \
           Tx     Ty     Tz     \
           MODEx  MODEy  MODEz  \
           SWx    SWy    SWz    \
           OBSx   OBSy   OBSz   \
           CARx   CARy   CARz

   set objInfo(data,hdrSize) 2048

   set error \
    [catch {readROI -ndim 3 -roi $Def3DROI -in $objInfo(data,name) \
                    -x X_AXIS -y Y_AXIS -z Z_AXIS}]

   if {$error} {return $error}

   set roiInfo [getROIInfo -roi $Def3DROI]
   set fdata   [lindex $roiInfo 1]

   set Nx    [expr int([getParm $fdata NDSIZE CUR_XDIM])] 
   set Tx    [expr int([getParm $fdata NDSIZE CUR_XDIM])] 

   set SWx   [getParm $fdata NDSW   CUR_XDIM]
   set OBSx  [getParm $fdata NDOBS  CUR_XDIM]
   set CARx  [getParm $fdata NDCAR  CUR_XDIM]

   set MODEx Real

   set Ny    [expr int([getParm $fdata NDSIZE CUR_YDIM])] 
   set Ty    [expr int([getParm $fdata NDSIZE CUR_YDIM])]  

   set SWy   [getParm $fdata NDSW   CUR_YDIM]
   set OBSy  [getParm $fdata NDOBS  CUR_YDIM]
   set CARy  [getParm $fdata NDCAR  CUR_YDIM]

   set MODEy Real

   set Nz    [expr int([getParm $fdata NDSIZE CUR_ZDIM])] 
   set Tz    [expr int([getParm $fdata NDSIZE CUR_ZDIM])]  

   set SWz   [getParm $fdata NDSW   CUR_ZDIM]
   set OBSz  [getParm $fdata NDOBS  CUR_ZDIM]
   set CARz  [getParm $fdata NDCAR  CUR_ZDIM]

   set MODEz Real

   return 0
}

proc newSpiff { spiffName fdataPtr rdataPtr } \
{
   upvar $fdataPtr fdata
   upvar $rdataPtr rdata

   global pcFlag exeExt spfHdr objInfo SYS_BYTESWAP FDATASIZE

   global  Nx     Ny     Nz     \
           Tx     Ty     Tz     \
           MODEx  MODEy  MODEz  \
           SWx    SWy    SWz    \
           OBSx   OBSy   OBSz   \
           CARx   CARy   CARz

   set inUnit [sysOpen $spiffName r]

   set spfHdr(error)      0
   set spfHdr(xsize)      0
   set spfHdr(ysize)      0
   set spfHdr(imag_size)  0
   set spfHdr(num_points) 0
   set spfHdr(first_wave) 0
   set spfHdr(last_wave)  0

   set fileBytes [getFileBytes $spiffName -estimate]

   if {$SYS_BYTESWAP} \
      {
       set swapFlag "-noswap"
      } \
   else \
      {
       set swapFlag "-swap"
      }

   catch \
    {set sInfo [exec showSpiff$exeExt $swapFlag -var spfHdr -in $spiffName]}

   set error [catch "eval $sInfo"]

   if {$spfHdr(error) != 0} \
      {
       showError ImageView "Error reading SPIFF header."
       return $error
      }

   if {$spfHdr(identity) > 18289} \
      {
       set objInfo(data,hdrSize) 1020

       if {$spfHdr(identity) > 18290} {set objInfo(data,hdrSize) 1024}

       set Nx    $spfHdr(xsize)
       set Tx    $spfHdr(xsize)

       set Ny    $spfHdr(ysize)
       set Ty    $spfHdr(ysize)

       set Nz    $spfHdr(num_points) 
       set Tz    $spfHdr(num_points) 

       set extra [expr $fileBytes - $Nx*$Ny*$Nz*4]

       if {$extra == 1024} {set objInfo(data,hdrSize) 1024}
      } \
   else \
      {
       set objInfo(data,hdrSize) 128

       if {$spfHdr(imag_size) == 0} {set spfHdr(imag_size) 128}

       set Nx    $spfHdr(imag_size)
       set Tx    $spfHdr(imag_size)

       set Ny    $spfHdr(imag_size)
       set Ty    $spfHdr(imag_size)

       set Nz    $spfHdr(num_points)
       set Tz    $spfHdr(num_points)
      }

   if {$spfHdr(first_wave) >= 0.0 && $spfHdr(last_wave) < 10000.0} \
      {
       set FIRSTz  $spfHdr(last_wave)
       set LASTz   $spfHdr(first_wave)
      }

   set aq2D   Image

   set SWx    $Nx
   set MODEx  Real
   set OBSx   1.0 
   set CARx   0.0
   set FTx    Freq
   set LABx   X 

   set SWy    $Ny
   set MODEy  Real
   set OBSy   1.0
   set CARy   0.0
   set FTy    Freq
   set LABy   Y

   set size  [expr $Nz/2]
   set mid   [expr 1.0 + $size/2]
   set SWz   [expr ($FIRSTz - $LASTz)*2.99792e+10]
   set CARz  [expr $LASTz*2.99792e+10 + $SWz*($size - $mid)/(1.0*$size)]
   set MODEz Real 
   set OBSz  1.00
   set FTz   Time 
   set LABz  IR 

   set totalSize [expr $Nx*$Ny*$Nz]

   if {$totalSize < $objInfo(data,hdrSize)} \
      {
       set totalSize $objInfo(data,hdrSize)
      }

   set error [catch {set fdata [sysMalloc $FDATASIZE float]}]
   set error [catch {set rdata [sysMalloc $totalSize float]}]

   if {$error} \
      {
       showError ImageView "Error allocating SPIFF data."
       return $error
      }

   buildHdr $fdata \
     -xN      $Nx     -yN      $Ny     -zN      $Nz     \
     -xT      $Tx     -yT      $Ty     -zT      $Tz     \
     -xMODE   $MODEx  -yMODE   $MODEy  -zMODE   $MODEz  \
     -xSW     $SWx    -ySW     $SWy    -zSW     $SWz    \
     -xOBS    $OBSx   -yOBS    $OBSy   -zOBS    $OBSz   \
     -xCAR    $CARx   -yCAR    $CARy   -zCAR    $CARz   \
     -xFT     Freq    -yFT     Freq    -zFT     Freq    \
     -xLAB    XPixel  -yLAB    YPixel  -zLAB    IR      \
     -ndim    3       -aq2D    Image

    set inUnit [sysOpen $spiffName r]

    if {$inUnit < 0} \
       {
        showError ImageView "Error opening data file $spiffName"
        return $error
       }

    set error [catch {sysRead $inUnit $rdata $objInfo(data,hdrSize) 1}]
    set rPtr  [incrPtr $rdata $totalSize]
    set jump  [expr $Nx*$Ny]

    for {set iz 0} {$iz < $Nz} {incr iz} \
        {
         set rPtr  [incrPtr $rPtr -$jump]
         set error [catch {sysRead $inUnit $rPtr $jump}]

         if {!$SYS_BYTESWAP} {vByteSwap $rPtr $jump}
        }

    if {$error} \
       {
        showError ImageView "Error reading header from file $spiffName"
        return $error
       }

   sysClose $inUnit

   return 0
}

proc initImageView {} \
{
   global objInfo

   initSpectralData
   init2DWindow
   init1DWindow
   initMinMaxWindow
   initIZWindow
   initHVDrawWindow
   initHVFileWindow
   initResize
   initPositions

   new2D
   new1D $objInfo(win2D,mouseX) $objInfo(win2D,mouseY)

   untrack1D
   untrack2D
}

#initVars ASPIRIN1.SPC
#initVars tube1.abs
#initVars may28b.spc

initVars ft/test%04d.dat 
initArgs
initImageView

#
# Bottom.
