#!/bin/csh

set thisVersion = "2022.333.14.25"

if (`flagLoc $argv -help`) then
   echo "ist2DMag.com $thisVersion"
   echo "Perform IST Recontruction of 2D Non-Uniform Sampled (NUS) Data;"
   echo "Magnitude-Mode (Absolute Value Spectrum) Version:"
   echo " -in    inName   [test.fid]   Expanded Magnitude-Mode 2D Time-Domain Input."
   echo " -mask  maskName [mask.fid]   Corresponding 2D Time-Domain Mask Input."
   echo " -out   outName  [ist.ft2]    IST Spectrum Output."
   echo " -title title    [None]       Title for Output."
   echo "IST Parameters:"
   echo " -istTMult  t [0.70]  Fraction for Threshold at Each Iteration"
   echo " -istCMult  c [0.30]  Fraction for Reducing Data Above Threshold." 
   echo " -istMaxRes r [1.0]   Target Max Height in Residual, as % of Original Max."
   echo " -istIter   n [1024]  Maximum Allowed Iterations."
   echo "Processing Parameters (Use -x... -y...):"
   echo " -xSOL    sName  [NULL]       Solvent Function Name and Options (X-Axis Only)."
   echo " -xAPOD   aName  [SP]         Window Function Name."
   echo " -xQ1     q1     [0.05]       Window Paramater Q1."
   echo " -xQ2     q2     [0.95]       Window Paramater Q2."
   echo " -xQ3     q3     [2]          Window Parameter Q3."
   echo " -xELB    elb    [0.0]        Exponential Line Broaden, Hz."
   echo " -xGLB    glb    [0.0]        Gaussian Line Broaden, Hz."
   echo " -xGOFF   goff   [0.0]        Gaussian Center, 0.0 to 1.0."
   echo " -xC1     c      [0.5]        First-Point Time-Domain Scale (0.5 1.0)."
   echo " -xZFARG  zfArgs [zf=1,auto]  ZF (Zero Fill) Option."
   echo " -xFTARG  ftArg  [None]       FT (Fourier Transform) Options (None neg alt)."
   echo " -xEXTX1  x1     [0%]         Extract Range Start (X-Axis Only) (% Hz ppm pts)."
   echo " -xEXTXN  xn     [100%]       Extract Range End   (X-Axis Only) (% Hz ppm pts)."
   echo " -scaleTo maxVal [None]       Scale to Given Max Value, or None for No Scaling."

   echo "Baseline Correction Function Name and Options:"
   echo " -xBASEARG_FT1   [NULL]       X-Axis Interferogram Correction."
   echo " -xBASEARG       [NULL]       X-Axis Spectrum Baseline Correction."
   echo " -xBASEARG_FINAL [NULL]       X-Axis Baseline Correction for Final IST Result."
   echo " -yBASEARG       [NULL]       Y-Axis Spectrum Baseline Correction."
   echo " -yBASEARG_FINAL [NULL]       Y-Axis Baseline Correction for Final IST Result."
   echo "Baseline Correction Applied to Final Residual:"
   echo " -rBASEARG       [POLY,auto,ord=0] Residual Baseline Correction."
   echo "Other Options:"
   echo "  -noclean     Do Not Delete Intermediate IST Results."
   echo "  -noprogress  Do Not Display Progress Bar Graphic."
   echo "Notes:"
   echo " 1. Options -xSOL -xBASEARG_FT1 -xBASEARG and -xBASEARG_FINAL are expanded"
   echo "    into NMRPipe processing function names and command line arguments."
   echo "    Options should contain no spaces. Flags are separated by commas, and"
   echo "    parameters with values are given with = signs. For example:"
   echo ""
   echo "      -xSOL     NULL              Becomes: nmrPipe -fn NULL (No Solvent Filter)"
   echo "      -xSOL     SOL               Becomes: nmrPipe -fn SOL"
   echo "      -xSOL     POLY,time         Becomes: nmrPipe -fn POLY -time"
   echo "      -xBASEARG POLY,auto,ord=0   Becomes: nmrPipe -fn POLY -auto -ord 0"
   echo ""
   echo " 2. Options -xZFARG -xFTARG are expanded to command line arguments for ZF"
   echo "    and FT, for example:"
   echo ""
   echo "      -xZFARG size=2048,auto  Becomes: nmrPipe -fn ZF -size 2048 -auto"
   echo "      -xFTARG neg             Becomes: nmrPipe -fn FT -neg"
   echo ""
   echo " 3. To stop an IST calculation in progress:"
   echo ""
   echo "       touch ist.stop"
   echo ""
   echo " 4. See utility nusExpand.tcl for expanding time-domain NUS data and"
   echo "    creating time-domain NUS schedule mask."
   exit 0
endif

set inName    = (`getArgD $argv -in    test.fid`)
set outName   = (`getArgD $argv -out   ist.ft2`)
set maskName  = (`getArgD $argv -mask  mask.fid`)
set titStr    = (`getArgD $argv -title None`)

set istPrefix = (`getArgD $argv -istPrefix ist`)
set tMult     = (`getArgD $argv -istTMult  0.70`)
set cMult     = (`getArgD $argv -istCMult  0.30`)
set istIter   = (`getArgD $argv -istIter   1024`)
set istMaxRes = (`getArgD $argv -istMaxRes 1.0`)

set scaleToVal   = (`getArgD $argv -scaleTo   None`)

set istSaveFlag  = (`getLogArgD $argv -noistSave -istSave 0 1`)
set istCleanFlag = (`getLogArgD $argv -clean     -noclean 1 0`)

set thisPID      = $$
set progressFlag = 0
set sendName     = nmrProgress.tcl

if (`flagLoc $argv -progress`) then
   set progressFlag = 1
   set sendName     = nmrProgress.tcl
endif

if (`flagLoc $argv -noprogress`) then
   set progressFlag = 0
   set sendName     = None
endif

if ($progressFlag != 0) then
   echo "Stopping any existing progress bar displays ..."
   nmrProgress.com -stop -noprogress
   echo "Starting new progress bar display ..."
   nmrProgress.tcl &
   sleep 3
   echo "Done starting new progress bar display."
endif

if (!(-e $inName)) then
   echo "ist2DMag.com Notice: Missing Input $inName"
   exit 1
endif

set xSOL   = (`getArgD $argv -xSOL   NULL`)
set xAPOD  = (`getArgD $argv -xAPOD  SP`)
set xQ1    = (`getArgD $argv -xQ1    0.05`)
set xQ2    = (`getArgD $argv -xQ2    0.95`)
set xQ3    = (`getArgD $argv -xQ3    2`)
set xELB   = (`getArgD $argv -xELB   0.0`)
set xGLB   = (`getArgD $argv -xGLB   0.0`)
set xGOFF  = (`getArgD $argv -xGOFF  0.0`)
set xC1    = (`getArgD $argv -xC1    0.5`)
set xZFARG = (`getArgD $argv -xZFARG zf=1,auto`)
set xFTARG = (`getArgD $argv -xFTARG None`)
set xP0    = 0.0
set xP1    = 0.0
set xEXTX1 = (`getArgD $argv -xEXTX1 0%`)
set xEXTXN = (`getArgD $argv -xEXTXN 100%`)

set xBASEARG_FT1   = (`getArgD $argv -xBASEARG_FT1   NULL`)
set xBASEARG       = (`getArgD $argv -xBASEARG       NULL`)
set xBASEARG_FINAL = (`getArgD $argv -xBASEARG_FINAL NULL`)

set rBASEARG       = (`getArgD $argv -rBASEARG       POLY,auto,ord=0`)

set yAPOD  = (`getArgD $argv -yAPOD  SP`)
set yQ1    = (`getArgD $argv -yQ1    0.05`)
set yQ2    = (`getArgD $argv -yQ2    0.95`)
set yQ3    = (`getArgD $argv -yQ3    2`)
set yELB   = (`getArgD $argv -yELB   0.0`)
set yGLB   = (`getArgD $argv -yGLB   0.0`)
set yGOFF  = (`getArgD $argv -yGOFF  0.0`)
set yC1    = (`getArgD $argv -yC1    0.5`)
set yZFARG = (`getArgD $argv -yZFARG zf=1,auto`)
set yFTARG = (`getArgD $argv -yFTARG None`)
set yP0    = 0.0
set yP1    = 0.0

set yBASEARG       = (`getArgD $argv -yBASEARG       NULL`)
set yBASEARG_FINAL = (`getArgD $argv -yBASEARG_FINAL NULL`)

set multDir  = ${istPrefix}mult
set specDir  = ${istPrefix}spec

set istName  = $outName
set ft1Name  = ${istPrefix}.ft1
set istMask  = ${istPrefix}_mask.ft1
set sumName  = ${istPrefix}_sum.ft2
set resName  = ${istPrefix}_res.ft2
set invName  = ${istPrefix}_inv.ft1
set tmp1Name = ${istPrefix}_tmp.ft1
set tmp2Name = ${istPrefix}_tmp.ft2

if (-d $multDir) then
   /bin/rm -rf $multDir
endif

if (-d $specDir) then
   /bin/rm -rf $specDir 
endif

foreach f ($tmp1Name $tmp2Name $ft1Name $sumName $invName $istName)
   if (-e $f) then
      /bin/rm -f $f
   endif 
end

if ($istIter < 1) then
   exit 0
endif

mkdir $multDir
mkdir $specDir

nmrPipe -in $inName \
| nmrPipe -fn `expandArg0 $xSOL` \
| nmrPipe -fn APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 \
| nmrPipe -fn ZF `expandArg $xZFARG` \
| nmrPipe -fn FT `expandArg $xFTARG` \
| nmrPipe -fn EXT -x1 $xEXTX1 -xn $xEXTXN -sw \
| nmrPipe -fn `expandArg0 $xBASEARG_FT1` \
| nmrPipe -fn TP \
| nmrPipe -fn APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1 \
| nmrPipe -fn ZF `expandArg $yZFARG` \
| nmrPipe -fn FT `expandArg $yFTARG` \
| nmrPipe -fn `expandArg0 $yBASEARG` \
| nmrPipe -fn TP \
| nmrPipe -fn `expandArg0 $xBASEARG` \
| nmrPipe -fn TP \
| nmrPipe -fn FT -inv \
| nmrPipe -fn EXT -time -sw \
  -out $ft1Name -ov

nmrPipe -in $maskName \
| nmrPipe -fn ZF `expandArg $xZFARG` \
| nmrPipe -fn MAC -macro $NMRTXT/setFirst.M \
| nmrPipe -fn FT -null \
| nmrPipe -fn EXT -x1 $xEXTX1 -xn $xEXTXN -sw \
| nmrPipe -fn TP \
  -out $istMask -ov

set thisInName = $ft1Name

set i  = 1
set ii = 1

echo ""

if ($progressFlag != 0) then
   set thisMsg = "nmrProgress -loc 0 -n $istIter -pct 0.0 -fromID 1 -pid $thisPID -title IST Convergence (steps get smaller) -axis None"
   nmrSend -norecv -send $sendName -msg "$thisMsg"
endif

if (-e ist.stop) then
   /bin/rm -f ist.stop
endif

set convFlag = 0

while( $i <= $istIter )
   if ($istSaveFlag) then
      set ii = $i
   else
      set ii = 1
   endif
 
   addNMR -in1 $thisInName -in2 $istMask -out $tmp1Name -mul

   nmrPipe -in $tmp1Name \
   | nmrPipe -fn ZF `expandArg $yZFARG` \
   | nmrPipe -fn FT -di \
      -out $tmp2Name -ov

#  set thresh = (`scale2D -brief $tmp2Name`)

   set minVal  = (`getParm -in $tmp2Name -parm FDMIN`)
   set maxVal  = (`getParm -in $tmp2Name -parm FDMAX`)
   set thresh  = (`EMATH "fMax(abs($minVal),abs($maxVal))"`)

   if ($i == 1) then
      set firstThresh = $thresh
   endif

   set thisThresh = (`MATH "100.0*div($thresh,$firstThresh)"`)

   nmrPrintf " %4d. Threshold: %8.3f%%\n" $i $thisThresh

   set thresh   = (`MATH "$thresh*$tMult"`)
   set multName = (`nmrPrintf $multDir/spec%03d.ft2 $ii`)
   set specName = (`nmrPrintf $specDir/spec%03d.ft2 $ii`)

   nmrPipe -in $tmp2Name -out $multName -ov -fn MULT -c $cMult -thresh  $thresh
   nmrPipe -in $tmp2Name -out $specName -ov -fn MULT -c $cMult -thresh -$thresh

   if ($i == 1) then
      nmrPipe -in $specName -out $sumName -fn SET -c 0.0 -ov
   endif

   addNMR -in1 $sumName -in2 $specName -out $tmp2Name -add
   /bin/mv $tmp2Name $sumName

   nmrPipe -in $multName \
   | nmrPipe -fn HT -auto -nozf \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn EXT -time -sw \
     -out $invName -ov

   if ($progressFlag != 0) then
      set progPct = (`MATH "100.0 - 100.0*div($thisThresh-$istMaxRes, 100.0-$istMaxRes)"`)
      set thisMsg = "nmrProgress -loc $i -pct `nmrPrintf %.3f $progPct` -n $istIter -fromID 1 -pid $thisPID -title IST Convergence (steps get smaller) -axis None"
      nmrSend -norecv -send $sendName -msg "$thisMsg"
   endif

   set thisInName = $invName 
   @ i++

   if (`IMATH "$thisThresh < $istMaxRes"`) then
      set convFlag = 1
      break
   endif 

  if (-e ist.stop) then
     echo "Found File ist.stop ... stopping IST Calculation."
     /bin/rm -f ist.stop
     @ convFlag = 1
     break
  endif
end

if ($progressFlag != 0) then
   if ($convFlag == 0) then
      set statusStr = "Iteration Count Exceeded."
   else
      set statusStr = "None"
   endif

   set thisMsg = "nmrProgress -loc $istIter -n $istIter -pct 100.0 -fromID 1 -pid $thisPID -title IST Convergence -axis None -status $statusStr"

   nmrSend -norecv -send $sendName -msg "$thisMsg"
endif

echo ""

nmrPipe -in $multName -out $resName -ov -fn `expandArg0 $rBASEARG`
addNMR  -in1 $sumName -in2 $resName -out $istName -add

nmrPipe -inPlace -in $resName -out $resName -fn TP -ov
nmrPipe -inPlace -in $sumName -out $sumName -fn TP -ov

nmrPipe -in $istName \
| nmrPipe -fn `expandArg0 $yBASEARG_FINAL` \
| nmrPipe -fn HT \
| nmrPipe -fn MC \
| nmrPipe -fn TP \
| nmrPipe -fn `expandArg0 $xBASEARG_FINAL` \
| nmrPipe -fn `expandArg0 $xBASEARG_FINAL` \
| nmrPipe -inPlace -out $istName -ov

if (`string notcsame $scaleToVal None`) then
   rescale.com -in $istName -out $istName -max $scaleToVal
endif

if (`string casecmp $titStr None`) then
   sethdr.com $istName -title $titStr
endif

series.com $multDir/*.ft2
series.com $specDir/*.ft2

if ($istCleanFlag) then
   set fList = (ist_sum.ft2 ist_res.ft2 ist_inv.ft1 ist_tmp.ft1 ist_mask.ft1 ist.ft1 istspec istmult)

   foreach f ($fList)
      if (-e $f) then
         /bin/rm -rf $f
      endif
   end
endif

exit 0
