#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Update Table from Local Fitting of Peaks:"
    puts " -in     inName  \[master.tab\]      Original Peak Table."
    puts " -local  locName \[nlin_local.tab\]  Local Fit Result Peak Table."
    puts " -out    outName \[nlin.tab\]        Output Table."
    puts " -ft     ftName  \[test.ft2\]        Reference Spectrum."
    puts " -ndim   ndim    \[Auto\]            Dimension Count of Peak Table."
    puts " -verb                             Verbose Mode ON (Default)."
    puts " -noverb                           Verbose Mode OFF."
    puts "Linewidth Clipping (-x... -y... etc) (ppm, Hz, pts or %), or Keywords Auto None:"
    puts " -xwMin  xwMin   \[0.0\]             X-Axis Minimum Linewidth."
    puts " -xwMax  xwMax   \[None\]            X-Axis Maximum Linewidth."
    puts "Automation Parameters (Use -x... -y... etc):"
    puts " -xwVar  xwVar   \[XW\]              Table Variable for X-Axis Linewidth." 
    puts " -xwMult xwMult  \[2.0\]             X-Axis Linewidth Multiplier."
    exit 0
   }

set inName     master.tab
set localName  nlin_local.tab
set outName    nlin.tab
set ftName     test.ft2

set dimCount   Auto

set xwVar   XW
set ywVar   YW
set zwVar   ZW
set awVar   AW

set xwMult  2.0
set ywMult  2.0
set zwMult  2.0
set awMult  2.0

set xwMin  0.0
set ywMin  0.0 
set zwMin  0.0 
set awMin  0.0 

set xwMax  None 
set ywMax  None 
set zwMax  None 
set awMax  None 

set verbose 1

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -local localName
getArgD $ARGV -ft    ftName

getArgD $ARGV -ndim  dimCount

getArgD $ARGV -xwVar xwVar
getArgD $ARGV -ywVar ywVar
getArgD $ARGV -zwVar zwVar
getArgD $ARGV -awVar awVar

getArgD $ARGV -xwMult xwMult 
getArgD $ARGV -ywMult ywMult 
getArgD $ARGV -zwMult zwMult 
getArgD $ARGV -awMult awMult 

getArgD $ARGV -xwMin xwMin 
getArgD $ARGV -ywMin ywMin 
getArgD $ARGV -zwMin zwMin 
getArgD $ARGV -awMin awMin 

getArgD $ARGV -xwMax xwMax 
getArgD $ARGV -ywMax ywMax 
getArgD $ARGV -zwMax zwMax 
getArgD $ARGV -awMax awMax 

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#-------------------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "localUpdate. Error finding original input peak table $inName"
    exit 1
   }

if {![file exists $localName]} \
   {
    puts stderr "localUpdate. Error finding local fit input peak table $localName"
    exit 1
   }

set thisName [format $ftName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "localUpdate. Error finding input spectrum $ftName ($thisName)"
    exit 1
   }

set t       [gdbRead table -in $inName]
set tLocal  [gdbRead table -in $localName]

set eList   [gdbGetInfo $t -parms entries]
set varList [gdbGetInfo $t -parms varNames]

set error [rdFDATA $ftName fdata]

if {![strcasecmp $dimCount Auto]} \
   {
    if {[lsearch -exact $varList $xwVar] >= 0} {set dimCount 1}
    if {[lsearch -exact $varList $ywVar] >= 0} {set dimCount 2}
    if {[lsearch -exact $varList $zwVar] >= 0} {set dimCount 3}
    if {[lsearch -exact $varList $awVar] >= 0} {set dimCount 4}
   }

set xwList ""
set ywList ""
set zwList ""
set awList ""

foreach e $eList \
   {
    set id [gdbGet $e INDEX]

    set nearList [gdbSelect -from $tLocal -cond "CLUSTID == $id"]

    if {[llength $nearList] < 1} \
       {
        puts stderr "localUpdate. Error finding CLUSTID $id in fit result $localName"
        exit 1
       }

    set eNew  [lindex $nearList 0]

    foreach var $varList \
       {
        if {[lsearch -exact "INDEX CLUSTID MEMCNT" $var] < 0} \
           {
            gdbSet $e $var [gdbGet $eNew $var]
           }
       }

    if {$dimCount >= 1} {lappend xwList [gdbGet $e $xwVar]}
    if {$dimCount >= 2} {lappend ywList [gdbGet $e $ywVar]}
    if {$dimCount >= 3} {lappend zwList [gdbGet $e $zwVar]}
    if {$dimCount >= 4} {lappend awList [gdbGet $e $awVar]}
   }

set xwAvg [getMedian $xwList]
set xwDev [getMAD    $xwList]

set ywAvg [getMedian $ywList]
set ywDev [getMAD    $ywList]

set zwAvg [getMedian $zwList]
set zwDev [getMAD    $zwList]

set awAvg [getMedian $awList]
set awDev [getMAD    $awList]

set thisXWMin [expr $xwAvg - $xwMult*$xwDev]
set thisXWMax [expr $xwAvg + $xwMult*$xwDev]

set thisYWMin [expr $ywAvg - $ywMult*$ywDev]
set thisYWMax [expr $ywAvg + $ywMult*$ywDev]

set thisZWMin [expr $zwAvg - $zwMult*$zwDev]
set thisZWMax [expr $zwAvg + $zwMult*$zwDev]

set thisAWMin [expr $awAvg - $awMult*$awDev]
set thisAWMax [expr $awAvg + $awMult*$awDev]

if {$thisXWMin < 0.0} {set thisXWMin 0.0}
if {$thisYWMin < 0.0} {set thisYWMin 0.0}
if {$thisZWMin < 0.0} {set thisZWMin 0.0}
if {$thisAWMin < 0.0} {set thisAWMin 0.0}

foreach e $eList \
   {
    if {$dimCount >= 1} \
       {
        set xw [gdbGet $e $xwVar]

        if {![strcasecmp $xwMin Auto]} \
           {
            if {$xw < $thisXWMin} {set xw $thisXWMin}
           } \
        elseif {[strcasecmp $xwMin None]} \
           {
            if {$xw < $xwMin} {set xw $xwMin}
           }

        if {![strcasecmp $xwMax Auto]} \
           {
            if {$xw > $thisXWMax} {set xw $thisXWMax}
           } \
        elseif {[strcasecmp $xwMax None]} \
           {
            if {$xw > $xwMax} {set xw $xwMax}
           }

        set xwHz [pnt2specW $fdata CUR_XDIM $xw Hz]

        gdbSet $e $xwVar $xw
        gdbSet $e XW_HZ  $xwHz
       }

    if {$dimCount >= 2} \
       {
        set yw [gdbGet $e $ywVar]

        if {![strcasecmp $ywMin Auto]} \
           {
            if {$yw < $thisXWMin} {set yw $thisYWMin}
           } \
        elseif {[strcasecmp $ywMin None]} \
           {
            if {$yw < $ywMin} {set yw $ywMin}
           }

        if {![strcasecmp $ywMax Auto]} \
           {
            if {$yw > $thisYWMax} {set yw $thisYWMax}
           } \
        elseif {[strcasecmp $ywMax None]} \
           {
            if {$yw > $ywMax} {set yw $ywMax}
           }

        set ywHz [pnt2specW $fdata CUR_YDIM $yw Hz]

        gdbSet $e $ywVar $yw
        gdbSet $e YW_HZ  $ywHz
       }

    if {$dimCount >= 3} \
       {
        set zw [gdbGet $e $zwVar]

        if {![strcasecmp $zwMin Auto]} \
           {
            if {$zw < $thisZWMin} {set zw $thisZWMin}
           } \
        elseif {[strcasecmp $zwMin None]} \
           {
            if {$zw < $zwMin} {set zw $zwMin}
           }

        if {![strcasecmp $zwMax Auto]} \
           {
            if {$zw > $thisZWMax} {set zw $thisZWMax}
           } \
        elseif {[strcasecmp $zwMax None]} \
           {
            if {$zw > $zwMax} {set zw $zwMax}
           }

        set zwHz [pnt2specW $fdata CUR_ZDIM $zw Hz]

        gdbSet $e $zwVar $zw
        gdbSet $e ZW_HZ  $zwHz
       }
   }

gdbWrite $t $outName

exit 0

