#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Map DYNAMO Table Data onto PDB File:"
    puts " -in     inPDBName   \[ref.pdb\]       PDB Input."
    puts " -out    outPDBName  \[map.pdb\]       PDB Output." 
    puts " -tab    inTabName   \[eval.tab\]      Table Input."
    puts " -inVar  inVar       \[ANGLE_VIO\]     Input Variable from Table."
    puts " -outVar outVar      \[TEMPFACTOR\]    Output Variable in PDB."
    puts " -vDef   vDef        \[0.0\]           Default Value for Output."
    puts " -vMin   vMin        \[0.0\]           Lower Range of Output."
    puts " -vMax   vMax        \[999.0\]         Upper Range of Output." 
    puts " -select cond        \[1\]             Table Selection Condition."
    puts " -all                                Map Values Onto Entire Residues."
    puts " -abs                                Use Absolute Value of Input."
    puts " -avg                                Use Average Values."
    puts " -seg                                Use Seg Name Info."
    puts " -noseg                              Ignore Seg Name Info (Default)."
    puts "Notes:"
    puts " Use keywords 'tMin' and 'tMax' to set vMin and vMax according"
    puts " to the min and max values in the input table."
    exit 1
   }

set inPDBName    ref.pdb 
set outPDBName   map.pdb 
set tabName      eval.tab
set inVar        ANGLE_VIO
set outVar       TEMPFACTOR
set allFlag      0
set avgFlag      0
set cond         1
 
set vDef       0.00 
set vMin       0.00 
set vMax     999.00 
set absFlag       0
set segFlag       0

getArgD $ARGV -in     inPDBName
getArgD $ARGV -out    outPDBName
getArgD $ARGV -tab    tabName
getArgD $ARGV -inVar  inVar
getArgD $ARGV -outVar outVar
getArgD $ARGV -vDef   vDef
getArgD $ARGV -vMin   vMin
getArgD $ARGV -vMax   vMax

getListArgD $ARGV -select cond

if {[flagLoc $ARGV -abs]}   {set absFlag 1}
if {[flagLoc $ARGV -noabs]} {set absFlag 0}

if {[flagLoc $ARGV -avg]}   {set avgFlag 1}
if {[flagLoc $ARGV -noavg]} {set avgFlag 0}

if {[flagLoc $ARGV -all]}   {set allFlag 1}
if {[flagLoc $ARGV -noall]} {set allFlag 0}

if {[flagLoc $ARGV -seg]}   {set segFlag 1}
if {[flagLoc $ARGV -noseg]} {set segFlag 0}

#
#---------------------------------------------------------------------------

if {![file exists $inPDBName]} \
   {
    puts "Error Finding PDB Input $inPDBName"
    exit 1
   }

if {![file exists $tabName]} \
   {
    puts "Error Finding Table Input $tabName"
    exit 1
   }

set pdbTab   [gdbRead table -in $inPDBName -pdb]
set vTab     [gdbRead table -in $tabName]

set varNames [gdbGetInfo $vTab -parms varNames]
set eList    [gdbSelect -from $vTab -cond [string trim $cond "{}"]]
set sList    ""

foreach s {"" "_I" "_J" "_K" "_L"} \
   {
    set vName [format "RESID%s" $s] 
    if {[lsearch -exact $varNames $vName] >= 0} {lappend sList $s} 
   }

if {[strcasecmp $vDef None]} \
   {
    set atomList [gdbGetInfo $pdbTab -parms entries]
    foreach atom $atomList {gdbSet $atom $outVar $vDef}
   }

if {$absFlag} \
   {
    foreach entry $eList \
       {
        set v [gdbGet $entry $inVar]
        gdbSet $entry $inVar [expr abs($v)]
       }
   }

if {$allFlag} \
   {
    set n [llength $sList]

    if {$n > 1} \
       {
        set resVar RESID[string trim [lindex $sList 1] "{}"]
       } \
    else \
       {
        set resVar RESID[string trim [lindex $sList 0] "{}"]
       }

    set r1 [getEListStat getMin $eList $resVar]
    set rN [getEListStat getMax $eList $resVar]

    for {set res $r1} {$res <= $rN} {incr res} \
       {
        set aList [gdbSelect -from $vTab -cond "$resVar == $res"]

        if {$avgFlag} \
           {
            set q [getEListStat getAvg $aList $inVar]
           } \
        else \
           {
            set q [getEListStat getSMaxAbs $aList $inVar]
           }

        foreach a $aList {gdbSet $a $inVar $q}
       }
   }

set tMin [getEListStat getMin $eList $inVar]
set tMax [getEListStat getMax $eList $inVar]

if {![strcasecmp $vMin tMin]} {set vMin $tMin}
if {![strcasecmp $vMax tMax]} {set vMax $tMax}

set tRange [expr $tMax - $tMin]
set vRange [expr $vMax - $vMin]

if {$tRange == 0.0} \
   {
    set r 0.0
   } \
else \
   {
    set r [expr $vRange/$tRange]
   }

foreach entry $eList \
   {
    set t [gdbGet $entry $inVar]

    set v [expr $vMin + $r*($t - $tMin)]

    foreach s $sList \
       {
        set vAtomName [format "ATOMNAME%s" $s]
        set vResName  [format "RESNAME%s"  $s]
        set vSegName  [format "SEGNAME%s"  $s]
        set vResID    [format "RESID%s"    $s]

        set atomName [gdbGet $entry $vAtomName]
        set resName  [gdbGet $entry $vResName]
        set segName  [gdbGet $entry $vSegName]
        set resID    [gdbGet $entry $vResID]

        if {$allFlag} \
           {
            set c "RESID == $resID"
           } \
        else \
           {
            set c "RESID == $resID && !strcasecmp( ATOMNAME, \"$atomName\" )"
           }

        if {$segFlag} {append c " && !strcasecmp( SEGNAME, \"$segName\" )"}

        set atomList [gdbSelect -from $pdbTab -cond $c]

        set fmt \
         "%4d %3s %-4s $outVar: %6.2f $inVar: %7.3f Limits: %7.3f %7.3f" 

        foreach atom $atomList \
           {
            puts [format $fmt $resID $resName $atomName $v $t $tMin $tMax]

            set vOld [gdbGet $atom $outVar]

            if {abs($v) > abs($vOld)} {gdbSet $atom $outVar $v}
           }
       }
   }

gdbWrite $pdbTab $outPDBName -pdb
 
exit 0
