#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- -notk "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set dNameList   [glob -nocomplain dObs*.tab]
set mfrName     mfr.tab
set alpha       3.0
set frac        0.10
set r1          First
set rN          Last
set dcNMin      15
set maxRank     5
set useFragName 0

set pdbDir   $env(PDBH_DIR)
set fragDir  None

if {[flagLoc $ARGV -help]} \
   {
    puts "Estimate Relative Tensor Angles from Molecular Fragment Search."
    puts " -in      dcList   \[dObs*.tab\]  Input DC Tables."
    puts " -mfr     mfrName  \[mfr.tab\]    Input Fragment Table."
    puts " -maxRank maxRank  \[5\]          Maximum Allowable Rank."
    puts " -dcNMin  nMin     \[15\]         Minimum Allowable Coupling Count."
    puts " -frac    f        \[0.1\]        Fraction of Best Fragments to Use."
    puts " -alpha   alpha    \[3.0\]        Exponential Weighting Factor."
    puts " -r1      r1       \[First\]      First Residue in Range."
    puts " -rN      rN       \[Last\]       Last Residue in Range."
    puts "PDB Input Sources:"
    puts " -pdbDir  pdbDir   \[PDBH_DIR\]   PDB Input Directory."
    puts " -fragDir fragDir  \[None\]       Alternate Fragment Input Directory."
    puts " -useFragName                   Use FRAG_NAME for fragment name;" 
    puts "                                Otherwise, RES1 RESN RANK are used."
    exit 1
   }

getListArgD $ARGV -in dNameList

getArgD $ARGV -mfr     mfrName
getArgD $ARGV -frac    frac
getArgD $ARGV -alpha   alpha
getArgD $ARGV -r1      r1
getArgD $ARGV -rN      rN
getArgD $ARGV -dcNMin  dcNMin
getArgD $ARGV -maxRank maxRank

getArgD $ARGV -pdbDir  pdbDir
getArgD $ARGV -fragDir fragDir

if {[flagLoc $ARGV -useFragName]}   {set useFragName 1}

#
#------------------------------------------------------------------------------

if {![file exists $mfrName]} \
   {
    puts "Error Opening MFR Input $mfrName"
    exit 1
   }

foreach dName $dNameList \
   {
    if {![file exists $dName]} \
       {
        puts "Error Opening DC Input $nName"
        exit 1
       }
   }

dynInit

set mfrTab [gdbRead table -in $mfrName]
set minRes [getTabMin $mfrTab D_RES1]
set maxRes [getTabMax $mfrTab D_RESN]

if {![strcasecmp $r1 First]} {set r1 $minRes}
if {![strcasecmp $rN Last]}  {set rN $maxRes}

gdbDestroy [gdbSelect -from $mfrTab -cond "DC_NMIN < $dcNMin"]
gdbDestroy [gdbSelect -from $mfrTab -cond "RANK    > $maxRank"]
gdbDestroy [gdbSelect -from $mfrTab -cond "D_RES1 < $r1 || D_RES1 > $rN"]

gdbSortTab $mfrTab DC_Q
set eList  [gdbGetInfo $mfrTab -parms entries]
set eList  [lrange $eList 0 [expr int($frac*[llength $eList])]]

set dSrcN     [dynRead -dc -in $dNameList]
set tCount    [expr [llength $dSrcN] - 1]
set dSrc(All) [lindex $dSrcN end]

set aList "A B C D E F G H I J K L M N O P Q R S T U V W X Y Z"
set tList [lrange $aList 0 [expr $tCount - 1]]

for {set i 0} {$i < $tCount} {incr i} \
   {
    set t [lindex $tList $i] 
    set dSrc($t) [lindex $dSrcN $i]
   }

foreach e $eList \
   {
    set r1   [gdbGet $e D_RES1]
    set rN   [gdbGet $e D_RESN]
    set a1   [gdbGet $e RES1]
    set aN   [gdbGet $e RESN]
    set rank [gdbGet $e RANK]

    set pdbName [addExt $pdbDir/[gdbGet $e PDB_NAME] pdb]

    if {![strcasecmp $fragDir None]} \
       {
        set fragName $pdbName 
        set q1       $a1
        set qN       $aN
       } \
    else \
       {
        set q1 $r1
        set qN $rN

        if {$useFragName} \
           {
            set fragName [addExt $fragDir/[gdbGet $e FRAG_NAME] pdb]
           } \
        else \
           {
            set fragName \
             [format "$fragDir/frag_%03d_%03d_%02d.pdb" $r1 $rN $rank]
           }
       }

    set pdbSrc   [dynRead -pdb -in $fragName -r1 $q1 -rN $qN]
    set rootName [file root [file tail $fragName]]

    foreach t $tList \
       {
        set dInfo($t) \
         [dynSimulate -dc $dSrc($t) -src $pdbSrc -r1 $r1 -rN $rN -a1 $q1]
    
        set dRMS($t) [lindex $dInfo($t) 0]
        set Da($t)   [lindex $dInfo($t) 3]
        set Dr($t)   [lindex $dInfo($t) 4]
        set dN($t)   [lindex $dInfo($t) end]
        set rXYZ($t) [lrange $dInfo($t) 6 8]

        if {![strcasecmp $t A]} \
           {
            set rx   [lindex $rXYZ($t) 0]
            set ry   [lindex $rXYZ($t) 1]
            set rz   [lindex $rXYZ($t) 2]

            dynRotate -src $pdbSrc -rx $rx -ry $ry -rz $rz -all 

            set dInfo($t) \
             [dynSimulate -dc $dSrc($t) -src $pdbSrc -r1 $r1 -rN $rN -a1 $q1]
    
            set dRMS($t) [lindex $dInfo($t) 0]
            set Da($t)   [lindex $dInfo($t) 3]
            set Dr($t)   [lindex $dInfo($t) 4]
           }

        dynSetTensor -dc $dSrc($t) -dadr -da $Da($t) -dr $Dr($t) -auto 
       }

    set dInfo(All) \
     [dynSimulate -dcList $dSrc(All) -src $pdbSrc -r1 $r1 -rN $rN -a1 $q1 \
                  -simultaneous -keepTensor -retry]

    for {set i 0} {$i < $tCount} {incr i} \
       {
        set t [lindex $tList $i]

        set dInfo($t) [getNthTensor $dInfo(All) [expr $i + 1]]
        set dRMS($t)  [lindex $dInfo($t) 0]
        set Da($t)    [lindex $dInfo($t) 3]
        set Dr($t)    [lindex $dInfo($t) 4]
        set rXYZ($t)  [lrange $dInfo($t) 6 8]

        if {![strcasecmp $t A]} continue

        puts "A$t $Da(A) $Da($t) [dynEuler $rXYZ(A) $rXYZ($t)] $r1"
       }
   
    dynFree $pdbSrc
   }

exit
