#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- -notk "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set mfrNameList mfr.tab
set maxRank     10 
set verbose     0
set outDir      mfrSurf
set outName     mfr%03d.dat
set inPDBName   ext.pdb
set outPDBName  mfrSurf.pdb
set xw          32 
set yw          32 
set atomNames   "C CA N"
set thresh      0.0
set rms         0.0
set pdbDir      $env(PDBH_DIR)
set zeroName    $env(SURF_DIR)/zero.dat
set fragDir     None
set wFlag       0

if {[flagLoc $ARGV -help]} \
   {
    puts "Create Phi/Psi Surfaces from Molecular Fragment Replacement Search."
    puts " -mfr     mfrList   \[mfr.tab\]       Input Fragment Tables."
    puts " -inPDB   inPDB     \[ext.pdb\]       PDB File Input."
    puts " -outPDB  outPDB    \[mfrSurf.pdb\]   PDB File Output (Or None)."
    puts " -out     outName   \[mfr%03d.dat\]   Output Template."
    puts " -outDir  outDir    \[mfrSurf\]       Output Directory."
    puts " -pdbDir  pdbDir    \[PDBH_DIR\]      PDB Input Directory."
    puts " -fragDir fragDir   \[None\]          MFR Fragment Input Directory."
    puts " -dPhi    dPhi      \[32\]            Gaussian +/- Phi Width."
    puts " -dPsi    dPsi      \[32\]            Gaussian +/- Psi Width."
    puts " -maxRank maxRank   \[10\]            Max Rank of Fragments."
    puts " -w                                Weight Values by Fragment Score."
    exit 1
   }

getListArgD $ARGV -mfr mfrNameList

getArgD $ARGV -dPhi    xw 
getArgD $ARGV -dPsi    yw 
getArgD $ARGV -out     outName
getArgD $ARGV -outDir  outDir
getArgD $ARGV -maxRank maxRank
getArgD $ARGV -inPDB   inPDBName
getArgD $ARGV -outPDB  outPDBName

getArgD $ARGV -fragDir fragDir
getArgD $ARGV -pdbDir  pdbDir

if {[flagLoc $ARGV -w]}   {set wFlag 1}
if {[flagLoc $ARGV -now]} {set wFlag 0}

#
#------------------------------------------------------------------------------

if {![file isdirectory $outDir]} \
   {
    catch "file mkdir $outDir"
   }

dynInit -fast

set refSrc 0

if {[strcasecmp $outPDBName None]} \
   {
    if {![file exists $inPDBName]} \
       {
        puts stderr "Error finding PDB reference input $inPDBName"
        exit 1
       }

    set refSrc [dynRead -pdb -in $inPDBName]
   }

readROI -roi 1 -ndim 2 -in $zeroName

set rInfo [getROIInfo -roi 1]
set rdata [lindex $rInfo 0]
set fdata [lindex $rInfo 1]
set sList [lindex $rInfo 2]
set xSize [lindex $sList 0] 
set ySize [lindex $sList 1] 
set tSize [expr $xSize*$ySize]
set sdata [sysMalloc $tSize float]

set resList ""

foreach mfrName $mfrNameList \
  {
   set mfrTab [gdbRead table -in $mfrName]
   set eList  [gdbGetInfo $mfrTab -parms entries]
   set j      0

   foreach entry $eList \
      {
       set pdbName  [addExt [gdbGet $entry PDB_NAME] pdb]
       set rootName [file root [file tail $pdbName]]

       set rank [gdbGet $entry RANK]

       if {$rank > $maxRank} continue

       set r1   [gdbGet $entry D_RES1]
       set rN   [gdbGet $entry D_RESN]
       set a1   [gdbGet $entry RES1]
       set aN   [gdbGet $entry RESN]
       set id   [gdbGet $entry INDEX]
       set q    [gdbGet $entry TOTAL_Q]

       set aa1  [expr $a1 - 1]
       set aaN  [expr $aN + 1]

       set fragName None

       if {![strcasecmp $fragDir None]} \
          {
           set pdbSrc [dynRead -pdb -in $pdbDir/$pdbName \
                               -r1 $aa1 -rN $aaN -atomName $atomNames]
          } \
       else \
          {
           set fragName [format frag_%03d_%03d_%02d $r1 $rN $rank]

           set pdbSrc [dynRead -pdb -in $fragDir/$fragName.pdb \
                               -r1 $r1 -rN $rN -atomName $atomNames]
          }

       set fmt "%4d %2d   %3d   %8s %s %3d % 8.2f % 8.2f"
       set r0  $r1

       for {set resID $a1} {$resID <= $aN} {incr resID} \
          {
           set refInfo [dynGetPhiPsi -src $pdbSrc -res $resID]
           set phi     [lindex $refInfo 0]
           set psi     [lindex $refInfo 1]
          
           if {$phi < 999.0 && $psi < 999.0} \
              {
               puts \
                [format $fmt $id $rank $r0 $rootName $fragName $resID $phi $psi]

               if {![info exists aInfo($r0)]} {lappend resList $r0}

               lappend aInfo($r0) $phi,$psi,$q
              }

           incr r0
          }

       dynFree $pdbSrc
      }

    gdbFree $mfrTab
   }

foreach r0 $resList \
   {
    if {![info exists aInfo($r0)]} continue

    set thisName [format $outDir/$outName $r0]

    vsSet $rdata 0.0 $tSize

    set qList ""
    set n 0

    foreach aItem $aInfo($r0) \
       {
        set refInfo [split $aItem ,]
        lappend qList [lindex $refInfo 2]
       }

    set minQ [getMin $qList]
    set maxQ [getMax $qList]
    set dev  [getRMS $qList]
    set val  1.0

    if {$dev == 0.0} {set dev 1.0}

    foreach aItem $aInfo($r0) \
       {
        set refInfo [split $aItem ,]

        set phi     [lindex $refInfo 0]
        set psi     [lindex $refInfo 1]
        set q       [lindex $refInfo 2]

        if {$wFlag} {set val [expr 1.0 + ($q - $minQ)/$dev]}

        while {$phi < -180.0} {set phi [expr $phi + 360.0]}
        while {$phi >  180.0} {set phi [expr $phi - 360.0]}

        while {$psi < -180.0} {set psi [expr $psi + 360.0]}
        while {$psi >  180.0} {set psi [expr $psi - 360.0]}

        vSurface -sPtr $rdata -val $val \
          -xSize $xSize -ySize $xSize -xfw $xw -yfw $yw -x0 $phi -y0 $psi

        incr n
       }

    if {!$n} continue

    vvMove $sdata $rdata $tSize
    vSort  $sdata $tSize
    vRev   $sdata $tSize 

    set sum   [vSum $sdata $tSize]
    set limit [expr 0.68*$sum]
    set rLoc  [vsIntegLoc $sdata $limit $tSize]

    if {$rLoc >= 0.0} \
       {
        set thresh [getPtrVal $sdata [expr int($rLoc)]] 
       } \
    else \
       {
        set thresh [expr 0.68*$n]
       }

    setParm $fdata FDF2LB  $xw     NULL_DIM
    setParm $fdata FDF1LB  $yw     NULL_DIM
    setParm $fdata FDTAU   $thresh NULL_DIM
    setParm $fdata FDNOISE $rms    NULL_DIM

    setParm $fdata FDUSER1 -180.0 NULL_DIM
    setParm $fdata FDUSER2  180.0 NULL_DIM
    setParm $fdata FDUSER3 -180.0 NULL_DIM
    setParm $fdata FDUSER4  180.0 NULL_DIM
    setParm $fdata FDUSER5  $n    NULL_DIM
 
    writeROI -roi 1 -out $thisName -ov

    if {!$refSrc} \
       {
        puts [format "%s %3d %.3e %8.4f" $thisName $n $sum $thresh]
        continue
       }

    set maxLoc [expr int([vMaxLoc $rdata $tSize])]
    
    for {set yLoc 1} {$maxLoc > $xSize} {incr maxLoc -$xSize} \
       {
        incr yLoc
       }

    set xLoc [expr 1 + $maxLoc]

    set phi  [expr $xLoc - 181]
    set psi  [expr $yLoc - 181]

    dynSetPhiPsi -src $refSrc -phi $phi -psi $psi -res $r0

    puts \
     [format "%s %3d %.3e %8.4f %4d %4d" $thisName $n $sum $thresh $phi $psi]
   }

if {$refSrc} {dynWrite -pdb -src $refSrc -out $outPDBName}

exit
