#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Assemble MFR Fragment Library with DC Alignment Tensors:"
    puts " -mfr     mfrName \[mfr.tab\]         MFR Input Table."
    puts " -ref     pdbName \[ext.pdb\]         PDB Input of Target Protein."
    puts " -dc      dcList  \[dObs*.tab\]       Input Dipolar Coupling Tables."
    puts " -r1      r1      \[First\]           First Residue in Range."
    puts " -rN      rN      \[Last\]            Last Residue in Range."
    puts " -minRank rank1   \[1\]               Minimum Rank to Accept."
    puts " -maxRank rankN   \[10\]              Maximum Rank to Accept."
    puts " -inDir   inDir   \[frag\]            Input Fragments."
    puts " -outDir  outDir  \[build\]           Rotated Output Fragments."
    puts "Optional Specification of Fixed Tensor Parameters:"
    puts " -da      daList  \[None\]            Target Da Values for Tensor."
    puts " -dr      drList  \[None\]            Target Dr Values for Tensor."
    puts " -dadr                                Enable Fixed Tensor Parameters."
    puts "Temporary Files:"
    puts " -tmp     tmpName \[tmp.pdb\]"
    exit 1
   }

set inDir      frag 
set outDir     build 
set mfrName    mfr.tab
set tmpName    tmp.pdb
set refName    ext.pdb
set firstRes   First
set lastRes    Last

set dcNames    [glob -nocomplain dObs*.tab]
set dadrFlag   0
set minOvCount 3
set skip       1
set minRank    1
set maxRank    10

set daList    ""
set drList    ""

set rxList    0.0 
set ryList    0.0 
set rzList    0.0 

getArgD $ARGV -inDir   inDir
getArgD $ARGV -outDir  outDir
getArgD $ARGV -mfr     mfrName
getArgD $ARGV -ref     refName
getArgD $ARGV -tmp     tmpName
getArgD $ARGV -skip    skip
getArgD $ARGV -minOv   minOvCount
getArgD $ARGV -minRank minRank
getArgD $ARGV -maxRank maxRank

getListArgD $ARGV -dc dcNames
getListArgD $ARGV -da daList
getListArgD $ARGV -dr drList

getListArgD $ARGV -rx rxList
getListArgD $ARGV -ry ryList
getListArgD $ARGV -rz rzList

if {[flagLoc $ARGV -dadr]}   {set dadrFlag 1}
if {[flagLoc $ARGV -nodadr]} {set dadrFlag 0}

#
#-------------------------------------------------------------------------

proc getTensorScore { rxR ryR rzR rxT ryT rzT }
{
    set rotTarget  [dynRotate -verb -rx $rxR -ry $ryR -rz $rzR]
    set rotCurrent [dynRotate -verb -rx $rxT -ry $ryT -rz $rzT]

    set n [llength $rotTarget]
    set s 0.0

    for {set i 0} {$i < $n} {inncr i} \
       {
        set qT [lindex $rotTarget  $i]
        set qC [lindex $rotCurrent $i]
        set d  [expr $qT - $qC]
        set s  [expr $s + $d*$d]
       }

    return $s 
}

proc saveBestRot { } \
{
   global pdbSrc rotSrc prevPDBSrc prevRotSrc r1 rN a1 aN tList

   foreach t $tList \
      {
       dynFree $prevPDBSrc($t) 
       dynFree $prevRotSrc($t) 

       set prevPDBSrc($t) $pdbSrc($t)
       set prevRotSrc($t) $rotSrc($t)

       set rxT [lindex $tOffset 0]
       set ryT [lindex $tOffset 1]
       set rzT [lindex $tOffset 2]

       dynRotate -src $pdbSrc($t) -all -rx $rxT -ry $ryT -rz $rzT
       dynRotate -src $rotSrc($t) -all -rx $rxT -ry $ryT -rz $rzT

       set outName [format "%s/%s_%s" $outDir $t $rootName]

       dynWrite -pdb -src $pdbSrc($t) -out $outName
      }

   set a1 $r1
   set aN $rN
}
 
#
#-------------------------------------------------------------------------

#
# Target PDB Info:

dynInit

if {![file exists $refName]} \
   {
    puts stderr "Error Finding Reference PDB $refName"
    exit 1
   }

set refSrc [dynRead -pdb -in $refName]
set pInfo  [dynGetInfo -src $refSrc]
set r1     [lindex $pInfo 0]
set rN     [lindex $pInfo 1]

if {![strcasecmp $firstRes First]} {set firstRes $r1}
if {![strcasecmp $LastRes  Last]}  {set lastRes  $rN}

#
# DC Info:

set tNames  {A B C D E F G H I J K L M N O P Q R S T U V W X Y Z}
set dcCount [llength $dcNames]

set tList   [lrange $tNames 0 [expr $dcCount - 1]]

if {$dadrFlag} \
   {
    if {[llength $daList] != $dcCount || [llength $drList] != $dcCount} \
       {
        puts stderr "Error: Da or Dr Count Mismatch."
        exit 1
       }
   }

set j 0

foreach dcName $dcNameList \
   {
    if {![file exists $dcName]} \
       {
        puts stderr "Error Finding DC Table $dcName"
        exit 1
       }

    set t [lindex $tList $j]

    set dSrc($t) [dynRead -dc -in $dcName -src $refSrc]

    set prevPDBSrc($t) 0
    set prevRotSrc($t) 0

    set pdbSrc($t) 0
    set rotSrc($t) 0

    incr j
   }

#
# Fragment Info:

if {![file exists $mfrTab]} \
   {
    puts stderr "Error Finding MFR Input Table $mfrName"
    exit 1
   }

set mfrTab [gdbRead table -in $mfrName]
set eList  [gdbGetInfo $mfrTab -parms entries]

#
# Fragment assembly:

if {![file isdirectory $outDir]} {file mkdir $outDir}

set eulerOffsets \
 {{0.0 0.0 0.0} {180.0 0.0 0.0} {0.0 180.0 0.0} {0.0 0.0 180.0}}

set i 0

foreach entry $eList \
   {
    set aName [gdbGet $entry PDB_NAME]
    set r1    [gdbGet $entry D_RES1]
    set rN    [gdbGet $entry D_RESN]
    set rank  [gdbGet $entry RANK]
    set rr1   [expr $r1 + $skip]
    set rrN   [expr $rN - $skip]

    if {$r1   < $firstRes || $r1   > $lastRes} continue
    if {$rank < $minRank  || $rank > $maxRank} continue

    puts [format "%2d %4d %2d %s" $r1 $rN $rank $aName]"

    set j 0

    foreach t $tList \
       {
        set pdbSrc($t) [dynRead -pdb -in $inDir/$aName]
        set rotSrc($t) [dynRead -pdb -in $inDir/$aName]
 
        if {$dadrFlag} \
           {
            set da [lindex $daList $j]
            set dr [lindex $drList $j]

            set dcInfo($t) \
             [dynSimulate -dc $dcSrc($t) -src $pdbSrc($t) -dadr \
                          -da $daA -dr $drA -r1 $rr1 -rN $rrN -a1 $rr1]
           } \
        else \
           {
            set dcInfo($t) \
             [dynSimulate -dc $dcSrc($t) -src $pdbSrc($t) -svd \
                          -r1 $rr1 -rN $rrN -a1 $rr1]
           }

        set rx [lindex $dcInfo($t) 6]
        set ry [lindex $dcInfo($t) 7]
        set rz [lindex $dcInfo($t) 8]

        dynRotate -src $pdbSrc($t) -all -rx $rx -ry $ry -rz $rz
        dynRotate -src $rotSrc($t) -all -rx $rx -ry $ry -rz $rz

        incr j
       }

    if {!$i} \
       {
        saveBestRot 
        continue
       }

    set aa1 [expr $a1 + $skip]
    set aaN [expr $aN - $skip]

    set s1  [getMax "$rr1 $aa1"]
    set sN  [getMin "$rrN $aaN"]

    set ovCount [expr 1 + $sN - $s1]

    set rmsScoreList ""
    set tScoreList   ""

    foreach aOffset $eulerOffsets \
       {
        set scoreItem ""
        set j 0

        set rxA [lindex $aOffset 0]
        set ryA [lindex $aOffset 1]
        set rzA [lindex $aOffset 2]

        dynRotate -src $rotSrc(A) -all -rx $rxA -ry $ryA -rz $rzZ

        foreach t $tList \
           {
            set rxTarget [lindex $rxList $j]
            set ryTarget [lindex $ryList $j]
            set rzTarget [lindex $rzList $j]

            set rmsSum 0.0
            set tSum   0.0

            foreach tOffset $eulerOffsets \
               {
                set rxT [lindex $tOffset 0]
                set ryT [lindex $tOffset 1]
                set rzT [lindex $tOffset 2]

                dynRotate -src $pdbSrc($t) -all -rx $rxT -ry $ryT -rz $rzT

                set aInfo \
                 [dynAlign -ref $rotSrc(A) -aSrc $pdbSrc($t) \
                           -r1 $r1 -rN $rN -a1 $r1 -noapply] 

                set rxCurrent [lindex $aInfo 0]
                set ryCurrent [lindex $aInfo 1]
                set rzCurrent [lindex $aInfo 2]

                set tScore \
                 [getTensorScore \
                   $rxTarget  $ryTarget  $rzTarget \
                   $rxCurrent $ryCurrent $rzCurrent]

                if {$ovCount > 0} \
                   {
                    set aInfo \
                     [dynAlign -ref $prevRotSrc($t) -aSrc $pdbSrc($t) \
                               -r1 $s1 -rN $sN -a1 $s1 -noapply]

                    set rmsScore [lindex aInfo end]
                   } \
                else \
                   {
                    set rmsScore 999.0
                   }
            
                set rmsSum [expr $rmsSum + $rmsScore] 
                set tSum   [expr $tSum   + $tScore] 

                dynRotate -src $pdbSrc($t) -all -rx $rxT -ry $ryT -rz $rzT -inv
               }

            lappend rmsScoreList "$rmsSum $aOffset $tOffset $ovCount rms"
            lappend tScoreList   "$tSum   $aOffset $tOffset $ovCount tensor"

            incr j
           }

        dynRotate -src $rotSrc(A) -all -rx $rxA -ry $ryA -rz $rzZ -inv
       }

     set offsetInfo [findBestEulerOffset $rmsScoreList $tScoreList]

     if {![llength $offsetInfo]} continue
       
     saveBestRot $offsetInfo
 
     foreach t $tList \
        {
         set tOffset [lindex $offsetInfo 2]

         set rxT [lindex $tOffset 0]
         set ryT [lindex $tOffset 1]
         set rzT [lindex $tOffset 2]

         dynRotate -src $pdbSrc($t) -all -rx $rxT -ry $ryT -rz $rzT
         dynRotate -src $rotSrc($t) -all -rx $rxT -ry $ryT -rz $rzT

         set outName [format "%s/%s_%s" $outDir $t $rootName]

         dynWrite -pdb -src $pdbSrc($t) -out $outName

         set prevPDBSrc($t) $pdbSrc($t)
         set prevRotSrc($t) $rotSrc($t)

         set a1 $r1
         set aN $rN
        }
    }

exit
