#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

#
# Display a modelXY modeling command given X/Y pairs, etc.

proc showModelCmnd { scriptUnit xList yList id ass height vol sigma } \
{
    global env pList

    set thisPList      $pList
    set macName        $env(NMRTXT)/exp.M
    set guessCount     21
    set confidInterval 0.68
    set outName        txt/fit$id.txt
    set modName        mod/mod$id.tab
    set count          [llength $xList]

    set sList ""
    set noise [format "% .4e" [expr abs($sigma/$height)]]

    foreach i $yList \
       {
        lappend sList $noise 
       }

#
# Save the X/Y pairs in a text file for plotting:

    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "GetXY Error opening output file $outName"
        return 1
       }

    for {set i 0} {$i < $count} {incr i} \
       {
        puts $outUnit "[lindex $xList $i] [lindex $yList $i] [lindex $sList $i]"
       }

    close $outUnit
#
# Print a nicely formatted modelXY command:

    puts $scriptUnit "modelXY -id $id -tol 1e-8 -p $thisPList \\"

    puts -nonewline $scriptUnit " -x     "
    foreach i $xList {puts -nonewline $scriptUnit [format " %6s" $i]}
    puts $scriptUnit " \\"

    puts -nonewline $scriptUnit " -y     "
    foreach i $yList {puts -nonewline $scriptUnit [format " %6s" $i]}
    puts $scriptUnit " \\"

    puts -nonewline $scriptUnit " -noise "
    foreach i $sList {puts -nonewline $scriptUnit [format " %6s" $i]}
    puts $scriptUnit " \\"

    puts $scriptUnit " -gc $guessCount -cf $confidInterval \\"
    puts $scriptUnit " -str ass \"$ass\" -macro $macName >& $modName"
    puts $scriptUnit ""
    puts $scriptUnit "cat $modName"
    puts $scriptUnit ""
}

#
# Initialize:

setOSType

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set error     0

set sigma     0.0

set tabName   ""
set listName  ""
set inNames   ""
set tauList   ""
set locList   ""
set pList     {1.0 100.0}
set comName   modelExp.com

set specCount 0
set zeroCount 0

#
# Print help text if needed:

if {$ARGC < 4 || [flagLoc $ARGV "-help"]} \
   {
    puts stderr "Exponential Fit of X/Y evolution curves from a series table:"
    puts stderr " $argv0 tableName listName noiseStdDev \[-p amp decay\]"
    puts stderr "Examples:"
    puts stderr " $argv0 relax.nlin.tab relax.list 38000"
    puts stderr " $argv0 relax.nlin.tab relax.list 38000 -p 1.0 2.0e-1"
    puts stderr " (Use 0.0 for auto noise estimate, assumes uniform noise.)"
    exit 1
   }

set tabName  [lindex $ARGV 3]
set listName [lindex $ARGV 4]
set sigma    [lindex $ARGV 5]

getListArgD $ARGV -p pList

#
# Extract the spectral names and tau values:

set error [catch {set inUnit [open $listName r]}]

if {$error} \
   {
    puts stderr "GetXY Error opening list file $listName"
    return 1
   }

set inNames [read $inUnit]

close $inUnit

set specCount [llength $inNames]

if {!$specCount} \
   {
    puts stderr "GetXY Error, empty list file $listName"
    return 1
   }

if {$sigma == 0.0} \
   {
    set sigma [estSpecNoise [lindex $inNames 0]]
   }

set zeroCount

foreach i $inNames \
   {
    set tau [exec getParm -in $i -parm FDTAU -fmt " %g"]

    if {$tau == 0} {incr zeroCount}

    lappend tauList [string trim $tau]
   }

if {$zeroCount == $specCount} \
   {
    puts stderr "GetXY Error, all tau values are zero."
    puts stderr "Try setting each one with: sethdr specName -tau tauValue"
    return 1
   }

#
# Read the peak table.
# Extract the locations of the evolutions scales, etc.

if {![file exists $tabName]} \
   {
    puts stderr "GetXY Error finding series peak table file $tabName"
    return 1
   }

set tab      [gdbRead table -in $tabName]
set vList    [gdbGetInfo $tab -parms varNames]
set eList    [gdbGetInfo $tab -parms entries]

set pkCount  [llength $eList]
set varCount [llength $vList]

set idLoc  [lsearch -exact $vList INDEX]
set assLoc [lsearch -exact $vList ASS]
set hiLoc  [lsearch -exact $vList HEIGHT]
set volLoc [lsearch -exact $vList VOL]

if {$idLoc < 0 || $hiLoc < 0 || $volLoc < 0} \
   {
    puts stderr "GetXY Error: missing INDEX/HEIGHT/VOL."
    return 1
   }

#
# Create the script output file.
# Create the directories used for data and graphics output.
#
# For each peak:
#  Extract the list of evolution amplitudes.
#  Construct the modeling command.

set error [catch {set scriptUnit [open $comName w 0744]}]

if {$error} \
   {
    puts stderr "Error opening script $comName"
    exit 1
   }

if {![file isdirectory plot]} {file mkdir plot}
if {![file isdirectory gnu]}  {file mkdir gnu}
if {![file isdirectory txt]}  {file mkdir txt}
if {![file isdirectory mod]}  {file mkdir mod}

for {set i 0} {$i < $specCount} {incr i} \
   {
    set loc [lsearch -exact $vList Z_A$i]

    if {$loc < 0} \
       {
        puts stderr "GetXY Error: missing Z_A$i."
        return 1
       }

    lappend locList $loc
   }

#
# For each peak:
#   Extract the scaling amplitudes etc.
#   Show the extracted results.

puts $scriptUnit "#!/bin/csh"
puts $scriptUnit ""
puts $scriptUnit "#"
puts $scriptUnit "# Noise Estimate: $sigma"
puts $scriptUnit ""

set ass None

for {set i 0} {$i < $pkCount} {incr i} \
   {
    set e [lindex $eList $i]

    set id     [gdbGet $e INDEX]
    set height [gdbGet $e HEIGHT]
    set vol    [gdbGet $e VOL]

    if {$assLoc >= 0} {set ass [gdbGet $e ASS]}

    set ampList ""

    for {set j 0} {$j < $specCount} {incr j} \
        {
         lappend ampList [gdbGet $e Z_A$j]
        }

    showModelCmnd $scriptUnit $tauList $ampList $id $ass $height $vol $sigma
   }

puts $scriptUnit "gnuplot gnu/*"

close $scriptUnit

set g 90x12+15+15
set error [catch {exec xterm -bg cyan -fg black -geom $g -e $comName}]

exit
