#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert Mass Spec Time Series to NMRPipe 2D Format:"
    puts " -in  inName    \[stdin\]   Input File, as Produced by MSConvert."
    puts " -out outName   \[Auto\]    NMRPipe-format Output"
    puts " -mzMin  mzMin  \[1.0\]     Min Value for m/z Dimension."
    puts " -mzMax  mzMax  \[Auto\]    Max Value for m/z Dimension."
    puts " -mzStep mzStep \[1.0\]     Step Size for m/z Dimension."
    puts " -tMin  tMin    \[0.0\]     Min Value for Time Dimension."
    puts " -tMax  tMax    \[Auto\]    Max Value for Time Dimension."
    puts " -tStep tStep   \[0.1\]     Step Size for Time Dimension."
    puts " -info                    Display Info Only, No Conversion."
    puts " -verb                    Verbose Mode ON."
    puts " -noverb                  Verbose Mode OFF."
    puts "Notes:"
    puts " 1. MSConvert is part of the following software:"
    puts "    http://proteowizard.sourceforge.net"
    puts "    MC Chambers et al: A cross-platform toolkit for mass spectrometry"
    puts "    and proteomics. Nature Biotechnology 30, 918–920 (2012)."
    puts " 2. Auto settimgs for -mzMax and -tMax require explicit input file."
    exit 0
   }

set inName   stdin
set inUnit   0
set outName  Auto 
set infoFlag 0 
set verbose  1

set xSize    0
set ySize    0
set fdata    0
set rdata    0

set mzMin    1
set mzMax    Auto
set mzStep   1.0
set mzRange  1.0

set tMin     0.0
set tMax     Auto
set tStep    0.1
set tRange   1.0

if {[flagLoc $ARGV -info]}   {set infoFlag 1}
if {[flagLoc $ARGV -noinfo]} {set infoFlag 0}

if {$infoFlag} \
   {
    set outName None
    set verbose 0
   }

getArgD $ARGV -in     inName
getArgD $ARGV -out    outName

getArgD $ARGV -mzMin  mzMin
getArgD $ARGV -mzMax  mzMax
getArgD $ARGV -mzStep mzStep

getArgD $ARGV -tMin   tMin
getArgD $ARGV -tMax   tMax
getArgD $ARGV -tStep  tStep

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {![strcasecmp $inName stdin] && (![strcasecmp $mzMax Auto] || ![strcasecmp $tMax Auto])} \
   {
    puts "Error: -mzMax Auto and -tMax Auto require expliciting input file."
    exit 1    
   }

#
#----------------------------------------------------------------------------------

proc getCSVItem { n str } \
{
   set tStr [string trim [getToken $n $str ","]]
   set rStr ""

   foreach t $tStr {lappend rStr $t}

   return $rStr
}

proc addMSData { mzList hiList t } \
{
   global rdata xSize ySize mzMin mzMax mzRange tMin tMax tRange

   if {$t < $tMin || $t > $tMax} return

   set n [llength $mzList]

   for {set i 0} {$i < $n} {incr i} \
      {
       set mz [lindex $mzList $i]
       set hi [lindex $hiList $i]

       if {$mz < $mzMin || $mz > $mzMax} continue

       set xLoc [expr int(($xSize - 1)*($mz - $mzMin)/$mzRange)] 
       set yLoc [expr int(($ySize - 1)*($t  - $tMin)/$tRange)] 

       set loc [expr $xLoc  + $yLoc*$xSize]
       set val [getPtrVal $rdata $loc]

       setPtr $rdata $loc [expr $hi + $val]
      } 
}

if {![strcasecmp $inName stdin]} \
   {
    set inUnit 0
   } \
else \
   {
    if {![file exists $inName]} \
       {
        puts "Error Finding Input File $inName"
        exit 1
       }

    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        puts "Error Opening Input File $inName"
        exit 1
       }
   }

if {![strcasecmp $outName Auto]} \
   {
    if {![strcasecmp $inName stdin]} \
       {
        set outName ms.dat
       } \
    else \
       {
        set rName [file root      $inName]
        set eName [file extension $inName]

        if {[strcmp $rName .dat]} \
           {
            set outName ${rName}.dat
           } \
        else \
           {
            set outName ${inName}.dat
           }
       }
   }

if {!$infoFlag} \
   {
    if {![strcasecmp $mzMax Auto] || ![strcasecmp $tMax Auto]} \
       {
        set msInfo [exec ms2pipe.tcl -in $inName -mzMin $mzMin -mzStep $mzStep -tMin $tMin -tStep $tStep -info]
       
        if {![strcasecmp $mzMax Auto]} {set mzMax [getNamedItem mzMax $msInfo]}
        if {![strcasecmp $tMax  Auto]} {set tMax  [getNamedItem tMax  $msInfo]}
       }

    set mzRange  [expr $mzMax - $mzMin]
    set tRange   [expr $tMax  - $tMin]

    if {$mzRange == 0.0} {set mzRange 1.0}
    if {$tRange  == 0.0} {set tRange  1.0}

    set xSize    [expr 1 + int($mzRange/$mzStep)]
    set ySize    [expr 1 + int($tRange/$tStep)]
    set fileSize [expr 2048*$xSize*$ySize*4/(1024*1024)]
   }

if {[strcasecmp $outName None]} \
   {
    set error [catch {set fdata [sysMalloc $FDATASIZE float]}]
    set error [catch {set rdata [sysMalloc [expr $xSize*$ySize] float]}]

    if {$error} \
       {
        puts "Error Allocating Memory for Output."
        exit 1
       }

    vsSet $rdata 0.0 [expr $xSize*$ySize]

    set error [catch {set outUnit [sysOpen $outName w]}]

    if {$error} \
       {
        puts "Error Opening Output File $outName"
        exit 1
       }

    set xMid [expr 1 + $xSize/2]
    set yMid [expr 1 + $ySize/2]

    if {$xSize > 1} \
       {
        set mzMid [expr $mzMin + $mzRange*($xMid - 1)/($xSize - 1)]
       } \
    else \
       {
        set mzMid $mzMin 
       }

    if {$ySize > 1} \
       {
        set tMid [expr $tMin + $tRange*($yMid - 1)/($ySize - 1)]
       } \
    else \
       {
        set tMid $tMin 
       }

    buildHdr $fdata -title [file root [file tail $outName]] \
     -xN      $xSize   -yN      $ySize  \
     -xT      $xSize   -yT      $ySize  \
     -xMODE   Real     -yMODE   Real    \
     -xSW     $mzRange -ySW     $tRange \
     -xOBS    1.0000   -yOBS    1.0000  \
     -xCAR    $mzMid   -yCAR    $tMid   \
     -xORIG   $mzMax   -yORIG   $tMax   \
     -xFT     Freq     -yFT     Freq    \
     -xLAB    m/z      -yLAB    Time    \
     -ndim    2        -aq2D    Real 

    set error [catch {sysWrite $outUnit $fdata $FDATASIZE}]

    if {$error} \
       {
        puts "Error Writing Header to File $outName"
        exit 1
       }
   }

set lineCount  0
set specCount  0
set minCoord   1.0e+16
set maxCoord  -1.0e+16
set minTime    1.0e+16
set maxTime   -1.0e+16
set maxHi      0.0

set coordList   ""
set coordType   None

set msLevel     -1
set specID      0 
set startTime   0.0
set arrayType   None
set errorCount  0

while {[gets $inUnit line] >= 0} \
   {
    incr lineCount

    set n   [llength $line]
    set key [lindex  $line 0]

    if {![strcmp $key "cvParam:"]} \
       {
        set parmStr [lrange $line 1 end]
        set parmKey [getCSVItem 1 $parmStr]
        set subKey  [lindex $parmKey end]

        if {![strcmp $parmKey "ms level"]} \
           {
            set msLevel [getCSVItem 2 $parmStr]
            if {$verbose} {puts "$lineCount: Found msLevel $msLevel"}
           } \
        elseif {![strcmp $parmKey "scan start time"]} \
           {
            set startTime [getCSVItem 2 $parmStr]
            if {$verbose} {puts "$lineCount: Found startTime $startTime"}
           } \
        elseif {![strcmp $parmKey "m/z array"]} \
           {
            set arrayType m/z
            set coordType $arrayType
            if {$verbose} {puts "$lineCount: Found m/z array"}
           } \
        elseif {![strcmp $parmKey "intensity array"]} \
           {
            set arrayType intensity 
            if {$verbose} {puts "$lineCount: Found intensity array"}
           } \
        elseif {![strcmp $subKey "array"]} \
           {
            set arrayType [lindex $parmKey 0] 
            set coordList ""
            set coordType $arrayType

            if {$verbose} {puts "$lineCount: Found other array $arrayType"}
           } \
       } \
    elseif {![strcmp $key "index:"]} \
       {
        set specID [lindex $line end]
        if {$verbose} {puts "$lineCount: Found index $specID"}
       } \
    elseif {![strcmp $key "binary:"]} \
       {
        set count [string trim [lindex $line 1] "\[\]"]

        if {[strcmp $msLevel 1]} \
           {
            if {$verbose} {puts "$lineCount: Skipping MS Level $msLevel binary actual: [expr $n - 2] expected: $count"}
            continue
           }

        if {![strcmp $arrayType "m/z"]} \
           {
            set coordList [lrange $line 2 end]
            set coordType $arrayType

            if {$verbose} {puts "$lineCount: Extracting coords $coordType binary actual: [llength $coordList] expected: $count"}
            continue
           }

        if {![strcmp $arrayType "intensity"]} \
           {
            if {[strcmp $coordType "m/z"]} \
               {
                if {$verbose} {puts "$lineCount: Skipping Intensity for Coord Type $coordType binary actual: [expr $n - 2] expected: $count"}
                continue
               }

            set hiList     [lrange $line 2 end]
            set coordCount [llength $coordList]
            set hiCount    [llength $hiList]

            if {$coordCount != $hiCount} \
               {
                puts "$lineCount: Error: Intensity for Coord Type $coordType binary mismatch actual: $hiCount coords: $coordCount expected: $count"
                incr errorCount
                continue
               }


            if {$verbose} {puts "$lineCount: Keeping Intensity for Coord Type $coordType binary actual: [llength $hiList] coords: [llength $coordList] expected: $count"}

            incr specCount

            set thisCoordMin [getMin $coordList]
            set thisCoordMax [getMax $coordList]
            set thisHiMax    [getMax $hiList]

            if {$specCount == 1} \
               {
                set minCoord $thisCoordMin
                set maxCoord $thisCoordMax
                set minTime  $startTime
                set maxTime  $startTime
                set maxHi    $thisHiMax
               }

            if {$thisCoordMin < $minCoord} {set minCoord $thisCoordMin}
            if {$thisCoordMax > $maxCoord} {set maxCoord $thisCoordMax}

            if {$startTime < $minTime} {set minTime $startTime}
            if {$startTime > $maxTime} {set maxTime $startTime}

            if {$thisHiMax > $maxHi}  {set maxHi $thisHiMax}

            if {[strcasecmp $outName None]} {addMSData $coordList $hiList $startTime} 

            continue
           }

        if {$verbose} {puts "$lineCount: Found unused binary actual: [expr $n - 2] expected: $count"}
       }

    if {!$n} continue
   }

if {[strcasecmp $outName None]} \
   {
    set error [catch {sysWrite $outUnit $rdata [expr $xSize*$ySize]}]

    if {$error} \
       {
        puts "Error Writing Data to File $outName"
        exit 1
       }

    catch "sysClose $outUnit"
   }

if {![strcasecmp $mzMax Auto]} \
   {
    set mzMax [expr 1.0 + int($maxCoord + 0.05*($maxCoord - $minCoord))]
    set xSize [expr 1 + int(($mzMax - $mzMin)/$mzStep)]
   }

if {![strcasecmp $tMax  Auto]} \
   {
    set tMax  [expr 1.0 + int($maxTime  + 0.05*($maxTime  - $minTime))]
    set ySize [expr 1 + int(($tMax  - $tMin)/$tStep)]
   }

if {$infoFlag} \
   {
    set fileSize [expr 2048*$xSize*$ySize*4/(1024*1024)]

    puts "inName $inName outName $outName specCount $specCount mzCoord $minCoord maxCoord $maxCoord minTime $minTime maxTime $maxTime maxHi $maxHi mzMin $mzMin mzMax $mzMax mzStep $mzStep tMin $tMin tMax $tMax tStep $tStep xSize $xSize ySize $ySize fileSize ${fileSize}MB errorCount $errorCount"
   }

exit 0 
