#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

#
# Event handler for building conversion from NIH Protocol Bruker parameters.
# Updated 10/2015 to use extra information in Bruker's own sequences.

proc readParams_BRUKER_NIH {} \
{
   global inName specialArgs dataSource DECIM DSPFVS GRPDLY BYTORDA DTYPA dmxMode interleaveInfo verbose
   global nusFlag nusRevFlag nusName nusOffset nusInfo nusDimCount nusSampleCount nusSampleCountMax
   global Tx Ty Tz Ta Nx Ny Nz Na dimCount dimCountName userTauCount expectedTauCount pseudo3DFlag
   global vendorScans vendorScale scaleByScansFlag scaleByVendorFlag vendorMsg dateText
   global acqus acqu2s acqu3s acqu4s

   global MODEy SWy OBSy CARy FTy LABy MODEz SWz OBSz CARz FTz LABz MODEa SWa OBSa CARa FTa LABa NUCx NUCy NUCz NUCa aq2D

   clearScript

#
# Reset default values.
# Extract the parameters from the Bruker parameter files.
# Add special bruker arguments.

   set dataSource       [getDataSource $inName]
   set nusInfo          None 
   set nusDimCount      0
   set expectedTauCount 0

   set nmrPath [tkFileFn dirname $inName]

   if {![info exists userTauCount]} {set userTauCount 0}

   if {![info exists NUCx]} {set NUCx None}
   if {![info exists NUCy]} {set NUCy None}
   if {![info exists NUCz]} {set NUCz None}
   if {![info exists NUCa]} {set NUCa None}

   if {![info exists verbose]} {set verbose 1}

   if {$nusFlag && [strcasecmp $nusName None] && [file exists $nusName]} \
      {
       set nusInfo None 

       if {$nusRevFlag} \
          {
           catch {set nusInfo [exec getNUSInfo.tcl -sample $nusName -sampleCount $nusSampleCount -off $nusOffset -rev -parmDir $nmrPath]}
          } \
       else \
          {
           catch {set nusInfo [exec getNUSInfo.tcl -sample $nusName -sampleCount $nusSampleCount -off $nusOffset -parmDir $nmrPath]}
          }

       if {![info exists nusInfo]} {set nusInfo None}

       set itemCount [llength $nusInfo]

       if {$verbose} \
          {
           if {$itemCount > 1} \
              {
               puts ""
               puts "NUS Schedule Information from $nusName:"

               for {set i 0} {$i < $itemCount} {incr i 2} \
                  {
                   puts [format " %-16s %6s" [lindex $nusInfo $i]: [lindex $nusInfo [expr $i + 1]]]
                  }
              } \
           else \
              {
               puts ""
               puts "NUS Schedule Information from $nusName: $nusInfo"
              }
          }

       set nusDimCount       [getNamedItemD nusDimCount    $nusInfo 0]
       set nusSampleCountMax [getNamedItem  sampleCountMax $nusInfo]

       if {![string is integer $nusSampleCountMax]} {set nusSampleCountMax 0}

       if {$nusDimCount >= 2} \
          {
           set dimCount $nusDimCount

           if {![strcasecmp $nusSampleCount Auto]} {set nusSampleCount [getNamedItemD sampleCount $nusInfo 0]}
           if {![strcasecmp $nusOffset Auto]}      {set nusOffset      [getNamedItemD offset      $nusInfo 0]}
          }
       
       if {$verbose} {puts ""}
      }

   getNIHBrukerND $inName

   updateSpecial_BRUKER_NIH

   if {[info exists acqus(DATE)]} {set dateText $acqus(DATE)}

   if {(![llength $NUCx] || ![strcasecmp $NUCx None]) && [info exists acqus(NUC1)]}  {set NUCx $acqus(NUC1)}
   if {(![llength $NUCy] || ![strcasecmp $NUCy None]) && [info exists acqu2s(NUC1)]} {set NUCy $acqu2s(NUC1)}
   if {(![llength $NUCz] || ![strcasecmp $NUCz None]) && [info exists acqu3s(NUC1)]} {set NUCz $acqu3s(NUC1)}
   if {(![llength $NUCa] || ![strcasecmp $NUCa None]) && [info exists acqu4s(NUC1)]} {set NUCa $acqu4s(NUC1)}

#
# Update parameters according to NUS schedule:

   set yqSize 2
   set zqSize 2
   set aqSize 2

   if {![strcasecmp $MODEy Real] || ![strcasecmp $MODEy TPPI]} {set yqSize 1}
   if {![strcasecmp $MODEz Real] || ![strcasecmp $MODEz TPPI]} {set zqSize 1}
   if {![strcasecmp $MODEa Real] || ![strcasecmp $MODEa TPPI]} {set aqSize 1}

   if {$nusDimCount >= 2} {set dimCount $nusDimCount}

   if {$nusDimCount >= 2} \
      {
       set yTNUS [getNamedItemD yTNUS $nusInfo 0]
       
       if {$yTNUS > 0} \
          {
           set Ty $yTNUS
           set Ny [expr $yqSize*$Ty]
          } 
      }

   if {$nusDimCount >= 3} \
      {
       set zTNUS [getNamedItemD zTNUS $nusInfo 0]

       if {$zTNUS > 0} \
          {
           set Tz $zTNUS
           set Nz [expr $zqSize*$Tz]
          }
      }

   if {$nusDimCount >= 4} \
      {
       set aTNUS [getNamedItemD aTNUS $nusInfo 0]

       if {$aTNUS > 0} \
          {
           set Ta $aTNUS
           set Na [expr $aqSize*$Ta]
          } 
      }

#
# Update the number of dimensions

    if {![strcasecmp $dimCountName N]} \
       {
        if {$pseudo3DFlag} \
           {
            if {[info exists acqu2s(FnMODE)] && [info exists acqu2s(TD)]} \
               {
                if {$acqu2s(FnMODE) <= 1 || $acqu2s(FnMODE) == 7} {set expectedTauCount $acqu2s(TD)}
               }

            if {[info exists acqu3s(FnMODE)] && [info exists acqu3s(TD)]} \
               {
                if {$acqu3s(FnMODE) <= 1 || $acqu3s(FnMODE) == 7} {set expectedTauCount $acqu3s(TD)}
               }

            set dimCount 3 
            setDimCount  N 
           } \
        elseif {$Ny < 1 && $dimCount > 1} \
           {
            set dimCount 1 
            setDimCount  N 
           } \
        elseif {$Nz < 1 && $dimCount > 2} \
           {
            set dimCount 2
            setDimCount  N 
           } \
        elseif {$Na < 1 && $dimCount > 3} \
           {
            set dimCount 3
            setDimCount  N 
           }
       }

   if {$nusSampleCount != $nusSampleCountMax} {showPoorEntries nusSampleCount}

#
# If no explicit settings via aqseq or acqu3s/acqu4s exist, move 15N 
# to the Y-Axis, which is more likely for Bruker:

   set trueDimCount $dimCount
   if {$nusDimCount > $trueDimCount} {set trueDimCount $nusDimCount}

   if {$trueDimCount >= 2 && ![strcasecmp $MODEy Real]} {set Ty $Ny}
   if {$trueDimCount >= 3 && ![strcasecmp $MODEz Real]} {set Tz $Nz}
   if {$trueDimCount >= 4 && ![strcasecmp $MODEa Real]} {set Ta $Na}

   if {$trueDimCount == 2 && ![strcasecmp $MODEy Real]} {set aq2D Magnitude}

   if {![info exists acqus(topspinVersion)]} {set acqus(topspinVersion) 0}
   if {![info exists acqus(aqseq)]}          {set acqus(aqseq)          0}
   if {![info exists acqus(NS)]}             {set acqus(NS)             1}
   if {![info exists acqus(NC)]}             {set acqus(NC)             0}

   if {[string is integer $acqus(NS)]} {set vendorScans $acqus(NS)}

   if {[string is integer $acqus(NC)]} {catch {set vendorScale [expr pow(2,$acqus(NC))]}}

   set vendorMsg "Scan Count acqus(NS): $acqus(NS) Scaling Parameter acqus(NC): $acqus(NC)"

   if {$verbose} \
      {
       puts $vendorMsg

       if {$scaleByVendorFlag && $scaleByScansFlag} \
          {
           puts "Bruker Conversion for NMRPipe Version 10.7 and Higher:"
           puts "Intensities Will Be Scaled According to acqus(NS) and acqus(NC)."
           puts "Use -noscale to Suppress Scaling, or Adjust Intensity Scale Settings."
          } \
       elseif {$scaleByVendorFlag} \
          {
           puts "Bruker Conversion for NMRPipe Version 10.7 and Higher:"
           puts "Intensities Will Be Scaled According to acqus(NC)."
           puts "Use -noscale to Suppress Scaling, or Adjust Intensity Scale Settings."
          } \
       elseif {$scaleByScansFlag} \
          {
           puts "Bruker Conversion for NMRPipe Version 10.7 and Higher:"
           puts "Intensities Will Be Scaled According to acqus(NS)."
           puts "Use -noscale to Suppress Scaling, or Adjust Intensity Scale Settings."
          }

       puts ""
      }

   updateDataScale

   if {![strcmp $acqus(aqseq) 21] || ![strcmp $acqus(aqseq) 4321]} \
      {
      } \
   elseif {$pseudo3DFlag && ![strcmp $acqus(aqseq) 321] && ($acqu2s(FnMODE) <= 1 || $acqu2s(FnMODE) == 7 || $acqu2s(FnILOOP) > 1)} \
      {
       if {$verbose} {puts "Adjusting Dimensions for Pseudo-3D aqseq 321, TopSpin $acqus(topspinVersion):"}

       if {$verbose} {puts " Swapping Y-Axis and Z-Axis Sizes and Modes." }

       rSwap Ny    Nz
       rSwap Ty    Tz
       rSwap MODEy MODEz

       if {$verbose} {puts " Swapping Y-Axis and Z-Axis Chemical Shift Calibrations." }

       clearAxisLabelPostfix

       rSwap SWy   SWz
       rSwap OBSy  OBSz
       rSwap CARy  CARz
       rSwap LABy  LABz

       updateAxisLabels
      } \
   elseif {$trueDimCount == 3 && ![strcmp $acqus(aqseq) 312]} \
      {
       if {$verbose} {puts "Adjusting Dimensions for aqseq 312, TopSpin $acqus(topspinVersion):"}

       if {$acqus(topspinVersion) == 2} \
          {
           if {$nusFlag} \
              {
               if {$verbose} {puts " Swapping Y-Axis and Z-Axis Modes (NUS Data)." }

               rSwap MODEy MODEz
              } \
           else \
              {
               if {$verbose} {puts " Swapping Y-Axis and Z-Axis Modes." }

               rSwap MODEy MODEz
              }

           if {$verbose} {puts " Swapping Y-Axis and Z-Axis Chemical Shift Calibrations." }

           clearAxisLabelPostfix

           rSwap SWy   SWz
           rSwap OBSy  OBSz
           rSwap CARy  CARz
           rSwap LABy  LABz

           updateAxisLabels
          } \
       elseif {$acqus(topspinVersion) == 3} \
          { 
           if {$nusFlag} \
              {
               if {$verbose} {puts " Swapping Y-Axis and Z-Axis Modes (NUS Data)." }

               rSwap MODEy MODEz
              } \
           else \
              {
               if {$verbose} {puts " Swapping Y-Axis and Z-Axis Sizes and Modes." }

               rSwap Ny    Nz
               rSwap Ty    Tz
               rSwap MODEy MODEz
              }

           if {$verbose} {puts " Swapping Y-Axis and Z-Axis Chemical Shift Calibrations." }

           clearAxisLabelPostfix

           rSwap SWy   SWz
           rSwap OBSy  OBSz
           rSwap CARy  CARz
           rSwap LABy  LABz

           updateAxisLabels
          } \
      elseif {$acqus(topspinVersion) >= 4} \
          {
           if {$nusFlag} \
              {
               if {$verbose} {puts " Swapping Y-Axis and Z-Axis Modes (NUS Data)." }

               rSwap MODEy MODEz
              } \
           else \
              {
               if {$verbose} {puts " Swapping Y-Axis and Z-Axis Sizes and Modes." }

               rSwap Ny    Nz
               rSwap Ty    Tz
               rSwap MODEy MODEz
              }

           if {$verbose} {puts " Swapping Y-Axis and Z-Axis Chemical Shift Calibrations." }

           clearAxisLabelPostfix

           rSwap SWy   SWz
           rSwap OBSy  OBSz
           rSwap CARy  CARz
           rSwap LABy  LABz

           updateAxisLabels
          }
      } \
   else \
      {
       if {$trueDimCount == 3} \
          {
           if {![info exists acqu3s(NUC1)]} {set acqu3s(NUC1) off}

           if {[strcasecmp $acqu3s(NUC1) 15N] && [string first 15N [string toupper $LABz]] >= 0} \
              {
               if {$verbose} {puts " Swapping to Place 15N From the Z-Axis to the Y-Axis."}

               clearAxisLabelPostfix

               rSwap OBSy OBSz
               rSwap CARy CARz
               rSwap LABy LABz

               updateAxisLabels
              }
          } \
       elseif {$trueDimCount == 4} \
          {
           if {![info exists acqu3s(NUC1)]} {set acqu3s(NUC1) off}
           if {![info exists acqu4s(NUC1)]} {set acqu4s(NUC1) off}

           if {[strcasecmp $acqu3s(NUC1) 15N] && [string first 15N [string toupper $LABz]] >= 0} \
              {
               if {$verbose} {puts " Swapping to Place 15N From the Z-Axis to the Y-Axis."}

               clearAxisLabelPostfix

               rSwap OBSy OBSz
               rSwap CARy CARz
               rSwap LABy LABz

               updateAxisLabels
              } \
           elseif {[strcasecmp $acqu4s(NUC1) 15N] && [string first 15N [string toupper $LABa]] >= 0} \
              {
               if {$verbose} {puts " Swapping to Place 15N From the A-Axis to the Y-Axis."}

               clearAxisLabelPostfix

               rSwap OBSy OBSa
               rSwap CARy CARa
               rSwap LABy LABa

               updateAxisLabels
              }
          }
      }

#
# Adjust Real dimension for non-CEST pseudo3D data:

   if {$dimCount == 3 && $pseudo3DFlag && ([string first cest [string tolower $acqus(PULPROG)]] < 0)} \
      {
       clearAxisLabelPostfix

       if {![strcasecmp $MODEy Real]} \
          {
           set SWy [expr 1.0*$Ny]
           set OBSy 1.0
           set CARy 0.0

           if {$Ny > 2} \
              {
               set LABy Tau
              } \
           else \
              {
               set LABy ID
              }
          } \
       elseif {![strcasecmp $MODEz Real]} \
          {
           set SWz [expr 1.0*$Nz]
           set OBSz 1.0
           set CARz 0.0

           if {$Nz > 2} \
              {
               set LABz Tau
              } \
           else \
              {
               set LABz ID
              }
          }

       updateAxisLabels
      }

#
# Check for "skipped" dimensions:

   if {$dimCount == 3 && $Ny <= 1} \
      {
       clearAxisLabelPostfix

       set dimCount 2

       set Ny    $Nz
       set Ty    $Tz
       set MODEy $MODEz
       set SWy   $SWz
       set OBSy  $OBSz
       set CARy  $CARz
       set LABy  $LABz
       set FTy   $FTz
       set NUCy  $NUCz

       setDimCount  N 

       updateAxisLabels
      } \
   elseif {$dimCount == 4 && $Ny <= 1} \
      {
       clearAxisLabelPostfix

       set dimCount 3

       set Ny    $Nz
       set Ty    $Tz
       set MODEy $MODEz
       set SWy   $SWz
       set OBSy  $OBSz
       set CARy  $CARz
       set LABy  $LABz
       set FTy   $FTz
       set NUCy  $NUCz

       set Nz    $Na
       set Tz    $Ta
       set MODEz $MODEa
       set SWz   $SWa
       set OBSz  $OBSa
       set CARz  $CARa
       set LABz  $LABa
       set FTz   $FTa
       set NUCz  $NUCa

       setDimCount  N 

       updateAxisLabels
      } \
   elseif {$dimCount == 4 && $Nz <= 1} \
      {
       clearAxisLabelPostfix

       set dimCount 3

       set Nz    $Na
       set Tz    $Ta
       set MODEz $MODEa 
       set SWz   $SWa
       set OBSz  $OBSa
       set CARz  $CARa
       set LABz  $LABa
       set FTz   $FTa
       set NUCz  $NUCa

       setDimCount  N 

       updateAxisLabels
      }

#
# Check for interleaved data (Topspin 4, FnILOOP > 1).

   if {$acqus(topspinVersion) >= 4} \
      {
       if {[info exists acqu2s(FnMODE)]} \
          {
           if {$dimCount == 2 && $acqu2s(FnMODE) == 6} {set MODEy "Echo-AntiEcho"}
          }
      }

   if {$acqus(topspinVersion) >= 4 && $acqu2s(FnILOOP) > 1} \
      {
       if {$verbose} {puts "Adjusting for Interleaved Data (acqu2s(FnILOOP) $acqu2s(FnILOOP)) yN $Ny"}

       if {![strcasecmp $MODEy "Echo-AntiEcho"] || ![strcasecmp $MODEy "Rance-Kay"]} \
          {
           set interleaveInfo "yTime yEchoAntiEcho tau" 
          } \
       else \
          {
           set interleaveInfo "yTime yQuadrature   tau" 
          }

       if {$pseudo3DFlag} \
          {
           set expectedTauCount $acqu2s(FnILOOP)
          } \
       else \
          {
           set Ny [expr $acqu2s(FnILOOP)*$Ny]
          }
      }

   if {[string is integer $userTauCount]} \
      {
       if {$userTauCount > 0} \
          {
           if {$verbose} {puts "Adjusting Tau Count to User Setting. Expected: $expectedTauCount User: $userTauCount"}
           set expectedTauCount $userTauCount
          }
      }
}

#
# Event handler for building conversion from NIH Protocol Bruker parameters.

proc updateSpecial_BRUKER_NIH {} \
{
   global DECIM DSPFVS GRPDLY BYTORDA DTYPA dmxMode verbose
   global specialArgs wordSize wordType autoUpdateFlag nusFlag

#
# Add the Bruker-specific flags to the conversion parameters:

   if {$autoUpdateFlag} \
      {
       if {$nusFlag} \
          {
           set specialArgs " `brukUpdate.com -nus` -nocheckArgs -bad 0.0 -ext"
          } \
       else \
          {
           set specialArgs " `brukUpdate.com` -nocheckArgs -bad 0.0 -ext"
          }
      } \
   else \
      {
       set specialArgs " -bad 0.0 -ext"
      }

   if {$BYTORDA != 0} \
      {
       append specialArgs " -noaswap"
      } \
   else \
      {
       append specialArgs " -aswap"
      }

   if {$DECIM > 1} \
      {
       if {$dmxMode} \
          {
           append specialArgs " -DMX -decim $DECIM -dspfvs $DSPFVS -grpdly $GRPDLY"
          } \
       else \
          {
           append specialArgs " -AMX -decim $DECIM -dspfvs $DSPFVS -grpdly $GRPDLY"
          }
      }

   set wordSize 4
   set wordType int

   if {![info exists DTYPA]} {set DTYPA -1}

   if {[string is integer $DTYPA]} \
      {
       if {$DTYPA == 1} \
          {
           if {$autoUpdateFlag} \
              {
               if {$verbose} {puts "Auto Update Mode: Skipping Options for 4-byte floating point input."}
              } \
           else \
              {
               if {$verbose} {puts "Setting Options for 4-byte floating point input."}
               append specialArgs " -noi2f"
              }

           set wordType float
          } \
       elseif {$DTYPA == 2} \
          {
           if {$autoUpdateFlag} \
              {
               if {$verbose} {puts "Auto Update Mode: Skipping Options for 8-byte floating point input."}
              } \
           else \
              {
               if {$verbose} {puts "Setting Options for 8-byte floating point input."}
               append specialArgs " -ws 8 -noi2f"
              }

           set wordType float
           set wordSize 8 
          }
      } 
}

#
# Read Bruker parameter files.

proc getBrukerAcq { serName } \
{
   global acqus acqu2s acqu3s acqu4s shutdown

   set acqFile1 acqus
   set acqFile2 acqu2s
   set acqFile3 acqu3s
   set acqFile4 acqu4s

   set nmrPath  [tkFileFn dirname $serName]

   catch "array unset acqus"
   catch "array unset acqu2s"
   catch "array unset acqu3s"
   catch "array unset acqu4s"

   if {[tkFileTest exists $nmrPath/$acqFile1]} \
      {
       rdJCAMP $nmrPath/$acqFile1 acqus
       getTopspinVersion acqus
      } \
   else \
      {
       puts stderr "Error: Unable to find file $nmrPath/$acqFile1"
       eval $shutdown
       return 1
      }

   if {[tkFileTest exists $nmrPath/$acqFile2]} \
      {
       rdJCAMP $nmrPath/$acqFile2 acqu2s
      }

   if {[tkFileTest exists $nmrPath/$acqFile3]} \
      {
       rdJCAMP $nmrPath/$acqFile3 acqu3s
      }

   if {[tkFileTest exists $nmrPath/$acqFile4]} \
      {
       rdJCAMP $nmrPath/$acqFile4 acqu4s
      }

#
# Set defaults for missing values:

   if {![info exists acqus(DTYPA)]}   {set acqus(DTYPA)      -1}
   if {![info exists acqus(BYTORDA)]} {set acqus(BYTORDA)     0}
   if {![info exists acqus(DECIM)]}   {set acqus(DECIM)       0}
   if {![info exists acqus(DSPFVS)]}  {set acqus(DSPFVS)      0}
   if {![info exists acqus(GRPDLY)]}  {set acqus(GRPDLY)      0}
   if {![info exists acqus(PULPROG)]} {set acqus(PULPROG)  None}
   if {![info exists acqus(TE)]}      {set acqus(TE)        273}

   if {![info exists acqus(topspinVersion)]} {set acqus(topspinVersion) 0}
   if {![info exists acqus(aqseq)]}          {set acqus(aqseq)          0}

   if {![info exists acqus(AQ_mod)]}  {set acqus(AQ_mod)    0}
   if {![info exists acqus(TD)]}      {set acqus(TD)     1024}
   if {![info exists acqus(SW_h)]}    {set acqus(SW_h) 1000.0}

   foreach v {FnMODE FnILOOP} \
      {
       set acqus(${v},orig)  -1 
       set acqu2s(${v},orig) -1 
       set acqu3s(${v},orig) -1 
       set acqu4s(${v},orig) -1 

       if {[info exists acqus($v)]} \
          {
           set acqus(${v},orig) $acqus($v)
          } \
       else \
          {
           set acqus($v) 0
          }

       if {[info exists acqu2s($v)]} \
          {  
           set acqu2s(${v},orig) $acqu2s($v)  
          } \
       else \
          {
           set acqu2s($v) 0
          }

       if {[info exists acqu3s($v)]} \
          {  
           set acqu3s(${v},orig) $acqu3s($v)  
          } \
       else \
          {
           set acqu3s($v) 0
          }

       if {[info exists acqu4s($v)]} \
          {  
           set acqu4s(${v},orig) $acqu4s($v)  
          } \
       else \
          {
           set acqu4s($v) 0
          }
      }

   return 0
}

#
# Get parameters from an NIH-protocol Bruker pulse sequence.
# updated to include Bruker's more recent settings.

proc getNIHBrukerND { serName } \
{
   global Nx Tx MODEx SWx OBSx CARx FTx LABx NUCx \
          Ny Ty MODEy SWy OBSy CARy FTy LABy NUCy \
          Nz Tz MODEz SWz OBSz CARz FTz LABz NUCz \
          Na Ta MODEa SWa OBSa CARa FTa LABa NUCa

   global INMULTx  INMULTy  INMULTz  INMULTa
   global xScaleIN yScaleIN zScaleIN aScaleIN

   global DTYPA BYTORDA DECIM DSPFVS GRPDLY EXPCODES EXPNAME 
   global outName catSerFlag dimCount serBytes serWordSize serPadSize

   global acqus acqu2s acqu3s acqu4s temperature carrierInfo userAQSeq
   global tcl_precision old_tcl_precision infoRequest infoResult verbose
   global swhFlag o1Flag carXSrc shutdown

#
# Initialize:

   set serBytes     -1
   set serWordSize   4
   set serPadSize  256

   set nmrPath  [tkFileFn dirname $serName] 
   set ppFile   pulseprogram

   if {![tkFileTest exists $nmrPath/$ppFile]} \
      {
       if {[tkFileTest exists $nmrPath/pulseprogram.precomp]} \
          {
           if {$verbose} {puts "Using file pulseprogram.precomp in place of missing pulseprogram file.\n"}
           set ppFile pulseprogram.precomp
          }
      }

   if {![info exists infoRequest]} {set infoRequest None}
   if {![info exists verbose]}     {set verbose 1}

   if {![info exists swhFlag]}  {set swhFlag  0}
   if {![info exists o1Flag]}   {set o1Flag   0}
   if {![info exists carXSrc]}  {set carXSrc  None}

   if {![info exists NUCx]}     {set NUCx     None}
   if {![info exists NUCy]}     {set NUCy     None}
   if {![info exists NUCz]}     {set NUCz     None}
   if {![info exists NUCa]}     {set NUCa     None}

   if {![info exists SWx]}      {set SWx      0.0}
   if {![info exists SWy]}      {set SWy      0.0}
   if {![info exists SWz]}      {set SWz      0.0}
   if {![info exists SWa]}      {set SWa      0.0}

#
# Set the precision for the PPM expressions, enable restore of precision later:

   if {[info exists tcl_precision]} {set old_tcl_precision $tcl_precision}

   set shutdown \
     {
      if {[info exists old_tcl_precision]} \
         {
          set tcl_precision $old_tcl_precision
         }
     }

   set tcl_precision 17

#
# Test for the existence of the parameter and data files.
# Read their contents into arrays.

   if {[tkFileTest exists $serName]} \
      {
       if {[tkFileTest exists $nmrPath/$ppFile]} \
          {
           set serBytes [getFileBytes $serName -estimate]
          } \
       else \
          {
           set serBytes [getFileBytes $serName -exact]
          }
      }
  
   set error [getBrukerAcq $serName]

   if {$error} \
      {
       eval $shutdown
       return
      }
 
   set DTYPA $acqus(DTYPA)

   if {[string is integer $DTYPA]} \
      {
       if {$DTYPA == 2} \
          {
           set serWordSize 8
           set serPadSize  128 
          }
      }

   if {![strcasecmp $temperature "From File"]} \
      {
       set temperature $acqus(TE)
      }

   if {[string is double $CARx]} \
      {
       set directCar $CARx
      } \
   else \
      {
       set directCar  0.0
      }

   set directNuc    X 
   set directGamma  1.0 
   set directO1     0.0
   set directObs    500.0
   set directBase   500.0
   set directSrc    User 
   set directChan   1

   set h2oPPM  [getH2Oppm $temperature]

   foreach i {1 2 3 4 5 6 7 8} \
      {
       if {![info exists acqus(NUC$i)]} {set acqus(NUC$i) off}
       if {[isOff $acqus(NUC$i)] || ![info exists acqus(SFO$i)]} continue
       if {![string is double $acqus(SFO$i)]} continue

       set directNuc   $acqus(NUC$i)
       set directObs   $acqus(SFO$i)
       set directBase  $acqus(SFO$i)
       set directGamma [getGamma $directNuc]
       set directChan  $i 

       if {[info exists acqus(BF$i)]} \
          {
           if {[string is double $acqus(BF$i)]} {set directBase $acqus(BF$i)}
          }

       if {[info exists acqus(O$i)]} \
          {
           if {[string is double $acqus(O$i)]} {set directO1 $acqus(O$i)}
          }

       break
      }

   foreach i {1 2 3 4 5 6 7 8} \
      {
       if {![info exists acqus(NUC$i)]}  {set acqus(NUC$i)  off}
       if {![info exists acqu2s(NUC$i)]} {set acqu2s(NUC$i) off}
       if {![info exists acqu3s(NUC$i)]} {set acqu3s(NUC$i) off}
       if {![info exists acqu4s(NUC$i)]} {set acqu4s(NUC$i) off}

       set acqus(NUC$i)  [string toupper $acqus(NUC$i)]
       set acqu2s(NUC$i) [string toupper $acqu2s(NUC$i)]
       set acqu3s(NUC$i) [string toupper $acqu3s(NUC$i)]
       set acqu4s(NUC$i) [string toupper $acqu4s(NUC$i)]

       if {![info exists acqus(O$i)]}  {set acqus(O$i)  0.0}
       if {![info exists acqu2s(O$i)]} {set acqu2s(O$i) 0.0}
       if {![info exists acqu3s(O$i)]} {set acqu3s(O$i) 0.0}
       if {![info exists acqu4s(O$i)]} {set acqu4s(O$i) 0.0}

       if {![string is double $acqus(O$i)]}  {set acqus(O$i)  0.0}
       if {![string is double $acqu2s(O$i)]} {set acqu2s(O$i) 0.0}
       if {![string is double $acqu3s(O$i)]} {set acqu3s(O$i) 0.0}
       if {![string is double $acqu4s(O$i)]} {set acqu4s(O$i) 0.0}

       if {![isOff acqus(NUC$i)]} \
          {
           if {![info exists acqus(SFO$i)]}       {set acqus(SFO$i) 0.0}
           if {![info exists acqus(BF$i)]}        {set acqus(BF$i)  $acqus(SFO$i)}

           if {![string is double $acqus(SFO$i)]} {set acqus(SFO$i) 0.0}
           if {![string is double $acqus(BF$i)]}  {set acqus(BF$i)  0.0}
          }

       if {![isOff acqu2s(NUC$i)]} \
          {
           if {![info exists acqu2s(SFO$i)]}       {set acqu2s(SFO$i) 0.0}
           if {![string is double $acqu2s(SFO$i)]} {set acqu2s(SFO$i) 0.0}

           if {![info exists acqu2s(BF$i)]}        {set acqu2s(BF$i)  $acqu2s(SFO$i)}
           if {![string is double $acqu2s(BF$i)]}  {set acqu2s(BF$i)  0.0}
          }

       if {![isOff acqu3s(NUC$i)]} \
          {
           if {![info exists acqu3s(SFO$i)]}       {set acqu3s(SFO$i) 0.0}
           if {![string is double $acqu3s(SFO$i)]} {set acqu3s(SFO$i) 0.0}

           if {![info exists acqu3s(BF$i)]}        {set acqu3s(BF$i)  $acqu3s(SFO$i)}
           if {![string is double $acqu3s(BF$i)]}  {set acqu3s(BF$i)  0.0}
          }

       if {![isOff acqu4s(NUC$i)]} \
          {
           if {![info exists acqu4s(SFO$i)]}       {set acqu4s(SFO$i) 0.0}
           if {![string is double $acqu4s(SFO$i)]} {set acqu4s(SFO$i) 0.0}

           if {![info exists acqu4s(BF$i)]}        {set acqu4s(BF$i)  $acqu4s(SFO$i)}
           if {![string is double $acqu4s(BF$i)]}  {set acqu4s(BF$i)  0.0}
          }
      }

   if {![strcasecmp $CARx H2O]} \
      {
       if {![strcasecmp $directNuc 1H] || ![strcasecmp $directNuc H1]} \
          {
           set directCar $h2oPPM
           set carXSrc   H2O
          } \
       else \
          {
           set directCar [expr $directO1/$directBase]
           set carXSrc   "O$directChan/BF$directChan"
          }

       set CARx $directCar
      } \
   elseif {![strcasecmp $carXSrc User]} \
      {
       if {![string is double $CARx]} {set CARx 0.0}
       set directCar $CARx
       set charXSrc  None
      } \
   elseif {![strcasecmp $carXSrc None]} \
      {
       set directCar [expr $directO1/$directBase]
       set carXSrc   "O$directChan/BF$directChan"
      }

   set LABx $directNuc
   set LABy $directNuc
   set LABz $directNuc
   set LABa $directNuc

   set OBSx $directObs
   set OBSy $directObs
   set OBSz $directObs
   set OBSa $directObs

   set CARx $directCar
   set CARy $directCar
   set CARz $directCar
   set CARa $directCar

#
# Set the parameters for the directly-detected dimension:

   set DTYPA    $acqus(DTYPA)
   set BYTORDA  $acqus(BYTORDA)
   set DECIM    $acqus(DECIM)
   set DSPFVS   $acqus(DSPFVS)
   set GRPDLY   $acqus(GRPDLY)

   switch $acqus(AQ_mod) \
      {
       0 {set MODEx Real}
       1 {set MODEx Complex}
       2 {set MODEx Sequential}
       3 {set MODEx DQD}
       default \
          {
           puts stderr "Error: unknown acqusition mode AQ_mod = $acqus(AQ_mod)"
           eval $shutdown
           return 1
          }
      }

   set pad [expr $acqus(TD) % $serPadSize]

   if {$pad} \
      {
       set Nx [expr $acqus(TD) + $serPadSize - $pad]
      } \
   else \
      {
       set Nx $acqus(TD)
      }

   if {$MODEx == "Sequential" || $MODEx == "Real"} \
      {
       set Tx $acqus(TD)
      } \
   else \
      {
       set Tx [expr $acqus(TD) / 2]
      }

   set xScaleIN 1.0
   set yScaleIN 2.0
   set zScaleIN 2.0
   set aScaleIN 2.0

   if {[strcasecmp $INMULTx Auto]} {set xScaleIN $INMULTx}
   if {[strcasecmp $INMULTy Auto]} {set yScaleIN $INMULTy}
   if {[strcasecmp $INMULTz Auto]} {set zScaleIN $INMULTz}
   if {[strcasecmp $INMULTa Auto]} {set aScaleIN $INMULTa}

   set SWx [expr $xScaleIN*$acqus(SW_h)]

#
# Set some initial parameters for the indirect dimensions:

   if {![info exists acqu2s(TD)]}  {set acqu2s(TD) 0}

   catch {set Ny   $acqu2s(TD)}
   catch {set Ty   [expr $Ny / 2]}
   catch {set IN0  [lindex $acqus(IN) 0]}
   catch {set SWy  [expr 1.0 / ($yScaleIN*$IN0)]}

   if {[info exists acqu3s(TD)]} \
      {
       catch {set Nz   $acqu3s(TD)}
      } \
   else \
      {
       catch {set Nz   [expr $serBytes / ($Nx * $Ny * $serWordSize)]}
      }

   catch {set Tz   [expr $Nz / 2]}
   catch {set IN6  [lindex $acqus(IN) 6]}
   catch {set SWz  [expr 1.0 / ($zScaleIN*$IN6)]}

   catch {set Na   [expr $serBytes / ($Nx * $Ny * 16 * $serWordSize)]}

   if {[info exists acqu4s(TD)]} \
      {
       if {[string is integer $acqu4s(TD)]} \
          {
           if {$acqu4s(TD) > 1} \
              {
               set Na $acqu4s(TD)
               set dimCount 4
              }
          }
      } \
   else \
      {
       catch {set Na   [expr $serBytes / ($Nx * $Ny * $serWordSize)]}
      }

   catch {set Ta   [expr $Na / 2]}
   catch {set IN7  [lindex $acqus(IN) 7]}
   catch {set SWa  [expr 1.0 / ($aScaleIN*$IN7)]}

#
# Read and parse the pulse program file if possible:

   set varList ""

   if {[tkFileTest exists $nmrPath/$ppFile]} \
      {
       set vFlag $verbose 
       rdPProg $nmrPath/$ppFile nArray dArray pArray loopCount varList $vFlag

       set vFlag 0
       parsePProg nArray dArray pArray $loopCount $vFlag

       if {[info exists userAQSeq]} \
          {
           if {[strcasecmp $userAQSeq None] && [strcasecmp $userAQSeq Auto]} \
              {
               if {$verbose} {puts "Changing User-Specified aqseq Value. Old: $acqus(aqseq) New: $userAQSeq"}
               set acqus(aqseq) $userAQSeq
              }
          }
      } \
   else \
      {
       if {$dimCount > 2} \
          {
           showPoorEntries {Ny Ty Nz Tz Na Ta}
          }
      }

   if {$dimCount == 2 && $Ny == 1} \
      {
       set dimCount 1
       setDimCount  N
      }

   if {$dimCount == 3 && $Nz == 1} \
      {
       set dimCount 2
       setDimCount  N
      }

#
# Reset SW values if requested:

   if {$swhFlag} \
      {
       if {[info exists acqu2s(SW_h)]} {set SWy $acqu2s(SW_h)}
       if {[info exists acqu3s(SW_h)]} {set SWz $acqu3s(SW_h)}
       if {[info exists acqu4s(SW_h)]} {set SWa $acqu4s(SW_h)}
      }

#
# Update the menus associated with the ambiguous values:

   set swList  {}
   set obsList {}
   set carList {}

#
# SW by increments:

   set k 0

   foreach i $acqus(IN) \
      {
       if {[lsearch -exact $varList IN$k] >= 0} \
          {
           if {$i == 0} continue
           set val  [safeFltFormat %9.3f [expr 1.0/(2.0*$i)]]
           set line [list "$val 1/(2*IN$k)" $val]
           lappend swList $line 
          }

       incr k
      }

   set k 0

   foreach i $acqus(IN) \
      {
       if {[lsearch -exact $varList IN$k] >= 0} \
          {
           if {$i == 0} continue
           set val  [safeFltFormat %9.3f [expr 1.0/$i]]
           set line [list "$val 1/(1*IN$k)" $val]
           lappend swList $line
          }

       incr k
      }

   set k 0

   foreach i $acqus(IN) \
      {
       if {[lsearch -exact $varList IN$k] >= 0} \
          {
           if {$i == 0} continue

           foreach swScaleIN "$xScaleIN $yScaleIN $zScaleIN $aScaleIN" \
              {
               if {$swScaleIN == 1.0 || $swScaleIN == 2.0} continue

               set val  [safeFltFormat %9.3f [expr 1.0/($swScaleIN*$i)]]
               set line [list "$val 1/($swScaleIN*IN$k)" $val]
               lappend swList $line
              }
          }

       incr k
      }

   if {[info exists acqus(SW_h)]} \
      {
       lappend swList [format "{%9.3f acqus(SW_h)} %9.3f" $acqus(SW_h)  $acqus(SW_h)]
      }

   if {[info exists acqu2s(SW_h)]} \
      {
       lappend swList [format "{%9.3f acqu2s(SW_h)} %9.3f" $acqu2s(SW_h) $acqu2s(SW_h)]
      }

   if {[info exists acqu3s(SW_h)]} \
      {
       lappend swList [format "{%9.3f acqu3s(SW_h)} %9.3f" $acqu3s(SW_h) $acqu3s(SW_h)]
      }

   if {[info exists acqu4s(SW_h)]} \
      {
       lappend swList [format "{%9.3f acqu4s(SW_h)} %9.3f" $acqu4s(SW_h) $acqu4s(SW_h)]
      }

   foreach i {1 2 3 4 5 6 7 8} \
      {
       if {![strcasecmp $acqus(NUC$i) off]} continue
       if {![strcasecmp $acqus(NUC$i) None]} continue

       set val [safeFltFormat %7.3f $acqus(SFO$i)]
       lappend obsList [list "$val (SFO$i) [format %3s $acqus(NUC$i)]" $val]
      }

   foreach i {1} \
      {
       if {[isOff $acqu2s(NUC$i)] || $acqu2s(FnMODE) == 0} continue
       set val [safeFltFormat %7.3f $acqu2s(SFO$i)]
       lappend obsList [list "$val (SFO$i/acqu2s) [format %3s $acqu2s(NUC$i)]" $val]
      }

   foreach i {1} \
      {
       if {[isOff $acqu3s(NUC$i)] || $acqu3s(FnMODE) == 0} continue
       set val [safeFltFormat %7.3f $acqu3s(SFO$i)]
       lappend obsList [list "$val (SFO$i/acqu3s) [format %3s $acqu3s(NUC$i)]" $val]
      }

  foreach i {1} \
      {
       if {[isOff $acqu4s(NUC$i)] || $acqu4s(FnMODE) == 0} continue
       set val  [safeFltFormat %7.3f $acqu4s(SFO$i)]
       set line [list "$val (SFO$i/acqu4s) [format %3s $acqu4s(NUC$i)]" $val]
      }

#
# Carrier according to gamma:

   set LABx $directNuc
   set OBSx $directObs
   set CARx $directCar

   lappend carList [list H2O H2O]

   set car [safeFltFormat %7.3f $directCar]
   lappend carList [list "$car ($directSrc)" $car]

   if {![strcasecmp $directNuc 1H] || ![strcasecmp $directNuc 1H]} \
      {
       set car [safeFltFormat %7.3f $h2oPPM]
       lappend carList [list "$car (H2O)" $car]
      }

   set j 0

   foreach i {1 2 3 4 5 6 7 8} \
      {
       if {[isOff $acqus(NUC$i)] || $acqus(SFO$i) == 0.0} continue

       set g      [getGamma $acqus(NUC$i)]
       set gRatio [expr $g/$directGamma]
       set thisObs $acqus(SFO$i)

       set val [expr (1.0e+6*$thisObs-((($directObs*1.0e+6)-($directObs*$directCar))*$gRatio))/$thisObs]
       set car [safeFltFormat %7.3f $val]
       set lab [format "%3s from %s %.3fppm" $acqus(NUC$i) $directNuc $directCar]

       lappend carList [list "$car (SFO$i) $lab" $car]

       set carrierInfo(SFO$i) [string trim $car]

       incr j

       set thisNuc [normalizeNucName $acqus(NUC$i)]

       set nucInfo($thisNuc,LAB) $acqus(NUC$i)
       set nucInfo($thisNuc,OBS) $acqus(SFO$i)
       set nucInfo($thisNuc,CAR) $val

       if {$j == 1} \
          {
           set LABy $acqus(NUC$i)
           set OBSy $acqus(SFO$i)
           set CARy $val

           set LABz $acqus(NUC$i)
           set OBSz $acqus(SFO$i)
           set CARz $val

           set LABa $acqus(NUC$i)
           set OBSa $acqus(SFO$i)
           set CARa $val
          } \
       elseif {$j == 2} \
          {
           set LABy $acqus(NUC$i)
           set OBSy $acqus(SFO$i)
           set CARy $val
          } \
       elseif {$j == 3} \
          {
           set LABz $acqus(NUC$i)
           set OBSz $acqus(SFO$i)
           set CARz $val
          }
      }

   if {![isOff $acqu2s(NUC1)] && $acqu2s(FnMODE) != 0 && $acqu2s(SFO1) != 0.0} \
      {
       set g      [getGamma $acqu2s(NUC1)]
       set gRatio [expr $g/$directGamma]
       set thisObs $acqu2s(SFO1)

       set val [expr (1.0e+6*$thisObs-((($directObs*1.0e+6)-($directObs*$directCar))*$gRatio))/$thisObs]
       set car [safeFltFormat %7.3f $val]
       set lab [format "%3s from %s %.3fppm" $acqu2s(NUC1) $directNuc $directCar]

       lappend carList [list "$car (SFO1/acqu2s) $lab" $car]

       set carrierInfo(SFO1/acqu2s) [string trim $car]

       set LABy $acqu2s(NUC1)
       set OBSy $acqu2s(SFO1)
       set CARy $val

       set thisNuc [normalizeNucName $acqu2s(NUC1)]

       set nucInfo($thisNuc,LAB) $acqu2s(NUC1)
       set nucInfo($thisNuc,OBS) $acqu2s(SFO1)
       set nucInfo($thisNuc,CAR) $val
      }

   if {![isOff $acqu3s(NUC1)] && $acqu3s(FnMODE) != 0 && $acqu3s(SFO1) != 0.0} \
      {
       set g      [getGamma $acqu3s(NUC1)]
       set gRatio [expr $g/$directGamma]
       set thisObs $acqu3s(SFO1)

       set val [expr (1.0e+6*$thisObs-((($directObs*1.0e+6)-($directObs*$directCar))*$gRatio))/$thisObs]
       set car [safeFltFormat %7.3f $val]
       set lab [format "%3s from %s %.3fppm" $acqu3s(NUC1) $directNuc $directCar]

       lappend carList [list "$car (SFO1/acqu3s) $lab" $car]

       set carrierInfo(SFO1/acqu3s) [string trim $car]

       set LABz $acqu3s(NUC1)
       set OBSz $acqu3s(SFO1)
       set CARz $val

       set thisNuc [normalizeNucName $acqu3s(NUC1)]
      
       set nucInfo($thisNuc,LAB) $acqu3s(NUC1)
       set nucInfo($thisNuc,OBS) $acqu3s(SFO1)
       set nucInfo($thisNuc,CAR) $val
      }

   if {![isOff $acqu4s(NUC1)] && $acqu4s(FnMODE) != 0 && $acqu4s(SFO1) != 0.0} \
      {
       set g      [getGamma $acqu4s(NUC1)]
       set gRatio [expr $g/$directGamma]
       set thisObs $acqu4s(SFO1)

       set val [expr (1.0e+6*$thisObs-((($directObs*1.0e+6)-($directObs*$directCar))*$gRatio))/$thisObs]
       set car [safeFltFormat %7.3f $val]
       set lab [format "%3s from %s %.3fppm" $acqu4s(NUC1) $directNuc $directCar]

       lappend carList [list "$car (SFO1/acqu4s) $lab" $car]

       set carrierInfo(SFO1/acqu4s) [string trim $car]

       set LABa $acqu4s(NUC1)
       set OBSa $acqu4s(SFO1)
       set CARa $val

       set thisNuc [normalizeNucName $acqu4s(NUC1)]
      
       set nucInfo($thisNuc,LAB) $acqu4s(NUC1)
       set nucInfo($thisNuc,OBS) $acqu4s(SFO1)
       set nucInfo($thisNuc,CAR) $val
      }

#
# Carrier according to O1/BF1

   set j 0

   foreach i {1 2 3 4 5 6 7 8} \
      {
       if {[isOff $acqus(NUC$i)] || ![info exists acqus(BF$i)]} continue

       set val [safeDiv $acqus(O$i) $acqus(BF$i)]
       if {$val == 0.0} continue

       set car [safeFltFormat %7.3f $val]
       lappend carList  [list "$car (O$i/BF$i) $acqus(NUC$i)" $car]

       set carrierInfo(O$i/BF$i) [string trim $car]

       if {$o1Flag} \
          {
           set thisNuc [normalizeNucName $acqus(NUC$i)]
           
           set nucInfo($thisNuc,LAB) $acqu2s(NUC$i)
           set nucInfo($thisNuc,OBS) $acqu2s(SFO$i)
           set nucInfo($thisNuc,CAR) $val
          }
      }

   if {![isOff $acqu2s(NUC1)] && $acqu2s(FnMODE) != 0 && [info exists acqu2s(BF1)]} \
      {
       set val [safeDiv $acqu2s(O1) $acqu2s(BF1)]

       if {$val != 0.0} \
          {
           if {$o1Flag} \
              {
               set LABy $acqu2s(NUC1)
               set OBSy $acqu2s(SFO1)
               set CARy $val

               set thisNuc [normalizeNucName $acqu2s(NUC1)]
           
               set nucInfo($thisNuc,LAB) $acqu2s(NUC1)
               set nucInfo($thisNuc,OBS) $acqu2s(SFO1)
               set nucInfo($thisNuc,CAR) $val
              }
 
           set car [safeFltFormat %7.3f $val]
           lappend carList  [list "$car (O1/BF1/acqu2s) $acqu2s(NUC1)" $car]

           set carrierInfo(O1/BF1/acqu2s) [string trim $car]
          }
      }

   if {![isOff $acqu3s(NUC1)] && $acqu3s(FnMODE) != 0 && [info exists acqu3s(BF1)]} \
      {
       set val [safeDiv $acqu3s(O1) $acqu3s(BF1)]
   
       if {$val != 0.0} \
          { 
           if {$o1Flag} \
              {
               set LABz $acqu3s(NUC1)
               set OBSz $acqu3s(SFO1)
               set CARz $val

               set thisNuc [normalizeNucName $acqu3s(NUC1)]
          
               set nucInfo($thisNuc,LAB) $acqu3s(NUC1)
               set nucInfo($thisNuc,OBS) $acqu3s(SFO1)
               set nucInfo($thisNuc,CAR) $val
              }

           set car [safeFltFormat %7.3f $val]
           lappend carList  [list "$car (O1/BF1/acqu3s) $acqu3s(NUC1)" $car]

           set carrierInfo(O1/BF1/acqu3s) [string trim $car]
          }
      }

   if {![isOff $acqu4s(NUC1)] && $acqu4s(FnMODE) != 0 && [info exists acqu4s(BF1)]} \
      {
       set val [safeDiv $acqu4s(O1) $acqu4s(BF1)]

       if {$val != 0.0} \
          { 
           if {$o1Flag} \
              {    
               set LABa $acqu4s(NUC1)
               set OBSa $acqu4s(SFO1)
               set CARa $val

               set thisNuc [normalizeNucName $acqu4s(NUC1)]
          
               set nucInfo($thisNuc,LAB) $acqu4s(NUC1)
               set nucInfo($thisNuc,OBS) $acqu4s(SFO1)
               set nucInfo($thisNuc,CAR) $val
              }

           set car [safeFltFormat %7.3f $val]
           lappend carList  [list "$car (O1/BF1/acqu4s) $acqu4s(NUC1)" $car]

           set carrierInfo(O1/BF1/acqu4s) [string trim $car]
          }
      }

#
# Check for user-suggested nucleus:

   if {[strcasecmp $NUCx None]} \
      {
       set thisNuc [normalizeNucName $NUCx]

       if {[info exists nucInfo($thisNuc,LAB)]} \
          {
           set LABx $nucInfo($thisNuc,LAB)
           set OBSx $nucInfo($thisNuc,OBS)
           set CARx $nucInfo($thisNuc,CAR)
          }
      }

   if {[strcasecmp $NUCy None]} \
      {
       set thisNuc [normalizeNucName $NUCy]

       if {[info exists nucInfo($thisNuc,LAB)]} \
          {
           set LABy $nucInfo($thisNuc,LAB)
           set OBSy $nucInfo($thisNuc,OBS)
           set CARy $nucInfo($thisNuc,CAR)
          }
      }

   if {[strcasecmp $NUCz None]} \
      {
       set thisNuc [normalizeNucName $NUCz]

       if {[info exists nucInfo($thisNuc,LAB)]} \
          {
           set LABz $nucInfo($thisNuc,LAB)
           set OBSz $nucInfo($thisNuc,OBS)
           set CARz $nucInfo($thisNuc,CAR)
          }
      }

   if {[strcasecmp $NUCa None]} \
      {
       set thisNuc [normalizeNucName $NUCa]

       if {[info exists nucInfo($thisNuc,LAB)]} \
          {
           set LABa $nucInfo($thisNuc,LAB)
           set OBSa $nucInfo($thisNuc,OBS)
           set CARa $nucInfo($thisNuc,CAR)
          }
      }

   foreach i {a z y x} \
      {
       if {[llength $swList]  > 0} {updateGenMenu SW  $i $swList}
       if {[llength $obsList] > 0} {updateGenMenu OBS $i $obsList}
       if {[llength $carList] > 0} {updateGenMenu CAR $i $carList}
      }

   showPoorEntries {CARx CARy CARz CARa SWy SWz SWa OBSy OBSz OBSa}

   if {![strcasecmp $MODEy "Rance-Kay"]} {set MODEy Complex}
   if {![strcasecmp $MODEz "Rance-Kay"]} {set MODEz Complex}
   if {![strcasecmp $MODEa "Rance-Kay"]} {set MODEa Complex}

#
# Adjust precision of calculated parameters:

   set SWx  [safeFltFormat "%.3f" $SWx]
   set SWy  [safeFltFormat "%.3f" $SWy]
   set SWz  [safeFltFormat "%.3f" $SWz]
   set SWa  [safeFltFormat "%.3f" $SWa]

   set CARx [safeFltFormat "%.3f" $CARx]
   set CARy [safeFltFormat "%.3f" $CARy]
   set CARz [safeFltFormat "%.3f" $CARz]
   set CARa [safeFltFormat "%.3f" $CARa]

   set OBSx [safeFltFormat "%.3f" $OBSx]
   set OBSy [safeFltFormat "%.3f" $OBSy]
   set OBSz [safeFltFormat "%.3f" $OBSz]
   set OBSa [safeFltFormat "%.3f" $OBSa]

   set temperature [safeFormat "%.3f" $temperature]

#
# Fix 1D name:

   if {$Ny < 1} \
      {
       set LABx $directNuc
      }

#
# Extract any requested values:

   if {[strcasecmp $infoRequest None]} \
      {
       set infoResult ""

       foreach var $infoRequest \
          {
           lappend infoResult $var

           if {[info exists acqus($var)]} \
              {
               lappend infoResult $acqus($var)
              } \
           elseif {[info exists $var]} \
              {
               lappend infoResult [subst -nocommands \$$var] 
              } \
           else \
              {
               lappend infoResult None
              }            
          }
      }

   eval $shutdown

   return 0
}

proc getTopspinVersion { parmPtr } \
{
    upvar $parmPtr parmList

#
# Can either be with or without "Version" in the text:
#
#  ##TITLE= Parameter file, TOPSPIN Version 3.2
#  ##TITLE= Parameter file, TopSpin 3.5 pl 6
#  ##TITLE= Parameter file, TopSpin 4.0.b.20161214 pl 0

    set parmList(topspinVersion) 0

    if {![info exists parmList(.hdr)]} return

    set tInfo ""
    set vInfo ""

    foreach item $parmList(.hdr) \
       {
        set key [string trim [lindex $item 0] "=#"]

        if {![strcasecmp $key TITLE]} \
           {
            set tInfo [string tolower $item]
            break
           }
       }

    set n [llength $tInfo]

    for {set i 0} {$i < $n} {incr i} \
       {
        set key [lindex $tInfo $i]

        if {[string first "topspin" $key] >= 0} \
           {
            foreach s [lrange $tInfo [expr $i + 1] [expr $i + 2]] \
               { 
                set c [lindex [split $s "."] 0]

                if {[string is integer $c]} \
                   {
                    set parmList(topspinVersion) $c
                    return
                   }
               }
           }
       }
}

#
# Return 1 if str is "off".

proc isOff { str } \
{
   if {![strcasecmp [string trim $str] off]} {return 1}
   return 0
}

#
# Bottom.
