#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

#
# check for hypercomplex input adjustment of qSize

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2022.063.10.03"

set knownFnList \
 {
  set       add       subtract divide   multiply clipBelow    clipAbove    binarize     selectMask    rampUp   \
  clipPos   clipNeg   log      power    abs      bin          img          label        rampDown \
  hole      plateau
 }

set vecFnList \
 {
  vs,Set     vs,Add    vs,Sub   vs,Div   vs,Mult  vs,ClipBelow vs,ClipAbove vs,Binarize  vs,SelectMask v,RampUp  \
  v,ClipPos  v,ClipNeg v,Log    vs,Power v,Abs    multi,vBin   multi,vImage multi,vLabel v,RampDown \
  multi,hole multi,plateau
 }

set knownValList {min  max  minAbs  maxAbs  amin  amax  sum  avg  stdDev  rms  variance  range  median  estNoise  thresh noise xSize ySize xSizeFn ySizeFn xSizeVal ySizeVal fnN valN N}
set valFnList    {vMin vMax vMinAbs vMaxAbs vAMin vAMax vSum vAvg vStdDev vRMS vVariance vRange vMedian vEstNoise thresh noise xSize ySize xSizeFn ySizeFn xSizeVal ySizeVal fnN valN N}
set skipList     {the from to by with in on of then and all data take perform do}

if {[flagLoc $ARGV -help]} \
   {
    puts "Clipping, Normalization, Binarization, etc. (Version $thisVersion):"
    puts " -in      inList  \[test.ft2\]  Input Spectra List."
    puts " -out     outName \[norm.ft2\]  Output Spectra, or Keyword Auto."
    puts " -outDir  outDir  \[norm\]      Output Directory (Used Only for -out Auto)."
    puts " -prefix  prefix  \[tmpnorm\$\$\] Prefix for Temporary Files (\$\$ is Process ID)."
    puts " -prog    pList   \[None ...\]  List of Functions to Apply."
    puts " -noise   noise   \[Auto\]      Noise Value."
    puts " -thresh  thresh  \[Auto\]      Threshold."
    puts " -factor  f       \[7.0\]       Factor for -thresh Auto."
    puts " -verb                        Verbose Mode ON (Default)."
    puts " -noverb                      Verbose Mode OFF (Default)."
    puts "Functions (value is a Numeric Value or Keyword):"
    puts " set        value             Set Data to Value."
    puts " add        value             Add value to Data."
    puts " subtract   value             Subtract value from Data." 
    puts " divide     value             Divide Data by value." 
    puts " multiply   value             Multiply Data by value." 
    puts " clipBelow  value             Set Data Below value to value."
    puts " clipAbove  value             Data Data Above value to value."
    puts " binarize   value             Set Data Above value to One, Data Below to Zero."
    puts " selectMask value             Set Data Equal to value to One, Other Data to Zero."
    puts " power      value             Set Data to value power."
    puts " clipPos                      Set Positive Data to Zero."
    puts " clipNeg                      Set Negative Data to Zero."
    puts " log                          Perform log() Function of Data."
    puts " abs                          Take Absolute Value of Data."
    puts "Special Functions:"
#   puts " bin       sizeArgs  \[1,1,...\]          Binning Sizes and Options for vBin."
    puts " label                                    Component Labeling (Requires Binarized Input)."
    puts " rampUp    regInfo                        1D Cos-square Ramp Up (0.0 to 1.0)."
    puts " rampDown  regInfo                        1D Cos-square Ramp Down (1.0 to 0.0)."
    puts " hole      hArgs     \[x1=0%,...y4=100%]  ND Cos-square Ramp Hole."
    puts " plateau   pArgs     \[x1=0%,...y4=100%]  ND Cos-square Ramp Plateau."
    puts " img       imgArgs   \[dx=1,dy=1,max\]    2D IMG Options (See nmrWish vImage -help)."
    puts "Keywords for value:"
    
    prettyPrintList stdout " " $knownValList

    puts "Modifier for Function or value:"
    puts " region    coordList \[0% 100% ...\]   Coordinates for Region of Interest."
    puts "Ignored Words:"
    puts " $skipList"
    puts ""
    puts "Program Examples:"
    puts ""
    puts " Set All Data to Zero:"
    puts ""
    puts "    -prog set 0.0"
    puts ""
    puts " Set Data in 2D Region (X: 8.0ppm 7.0ppm, Y: 121ppm 120.0ppm) to Zero:"
    puts ""
    puts "    -prog set region 8.0ppm 7.0ppm 121ppm 120.0ppm 0.0"
    puts ""
    puts "    Or:"
    puts ""
    puts "    -prog set region 8.0ppm 7.0ppm 121ppm 120.0ppm to 0.0"
    puts ""
    puts " Subtract Average of All Data:"
    puts ""
    puts "    -prog subtract avg"
    puts ""
    puts "    Or:"
    puts ""
    puts "    -prog subtract avg of all data"
    puts ""
    puts " Divide Data by Maximum Intensity of a 2D Central Region:"
    puts ""
    puts "    -prog divide maxAbs region 25% 75% 40% 60%" 
    puts ""
    puts "    Or:"
    puts ""
    puts "    -prog divide all data by maxAbs of region 25% 75% 40% 60%" 
    puts ""
    puts " 5x5 (X+/-2, Y+/-2) Filter Replacing Points by Local RMS:"
    puts ""
    puts "    -prog IMG dx=2,dy=2,rms"
    puts ""
    exit 0
   }

set inList   test.ft2
set outList  norm.ft2
set outDir   norm

set prefix   tmpnorm[pid]
set progInfo None

set noise    Auto
set thresh   Auto
set factor   7.0

set verbose  1

getListArgD $ARGV -in  inList
getListArgD $ARGV -out outList

getArgD $ARGV -outDir outDir
getArgD $ARGV -prefix prefix
getArgD $ARGV -thresh thresh
getArgD $ARGV -factor factor

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

set loc [flagLoc $ARGV -prog]

if {$loc > 1} {set progInfo [lrange $ARGV [expr $loc + 1] end]}

if {![llength $progInfo]} {set progInfo None}

foreach inName $inList {lappend outList Auto}

#
#-------------------------------------------------------------------------------------------------------------------------

proc getFnArgCount { thisFn } \
{
   global knownFnList vecFnList

   set loc [lsearch -exact [string tolower $knownFnList] [string tolower $thisFn]]

   if {$loc < 0} {return -1}

   set vecFn     [lindex $vecFnList $loc]
   set vecInfo   [split $vecFn ","]

   set vecPrefix [lindex $vecInfo 0]
   set vecCall   [lindex $vecInfo 1]

   if {![strcasecmp $vecPrefix v]}     {return 0}
   if {![strcasecmp $vecPrefix vs]}    {return 1}
   if {![strcasecmp $vecPrefix multi]} {return 1}

   return -1
}

proc getFnVal { thisFn thisVal } \
{
   global valReg valN rdata valBuff valReg dimCount n knownValList valFnList coordListVal noise thresh verbose
   global xSize ySize zSize aSize xSizeFn ySizeFn zSizeFn aSizeFn xSizeVal ySizeVal zSizeVal aSizeVal 

   set loc     [lsearch -exact [string tolower $knownValList] [string tolower $thisVal]]

   set newVal  $thisVal
   set infoStr $thisVal
   set valFn   None

   if {![strcasecmp $thisFn bin]} \
      {
       set infoStr "Comma-separated Options"
      } \
   elseif {![strcasecmp $thisFn hole] || ![strcasecmp $thisFn plateau]} \
      {
       set infoStr "Comma-separated Options"
      } \
   elseif {$loc < 0 && [string is double $thisVal]} \
      {
       set infoStr "Numeric Value"
      } \
   elseif {$loc < 0} \
      {
       puts "NormSpec. Error: Unknown Value Type $thisVal"
       exit 1
      } \
   elseif {![strcasecmp $thisVal noise]} \
      {
       set infoStr "User-supplied noise"
       set newVal $noise
      } \
   elseif {![strcasecmp $thisVal N]} \
      {
       set infoStr "Number of Points in Input Spectrum"
       set newVal $n
      } \
   elseif {![strcasecmp $thisVal fnN]} \
      {
       set infoStr "Number of Points Function Region"
       set newVal $fnN
      } \
   elseif {![strcasecmp $thisVal valN]} \
      {
       set infoStr "Number of Points in Value Region"
       set newVal $valN
      } \
   elseif {![strcasecmp $thisVal thresh]} \
      {
       set infoStr "User-supplied thresh"
       set newVal $thresh
      } \
   elseif {![strcasecmp $thisVal xSize]} \
      {
       set infoStr "Full X-Axis Size of Input Spectrum"
       set newVal $xSize
      } \
   elseif {![strcasecmp $thisVal ySize]} \
      {
       set infoStr "Full Y-Axis Size of Input Spectrum"
       set newVal $ySize
      } \
   elseif {![strcasecmp $thisVal zSize]} \
      {
       set infoStr "Full Z-Axis Size of Input Spectrum"
       set newVal $zSize
      } \
   elseif {![strcasecmp $thisVal aSize]} \
      {
       set infoStr "Full A-Axis Size of Input Spectrum"
       set newVal $aSize
      } \
   elseif {![strcasecmp $thisVal xSizeFn]} \
      {
       set infoStr "X-Axis Size of Function Region"
       set newVal $xSizeFn
      } \
   elseif {![strcasecmp $thisVal ySizeFn]} \
      {
       set infoStr "Y-Axis Size of Function Region"
       set newVal $ySizeFn
      } \
   elseif {![strcasecmp $thisVal zSizeFn]} \
      {
       set infoStr "Z-Axis Size of Function Region"
       set newVal $zSizeFn
      } \
   elseif {![strcasecmp $thisVal aSizeFn]} \
      {
       set infoStr "A-Axis Size of Function Region"
       set newVal $aSizeFn
      } \
   elseif {![strcasecmp $thisVal xSizeVal]} \
      {
       set infoStr "X-Axis Size of Value Region"
       set newVal $xSizeVal
      } \
   elseif {![strcasecmp $thisVal ySizeVal]} \
      {
       set infoStr "Y-Axis Size of Value Region"
       set newVal $ySizeVal
      } \
   elseif {![strcasecmp $thisVal zSizeVal]} \
      {
       set infoStr "Z-Axis Size of Value Region"
       set newVal $zSizeVal
      } \
   elseif {![strcasecmp $thisVal aSizeVal]} \
      {
       set infoStr "A-Axis Size of Value Region"
       set newVal $aSizeVal
      } \
   else \
      {
       set valFn  [lindex $valFnList $loc]
       set newVal [$valFn $valBuff $valN] 

       set infoStr "Function $valFn valBuff $valN"
      }

   if {$verbose} \
      {
       puts "NormSpec.  Parameter Value Type:              $infoStr"
       puts "NormSpec.  Value Region Limits:               $valReg"
       puts "NormSpec.  Value Region Point Coordinates:    $coordListVal"
       puts "NormSpec.  Value:                             $newVal"
       puts ""
      }

   return $newVal
}

proc updateFnRegion {} \
{
   global fdata rdata fnBuff fnReg fnN n sizeList coordListFn dimCount fnCount verbose
   global xSize ySize zSize aSize xSizeFn ySizeFn zSizeFn aSizeFn

   set coordListFn [getPointCoords $fdata $fnReg]
   set fnN         [getThisRegion $fnBuff $rdata $sizeList $coordListFn]

   set xSizeFn 1
   set ySizeFn 1
   set zSizeFn 1
   set aSizeFn 1

   if {$dimCount >= 1} \
      {
       set ix1 [lindex $coordListFn 0]
       set ixn [lindex $coordListFn 1]

       if {$ix1 > $ixn} \
          {
           set itmp $ix1
           set ix1  $ixn
           set ixn  $itmp
          }

       set xSizeFn [expr 1 + $ixn - $ix1]
      }

   if {$dimCount >= 2} \
      {
       set iy1 [lindex $coordListFn 2]
       set iyn [lindex $coordListFn 3]

       if {$iy1 > $iyn} \
          {
           set itmp $iy1
           set iy1  $iyn
           set iyn  $itmp
          }

       set ySizeFn [expr 1 + $iyn - $iy1]
      }

   if {$dimCount >= 3} \
      {
       set iz1 [lindex $coordListFn 4]
       set izn [lindex $coordListFn 5]

       if {$iz1 > $izn} \
          {
           set itmp $iz1
           set iz1  $izn
           set izn  $itmp
          }

       set zSizeFn [expr 1 + $izn - $iz1]
      }

   if {$dimCount >= 4} \
      {
       set ia1 [lindex $coordListFn 6]
       set ian [lindex $coordListFn 7]

       if {$ia1 > $ian} \
          {
           set itmp $ia1
           set ia1  $ian
           set ian  $itmp
          }

       set aSizeFn [expr 1 + $ian - $ia1]
      }

   if {$verbose} \
      {
       if {$fnCount} \
          {
           puts "NormSpec.  Function Region Limits:            $fnReg"
           puts "NormSpec.  Function Region Point Coordinates: $coordListFn ($fnN Points)"
          } \
       else \
          {
           puts "NormSpec. Function Region Limits:             $fnReg"
           puts "NormSpec. Function Region Point Coordinates:  $coordListFn ($fnN Points)"
           puts ""
          }
      }
}

proc updateValRegion {} \
{
   global fdata rdata valBuff valReg valN n coordListVal dimCount fnCount verbose sizeList
   global xSize ySize zSize aSize xSizeVal ySizeVal zSizeVal aSizeVal

   set coordListVal [getPointCoords $fdata $valReg]
   set valN         [getThisRegion $valBuff $rdata $sizeList $coordListVal]

   set xSizeVal 1
   set ySizeVal 1
   set zSizeVal 1
   set aSizeVal 1

   if {$dimCount >= 1} \
      {
       set ix1 [lindex $coordListVal 0]
       set ixn [lindex $coordListVal 1]

       if {$ix1 > $ixn} \
          {
           set itmp $ix1
           set ix1  $ixn
           set ixn  $itmp
          }

       set xSizeVal [expr 1 + $ixn - $ix1]
      }

   if {$dimCount >= 2} \
      {
       set iy1 [lindex $coordListVal 2]
       set iyn [lindex $coordListVal 3]

       if {$iy1 > $iyn} \
          {
           set itmp $iy1
           set iy1  $iyn
           set iyn  $itmp
          }

       set ySizeVal [expr 1 + $iyn - $iy1]
      }

   if {$dimCount >= 3} \
      {
       set iz1 [lindex $coordListVal 4]
       set izn [lindex $coordListVal 5]

       if {$iz1 > $izn} \
          {
           set itmp $iz1
           set iz1  $izn
           set izn  $itmp
          }

       set zSizeVal [expr 1 + $izn - $iz1]
      }

   if {$dimCount >= 4} \
      {
       set ia1 [lindex $coordListVal 6]
       set ian [lindex $coordListVal 7]

       if {$ia1 > $ian} \
          {
           set itmp $ia1
           set ia1  $ian
           set ian  $itmp
          }

       set aSizeVal [expr 1 + $ian - $ia1]
      }

   if {$verbose} \
      {
       if {$fnCount} \
          {
           puts "NormSpec.  Value Region Limits:               $valReg"
           puts "NormSpec.  Value Region Point Coordinates:    $coordListVal ($valN Points)"
           puts ""
          } \
       else \
          {
           puts "NormSpec. Value Region Limits:                $valReg"
           puts "NormSpec. Value Region Point Coordinates:     $coordListVal ($valN Points)"
           puts ""
          }
      }
}

proc applyFn { thisFn argInfo } \
{
   global fdata rdata fnBuff fnReg fnN fnCount coordListFn n sizeList dimCount knownFnList vecFnList verbose
   global xSize ySize zSize aSize xSizeFn ySizeFn zSizeFn aSizeFn

   set loc [lsearch -exact [string tolower $knownFnList] [string tolower $thisFn]]

   if {$loc < 0} \
      {
       puts "NormSpec. Error: Unknown Function $thisFn"
       exit 1
      }

   set vecFn     [lindex $vecFnList $loc]
   set vecInfo   [split $vecFn ","]

   set vecPrefix [lindex $vecInfo 0]
   set vecCall   [lindex $vecInfo 1]
   set thisVal   [lindex $argInfo 0]

   set infoStr $thisFn

   set minValInit [vMin $fnBuff $fnN]
   set maxValInit [vMax $fnBuff $fnN]

   if {![strcasecmp $vecPrefix v]} \
      {
       v$vecCall $fnBuff $fnN
       set infoStr "v$vecCall fnBuff $fnN"
      } \
   elseif {![strcasecmp $vecPrefix vs]} \
      {
       vs$vecCall $fnBuff $thisVal $fnN
       set infoStr "vs$vecCall fnBuff $thisVal $fnN"
      } \
   else \
      {
       if {![strcasecmp $vecCall vLabel]} \
          {
           if {$dimCount == 1} \
              {
               vLabel $fnBuff $xSizeFn
               set infoStr "vLabel fnBuff $xSizeFn"
              } \
           elseif {$dimCount == 2} \
              {
               vLabel $fnBuff $xSizeFn $ySizeFn
               set infoStr "vLabel fnBuff $xSizeFn $ySizeFn"
              } \
           elseif {$dimCount == 3} \
              {
               vLabel $fnBuff $xSizeFn $ySizeFn $zSizeFn
               set infoStr "vLabel fnBuff $xSizeFn $ySizeFn $zSizeFn"
              } \
           elseif {$dimCount == 4} \
              {
               vLabel $fnBuff $xSizeFn $ySizeFn $zSizeFn $aSizeFn
               set infoStr "vLabel fnBuff $xSizeFn $ySizeFn $zSizeFn $aSizeFn"
              }
          } \
       elseif {![strcasecmp $vecCall vBin]} \
          {
           set thisInfo [split $thisVal ","]

           if {[llength $thisInfo] < $dimCount} \
              {
               puts "NormSpec. Error: Binning for ${dimCount}D Data Needs $dimCount Values."
               exit 1
              }

           if {$dimCount == 1} \
              {
               set newVal [join [lrange $thisInfo 1 end] ","]

               eval "vBin -ptr $fnBuff -xSize $xSizeFn [expandArg $newVal] -xBinSize [lindex $thisInfo 0]"
               set infoStr "vBin -ptr fnBuff -xSize $xSizeFn [expandArg $newVal] -xBinSize [lindex $thisInfo 0]"
              } \
           else \
              {
               set newVal [join [lrange $thisInfo 2 end] ","]

               eval "vBin -ptr $fnBuff -xSize $xSizeFn -ySize $ySizeFn [expandArg $newVal] -xBinSize [lindex $thisInfo 0] -yBinSize [lindex $thisInfo 1]"
               set infoStr "vBin -ptr fnBuff -xSize $xSizeFn -ySize $ySizeFn [expandArg $newVal] -xBinSize [lindex $thisInfo 0] -yBinSize [lindex $thisInfo 1]"
              }
          } \
       elseif {![strcasecmp $vecCall vImage]} \
          {
           if {$dimCount == 1} \
              {
               eval "vImage -src $fnBuff -xSize $xSizeFn -ySize 1 [expandArg $thisVal]"
               set infoStr "vImage -src fnBuff -xSize $xSizeFn [expandArg $thisVal]"
              } \
           else \
              {
               eval "vImage -src $fnBuff -xSize $xSizeFn -ySize $ySizeFn [expandArg $thisVal]"
               set infoStr "vImage -src fnBuff -xSize $xSizeFn -ySize $ySizeFn [expandArg $thisVal]"
              }
          } \
       elseif {![strcasecmp $vecCall hole]} \
          {
           eval "vHole -src $fnBuff -fdata $fdata -xSize $xSizeFn -ySize $ySizeFn [expandArg $thisVal]"
           set infoStr "vHole -src $fnBuff -fdata $fdata -xSize $xSizeFn -ySize $ySizeFn [expandArg $thisVal]"
          } \
       elseif {![strcasecmp $vecCall plateau]} \
          {
           eval "vPlateau -src $fnBuff -fdata $fdata -xSize $xSizeFn -ySize $ySizeFn [expandArg $thisVal]"
           set infoStr "vPlateau -src $fnBuff -fdata $fdata -xSize $xSizeFn -ySize $ySizeFn [expandArg $thisVal]"
          } \
       else \
          {
           puts "NormSpec. Error: Unknown Function $thisFn"
           exit 1
          }
      }
 
   putThisRegion $fnBuff $rdata $sizeList $coordListFn
 
   set minValFinal [vMin $fnBuff $fnN]
   set maxValFinal [vMax $fnBuff $fnN]

   set fmt "% .4e"

   if {$verbose} \
      {
       puts "NormSpec.  Function $fnCount Command:                $infoStr"
       puts "NormSpec.  Region Min and Max Before:        [format $fmt $minValInit] [format $fmt $maxValInit]"
       puts "NormSpec.  Region Min and Max After          [format $fmt $minValFinal] [format $fmt $maxValFinal]"
       puts ""
      }
}

proc saveSpectrum { fdata rdata sizeList dimCount outName } \
{
   global FDATASIZE

   set fmtCount [getFmtCount $outName]

   set xSize 1
   set ySize 1
   set zSize 1
   set aSize 1

   if {$dimCount >= 1} {set xSize [lindex $sizeList 0]}
   if {$dimCount >= 2} {set ySize [lindex $sizeList 1]}
   if {$dimCount >= 3} {set zSize [lindex $sizeList 2]}
   if {$dimCount >= 4} {set aSize [lindex $sizeList 3]}

   if {$fmtCount == 0} \
      {
       set n [expr $xSize*$ySize*$zSize*$aSize]

       if {$dimCount >= 3} \
          {
           setParm $fdata FDPIPEFLAG 1 NULL_DIM
          } \
       else \
          {
           setParm $fdata FDPIPEFLAG 0 NULL_DIM
          }

       setParm $fdata FDCUBEFLAG 0 NULL_DIM

       set error [catch {set outUnit [sysOpen $outName w]}]

       if {$error} \
          {
           puts "NormSpec. Error Opening Output File $outName"
           return $error 
          }

       sysWrite $outUnit $fdata $FDATASIZE
       sysWrite $outUnit $rdata $n

       catch "sysClose $outUnit"
      } \
   elseif {$fmtCount == 1 && $dimCount == 3} \
      {
       set n    [expr $xSize*$ySize]
       set rPtr $rdata

       setParm $fdata FDPIPEFLAG 0 NULL_DIM
       setParm $fdata FDCUBEFLAG 0 NULL_DIM

       for {set iz 1} {$iz <= $zSize} {incr iz} \
          {
           set thisName [format $outName $iz]

           set error [catch {set outUnit [sysOpen $thisName w]}]

           if {$error} \
              {
               puts "NormSpec. Error Opening Output File $thisName"
               return $error
              }

           sysWrite $outUnit $fdata $FDATASIZE
           sysWrite $outUnit $rPtr  $n

           catch "sysClose $outUnit"

           set rPtr [incrPtr $rPtr $n]
          }
      } \
   elseif {$fmtCount == 1 && $dimCount == 4} \
      {
       set n    [expr $xSize*$ySize*$zSize]
       set rPtr $rdata

       setParm $fdata FDPIPEFLAG 0 NULL_DIM
       setParm $fdata FDCUBEFLAG 1 NULL_DIM

       for {set ia 1} {$ia <= $aSize} {incr ia} \
          {
           set thisName [format $outName $ia]

           set error [catch {set outUnit [sysOpen $thisName w]}]

           if {$error} \
              {
               puts "NormSpec. Error Opening Output File $thisName"
               return $error
              }


           sysWrite $outUnit $fdata $FDATASIZE
           sysWrite $outUnit $rPtr  $n

           catch "sysClose $outUnit"

           set rPtr [incrPtr $rPtr $n]
          }
      } \
   elseif {$fmtCount == 2 && $dimCount == 4} \
      {
       set n    [expr $xSize*$ySize]
       set rPtr $rdata

       setParm $fdata FDPIPEFLAG 0 NULL_DIM
       setParm $fdata FDCUBEFLAG 0 NULL_DIM

       for {set ia 1} {$ia <= $aSize} {incr ia} \
          {
           for {set iz 1} {$iz <= $zSize} {incr iz} \
              {
               set thisName [format $outName $ia $iz]

               set error [catch {set outUnit [sysOpen $thisName w]}]

               if {$error} \
                  {
                   puts "NormSpec. Error Opening Output File $thisName"
                   return $error
                  }

               sysWrite $outUnit $fdata $FDATASIZE
               sysWrite $outUnit $rPtr  $n

               catch "sysClose $outUnit"

               set rPtr [incrPtr $rPtr $n]
              }
          }
      } \
   else \
      {
       puts "NormSpec. Error: output name/dimension count mismatch: $outName $dimCount"
       return 1
      }

   return 0
}

set progInfoOrig $progInfo
set threshOrig   $thresh
set noiseOrig    $noise

set skipList "$skipList ,"
set progInfo ""

foreach origItem $progInfoOrig \
   {
    set item [string trim $origItem ","]
    if {[strcasecmp $item None] && ([lsearch -exact $skipList [string tolower $item]] < 0)} {lappend progInfo $item}
   }

set coordListFn  None
set coordListVal None

set fnCount   0
set dimCount  0

set sizeList  1

set xSizeFn   1
set ySizeFn   1
set zSizeFn   1
set aSizeFn   1

set xSize     1
set ySize     1
set zSize     1
set aSize     1

set xSizeVal  1
set ySizeVal  1
set zSizeVal  1
set aSizeVal  1

set k 0

foreach inName $inList \
   {
    if {$verbose} {puts ""}

    set thisName [format $inName 1 1]

    if {![file exists $thisName]} \
       {
        puts "NormSpec. Error Finding Input $inName ($thisName)"
        exit 1
       }

    set noise  $noiseOrig
    set thresh $threshOrig

    set outName [lindex $outList $k]

    set error [rdFDATA $inName fdata]

    set sizeInfo [getSpecSizeInfo $inName]

    set totalSize [lindex $sizeInfo 0]
    set dimCount  [lindex $sizeInfo 1]

    set qSize     [lindex $sizeInfo 2]
    set xSize     [lindex $sizeInfo 3]
    set ySize     [lindex $sizeInfo 4]
    set zSize     [lindex $sizeInfo 5]
    set aSize     [lindex $sizeInfo 6]

    set xSize     [expr $qSize*$xSize]

    if {$dimCount == 2 && $ySize == 1} {set dimCount 1}
    if {$dimCount == 3 && $zSize == 1} {set dimCount 2}
    if {$dimCount == 4 && $aSize == 1} {set dimCount 3}

    if {$dimCount == 1} \
       {
        set fnReg  "0% 100%"
        set valReg "0% 100%"

        set sizeList "$xSize"
        
        readROI -roi 1 -ndim 1 -in $inName -x X_AXIS 0% 100%
       } \
    elseif {$dimCount == 2} \
       {
        set fnReg  "0% 100% 0% 100%"
        set valReg "0% 100% 0% 100%"

        set sizeList "$xSize $ySize"

        readROI -roi 1 -ndim 2 -in $inName -x X_AXIS 0% 100% -y Y_AXIS 0% 100%
       } \
    elseif {$dimCount == 3} \
       {
        set fnReg  "0% 100% 0% 100% 0% 100%"
        set valReg "0% 100% 0% 100% 0% 100%"

        set sizeList "$xSize $ySize $zSize"

        readROI -roi 1 -ndim 3 -in $inName -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -z Z_AXIS 0% 100%
       } \
    elseif {$dimCount == 4} \
       {
        set fnReg  "0% 100% 0% 100% 0% 100% 0% 100%"
        set valReg "0% 100% 0% 100% 0% 100% 0% 100%"

        set sizeList "$xSize $ySize $zSize $aSize"

        readROI -roi 1 -ndim 4 -in $inName -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -z Z_AXIS 0% 100% -z A_AXIS 0% 100%
       } \
    else \
       {
        puts "NormSpec. Error: Unsupported Dimension Count $dimCount"
        exit 1
       }

    set rdata     0
    set thisFDATA 0
    set xSize     0
    set ySize     0
    set zSize     0
    set aSize     0

    set n [getROIInfoVals 1 rdata thisFDATA xSize ySize zSize aSize]

    set fnBuff  [sysMalloc $n float]
    set valBuff [sysMalloc $n float]

    if {![strcasecmp $noise  Auto]} {set noise  [vEstNoise $rdata $n]}
    if {![strcasecmp $thresh Auto]} {set thresh [expr $factor*$noise]}

    if {$verbose} \
       {
        puts "NormSpec. Input Spectrum:                     $inName"
        puts "NormSpec. Input Sizes:                        $sizeList ($n Points)"
        puts "NormSpec. Current Noise:                      $noise"
        puts "NormSpec. Current Thresh:                     $thresh"
        puts ""
       }

    set itemCount [llength $progInfo]
    set fnCount   0

    updateFnRegion
    updateValRegion

    for {set i 0} {$i < $itemCount} {incr i} \
       {
        set nextArg [lindex $progInfo [expr $i + 1]]

        incr fnCount

        set thisFn  [string tolower [lindex $progInfo $i]]
        set thisVal None

        if {$verbose} \
           {
            puts "NormSpec. Parsing Function $fnCount:                 $thisFn"
           }

        if {![strcasecmp $nextArg region]} \
           {
            if {$dimCount == 1} \
               {
                set fnReg [lrange $progInfo [expr $i + 2] [expr $i + 3]]
                incr i 3
               } \
            elseif {$dimCount == 2} \
               {
                set fnReg [lrange $progInfo [expr $i + 2] [expr $i + 5]]
                incr i 5
               } \
            elseif {$dimCount == 3} \
               {
                set fnReg [lrange $progInfo [expr $i + 2] [expr $i + 7]]
                incr i 7
               } \
            elseif {$dimCount == 4} \
               {
                set fnReg [lrange $progInfo [expr $i + 2] [expr $i + 9]]
                incr i 9
               } \

            if {[llength $fnReg] != 2*$dimCount} \
               {
                puts "NormSpec. Error Parsing Function Region Values: $fnReg"
                exit 1
               }

            updateFnRegion
           }

        set argCount [getFnArgCount $thisFn]
        set argInfo  ""

        if {$argCount < 0} \
           {
            puts "NormSpec. Error: Unsupported Function $thisFn"
            exit 1
           }

        for {set j 0} {$j < $argCount} {incr j} \
           {
            incr i

            set thisVal [lindex $progInfo $i]
            set nextArg [lindex $progInfo [expr $i + 1]]

            if {![strcasecmp $nextArg region]} \
               {
                if {$dimCount == 1} \
                   {
                    set valReg [lrange $progInfo [expr $i + 2] [expr $i + 3]]
                    incr i 3
                   } \
                elseif {$dimCount == 2} \
                   {
                    set valReg [lrange $progInfo [expr $i + 2] [expr $i + 5]]
                    incr i 5
                   } \
                elseif {$dimCount == 3} \
                   {
                    set valReg [lrange $progInfo [expr $i + 2] [expr $i + 7]]
                    incr i 7
                   } \
                elseif {$dimCount == 4} \
                   {
                    set valReg [lrange $progInfo [expr $i + 2] [expr $i + 9]]
                    incr i 9
                   } \
    
                if {[llength $valReg] != 2*$dimCount} \
                   {
                    puts "NormSpec. Error Parsing Value Region Values: $valReg"
                    exit 1
                   }

                updateValRegion
               }

            set nextVal [getFnVal $thisFn $thisVal]

            if {![strcasecmp $nextVal error]} \
               {
                puts "NormSpec. Error Finding Value for $thisVal"
                exit 1
               }

            lappend argInfo $nextVal
           } 

        applyFn $thisFn $argInfo
       }

    if {$verbose} \
       {
        puts "NormSpec. Output Spectrum:                   $outName"
        puts "NormSpec. Output Size:                       $sizeList ($n Points)"
       }

    if {![strcasecmp $outName Auto]} \
       {
        if {![file exists $outDir]}  {catch "file mkdir $outDir"}

        set thisName $outDir/$inName
       } \
    else \
       {
        set thisName $outName
       }

    set thisDir [file dirname $thisName]

    if {![file exists $thisDir]} {catch "file mkdir $thisDir"}

    set errror [saveSpectrum $fdata $rdata $sizeList $dimCount $thisName]

    if {$error} \
       {
        puts "NormSpec. Error Saving Output File $thisName"
        exit 1
       }

    sysFree $fdata 
    sysFree $fnBuff
    sysFree $valBuff

    incr k
   }

exit 0
