#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Sort/Distribute Non-Uniformly Sampled (NUS) Vectors in Bruker ser Data:"
    puts " -in     inName   \[ser\]      Bruker ser Input Name, or Keyword stdin"
    puts " -out    outName  \[ser_full\] Adjusted ser Output Name, or Keyword stdout." 
    puts " -prof   profName \[Auto\]     Sampling Schedule Output Name." 
    puts " -verb                       Verbose Mode ON (Default)."
    puts " -noverb                     Verbose Mode OFF."
    puts "Parameter File Inputs:"
    puts " -sample sName    \[vclist\]   Sample Point Input Name."
    puts " -acqus  a1Name   \[acqus\]    Dimension 1 Parameter File."
    puts " -acqu2s a2Name   \[acqu2s\]   Dimension 2 Parameter File."
    puts "Sizes of Input ser File:"
    puts " -xSize  xSize    \[Auto\]     Vector Size of ser Input, Complex Points."
    puts " -ySize  ySize    \[Auto\]     Number of Complex Vectors in ser Input."
    puts "Sizes of Sorted, Expanded Output ser File:"
    puts " -ndim   ndim     \[Auto\]     Dimension Count of Output, 2-4."
    puts " -xT     xT       \[Auto\]     X-Size of Output, Complex Points."
    puts " -yT     yT       \[Auto\]     Y-Size of Output, Complex Points."
    puts " -zT     zT       \[Auto\]     Z-Size of Output, Complex Points."
    puts " -aT     aT       \[Auto\]     A-Size of Output, Complex Points."
    exit 1
   }

set inName     ser
set outName    ser_full
set sampleName vclist
set profName   Auto
set a1Name     acqus
set a2Name     acqu2s

set xSize      Auto
set ySize      Auto

set xT         Auto
set yT         Auto
set zT         Auto
set aT         Auto

set dimCount   Auto

set revFlag    0
set verbose    1

getArgD $ARGV -in     inName
getArgD $ARGV -out    outName
getArgD $ARGV -prof   profName
getArgD $ARGV -sample sampleName

getArgD $ARGV -acqus  a1Name
getArgD $ARGV -acqu2s a2Name

getArgD $ARGV -xSize  xSize
getArgD $ARGV -ySize  ySize

getArgD $ARGV -xT     xT
getArgD $ARGV -yT     yT
getArgD $ARGV -zT     zT
getArgD $ARGV -aT     aT

if {[flagLoc $ARGV -rev]}   {set revFlag 1}
if {[flagLoc $ARGV -norev]} {set revFlag 0}

set loc [flagLoc $ARGV -verb]

if {$loc} \
   {
    set nextFlag [nextFlag $ARGV $loc]
    set verbose  1

    if {$nextFlag == $loc + 2} \
       {
        getArgD $ARGV -verb verbose
       }
   }

if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#--------------------------------------------------------------------------

proc getSampleParm { } \
{
    global inName xSize ySize xT acqus acqu2s a1Name a2Name verbose

    if {[file exists $a1Name]} {rdJCAMP $a1Name acqus}
    if {[file exists $a2Name]} {rdJCAMP $a2Name acqu2s}

    if {[info exists acqus(TD)]} \
       {
        set xSize [expr int($acqus(TD)/2)]
        if {$verbose} {puts stderr "NUS2SER. Input Vector Size:  $xSize complex points." }
       }

    if {[info exists acqu2s(TD)]} \
       {
        set ySize [expr int($acqu2s(TD))]
        if {$verbose} {puts stderr "NUS2SER. Input Vector Count: $ySize complex vectors." }
       }

    if {![strcasecmp $xSize Auto] || ![strcasecmp $ySize Auto]} \
       {
        puts stderr "NUS2SER Error Extracting Input Sizes from $a1Name and $a2Name"
        exit 1
       }

    if {![strcasecmp $xT Auto]} {set xT $xSize}
 
    if {[strcasecmp $inName stdin] && [file exists $inName]} \
       {
        set thisSize [file size $inName]
        set predSize [expr $xSize*2*4*$ySize]

        if {$verbose} \
           {
            puts stderr [format "NUS2SER. Size of Input: %d bytes." $thisSize]
            puts stderr [format "NUS2SER. Expected Size: %d bytes." $predSize]
           }
       }
 
    return 0 
}

#
# Interpret sample scheme:

proc getSampleSchedule { } \
{
   global xSize ySize sList yInfo zInfo aInfo locInfo sampleName dimCount verbose
   global qxSize qySize qzSize qaSize xT yT zT aT revFlag

   set sList ""

   if {![strcasecmp $sampleName None]} \
      {
       if {![strcasecmp $dimCount Auto]} \
          {
           if {$verbose} {puts stderr "NUS2SER. No Sampling Schedule, Assuming 2D Data."}
           set dimCount 2
          }

       if {![strcasecmp $xT Auto]} {set xT $xSize}
       if {![strcasecmp $yT Auto]} {set yT $ySize}
       if {![strcasecmp $zT Auto]} {set zT 1}
       if {![strcasecmp $aT Auto]} {set aT 1}

       set nusDim [expr $dimCount - 1]

       for {set ia 0} {$ia < $aT} {incr ia} \
          {
           for {set iz 0} {$iz < $zT} {incr iz} \
              {
               for {set iy 0} {$iy < $yT} {incr iy} \
                  {
                   set iList [join [lrange "$iy $iz $ia" 0 [expr $nusDim - 1]] ","]
                   lappend sList $iList
                  }
              }
          }

       return
      }

   if {![file exists $sampleName]} \
      {
       puts stderr "NUS2SER Notice: No Sampling Input File."
       exit 1 
      }

   set prevN 0
   set n     0
   set str   ""

   set mxSize 0
   set mySize 0
   set mzSize 0
   set maSize 0
  
   set error [catch {set sUnit [open $sampleName r]}]

   if {$error} \
      {
       puts stderr "NUS2SER Error Opening Sampling Input File $sampleName"
       exit 1 
      }

   while {[gets $sUnit line] >= 0} \
      {
       if {![strcasecmp $dimCount Auto]} \
          {
           set nI [llength $line]

           if {$nI == 2} \
              {
               set dimCount 3
              } \
           elseif {$nI == 3} \
              {
               set dimCount 4
              } \
           else \
              {
               set dimCount 2
              }

           if {$verbose} {puts stderr "NUS2SER. Output Dimensions: $dimCount"}
          }

       if {$revFlag} {set line [lreverse [string trim $line]]}

       set k [expr int(-1*[llength $str])]
 
       append str " $line"
       incr n

       set nusDim [expr $dimCount - 1]

       while {[llength $str] >= $nusDim} \
          {
           set iList [lrange $str 0 [expr $nusDim - 1]]
           set str   [lrange $str $nusDim end]
           set j 1

           lappend sList [join $iList ","]

           foreach item $iList \
              {
               incr j
               incr k

               if {![string is integer $item]} \
                  {
                   puts -nonewline stderr "NUS2SER Warning: Bad Sample Value, Near"

                   if {$k} \
                      {
                       puts stderr " Line $n Item $k: $item"
                      } \
                   else \
                      {
                       puts stderr " Line [expr $n - 1] Item [expr $prevN - $k]: $item"
                      }
                  } \
               else \
                  {
                   if {$item >= 0} \
                      {
                       if {$j == 2} {set mySize [getMax "$mySize $item"]}
                       if {$j == 3} {set mzSize [getMax "$mzSize $item"]}
                       if {$j == 4} {set maSize [getMax "$maSize $item"]}
                      }
                  }
              }
          }

       set prevN [llength $line]
      }

   close $sUnit

   if {$dimCount < 1 || $dimCount > 4} \
      {
       puts stderr "NUS2SER Error: Unsupported Dimension Count $dimCount."
       exit 1
      }

   incr mySize
   incr mzSize
   incr maSize

   set sSize [llength $sList]

   if {![strcasecmp $xT Auto]} {set xT $xSize}
   if {![strcasecmp $yT Auto]} {set yT $mySize}
   if {![strcasecmp $zT Auto]} {set zT $mzSize}
   if {![strcasecmp $aT Auto]} {set aT $maSize}

   if {$dimCount >= 1} {set qxSize 2}
   if {$dimCount >= 2} {set qySize 2}
   if {$dimCount >= 3} {set qzSize 2}
   if {$dimCount >= 4} {set qaSize 2}

   if {$xT < 1} {set xT 1}
   if {$yT < 1} {set yT 1}
   if {$zT < 1} {set zT 1}
   if {$aT < 1} {set aT 1}

   if {$dimCount < 4} {set aT 1}
   if {$dimCount < 3} {set zT 1}
   if {$dimCount < 2} {set yT 1}

   if {$verbose} \
      {
       puts stderr "NUS2SER. Samples:           $sSize"

       if {$dimCount >= 1} {puts stderr "NUS2SER. Output X-Size:     $xSize"}
       if {$dimCount >= 2} {puts stderr "NUS2SER. Output Y-Size:     $yT"}
       if {$dimCount >= 3} {puts stderr "NUS2SER. Output Z-Size:     $zT"}
       if {$dimCount >= 4} {puts stderr "NUS2SER. Output A-Size:     $aT"}

       set nS  [expr $yT*$zT*$aT*$qySize*$qzSize*$qaSize]
       set pct 0.0

       if {$nS} {set pct [expr 100.0*$ySize/$nS]}

       puts stderr [format "NUS2SER  Sampling Density:  %.1f%%" $pct]
      }

   if {$sSize*4 < $ySize} \
      {
       puts stderr "NUS2SER Warning: Sample Count is Smaller than Data Size."
      }

   if {$sSize*4 > $ySize} \
      {
       puts stderr "NUS2SER Warning: Sample Count is Larger than Data Size."
      }

   for {set ia 0} {$ia < $aT} {incr ia} \
      {
       for {set iz 0} {$iz < $zT} {incr iz} \
          {
           set yInfo($ia,$iz) ""
          }
      }
  
   for {set ia 0} {$ia < $aT} {incr ia} \
      {
       for {set iy 0} {$iy < $yT} {incr iy} \
          {
           set zInfo($ia,$iy) ""
          }
      }

   for {set iz 0} {$iz < $zT} {incr iz} \
      {
       for {set iy 0} {$iy < $yT} {incr iy} \
          {
           set aInfo($iz,$iy) ""
          }
      }

   set iy 0
   set iz 0
   set ia 0

   foreach s $sList \
      {
       set iList [split $s ","]

       if {$nusDim > 0} {set iy [lindex $iList 0]}
       if {$nusDim > 1} {set iz [lindex $iList 1]}
       if {$nusDim > 2} {set ia [lindex $iList 2]}

       if {$iy >= 0} {lappend yInfo($ia,$iz) $iy}
       if {$iz >= 0} {lappend zInfo($ia,$iy) $iz}
       if {$ia >= 0} {lappend aInfo($iz,$iy) $ia}
      }

   set i 0

   for {set ia 0} {$ia < $aT} {incr ia} \
      {
       for {set iz 0} {$iz < $zT} {incr iz} \
          {
           for {set iy 0} {$iy < $yT} {incr iy} \
              {
               set locInfo($i) "$iy $iz $ia"
               incr i
              }
          }
      }

   return 
}

proc getNUSOffset { ia iaq iz izq iy loc } \
{
   global xSize xT yT zT aT qxSize qySize qzSize qaSize locInfo dimCount 

   set off [expr $loc*$xSize*$qxSize*$qySize*$qzSize]

   if {$dimCount == 3} \
      {
       if {$izq == 1} {incr off [expr $xSize*$qxSize*$qySize]}
      } \
   elseif {$dimCount == 4} \
      {
       if {$iaq == 1} {incr off [expr $xSize*$qxSize*$qySize*$qzSize]}
       if {$izq == 1} {incr off [expr $xSize*$qxSize*$qySize]}
      }

   return $off
}

proc getSER {} \
{
   global inName xSize ySize serBuff zeroBuff nTotal nZero verbose

   set serBuff   0
   set zeroBuff  0

   set nTotal    0
   set nZero     0

   if {![strcasecmp $xSize Auto] || ![strcasecmp $ySize Auto]} \
      {
       puts stderr "NUS2SER Error: Missing -xSize or -ySize"
       exit 1 
      }

   set nTotal [expr $xSize*2*$ySize]
   set nZero  [expr $xSize*4]

   set error [catch {set serBuff [sysMalloc $nTotal int]}]

   if {$error} \
      {
       puts stderr "NUS2SER Error Allocating Input Buffer."
       exit 1
      }

   set error [catch {set zeroBuff [sysMalloc $nZero int]}]

   if {$error} \
      {
       puts stderr "NUS2SER Error Allocating Zero Buffer."
       exit 1
      }

   for {set i 0} {$i < $nZero} {incr i} {setPtrVal $zeroBuff $i 0 int}

   if {![strcasecmp inName stdin]} \
      {
       if {$verbose} {puts stderr "NUS2SER. Reading $nTotal Points from Input Stream ..."}

       set inUnit 0

       sysReadB $inUnit $serBuff $nTotal
      } \
   else \
      {
       if {![file exists $inName]} \
          {
           puts stderr "NUS2SER Error Finding Input File $inName"
           exit 1
          }

       if {$verbose} {puts stderr "NUS2SER. Reading $nTotal Points from Input File ..."}

       set inUnit [sysOpen $inName r]

       sysRead  $inUnit $serBuff $nTotal
       sysClose $inUnit
      }
}

proc saveProf {} \
{
   global profName fdataP FDATASIZE profBuff nProf xT yT zT aT dimCount verbose
   global yInfo zInfo aInfo

   if {![strcasecmp $profName None]} {return 0}

   set nusDim [expr $dimCount - 1]

   if {![strcasecmp $profName Auto]} \
      {
       if {$dimCount == 2} \
          {
           set profName profY.dat
          } \
       elseif {$dimCount == 3} \
          {
           set profName profYZ.dat
          } \
       elseif {$dimCount == 4} \
          {
           set profName prof/prof%03d.dat
          } \
       else \
          {
           set profName None
           return
          }
      }

   if {$verbose} {puts stderr "NUS2SER. Creating ${nusDim}D Sampling Kernel $profName ..."}

   set error [catch {set profBuff [sysMalloc $yT        float]}]
   set error [catch {set fdataP   [sysMalloc $FDATASIZE float]}]

   set nProf $yT

   set thisName [format $profName 1]
   set dirName  [file dirname $thisName]

   if {![file exists $dirName]} {catch {file mkdir $dirName}}
   
   if {![file isdirectory $dirName]} \
      {
       puts stderr "NUS2SER Error Finding Sampling Schedule Output Directory $dirName"
       exit 1
      }

   switch -exact $nusDim \
      {
       1 {
          buildHdr $fdataP -ndim $nusDim -aq2D Real \
             -xMODE Real -xFT Time -xN  $yT -xT $yT -xSW $yT -xOBS 1. 0 -xCAR 0.0 -xLAB Y_INC  
         }

       2 {
          buildHdr $fdataP -ndim $nusDim -aq2D Real \
             -xMODE Real -xFT Time -xN  $yT -xT $yT -xSW $yT -xOBS 1. 0 -xCAR 0.0 -xLAB Y_INC \
             -yMODE Real -yFT Time -yN  $zT -yT $zT -ySW $zT -yOBS 1. 0 -yCAR 0.0 -yLAB Z_INC  
         }

       3 {
          buildHdr $fdataP -ndim $nusDim -aq2D Real \
             -xMODE Real -xFT Time -xN  $yT -xT $yT -xSW $yT -xOBS 1. 0 -xCAR 0.0 -xLAB Y_INC \
             -yMODE Real -yFT Time -yN  $zT -yT $zT -ySW $zT -yOBS 1. 0 -yCAR 0.0 -yLAB Z_INC \
             -zMODE Real -zFT Time -zN  $aT -zT $aT -zSW $aT -yOBS 1. 0 -aCAR 0.0 -zLAB A_INC  
         }
      }

   setParm $fdataP FDMIN       0.0 NULL_DIM
   setParm $fdataP FDMAX       1.0 NULL_DIM
   setParm $fdataP FDSCALEFLAG 1.0 NULL_DIM

   for {set ia 0} {$ia < $aT} {incr ia} \
      {
       set thisName [format $profName [expr $ia + 1]]
       set outUnit  [sysOpen $thisName w]

       if {$error} \
          {
           puts stderr "NUS2SER Error Opening Sampling Schedule Output File $thisName"
           exit 1
          }

       sysWrite $outUnit $fdataP $FDATASIZE

       for {set iz 0} {$iz < $zT} {incr iz} \
          {
           vsSet $profBuff 0.0 $nProf

           foreach iy $yInfo($ia,$iz) \
             {
              setPtrVal $profBuff $iy 1.0
             }

           sysWrite $outUnit $profBuff $nProf
          }

       sysClose $outUnit
      }
}

#
#------------------------------------------------------------------------------

set sList     "" 

set fdataP    0
set serBuff   0
set zeroBuff  0
set profBuff  0

set qxSize    1
set qySize    1
set qzSize    1
set qaSize    1

set nTotal    0
set nZero     0
set nProf     0

set outUnit   1

getSampleParm 
getSampleSchedule
getSER
saveProf

#
# Write the data in sorted order:

if {![strcasecmp $outName None]} {exit 0} 

if {![strcasecmp $outName stdout]} \
   {
    set outUnit 1
   } \
else \
   {
    if {[file exists $outName]} {catch "file delete $outName"}

    set error [catch {set outUnit [sysOpen $outName w]}]

    if {$error} \
       {
        puts stderr "NUS2SER Error Opening Output File $outName"
        exit 1
       }
   }

set nusDim     [expr $dimCount - 1]
set vSize      [expr $xSize*$qxSize*$qySize]
set planeCount [expr $zT*$qzSize*$aT*$qaSize]
set maxOff     [expr $nTotal - $vSize]

set maxUsed 0
set nBad    0
set iip     0
 
for {set ia 0} {$ia < $aT} {incr ia} \
   {
for {set iaq 0} {$iaq < $qaSize} {incr iaq} \
   {
    for {set iz 0} {$iz < $zT} {incr iz} \
       {
    for {set izq 0} {$izq < $qzSize} {incr izq} \
       {
        incr iip 

        if {$verbose == 1} {puts -nonewline stderr "Plane $iip of $planeCount\015"}

        for {set iy 0} {$iy < $yT} {incr iy} \
           {
            set iList  [join [lrange "$iy $iz $ia" 0 [expr $nusDim - 1]] ","]
            set loc    [lsearch -exact $sList $iList]
            set src    $zeroBuff
            set srcStr zero

            if {$loc >= 0} \
               {
                set off [getNUSOffset $ia $iaq $iz $izq $iy $loc] 

                if {$off > $maxOff} \
                   {
                    puts stderr ""
                    puts stderr "NUS2SER Warning: Bad Sample:" 
                    puts stderr " Y: $iy Z: $iz $izq A: $ia $iaq Sample: $loc Src Offset: $srcStr"
                    puts stderr ""

                    set srcStr error 
                    incr nBad
                   } \
                else \
                   {
                    set src [incrPtr $serBuff $off]
                    set srcStr $off
                   }
               }

            if {$off > $maxUsed} {set maxUsed $off}

            if {$verbose > 1} \
               {
                if {$dimCount == 2} \
                   {
                    puts stderr " Y: $iy Sample: $loc Src Offset: $srcStr"
                   } \
                elseif {$dimCount == 3} \
                   {
                    puts stderr " Y: $iy Z: $iz $izq Sample: $loc Src Offset: $srcStr"
                   } \
                else \
                   {
                    puts stderr " Y: $iy Z: $iz $izq A: $ia $iaq Sample: $loc Src Offset: $srcStr"
                   }
               }


            sysWrite $outUnit $src $vSize
           }
       }
       }
    }
    }

if {$verbose == 1} {puts stderr ""}
if {$verbose >= 1} {puts stderr "NUS2SER: Maximum Allowed Offset: $maxOff: Used: $maxUsed"}

if {$nBad} {puts stderr "NUS2SER Warning: $nBad Bad Sample Locations."}


if {![strcasecmp $outName stdout]} {catch "sysClose $outUnit"}

exit 0
