#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2020.088.18.20"

if {[flagLoc $ARGV -help]} \
   {
    puts "Generate a Proper NMRPipe Peak Table from Basic Peak Parameters:"
    puts " -in        parmTab \[shift.tab\] Input Table of Chemical Shifts."
    puts " -out       pkTab   \[peak.tab\]  Output Peak Table."
    puts " -verb                          Verbose Mode ON (Default)."
    puts " -noverb                        Verbose Mode OFF."
    puts "Other Inputs:"
    puts " -fid       fidName \[test.fid\]  Corresponding Time-Domain Data."
    puts " -ft        ftName  \[test.ft2\]  Corresponding Spectrum."
    puts "Linewidth Conversion (Use -x -y ...):"
    puts " -mxw       mxwVal  \[1.0\]       Scale Factor for Linewidth."
    puts " -dxw       dxwVal  \[0.0\]       Offset for Linewidth."
    puts "Peak Region Additional Widths (For X1 X3 Y1 Y3 etc. Use -x -y...):"
    puts " -mx        mxVal   \[1.0\]       X-Axis Width Multiplier."
    puts " -dx        dxVal   \[1\]         X-Axis Peak Region +/- Width (Pts Hz ppm %)." 
    puts "Default Parameter Values:"
    puts " -def       defList \[None\]      List of Parameter/Value Pairs."
    puts "Notes:"
    puts " 1. The minimum required input table parameters are the chemical shifts:"
    puts "      X_PPM Y_PPM Z_PPM A_PPM"
    puts ""
    puts " 2. Optional linewidths are in Hz, and will be zero by default:"
    puts "      XW_HZ YW_HZ ZW_HZ AW_HZ"
    puts ""
    puts " 3. Other optional axis-specific parameters are (X... Y... Z... A...):"
    puts "      X_P0 ... X_COSJ1_HZ ... X_COSJ4_HZ ..."
    puts ""
    puts " 4. Other optional parameters in the input table:"
    puts "      INDEX CLUSTID ASS HEIGHT"
    puts ""
    puts " 5. The values for linewidths will be adjusted according to the"
    puts "    values for -mxw -dxw and linewidth: dxw + mxw*linewidth."
    puts ""
    puts " 6. The values for peak regions X1 X3 will be set according to the"
    puts "    values for -dx -mx and linewidth: +/-(dx + mx*linewidth)."
    puts ""
    puts " 7. Use the -def option to det default values for parameters that"
    puts "    are not given in the input table, for example:"
    puts ""
    puts "       -def XW_HZ 22.0 YW_HZ 5.0 HEIGHT 100.0"
    exit 0
   }

set parmTabName   shift.tab
set pkTabName     peak.tab
set fidScriptName fid.com
set fidScriptName None
 
set ftName        Auto
set fidName       Auto
set defList       None
set verbose       1

set noiseVal      Auto
set noiseVal      0.0 

getArgD $ARGV -in        parmTabName
getArgD $ARGV -out       pkTabName
getArgD $ARGV -fidScript fidScriptName
getArgD $ARGV -fid       fidName
getArgD $ARGV -ft        ftName
getArgD $ARGV -noise     noiseVal

foreach dim {X Y Z A} \
   {
    set regW($dim)  1
    set offLW($dim) 0.0

    getArgD $ARGV -d[string tolower $dim]  regW($dim)
    getArgD $ARGV -d[string tolower $dim]w offLW($dim)
   }

foreach dim {X Y Z A} \
   {
    set multW($dim)  1.0
    set multLW($dim) 1.0

    getArgD $ARGV -m[string tolower $dim]  multW($dim)
    getArgD $ARGV -m[string tolower $dim]w multLW($dim)
   }

getListArgD $ARGV -def defList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {![strcasecmp $defList None]} {set defList ""}

#
#---------------------------------------------------------------------------------------------------------------------------------------------

proc showMsg { msg } \
{
   global verbose

   if {!$verbose} return

   puts stderr [string trim $msg "{}"]
}

proc extractScriptCmnd { inName cmndList } \
{
   set cmnd ""

   set error [catch {set inUnit [open $inName r]}]
   if {$error} {return $cmnd}

   while {[gets $inUnit lineOrig] >= 0} \
      {
       set line [string trim $lineOrig]
       set key  [lindex $line 0]

       if {[lsearch -exact $cmndList $key] < 0} continue

       set cmnd  [string trim $line "\\"]
       set lastC [string index $line end]

       while {![strcmp $lastC "\\"]} \
          {
           if {[gets $inUnit lineOrig] < 0} break 
          
           set line [string trim $lineOrig]

           append cmnd " " 
           append cmnd [string trim $line "\\"]

           set lastC   [string index $lineOrig end]
          }

       catch "close $inUnit"

       return $cmnd
      }

   catch "close $inUnit"

   return $cmnd
}

if {[llength $defList] % 2} \
   {
    puts stderr "Parm2Peak. Error with -def Option: an Even Number of Items are Required."
    exit 1
   }

if {![file exists $parmTabName]} \
   {
    puts stderr "Parm2Peak. Error Finding Parameter Table Input $parmTabName"
    exit 1
   }

set parmTab [gdbRead table -in $parmTabName]
set eList   [gdbGetInfo $parmTab -parms entries]
set vList   [gdbGetInfo $parmTab -parms varNames]
set vCount  [llength $vList]

set dimCount 0

foreach dim {X Y Z A} \
   {
    if {[lsearch -exact $vList ${dim}_PPM] < 0} break
    incr dimCount
   }

if {$dimCount == 0} \
   {
    puts stderr "Parm2Peak. No Required Variables X_PPM ... in Input Table $parmTabName"
    exit 1
   }

if {$dimCount > 4} {set dimCount 4}

set dimList [lrange "X Y Z A" 0 [expr $dimCount - 1]]

if {![strcasecmp $ftName Auto]} \
   {
    if {$dimCount >= 3} \
       {
        set ftName  ft/test%03d.ft$dimCount
       } \
    else \
       {
        set ftName  test.ft$dimCount
       }
   }

if {![strcasecmp $fidName Auto]} \
   {
    if {$dimCount >= 3} \
       {
        set fidName fid/test%03d.fid
       } \
    else \
       {
        set fidName test.fid
       }
   }

set thisFIDName [format $fidName 1 1]
set thisFTName  [format $ftName  1 1]

if {![file exists $thisFIDName]} \
   {
    puts stderr "Parm2Peak. Error Finding Input data $fidName ($thisFIDName)."
    exit 1
   }

set error [rdFDATA $thisFIDName fdataT]

if {$error} \
   {
    puts stderr "Parm2Peak. Error Reading Header in Input data $fidName ($thisFIDName)."
    exit 1
   }

if {![file exists $thisFTName]} \
   {
    puts stderr "Parm2Peak. Error Finding Input Spectrum $ftName ($thisFTName)."
    exit 1
   }

set error [rdFDATA $thisFTName fdataF]

if {$error} \
   {
    puts stderr "Parm2Peak. Error Reading Header in Input data $ftName ($thisFTName)."
    exit 1
   }

set inVars   ""
set thisList ""

foreach v "INDEX" \
   {
    if {[lsearch -exact $vList $v] >= 0} \
       {
        lappend thisList $v
        lappend inVars   $v
       }
   }

if {[llength $thisList]} {showMsg "Parm2Peak. Initial Input Variables:    $thisList"}

foreach dim $dimList \
   {
    set thisList ""

    foreach v "_PPM W_HZ _P0 1 3 _COSJ1_HZ _COSJ2_HZ _COSJ3_HZ _COSJ4_HZ" \
       {
        if {[lsearch -exact $vList $dim$v] >= 0} \
           {
            lappend thisList $dim$v
            lappend inVars   $dim$v
           }
       }

    if {[llength $thisList]} {showMsg "Parm2Peak. ${dim}-Axis Input Variables:     $thisList"}
   }

set thisList ""

foreach v "HEIGHT DHEIGHT VOL PCHI2 TYPE ASS CLUSTID" \
   {
    if {[lsearch -exact $vList $v] >= 0} \
       {
        lappend thisList $v
        lappend inVars   $v
       }
   }

set evList ""

for {set i 0} {$i <= $vCount} {incr i} \
   {
    set v Z_A$i

    if {[lsearch -exact $vList $v] >= 0} \
       {
        lappend evList $v
       }
   }

if {[llength $thisList]} {showMsg "Parm2Peak. General Input Variables:    $thisList"}

if {[llength $evList]} \
   {
    showMsg "Parm2Peak. Evolution Variables:        $evList"
   } \
else \
   {
    showMsg "Parm2Peak. Evolution Variables:        None"
   }

set nPk  [llength $eList]
set nDef [llength $defList]

set thisList "" 

for {set i 0} {$i < $nDef} {incr i 2} \
   {
    set v   [lindex $defList $i]
    set val [lindex $defList [expr $i + 1]]

    lappend thisList $v

    set defInfo($v) $val
   }

if {[llength $thisList]} {showMsg "Parm2Peak. User Default Values:        $thisList"}

set thisList ""

if {![info exists defInfo(INDEX)]} \
   {
    lappend thisList INDEX
    set defInfo(INDEX) Auto
   }

foreach dim $dimList \
   {
    set v ${dim}W_HZ

    if {![info exists defInfo($v)]} \
       {
        lappend thisList $v
        set defInfo($v) 0.0
       }
   }

foreach vv {HEIGHT=100.0 DEHIGHT=0.0 VOL=Auto PCHI=0.0 TYPE=1 ASS=None CLUSTID=Auto MEMCNT=Auto} \
   {
    set v   [lindex [split $vv "="] 0]
    set val [lindex [split $vv "="] 1]

    if {![info exists defInfo($v)]} \
       {
        lappend thisList $v
        set defInfo($v) $val
       }
   }

if {[llength $thisList]} {showMsg "Parm2Peak. Program Adjusted Variables: $thisList"}

if {![strcasecmp $fidScriptName None]} \
   {
    set fidCmnd ""
   } \
else \
   {
    if {![file exists $fidScriptName]} \
       {
        puts stderr "Parm2Peak. Error Finding Input Script $fidScriptName"
        exit 1
       }

    set fidCmnd [extractScriptCmnd $fidScriptName "bruk2pipe var2pipe"]

    if {![llength $fidCmnd]} \
       {
        puts stderr "Parm2Peak. Error Finding Command in Input Script $fidScriptName"
        exit 1
       }
   }

if {![strcasecmp $noiseVal Auto]} {set noiseVal [estSpecNoise $ftName]}

set error [catch {set outUnit [open $pkTabName w]}]

if {$error} \
   {
    puts "Parm2Peak. Error Creating Peak Table $pkTabName"
    exit 1
   }

#
# INDEX X_PPM X_AXIS X_HZ DX XW_HZ XW X1 X3 HEIGHT DHEIGHT VOL PCHI2 TYPE ASS CLUSTID MEMCNT

puts -nonewline $outUnit "VARS  "
puts -nonewline $outUnit [format " %-9s" INDEX]

foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" ${dim}_PPM]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" ${dim}_AXIS]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" ${dim}_HZ]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" D${dim}]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" ${dim}W_HZ]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" ${dim}W]}

foreach dim $dimList \
   {
    foreach i {1 2 3 4} \
       {
        set v ${dim}_COSJ${i}_HZ 

        if {[lsearch -exact $inVars $v] >= 0 || [info exists defInfo($v)]} \
           {
            puts -nonewline $outUnit [format " %-9s" ${dim}_COSJ${i}]
           }
       }
   }

foreach dim $dimList {puts -nonewline $outUnit [format " %-9s %-9s" ${dim}1 ${dim}3]}

foreach v "HEIGHT DHEIGHT VOL PCHI2 TYPE ASS CLUSTID MEMCNT" {puts -nonewline $outUnit [format " %-9s" $v]}
foreach v $evList {puts -nonewline $outUnit [format " %-9s" $v]}

puts $outUnit ""

puts -nonewline $outUnit "FORMAT"
puts -nonewline $outUnit [format " %-9s" %5d]

foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" %9.4f]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" %8.3f]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" %8.3f]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" %8.3f]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" %8.3f]}
foreach dim $dimList {puts -nonewline $outUnit [format " %-9s" %8.3f]}

foreach dim $dimList \
   {
    foreach i {1 2 3 4} \
       {
        set v ${dim}_COSJ${i}_HZ 

        if {[lsearch -exact $inVars $v] >= 0 || [info exists defInfo($v)]} \
           {
            puts -nonewline $outUnit [format " %-9s" %8.3f]
           }
       }
   }

foreach dim $dimList {puts -nonewline $outUnit [format " %-9s %-9s" %5d %5d]}

puts -nonewline $outUnit " %e        %e        %e        %.5f      %d        %s        %4d       %4d"

foreach v $evList {puts -nonewline $outUnit "        %e"}

puts $outUnit ""
puts $outUnit ""

set id 0

foreach e $eList \
   {
    incr id

    set v INDEX

    if {[lsearch -exact $inVars $v] < 0} \
       {
        set val $id
       } \
    else \
       {
        set val [gdbGet $e $v] 
       }

    puts -nonewline $outUnit [format " %5d" $val]

#
# X_PPM X_AXIS X_HZ DX XW_HZ XW X1 X3 

    foreach dim $dimList \
       {
        set v   ${dim}_PPM
        set val [gdbGet $e $v]

        puts -nonewline $outUnit [format " %9.4f" $val]
       }

   foreach dim $dimList \
       {
        set v   ${dim}_PPM
        set val [gdbGet $e $v]

        set valPts [spec2pnt $fdataF ${dim}_AXIS ${val}ppm]

        set locPts($dim) $valPts

        puts -nonewline $outUnit [format " %8.3f" $valPts]
       }

   foreach dim $dimList \
       {
        set v   ${dim}_PPM
        set val [gdbGet $e $v]

        set valPts [spec2pnt $fdataF ${dim}_AXIS ${val}ppm]
        set valHz  [pnt2spec $fdataF ${dim}_AXIS ${val} Hz]

        puts -nonewline $outUnit [format " %8.3f" $valHz]
       }

   foreach dim $dimList \
       {
        set v D${dim}

        if {[lsearch -exact $inVars $v] >= 0} \
           {
            set val [gdbGet $e $v]
           } \
        elseif {[info exists defInfo($v)]} \
           {
            set val $defInfo($v)
           } \
        else \
           {
            set val 0.1
           }

        puts -nonewline $outUnit [format " %8.3f" $val]
       }

   foreach dim $dimList \
       {
        set v ${dim}W_HZ

        if {[lsearch -exact $inVars $v] >= 0} \
           {    
            set val [gdbGet $e $v]
           } \
        elseif {[info exists defInfo($v)]} \
           {
            set val $defInfo($v) 
           } \
        else \
           {
            set val 0.0
           }

        set val [expr $offLW($dim) + $multLW($dim)*$val]
 
        puts -nonewline $outUnit [format " %8.3f" $val]
       }

   foreach dim $dimList \
       {
        set v ${dim}W_HZ

        if {[lsearch -exact $inVars $v] >= 0} \
           {
            set val [gdbGet $e $v]
           } \
        elseif {[info exists defInfo($v)]} \
           {
            set val $defInfo($v)
           } \
        else \
           {
            set val 0.0
           }

        set valPts [spec2pntW $fdataF ${dim}_AXIS ${val}Hz]

        set lwPts($dim) $valPts

        set val    [expr $offLW($dim) + $multLW($dim)*$val]
        set valPts [spec2pntW $fdataF ${dim}_AXIS ${val}Hz]

        puts -nonewline $outUnit [format " %8.3f" $valPts]
       }

    foreach dim $dimList \
       {
        set w [expr $regW($dim) + $multW($dim)*$lwPts($dim)]

        puts -nonewline $outUnit [format " %5d" [expr int($locPts($dim) - $w)]]
        puts -nonewline $outUnit [format " %5d" [expr int($locPts($dim) + $w)]]
       }

    foreach dim $dimList \
       {
        foreach i {1 2 3 4} \
           {
            set v ${dim}_COSJ${i}_HZ
    
            if {[lsearch -exact $inVars $v] >= 0} \
               {
                set val    [gdbGet $e $v]
                set valPts [spec2pntW $fdataT X_AXIS ${val}Hz]

                puts -nonewline $outUnit [format " %8.3f" $valPts]
               } \
            elseif {[info exists defInfo($v)]} \
               {
                set val    $defInfo($v)
                set valPts [spec2pntW $fdataT X_AXIS $defInfo($v)Hz]

                puts -nonewline $outUnit [format " %8.3f" $valPts]
               }
           }
       }

    set v HEIGHT
   
    if {[lsearch -exact $inVars $v] >= 0} \
       {
        set val [gdbGet $e $v]
       } \
    elseif {[info exists defInfo($v)]} \
       {
        set val $defInfo($v)
       } \
    else \
       {
        set val 100.0 
       } 

    set thisHi $val

    puts -nonewline $outUnit [format " % e" $val]

    set v DHEIGHT

    if {[lsearch -exact $inVars $v] >= 0} \
       {
        set val [gdbGet $e $v]
       } \
    elseif {[info exists defInfo($v)]} \
       {
        set val $defInfo($v) 
       } \
    else \
       {
        set val 0.0
       } 

    if {![strcasecmp $val Auto]} {set val 0.0}

    puts -nonewline $outUnit [format " % e" $val]

    set v VOL

    if {[lsearch -exact $inVars $v] >= 0} \
       {
        set val [gdbGet $e $v]
       } \
    elseif {[info exists defInfo($v)]} \
       {
        set val $defInfo($v)
       } \
    else \
       {
        set val $thisHi 
       }

    if {![strcasecmp $val Auto]} \
       {
        set val $thisHi 

        foreach dim $dimList \
           {
            if {$lwPts($dim) > 0.0} {set val [expr $val*$lwPts($dim)]}
           }
       }

    puts -nonewline $outUnit [format " % e" $val]

    set v PCHI2

    if {[lsearch -exact $inVars $v] >= 0} \
       {
        set val [gdbGet $e $v]
       } \
    elseif {[info exists defInfo($v)]} \
       {
        set val $defInfo($v)
       } \
    else \
       {
        set val 0.0
       }

    puts -nonewline $outUnit [format " %.5f" $val]

    set v TYPE

    if {[lsearch -exact $inVars $v] >= 0} \
       {
        set val [gdbGet $e $v]
       } \
    elseif {[info exists defInfo($v)]} \
       {
        set val $defInfo($v)
       } \
    else \
       {
        set val 1
       }

    puts -nonewline $outUnit [format " %d" $val]

    set v ASS

    if {[lsearch -exact $inVars $v] >= 0} \
       {
        set val [gdbGet $e $v]
       } \
    elseif {[info exists defInfo($v)]} \
       {
        set val $defInfo($v)
       } \
    else \
       {
        set val None
       }

    puts -nonewline $outUnit [format " %10s" $val]

    set v CLUSTID

    if {[lsearch -exact $inVars $v] >= 0} \
       {
        set val [gdbGet $e $v]
       } \
    elseif {[info exists defInfo($v)]} \
       {
        set val $defInfo($v)
       } \
    else \
       {
        set val $id
       }

    if {![strcasecmp $val Auto]} {set val $id}

    puts -nonewline $outUnit [format " %4d" $val]
    puts -nonewline $outUnit [format " %4d" 1]

    foreach v $evList {puts -nonewline $outUnit [format " % e" [gdbGet $e $v]]}

    puts $outUnit ""
   }

catch "close $outUnit"

exit 0
