#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set pcaDir    pca001
set ftName    test%05d.dat
set extraDir  extra001
set extraList None
set tStr      PCA

set dbgFlag   0
set verbose   1

getArgD $ARGV -inDir   pcaDir
getArgD $ARGV -outDir  extraDir
getArgD $ARGV -ftName  ftName

getListArgD $ARGV -extra extraList
getListArgD $ARGV -title tStr

if {[flagLoc $ARGV -dbg]}    {set dbgFlag 1}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -help]} \
   {
    puts "Add Spectra to an Existing PCA Result (2022.166.15.06):"
    puts " -inDir  pcaDir    \[pca001\]    Input Directory of Existing PCA Result."
    puts " -outDir extraDir  \[extra001\]  Output PCA Directory with New Spectra Added."
    puts " -extra  extraList \[None\]      List of Spectra to Add to PCA Result."
    puts " -verb                         Verbose Mode ON (Default)."
    puts " -noverb                       Verbose Mode OFF."
    puts "Notes:"
    puts " 1. The new spectra will automatically be interpolated to match the size and"
    puts "    spectral window of the existing PCA result."
    puts " 2. No normalization will be applied to the new spectra."
    puts " 3. The original scores and loadings are retained."
    puts " 4. Scores for the new spectra are computed using the existing loading spectra."
    puts " 5. Since the original scores are retained, the new scores will no longer have"
    puts "    a total variance of 1."
    puts " 6. Display results with: specView.tcl -pcaInit extra001"
    exit 0
   }

#
#-----------------------------------------------------------------------------

if {![file isdirectory $pcaDir]} \
   {
    puts stderr "pcaExtra. Error finding input PCA directory $pcaDir"
    exit 1
   }

if {![file isdirectory $pcaDir/ft]} \
   {
    puts "pcaExtra. Error finding input PCA spectrum directory $pcaDir/ft"
    exit 1
   }

foreach f {pca.tab pcaCoords.tab score001.dat load.dat} \
   {
    if {![file exists $pcaDir/$f]} \
       {
        puts stderr "pcaExtra. Error finding PCA input file $pcaDir/$f"
        exit 1
       }
   }

set refName   [format $pcaDir/ft/$ftName 1]
set scoreName $pcaDir/score%03d.dat
set loadName  $pcaDir/load.dat

set error [rdFDATA $refName fdata(ref)]

if {$error} \
   {
    puts stderr "pcaExtra. Error reading header in $refName"
    exit 1
   }

set error [rdFDATA $pcaDir/load.dat fdata(load)]

if {$error} \
   {
    puts stderr "pcaExtra. Error reading header in $refName"
    exit 1
   }

set dimCount   [getParmI $fdata(ref)  FDDIMCOUNT NULL_DIM]
set specCount  [getParmI $fdata(load) NDSIZE     CUR_XDIM]
set pcCount    [getParmI $fdata(load) NDSIZE     CUR_YDIM]

set extraCount [llength $extraList]
set finalCount [expr $specCount + $extraCount]

if {$verbose} {puts "pcaExtra. specCount $specCount extraCount $extraCount pcCount $pcCount dimCount $dimCount"}

catch [file delete -force $extraDir]

catch [file mkdir $extraDir]
catch [file mkdir $extraDir/ft]
catch [file mkdir $extraDir/pca]

set t     [gdbRead table -in $pcaDir/pcaCoords.tab]
set tStr "[getTabKeyItem $t TITLE 0], $extraCount Added Spectra"

if {$dbgFlag} {puts  "pcaExtra. nmrPipe -in $pcaDir/load.dat -fn ZF -pad $extraCount $loadName -td -out $extraDir/load.dat -ov"}

catch "exec nmrPipe -in $pcaDir/load.dat -fn ZF -pad $extraCount $loadName -td -out $extraDir/load.dat -ov"

set scoreIDList ""
set loadIDList  ""

set extraIDList ""
set specIDList  ""
set pcaList     ""

set thisName $extraDir/load.dat
set thisID   1

readROI -roi $thisID -in $thisName -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%

set loadInfo [getROIInfo -roi $thisID]

set rdata(loadNew) [lindex $loadInfo 0]

incr thisID

for {set pc 1} {$pc <= $pcCount} {incr pc} \
   {
    set thisName $extraDir/load.dat

    readROI -roi $thisID -in $thisName -ndim 1 -x X_AXIS 0% 100% -dy Y_AXIS $pc 0

    set loadInfo [getROIInfo -roi $thisID]

    set rdata(load,$pc) [lindex $loadInfo 0]

    lappend loadIDList $thisID
    incr thisID
   }

if {$dimCount == 1} \
   {
    if {$dbgFlag} {puts  "pcaExtra. /bin/cp $pcaDir/pca.tab $extraDir"}

    catch "exec /bin/cp $pcaDir/pca.tab $extraDir" 

    set thisName [format $scoreName 1]

    if {$dbgFlag} {puts  "pcaExtra. /bin/cp $thisName $extraDir"}

    catch "exec /bin/cp $thisName $extraDir" 

    set xSize [getParmI $fdata(ref) NDSIZE CUR_XDIM]
    set ySize 1

    if {$dbgFlag} {puts "pcaExtra. dimCount $dimCount size $xSize"}
    
    for {set pc 1} {$pc <= $pcCount} {incr pc} \
       {
        set thisName [format $scoreName 1]

        readROI -roi $thisID -in $thisName -ndim 1 -x X_AXIS 0% 100% -dy Y_AXIS $pc 0

        set scoreInfo [getROIInfo -roi $thisID]

        set rdata(score,$pc) [lindex $scoreInfo 0]

        lappend scoreIDList $thisID
        incr thisID
       }

    set j 1

    for {set i 1} {$i <= $specCount} {incr i} \
       {
        set thisInName  [format $pcaDir/ft/$ftName   $i]
        set thisOutName [format $extraDir/ft/$ftName $j]

        lappend pcaList $thisInName

        if {$dbgFlag} {puts "pcaExtra. /bin/cp $thisInName $thisOutName"}

        catch "exec /bin/cp $thisInName $thisOutName"

        readROI -roi $thisID -in $thisOutName -ndim 1 -x X_AXIS 0% 100%

        set specInfo [getROIInfo -roi $thisID]

        set rdata(spec,$j) [lindex $specInfo 0]

        lappend specIDList $thisID

        incr thisID
        incr j
       }

    set k 1

    foreach thisInName $extraList \
       {
        lappend pcaList $thisInName

        if {![file exists $thisInName]} \
           {
            puts stderr "pcaExtra. Error finding extra spectrum input $thisInName"
            exit 1
           }

        set thisOutName [format $extraDir/ft/$ftName $j]

        if {$dbgFlag} {puts "pcaExtra. interpNMR -ref $refName -in $thisInName -out $thisOutName"}

        catch "exec interpNMR -ref $refName -in $thisInName -out $thisOutName"

        if {![file exists $thisOutName]} \
           {
            if {$dbgFlag} {puts stderr "pcaExtra. Error creating interpolated spectrum output $thisOutName"}
            exit 1
           }

        if {$dbgFlag} {puts "pcaExtra. sethdr $thisOutName -id $j"}

        catch "exec sethdr $thisOutName -id $j"

        readROI -roi $thisID -in $thisOutName -ndim 1 -x X_AXIS 0% 100% 

        set specInfo [getROIInfo -roi $thisID]
 
        set rdata(spec,$j)  [lindex $specInfo 0]
        set rdata(extra,$k) [lindex $specInfo 0]

        lappend extraIDList $j

        incr thisID
        incr j
        incr k
       }
   } \
elseif {$dimCount == 2} \
   {
    if {$dbgFlag} {puts "pcaExtra. /bin/cp $pcaDir/pca.tab $extraDir"}

    catch "exec /bin/cp pca.tab $extraDir" 

    set xSize [getParmI $fdata(ref) NDSIZE CUR_XDIM]
    set ySize [getParmI $fdata(ref) NDSIZE CUR_YDIM] 

    if {$verbose} {puts "pcaExtra. dimCount $dimCount xSize $xSize ySize $ySize"}
    
    for {set pc 1} {$pc <= $pcCount} {incr pc} \
       {
        set thisName [format $scoreName $pc]

        if {$dbgFlag} {puts  "pcaExtra. /bin/cp $thisName $extraDir"}

        catch "exec /bin/cp $thisName $extraDir" 

        readROI -roi $thisID -in $thisName -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%

        set scoreInfo [getROIInfo -roi $thisID]

        set rdata(score,$pc) [lindex $scoreInfo 0]

        lappend scoreIDList $thisID
        incr thisID
       }

    set j 1

    for {set i 1} {$i <= $specCount} {incr i} \
       {
        set thisInName  [format $pcaDir/ft/$ftName   $i]
        set thisOutName [format $extraDir/ft/$ftName $j]

        lappend pcaList $thisInName

        if {$dbgFlag} {puts "pcaExtra. /bin/cp $thisInName $thisOutName"}

        catch "exec /bin/cp $thisInName $thisOutName"

        readROI -roi $thisID -in $thisOutName -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%

        set specInfo [getROIInfo -roi $thisID]

        set rdata(spec,$j) [lindex $specInfo 0]

        lappend specIDList $thisID

        incr thisID
        incr j
       }

    set k 1

    foreach thisInName $extraList \
       {
        lappend pcaList $thisInName

        if {![file exists $thisInName]} \
           {
            puts stderr "pcaExtra. Error finding extra spectrum input $thisInName"
            exit 1
           }

        set thisOutName [format $extraDir/ft/$ftName $j]

        if {$dbgFlag} {puts "pcaExtra. interpNMR -ref $refName -in $thisInName -out $thisOutName"}

        catch "exec interpNMR -ref $refName -in $thisInName -out $thisOutName"

        if {![file exists $thisOutName]} \
           {
            puts stderr "pcaExtra. Error creating interpolated spectrum output $thisOutName"
            exit 1
           }

        if {$dbgFlag} {puts "pcaExtra. sethdr $thisOutName -id $j"}

        catch "exec sethdr $thisOutName -id $j"

        readROI -roi $thisID -in $thisOutName -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%

        set specInfo [getROIInfo -roi $thisID]
 
        set rdata(spec,$j)  [lindex $specInfo 0]
        set rdata(extra,$k) [lindex $specInfo 0]

        lappend extraIDList $j

        incr thisID
        incr j
        incr k
       }
   } \
else \
   {
    puts stderr "pcaExtra. Unsupported data dimension count $dimCount."
    exit 1
   }

set n [expr $xSize*$ySize]

set sumBuff  [sysMalloc $n float]
set workBuff [sysMalloc $n float]

for {set pc 1} {$pc <= $pcCount} {incr pc} \
   {
    if {$verbose} {puts -nonewline [format "pcaExtra. PC: %2d" $pc]}

    vsSet $sumBuff 0.0 $n

    set ratList ""

    for {set i 1} {$i <= $specCount} {incr i} \
       {
        set specMag  [vSumSq     $rdata(spec,$i)   $n]
        set scoreVal [getPtrVal  $rdata(load,$pc)  [expr $i - 1]]
        set scaleVal [vvGetScale $rdata(score,$pc) $rdata(spec,$i) $n]

        set ratio    [safeDiv $scoreVal [expr $scaleVal*$specMag]]

        vsMult $workBuff $rdata(spec,$i) $scoreVal $n]
        vvAdd  $sumBuff  $workBuff $n

        lappend ratList $ratio
       }

    set rAvg [getAvg $ratList]

    foreach i $extraIDList \
       {
        set scaleVal [vvGetScale $rdata(score,$pc) $rdata(spec,$i) $n]
        set specMag  [vSumSq $rdata(spec,$i) $n]
        set scoreVal [expr $scaleVal*$rAvg*$specMag]
        set loc      [expr ($i - 1) + ($pc - 1)*$finalCount]

        if {$verbose} {puts -nonewline [format " Score %2d: % .4e" $i $scoreVal]}

        setPtrVal $rdata(loadNew) $loc $scoreVal
       }

    if {$verbose} {puts ""}
   }

writeROI -roi 1 -out $extraDir/load.dat -ov

set    cmnd "pcaCoords.tcl -in $extraDir/ft/$ftName -pcaCoords $extraDir/pcaCoords.tab -load $extraDir/load.dat"
append cmnd " -score $extraDir/score001.dat -extraCount $extraCount -title $tStr"

if {$dbgFlag} {puts $cmnd}

catch {eval "exec $cmnd"}

if {![file exists $extraDir/pcaCoords.tab]} \
   {
    puts "pcaExtra. Error generating $extraDir/pcaCoords.tab"
    exit 1
   }

set error [catch {set outUnit [open $extraDir/pca.list w]}]

foreach thisName $pcaList {puts $outUnit $thisName}

catch "close $outUnit"

exit 0
