#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Simple Utility for 2D HN or HC Peak Detection."
    puts " -in    specName \[test.ft2\] Spectrum Input."
    puts " -out   tabName  \[test.tab\] Peak Table Output."
    puts " -xName xName    \[Auto\]     Spectrum Directly-Detected 1H Axis Name." 
    puts " -yName yName    \[Auto\]     Spectrum 15N or 13C Axis Name." 
    puts " -noise sigma    \[Auto\]     Noise for Peak Detection."
    puts " -hi    thresh   \[Auto\]     Threshold for Peak Detection."
    puts " -proj                      Projection Mode; Requires 3D Spectrum."
    exit 1
   }

set tabName  test.tab
set specName test.ft2 

set xName    Auto
set yName    Auto
set noise    Auto
set hi       Auto

set projMode 0

getArgD $ARGV -in    specName
getArgD $ARGV -out   tabName
getArgD $ARGV -xName xName
getArgD $ARGV -yName yName
getArgD $ARGV -noise noise
getArgD $ARGV -hi    hi

if {[flagLoc $ARGV -proj]}   {set projMode 1}
if {[flagLoc $ARGV -noproj]} {set projMode 0}

if {[flagLoc $ARGV -hn]}     {set specMode hn}
if {[flagLoc $ARGV -hc]}     {set specMode hc}

#
#------------------------------------------------------------------------

set fmtCount     [getFmtCount $specName]
set thisSpecName [format $specName 1 1]
set fdata 0

if {![file exists $thisSpecName]} \
   {
    puts "Peak Detection Error Finding Spectrum $thisSpecName"
    exit 1
   }

set error    [rdFDATA $thisSpecName fdata]
set specType [getSpecType $fdata]

puts "Spectrum Type:                 $specType"

if {$projMode} \
   {
    if {![strcasecmp $xName Auto]} \
       {
        set xName [getDirectDim $fdata]
        puts "Auto Mode Direct Dimension:    $xName"
       }

    if {![strcasecmp $yName Auto]} \
       {
        if {![strcasecmp $specType HN]} \
           {
            set yName [get15NDim $fdata]
            puts "Auto Mode 15N Dimension:       $yName"
           } \
        elseif {![strcasecmp $specType HC]} \
           {
            set yName [get13CDim $fdata]
            puts "Auto Mode 13C Dimension:       $yName"
           } \
        else \
           {
            set yName [getSecondDim $fdata]
            puts "Auto Mode Indirect Dimension:  $yName"
           }
       }

    if {[file exists $yName.$xName.dat]} {set thisSpecName $yName.$xName.dat}
    if {[file exists $xName.$yName.dat]} {set thisSpecName $xName.$yName.dat}
 
    puts "Projection Mode Input:         $thisSpecName"

    sysFree $fdata

    set error [rdFDATA $thisSpecName fdata]
   }

if {!$fmtCount} \
   {
    readROI -ndim 2 -roi 1 -in $thisSpecName -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -dz 1 0
   } \
else \
   {
    readROI -ndim 2 -roi 1 -in $thisSpecName -x X_AXIS 0% 100% -y Y_AXIS 0% 100%
   }

set rInfo [getROIInfo -roi 1]
set rdata [lindex $rInfo 0]
set sList [lindex $rInfo 2]
set n     [expr [lindex $sList 0] * [lindex $sList 1]]

set maxVal [vMax $rdata $n]
set minVal [vMin $rdata $n]

if {![strcasecmp $noise Auto]} \
   {
    set noise [vEstNoise $rdata $n]
    puts "Auto Mode Noise Estimate:      [format %.4e $noise]"
   }

if {![strcasecmp $hi Full]} \
   {
    set hi [expr 0.1*[getMaxAbs "$minVal $maxVal"]] 
    puts "Full Mode Detection Threshold: [format %.4e $hi]"
   }

if {![strcasecmp $hi Auto]} \
   {
    set hi [expr 7.0*$noise]
    if {$minVal >= 0.0} {set hi [expr 2.0*$hi]}
    puts "Auto Mode Detection Threshold: [format %.4e $hi]"
   }

if {[string last % $hi] > 0} \
   {
    set pct [string trim $hi "% "]
    set hi  [expr 0.01*$pct*[getMaxAbs "$minVal $maxVal"]] 
    puts "Detection Threshold at [format "%-6s " $pct%:] [format %.4e $hi]"
   }

if {[string last @ $hi] > 0} \
   {
    set hiInfo [getSpecContHi $thisSpecName $hi 10 10 7.0]
    set pct    [string trim $hi "@ "]
    set hi     [lindex $hiInfo 0]
    set str    "Threshold Fraction at"

    puts "$str [format "%-6s  " $pct%:] [format %.4e $hi]"
   }

set tabDir [file dirname $tabName]

if {![file exists $tabDir]} {file mkdir $tabDir}

set xfSize [getParmI $fdata NDSIZE CUR_XDIM]
set xtSize [getParmI $fdata NDAPOD CUR_XDIM]

set yfSize [getParmI $fdata NDSIZE CUR_YDIM]
set ytSize [getParmI $fdata NDAPOD CUR_YDIM]

set dx 1
set dy 1

if {$xfSize > 0 && $xtSize > 0} \
   {
    set dx [expr int(sqrt(double($xfSize)/$xtSize))]
    if {$dx < 1} {set dx 1}
   }

if {$yfSize > 0 && $ytSize > 0} \
   {
    set dy [expr int(sqrt(double($yfSize)/$ytSize))]
    if {$dy < 1} {set dy 1}
   }

set x1      1
set xN      $xfSize
set xInc    $xfSize 
set xExtra  $dx 
set xLast   [expr $xN + $xExtra + 1]

set y1      1
set yN      $yfSize 
set yInc    $yfSize 
set yExtra  $dy 
set yLast   [expr $yN + $yExtra + 1]

set yFirst  $y1

while {$yFirst <= 1 + $yN - $yExtra} \
   {
    set yNext [expr $yFirst+$yInc+2*$yExtra-1]
    if {$yNext > $yLast} {set yNext $yLast}

    set xFirst  $x1

while {$xFirst <= 1 + $xN - $xExtra} \
   {
    set xNext [expr $xFirst+$xInc+2*$xExtra-1]
    if {$xNext > $xLast} {set xNext $xLast}

    readROI -roi 1 \
       -ndim 2 -in $thisSpecName \
       -x X_AXIS $xFirst $xNext           \
       -y Y_AXIS $yFirst $yNext           \
       -verb

    pkFindROI -roi 1 \
      -sigma $noise -pChi 0.0001 -plus $hi -minus -$hi \
      -dx      $dx   $dy \
      -idx     $dx   $dy \
      -tol    2.00  2.00 \
      -hiAdj  1.20  1.80 \
      -lw    15.00  0.00 \
      -reject  -sinc -mask -out $tabName -verb

    set xFirst [expr 1 + $xNext - 2*$xExtra]
   }
    set yFirst [expr 1 + $yNext - 2*$yExtra]
   }

exit
