#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
  {
   puts "Detect Peaks in a 1D Spectrum Over One or More Regions:"
   puts " -in   inName    \[test.ft1\] Input Spectrum."
   puts " -out  tabName   \[test.tab\] Peak Table Output."
   puts " -tmp  tmpName   \[tmp.tab\]  Temporary Output."
   puts " -reg  regList   \[0% 100%\]  List of Analysis Region Limits (pts ppm %)."
   puts " -clust                     Group All Peaks Into One Cluster for Fitting."
   puts " -verb                      Verbose Mode ON (Default)."
   puts " -noverb                    Verbose Mode OFF."
   puts "Noise Analysis:"
   puts " -noiseX1 nx1    \[None\]     X-Axis Start of Signal-Free Region (pts ppm %)."
   puts " -noiseXN nxn    \[None\]     X-Axis End of Signal-Free Region (pts ppm %)."
   puts " -noise   sigma  \[Auto\]     Noise Value."
   puts "Peak Detection Parameters:"
   puts " -maxVal  maxVal \[Auto\]     Maximum Value in Spectrum."
   puts " -thresh  t      \[Auto\]     Peak Detection Height Threshold."
   puts " -mult    m      \[50.0\]     Multiplication Factor for -thresh Auto."
   puts " -ddx     ddx    \[Auto\]     X-Axis +/- Width of Detection Region, Pts."
   puts " -idx     idx    \[Auto\]     X-Axis +/- Width of Interpolation Region, Pts."
   puts " -tolx    tolx   \[Auto\]     Maximum Allowable X-Axis Position Change, +/- Pts."
   puts "Options for nmrWish pkFindROI Command (Should be Last on Command Line):"
   puts " -pkArgs  args   \[None\]     Arguments for pkFindROI."
   puts "Notes:"
   puts " 1. Analysis region limits are specified in pairs of values x1 xn ..."
   puts " 2. The -ddx option specifies how many points are considered when"
   puts "    testing whether a given point could be a peak because it is"
   puts "    larger than all of its neighboring points."
   puts " 3. If no noise region is specified via -noiseX1 -noiseXN options, and"
   puts "    no noise value is specified via the -noise option, the noise will be"
   puts "    estimated automatically from the distribution of spectral intensities."
   puts " 4. If -thresh Auto is used, the threshold will be set to the noise"
   puts "    value times the -mult multiplication factor."
   puts " 5. If a percent value is used for -thresh (such as -thresh 10%) then"
   puts "    the detection threshold will be set as a percentage of the highest"
   puts "    value in the entire spectrum."
   exit 1
  }

set inName     test.ft1
set tabName    test.tab
set tmpName    tmp.tab
set pkArgs     None

set threshMult 50.0
set regList    "0% 100%"
set ddx        1 
set idx        1
set tolx       2.0

set noiseX1    None
set noiseXN    None
set noiseRMS   Auto
set thresh     Auto
set maxVal     Auto

set clustFlag  0
set verbose    1

set loc [flagLoc $ARGV -pkArgs]

if {$loc > 0} \
   {
    set pkArgs [lrange $ARGV [expr $loc + 1] end]
    set ARGV   [lrange $ARGV 0 [expr $loc - 1]]
   }

if {![strcasecmp $pkArgs None]} {set pkArgs ""}

getListArgD $ARGV -reg regList

getArgD $ARGV -in  inName
getArgD $ARGV -out tabName
getArgD $ARGV -tab tabName

getArgD $ARGV -ddx  ddx
getArgD $ARGV -idx  idx
getArgD $ARGV -tolx tolx

getArgD $ARGV -noiseX1 noiseX1
getArgD $ARGV -noiseXN noiseXN

getArgD $ARGV -noise   noiseRMS
getArgD $ARGV -maxVal  maxVal
getArgD $ARGV -thresh  thresh
getArgD $ARGV -mult    threshMult 

if {[flagLoc $ARGV -clust]}   {set clustFlag 1}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {$verbose} \
   {
    set vArg "verb"
   } \
else \
   {
    set vArg "noverb"
   }

#
#--------------------------------------------------------------------------

set fdata 0

if {![file exists $inName]} \
   {
    puts stderr "1D Peak Detection. Error Finding Input Spectrum $inName"
    exit 1
   }

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "1D Peak Detection. Error Reading Header from Spectrum $inName"
    exit 1
   }

set xtSize [getParmI $fdata NDTDSIZE CUR_XDIM]
set xfSize [getParmI $fdata NDFTSIZE CUR_XDIM]

if {$xtSize > 0 && $xfSize > 0} \
   {
    set dxDef   [expr int(sqrt( 0.5*double($xfSize)/double($xtSize) ))]
    set xTolDef [expr 0.5*double($xfSize)/double($xtSize)]

    if {$dxDef < 1}     {set dxDef   1}
    if {$xTolDef < 2.0} {set xTolDef 2.0}
   } \
else \
   {
    set dxDef   1
    set xTolDef 2.0
   }

set n [llength $regList]

if {$n % 2} \
   {
    puts stderr "1D Peak Detection. Error: Region List -reg Must Have an Even Number of Values."
    puts stderr $regList
    exit 1
   }

if {![strcasecmp $ddx  Auto]} {set ddx  $dxDef}
if {![strcasecmp $idx  Auto]} {set idx  $dxDef}
if {![strcasecmp $tolx Auto]} {set tolx $xTolDef}

#
# Noise estimate:

if {![strcasecmp $noiseX1 None] || ![strcasecmp $noiseXN None]} \
   {
    if {![strcasecmp $noiseRMS Auto]} \
       {
        set noiseRMS [estSpecNoiseND $inName]
        set noiseStr "(Auto)"
       } \
    else \
       {
        set noiseStr "(User-Supplied)"
       }
   } \
else \
   {
    readROI -roi 1 -in $inName -ndim 1 -x X_AXIS $noiseX1 $noiseXN

    set noiseStr "(RMS of $noiseX1 to $noiseXN)"
    set noiseRMS [getROIStat 1 vRMS]
   }

#
# Detection threshold:

if {![strcasecmp $maxVal Auto]} \
   {
    readROI -roi 1 -in $inName -ndim 1 -x X_AXIS 0% 100%
    set maxVal [getROIStat 1 vMaxAbs]
   }

set threshStr ""

if {![strcasecmp $thresh Auto]} \
   {
    set threshStr " ($threshMult*Noise RMS)"
    set thresh    [expr $threshMult*$noiseRMS]
   }

if {![strcmp [string index $thresh end] "%"]} \
   {
    set threshStr " ($thresh of Max Value)"
    set pct       [string trim $thresh "%"]
    set thresh    [expr $maxVal*$pct/100.0]
   }

if {$verbose} \
   {
    puts "1D Peak Detection. Input:              $inName"
    puts "1D Peak Detection. Max Value:          [format %.3e $maxVal]"
    puts "1D Peak Detection. Noise RMS:          [format %.3e $noiseRMS] $noiseStr"
    puts "1D Peak Detection. Threshold:          [format %.3e $thresh]$threshStr"
    puts "1D Peak Detection. X-Axis Detection:   ddx $ddx idx $idx tolx $tolx"
    puts ""
   }

#
# Peak detection for each specified analysis region:

set regCount [expr $n/2]
set loc      0

for {set i 1} {$i <= $regCount} {incr i} \
   {
    if {$regCount == 1} \
       {
        set thisName $tabName
       } \
    else \
       {
        set thisName $tmpName
       }

    set rx1 [lindex $regList $loc]
    incr loc

    set rxn [lindex $regList $loc]
    incr loc

    readROI -roi 1 -ndim 1 -in $inName -x X_AXIS $rx1 $rxn

    if {$verbose} \
       {
        set localMin [getROIStat 1 vMin]
        set localMax [getROIStat 1 vMax]
       }

    eval pkFindROI $pkArgs -roi 1 \
         -sigma $noiseRMS -pChi 1.0e-04 -plus $thresh -minus -$thresh \
         -dx      $ddx   \
         -idx     $idx   \
         -tol     $tolx  \
         -hiAdj   1.20   \
         -lw      1.00   \
         -parent -noreject -mask -out $thisName -$vArg

    set tabInfo($i)   [gdbRead table -in $thisName]
    set thisCount($i) [gdbGetInfo $tabInfo($i) -parms entryCount]

    if {$verbose} \
       {
        puts "1D Peak Detection. Region $i Limits X:  $rx1 $rxn"
        puts "1D Peak Detection. Region $i Min:      [format %.3e $localMin]"
        puts "1D Peak Detection. Region $i Max:       [format %.3e $localMax]"
        puts "1D Peak Detection. Region $i Peaks:     $thisCount($i)"
       }
   }

#
# Join peak tables if multiple regions were analyzed:

if {$regCount > 1} \
   {
    for {set i 2} {$i <= $regCount} {incr i} \
       {
        if {!$thisCount($i)} continue
        gdbAppend -dest $tabInfo(1) -src $tabInfo($i)        
       }
   }

set totalCount [gdbGetInfo $tabInfo(1) -parms entryCount]

if {$verbose} \
   {
    puts "1D Peak Detection. Total Peaks:        $totalCount"
    puts "1D Peak Detection. Output:             $tabName"
   }

gdbWrite $tabInfo(1) $tabName

#
# Renumber peaks if multiple tables were joined:

if {$regCount > 1} \
   {
    set t     [gdbRead table -in $tabName]
    set eList [gdbGetInfo $t -parms entries]
    set n     [llength $eList]

    set i     0

    foreach e $eList \
       {
        incr i
        gdbSet $e INDEX $i
       }

    gdbWrite $t $tabName
   }

if {$clustFlag} \
   {
    set t     [gdbRead table -in $tabName]
    set eList [gdbGetInfo $t -parms entries]
    set n     [llength $eList]

    foreach e $eList \
       {
        gdbSet $e CLUSTID 1
        gdbSet $e MEMCNT  $n
       }

    gdbWrite $t $tabName
   }

if {$regCount > 1} {catch "file delete -force $tmpName"}

exit 0
