#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Process 1D Time-Domain Data to Match an Existing Spectrum (1/24/2023):"
   echo " -in    inName  [test.fid]  Input Time-Domain Data."
   echo " -out   outName [test.ft1]  Output Spectrum."
   echo " -ref   refName [ref.ft1]   Reference Input Spectrum."
   echo " -scaleTo s     [None]      Desired Max Value, Keyword None or Auto."
   echo " -calib                     Copy Chemical Shift Calibration from Reference."
   echo " -rescale                   Rescale Output to Best Match Reference."
   echo " -procArgs      [None]      Additional Options for basicFT1.com Processing."
   echo "Notes:"
   echo " 1. The output spectrum will be processed to have the same window"
   echo "    function, zero fill, phase correction, and extracted region as"
   echo "    the reference spectrum."
   echo " 2. The -calib option will copy: NDSW NDOBS NDCAR NDORIG NDCENTER NDLABEL" 
   echo " 3. Option -scaleTo Auto will scale the output to best match the reference."
   echo " 4. Use additional command-line options to override the processing"
   echo "    paremeters taken from the reference spectrum:"
   echo ""
   echo "     -xQ1 -xQ2 -xQ3 -xELB -xGLB -xGOFF -xC1 -xZFARG -xP0 -xP1 -xEXTX1 -xEXTXN"
   exit 0
endif

set procArgs = None
set loc      = (`flagLoc $argv -procArgs`)
set thisArgv = ($argv)

if ($loc >= 1) then
   @ locP = $loc - 1
   @ locN = $loc + 1

   set procArgs = ($argv[$locN-])
   set thisArgv = ($argv[1-$locP])
endif

if (`string csame $procArgs None`) then
   set procArgs = ""
endif

@ calibFlag = 0

set inName  = (`getArgD $thisArgv -in      test.fid`)
set outName = (`getArgD $thisArgv -out     test.ft1`)
set refName = (`getArgD $thisArgv -ref     ref.ft1`)
set scaleTo = (`getArgD $thisArgv -scaleTo None`)

if (`flagLoc $thisArgv -calib`) then
   @ calibFlag = 1
endif

if (`flagLoc $thisArgv -nocalib`) then
   @ calibFlag = 0
endif

if (!(-e $inName)) then
   echo "procSameAs1D. Error finding input time-domain data ($inName)."
   exit 1
endif

if (!(-e $refName)) then
   echo "procSameAs1D. Error finding input reference spectrum ($refName)."
   exit 1
endif

set xQ1    = (`getParm -in $refName -parm NDAPODQ1 -dim CUR_XDIM`)
set xQ2    = (`getParm -in $refName -parm NDAPODQ2 -dim CUR_XDIM`)
set xQ3    = (`getParm -in $refName -parm NDAPODQ3 -dim CUR_XDIM`)
set xC1    = (`getParm -in $refName -parm NDC1     -dim CUR_XDIM`)

set xELB   = (`getParm -in $refName -parm NDLB     -dim CUR_XDIM`)
set xGLB   = (`getParm -in $refName -parm NDGB     -dim CUR_XDIM`)
set xGOFF  = (`getParm -in $refName -parm NDGOFF   -dim CUR_XDIM`)

set p0     = (`getParm -in $refName -parm NDP0     -dim CUR_XDIM`)
set p1     = (`getParm -in $refName -parm NDP1     -dim CUR_XDIM`)

set x1     = (`getParm -in $refName -parm NDX1     -dim CUR_XDIM -fmt %.0f`)
set xn     = (`getParm -in $refName -parm NDXN     -dim CUR_XDIM -fmt %.0f`)
set xSize  = (`getParm -in $refName -parm NDFTSIZE -dim CUR_XDIM -fmt %.0f`)

set xZFARG = size=$xSize
set xC1    = (`MATH "1.0 + $xC1"`)

set xQ1    = (`getArgD $thisArgv -xQ1    $xQ1`)
set xQ2    = (`getArgD $thisArgv -xQ2    $xQ2`)
set xQ3    = (`getArgD $thisArgv -xQ3    $xQ3`)
set xC1    = (`getArgD $thisArgv -xC1    $xC1`)

set xELB   = (`getArgD $thisArgv -xELB   $xELB`)
set xGLB   = (`getArgD $thisArgv -xGLB   $xGLB`)
set xGOFF  = (`getArgD $thisArgv -xGOFF  $xGOFF`)
set xZFARG = (`getArgD $thisArgv -xZFARG $xZFARG`)

set p0     = (`getArgD $thisArgv -xP0    $p0`)
set p1     = (`getArgD $thisArgv -xP1    $p1`)

set x1     = (`getArgD $thisArgv -xEXTX1 $x1`)
set xn     = (`getArgD $thisArgv -xEXTXN $xn`)

set cmnd   = (basicFT1.com -in $inName -out $outName)
set cmnd   = ($cmnd $procArgs -xQ1 $xQ1 -xQ2 $xQ2 -xQ3 $xQ3 -xELB $xELB -xGLB $xGLB -xGOFF $xGOFF -xC1 $xC1 -xZFARG $xZFARG -xP0 $p0 -xP1 $p1)

if (`string notsame $x1 0` || `string notsame $xn 0`) then
   set cmnd = ($cmnd -xEXTX1 $x1 -xEXTXN $xn)
endif

echo "procSameAs1D. Reference:    $refName"
echo "procSameAs1D. Command:      $cmnd"

echo ""
$cmnd

if ($calibFlag) then
   set sw     = (`getParm -in $refName -parm NDSW     -dim CUR_XDIM`)
   set obs    = (`getParm -in $refName -parm NDOBS    -dim CUR_XDIM`)
   set car    = (`getParm -in $refName -parm NDCAR    -dim CUR_XDIM`)
   set orig   = (`getParm -in $refName -parm NDORIG   -dim CUR_XDIM`)
   set center = (`getParm -in $refName -parm NDCENTER -dim CUR_XDIM -fmt %.0f`)
   set lab    = (`getParm -in $refName -parm NDLABEL  -dim CUR_XDIM -fmt %s -text`)

   echo "procSameAs1D. Calibration:  -xSW $sw -xOBS $obs -xCAR $car -xORIG $orig -xCENTER $center -xLAB $lab"

   sethdr $outName -xSW $sw -xOBS $obs -xCAR $car -xORIG $orig -xCENTER $center -xLAB $lab 
endif

if (`string csame $scaleTo Auto`) then
   set sInfo = (`findScale.tcl -in1 $refName -in2 $outName -out $outName`)
   echo "procSameAs1D. Scaling:      $sInfo (Auto Mode)."
else if (`string csame $scaleTo None`) then
   echo "procSameAs1D. Scaling:      None"
else
   echo "procSameAs1D. Scaling:      $scaleTo"
   rescale.com -in $outName -out $outName -max $scaleTo
endif

