
#
# procpar.tcl: modules to read Varian procpar parameters into a parameter array.
#
#  rdProcPar: main module for reading and returning parameter values.
#  isVParm:   returns 1 if parmName is an original Varian parameter.

#-------------------------------------------------------------------------------
# Return 1 if the given parameter name is an original Varian 
# parameter, rather than a "synthetic" one created by these
# modules, for instance "ni" (original) vs "ni.type" (synthetic).

proc isVParm { parmName } \
{
   if {-1 == [string first "." $parmName]} \
      {
       return 1
      }

   return 0
}

#
# Removes quote (") character, returns count of quotes removed.

proc deQuote { oldStrPtr newStrPtr } \
{
   upvar $oldStrPtr oldStr 
   upvar $newStrPtr newStr 

   set iq     0
   set newStr ""
   set n      [string length $oldStr]

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $oldStr $i]

       if {"$c" == "\""} \
          {
           incr iq

           if {$i < [expr $n-1]} \
              {
               set c [string index $oldStr [expr $i+1]]

               if {"$c" == "\""} \
                  {
                   append newStr "(Null)"
                   incr   iq
                   incr   i
                  }
              }
          } \
       else \
          {
           append newStr $c
          }
      }

   return $iq
}

#------------------------------------------------------------------------------------
# Read parameters from Varian procpar file to a TCL array.
# Values in the array (e.g. "var(np)") are the original parameter values.
# Types  in the array (e.g. "var(np.type)")  are the Varian parameter types.
# Info   in the array (e.g. "var(np.info)")  are the Varian info lists.
# Enum   in the array (e.g. "var(np.enum)")  are the Varian enumeration codes.
# Value for parmList  (e.g. "var(parmList)") are the Varian parameter names found.
# Returns number of parameters read.
# Returns -1 on failure.
#
#  inName:  file name of input, or "stdin".
#  parmPtr: name of TCL array for storing parameters.

proc rdProcPar { inName parmPtr msgUnit } \
{
    upvar $parmPtr parmList

    set errorCount 0
    set error      0
    set inUnit     ""
    set eStr       "RdProcPar Error"

    set parmList(parmList) ""

    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        if {[strcasecmp $msgUnit None]} \
           {
            puts $msgUnit "$eStr opening input file $inName"
           }

        catch [close $inUnit]
        incr   errorCount
        return -1
       }

    set parmCount 0
    set linesLeft 1
    set lineCount 0

    while { $linesLeft } \
       {
        set  error [catch {set status [gets $inUnit line]}]
        incr lineCount

        if {![info exists status]} {set status -1}

        if {$status < 0} \
           {
            if {!$parmCount} {incr errorCount}

            catch [close $inUnit]
            return $parmCount
           }

        if {![isPParmLine $line]} {continue}

        set parmName [lindex $line 0]
        set parmInfo [lrange $line 1 end]
        set parmType [getPParmType $parmInfo]

        set  error [catch {set status [gets $inUnit str]}]
        set  iq    [deQuote str line]
        incr lineCount

        if {$status < 0} \
           {
            if {[strcasecmp $msgUnit None]} \
               {
                puts $msgUnit \
                 "$eStr reading value line from $inName near line $lineCount:"

                puts $msgUnit $line
               }

            catch  [close $inUnit]
            incr   errorCount
            return -1
           }

        set itemCount [lindex $line 0]

        if {[llength $line] == 1} \
           {
            set error [catch {set status [gets $inUnit str]}]
            set iq    [deQuote str line]
            incr lineCount

            set line "$itemCount $line" 
           }

        set parmVal [lrange $line 1 end]

        if {![string compare $parmType STRING]} \
           {
            while {$iq < [expr 2*$itemCount]} \
               {
                set  error [catch {set status [gets $inUnit str]}]
                incr iq    [deQuote str line]
                incr lineCount

                append parmVal " $line"
               }
           }

        set error [catch {set status [gets $inUnit str]}]
        set iq    [deQuote str line]
        incr lineCount

        if {$status < 0} \
           {
            if {[strcasecmp $msgUnit None]} \
               {
                puts $msgUnit \
                 "$eStr reading choice line from $inName near line $lineCount:"

                puts $msgUnit $line
               }

            catch  [close $inUnit]
            incr   errorCount
            return -1
           }

        if {![isPParmEnum $line]} \
           {
            if {[strcasecmp $msgUnit None]} \
               {
                puts $msgUnit \
                 "$eStr in choice line from $inName near line $lineCount:"

                puts $msgUnit $line
               }

            incr errorCount
            set  parmEnum 0
           } \
        else \
           {
            set parmEnum $line
           }

        lappend parmList(parmList) $parmName

        set parmList($parmName)      $parmVal
        set parmList($parmName.type) $parmType
        set parmList($parmName.info) $parmInfo
        set parmList($parmName.enum) $parmEnum

        incr parmCount

#      if {[strcasecmp $msgUnit None]} \
#         {
#          puts $msgUnit [format "%2d. %12s: %s" $parmCount $parmName $parmVal]
#         }
       }

    catch [close $inUnit]

    return $parmCount 
}

#
# Return data type of parameter entry according to its info list.

proc getPParmType { info } \
{
   switch [lindex $info 0] \
      {
       0 { return UNDEFINED }
       1 { return REAL }
       2 { return STRING }
       3 { return DELAY }
       4 { return FLAG }
       5 { return FREQUENCY }
       6 { return PULSE }
       7 { return INTEGER }
       default {return BAD }
      }

   return NONE
}

#
# Return 1 if line seems to be a valid enumerated choice line.

proc isPParmEnum { line } \
{
   set trueLen [llength $line]

   if {!$trueLen} {return 0}

   set item     [lindex $line 0]
   set givenLen -1

   if {[regexp {^([0-9]+)$} $item]} \
      {
       set givenLen [expr 1 + $item]
      }

   if {$givenLen > $trueLen} {return 0}

   return 1
}

#
# Return 1 is line seems to be a valid parameter name line.

proc isPParmLine { line } \
{
   if {![llength $line]} {return 0}

   set key [lindex $line 0]

   if {[string match \[a-z\]* $key]} {return 1}
   if {[string match \[A-Z\]* $key]} {return 1}

   return 0
}

#
# Bottom.
