#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Split Interleaved 2D NMRPipe-format Data into a Pseudo-3D Series:"
   echo " -in      inName   [test.fid]     2D Input Name."
   echo " -outDir  outDir   [fid]          Output Directory."
   echo " -outName outName  [test%03d.dat] Output Name Template."
   echo " -tau     tList    [0.0 ...]      List of Tau Values."
   echo " -grad                            Perform Gradient Shuffling."
   echo " -sign                            Perform Z-Axis Sign Adjustment."
   echo " -time                            Reduce Y-Axis Time Domain Size."
   echo " -title                           Update Spectrum Title to Include Tau."
   echo "Sorting of Associated Non-Uniform Sampling (NUS) Schedule:"
   echo " -sample  sNameIn  [None]         NUS Schedule Input."
   echo " -sorted  sNameOut [nuslist.sort] Sorted NUS Schedule Output."
   echo "Order of Interleaving Loop and Quad Detection Loop:"
   echo " -inner                           Interleaving is Inner Loop (Default)."
   echo " -outer                           Interleaving is Outer Loop."
   exit 0
endif

if ((-e pulseprogram) || (-e pulseprogram.precomp) || (-e ser) || (-e ser.Z) || (-e ser.gz)) then
   set gradName = bruk_ranceY.M
   set outDir   = (`getArgD $argv -outDir fid`)
else
   set gradName = var_ranceY.M
   set outDir   = (`getArgD $argv -outDir data`)
endif

if (`flagLoc $argv -grad`) then
   set gradFlag = 1
else
   set gradFlag = 0
endif

if (`flagLoc $argv -title`) then
   set titleFlag = 1
else
   set titleFlag = 0
endif

if (`flagLoc $argv -sign`) then
   set signFlag = 1
else
   set signFlag = 0
endif

if (`flagLoc $argv -time`) then
   set timeArg = "-time"
else
   set timeArg = ""
endif

set innerFlag = 1

if (`flagLoc $argv -inner`) then
   set innerFlag = 1
endif

if (`flagLoc $argv -outer`) then
   set innerFlag = 0
endif

set inName    = (`getArgD $argv -in      test.fid`)
set outName   = (`getArgD $argv -outName test%03d.fid`)
set schedName = (`getArgD $argv -sample  None`)
set sortName  = (`getArgD $argv -sorted  nuslist.sort`)

set tauList   = (`getListArgD $argv -tau -- 0.0`)

#
#------------------------------------------------------------------------

if (!(-e $inName)) then
   echo "Error finding 2D Input $inName"
   exit 1
endif

if (`string notcsame $schedName None`) then
   if (!(-e $schedName)) then
      echo "Error Finding NUS Schedule $schedName"
      exit 1
   endif
endif

if (!(-d $outDir)) then
   mkdir $outDir 
endif

set ne     = $#tauList
set rrList = (0 0)
set riList = (0 0)
set cList  = 0
set sList  = 1
set i      = 1

if ($signFlag == 0) then
   set k =  1
else
   set k = -1
endif

while ($i < $ne)
   set cList = ($cList 0)
   set sList = ($sList $k)

   set rrList = ($rrList 0 0)
   set riList = ($riList 0 0)

   if ($signFlag == 1) then
      if ($k == 1) then
         set k = -1
      else
         set k =  1
      endif
   endif

   @ i++
end

set nRI = $#riList

#
# Prepare sorted NUS schedule if needed.
# Extract each 2D plane.
# Update header.

if (`string notcsame $schedName None`) then
   sortSched.tcl -in $schedName -out $sortName -loc $jList -col 1
endif

echo "Creating $ne 2D planes from $inName"

if ($gradFlag) then
   echo "Gradient Sorting Macro: $NMRTXT/$gradName"
endif

set jList = (`getStat.tcl -x $tauList -stat VecSortLoc -noverb`)
set i  = 1

while ($i <= $ne)
   set j = $jList[$i]
   set s = $sList[$i]

   @ iR = 2 * $j - 1 
   @ iI = 2 * $j

   set thisName    =  (`nmrPrintf $outDir/$outName $i`)
   set cList[$j]   = 1
   set rrList[$iR] = 1
   set riList[$iI] = 1
   set tau = $tauList[$j]

  nmrPrintf " %2d. Input Plane: %2d Output Name: %s Tau: %.3e Sign: %2d\n" $i $j $thisName $tau $s

   if ($innerFlag == 1) then
      set mixArg = "COADD -cList $cList $timeArg -axis Y"
   else
      set qList = ""
      set k = 1

      foreach iQ ($rrList)
         set qList = ($qList $iQ $riList[$k])
         @ k++
      end

      set mixArg = "QMIX  -cList $qList -ic $nRI -oc 2 $timeArg"
   endif

   if ($gradFlag == 1) then
      nmrPipe -in $inName \
      | nmrPipe -fn $mixArg  \
      | nmrPipe -fn MAC -macro $NMRTXT/$gradName -noRd -noWr \
      -out $thisName -ov
   else
      nmrPipe -in $inName \
      | nmrPipe -fn $mixArg \
      -out $thisName -ov
   endif

   if ($signFlag == 1 && $s == -1) then
      nmrPipe -in $thisName \
      | nmrPipe -fn TP -hyper \
      | nmrPipe -fn SIGN -i \
      | nmrPipe -fn TP -hyper -out $thisName -inPlace -ov
   endif

   set rrList[$iR] = 0
   set riList[$iI] = 0
   set cList[$j]   = 0

   if ($titleFlag) then
      set tStr = (`getParm -in $thisName -parm FDTITLE -dim NULL_DIM -text -fmt %s`)

      if (`string length $tStr`) then
         set tStr = ${tStr}-$tauList[$j]
      else
         set tStr = (`nmrPrintf "%03d-%s" $i $tauList[$j]`)
      endif

      sethdr $thisName -tau $tauList[$j] -title $tStr
   else
      sethdr $thisName -tau $tauList[$j]
   endif

   @ i++
end

seriesT.com $outDir/*
