#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Report Statistics of 2D Regions Listed in a Table:"
   echo " -in   inName  [test.ft2]  Input Spectrum."
   echo " -tab  tabName [reg.tab]   Input Table of Region Limits."
   echo " -stat sList   [Sum RMS]   List of Statistics to Compute."
   echo " -fmt  fList   [%.4e ...]  List of Corresponding Output Formats."
   echo " -nohdr                    Do Not Display Output Header."
   echo " -brief                    Display Statistic Values Only."
   echo "Notes:"
   echo " 1. Input table requires columns X1 XN Y1 YN."
   echo " 2. Input table can have optional assignment column ASS."
   echo " 3. X1 and XN correspond to the current X-Axis of the spectrum."
   echo " 4. Y1 and YN correspond to the current Y-Axis of the spectrum."
   echo " 5. Limits can be specified with units: pts ppm Hz %"
   echo ""
   echo "Statistics Functions (Case Sensitive):"
   echo ""
   echo "    Sum Avg StdDev RMS Median Min Max MinAbs MaxAbs MinLoc MaxLoc EstNoise"
   echo ""
   echo "Example Input Table:"
   echo ""
   echo "   VARS   X1 XN Y1 YN ASS"
   echo "   FORMAT %s %s %s %s %s"
   echo ""
   echo "   10.3ppm  10.1ppm  124.4ppm 124.0ppm RegionA"
   echo "   10.0ppm   9.8ppm  124.4ppm 124.0ppm RegionB"
   echo "   10.9ppm  10.7ppm  123.5ppm 123.1ppm RegionC"
   echo "   10.8ppm  10.6ppm  122.5ppm 122.1ppm RegionE"
   echo "    9.7ppm   9.5ppm  122.3ppm 121.9ppm RegionF"
   echo "   10.2ppm  10.0ppm  120.8ppm 120.4ppm EmptyRegion"
   echo ""
   exit 0
endif

set specName = (`getArgD $argv -in  test.ft2`)
set tabName  = (`getArgD $argv -tab reg.tab`)
set fnList   = (`getListArgD $argv -stat -- Sum RMS`)

set statList = ""
set varList  = ""
set fmtList  = ""

foreach i ($fnList)
   set statList = ($statList v$i)
   set varList = ($varList `string toupper $i`)
   set fmtList = ($fmtList %.4e)
end

set varList  = (`getListArgD $argv -var  -- $varList`)
set fmtList  = (`getListArgD $argv -fmt  -- $fmtList`)

set hdrFlag   = 1
set briefFlag = 0

if (`flagLoc $argv -hdr`) then
   set hdrFlag = 1
endif

if (`flagLoc $argv -nohdr`) then
   set hdrFlag = 0
endif

if (`flagLoc $argv -brief`) then
   set briefFlag = 1
   set hdrFlag   = 0
endif

if (`flagLoc $argv -nobrief`) then
   set briefFlag = 0
endif

if (!(-e $specName)) then
   echo "Error finding input spectrum $specName"
   exit 0
endif

if (!(-e $tabName)) then
   echo "Error finding input region table $tabName"
   exit 0
endif

if ($#statList != $#fmtList || $#statList != $#varList) then
   echo "Error: mismatch in -stat / -fmt / -var options."
   exit 1
endif

set x1List  = (`getTabCol.tcl -in $tabName -var X1`)
set xnList  = (`getTabCol.tcl -in $tabName -var XN`)
set y1List  = (`getTabCol.tcl -in $tabName -var Y1`)
set ynList  = (`getTabCol.tcl -in $tabName -var YN`)
set assList = (`getTabCol.tcl -in $tabName -var ASS`)

if ($#x1List == 0 || $#xnList == 0 || $#y1List == 0 || $#ynList == 0) then
   echo "Error: Input Table is Missing X1 / XN / Y1 / YN."
   exit 1
endif

set n = $#x1List

foreach x1 ($x1List)
   set assList = ($assList None)
end

if ($hdrFlag != 0) then
   echo "VARS   INDEX X1 XN Y1 YN $varList ASS"
   echo "FORMAT %4d   %s %s %s %s $fmtList %s"
   echo ""
endif

set i = 1

while ($i <= $n)
   set statInfo = (`readROI -in $specName -ndim 2 -out None -silent -vec $statList -x X_AXIS $x1List[$i] $xnList[$i] -y Y_AXIS $y1List[$i] $ynList[$i]`)

   if ($briefFlag == 0) then
      nmrPrintf " %4d %10s %10s %10s %10s $fmtList %s\n" \
                $i $x1List[$i] $xnList[$i] $y1List[$i] $ynList[$i] $statInfo $assList[$i]
   else
      nmrPrintf "$fmtList\n" $statInfo
   endif

   @ i++
end


