#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Generate New 2D FIDs by Resampling a Collection of Replicates:"
    puts " -in      inList  \[fid/*.fid\]    List of Identically-Sized Time-Domain Inputs."
    puts " -outDir  outDir  \[resample\]     Output Directory."
    puts " -outName outName \[test%03d.fid\] Output Name." 
    puts " -iz      iz      \[1\]            Starting Index Number for Output Names."
    puts " -iseed   iseed   \[1925677\]      Random Number Seed."
    puts " -n       n       \[32\]           Number of Resampled Time-Domain Output Files."
    puts " -exact                          Use Each Increment Exactly Once (Sets -n to Input Count)."
    puts " -noexact                        Increments are Selected Randomly (Default)."
    puts " -del                            Delete Existing Output Directory (Default)."
    puts " -nodel                          Do Not Delete Existing Output Directory."
    puts " -verb                           Verbose Mode ON."
    puts " -noverb                         Verbose Mode OFF (Default)."
    exit 0
   }

set inList    [glob -nocomplain fid/*.fid]
set outDir    resample
set outName   test%03d.fid
set iseed     1925677
set outCount  32
set iz        1
set exactFlag 0
set delFlag   1
set verbose   0
 
getListArgD $ARGV -in inList

getArgD $ARGV -outDir  outDir
getArgD $ARGV -outName outName
getArgD $ARGV -iz      iz
getArgD $ARGV -n       outCount 
getArgD $ARGV -iseed   iseed

if {[flagLoc $ARGV -exact]}   {set exactFlag 1}
if {[flagLoc $ARGV -noexact]} {set exactFlag 0}

if {[flagLoc $ARGV -del]}   {set delFlag 1}
if {[flagLoc $ARGV -nodel]} {set delFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

proc getRandLocList { inCount outCount exactFlag } \
{
   set rList ""
   set iList ""

   if {$exactFlag} \
      {
       for {set j 0} {$j < $outCount} {incr j} {lappend rList [rand]}

       set kList [getVecSortLoc $rList]

       foreach k $kList {lappend iList [expr $k % $inCount]}
      } \
   else \
      {
       for {set j 0} {$j < $outCount} {incr j} \
          {
           set i [expr int($inCount*[rand])]

           if {$i > $inCount - 1} {set i [expr $inCount - 1]}

           lappend iList $i
          }
      }

   return $iList
}

set inCount [llength $inList]

if {$exactFlag} {set outCount $inCount}

for {set i 0} {$i < $inCount} {incr i} \
   {
    set thisInName [lindex $inList $i]

    if {![file exists $thisInName]} \
       {
        puts stderr "Resample Error Finding Input $thisInName"
        exit 1
       }

    set fdata($i) 0

    catch {set error [rdFDATA $thisInName fdata($i)]}

    if {$error} \
       {
        puts stderr "Resample Error Reading Header in Input $thisInName"
        exit 1
       }

    set xSize($i) [getParmI $fdata($i) NDSIZE CUR_XDIM]
    set ySize($i) [getParmI $fdata($i) NDSIZE CUR_YDIM]

    set qxSize($i) [getQuadSize [getParmI $fdata($i) NDQUADFLAG CUR_XDIM]]
    set qySize($i) [getQuadSize [getParmI $fdata($i) NDQUADFLAG CUR_YDIM]]

    if {$xSize($i) != $xSize(0) || $ySize($i) != $ySize(0)} \
       {
        puts stderr "Resample Error: Size Mismatch for $thisInName $xSize($i) $ySize($i) (Expected $xSize(0) $ySize(0))"
        exit 1
       }

    set n [expr $xSize($i)*$ySize($i)*$qxSize($i)] 

    puts stderr "$thisInName X $xSize($i) $qxSize($i) Y $ySize($i) $qySize($i) Bytes: [expr 4*($FDATASIZE + $n)]"

    set rdata($i) [sysMalloc $n float]

    set error [catch {set inUnit [sysOpen $thisInName r]}]

    set error [catch {sysRead $inUnit $fdata($i) $FDATASIZE}]
    set error [catch {sysRead $inUnit $rdata($i) $n}]

    set thisPtr $rdata($i)

    set nY    [expr $ySize($i)/2]
    set jumpY [expr $qxSize($i)*$qySize($i)*$xSize($i)]

    for {set iy 0} {$iy < $nY} {incr iy} \
       {
        set rPtr($i,$iy) $thisPtr

        set thisPtr [incrPtr $thisPtr $jumpY]
       }

    catch "sysClose $inUnit"
   }

if {$delFlag} {catch "file delete -force $outDir"}

if {![file isdirectory $outDir]} {catch "file mkdir $outDir"}

srand $iseed

set tStr  [getParmStr $fdata(0) FDTITLE NULL_DIM]

set nY    [expr $ySize(0)/$qySize(0)]
set jumpY [expr $qxSize(0)*$qySize(0)*$xSize(0)]

for {set iy 0} {$iy < $nY} {incr iy} \
   {
    set j 0

    foreach i [getRandLocList $inCount $outCount $exactFlag] \
       {
        set iInfo($j,$iy) $i
        incr j
       }
   }

for {set j 0} {$j < $outCount} {incr j} \
   {
    set thisOutName [format %s/$outName $outDir [expr $j + $iz]]

    setParmStr $fdata(0) FDTITLE ${tStr}-[format r%03d [expr $j + 1]] NULL_DIM

    set error [catch {set outUnit [sysOpen $thisOutName w]}]
    set error [catch {sysWrite $outUnit $fdata(0) $FDATASIZE}]

    for {set iy 0} {$iy < $nY} {incr iy} \
       {
        set i $iInfo($j,$iy)

        if {$verbose} {puts [format " %s %4d %4d" $thisOutName $iy $i]}

        set error [catch {sysWrite $outUnit $rPtr($i,$iy) $jumpY}]
       }

    catch "sysClose $outUnit"
   }

exit 0

