#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2023.067.12.51"

if {[flagLoc $ARGV -help]} \
   {
    puts "SAND 1D Spectral Decomposition (Script Version $thisVersion)."
    puts "Y Wu, O Sanati, M Uchimiya, K Krishnamurthy, A Edison, and F Delaglio."
    puts "Conversion of SAND Results to NMRPipe Format Table and Data:"
    puts " -in        inName    \[sand.txt\]   SAND Signal Table Input."
    puts " -fidRef    refFID    \[ref.fid\]    Reference 1D Time-Domain Data Input."
    puts " -ftRef     refFT     \[ref.ft1\]    Corresponding Reference 1D Spectrum Input."
    puts " -out       outName   \[sim.tab\]    NMRPipe-format Peak Table Output."
    puts " -fid       fidName   \[test.fid\]   Optional Output FID, or Keyword None."
    puts " -ft        ftName    \[test.ft1\]   Optional Output Spectrum, or Keyword None."
    puts " -simFIDDir simFIDDir \[simFIDAll\]  Output Subdirectory for Individual Peak Simulations, or Keyword None."
    puts " -simDir    simFTDir  \[simAll\]     Output Subdirectory of Corresponding Peak Spectra, or Keyword None."
    puts " -specID    specID    \[None\]       Optional Spectrum ID."
    puts " -lb        elbHz     \[Default\]    Exponential Broadening to Apply (Default = Same as Input Data)."
    puts " -p0        p0        \[Default\]    Zero Order Phase of Simulation  (Default = Same as Input Data)."
    puts " -p1        p1        \[Default\]    First Order Phase of Simulation (Default = Same as Input Data)."
    puts " -scale     s         \[1.0\]        Scale Factor for Time-Domain data."
    puts "Adjustment of SAND Linewidth Estimate According to Exponential Broadening Applied to the Input:"
    puts " -lwAdj                            Reduce SAND Linewidth by the Exponential Broadening (Default)."
    puts " -nolwAdj                          Use Original SAND Linewidth, Sets -lb 0 (basicFT1.com -xELB 0)."
    puts "Apodization of Simulated Data:"
    puts "Scaling Options:"
    puts " -rescale                          Rescale Normalized SAND Output to Match Input (Default)."
    puts " -norescale                        Do Not Rescale SAND Output."
    puts "Notes:"
    puts " 1. The SAND signal table should have the following columns, with frequency"
    puts "    given in Hz from the center of the spectrum, lambda in Hz specifying"
    puts "    decay as exp( -lambda*t ), and phi as phase in degrees:"
    puts ""
    puts "    frequency        lambda           amplitude           phi"
    puts "    351.481732028563 11.2993577258213 0.00016221793732997 0"
    puts "    ..."
    puts ""
    exit 0
   }

set fidRefName ref.fid
set ftRefName  ref.ft1

set fidName    test.fid
set ftName     test.ft1

set inName  sand.tab
set outName sim.tab

set simDir    simAll
set simFIDDir simFIDAll
set simName   %04d.fid

set specID      None
set lbVal       Default
set p0Val       Default
set p1Val       Default
set scaleVal    1.0
set rescaleFlag 0
set lwAdjFlag   1

set tmpTabName    tmp_[pid].tab
set tmpScriptName tmp_[pid].com
set tmpFIDName    tmp_[pid].fid
set tmpFTName     tmp_[pid].ft1

getArgD $ARGV -in      inName
getArgD $ARGV -out     outName
getArgD $ARGV -fid     fidName
getArgD $ARGV -ft      ftName
getArgD $ARGV -specID  specID 

getArgD $ARGV -lb      lbVal
getArgD $ARGV -p0      p0Val
getArgD $ARGV -p1      p1Val
getArgD $ARGV -scale   scaleVal

getArgD $ARGV -simDir    simDir
getArgD $ARGV -simFIDDir simFIDDir
getArgD $ARGV -simName   simName
getArgD $ARGV -fidRef    fidRefName
getArgD $ARGV -ftRef     ftRefName

if {[flagLoc $ARGV -lwAdj]}   {set lwAdjFlag 1}
if {[flagLoc $ARGV -nolwAdj]} {set lwAdjFlag 0}

if {[flagLoc $ARGV -rescale]}   {set rescaleFlag 1}
if {[flagLoc $ARGV -norescale]} {set rescaleFlag 0}

if {![file exists $inName]} \
   {
    puts "sand2fid. Error finding SAND input text file $inName"
    exit 1
   }

if {![file exists $fidRefName]} \
   {
    puts "sand2fid. Error finding NMRPipe-format reference time-domain input file $fidRefName"
    exit 1
   }

set error [rdFDATA $fidRefName fdataT]

if {$error} \
   {
    puts "sand2fid. Error reading header in NMRPipe-format reference time-domain input file $fidRefName"
    exit 1
   }

if {![file exists $ftRefName]} \
   {
    puts "sand2fid. Error finding NMRPipe-format reference spectrum input file $ftRefName"
    exit 1
   }

set error [rdFDATA $ftRefName fdataF]

if {$error} \
   {
    puts "sand2fid. Error reading header in NMRPipe-format reference spectrum input file $ftRefName"
    exit 1
   }

set carPPM  [getParm  $fdataT NDCAR    CUR_XDIM]
set obsMHz  [getParm  $fdataT NDOBS    CUR_XDIM]
set swHz    [getParm  $fdataT NDSW     CUR_XDIM]
set dmxVal  [getParm  $fdataT FDDMXVAL NULL_DIM]

set tSize   [getParmI $fdataT NDSIZE   CUR_XDIM]
set fSize   [getParmI $fdataF NDSIZE   CUR_XDIM]

set extX1   [getParmI $fdataF NDX1     CUR_XDIM]
set extXN   [getParmI $fdataF NDXN     CUR_XDIM]

set xP0     [getParm  $fdataF NDP0     CUR_XDIM]
set xP1     [getParm  $fdataF NDP1     CUR_XDIM]

set xAPODQ1 [getParm  $fdataF NDAPODQ1 CUR_XDIM]
set xAPODQ2 [getParm  $fdataF NDAPODQ2 CUR_XDIM]
set xAPODQ3 [getParm  $fdataF NDAPODQ3 CUR_XDIM]
set xELB    [getParm  $fdataF NDLB     CUR_XDIM]
set xGLB    [getParm  $fdataF NDGB     CUR_XDIM]
set xZFSize [getParmI $fdataF NDFTSIZE CUR_XDIM]

if {[strcasecmp $p0Val Default]} {set xP0 $p0Val}
if {[strcasecmp $p1Val Default]} {set xP1 $p1Val}

if {$lwAdjFlag} \
   {
    set lwAdjVal $xELB
   } \
else \
   {
    set lwAdjVal 0.0
   }

if {$extX1 != 0 || $extXN != 0} \
   {
    set procX1 [pnt2spec $fdataF CUR_XDIM $extX1 ppm]ppm 
    set procXN [pnt2spec $fdataF CUR_XDIM $extXN ppm]ppm 
    set procX1 $extX1
    set procXN $extXN
   } \
else \
   {
    set procX1 0%
    set procXN 100%
   }

if {$obsMHz == 0.0} {set obsMHz 1.0}
if {$swHz   == 0.0} {set swHz   1.0}

if {[strcasecmp $simDir    None] && ![file exists $simDir]}    {catch "file mkdir $simDir"}
if {[strcasecmp $simFIDDir None] && ![file exists $simFIDDir]} {catch "file mkdir $simFIDDir"}

set error [catch {set inUnit  [open $inName  r]}]

if {$error} \
   {
    puts "sand2fid. Error opening SAND input file $inName"
    exit 1
   }

set error [catch {set outUnit [open $outName w]}]

if {$error} \
   {
    puts "sand2fid. Error opening NMRPipe simulation table $outName"
    exit 1
   }

set vListRef  "frequency lambda amplitude phi"
set vListIn   None

set lineCount 0
set pkCount   0

while {[gets $inUnit line] >= 0} \
   {
    incr lineCount

    if {[llength $line] == 0} continue

    set key [lindex $line 0]
    set c   [string index $key 0]

    if {![strcmp $c "#"]} continue

    if {![strcasecmp $vListIn None]} \
       {
        set vListIn $line

        if {![sameList $vListIn $vListRef]} \
           {
            puts "sand2fid. Error in SAND input format."
            puts "sand2fid. Expected: $vListRef Found: $vListIn"
           }

        continue
       }

    incr pkCount
   }

puts $outUnit ""
puts $outUnit "DATA SAND_INFO sandName $inName refFIDName $fidName refFTName $ftName carPPM $carPPM obsMHz $obsMHz swHz $swHz tSize $tSize fSise $fSize zfSize $xZFSize extX2 $extX1 extXN $extXN dfVal $dmxVal lb $lbVal elb $xELB glb $xGLB lwAdj $lwAdjVal scale $scaleVal p0 $xP0 p1 $xP1 specID $specID pkCount $pkCount"
puts $outUnit ""

puts $outUnit "VARS   INDEX X_AXIS DX    X_PPM X_HZ  XW    XW_HZ X1  X3  HEIGHT DHEIGHT VOL PCHI2 TYPE ASS CLUSTID MEMCNT"
puts $outUnit "FORMAT %5d   %9.3f  %6.3f %9.4f %9.3f %7.3f %8.3f %4d %4d %+e    %+e     %+e %.5f  %d   %s  %4d     %4d"
puts $outUnit ""

seek $inUnit 0

set hiList ""
set id     0

while {[gets $inUnit line] >= 0} \
   {
    if {[llength $line] == 0} continue

    set key [lindex $line 0]
    set c   [string index $key 0]

    if {![strcmp $c "#"]} continue

    if {$id == 0} \
       {
        incr id
        continue
       }

    set pChi2   0.0
    set pkType  1
    set ass     None
    set clustID $id
    set memCnt  1
    set dx      0.1

    set xMidHz [lindex $line 0]
    set xwHz   [lindex $line 1]
    set hi     [lindex $line 2]
    set dHi    [expr 0.01*$hi]

    set xPPM   [expr $carPPM + $xMidHz/$obsMHz]
    set xPts   [spec2pnt $fdataF CUR_XDIM ${xPPM}ppm]
    set xHz    [pnt2spec $fdataF CUR_XDIM $xPts Hz]

    set xwHz   [expr $xwHz - $lwAdjVal]
    if {$xwHz < 0.0} {set xwHz 0.0}

    set xwPts  [expr $xwHz*$tSize/$swHz]
    set x1     [expr int($xPts - 2*$xwPts)]
    set xn     [expr int($xPts + 2*$xwPts)]

    set vol    [expr $hi*$xwPts]

    puts -nonewline $outUnit [format " %5d" $id]
    puts -nonewline $outUnit [format " %9.3f  %6.3f %9.4f %9.3f" $xPts $dx $xPPM $xHz]
    puts -nonewline $outUnit [format " %7.3f %8.3f %4d %4d" $xwPts $xwHz $x1 $xn]
    puts -nonewline $outUnit [format " % e % e % e" $hi $dHi $vol]
    puts -nonewline $outUnit [format " %.5f %d %s %4d %4d"  $pChi2 $pkType $ass $clustID $memCnt]
    puts $outUnit ""

    lappend hiList [expr abs($hi)]

    incr id
   }

catch "close $outUnit"

set sandTab [gdbRead table -in $outName]

gdbSortTab $sandTab X_PPM -down

set eList [gdbGetInfo $sandTab -parms entries]
set id    0

foreach e $eList \
   {
    incr id
    gdbSet $e INDEX $id
   }

gdbWrite $sandTab $outName

set sandTab [gdbRead table -in $outName]

set thisFIDName $fidName
set thisFTName  $ftName

# fd 1/19/2023 adjusting for -nosim, then add csv format output

if {$rescaleFlag || [strcasecmp $thisFIDName None] || [strcasecmp $thisFTName None]} \
   {
    if {![strcasecmp $thisFIDName None]} {set thisFIDName $tmpFIDName}
    if {![strcasecmp $thisFTName  None]} {set thisFTName  $tmpFTName}
   }

if {[strcasecmp $thisFIDName None] || [strcasecmp $thisFTName None]} \
   {
    catch "file delete -force $tmpScriptName $tmpTabName"

    if {[strcasecmp $thisFIDName None]} {catch "file delete -force $thisFIDName"}
    if {[strcasecmp $thisFTName None]}  {catch "file delete -force $thisFTName"}

    set cmnd "pk2fid.tcl -in $outName -out $thisFIDName -fid $fidRefName -ft $ftRefName -fidScript None -title All-All-100.0 -simScript $tmpScriptName -nots -tdd"

    puts "sand2fid. Full Simulation: $cmnd"
    eval "exec $cmnd"

    if {![strcasecmp $lbVal Default]} \
       {
        set thisLB $xELB
       } \
    else \
       {
        set thisLB $lbVal
       }

    if {[strcasecmp $thisFTName None]} \
       {
        set    cmnd "basicFT1.com -in $thisFIDName -out $thisFTName -script $tmpScriptName"
        append cmnd " -xP0 $xP0 -xP1 $xP1 -xELB $thisLB -xEXTX1 $procX1 -xEXTXN $procXN -xZFARG size=$xZFSize -ioargs"

        puts "sand2fid. Spectral Processing: $cmnd"
        eval "exec $cmnd"
       }

    if {$rescaleFlag} \
       {
        set thisScale [exec findScale.tcl -in1 $ftRefName -in2 $thisFTName]
        set thisScale [expr $scaleVal*$thisScale]

        puts [format "sand2fid. Rescale Value: %.4e" $thisScale]
       } \
    else \
       {
        set thisScale $scaleVal 

        puts [format "sand2fid. Rescale Value: %.4e (No Auto-Scaling)" $thisScale]
       }

    exec nmrPipe -in $thisFIDName -out $thisFIDName -inPlace -fn MULT -c $thisScale
    exec nmrPipe -in $thisFTName  -out $thisFTName  -inPlace -fn MULT -c $thisScale

    set eList [gdbGetInfo $sandTab -parms entries]

    foreach e $eList \
       {
        gdbSet $e HEIGHT  [expr $thisScale*[gdbGet $e HEIGHT]]
        gdbSet $e DHEIGHT [expr $thisScale*[gdbGet $e DHEIGHT]]
        gdbSet $e VOL     [expr $thisScale*[gdbGet $e VOL]]
       }

    set maxHi [getTabStat getMaxAbs $sandTab HEIGHT]
    if {$maxHi == 0.0} {set maxHi 1.0}

    gdbAddNote $sandTab "DATA SAND_SCALE [format "%e" $thisScale]"
    gdbWrite   $sandTab $outName

    catch "file delete -force $tmpTabName $tmpFIDName $tmpFTName"

    if {[strcasecmp $simDir None]} \
       {
        for {set id 1} {$id <= $pkCount} {incr id} \
           {
            set thisFIDName [format $simFIDDir/%04d.fid $id]
            set thisFTName  [format $simDir/%04d.ft1    $id]

            catch "file delete -force $tmpTabName $thisFIDName $thisFTName"

            set oneTab [gdbSelect -from $sandTab -new -cond "INDEX == $id"]
            set eList  [gdbGetInfo $oneTab -parms entries]
            set e      [lindex $eList 0]

            set thisPPM [format %.3f [gdbGet $e X_PPM]]

            if {![strcmp [string index $thisPPM 0] "-"]} {set thisPPM n[string range $thisPPM 1 end]}

            set pctHi [expr 100.0*[gdbGet $e HEIGHT]/$maxHi]
            set tStr  [format "%04d-%sppm-%.1fpct" $id $thisPPM $pctHi]

            gdbWrite $oneTab  $tmpTabName 

            set cmnd "pk2fid.tcl -in $tmpTabName -out $thisFIDName -fid $fidRefName -ft $ftRefName -fidScript None -title $tStr -simScript $tmpScriptName -nots -tdd"

            puts ""
            puts "sand2fid. Simulation ${id} of ${pkCount}: $cmnd"
            eval "exec $cmnd"

            set    cmnd "basicFT1.com -in $thisFIDName -out $thisFTName -script $tmpScriptName"
            append cmnd " -xP0 $xP0 -xP1 $xP1 -xELB $thisLB -xEXTX1 $procX1 -xEXTXN $procXN -xZFARG size=$xZFSize -ioargs"

            puts ""
            puts "sand2fid. Processing ${id} of ${pkCount}: $cmnd"
            eval "exec $cmnd"

            catch "file delete -force $tmpTabName $tmpFIDName $tmpFTName"

            gdbFree $oneTab
           }
       }

    catch "file delete -force $tmpScriptName $tmpTabName $tmpFIDName $tmpFTName"
   }

exit 0
