#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Prepare 1D Pool Spectra for Screening, Perform Scoring."
   echo "Input Files:"
   echo " -blank    bName      blank.ft1"
   echo " -screen   sName      screen.ft1"
   echo " -cList    cList      c1.ft1 c2.ft1 ..."
   echo " -verb                Verbose Mode ON."
   echo "Input Parameters:"
   echo " -jobID    jobID      date +%Y%m%d"
   echo " -poolID   poolID     1"
   echo " -rackID   rackID     1"
   echo " -expID    expID      1 (Such as 1 ... 96)"
   echo " -x1       regX1      8.9ppm"
   echo " -xn       regXN      6.4ppm"
   echo "Dark Region List (Always in Pairs, pts ppm hz % None):"
   echo " -darkList regX1 regX1 ..."
   echo "Output Files and Naming:"
   echo " -outDir   oDir       ."
   echo " -prefix   pName      ext (Such as aro ali ...)"
   echo " -darkName dName      orig_dark.ft1"
   echo " -tmpName  tName      scorePool_tmp.tmp"
   echo "Interpolated Size in Points for All Output:"
   echo " -outSize  oSize      Auto"
   echo "Multiplier for Noise Threshold:"
   echo " -mult     noiseMult    3.0"
   echo "Scaling, Largest Allowable Negative Difference in Pct:"
   echo " -sDifLim  screenPct    5.0  (Or Keyword None for No Limit)"
   echo " -cDifLim  compoundPct  50.0 (Or Keyword None for No Limit)"
   echo "Limits for Determining Optimal Shift, Points:"
   echo " -shiftMin sMin         -10"
   echo " -shiftMax sMax          10"
   echo "Analysis of Decomposition Coefficients;"
   echo "Minimum Significant Pct Change in Coefficients:"
   echo " -minPct   minPct        1.0"
   exit 0
endif

set thisID = (`date +%Y%m%d`)

set jobID      = (`getArgD $argv -jobID    $thisID`)
set poolID     = (`getArgD $argv -poolID   1`)
set rackID     = (`getArgD $argv -rackID   1`)
set expID      = (`getArgD $argv -expID    1`)
set x1         = (`getArgD $argv -x1       8.9ppm`)
set xn         = (`getArgD $argv -xn       6.4ppm`)
set shiftMin   = (`getArgD $argv -shiftMin -10`)
set shiftMax   = (`getArgD $argv -shiftMin  10`)
set outSize    = (`getArgD $argv -outSize  Auto`)
set blankName  = (`getArgD $argv -blank    blank.ft1`)
set screenName = (`getArgD $argv -screen   screen.ft1`)
set darkName   = (`getArgD $argv -darkName orig_dark.ft1`)
set tmpName    = (`getArgD $argv -tmpName  scorePool_tmp.tmp`)
set prefix     = (`getArgD $argv -prefix   ext`)
set outDir     = (`getArgD $argv -outDir   .`)
set mult       = (`getArgD $argv -mult     3.0`)
set sDifLim    = (`getArgD $argv -sDifLim  5.0`)
set cDifLim    = (`getArgD $argv -sDifLim  50.0`)
set minPct     = (`getArgD $argv -minPct   1.0`)

set cList      = (`getListArgD $argv -in       -- c1.ft1 c2.ft1 c3.ft1 c4.ft1 c5.ft1`)
set darkList   = (`getListArgD $argv -darkList -- None None`)

set scaleVal   = 100.0
set scaleFlag  = 1
set vFlag      = 0

if (`flagLoc $argv -verb`) then
   set vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

if (`flagLoc $argv -scale`) then
   set scaleFlag  = 1
endif

if (`flagLoc $argv -noscale`) then
   set scaleFlag  = 0
endif

#
#------------------------------------------------------------------------------------------------------

foreach inName ($blankName $screenName $cList)
   if (!(-e $inName)) then
      echo "ScorePool Error Finding Input $inName"
      exit 1
   endif
end

if (!(-d $outDir)) then
   mkdir $outDir
endif

set origDir = (`pwd`)
cd  $outDir

set fullDir = (`pwd`)
cd $origDir

if (-e $outDir/${prefix}_score.tab) then
   /bin/rm -f $outDir/${prefix}_score.tab
endif

if (-e $outDir/${prefix}_blank.flist) then
   /bin/rm -f $outDir/${prefix}_blank.flist
endif

if (-e $outDir/${prefix}_screen.flist) then
   /bin/rm -f $outDir/${prefix}_screen.flist
endif

if (-e $outDir/${prefix}_pool.flist) then
   /bin/rm -f $outDir/${prefix}_pool.flist
endif

#
# Build the mask of dark regions:

nmrPipe -in $blankName -out $outDir/$darkName -fn SET -c 1.0 -ov

set ix1 = 1
set ixn = 2
set k   = 0

while ($ixn <= $#darkList)
   if (`string csame $darkList[$ix1]` == 0 && `string csame $darkList[$ixn]` == 0) then
      @ k++

      if ($vFlag) then
         echo "In: $outDir/$darkName Out: $outDir/$darkName SET 0.0: $darkList[$ix1] $darkList[$ixn]"
      endif

      nmrPipe -in $outDir/$darkName -out $outDir/$darkName -inPlace -ov \
              -fn SET -c 0.0 -x1 $darkList[$ix1] -xn $darkList[$ixn]
   endif
 
   @ ix1 += 2
   @ ixn += 2
end

#
# Read the selected regions and extract their original sizes:

readROI -in $blankName        -out $outDir/${prefix}_blank.ft1  -x X_AXIS $x1 $xn -ndim 1 -noverb -silent
readROI -in $screenName       -out $outDir/${prefix}_screen.ft1 -x X_AXIS $x1 $xn -ndim 1 -noverb -silent
readROI -in $outDir/$darkName -out $outDir/${prefix}_dark.ft1   -x X_AXIS $x1 $xn -ndim 1 -noverb -silent

set titleInfo = (`getParm -in $outDir/${prefix}_dark.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM | tr '-' ' '`)
sethdr $outDir/${prefix}_dark.ft1 -title $titleInfo[1]-dark-mask

set blankSize  = (`getParm -in $outDir/${prefix}_blank.ft1  -parm NDSIZE -dim CUR_XDIM -fmt %.0f`)
set screenSize = (`getParm -in $outDir/${prefix}_screen.ft1 -parm NDSIZE -dim CUR_XDIM -fmt %.0f`)

set sizeList = ""
set i        = 1

foreach inName ($cList)
   readROI -in $inName -out $outDir/${prefix}_c$i.ft1 -x X_AXIS $x1 $xn -ndim 1 -noverb -silent
   set sizeList = ($sizeList `getParm -in $outDir/${prefix}_c$i.ft1 -parm NDSIZE -dim CUR_XDIM -fmt %.0f`)
   @ i++
end

set minSize = (`getStat.tcl -x $blankSize $screenSize $sizeList -int -stat Max -noverb`)
set maxSize = (`getStat.tcl -x $blankSize $screenSize $sizeList -int -stat Max -noverb`)

if (`string csame $outSize Auto`) then
   set outSize = $maxSize
else
   if ($outSize < $maxSize) then
      set outSize = $maxSize
   endif
endif

#
# Interpolate all data to uniform size:

if ($blankSize != $outSize) then
   interp2D $outDir/${prefix}_blank.ft1 $outDir/${prefix}_$tmpName $outSize 1
   /bin/mv $outDir/${prefix}_$tmpName $outDir/${prefix}_blank.ft1

   interp2D $outDir/${prefix}_dark.ft1 $outDir/${prefix}_$tmpName $outSize 1
   /bin/mv $outDir/${prefix}_$tmpName $outDir/${prefix}_dark.ft1

   if ($vFlag) then
      echo "Interpolated $outDir/${prefix}_blank.ft1"
      echo "Interpolated $outDir/${prefix}_dark.ft1"
   endif
endif

if ($screenSize != $outSize) then
   interp2D $outDir/${prefix}_screen.ft1 $outDir/${prefix}_$tmpName $outSize 1
   /bin/mv $outDir/${prefix}_$tmpName $outDir/${prefix}_screen.ft1

   if ($vFlag) then
      echo "Interpolated $outDir/${prefix}_screen.ft1"
   endif
endif

set i = 1

foreach inName ($cList)
   if ($sizeList[$i] != $outSize) then
      interp2D $outDir/${prefix}_c$i.ft1 $outDir/${prefix}_$tmpName $outSize 1
      /bin/mv $outDir/${prefix}_$tmpName $outDir/${prefix}_c$i.ft1

      if ($vFlag) then
         echo "Interpolated $outDir/${prefix}_c$i.ft1"
      endif
   endif

   set title = (`getParm -in $outDir/${prefix}_c$i.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM | sed -e "s/-lib-/-c${i}-/gp"`)
   sethdr $outDir/${prefix}_c$i.ft1 -title $title

   @ i++
end

#
# Apply the dark region mask:

addNMR -in1 $outDir/${prefix}_blank.ft1  -in2 $outDir/${prefix}_dark.ft1 \
       -mul -out $outDir/${prefix}_blank_dark.ft1

set title = (`getParm -in $outDir/${prefix}_blank.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
sethdr $outDir/${prefix}_blank_dark.ft1 -title ${title}-dark


addNMR -in1 $outDir/${prefix}_screen.ft1 -in2 $outDir/${prefix}_dark.ft1 \
       -mul -out $outDir/${prefix}_screen_dark.ft1

set title = (`getParm -in $outDir/${prefix}_screen.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
sethdr $outDir/${prefix}_screen_dark.ft1 -title ${title}-dark

set i = 1

foreach inName ($cList)
   addNMR -in1 $outDir/${prefix}_c$i.ft1 -in2 $outDir/${prefix}_dark.ft1 \
          -mul -out $outDir/${prefix}_c${i}_dark.ft1

   set title = (`getParm -in $outDir/${prefix}_c${i}.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
   sethdr $outDir/${prefix}_c${i}_dark.ft1 -title ${title}-dark

   @ i++
end

#
# Scale the spectra to uniform max intensity:

if ($scaleFlag != 0) then
   set blankMaxVal  = (`scale2D -brief -nosave $outDir/${prefix}_blank_dark.ft1`) 
   set screenMaxVal = (`scale2D -brief -nosave $outDir/${prefix}_screen_dark.ft1`) 
   set maxVal       = (`getStat.tcl -x $blankMaxVal $screenMaxVal -stat Max -noverb`)

   echo $blankMaxVal $scaleVal > $outDir/${prefix}_blank_mult.txt

   nmrPipe -in $outDir/${prefix}_blank.ft1 \
   | nmrPipe -fn MULT -c $blankMaxVal -noise -inv \
   | nmrPipe -fn MULT -c $scaleVal    -noise \
     -out $outDir/${prefix}_blank.ft1 -inPlace -ov

   nmrPipe -in $outDir/${prefix}_blank_dark.ft1 \
   | nmrPipe -fn MULT -c $blankMaxVal -noise -inv \
   | nmrPipe -fn MULT -c $scaleVal    -noise \
     -out $outDir/${prefix}_blank_dark.ft1 -inPlace -ov

   if ($vFlag) then
      echo "Scale $blankMaxVal $scaleVal $outDir/${prefix}_blank.ft1"
      echo "Scale $blankMaxVal $scaleVal $outDir/${prefix}_blank_dark.ft1"

      scale2D $outDir/${prefix}_blank.ft1
      scale2D $outDir/${prefix}_blank_dark.ft1
   endif

   echo $screenMaxVal $scaleVal > $outDir/${prefix}_screen_mult.txt

   nmrPipe -in $outDir/${prefix}_screen.ft1 \
   | nmrPipe -fn MULT -c $screenMaxVal -noise -inv \
   | nmrPipe -fn MULT -c $scaleVal     -noise \
     -out $outDir/${prefix}_screen.ft1 -inPlace -ov

   nmrPipe -in $outDir/${prefix}_screen_dark.ft1 \
   | nmrPipe -fn MULT -c $screenMaxVal -noise -inv \
   | nmrPipe -fn MULT -c $scaleVal     -noise \
     -out $outDir/${prefix}_screen_dark.ft1 -inPlace -ov

   if ($vFlag) then
      echo "Scale $screenMaxVal $scaleVal $outDir/${prefix}_screen.ft1"
      echo "Scale $screenMaxVal $scaleVal $outDir/${prefix}_screen_dark.ft1"

      scale2D $outDir/${prefix}_screen.ft1
      scale2D $outDir/${prefix}_screen_dark.ft1
   endif

   set i = 1

   foreach inName ($cList)
      set maxVal = (`scale2D -brief -nosave $outDir/${prefix}_c${i}_dark.ft1`)

      echo $maxVal $scaleVal > $outDir/${prefix}_c${i}_mult.txt

      nmrPipe -in $outDir/${prefix}_c$i.ft1 \
      | nmrPipe -fn MULT -c $maxVal   -noise -inv \
      | nmrPipe -fn MULT -c $scaleVal -noise \
        -out $outDir/${prefix}_c$i.ft1 -inPlace -ov

      nmrPipe -in $outDir/${prefix}_c${i}_dark.ft1 \
      | nmrPipe -fn MULT -c $maxVal   -noise -inv \
      | nmrPipe -fn MULT -c $scaleVal -noise \
        -out $outDir/${prefix}_c${i}_dark.ft1 -inPlace -ov

      if ($vFlag) then
         echo "Scale $maxVal $scaleVal $outDir/${prefix}_c$i.ft1"
         echo "Scale $maxVal $scaleVal $outDir/${prefix}_c${i}_dark.ft1"

         scale2D $outDir/${prefix}_c$i.ft1
         scale2D $outDir/${prefix}_c${i}_dark.ft1
      endif

      @ i++
   end
endif

#
# Find the optimal shift between blank and screen pool data:

if ($shiftMin != 0 || $shiftMax != 0) then
   modelXY -noverb -silent -data $outDir/${prefix}_blank_dark.ft1 -aData $outDir/${prefix}_screen_dark.ft1  \
           -macro $NMRTXT/vvScaleShiftI.M -p 1.0 -gc 1 -var sMin $shiftMin sMax $shiftMax sStep 1 \
           >& $outDir/${prefix}_screen_shift.txt

   set shiftInfo = (`cat $outDir/${prefix}_screen_shift.txt` lshift 0 scale 1 RMS 1.0e+00)

   if (`string csame $shiftInfo[1] lshift`) then
      nmrPipe -in $outDir/${prefix}_screen_dark.ft1 -out $outDir/${prefix}_screen_dark.ft1 -inPlace -ov -fn CS -ls $shiftInfo[2]
      nmrPipe -in $outDir/${prefix}_screen.ft1      -out $outDir/${prefix}_screen.ft1      -inPlace -ov -fn CS -ls $shiftInfo[2]

      if ($vFlag) then
         echo "Shift $shiftInfo[2] $outDir/${prefix}_screen_dark.ft1"
         echo "Shift $shiftInfo[2] $outDir/${prefix}_screen.ft1"
      endif
   endif
endif

#
# Build noise masks for all data:

nmrPipe -in $outDir/${prefix}_blank_dark.ft1  -out $outDir/${prefix}_blank_mask.ft1  -ov \
        -fn MAC -macro $NMRTXT/noiseMask.M -all -var mult $mult

set title = (`getParm -in $outDir/${prefix}_blank.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
sethdr $outDir/${prefix}_blank_mask.ft1 -title ${title}-mask

nmrPipe -in $outDir/${prefix}_screen_dark.ft1 -out $outDir/${prefix}_screen_mask.ft1 -ov \
        -fn MAC -macro $NMRTXT/noiseMask.M -all -var mult $mult

set title = (`getParm -in $outDir/${prefix}_screen.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
sethdr $outDir/${prefix}_screen_mask.ft1 -title ${title}-mask

set i = 1

foreach inName ($cList)
   nmrPipe -in $outDir/${prefix}_c${i}_dark.ft1 -out $outDir/${prefix}_c${i}_mask.ft1 -ov \
           -fn MAC -macro $NMRTXT/noiseMask.M -all -var mult $mult

   set title = (`getParm -in $outDir/${prefix}_c${i}.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
   sethdr $outDir/${prefix}_c${i}_mask.ft1 -title ${title}-mask

   if ($vFlag) then
      echo "Before Mask: `scale2D $outDir/${prefix}_c${i}_dark.ft1`"
      echo "After Mask:  `scale2D $outDir/${prefix}_c${i}_mask.ft1`"
   endif

   @ i++
end

#
# Combine compound spectra for decomposition of the blank and screen pools: 

set appendList = ""
set pList      = ""
set i = 1

foreach inName ($cList)
   set appendList = ($appendList $outDir/${prefix}_c${i}_dark.ft1)
   set pList      = ($pList 1.0)
   @ i++
end

appendNMR.tcl -in $appendList -out $outDir/${prefix}_append.ft2 -ndim 2

#
# Perform the compound decomposition of the blank pool:

modelXY -data  $outDir/${prefix}_blank_dark.ft1  \
        -aData $outDir/${prefix}_append.ft2      \
        -macro $NMRTXT/vvBasis.M -p $pList -silent -noverb -gc 1 >& $outDir/${prefix}_blank_clist.txt

set pBlank = (`cat $outDir/${prefix}_blank_clist.txt`)

nmrPipe -in $outDir/${prefix}_blank.ft1 -out $outDir/${prefix}_blank_deco.ft1 -fn SET -c 0.0 -ov

if ($vFlag) then
    echo "Blank Coef:  $pBlank"
endif

set i = 1

foreach inName ($cList)
   addNMR -in1 $outDir/${prefix}_blank_deco.ft1 -in2 $outDir/${prefix}_c$i.ft1 -c1 1.0 \
          -c2 $pBlank[$i] -out $outDir/${prefix}_$tmpName -add

   mv $outDir/${prefix}_$tmpName $outDir/${prefix}_blank_deco.ft1 
   @ i++
end

set title = (`getParm -in $outDir/${prefix}_blank.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
sethdr $outDir/${prefix}_blank_deco.ft1 -title ${title}-deco

#
# Perform the compound decomposition of the screen pool:

modelXY -data  $outDir/${prefix}_screen_dark.ft1 \
        -aData $outDir/${prefix}_append.ft2      \
        -macro $NMRTXT/vvBasis.M -p $pList -silent -noverb -gc 1 >& $outDir/${prefix}_screen_clist.txt

set pScreen = (`cat $outDir/${prefix}_screen_clist.txt`)

nmrPipe -in $outDir/${prefix}_screen.ft1 -out $outDir/${prefix}_screen_deco.ft1 -fn SET -c 0.0 -ov

if ($vFlag) then
    echo "Screen Coef: $pScreen"
endif

set i = 1

foreach inName ($cList)
   addNMR -in1 $outDir/${prefix}_screen_deco.ft1 -in2 $outDir/${prefix}_c$i.ft1 \
          -c1 1.0 -c2 $pScreen[$i] -out $outDir/${prefix}_$tmpName -add

   mv $outDir/${prefix}_$tmpName $outDir/${prefix}_screen_deco.ft1
   @ i++
end

set title = (`getParm -in $outDir/${prefix}_screen.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
sethdr $outDir/${prefix}_screen_deco.ft1 -title ${title}-deco

#
# Form the differences (residuals) for the measured blank and screen data.
# Prepare a mask of the residual spectrum.

if ($scaleFlag != 0) then
   scale.tcl -pos -ix1 3% -ixn 97% -in1 $outDir/${prefix}_blank.ft1 -in2 $outDir/${prefix}_screen.ft1 \
             -mask $outDir/${prefix}_dark.ft1 -difLim $sDifLim > $outDir/${prefix}_screen_dscale.txt
else
   echo 1.0 > $outDir/${prefix}_screen_dscale.txt
endif

set scaleInfo = (`cat $outDir/${prefix}_screen_dscale.txt` 1.0)

if ($vFlag) then
   echo "Data Diff Scale: $scaleInfo[1] $outDir/${prefix}_blank.ft1 $outDir/${prefix}_screen.ft1 mask $outDir/${prefix}_dark.ft1"
endif

addNMR -in1 $outDir/${prefix}_blank.ft1 -in2 $outDir/${prefix}_screen.ft1 -c2 $scaleInfo[1] \
       -sub -out $outDir/${prefix}_screen_dif.ft1

set title = (`getParm -in $outDir/${prefix}_screen.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
sethdr $outDir/${prefix}_screen_dif.ft1 -title ${title}-dif

nmrPipe -in $outDir/${prefix}_screen_dif.ft1 -out $outDir/${prefix}_screen_dif_mask.ft1 -ov \
        -fn MAC -macro $NMRTXT/noiseMask.M -all -var mult $mult

addNMR -in1 $outDir/${prefix}_screen_dif_mask.ft1 -in2 $outDir/${prefix}_dark.ft1 \
       -mul -out $outDir/${prefix}_$tmpName

/bin/mv $outDir/${prefix}_$tmpName $outDir/${prefix}_screen_dif_mask.ft1

set title = (`getParm -in $outDir/${prefix}_screen_dif.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
sethdr $outDir/${prefix}_screen_dif_mask.ft1 -title ${title}-mask

#
# Form the differences (residuals) for the synthetic blank and screen data:

if ($scaleFlag != 0) then
   scale.tcl -pos -ix1 3% -ixn 97% -in1 $outDir/${prefix}_blank_deco.ft1 -in2 $outDir/${prefix}_screen_deco.ft1 \
             -mask $outDir/${prefix}_dark.ft1 -difLim $sDifLim > $outDir/${prefix}_screen_deco_dscale.txt
else
   echo 1.0 > $outDir/${prefix}_screen_deco_dscale.txt
endif

set scaleInfo = (`cat $outDir/${prefix}_screen_deco_dscale.txt` 1.0)

if ($vFlag) then
   echo "Deco Scale: $scaleInfo[1] $outDir/${prefix}_blank_deco.ft1 $outDir/${prefix}_screen_deco.ft1"
endif

addNMR -in1 $outDir/${prefix}_blank_deco.ft1 -in2 $outDir/${prefix}_screen_deco.ft1 -c2 $scaleInfo[1] \
       -sub -out $outDir/${prefix}_screen_deco_dif.ft1

set title = (`getParm -in $outDir/${prefix}_screen_deco.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
sethdr $outDir/${prefix}_screen_deco_dif.ft1 -title ${title}-dif

#
# Form the differences between the screen residual and the individual compounds:

set cScaleList = ""
set i = 1

foreach inName ($cList)
   scale.tcl -pos -ix1 3% -ixn 97% -in1 $outDir/${prefix}_screen_dif.ft1 -in2 $outDir/${prefix}_c$i.ft1 \
             -mask $outDir/${prefix}_dark.ft1 -difLim $cDifLim > $outDir/${prefix}_c${i}_dscale.txt

   set scaleInfo  = (`cat $outDir/${prefix}_c${i}_dscale.txt` 1.0)
   set cScaleList = ($cScaleList $scaleInfo[1])
 
   addNMR -in1 $outDir/${prefix}_screen_dif.ft1 -in2 $outDir/${prefix}_c$i.ft1 -c2 $scaleInfo[1] \
          -sub -out $outDir/${prefix}_c${i}_dif.ft1

   nmrPipe -in $outDir/${prefix}_c$i.ft1 -out $outDir/${prefix}_c${i}_dscale.ft1 -fn MULT -c $scaleInfo[1] -ov

   set title = (`getParm -in $outDir/${prefix}_c$i.ft1 -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
   sethdr $outDir/${prefix}_c${i}_dif.ft1    -title ${title}-dif
   sethdr $outDir/${prefix}_c${i}_dscale.ft1 -title ${title}-scale

   @ i++
end

if ($vFlag) then
   echo "Compound Scales: $cScaleList"
endif

#
# Record the origins of the data:

echo $blankName  > $outDir/${prefix}_blank.flist
echo $screenName > $outDir/${prefix}_screen.flist

if (-e $outDir/${prefix}_pool.flist) then
   /bin/rm -f $outDir/${prefix}_pool.flist
endif

set i = 1

foreach inName ($cList)
   echo $inName >> $outDir/${prefix}_pool.flist
   @ i++
end

#
# Record the spectral difference scores:

if (-e $outDir/${prefix}_score.tab) then
   /bin/rm -f $outDir/${prefix}_score.tab
endif

echo ""  >> $outDir/${prefix}_score.tab
echo "#" >> $outDir/${prefix}_score.tab
echo "# ScorePool (c) 2012  F. Delaglio www.nmrscience.com"  >> $outDir/${prefix}_score.tab
echo ""  >> $outDir/${prefix}_score.tab

set dName = $fullDir/${prefix}_screen_dif.ft1

echo "DATA JOB_ID  $jobID"          >> $outDir/${prefix}_score.tab
echo "DATA POOL_ID $poolID"         >> $outDir/${prefix}_score.tab
echo "DATA RACK_ID $rackID"         >> $outDir/${prefix}_score.tab
echo "DATA NC      $#cList"         >> $outDir/${prefix}_score.tab
echo ""                             >> $outDir/${prefix}_score.tab
echo "DATA BLANK_NAME  $fullDir/${prefix}_blank.ft1"      >> $outDir/${prefix}_score.tab
echo "DATA SCREEN_NAME $fullDir/${prefix}_screen.ft1"     >> $outDir/${prefix}_score.tab
echo "DATA DIF_NAME    $fullDir/${prefix}_screen_dif.ft1" >> $outDir/${prefix}_score.tab
echo ""                             >> $outDir/${prefix}_score.tab

set i = 1

foreach inName ($cList)
   echo "DATA C${i}_NAME  $fullDir/${prefix}_c$i.ft1" >> $outDir/${prefix}_score.tab
   @ i++
end

echo ""                            >> $outDir/${prefix}_score.tab

set i = 1

foreach inName ($cList)
   set cTitle = (`getParm -in $inName -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)
   echo "DATA C${i}_TITLE $cTitle" >> $outDir/${prefix}_score.tab
   @ i++
end
 
echo ""                  >> $outDir/${prefix}_score.tab
echo "VARS   PNAME  VAL" >> $outDir/${prefix}_score.tab
echo "FORMAT %26s  %12s" >> $outDir/${prefix}_score.tab
echo ""                  >> $outDir/${prefix}_score.tab

score.tcl -ix1 3% -ixn 97% -in1 $outDir/${prefix}_blank_dark.ft1 -in2 $outDir/${prefix}_screen_dark.ft1 \
          -lab1 BLANK -lab2 SCREEN -labD BSDIF >> $outDir/${prefix}_score.tab

echo "" >> $outDir/${prefix}_score.tab

score.tcl -no1 -noN -ix1 3% -ixn 97% \
          -in1 $outDir/${prefix}_blank_dark.ft1 -in2 $outDir/${prefix}_blank_deco.ft1 \
          -lab1 BLANK -lab2 BLANKDECO -labD BDDIF >> $outDir/${prefix}_score.tab

echo "" >> $outDir/${prefix}_score.tab

score.tcl -no1 -noN -ix1 3% -ixn 97% \
          -in1 $outDir/${prefix}_screen_dark.ft1 -in2 $outDir/${prefix}_screen_deco.ft1 \
          -lab1 SCREEN -lab2 SCREENDECO -labD SDDIF >> $outDir/${prefix}_score.tab

echo "" >> $outDir/${prefix}_score.tab

set i = 1

foreach inName ($cList)
    score.tcl -no1 -noN -ix1 3% -ixn 97% \
              -in1 $outDir/${prefix}_screen_dif.ft1 -in2 $outDir/${prefix}_c${i}_dscale.ft1 \
              -lab1 BS_DIF -lab2 C$i -labD C${i}DIF >> $outDir/${prefix}_score.tab

    echo "" >> $outDir/${prefix}_score.tab

    score.tcl -no1 -no2 -noN -ix1 3% -ixn 97% \
              -mask $outDir/${prefix}_screen_dif_mask.ft1 \
              -in1 $outDir/${prefix}_screen_dif.ft1 -in2 $outDir/${prefix}_c${i}_dscale.ft1 \
              -lab1 BS_DIF -lab2 C$i -labD C${i}DIF_DMASK >> $outDir/${prefix}_score.tab

    echo "" >> $outDir/${prefix}_score.tab

    score.tcl -no1 -no2 -noN -ix1 3% -ixn 97% \
              -mask $outDir/${prefix}_c${i}_mask.ft1 \
              -in1 $outDir/${prefix}_screen_dif.ft1 -in2 $outDir/${prefix}_c${i}_dscale.ft1 \
              -lab1 BS_DIF -lab2 C$i -labD C${i}DIF_CMASK >> $outDir/${prefix}_score.tab

    echo "" >> $outDir/${prefix}_score.tab
   @ i++
end

scorePool.tcl -minPct $minPct \
 -dir $outDir -prefix $prefix \
 -in $outDir/${prefix}_score.tab \
 -bCoef `cat $outDir/${prefix}_blank_clist.txt` \
 -sCoef `cat $outDir/${prefix}_screen_clist.txt` > $outDir/${prefix}_$tmpName

cat $outDir/${prefix}_$tmpName >> $outDir/${prefix}_score.tab

#
# Cleanup:

if (-e $outDir/$darkName) then
   /bin/rm -rf $outDir/$darkName
endif

if (-e $outDir/${prefix}_$tmpName) then
   /bin/rm -f $outDir/${prefix}_$tmpName
endif
