#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

set pdbName ref.pdb
set inName  None
set outName csSim.tab
set hmsName hmsInfo.tab
set hmsFlag hmsAdd

set verbose 1

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Prepare Simulated-Only Backbone Chemical Shifts from PDB:"
    puts " -pdb  pdbName   \[ref.pdb\]     PDB Input Name."
    puts " -in   csInName  \[None\]        Optional CS Input Name (Also: -cs)."
    puts " -out  csOutName \[csSim.tab\]   Chemical Shift Output Name."
    puts " -verb                         Verbose Table Output ON (Default)."
    puts " -noverb                       Verbose Table Output OFF."
    puts "Ring Current Shift Calculation:"
    puts " -hms  hmsName   \[hmsInfo.tab\] Output for Ring Definitions;"
    puts "                                 use 'None' for no Calculation."
    puts " -hmsAdd                         Add Ring Current Shifts to Predicted"
    puts "                                 Shifts (Default)."
    puts " -hmsSub                         Subtract Ring Current Shifts from"
    puts "                                 Observed Shifts."
    puts " -hmsNone                        Ring Current Shifts are calculated"
    puts "                                 but not used for adjustment."
    exit 1
   }

getArgD $ARGV -pdb pdbName
getArgD $ARGV -cs  inName
getArgD $ARGV -in  inName
getArgD $ARGV -hms hmsName
getArgD $ARGV -out outName

if {[flagLoc $ARGV -hmsAdd]}  {set hmsFlag hmsAdd}
if {[flagLoc $ARGV -hmsSub]}  {set hmsFlag hmsSub}
if {[flagLoc $ARGV -hmsNone]} {set hmsFlag hmsNone}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#-----------------------------------------------------------------------------

if {![file exists $pdbName]} \
   {
    puts stderr "Error finding PDB input file $pdbName"
    exit 1
   }

if {[strcasecmp $inName None]} \
   {
    if {![file exists $inName]} \
       {
        puts stderr "Error finding CS input file $inName"
        exit 1
       }
   }

dynInit
initResNames

#
# Extract PDB information:

set pdbSrc [dynRead -pdb -in $pdbName]
set pInfo  [dynGetInfo -src $pdbSrc]
set r1     [lindex $pInfo 0]
set rN     [lindex $pInfo 1]

set hmsSrc 0

if {[strcasecmp $hmsName None]} \
   {
    catch {file delete $hmsName}
    catch {exec pdb2hms.tcl -pdb $pdbName -ignore -seg None > $hmsName}

    if {![file exists $hmsName]} \
       {
        puts stderr "Error Creating HMS File $hmsName"
        exit 1
       }

    set hmsSrc [dynRead -src $pdbSrc -hms -in $hmsName]
   }

#
# Prepare an empty chemical shift table if needed:

if {![strcmp $inName None]} \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error opening output file $outName"
        exit 1
       }

    puts $outUnit [dynGetPDBSeqStr $pdbSrc]
    puts $outUnit ""

    puts $outUnit "VARS   RESID RESNAME ATOMNAME SHIFT"
    puts $outUnit "FORMAT %4d   %1s     %-4s      %8.3f"
    puts $outUnit ""

    set seqList [dynGetPDBSeqList $pdbSrc res1 resN]

    for {set res [expr $r1 + 1]} {$res < $rN} {incr res} \
       {
        set resName [dynGetInfo -src $pdbSrc -resName -res $res]
        set resType [lindex $seqList [expr $res - $res1]]
        set shift   0.0 

        switch -exact $resName \
           {
            GLY     {set atomList "HN HA2 HA3 CA C N"}
            PRO     {set atomList "HA CA CB C N"}
            default {set atomList "HN HA CA CB C N"}
           }

        foreach atom $atomList \
           {
            set fmt " %3d %s %-4s %8.3f" 
            puts $outUnit [format $fmt $res $resType $atom $shift]
           }
       }

    close $outUnit
   }

#
# If there is no CS input:
#
# The first simulation will just have all 0.0 values for "CS_OBS".
#
# Redo the simulation with random coil shifts instead of
# 0.0 values.  This will not effect the simulation values,
# but will provide an output file with both "CS_OBS" and "SHIFT"
# values.

if {![strcasecmp $inName None]} \
   {
    set csSrc  [dynRead -cs -in $outName]
    set csInfo [dynSimulate -cs $csSrc -src $pdbSrc]

    dynWrite -cs $csSrc -out $outName -verb
    dynFree  -cs $csSrc

    set csTab [gdbRead table -in $outName]
    set eList [gdbGetInfo $csTab -parms entries]

    foreach e $eList \
       {
        set rcShift [gdbGet $e RC_SHIFT]

        gdbSet $e SHIFT  $rcShift
        gdbSet $e CS_OBS $rcShift
       } 

    gdbWrite $csTab $outName

    set inName $outName
   }

set csSrc  [dynRead -cs -in $inName]
set csInfo [dynSimulate -cs $csSrc -src $pdbSrc -hms $hmsSrc -$hmsFlag]

if {$verbose} \
   {
    dynWrite -cs $csSrc -out $outName -verb
   } \
else \
   {
    dynWrite -cs $csSrc -out $outName
   }
 
exit
