#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]
  
if {[flagLoc $ARGV -help]} \
   {
    puts "Convert a SMILE Log File to an NMRPipe Peak Table:"
    puts " -in     logName \[smile.log\]  SMILE Log Output."
    puts " -out    tabName \[smile.tab\]  Peak Table Output."
    puts " -ref    refName \[test.ft2\]   Spectrum Reference."
    puts " -scale  scale   \[1.0\]        Factor for Scaling SMILE Amplitudes."
    puts " -hi     thresh  \[Auto\]       Threshold for Marking Peaks as Noise."
    puts " -factor factor  \[7.0\]        Factor for Setting -hi Auto Threshold."
    puts " -lwf    lwf     \[2.0\]        Factor for Setting Peak Bounds from Linewidth."
    puts " -reject                      Reject Noise Peaks."
    puts " -verb                        Verbose Mode ON (Default)."
    puts " -noverb                      Verbose Mode OFF."
    puts "Notes:"
    puts " 1. The SMILE log file should be created via SMILE -report 2."
    puts " 2. Using -scale Auto sets the scale value so that the highest peak height"
    puts "    in the output will match the maximum height in the spectrum."
    puts " 3. Using -hi Auto sets the threshold to factor*scale*THERMAL_NOISE."
    puts " 4. Use -hi None if no peaks should be marked as noise."
    exit 0
   }

set logName    smile.log
set outName    smile.tab
set refName    test.ft2
set scaleVal   1.0 
set hi         Auto
set f          7.0
set lwf        2.0
set rejectFlag 0
set verbose    1

getArgD $ARGV -in     logName
getArgD $ARGV -out    outName
getArgD $ARGV -ref    refName
getArgD $ARGV -scale  scaleVal
getArgD $ARGV -hi     hi
getArgD $ARGV -factor f
getArgD $ARGV -lwf    lwf

if {[flagLoc $ARGV -reject]}   {set rejectFlag 1}
if {[flagLoc $ARGV -noreject]} {set rejectFlag 0}

if {[flagLoc $ARGV -verb]}     {set verbose 1}
if {[flagLoc $ARGV -noverb]}   {set verbose 1}

#
#------------------------------------------------------------------------------------------------------------

if {![file exists $logName]} \
   {
    puts "Error Finding SMILE Log $logName"
    exit 1
   }

set thisName [format $refName 1 1]

if {![file exists $thisName]} \
   {
    puts "Error Finding Reference Spectrum $refName"
    exit 1
   }

set error [rdFDATA $thisName fdata]

if {$error} \
   {
    puts "Error Reading Header from Reference Spectrum $refName"
    exit 1
   }

set maxSpecVal  0.0
set maxPeakVal  0.0
set noiseVal    0.0
set dimCount    0
set pkCount     0
set threshCount 0

set dimName(2) X
set dimName(1) Y
set dimName(3) Z
set dimName(4) A 

set sizeInfo(X) [getParmI $fdata NDSIZE CUR_XDIM]
set sizeInfo(Y) [getParmI $fdata NDSIZE CUR_YDIM]
set sizeInfo(Z) [getParmI $fdata NDSIZE CUR_ZDIM]
set sizeInfo(A) [getParmI $fdata NDSIZE CUR_ADIM]

set offInfo(X)  [getParmI $fdata NDX1 CUR_XDIM]
set offInfo(Y)  [getParmI $fdata NDX1 CUR_YDIM]
set offInfo(Z)  [getParmI $fdata NDX1 CUR_ZDIM]
set offInfo(A)  [getParmI $fdata NDX1 CUR_ADIM]

if {$offInfo(X) > 0} {set offInfo(X) [expr $offInfo(X) - 1]}
if {$offInfo(Y) > 0} {set offInfo(Y) [expr $offInfo(Y) - 1]}
if {$offInfo(Z) > 0} {set offInfo(Z) [expr $offInfo(Z) - 1]}
if {$offInfo(A) > 0} {set offInfo(A) [expr $offInfo(A) - 1]}

set dimOrder1 [getParmI $fdata FDDIMORDER1 NULL_DIM]
set dimOrder2 [getParmI $fdata FDDIMORDER2 NULL_DIM]
set dimOrder3 [getParmI $fdata FDDIMORDER3 NULL_DIM]
set dimOrder4 [getParmI $fdata FDDIMORDER4 NULL_DIM]

set error [catch {set inUnit [open $logName r]}]

if {$error} \
   {
    puts "Error Opening SMILE Log $logName"
    exit 1
   }

while {[gets $inUnit line] >= 0} \
   {
    if {[lsearch -exact $line THERMAL_NOISE] >= 0} {set noiseVal [getNamedItem THERMAL_NOISE $line]}

    if {![lineBegins "SMILE SIGNAL INFO INDEX" $line]} continue

    incr pkCount

    if {$dimCount == 0} \
       {
        if {[lsearch -exact $line X_PPM] >= 0} {set dimCount 1}
        if {[lsearch -exact $line Y_PPM] >= 0} {set dimCount 2}
        if {[lsearch -exact $line Z_PPM] >= 0} {set dimCount 3}
        if {[lsearch -exact $line A_PPM] >= 0} {set dimCount 4}
       }

    set ampVar AMP

    if {[lsearch -exact $line AMP_0] >= 0} {set ampVar AMP_0}

    set thisHi [expr abs([getNamedItem $ampVar $line])]
    if {$thisHi > $maxPeakVal} {set maxPeakVal $thisHi}
   }

if {$dimCount == 0} \
   {
    puts "Error finding SMILE SIGNAL INFO in SMILE Log File $logName"
    exit 1
   }

if {$dimCount == 1} \
   {
    set specDimList  "$dimName($dimOrder1)" 
    set pkDimList    "X"
   } \
elseif {$dimCount == 2} \
   {
    set specDimList "$dimName($dimOrder2) $dimName($dimOrder1)"
    set pkDimList   "X Y"
   } \
elseif {$dimCount == 3} \
   {
    set specDimList "$dimName($dimOrder2) $dimName($dimOrder3) $dimName($dimOrder1)"
    set pkDimList   "X Y Z"
   } \
else \
   {
    set specDimList "$dimName($dimOrder2) $dimName($dimOrder3) $dimName($dimOrder4) $dimName($dimOrder1)"
    set pkDimList   "X Y Z A"
   }

if {![strcasecmp $hi Auto]} {set hi [expr $f*$noiseVal]} 

if {![strcasecmp $hi None]} \
   {
    set threshStr None
   } \
else \
   {
    set threshStr [format %.4e $hi]
   }

if {$verbose} \
   {
    puts ""   
    puts [format "smile2tab. Spectrum Dimension List:     %s"   $specDimList]
   }

if {![strcasecmp $scaleVal Auto]} \
   {
    set maxSpecVal [exec specStat.com -in $refName -stat vMaxAbs -brief]
    set scaleVal   [safeDiv $maxSpecVal $maxPeakVal]

    if {$verbose} {puts [format "Maximum Height in Spectrum:  %.4e" $maxSpecVal]}
   }

if {$verbose} \
   {
    puts [format "smile2tab. Maximum Height in Peak List: %.4e" $maxPeakVal]
    puts [format "smile2tab. Noise Peak Threshold:        %s"   $threshStr]
    puts [format "smile2tab. SMILE to Spectrum Scale:     %.4e" $scaleVal]
    puts [format "smile2tab. SMILE Thermal Noise Value:   %.4e" $noiseVal]
    puts [format "smile2tab. SMILE Peak Count:            %d"   $pkCount]
   }

catch "seek $inUnit 0"

set error [catch {set outUnit [open $outName w]}]

puts $outUnit ""
puts -nonewline $outUnit "VARS   INDEX"

foreach dim $pkDimList {puts -nonewline $outUnit " ${dim}_AXIS"}
foreach dim $pkDimList {puts -nonewline $outUnit " D${dim}"}
foreach dim $pkDimList {puts -nonewline $outUnit " ${dim}_PPM"}
foreach dim $pkDimList {puts -nonewline $outUnit " ${dim}_HZ"}
foreach dim $pkDimList {puts -nonewline $outUnit " ${dim}W"}
foreach dim $pkDimList {puts -nonewline $outUnit " ${dim}W_HZ"}
foreach dim $pkDimList {puts -nonewline $outUnit " ${dim}1 ${dim}3"}

puts $outUnit " HEIGHT DHEIGHT VOL PCHI2 TYPE ASS CLUSTID MEMCNT"

puts -nonewline $outUnit "FORMAT %5d"

foreach dim $pkDimList {puts -nonewline $outUnit " %9.3f"}
foreach dim $pkDimList {puts -nonewline $outUnit " %6.3f"}
foreach dim $pkDimList {puts -nonewline $outUnit " %8.3f"}
foreach dim $pkDimList {puts -nonewline $outUnit " %9.3f"}
foreach dim $pkDimList {puts -nonewline $outUnit " %8.3f"}
foreach dim $pkDimList {puts -nonewline $outUnit " %8.3f"}
foreach dim $pkDimList {puts -nonewline $outUnit " %5d %5d"}

puts $outUnit " %+.4e %+.4e %+.4e %.4f %d %s %5d %5d"
puts $outUnit ""

set k 0

while {[gets $inUnit line] >= 0} \
   {
    if {![lineBegins "SMILE SIGNAL INFO INDEX" $line]} continue

    set ampVar AMP

   if {[lsearch -exact $line AMP_0] >= 0} {set ampVar AMP_0}

    set pkInfo(INDEX)   [getNamedItem INDEX  $line] 
    set pkInfo(HEIGHT)  [expr $scaleVal*[getNamedItem $ampVar $line]]
    set pkInfo(DHEIGHT) $noiseVal
    set pkInfo(TYPE)    1

    if {[strcasecmp $hi None]} \
       {
        if {$pkInfo(HEIGHT) > $hi} \
           {
            incr threshCount
           } \
        else \
           {
            set pkInfo(TYPE) 2
            if {$rejectFlag} continue
           }
       }

    incr k

    set pkInfo(CLUSTID) $k

    puts -nonewline $outUnit [format " %5d" $pkInfo(INDEX)]

    set lwMult 1.0

    for {set i 0} {$i < $dimCount} {incr i} \
       {
        set smileDimCode [lindex "X Y Z A"     $i]
        set specDimCode  [lindex $specDimList  $i]

        set pkInfo(${specDimCode}_PPM)  [getNamedItem ${smileDimCode}_PPM  $line] 
        set pkInfo(${specDimCode}W_HZ)  [getNamedItem ${smileDimCode}W_HZ  $line] 
        set pkInfo(${specDimCode}W)     [getNamedItem ${smileDimCode}W_PTS $line] 
        set pkInfo(${specDimCode}W)     [spec2pntW $fdata $specDimCode $pkInfo(${specDimCode}W_HZ)hz]

        set pkInfo(${specDimCode}_AXIS) [getNamedItem ${smileDimCode}_PTS $line]
        set pkInfo(${specDimCode}_AXIS) [spec2pnt  $fdata $specDimCode $pkInfo(${specDimCode}_PPM)ppm]
        set pkInfo(${specDimCode}_HZ)   [pnt2spec  $fdata $specDimCode $pkInfo(${specDimCode}_AXIS) hz]
        set pkInfo(${specDimCode}1)     [expr int(-0.5 + $pkInfo(${specDimCode}_AXIS) - $lwf*$pkInfo(${specDimCode}W))]
        set pkInfo(${specDimCode}3)     [expr int( 0.5 + $pkInfo(${specDimCode}_AXIS) + $lwf*$pkInfo(${specDimCode}W))]

        set pkInfo(${specDimCode}_AXIS) [expr $pkInfo(${specDimCode}_AXIS) + $offInfo($specDimCode)]

        set lwMult [expr $lwMult*$pkInfo(${specDimCode}W)]
       }

    set pkInfo(VOL) [expr $lwMult*$pkInfo(HEIGHT)]

    foreach dim $pkDimList {puts -nonewline $outUnit [format " %9.3f"   $pkInfo(${dim}_AXIS)]}
    foreach dim $pkDimList {puts -nonewline $outUnit [format " %6.3f"   0.14]}
    foreach dim $pkDimList {puts -nonewline $outUnit [format " %8.3f"   $pkInfo(${dim}_PPM)]}
    foreach dim $pkDimList {puts -nonewline $outUnit [format " %9.3f"   $pkInfo(${dim}_HZ)]}
    foreach dim $pkDimList {puts -nonewline $outUnit [format " %8.3f"   $pkInfo(${dim}W)]}
    foreach dim $pkDimList {puts -nonewline $outUnit [format " %8.3f"   $pkInfo(${dim}W_HZ)]}
    foreach dim $pkDimList {puts -nonewline $outUnit [format " %5d %5d" $pkInfo(${dim}1) $pkInfo(${dim}3)]}

    set fmt " % .4e % .4e % .4e %.4f %d %s %5d %5d"

    puts $outUnit [format $fmt $pkInfo(HEIGHT) $pkInfo(DHEIGHT) $pkInfo(VOL) 0.0 $pkInfo(TYPE) None $pkInfo(CLUSTID) 1]
   }

if {[strcasecmp $hi None]} \
   {
    if {$verbose} {puts [format "smile2tab. Peaks Above Threshold:       %d"   $threshCount]}
   }

if {$verbose} {puts ""}

# X_AXIS DX X_PPM X_HZ XW XW_HZ X1 X3 HEIGHT DHEIGHT VOL PCHI2 TYPE ASS CLUSTID MEMCNT
#
# SMILE SIGNAL INFO INDEX  1 X_PTS 4834.589 Y_PTS 1300.253 XW_PTS  8.692 YW_PTS  5.093 X_PPM 116.358
#                   Y_PPM 8.924 XW_HZ   2.122 YW_HZ   6.728 HEIGHT  4.369134e+05 AMP_0  8.461934e+00

catch "close $outUnit"
catch "close $inUnit"

exit 0
