#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -backingstore -- "$@"

#
# Creates contour plot images, mask images, or pixel coord lists from list of related spectra.
# frank.delaglio@nist.gov 

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

set thisVersion  2024.008.22.22

set specList     test.ft2
set outName      Auto
set outDir       Auto
set mode         cont

set titleFlag    0
set greenFlag    0

set imSizeX      256
set imSizeY      256

set topMargin    10
set bottomMargin 10

set leftMargin   10
set rightMargin  10

set hiList       7.0

set pSat1    1.0
set pSatN    1.0
set pHue1    0.667
set pHueN    0.667
set pVal1    0.75
set pValN    1.00

set nSat1    1.0
set nSatN    1.0
set nHue1    0.0
set nHueN    0.0
set nVal1    0.75
set nValN    1.00

set plev      16
set nlev      16
set level     0

set x1       0%
set xn     100%
set y1       0%
set yn     100%

getArgD $ARGV -mode mode

if {[flagLoc $ARGV -multiColor]} \
   {
    set pSat1    1.0
    set pSatN    1.0
    set pHue1    0.5
    set pHueN    0.7
    set pVal1    0.8
    set pValN    1.0

    set nSat1    1.0
    set nSatN    1.0
    set nHue1    0.0
    set nHueN    0.2
    set nVal1    0.8
    set nValN    1.0
   }

if {![strcasecmp $mode mask]} \
   {
    set outDir mask
    set hiList 0.5
   } \
elseif {![strcasecmp $mode cont]} \
   {
    set outDir cont
   } \
elseif {![strcasecmp $mode coords]} \
   {
    set outDir coords 
   } elseif {![strcasecmp $mode pix]} \
   {
    set outDir pix 
   }

getListArgD $ARGV -in specList
getListArgD $ARGV -hi hiList

if {[flagLoc $ARGV -greenMask]} {set greenFlag 1}
if {[flagLoc $ARGV -redMask]}   {set greenFlag 0}

if {[flagLoc $ARGV -useTitle]}   {set titleFlag 1}
if {[flagLoc $ARGV -nouseTitle]} {set titleFlag 0}

set specCount [llength $specList]

set specName  [lindex  $specList 0]
set hi        [lindex  $hiList end]

set vList {
           "outDir  Output Directory."
           "outName Output Name, or Keyword Auto."
           "mode    Output Mode (Keywords: cont mask coords)."
           "plev    Positive Contour Count."
           "nlev    Negative Contour Count."
           "level   Contour Level Number (For -mode coords)."
           "- Region Coords (ppm Hz pts %):"
           "x1      X-Axis Region Start."
           "xn      X-Axis Region End."
           "y1      Y-Axis Region Start."
           "yn      Y-Axis Region End."
           " - Image Size:"
           "imSizeX X-Axis Size of Output Image, Pixels."
           "imSizeY Y-Axis Size of Output Image, Pixels."
           "leftMargin -"
           "rightMargin -"
           "topMargin -"
           "bottomMargin -"
           "- Positive Contour Colors (Values 0.0 to 1.0):"
           "pHue1   Start Hue."
           "pHueN   End Hue."
           "pSat1   Start Saturation." 
           "pSatN   End Saturation."
           "pVal1   Start Value."
           "pValN   End Value."
           "- Negative Contour Colors (Values 0.0 to 1.0):"
           "nHue1   Start Hue."
           "nHueN   End Hue."
           "nSat1   Start Saturation."
           "nSatN   End Saturation."
           "nVal1   Start Value."
           "nValN   End Value."
          }

foreach vInfo $vList \
   {
    set v [lindex $vInfo 0]

    if {![strcmp $v "-"]} continue

    getArgD $ARGV -${v} $v
   }

if {$titleFlag} {set outName Auto}

set cx1 [expr $leftMargin + 1]
set cxn [expr $cx1 + $imSizeX - 1]

set cy1 [expr $topMargin + 1]
set cyn [expr $cy1 + $imSizeY - 1]

set winSizeX [expr $leftMargin   + $imSizeX + $rightMargin]
set winSizeY [expr $bottomMargin + $imSizeY + $topMargin]

if {[flagLoc $ARGV -help]} \
   {
    puts "Create contour images, mask images, or contour coord lists ($thisVersion)."
    puts "(Requires the image utility mogrify):"

    puts [format " -%-8s %-10s  %s" in \[$specList\] "List of Input Spectra."]
    puts [format " -%-8s %-10s  %s" hi \[$hiList\]   "List of Contour Heights."]

    foreach vInfo $vList \
       {
        set v   [lindex $vInfo 0]
        set str [lrange $vInfo 1 end]

        if {![strcmp $v "-"]} \
           {
            puts $str
           } \
        else \
           {
            if {[strcmp $str "-"]} {puts [format " -%-8s %-10s  %s" $v \[[subst \$$v]\] $str]}
           }
       }

    puts "Other Options:"
    puts " -multiColor           Use nmrDraw Color Scheme."
    puts " -useTitle             Use Spectrum Title for Output Name."
    puts " -redMask              Use Red Values in Mask Output Image (Default)."
    puts " -greenMask            Use Green Values in Mask Output."
    exit 0
   }

#
#-----------------------------------------------------------------------------------------------

if {![file isdirectory $outDir]} {catch "file mkdir $outDir"} 

canvas .c -width $winSizeX -height $winSizeY -border 0

pack   .c -side top

.c create pixmap 0 0 -width $winSizeX -height $winSizeY -win 1

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -verb -win 1

foreach specName $specList {lappend hiList $hi}

set hiList [lrange $hiList 0 [expr $specCount - 1]]
set nn     [llength $specList]

set i 0

foreach specName $specList \
   {
    if {![file exists $specName]} \
       {
        puts "spec2image. Error finding input spectrum $specName"
        exit 1
       }

    if {$nn == 1} \
       {
        set str ""
       } \
    else \
       {
        set str "[expr $i + 1] of ${nn}."
       }

    set hi [lindex $hiList $i]

    readROI -ndim 2 -roi 1 -in $specName -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn -naux 1

    if {$titleFlag} \
       {
        set rootName [getParmStr [lindex [getROIInfo -roi 1] 1] FDTITLE NULL_DIM]
       } \
    else \
       {
        set rootName [file root [file tail $specName]]
       }

    if {![strcasecmp $outName Auto]} \
       {
        set imageName $outDir/${rootName}.tiff
        wm title . "$str $specName $imageName $mode"
       } \
    else \
       {
        set imageName $outDir/$outName
        wm title . " $str $specName $imageName $mode"
       }

    update

    drawRec -x 0 $winSizeX -y 0 $winSizeY -fg dimGray

     if {![strcasecmp $mode cont]} \
        {
         set tmpName tmp.gif

         puts "spec2image. Mode: $mode Spectrum: $specName Hi: $hi Output: $imageName"

         drawROI -roi 1 -cell 1 -bg black -x 0% 100% -y 0% 100% -nobox -outPixmap $tmpName \
                 -pHue1 $pHue1 -pHueN $pHueN -pVal1 $pVal1 -pValN $pValN -pSat1 $pSat1 -pSatN $pSatN -pcc $plev -plev $plev \
                 -nHue1 $nHue1 -nHueN $nHueN -nVal1 $nVal1 -nValN $nValN -nSat1 $nSat1 -nSatN $nSatN -ncc $nlev -nlev $nlev \
                 -color1 multi -color2 multi -hi $hi

         exec mogrify -type TrueColor -format tiff -compress lzw -write $imageName $tmpName

         catch "file delete $tmpName"
        } \
     elseif {![strcasecmp $mode pix]} \
        {
         set tmpName tmp.gif

         puts "spec2image. Mode: $mode Spectrum: $specName Output: $imageName"

         writeImage -tiff -out $imageName -ov -cell 1 \
                    -roiR  1      -roiG  1   -roiB  1   \
                    -rRMin 0.0    -rGMin 0.0 -rBMin 0.0 \
                    -rRMax 255.0  -rGMax 0.0 -rBMax 0.0 \
                    -iRMin 0      -iGMin 0   -iBMin 0   \
                    -iRMax 255    -iGMax 0   -iBMax 0

         catch "file delete $tmpName"
        } \
     elseif {![strcasecmp $mode mask]} \
        {
         set tmpName tmp.tiff

         puts "spec2image. Mode: $mode Spectrum: $specName Output: $imageName"

         set rInfo  [getROIInfo -roi 1 -n -aux]
         set sInfo  [lindex $rInfo 2]
         set n      [expr [lindex $sInfo 0]*[lindex $sInfo 1]]

         set rdata  [lindex $rInfo 0]
         set mdata  [lindex [lindex $rInfo 4] 0]

         vvMove $mdata $rdata $n

         vsMult $rdata  10.0  $n
         vsAdd  $rdata 100.0  $n 

         vvMask $rdata $mdata $n

         if {$greenFlag} \
            {
             writeImage -tiff -out $tmpName -ov -cell 1 \
                        -roiR  1      -roiG  1     -roiB  1   \
                        -rRMin 0.0    -rGMin 0.0   -rBMin 0.0 \
                        -rRMax 0.0    -rGMax 255.0 -rBMax 0.0 \
                        -iRMin 0      -iGMin 0     -iBMin 0   \
                        -iRMax 0      -iGMax 255   -iBMax 0
            } \
         else \
            {
             writeImage -tiff -out $tmpName -ov -cell 1 \
                        -roiR  1      -roiG  1   -roiB  1   \
                        -rRMin 0.0    -rGMin 0.0 -rBMin 0.0 \
                        -rRMax 255.0  -rGMax 0.0 -rBMax 0.0 \
                        -iRMin 0      -iGMin 0   -iBMin 0   \
                        -iRMax 255    -iGMax 0   -iBMax 0
            }

         exec mogrify -type TrueColor -format tiff -compress lzw -write $imageName $tmpName
        } \
     elseif {![strcasecmp $mode coords]} \
        {
         if {![strcasecmp $outName Auto]} \
            {
             set coordName $outDir/${rootName}.txt
            } \
         else \
            {
             set coordName $outDir/$outName
            }

         puts "spec2image. Mode: $mode Spectrum: $specName Hi: $hi Output: $coordName Level: $level"

         drawROI -roi 1 -cell 1 -bg black -x 0% 100% -y 0% 100% -nobox -outCoords $coordName -level $level \
                 -pHue1 $pHue1 -pHueN $pHueN -pVal1 $pVal1 -pValN $pValN -pSat1 $pSat1 -pSatN $pSatN -pcc $plev -plev $plev \
                 -nHue1 $nHue1 -nHueN $nHueN -nVal1 $nVal1 -nValN $nValN -nSat1 $nSat1 -nSatN $nSatN -ncc $nlev -nlev $nlev \
                 -color1 multi -color2 multi -hi $hi
        } \
     else \
        {
         puts "spec2image. Error: unknown mode $mode."
         exit 1
        }

    incr i 
   }

exit 0

