#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2020.193.20.53"

#
# pcaNMR.com -pcaTab pca005/pca.tab -pcaArgs -in pca005/ft/test%05d.dat -nc 6 \
#            -score pca005/score%03d.dat -load pca005/load.dat -pca pca005/pca/pca%03d.dat \
#            -noscale -avg -itmin 8 -conv 1.0e-6

if {[flagLoc $ARGV -help]} \
   {
    puts "Calculate Spectral Components (Version $thisVersion):"
    puts " -in       inName    \[ft/test%03d.ft2\] Input Series (Series of 1D or 2D)."
    puts " -nc       nC        \[6\]               Number of Components to Calculate." 
    puts "Outputs:"
    puts " -pcaTab   pcaTab    \[specVar.tab\]     Table of Component Statistics."
    puts " -coordTab coordTab  \[specCoords.tab\]  Table of Component Coordinates."
    puts " -load     loadName  \[load.dat\]        Component Loadings, or Keyword None."
    puts " -score    scoreName \[score%03d.dat\]   Component Scores, or Keyword None."
    puts " -pca      pcaName   \[pca%03d.dat\]     Component Spectra, or Keyword None."
    puts " -norm     normName  \[norm%03d.dat\]    Sign-Adjusted Normalized Spectra, or Keyword None."
    puts " -prefix   prefix    \[tmpcomp\$\$]    Prefix for Temporary Files (\$\$ is the Process ID)."
    puts "Component Calculation:"
    puts " -metric   metric    \[L2\]              Component Metric: L0 L1 L2 LINF CHI2 KL EMD."
    puts " -weightFn weightFn  \[GaussNearest\]    Weighting Function: GaussNearest GaussAll Linear."
    puts " -gFactor  gFactor   \[1.0\]             Gauss Factor for Weight Function."
    puts " -thresh   thresh    \[0.0\]             Threshold for Metric (Used for CHI2 KL EMD)."
    puts " -sort                                 Sort Components from Largest to Smallest (Default)."
    puts " -nosort                               Do Not Sort Components."
    puts "Normalization:"
    puts " -signMode signMode  \[None\]            Sign Adjustment: Abs KeepPos KeepNeg None."
    puts " -normFn   normFn    \[None\]            Normalization Function, or Keyword None."
    puts " -mult     mult      \[1.0\]             Post-Normalization Multiplier."
    puts " -clipMin  minVal    \[None\]            Lower Threshold for Intensity Clipping."
    puts " -clipMax  maxVal    \[None\]            Upper Threshold for Intensity Clipping."
    puts " -center                               Use Centering (Subtraction of Average)."
    exit 0
   }

set inName ft/test%03d.ft2
set nC     6

set loadName   load.dat
set scoreName  score%03d.dat
set pcaName    pca%03d.dat
set normName   norm%03d.dat
set resDir     res
set colorMode  Title

set coordTabName specCoords.tab
set varTabName   specVar.tab
set prefix       tmpcomp[pid]

set compMetric L2
set weightFn   GaussNearest
set normFn     None
set clipMin    None
set clipMax    None
set signMode   None
set thresh     0.0
set gFactor    1.0
set mult       1.0
set centerFlag 0
set sortFlag   1

set colorList {
               cyan    yellow      red    darkOrchid green     blue  orange \
               magenta forestGreen violet pink       firebrick brown
              }

getArgD $ARGV -in       inName
getArgD $ARGV -nc       nC

getArgD $ARGV -pcaTab   varTabName
getArgD $ARGV -coordTab coordTabName
getArgD $ARGV -load     loadName
getArgD $ARGV -score    scoreName
getArgD $ARGV -pca      pcaName
getArgD $ARGV -norm     normName
getArgD $ARGV -resDir   resDir
getArgD $ARGV -prefix   prefix

getArgD $ARGV -signMode signMode
getArgD $ARGV -metric   compMetric
getArgD $ARGV -weightFn weightFn
getArgD $ARGV -gFactor  gFactor
getArgD $ARGV -normFn   normFn
getArgD $ARGV -thresh   thresh
getArgD $ARGV -mult     mult

getArgD $ARGV -compColorMode colorMode

getListArgD $ARGV -compColors colorList

if {[flagLoc $ARGV -sort]}   {set sortFlag 1}
if {[flagLoc $ARGV -nosort]} {set sortFlag 0}

if {[flagLoc $ARGV -center]}   {set centerFlag 1}
if {[flagLoc $ARGV -nocenter]} {set centerFlag 0}

#
#--------------------------------------------------------------------------------------------

proc getOffDiagonalScoreList { i } \
{
   global distMatrix specCount

   set sList ""

   for {set j 1} {$j <= $specCount} {incr j} \
      {
       if {$i != $j} {lappend sList $distMatrix($i,$j)}
      } 

   return $sList
}

proc applySignAdjustment { i } \
{
   global rdata n signMode

   if {![strcasecmp $signMode None]} \
      {
       return
      } \
   elseif {![strcasecmp $signMode Abs]} \
      {
       vAbs $rdata($i) $n
      } \
   elseif {![strcasecmp $signMode KeepPos]} \
      {
       vClipNeg $rdata($i) $n
      } \
   elseif {![strcasecmp $signMode KeepNeg]} \
      {
       vClipPos $rdata($i) $n
      } \
   else \
      {
       puts "specComp. Error: Unknown Sign Adjustment Mode $signMode (-signMode)."
       exit 1
      }
}

proc applyClipping { i } \
{
   global rdata n clipMin clipMax

   if {[strcasecmp $clipMin None]} {vsClipBelow $rdata($i) $clipMin $n} 
   if {[strcasecmp $clipMax None]} {vsClipAbove $rdata($i) $clipMax $n} 
}

proc applyNorm { i } \
{
   global rdata n normFn mult thresh adjustedThresh

   set val 1.0

   if {![strcasecmp $normFn None]} \
      {
       set val 1.0
      } \
   elseif {![strcasecmp $normFn Max]} \
      {
       set val [vMaxAbs $rdata($i) $n] 
      } \
   elseif {![strcasecmp $normFn Sum]} \
      {
       set val [vSum $rdata($i) $n]
      } \
   elseif {![strcasecmp $normFn Range]} \
      {
       set val [vRange $rdata($i) $n]
      } \
   elseif {![strcasecmp $normFn Variance]} \
      {
       set val [vVariance $rdata($i) $n]
      } \
   elseif {![strcasecmp $normFn RMS]} \
      {
       set val [vRMS $rdata($i) $n]
      } \
   elseif {![strcasecmp $normFn Noise]} \
      {
       set val [vEstNoise $rdata($i) $n]
      } \
   else \
      {
       puts "specComp. Error: Unknown Normalization Function $normFn (-normFn)."
       exit 1
      }

   set s [safeDiv $mult $val]

   if {[string is double $thresh]} {set thresh $thresh*$s}

   vsMult $rdata($i) $s $n
}

proc applyScaling { i } \
{
}

proc applyCentering { i } \
{
}

proc getScoreList { i } \
{
   global distMatrix specCount

   set sList ""

   for {set j 1} {$j <= $specCount} {incr j} \
      {
       lappend sList $distMatrix($i,$j)
      } 

   return $sList
}

proc getAvgSpecScore { i } \
{
   global distMatrix specCount

   set d [getAvg [getOffDiagonalScoreList $i]]

   return $d
}

proc buildComponent { i iComp } \
{
   global specCount compID n rdata work1 scaleMatrix weightFn
 
   set mList ""
 
   for {set j 1} {$j <= $specCount} {incr j} \
      {
       lappend mList [getMin [getOffDiagonalScoreList $j]]
      }
 
   set sList [getScoreList            $i]
   set oList [getOffDiagonalScoreList $i]

   set devNearest [getAvg    $mList]
   set devAll     [getStdDev $oList]
   set maxVal     [getMax    $oList]

   if {$devNearest == 0.0} {set devNearest 1.0}
   if {$devAll     == 0.0} {set devAll     1.0}

   set qList ""

   for {set j 1} {$j <= $specCount} {incr j} \
      {
       set s [lindex $sList [expr $j - 1]]
       set q 0.0

       if {![strcasecmp $weightFn Linear]} \
          {
           if {$maxVal != 0.0} {set q [expr (1.0/$specCount) + 1.0 - ($s/$maxVal)]}
          } \
       elseif {![strcasecmp $weightFn GaussAll]} \
          {
           set q [expr exp( -($s*$s)/($devAll*$devAll) )]
          } \
       elseif {![strcasecmp $weightFn GaussNearest]} \
          {
           set q [expr exp( -($s*$s)/(2.0*$devNearest*$devNearest) )]
          } \
       else \
          {
           puts "specComp. Error: Unknown Weight Function $weightFn (-weightFn)."
           exit 1
          }

       lappend qList $q
      }

#  foreach s $sList {lappend qList [expr exp( -($s*$s)/($devAll*$devAll) )]}
#  foreach s $sList {lappend qList [expr (1.0/$specCount) + 1.0 - ($s/$maxVal)]}

   set sum [getSum $qList]

   if {$sum == 0.0} {set sum 1.0}

   vsSet $rdata($compID) 0.0 $n

   for {set j 1} {$j <= $specCount} {incr j} \
      {
       set q [lindex $qList [expr $j - 1]]

       set scaleMatrix($iComp,$j) [expr $q/$sum]

       vsMult $work1 $rdata($j) $scaleMatrix($iComp,$j) $n
       vvAdd $rdata($compID) $work1 $n
      }
}

proc getThisDist { i j } \
{
   global compMetric rdata n work1 work2 thresh

   if {![strcasecmp $compMetric L0]} \
      {
       vvMove $work1 $rdata($i) $n
       vvMove $work2 $rdata($j) $n

       vBinarize $work1 $thresh $n
       vBinarize $work2 $thresh $n

       vAbs $work1 $n
       vAbs $work2 $n

       vvSub $work1 $work2 $n
       vAbs  $work1 $n

       set d [vSum $work1 $n]
      } \
   elseif {![strcasecmp $compMetric L1]} \
      {
       vvSub $work1 $rdata($i) $rdata($j) $n
       vAbs  $work1 $n

       set d [vSum $work1 $n]
      } \
   elseif {![strcasecmp $compMetric L2]} \
      {
       vvSub $work1 $rdata($i) $rdata($j) $n
       set d [vRMS $work1 $n]
      } \
   elseif {![strcasecmp $compMetric LInf]} \
      {
       vvSub $work1 $rdata($i) $rdata($j) $n
       set d [vMaxAbs $work1 $n]
      } \
   elseif {![strcasecmp $compMetric Chi2]} \
      {
       vAbs  $work1 $rdata($i) $n
       vAbs  $work2 $rdata($j) $n

       vvAdd $work2 $work1 $n
       vvSub $work1 $rdata($i) $rdata($j) $n
       vvDiv $work1 $work2 $n
    
       set d [vRMS $work1 $n]
      } \
   elseif {![strcasecmp $compMetric EMD]} \
      {
       puts "specComp. Unsupported metric $compMetric."
       exit 1
      } \
   else \
      {
       puts "specComp. Unknown metric $compMetric."
       exit 1
      }

   return $d
}

proc getCompColor { i } \
{
   global specCount inName colorList colorMode title clustID

   set nColors [llength $colorList]
   set mode    [string tolower $colorMode]
   set modeVal $mode

   if {[string is int $mode]} {set mode int}

   set thisColor gray

   switch -exact $mode \
      {
       title -
       int   -
       last {
             set sList ""
      
             if {![strcmp $mode last]} \
                {
                 set item end
                } \
             elseif {![strcmp $mode int]} \
                {
                 set mode $modeVal
                } \
             else \
                {
                 set item $mode
                }

             set t [string toupper $title($i)]

             if {![strcmp $mode last]} \
                {
                 set s [lindex [split $t -] end]
                } \
             elseif {![strcmp $mode title]} \
                {
                 set s $t
                } \
             else \
                {
                 set s [lindex [split $t -] [expr $mode - 1]]
                }

             set thisColor [lindex $colorList [hashCode item$s $nColors]]
            }

       file {
             set s [file tail [format $inName $i]]

             set thisColor [lindex $colorList [hashCode $s $nColors]]
            }

       number {
               set thisColor [lindex $colorList [expr ($i - 1) % $nColors]]
              }

       cluster {
                if {[info exists clustID($i)]} \
                   {
                    set cID $clustID($i)
                   } \
                else \
                   {
                    set cID 0
                   }

                set thisColor [lindex $colorList [expr ($cID - 1) % $nColors]]
               }

       default {
                set thisColor [lindex $colorList [expr ($i - 1) % $nColors]]
               }
      }

   return $thisColor
}

set adjustedThresh $thresh

set cleanList ""

if {$nC < 1} \
   {
    puts "specComp. Error: Number of Components Must Be Greater Than Zero ($nC)."
    exit 1
   }

set thisName [format $inName 1]

if {![file exists $thisName]} \
   {
    puts "specComp. Error Finding Input $thisName ($inName)."
    exit 1
   }

set error [rdFDATA $thisName fdataOrig]

if {$error} \
   {
    puts "specComp. Error Reading Header in Input $thisName ($inName)."
    exit 1
   }

set dimCount  [getParmI $fdataOrig FDDIMCOUNT NULL_DIM]
set xSizeOrig [getParmI $fdataOrig NDSIZE     CUR_XDIM]
set ySizeOrig [getParmI $fdataOrig NDSIZE     CUR_YDIM]

if {$dimCount > 1 && $ySizeOrig == 1} {set dimCount 1}

set specCount 0

for {set i 1} {$i < 99999} {incr i} \
    {
     set thisName [format $inName $i]

     if {![file exists $thisName]} \
        {
         incr i -1
         set specCount $i
         break
        }
    }

if {$specCount < 2} \
   {
    puts "specComp. Error Finding Two or More Spectra ($inName)."
    exit 1
   }

puts "specComp. Input:            $inName"
puts "specComp. Spectrum Count:   $specCount"
puts "specComp. Component Count:  $nC"
puts "specComp. Component Metric: $compMetric"
puts "specComp. Metric Weighing:  $weightFn"
puts "specComp. Weighing Factor:  $gFactor"
puts "specComp. Sign Adjustment:  $signMode"

if {$nC < 1} {exit 0}

if {[strcasecmp $resDir None]} \
   {
    catch "file delete -force $resDir"
    catch "file mkdir $resDir"
   }

set compOffset [expr 100 + 100*int($specCount/100)]
set nTMax      0

for {set i 1} {$i <= $specCount} {incr i} \
   {
    if {$dimCount == 1} \
       {
        readROI -ndim 1 -roi $i -in [format $inName $i] -x X_AXIS 0% 100%
       } \
    else \
       {
        readROI -ndim 2 -roi $i -in [format $inName $i] -x X_AXIS 0% 100% -y Y_AXIS 0% 100%
       }

    set rInfo($i) [getROIInfo -roi $i]

    set rdata($i)   [lindex $rInfo($i) 0]
    set fdata($i)   [lindex $rInfo($i) 1]

    set title($i)   [getParmStr $fdata($i) FDTITLE NULL_DIM]
    set clustID($i) $i

    set nT [string length $title($i)]
    if {$nT > $nTMax} {set nTMax $nT}
   }

incr nTMax 1

for {set iComp 1} {$iComp <= $nC} {incr iComp} \
   {
    set i [expr $iComp + $compOffset]

    if {$dimCount == 1} \
       {
        readROI -ndim 1 -roi $i -in [format $inName 1] -x X_AXIS 0% 100%
       } \
    else \
       {
        readROI -ndim 2 -roi $i -in [format $inName 1] -x X_AXIS 0% 100% -y Y_AXIS 0% 100%
       }

    set rInfo($i) [getROIInfo -roi $i]

    set rdata($i) [lindex $rInfo($i) 0]
    set fdata($i) [lindex $rInfo($i) 1]

    set title($i) COMP$iComp
   }

set compID [expr 1 + $compOffset]

if {$dimCount == 1} \
   {
    set xSize [lindex [lindex $rInfo(1) 2] 0]
    set ySize 1
   } \
else \
   {
    set xSize [lindex [lindex $rInfo(1) 2] 0]
    set ySize [lindex [lindex $rInfo(1) 2] 1]
   }

set n     [expr $xSize*$ySize]
set k     0

set work1    [sysMalloc $n float]
set work2    [sysMalloc $n float]
set loadBuff [sysMalloc [expr $specCount*$nC] float]
set fdataL   [sysMalloc $FDATASIZE float]

set pctList   ""
set bestList  ""

set vOrig     0.0
set maxVal    0.0

set error [catch {set outUnit [open $coordTabName w]}]

for {set i 1} {$i <= $specCount} {incr i} \
   {
    applySignAdjustment $i
    applyClipping       $i
    applyNorm           $i
    applyScaling        $i
    applyCentering      $i

    set thisMax [vMaxAbs $rdata($i) $n]
    if {$thisMax > $maxVal} {set maxVal $thisMax}

    set vOrig [expr $vOrig + [vVariance $rdata($i) $n]]
   }

set varianceInfo(0) $vOrig
set maxInfo(0)      $maxVal

for {set iComp 1} {$iComp <= $nC} {incr iComp} \
   {
    set compID [expr $iComp + $compOffset]

    set vInit  0.0
    set vFinal 0.0

    for {set i 1} {$i <= $specCount} {incr i} {set vInit [expr $vInit + [vVariance $rdata($i) $n]]}

    for {set i 1} {$i <= $specCount} {incr i} \
       {
        set thisNameI [format $inName $i]

        for {set j 1} {$j <= $specCount} {incr j} \
           {
            set d 0.0

            if {$i == $j} \
               {
                set distMatrix($i,$j) 0.0
               } \
            elseif {$i < $j} \
               {
                incr k 

                set d [getThisDist $i $j]

#               puts [format " %d %3d. Pair: %3d %3d d$compMetric: % .4e (%s %s)" $iComp $k $i $j $d $title($i) $title($j)]

                set distMatrix($i,$j) $d
                set distMatrix($j,$i) $d
               }
           }
       }

    set bestScore($iComp) 0.0
    set bestSpec($iComp)  0

    for {set i 1} {$i <= $specCount} {incr i} \
       {
        set d [getAvgSpecScore $i]

#       if {[lsearch -exact $bestList $i] >= 0} continue

        if {!$bestSpec($iComp) || $d < $bestScore($iComp)} \
           {
            set bestScore($iComp) $d
            set bestSpec($iComp)  $i
           }
       }

    lappend bestList $bestSpec($iComp)

    buildComponent $bestSpec($iComp) $iComp

    vvMove $work2 $rdata($compID) $n

    set maxVal 0.0

    for {set i 1} {$i <= $specCount} {incr i} \
       {
        set scoreMatrix($iComp,$i) [vvGetScale $rdata($i) $work2 $n]

        vsMult $work1 $work2 $scoreMatrix($iComp,$i) $n
        vvSub  $rdata($i) $work1 $n

        set vFinal  [expr $vFinal + [vVariance $rdata($i) $n]]
        set thisMax [vMaxAbs $rdata($i) $n]

        if {$thisMax > $maxVal} {set maxVal $thisMax}

        if {[strcasecmp $resDir None]} \
           {
            writeROI -roi $i -out [format $resDir/res%05d.dat $i] -ov
           }
       }

    if {$vOrig == 0.0} \
       {
        set pct 0.0
       } \
    else \
       {
        set pct [expr 100.0*($vInit - $vFinal)/$vOrig]
       }

    set varianceInfo($iComp) $vFinal
    set maxInfo($iComp)      $maxVal

    lappend pctList $pct
   }

if {$sortFlag} \
   {
    set iCompList [lreverse [getVecSortLoc $pctList]]
   } \
else \
   {
    set iCompList [getIota 1 1 $nC]
   }

set k 1

puts "specComp. Final Order:      $iCompList"
puts ""

puts $outUnit ""
puts $outUnit "#"

prettyPrintCmnd $outUnit "# " "[file tail [lindex $ARGV 0]] [lrange $ARGV 3 end]"

puts $outUnit ""
puts $outUnit "DATA COMPONENT_ORDER $iCompList"

foreach iComp $iCompList \
   {
    set compID [expr $iComp + $compOffset]

    setParmStr $fdata($compID) FDTITLE COMP$k NULL_DIM

    if {[strcasecmp $pcaName   None]} {writeROI -roi $compID -out [format $pcaName   $k] -ov}
    if {[strcasecmp $scoreName None]} {writeROI -roi $compID -out [format $scoreName $k] -ov}

    set sList ""

    for {set i 1} {$i <= $specCount} {incr i} {lappend sList $scoreMatrix($iComp,$i)}

    set range($iComp) [getRange $sList]

    if {$range($iComp) == 0.0} {set range($iComp) 1.0}

    puts [format "Component: %d Best Spectrum: %3d %-${nTMax}s d$compMetric: % .4e Explained: %6.2f%%" $k $bestSpec($iComp) $title($bestSpec($iComp)) $bestScore($iComp) [lindex $pctList [expr $iComp - 1]]]

    puts $outUnit [format "DATA COMPONENT_INFO %2d BEST_SPECTRUM %3d %-${nTMax}s METRIC $compMetric % .4e PCT_EXPLAINED %6.2f%%" $k $bestSpec($iComp) $title($bestSpec($iComp)) $bestScore($iComp) [lindex $pctList [expr $iComp - 1]]]

    incr k
   }

puts $outUnit ""
puts ""

puts -nonewline $outUnit "VARS   INDEX FILE_NAME TITLE COLOR"

for {set i 1} {$i <= $nC} {incr i} {puts -nonewline $outUnit " SCALE$i"} 
for {set i 1} {$i <= $nC} {incr i} {puts -nonewline $outUnit " PCA$i"} 
for {set i 1} {$i <= $nC} {incr i} {puts -nonewline $outUnit " PCANORM$i"} 

puts $outUnit ""
puts -nonewline $outUnit "FORMAT %5d   %s        %-${nTMax}s %-10s"

for {set i 1} {$i <= $nC} {incr i} {puts -nonewline $outUnit " %.4e"} 
for {set i 1} {$i <= $nC} {incr i} {puts -nonewline $outUnit " %.4e"} 
for {set i 1} {$i <= $nC} {incr i} {puts -nonewline $outUnit " %.4e"} 

puts $outUnit ""
puts $outUnit ""

set k 0

foreach iComp $iCompList \
   {
    for {set i 1} {$i <= $specCount} {incr i} \
       {
        setPtrVal $loadBuff $k $scoreMatrix($iComp,$i)

        incr k
       }
   }

for {set i 1} {$i <= $specCount} {incr i} \
   {
    puts -nonewline $outUnit [format " %3d %s %-${nTMax}s %12s " $i [format $inName $i] $title($i) [getCompColor $i]]

    foreach iComp $iCompList \
       {
        set sVal [expr $specCount*$scoreMatrix($iComp,$i)]

        puts -nonewline $outUnit [format " % .4e" $sVal]
       }

    foreach iComp $iCompList \
       {
        puts -nonewline $outUnit [format " % .4e" $scoreMatrix($iComp,$i)]
       }

    foreach iComp $iCompList \
       {
        puts -nonewline $outUnit [format " % .4e" [expr $scoreMatrix($iComp,$i)/$range($iComp)]]
       }

    puts $outUnit ""
   }

catch "close $outUnit"

set error [catch {set outUnit [open $varTabName w]}]

puts $outUnit ""
puts $outUnit "VARS   COMP ITER VAR    MAX    CHI2    PCHI     EXPL    SIG"
puts $outUnit "FORMAT %3d  %3d  %.4e   %.4e   %8.3f   %7.3f    %7.2f   %s"
puts $outUnit ""

set k 0

foreach iComp $iCompList \
   {
    incr k

    set compID [expr $iComp + $compOffset]
    set fmt    " %3d %d % .4e %.4e %.4e %7.3f %7.2f %s"

    puts $outUnit [format $fmt $k 1 $varianceInfo($iComp) $maxInfo($iComp) 0.0 0.0 [lindex $pctList [expr $iComp - 1]] U]
   }

catch "close $outUnit"

if {[strcasecmp $loadName None]} \
   {
    buildHdr $fdataL \
     -xN      $specCount -yN      $nC     \
     -xT      $specCount -yT      $nC     \
     -xMODE   Real       -yMODE   Real    \
     -xSW     $specCount -ySW     $nC     \
     -xOBS    1.0000     -yOBS    1.0000  \
     -xCAR    0.0000     -yCAR    0.0000  \
     -xFT     Freq       -yFT     Freq    \
     -xLAB    ID         -yLAB    PCA     \
     -ndim    2          -aq2D    States 

    set error [catch {set outUnit [sysOpen $loadName w]}]

    set error [catch "sysWrite $outUnit $fdataL   $FDATASIZE"]
    set error [catch "sysWrite $outUnit $loadBuff [expr $specCount*$nC]"]

    set error [catch "sysClose $outUnit"]
   }

exit 0
