#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Polynomial Calc or SVD Polynomial Fit of X/Y Data."
    puts "Specification of X Values; Use Only One:"
    puts " -x       xList   \[0.0 1.0 2.0 ...\]  List of X-Values."
    puts " -nx      xCount  \[None\]             Number of X-Values, 0.0, 1.0 ..."
    puts "Specification of Order and Y Values; Used Only for SVD Fit:"
    puts " -y       yVar    \[None\]             List of Y-Values."
    puts " -n       maxOrd  \[1\]                Maximum Polynomial Order."
    puts " -nMin    minOrd  \[0\]                Minimum Polynomial Order."
    puts "Specification of Coefficients; Used Only for Polynomial Calc:"
    puts " -c  cList        \[None\]             List of Polynomial Coeffients."
    puts "Specification of X-Matrix; Used as Alternative to Polynomials:"
    puts " -xMatrix xMatrix \[None\]             Matrix of X Basis Set Values."
    puts "Report Mode; Use Only One:"
    puts " -verb           Verbose Mode ON."
    puts " -brief          Format Output as Arguments for Script Use."
    exit 1
   }

set xList     None
set yList     None
set cList     None
set xMatrix   None

set nx        None
set minOrder  0
set maxOrder  1
set verbose   0
set briefFlag 0

getArgD     $ARGV -nMin minOrder 
getArgD     $ARGV -n    maxOrder 
getArgD     $ARGV -nx   nx

getListArgD $ARGV -xMatrix xMatrix
getListArgD $ARGV -x       xList
getListArgD $ARGV -y       yList
getListArgD $ARGV -c       cList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -brief]} \
   {
    set briefFlag 1
    set verbose   0
   }

#
#-----------------------------------------------------------------

#
# Build a default X list if required:

if {[strcasecmp $nx None]} \
   {
    set xList ""

    for {set i 0} {$i < $nx} {incr i} {lappend xList [expr double($i)]}
   }

#
# In SVD mode, require Y values.
# In calc-only mode, require X values.

if {![strcasecmp $cList None]} \
   {
    set ny [llength $yList]

    if {![strcasecmp $yList None] || !$ny} \
       {
        puts stderr "Error: No Y Values for SVD Fitting Mode."
        exit 1
       }

    if {![strcasecmp $xList None]} \
       {
        set xList ""
        for {set i 0} {$i < $ny} {incr i} {lappend xList [expr double($i)]} 
       }
   } \
else \
   {
    set maxOrder [expr [llength $cList] - 1]
    set nx       [llength $xList]

    if {![strcasecmp $xList None] || !$nx} \
       {
        puts stderr "Error: No X Values for Polynomial Calculation Mode."
        exit 1
       }

    if {![strcasecmp $yList None]} \
       {
        set yList ""

        for {set i 0} {$i < $nx} {incr i} {lappend yList 0.0}
       }
   }

#
# Build the matrix of polynomial basis values.

set nx [llength $xList]
set ny [llength $yList]

if {$nx != $ny} \
   {
    puts stderr "Error: X/Y Mismatch."
    exit 1
   }

if {$minOrder > $maxOrder} \
   {
    set itemp    $minOrder
    set minOrder $maxOrder
    set maxOrder $itemp
   }

if {![strcasecmp $xMatrix None]} \
   {
    set xMatrix ""

    foreach xVal $xList \
       {
        for {set i $maxOrder} {$i >= $minOrder} {incr i -1} \
           {
            lappend xMatrix [expr pow( double($xVal), $i )]
           }
       }
   }

#
# In SVD mode, find the coefficients, report as needed:

if {![strcasecmp $cList None]} \
   {
    if {([llength $xMatrix] % [llength $yList])} \
       {
        puts stderr "Error: X-Matrix/Y Size Mismatch."
        exit 1
       }

    set cList [svdLinSol -x $xMatrix -y $yList]

    if {!$briefFlag} \
       {
        if {$verbose} \
           {
            set i $maxOrder 

            foreach c $cList \
               {
                puts [format " Power: %2d Coefficient: % 10.6f" $i $c]
                incr i -1
               }
           } \
        else \
           {
            puts $cList
            exit 0
           }
       }
   }

if {[llength $cList]*[llength $xList] != [llength $xMatrix]} \
   {
    puts stderr "Error: Coefficient/X-Matrix Size Mismatch."
    exit 1
   }

#
# Report the Y values as needed:

if {$briefFlag} \
   {
    puts -nonewline "-nc [llength $cList] -nx [llength $xList] -c $cList -x $xList -y $yList"
   } \
elseif {$verbose} \
   {
    puts ""
    puts " X           Y           Polynomial  Difference"
    puts ""
   }

set yModel ""
set dList  ""

set i 0
set j 0

foreach y $yList \
   {
    set x [lindex $xList $i]
    set s 0.0

    foreach a $cList \
       {
        set b [lindex $xMatrix $j]
        set s [expr $s + $a*$b]
        incr j
       } 

    set d [expr $y - $s]
    incr i

    lappend yModel $s
    lappend dList  $d

    if {!$briefFlag} \
       {
        if {$verbose} \
           {
            puts [format "% .4e % .4e % .4e % .4e" $x $y $s $d]
           } \
        else \
           {
            puts [format "% .4e" $s]
           }
       }
   }

if {$briefFlag} \
   {
    puts " -yModel $yModel -residual $dList -rms [getRMS $dList]"
   }

exit 0
