#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#-------------------------------------------------------------------------------

set talosName pred.tab
set pdbName   None 
set csName    None 
set badFlag   0

set factor    2.0
set fc        1.0
set ip        2
set id        1
set dPhiMin   0.0
set dPsiMin   0.0

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert TALOS prediction output to XPLOR torsion restraints:"
    puts " -in       talosName  \[pred.tab\]  TALOS Prediction Output."
    puts " -fc       forceConst \[1.0]\       Force Constant Value."
    puts " -pow      exp        \[2]\         Torsion Term Exponent."
    puts " -dPhiMin  pMin       \[0.0]\       Minimum Value for DPHI Input."
    puts " -dPsiMin  pMin       \[0.0]\       Minimum Value for DPSI Input."
    puts " -f        factor     \[2.0]\       Factor for +/- Angle Range."
    puts " -keepBad                         Use All TALOS Predictions."
    puts "Sequence Info (Use Only One):"
    puts " -cs       csName     \[None\]      TALOS Chemical Shift Table Input."
    puts " -pdb      pdbName    \[None\]      PDB Input File."
    exit 0
   }

getArgD $ARGV -in      talosName
getArgD $ARGV -pdb     pdbName
getArgD $ARGV -cs      csName
getArgD $ARGV -id      id
getArgD $ARGV -fc      fc
getArgD $ARGV -pow     ip
getArgD $ARGV -f       factor
getArgD $ARGV -dPhiMin dPhiMin
getArgD $ARGV -dPsiMin dPsiMin

if {[flagLoc $ARGV -keepBad]} {set badFlag 1}

#
#-------------------------------------------------------------------------------

proc getThisRes { resID } \
{
   global pdbTab seqList r1 rN

   set resName NONE

   if {$pdbTab} \
      {
       set resName [getNthResName $pdbTab $resID]
      } \
   else \
      {
       if {$resID >= $r1 && $resID <= $rN} \
          {
           set resType [lindex $seqList [expr $resID - $r1]]
           set resName [string toupper [getResName3 $resType]]
          }
      }

   return $resName
}

proc printThisAngle { aName id aVal daVal aLo aHi } \
{
    global ATOMNAME_I ATOMNAME_J ATOMNAME_K ATOMNAME_L
    global RESNAME_I  RESNAME_J  RESNAME_K  RESNAME_L
    global RESID_I    RESID_J    RESID_K    RESID_L

    global RESNAME RESID
    global fc ip

    set str ""

    append str "assign"

    append str [format " (resid %3d and name %2s)"    $RESID_I $ATOMNAME_I]
    append str [format " (resid %3d and name %2s) \n" $RESID_J $ATOMNAME_J]

    append str "      "

    append str [format " (resid %3d and name %2s)"    $RESID_K $ATOMNAME_K]
    append str [format " (resid %3d and name %2s)"    $RESID_L $ATOMNAME_L]
    append str [format " %s % 8.2f % 8.2f %d\n" $fc $aVal $daVal $ip]

    set fmt "! %d. %s %d %s %.2f +/- %.2f (%.2f to %.2f)\n"

    puts [format $fmt $id $RESNAME $RESID $aName $aVal $daVal $aLo $aHi]
    puts $str
}

#
#-------------------------------------------------------------------------------

set seqList   "" 
set pdbTab    0
set r1        0
set rN        0

initResNames

if {![file exists $talosName]} \
   {
    puts stderr "Error Finding TALOS prediction file $talosName"
    exit 1
   } 

if {[file exists $pdbName]} \
   {
    set pdbTab [gdbRead table -in $pdbName -pdb]
   }

if {[file exists $csName]} \
   {
    set csTab   [gdbRead table -in $csName]
    set seqList [getSeqList $csTab r1 rN]
   }

if {!$pdbTab && ![llength $seqList]} \
   {
    puts stderr "Error: No Sequence Input; use the -cs or -pdb arguments."
    exit 1
   }

set predTab [gdbRead table -in $talosName]
set eList   [gdbGetInfo $predTab -parms entries]

#-------------------------------------------------------------------------------
# phi(i): C(i-1) N(i)  CA(i) C(i)


set ATOMNAME_I C
set ATOMNAME_J N
set ATOMNAME_K CA
set ATOMNAME_L C 
 
foreach entry $eList \
   {
    foreach v {RESID RESNAME PHI PSI DPHI DPSI CLASS} \
       {
        set $v [gdbGet $entry $v]
       }

    if {!$badFlag} \
       {
        if {[strcasecmp $CLASS Good]} continue
       }
 
    set  str ""

    if {$DPHI < $dPhiMin} {set DPHI $dPhiMin}

    set DPHI [expr $factor*$DPHI]

    set aLo  [expr $PHI - $DPHI]
    set aHi  [expr $PHI + $DPHI]

    set RESID_I   [expr $RESID - 1]
    set RESID_J   $RESID
    set RESID_K   $RESID
    set RESID_L   $RESID

    set RESNAME_I [getThisRes $RESID_I]
    set RESNAME_J [getThisRes $RESID_J]
    set RESNAME_K [getThisRes $RESID_K]
    set RESNAME_L [getThisRes $RESID_L]

    printThisAngle Phi $id $PHI $DPHI $aLo $aHi

    incr id
   }

puts ""

#-------------------------------------------------------------------------------
# psi(i): N(i) CA(i) C(i)  N(i+1)

set ATOMNAME_I N
set ATOMNAME_J CA 
set ATOMNAME_K C
set ATOMNAME_L N 
 
foreach entry $eList \
   {
    foreach v {RESID RESNAME PHI PSI DPHI DPSI CLASS} \
       {
        set $v [gdbGet $entry $v]
       }

    if {!$badFlag} \
       {
        if {[strcasecmp $CLASS Good]} continue
       }
 
    set  str ""

    if {$DPSI < $dPsiMin} {set DPSI $dPsiMin}

    set DPSI [expr $factor*$DPSI]
    set aLo  [expr $PSI - $DPSI]
    set aHi  [expr $PSI + $DPSI]

    set RESID_I   $RESID
    set RESID_J   $RESID
    set RESID_K   $RESID
    set RESID_L   [expr $RESID + 1]

    set RESNAME_I [getThisRes $RESID_I]
    set RESNAME_J [getThisRes $RESID_J]
    set RESNAME_K [getThisRes $RESID_K]
    set RESNAME_L [getThisRes $RESID_L]

    printThisAngle Psi $id $PSI $DPSI $aLo $aHi

    incr id
   }

exit
