#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# TALOS: Torsion Angle Likeness Optimized by Shifts
# (C) Shen, Cornilescu, Delaglio, and Bax 1997-2010
#     NIH Lab of Chemical Physics NIDDK
#
# talos.tcl: finds chemical shift homology matches in database.
# vina.tcl:  classifies and summarizes predictions.
# rama.tcl:  interactive display and adjustment of predictions.

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

set talosVersion 2010.160.15.01 

#
# Initialize variables, extract command line arguments.

proc getTalosArgs {} \
{
   global rcTab adjTab prevTab nextTab homoTab wTab 
   global inName sumName homoName mapName wName dbName 
   global predDir talosDir shiftDir tabDir pdbDir
   global rcName adjName prevName nextName clipName
   global exclList clipFlag fastFlag compileFlag sleepTime
   global dbDefPct targetDefPct

   global varInfo wInfo cInfo thresh matchCount firstRes lastRes r1 rN
   global ARGC ARGV argv env talosVersion wishFlag

   puts "TALOS: Torsion Angle Likeness Optimized By Shifts."
   puts "Version $talosVersion Shen, Cornilescu, Delaglio, and Bax"
   puts ""

   set rcTab   0
   set adjTab  0
   set prevTab 0
   set nextTab 0
   set homoTab 0
   set wTab    0

   set varInfo \
      {
       {RESID    int   %4d}
       {RESNAME  text  %s}
       {HN       float %8.3f}
       {N        float %8.3f}
       {HA       float %8.3f}
       {C        float %8.3f}
       {CA       float %8.3f}
       {CB       float %8.3f}
      }

   if {[info exists env(TALOS_DIR)]} \
      {
       set talosDir $env(TALOS_DIR)
      } \
   else \
      {
       set talosDir .
      }

   set inName       None
   set binType      None
   set sumName      pred.tab
   set mapName      vina.dat
   set predDir      pred
   set compileFlag  0
   set clipFlag     1
   set fastFlag     1
   set thresh       200.0
   set sleepTime    5
   set matchCount   10
   set dbDefPct     59.0
   set targetDefPct 15.0
   set firstRes     First 
   set lastRes      Last 
   set r1           0
   set rN           0
   set exclList     ""

 if {$ARGC == 1 || [flagLoc $ARGV -help]} \
    {
     puts "talos.tcl: predicts angles from chemical shift homology."
     puts ""
     puts "NOTICE: this help text is for the old version of TALOS;"
     puts "        the TALOS system has been replaced with an improved"
     puts "        version called TALOS+ (TALOS plus)."
     puts ""
     puts "TALOS Old Version Arguments:"
     puts " -old                          Use the old version of TALOS."
     puts "TALOS Directory Settings:"
     puts " -talosDir mDir   \[.\]          TALOS Main Directory: TALOS_DIR."
     puts " -shiftDir sDir   \[shifts\]     Subdirectory of master shifts."
     puts " -pdbDir   pDir   \[pdb\]        Subdirectory of master structures."
     puts " -tabDir   tDir   \[tab\]        Subdirectory of master tables."
     puts " -predDir  pDir   \[pred\]       Directory for prediction results."
     puts " -notk                         Suppress Graphics Windows."
     puts "TALOS File Name Settings:"
     puts " -in       inName  \[None\]           Input Shift Table."
     puts " -sum      sumName \[pred.tab\]       Prediction Summary Output."
     puts " -homo     hName   \[homology.tab\]   Residue Homology Table."  
     puts " -map      mName   \[vina.dat\]       Ramachandran Map Name."
     puts " -db       dbName  \[talos.tab\]      TALOS triplet database file."
     puts " -w        wName   \[weight.tab\]     TALOS triplet weighting file."
     puts "TALOS Random Coil and Secondary Shift Tables:"
     puts " -rand     rName   \[randcoil.tab\]   Random Coil Shift Table."
     puts " -adj      aName   \[rcadj.tab\]      Adjustment Table."
     puts "                                      (None for no adjustment)."
     puts " -prev     pName   \[rcprev.tab\]     Previous Residue Adjustment."
     puts "                                      (None for no adjustment)."
     puts " -next     nName   \[rcnext.tab\]     Next Residue Adjustment."
     puts "                                      (None for no adjustment)."
     puts " -limits   lName   \[sslimit.tab\]    Secondary Shift Limits."
     puts "                                      (Used only with -clip)."
     puts "TALOS Other Options:"
     puts " -excl     eList                      List of Proteins to Exclude."
     puts " -first    resID1 \[First]\           First RESID to use."
     puts " -last     resIDN \[Last\]            Last RESID to use."
     puts " -thresh   tVal   \[200.0\]           Max Distance Threshold."
     puts " -count    mc     \[10\]              Max Match Count per Triplet."
     puts " -clip                                Clip Extreme Shifts (Default)."
     puts " -noclip                              No Clipping of Shifts."
     puts " -compile                             Re-compile triplet database."
     puts "TALOS Allowable Percentage of Undefined Shifts"
     puts " -dbpct    defPct \[59.0\]            Minimum, Database Triplet."
     puts " -tpct     tPct   \[15.0\]            Minimum, Target Triplet."
     puts "TALOS Special Fast-Mode Searching (Only Availble on Some Systems):"
     puts " -fast                      Use Fast Mode When Available (Default)."
     puts " -nofast                    Use Ordinary TCL-Based Search." 
     puts " -sleep    sTime  \[5\]     Sleep Time After Fast Mode Search, Sec."
     puts "TALOS Examples:"
     puts " talos.tcl -in ubiq.tab     Form predictions from the given shifts."
     puts " talos.tcl -compile         Re-compile the triplet database."
     puts "See Also:"
     puts " vina.tcl                   Summarize predictions in a single table."
     puts " rama.tcl                   Graphical interface to inspect results."
     exit 0
    }

   if {[info exists env(NMRBINTYPE)]} \
      {
       set binType $env(NMRBINTYPE)
      } \
   else \
      {
       if {[info exists env(NMRBIN)]} \
          {
           set binType [file tail $env(NMRBIN)]
           set eStr    [string trim [file extension $binType] "."]

           if {[string length $eStr]} {set binType $eStr}
          }
      }

   getArgD $ARGV -first    firstRes
   getArgD $ARGV -last     lastRes

   getArgD $ARGV -thresh   thresh
   getArgD $ARGV -count    matchCount

   getArgD $ARGV -in       inName
   getArgD $ARGV -sum      sumName
   getArgD $ARGV -out      sumName

   if {[flagLoc $ARGV -clip]}      {set clipFlag 1}
   if {[flagLoc $ARGV -noclip]}    {set clipFlag 0}

   if {[flagLoc $ARGV -fast]}      {set fastFlag 1}
   if {[flagLoc $ARGV -nofast]}    {set fastFlag 0}

   if {[flagLoc $ARGV -compile]}   {set compileFlag 1}
   if {[flagLoc $ARGV -nocompile]} {set compileFlag 0}

   if {$compileFlag} {set fastFlag 0}

   getArgD $ARGV -talosDir talosDir

   set shiftDir $talosDir/shifts
   set pdbDir   $talosDir/pdb
   set tabDir   $talosDir/tab
   set mapName  $talosDir/$mapName

   getArgD $ARGV -shiftDir shiftDir
   getArgD $ARGV -predDir  predDir
   getArgD $ARGV -pdbDir   pdbDir
   getArgD $ARGV -tabDir   tabDir

   set rcName   $tabDir/randcoil.tab
   set adjName  $tabDir/rcadj.tab
   set prevName $tabDir/rcprev.tab
   set nextName $tabDir/rcnext.tab
   set clipName $tabDir/sslimit.tab

   set homoName $tabDir/homology.tab
   set dbName   $tabDir/talos.tab
   set wName    $tabDir/weight.tab
   
   getArgD $ARGV -rand   rcName
   getArgD $ARGV -adj    adjName
   getArgD $ARGV -prev   prevName
   getArgD $ARGV -next   nextName
   getArgD $ARGV -limits clipName

   getArgD $ARGV -map   mapName
   getArgD $ARGV -homo  homoName
   getArgD $ARGV -db    dbName
   getArgD $ARGV -w     wName
   getArgD $ARGV -sleep sleepTime

   getListArgD $ARGV -excl exclList

   set newList ""

   foreach eName $exclList \
      {
       lappend newList [file root [file tail $eName]]
      }

   set exclList $newList

   showTalosLogo

#
#-----------------------------------------------------------------------------

   
   if {$fastFlag} \
      {
       set fastName $talosDir/bin/TALOS.$binType

       puts stderr "Looking for Fast Mode Executable: $fastName"

       if {![file exists $fastName]} \
          {
           puts stderr " No Fast Mode Executable; Using Standard Mode."
          } \
       else \
          {
           catch "file delete $sumName"

           if {$wishFlag} \
              {
               if {![strcasecmp $binType winxp] || \
                      ![strcasecmp $binType sgi6x]} \
                  {
                   set cmnd "exec xterm -e $fastName $argv"
                   if {$sleepTime} {append cmnd "; sleep $sleepTime"}
                  } \
               else \
                  {
                   set cmnd "exec xterm -e \"$fastName $argv"
                   if {$sleepTime} {append cmnd "; sleep $sleepTime"}
                   append cmnd "\""
                  }

               puts stderr "Using Fast Mode Command: $cmnd"
               catch {eval $cmnd}
              } \
           else \
              {
               set bList "-tk -notk --"
               set bList ""

               set cmnd   "exec $fastName"
               set result "None"

               foreach s $argv \
                  {
                   if {[lsearch -exact $bList $s] < 0} \
                      {
                       append cmnd " $s"
                      }
                  }

               puts stderr "Using Fast Mode Command: $cmnd"

               catch {set result [eval $cmnd]}

               if {[info exists result]} {puts $result}
              }

           if {[file exists $sumName]} \
              {
               exit 0
              } \
           else \
              {
               puts stderr "Fast Mode Failed, Using Standard Mode."
               set fastFlag 0
              }
          }
      }
}

#
# Create an empty table for compiling a triplet database.

proc initTripletTab {} \
{
   global varInfo wInfo cInfo

   set tripTab [gdbCreate table -name triplet]

   foreach i {_R1 _R2 _R3} \
      {
       foreach var $varInfo \
          {
           set name [lindex $var 0]
           set type [lindex $var 1]
           set fmt  [lindex $var 2]

           gdbCreate variable -parent $tripTab \
              -name $name$i -varType $type -varFmt $fmt
          }
      }

   gdbCreate variable -parent $tripTab \
      -name PHI -varType float -varFmt %8.3f

   gdbCreate variable -parent $tripTab \
      -name PSI -varType float -varFmt %8.3f

   gdbCreate variable -parent $tripTab \
      -name SOURCE -varType text -varFmt %s

   gdbCreate variable -parent $tripTab \
      -name CLASS -varType text -varFmt %s

   return $tripTab
}

#
# Initialize table of weighting factors.

proc initWeights {} \
{
   global wInfo cInfo pInfo wName wTab

   puts "Reading TALOS Weight Factors from $wName"

   set wTab   [gdbRead table -in $wName -name "TALOS Weights"]
   set entry  [gdbFirst $wTab]

   set cInfo  ""
   set wInfo  ""
   set pInfo  ""

#
# Extract the atom type weights:

   foreach i {1 2 3 4} \
      {
       lappend cInfo [gdbGet $entry W]
       set     t$i   [gdbGet $entry {HN N HA C CA CB}]

       set entry [gdbNext $entry]
      }

   set wInfo [list $t1 $t2 $t3]

#
# Extract the missing value replacements:

   set c     [lindex $cInfo 3]

   foreach i $t4 \
      {
       lappend pInfo [expr $c*$i]
      }
}

#
# Read the PHI/PSI classification map.

proc initMap {} \
{
   global mapName mapPtr alphaClass betaClass posClass nullClass classArray

   readROI -ndim 2 -in $mapName -roi 1

   set roiInfo  [getROIInfo -roi 1]

   set mapPtr   [lindex $roiInfo 0]
   set sizeList [lindex $roiInfo 2]
   set xSize    [lindex $sizeList 0]
   set ySize    [lindex $sizeList 1]
   set n        [expr $xSize*$ySize]

   set maxVal   [expr 1 + int([vMax $mapPtr $n])]

   for {set i 0} {$i <= $maxVal} {incr i} \
      {
       set classArray($i) Class$i
      }

   set alphaClass [getClassID  -57.0 -46.0]
   set betaClass  [getClassID -102.0 118.0]
   set posClass   [getClassID   90.0   2.0]
   set nullClass  [getClassID    0.0   0.0]

   set classArray(-1)          Undefined
   set classArray($alphaClass) Alpha
   set classArray($betaClass)  Beta
   set classArray($posClass)   Pos 
}

#
# Get the PHI/PSI class ID from the classification map.

proc getClassID { phi psi } \
{
    global mapPtr

    set x   [expr int(0.5 + $phi + 180.0)]
    set y   [expr int(0.5 + $psi + 180.0)]

    if {$x < 0} {set x 0}
    if {$y < 0} {set y 0}

    if {$x > 360} {set x 360}
    if {$y > 360} {set y 360}

    set loc [expr $x + 361*$y]

    set val [expr int([getPtrVal $mapPtr $loc])]

    return $val
}

#
# Get the PHI/PSI class name from the classification map.

proc getClassName { phi psi } \
{
   global classArray

   if {$phi > 999.0 || $psi > 999.0} {return Undefined}

   set id   [getClassID $phi $psi]
   set name $classArray($id)

   return $name
}

#
# Get the list of useful shifts from a given residue.

proc getResInfo { resID assTab resName prevName nextName thisClipFlag } \
{
    global rcArray adjArray prevArray nextArray

#
# Initialize:

    set HN    9999
    set N     9999
    set H     9999
    set HA    9999
    set HA2   9999
    set HA3   9999
    set C     9999
    set CA    9999
    set CB    9999
   
#
# Extract all shifts for this residue:
 
    set eList [gdbSelect -mSQL -from $assTab -cond "RESID == $resID"]

    foreach entry $eList \
       {
        set atomName  [gdbGet $entry ATOMNAME]
        set $atomName [gdbGet $entry SHIFT]
       }

#
# Convert GLY HA2/HA3 to HA:

   if {![strcasecmp $resName G]} \
      {
       if {$HA2 < 999} \
          {
           if {$HA3 < 999} \
              {
               set HA [expr 0.5*($HA2 + $HA3)]
              } \
           else \
              {
               set HA $HA2
              }
          } \
       else \
          {
           if {$HA3 < 999} \
              {
               set HA $HA3
              }
          }
      }

#
# Form secondary shifts for useful atoms using random coil shifts:

   set i 0

   foreach shift [list $HN $N $HA $C $CA $CB] \
      {
       if {$shift < 999} \
          {
           set atomName  [lindex {HN N HA C CA CB} $i]
           set $atomName 9999
           set rcVal     0.0

           if {[info exists rcArray($resName,$atomName)]} \
              {
               set rc $rcArray($resName,$atomName)

               if {$rcVal < 999} \
                  {
                   set rcAdj   $adjArray($resName,$atomName)
                   set prevAdj $prevArray($prevName,$atomName)
                   set nextAdj $nextArray($nextName,$atomName)

                   set rcVal [expr $rc + $rcAdj + $prevAdj + $nextAdj]
                   set ssVal [expr $shift - $rcVal]

                   if {$thisClipFlag} \
                      {
                       set ssVal [clipSShift $ssVal $resID $resName $atomName]
                      }

                   set $atomName $ssVal
                  }
              }
          }

       incr i
      }

   return [list $resID $resName $HN $N $HA $C $CA $CB]
}

#
# Clip secondary shift value.

proc clipSShift { ssVal resID resName atomName } \
{
   global clipArray

   set minVal $clipArray($resName,$atomName,min)
   set maxVal $clipArray($resName,$atomName,max)

   set fmt " Clipped %3d %s %3s Secondary Shift: %8.3f Limit: %8.3f"

   if {$ssVal < $minVal} \
      {
       puts [format $fmt $resID $resName $atomName $ssVal $minVal]
       set ssVal $minVal
      }

   if {$ssVal > $maxVal} \
      {
       puts [format $fmt $resID $resName $atomName $ssVal $maxVal]
       set ssVal $maxVal
      }

   return $ssVal
}

#
# Create the information associated with a given triplet.

proc createTriplet { resID assTab rList pList nList thisClipFlag } \
{
   foreach i {0 1 2} \
      {
       set a0 [lindex $rList $i]
       set aP [lindex $pList $i]
       set aN [lindex $nList $i]

       set  res$i  [getResInfo $resID $assTab $a0 $aP $aN $thisClipFlag]
       incr resID
      }

   return [list $res0 $res1 $res2]
}

#
# Add the triplet info to the triplet table.
 
proc addTriplet args \
{
   global varInfo

   set tripTab [lindex $args 0]
   set triplet [lindex $args 1]
   set source  [lindex $args 2]

   if {[llength $args] > 3} \
      {
       set angInfo [lindex $args 3]
       set phi     [lindex $angInfo 0]
       set psi     [lindex $angInfo 1]

       if {$phi > 999 || $psi > 999} {return 0}
      }

   set entry [gdbCreate entry -parent $tripTab]
   set j     0

   foreach i {_R1 _R2 _R3} \
      {
       set  resInfo [lindex $triplet $j]
       set  k       0
       incr j

       foreach var {RESID RESNAME HN N HA C CA CB} \
          {
           set  val [lindex $resInfo $k]
           incr k

           gdbSet $entry $var$i $val
          }
      }

   set class [getClassName $phi $psi]

   if {[llength $args] > 3} \
      {
       gdbSet $entry PHI $phi
       gdbSet $entry PSI $psi
      }

   gdbSet $entry SOURCE $source
   gdbSet $entry CLASS  $class

   return $entry
}

#
# Get triplet information from a triplet table.

proc getTriplet { entry } \
{
   global tInfo

   if {[info exists tInfo($entry)]} {return $tInfo($entry)}

#
# Following entry format is required:
#   RESID_R1 RESNAME_R1 HN_R1 N_R1 HA_R1 C_R1 CA_R1 CB_R1

#  set cell  [gdbGetInfo $entry -parms cell]
   set vList [gdbGetInfo $entry -parms values]

if { [llength $vList] != 28} {puts $vList}
 
   set t1 [lrange $vList  0  7]
   set t2 [lrange $vList  8 15]
   set t3 [lrange $vList 16 23]

   set t  [list $t1 $t2 $t3]

   set tInfo($entry) $t

   return $t
}

#
# Find percent of defined shifts, blank residue count; 
# Returns defined count for central residue. 

proc testTriplet { triplet pPtr bPtr } \
{
   upvar $pPtr p
   upvar $bPtr b

# 0     1       2  3 4  5 6  7
# RESID RESNAME HN N HA C CA CB

    set p 0.0
    set b 0
    set n 0

    foreach i {0 1 2} \
       {
        set t      [lindex $triplet $i]
        set nn($i) 0
        set jj     0

# HN does not count towards percent defined:

        foreach j [lrange $t 2 7] \
           {
            if {$j > 999} \
               {
                incr nn($i)
                if {!$jj} {incr n}
               }

            incr jj
           }

        if {$nn($i) == 6} {incr b} 
       }

    set p [expr 100.0*(1.0 - $n/15.0)]

    return [expr 6 - $nn(1)]
}

#
# Aux function to sort homology distances.

proc dSort { e1 e2 } \
{
    set d1 [lindex $e1 1]
    set d2 [lindex $e2 1]

    if {$d1 < $d2} \
       {
        return -1
       } \
    elseif {$d1 > $d2} \
       {
        return 1
       }

    return 0
}

#
# Recreate the TALOS triplet database file.

proc compileTalos { shiftDir pdbDir dbName } \
{
    global exclList r1 rN
    global dbDefPct targetDefPct

    puts ""
    puts "Compiling."
    puts " Shifts: $shiftDir"
    puts " PDB:    $pdbDir"
    puts " DBA:    $dbName"
    puts ""

#
# Create the empty triplet table.
# Find the list of input files.

   set tripTab   [initTripletTab]
   set rawList   [glob -nocomplain $shiftDir/*.tab]
   set inList    ""

   foreach i $rawList \
      {
       set rootName [file root [file tail $i]]
 
       if {[lsearch -exact $exclList $rootName] < 0} \
          {
           lappend inList $rootName
          }
      }

   set inCount [llength $inList]

   set time1 [join [sysTime] .]
   set i 0

   set pctDefined    0.0
   set blankResCount 0

#
# For each input:
#  Read the shifts and structure.
#  Find the number of residues in this input.
#
#  For each triplet of residue IDs:
#     Form the triplet.
#     Add it to the table.
#
#  Destroy the original internal shift and structure table.

   foreach inName $inList \
      {
       set assTab \
        [gdbRead table -in $shiftDir/$inName.tab -name "$inName shifts"]

       set pdbTab \
        [gdbRead table -in $pdbDir/$inName.pdb   -name "$inName PDB" -pdb]

       set seqList [getSeqList $assTab r1 rN]
       set n       [expr $rN - 1]
       set npList  "?"
       set cCount  0

       foreach res $seqList {lappend npList $res}

       lappend npList "?"

       incr i

       puts "Compiling $inName, Protein $i of $inCount:"
 
       for {set j $r1} {$j < $n} {incr j} \
          {
           puts -nonewline " Residues $j-[expr $j+2] of $r1 to $rN\015"
           flush stdout

           set jj      [expr $j - $r1]
           set rList   [lrange $seqList $jj          [expr $jj+2]]
           set pList   [lrange $npList  $jj          [expr $jj+2]]
           set nList   [lrange $npList  [expr $jj+2] [expr $jj+4]]

           set triplet [createTriplet $j $assTab $rList $pList $nList 0]
           set phiPsi  [getAngles     $j $pdbTab]

           testTriplet $triplet pctDefined blankResCount

           if {$pctDefined > $dbDefPct && $blankResCount == 0} \
              {
               set newEntry [addTriplet $tripTab $triplet $inName $phiPsi]

               if {$newEntry} {incr cCount}
              }
          }

       gdbDestroy $assTab
       gdbDestroy $pdbTab

       update

       puts ""
       puts " Added $cCount triplets to the database."
      }

   set time2 [join [sysTime] .]

   puts "[expr $time2 - $time1] seconds."

#
# Save the results:

   gdbWrite $tripTab $dbName

   return $tripTab
}

#
# Find the shift homology distances for the given triplet.

proc getTalosDist { tripTab tripletU } \
{
   global homoArray rcArray cInfo wInfo pInfo thresh matchCount

   set entry [gdbFirst $tripTab]

   set bList ""
   set sList ""

   set ii 0
  
   while {$entry} \
      {
       set tripletD [getTriplet $entry]
       set dist 0.0

       incr ii

       foreach j {0 1 2} \
          {
           set  tInfoU [lindex $tripletU $j]
           set  tInfoD [lindex $tripletD $j]
           set  wList  [lindex $wInfo    $j]

           set  c      [lindex $cInfo    $j]

           set  aU [lindex $tInfoU 1]
           set  aD [lindex $tInfoD 1]

           set  h  $homoArray($aU,$aD)

           set  k  0
           set  kk 2
           set  q  0.0
           set  f  1.0

           foreach var {HN N HA C CA CB} \
              {
               set w   [lindex $wList  $k]
               set sU  [lindex $tInfoU $kk]
               set sD  [lindex $tInfoD $kk]

               if {$sU > 999 || $sD > 999} \
                  {
                   set d [lindex $pInfo $k]
                   set q [expr $q + $w*$d]
                  } \
               else \
                  {
                   set d [expr $sU - $sD]
                   set q [expr $q + $w*$d*$d]
                  }

               incr k
               incr kk
              }

           set dist [expr $dist + $c*$h*$h + $q]
          }

       if {$dist < $thresh} {lappend bList [list $entry $dist]}

       set entry [gdbNext $entry]
       update
      }

   set sList [lsort -command dSort $bList]

   return [lrange $sList 0 [expr $matchCount-1]]
}

#
# Save and show the results of matches tallied for a given triplet.

proc saveTalosDist { resID1 dList predDir rList } \
{
   global targetDefPct

   set j 1

   set fmt1     "   %2d % 8.3f % 8.3f  %8.3f %5.3f  %-4s %-4s %-4s %s"
   set fmt2     "   %2d % 8.3f % 8.3f  %8.3f  %-4s %-4s %-4s %s"

   set count    [llength $dList]
   set resID2   [expr $resID1 + 1]
   set resID3   [expr $resID1 + 2]
   set thisName [lindex $rList 1]
   set thisRes  $resID2

   set predName [format $predDir/res%03d.tab $thisRes]

   set error [catch {set outUnit [open $predName w]}]

   if {$error} \
      {
       puts "Error opening prediction file $predName"
       set  outUnit stdout
      } \
   else \
      {
       puts $outUnit ""
       puts $outUnit "REMARK Prediction Matches for Residue $thisName$thisRes."
       puts $outUnit ""

       puts $outUnit \
         "VARS   INDEX PHI   PSI   DIST   W     R1   R2   R3   SOURCE"

       puts $outUnit \
         "FORMAT %2d   %9.3f %9.3f %8.3f  %5.3f %-4s %-4s %-4s %s"

       puts $outUnit ""
       puts $outUnit "DATA RESIDS   $resID1 $resID2 $resID3"
       puts $outUnit "DATA RESNAMES $rList"
       puts $outUnit ""
      }

   if {!$count} \
      {
       puts $outUnit "REMARK Notice: No Matches for Residue $resID1."
       puts "    Notice: No Matches for Residue $thisName$thisRes."
      }

   foreach i $dList \
      {
       set entry [lindex $i 0]
       set d     [lindex $i 1]
       set w     1.0

       set phi   [gdbGet $entry PHI]
       set psi   [gdbGet $entry PSI]
       set src   [gdbGet $entry SOURCE]

       set a1    [gdbGet $entry RESNAME_R1]
       set a2    [gdbGet $entry RESNAME_R2]
       set a3    [gdbGet $entry RESNAME_R3]

       append a1 [gdbGet $entry RESID_R1]
       append a2 [gdbGet $entry RESID_R2]
       append a3 [gdbGet $entry RESID_R3]

       puts $outUnit [format $fmt1 $j $phi $psi $d $w $a1 $a2 $a3 $src]
       puts stdout   [format $fmt2 $j $phi $psi $d $a1 $a2 $a3 $src]

       incr j
      }

    if {[strcmp $outUnit stdout]} {close $outUnit}
}

#
# Predict the angles for the given shift table.

proc talosPredict { inName tripTab predDir } \
{
    global firstRes lastRes r1 rN clipFlag targetDefPct

#
# Prepare the output:

    if {![file isdirectory $predDir]} \
       {
        puts "Creating Prediction Directory $predDir"
        file mkdir $predDir
       }

#
# Read the input:

    puts ""
    puts -nonewline "Analyzing $inName"
    flush stdout

    set  inTab   [gdbRead table -in $inName -name unknown]
    set  seqList [getSeqList $inTab r1 rN]
    set  seqStr  [getSeqListSeqTxt $seqList $r1 $rN " "]
    set  npList  "?"

    puts " Residues $r1 to $rN:"
    puts $seqStr

    foreach res $seqList {lappend npList $res}

    lappend npList "?"

#
# Find the range of residues to analyze:

    if {![strcasecmp $firstRes First]} {set firstRes $r1}
    if {![strcasecmp $lastRes  Last]}  {set lastRes  $rN}

    if {$firstRes < $r1} {set firstRes $r1}
    if {$lastRes  < $r1} {set lastRes  $r1}

    if {$firstRes > $rN} {set firstRes $rN}
    if {$lastRes  > $rN} {set lastRes  $rN}

    if {$firstRes > $lastRes} \
       {
        set itemp    $firstRes
        set firstRes $lastRes
        set $lastRes $itemp
       }

    set n [expr $lastRes - 1]

#
# For each triplet in the unknown:
#  Extract that triplet.
#  If it is a reasonable triplet, perform distance analysis.
#  Show the results.

    set pctDefined    0.0
    set blankResCount 0

    for {set j $firstRes} {$j < $n} {incr j} \
       {
        set jj      [expr $j - $r1]
        set rList   [lrange $seqList $jj          [expr $jj+2]]
        set pList   [lrange $npList  $jj          [expr $jj+2]]
        set nList   [lrange $npList  [expr $jj+2] [expr $jj+4]]

        set triplet [createTriplet $j $inTab $rList $pList $nList $clipFlag]

        puts " Triplet $rList $j-[expr $j+2] of $r1 to $rN"

        set defCount [testTriplet $triplet pctDefined blankResCount]

        if {$pctDefined > $targetDefPct && $defCount > 0} \
           {
            set dList [getTalosDist $tripTab $triplet]
            saveTalosDist $j $dList $predDir $rList
           }
       }
}

#
# Summarize TALOS predictions.

proc talosSummarize { inName outName predDir } \
{
   global firstRes lastRes wishFlag

   if {![strcasecmp $outName  None]}  return
   if {![strcasecmp $firstRes First]} return
   if {![strcasecmp $lastRes  Last]}  return

if {$wishFlag} \
   {
    catch \
     {exec vinaOrig.tcl -in $inName -sum $outName -predDir $predDir -auto}
   } \
else \
   {
    catch \
     {exec vinaOrig.tcl -notk -in $inName -sum $outName -predDir $predDir -AUTO}
   }
}

#
# Display the TALOS Logo.

proc showTalosLogo {} \
{
   global talosDir talosVersion wishFlag

   if {!$wishFlag} return

   frame .top
   frame .bottom

   image create photo talosImage -file $talosDir/talos.gif

   label  .verLabel -text  "TALOS Version $talosVersion"
   label  .imLabel  -image talosImage -bd 2 -relief raised
   button .imQuit   -text  Quit -command exit

   pack .verLabel .imLabel .imQuit -in . -side top -expand 1 -fill x
   update
}

#
# Main application.

#
# Extract command line arguments and initialize values.
# Initialize the database engine.
# Establish the random coil shifts, residue homology values, and weights.

getTalosArgs

initRandCoil
initSSLimits
initHomology
initWeights
initMap

#
# Build or read the triplet database:

if {$compileFlag} \
   {
    set  tripTab [compileTalos $shiftDir $pdbDir $dbName]

    set  eCount  [gdbGetInfo $tripTab -parms entryCount]
    puts "Created $dbName, $eCount Triplets."
   } \
else \
   {
    set initTab [gdbRead table -in $dbName -name triplet]

    if {[llength $exclList]} \
       {
        set c       "\[lsearch -exact \{$exclList\} \$SOURCE\] < 0"
        set tripTab [gdbSelect -from $initTab -tcl -new -cond $c]
       } \
    else \
       {
        set tripTab $initTab
       }

    set  eCount  [gdbGetInfo $tripTab -parms entryCount]
    puts "Read $dbName, $eCount Triplets."
   }

#
# Analyze the input if one has been given.

if {[strcasecmp $inName None]} \
   {
    talosPredict   $inName $tripTab $predDir
    talosSummarize $inName $sumName $predDir
   }

exit
