#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#--------------------------------------------------------------

set inName  None
set tName   None
set pdbName None
set segName None
set range   None

set assID     1
set defaultFC 1.0

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Convert XPLOR Torsion Input to DYNAMO Format."
    puts " -in    inName      XPLOR Input (Torsion Information Only)."
    puts " -tor   tName       Existing Torsion Input, If Any."
    puts " -pdb   pdbName     Corresponding PDB (for Sequence and Atom Names)."
    puts " -seg   segName     Default Seg Name."
    puts " -id    firstID     First Torsion ID."
    puts " -fc    fc          Force Constant."
    puts "Optional Reset of Torsion Angle Range:" 
    puts " -da    daDegrees   Replacement +/- Degrees for Angular Range."
    exit 0
   }

getArgD $ARGV -in  inName
getArgD $ARGV -tor tName
getArgD $ARGV -pdb pdbName
getArgD $ARGV -seg segName
getArgD $ARGV -da  range
getArgD $ARGV -id  assID
getArgD $ARGV -fc  defaultFC

#
#--------------------------------------------------------------

proc parseTorsionAss { assID line } \
{
   global range defaultFC

   parseXPLORAss $assID $line 4 assList tail

   set assI  [lindex $assList 0]
   set assJ  [lindex $assList 1]
   set assK  [lindex $assList 2]
   set assL  [lindex $assList 3]

   set fc    [lindex $tail 0]
   set d0    [lindex $tail 1]
   set dx    [lindex $tail 2]
   set ipow  [lindex $tail 3]

   if {[strcasecmp $range None]} {set dx $range}
 
   set dHi [expr $d0 + $dx]
   set dLo [expr $d0 - $dx]

   set assText $assI,$assJ,$assK,$assL
   set pList   [permList $assText]
   set groupID 1

   set fc      $defaultFC
   set w       1.0
   set s       1.0
 
   foreach p $pList \
      {
       printTorsionAss $assID $groupID $p $d0 $dLo $dHi $fc $w $s
       incr groupID
      }
}

proc printTorsionAss { assID groupID p d dLo dHi fc w s } \
{
   global seqInfo

   set str [string toupper [split $p "~,"]]

   set resIDI    [lindex $str 0]
   set atomNameI [lindex $str 1]
   set segNameI  [lindex $str 2]
   set resNameI  $seqInfo($resIDI,$segNameI)
   set atomListI [expandXPLORAtom $resIDI $atomNameI $segNameI]

   set resIDJ    [lindex $str 3]
   set atomNameJ [lindex $str 4]
   set segNameJ  [lindex $str 5]
   set resNameJ  $seqInfo($resIDJ,$segNameJ)
   set atomListJ [expandXPLORAtom $resIDJ $atomNameJ $segNameJ]

   set resIDK    [lindex $str 6]
   set atomNameK [lindex $str 7]
   set segNameK  [lindex $str 8]
   set resNameK  $seqInfo($resIDK,$segNameK)
   set atomListK [expandXPLORAtom $resIDK $atomNameK $segNameK]

   set resIDL    [lindex $str 9]
   set atomNameL [lindex $str 10]
   set segNameL  [lindex $str 11]
   set resNameL  $seqInfo($resIDL,$segNameL)
   set atomListL [expandXPLORAtom $resIDL $atomNameL $segNameL]

   foreach atomI $atomListI \
      {
   foreach atomJ $atomListJ \
      {
   foreach atomK $atomListK \
      {
   foreach atomL $atomListL \
      {
       puts -nonewline [format "%4d %2d" $assID $groupID]
       puts -nonewline [format " %3d %-4s %-4s" $resIDI $resNameI $atomI]
       puts -nonewline [format " %-4s" $segNameI]
       puts -nonewline [format " %3d %-4s %-4s" $resIDJ $resNameJ $atomJ]
       puts -nonewline [format " %-4s" $segNameJ]
       puts -nonewline [format " %3d %-4s %-4s" $resIDK $resNameK $atomK]
       puts -nonewline [format " %-4s" $segNameK]
       puts -nonewline [format " %3d %-4s %-4s" $resIDL $resNameL $atomL]
       puts -nonewline [format " %-4s" $segNameL]

       puts \
        [format " % 7.2f % 7.2f % 7.2f %.3f %.1f %.1f" $d $dLo $dHi $fc $w $s]
      }
      }
      }
      }
}

proc printTorsionHdr {} \
{
   puts -nonewline "VARS INDEX GROUP"
   puts -nonewline " RESID_I RESNAME_I ATOMNAME_I SEGNAME_I"
   puts -nonewline " RESID_J RESNAME_J ATOMNAME_J SEGNAME_J"
   puts -nonewline " RESID_K RESNAME_K ATOMNAME_K SEGNAME_K"
   puts -nonewline " RESID_L RESNAME_L ATOMNAME_L SEGNAME_L"
   puts -nonewline " ANGLE ANGLE_LO ANGLE_HI FC W S"
   puts ""

   puts -nonewline "FORMAT %3d %3d"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %9.3f %9.3f %9.3f %.3f %.3f %.3f"
   puts ""
   puts ""
}

proc printExistingTorsions {} \
{
   global tName segName assID group

   if {![strcasecmp $tName None] || ![file exists $tName]} return 

   set tTab   [gdbRead table -in $tName]
   set eList  [gdbGetInfo $tTab -parms entries]
   set vList  [gdbGetInfo $tTab -parms varNames]

   set n [llength $eList]
   set k 0

   set defVal(SEGNAME)  $segName
   set defVal(ATOMNAME) None
   set defVal(RESNAME)  None
   set defVal(RESID)    0 

   foreach entry $eList \
      {
       wm title . "Appending Torsion $k/$n"
       update

       set ANGLE_HI 9999.0
       set ANGLE_LO 9999.0
       set ANGLE    9999.0

       set INDEX    $assID
       set GROUP    1
       set FC       1.0
       set W        1.0
       set S        1.0

       foreach i {I J K L} \
          {
           foreach t {RESID RESNAME ATOMNAME SEGNAME} \
              {
               set v  [format "%s_%s" $t $i]
               set $v $defVal($t)

               if {[lsearch -exact $vList $v] < 0} continue

               set $v [gdbGet $entry $v]
              }
          }

       foreach v {INDEX GROUP FC W S ANGLE ANGLE_HI ANGLE_LO} \
          {
           if {[lsearch -exact $vList $v] < 0} continue

           set $v [gdbGet $entry $v]
          }

       if {$ANGLE > 999 && $ANGLE_LO < 999 && $ANGLE_HI < 999} \
          {
           set angle [getAvgAngle "$ANGLE_LO $ANGLE_HI"]
          }

       if {$ANGLE_LO > 999 && $ANGLE    < 999} {set ANGLE_LO $ANGLE} 
       if {$ANGLE_LO > 999 && $ANGLE_HI < 999} {set ANGLE_LO $ANGLE_HI} 

       if {$ANGLE_HI > 999 && $ANGLE    < 999} {set ANGLE_HI $ANGLE} 
       if {$ANGLE_HI > 999 && $ANGLE_LO < 999} {set ANGLE_HI $ANGLE_LO} 

       if {[lsearch -exact $vList INDEX] < 0}  {incr assID} 

       set assI [format "%s~%s~%s" $RESID_I $ATOMNAME_I $SEGNAME_I]
       set assJ [format "%s~%s~%s" $RESID_J $ATOMNAME_J $SEGNAME_J]
       set assK [format "%s~%s~%s" $RESID_K $ATOMNAME_K $SEGNAME_K]
       set assL [format "%s~%s~%s" $RESID_L $ATOMNAME_L $SEGNAME_L]

       set p $assI,$assJ,$assK,$assL

       printTorsionAss $INDEX $GROUP $p $ANGLE $ANGLE_LO $ANGLE_HI $FC $W $S

       incr k
      }

   if {[lsearch -exact $vList INDEX] >= 0} \
      {
       set assID [getTabMax $tTab INDEX]
       incr assID
      }

   puts ""
   puts ""
}

proc parseTorsion {} \
{
   global inUnit assID

   set itemCount 0
   set assInfo($itemCount) ""

   while {[gets $inUnit line] >= 0} \
      {
       set keyword [lindex $line 0]

       if {"$keyword" == "assign"} \
          {
           set line [lrange $line 1 end]
           set line [stripXPLORComment $line]

           incr itemCount

           wm title . "Reading $itemCount"
           update
          }

       append assInfo($itemCount) " $line"
      }

   for {set i 1} {$i <= $itemCount} {incr i} \
      {
       wm title . "Interpreting $i/$itemCount"
       update

       set result ""
       set error  0

       set error [catch {parseTorsionAss $assID $assInfo($i)}]

       if {$error} \
          {
           puts stderr "DYNAMO Warning: Error interpreting Restraint $assID:"
           puts stderr "  Error Code: $error $result"
           puts stderr "  Line: $assInfo($assID)"
          }

       incr assID
      }
}

if {![file exists $inName]} \
   {
    puts stderr "Error finding input restraint file $inName"
    exit 1
   }

if {![file exists $pdbName]} \
   {
    puts stderr "Error finding input PDB file $pdbName"
    exit 1
   }

dynInit -fast

showDynLogo .

wm title . "Reading $pdbName"
update

set inUnit [open $inName r]

getXPLORPDBInfo $pdbName

wm title . "Reading $inName"
update

printTorsionHdr
printExistingTorsions
parseTorsion

exit
