#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Update HEIGHT and VOL Values in an NMRPipe Peak Table."
    puts " -in   inName  \[test.tab\] Input Table or PDB File." 
    puts " -out  outName \[new.tab\]  Output Table." 
    puts " -ft   ftName  \[test.ft2\] Input Spectrum."
    puts " -noverb                  Verbose Mode OFF."
    puts "Region Size (Use -dx dy -dz -da):"
    puts " -dx   dx      \[1\]        X-Axis +/- Region Size (Pts Hz ppm %)."
    puts "Notes:"
    puts " 1. The HEIGHT value will be set to the maximum signed intensity"
    puts "    in the region for each peak."
    puts " 2. The VOL value will be set to sum of intensities in the region"
    puts "    for each peak."
    exit 1 
   }

set inName      test.tab
set outName     new.tab
set ftName      test.ft2 
set dimCount    Auto
set verbose     1

set dx          1
set dy          1
set dz          1
set da          1
 
getArgD $ARGV -in   inName
getArgD $ARGV -out  outName
getArgD $ARGV -ft   ftName
getArgD $ARGV -ndim dimCount

getArgD $ARGV -dx   dx
getArgD $ARGV -dy   dy
getArgD $ARGV -dz   dz
getArgD $ARGV -da   da

if {[flagLoc $ARGV -verb]}   {set verbose 1} 
if {[flagLoc $ARGV -noverb]} {set verbose 0} 

#
#--------------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts "UpdateHi Error Finding Input Table $inName"
    exit 1
   }

if {![file exists [format $ftName 1 1]]} \
   {
    puts "UpdateHi Error Finding Input Spectrum $ftName"
    exit 1
   }

set t     [gdbRead table -in $inName]
set eList [gdbGetInfo $t -parms entries]
set vList [gdbGetInfo $t -parms varNames]

if {![strcasecmp $dimCount Auto]} \
   {
    set dimCount 0

    foreach v {X_PPM Y_PPM Z_PPM A_PPM} \
       {
        if {[lsearch -exact $vList $v] < 0} break
        incr dimCount
       }
   }

if {$verbose} \
   {
    puts ""
    puts "UpdateHi. Input Spectrum:          $ftName"
    puts "UpdateHi. Input Peak Table:        $inName"
    puts "UpdateHi. Peak Count:              [llength $eList]"
    puts "UpdateHi. Peak Table Dimensions:   $dimCount"
    puts "UpdateHi. Ouput Table:             $outName"
    puts ""
   }

foreach e $eList \
   {
    if {$dimCount == 1} \
       {
        readROI -roi 1 -in $ftName -ndim 1 -dx X_AXIS [gdbGet $e X_PPM]ppm $dx
       } \
    elseif {$dimCount == 2} \
       {
        readROI -roi 1 -in $ftName -ndim 2 -dx X_AXIS [gdbGet $e X_PPM]ppm $dx \
                                           -dy Y_AXIS [gdbGet $e Y_PPM]ppm $dy
       } \
    elseif {$dimCount == 3} \
       {
        readROI -roi 1 -in $ftName -ndim 3 -dx X_AXIS [gdbGet $e X_PPM]ppm $dx \
                                           -dy Y_AXIS [gdbGet $e Y_PPM]ppm $dy \
                                           -dz Z_AXIS [gdbGet $e Z_PPM]ppm $dz
       } \
    elseif {$dimCount == 4} \
       {
        readROI -roi 1 -in $ftName -ndim 4 -dx X_AXIS [gdbGet $e X_PPM]ppm $dx \
                                           -dy Y_AXIS [gdbGet $e Y_PPM]ppm $dy \
                                           -dz Z_AXIS [gdbGet $e Z_PPM]ppm $dz \
                                           -da A_AXIS [gdbGet $e A_PPM]ppm $da
       } \
    else \
       {
        puts "UpdateHi. Error: Unsupported Dimension Count."
        exit 1
       }

    set minVal [getROIStat 1 vMin]
    set maxVal [getROIStat 1 vMax]
    set sum    [getROIStat 1 vSum]

    if {abs($minVal) > abs($maxVal)} \
       {
        set hi $minVal
       } \
    else \
       {
        set hi $maxVal
       }

    if {$verbose} {puts [format "UpdateHi. Peak: %4d Height: % .3e Volume: % .3e" [gdbGet $e INDEX] $hi $sum]}

    gdbSet $e HEIGHT $hi
    gdbSet $e VOL    $sum
   }

if {$verbose} \
   {
    puts ""
    puts "UpdateHi. Ouput Table: $outName"
    puts ""
   }

gdbWrite $t $outName

exit 0
