#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set thisVersion "2016.048.16.30"

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "vdExpand.tcl Version $thisVersion"
    puts stderr "Expand Binary Data According to a Keep/Skip Non-Unform Sampling (NUS) Schedule:"
    puts stderr " -mode   eMode     \[bruker\]        Input Type Keyword: bruker pipe."
    puts stderr " -sample vdName    \[vdlist\]        NUS Schedule Input File."
    puts stderr " -xN     xN        \[Auto\]          Number of Total Points per Vector."
    puts stderr " -keep   keepText  \[3u\]            Schedule Text for Increments That Were Kept."
    puts stderr " -mask                             Output a Mask Instead of Data."
    puts stderr " -expand                           Expand Data on Output (Default)."
    puts stderr " -noexpand                         Input Data is Already Expanded."
    puts stderr " -verb                             Verbose Mode ON (Default)."
    puts stderr " -noverb                           Verbose Mode Off."
    puts stderr "Options and defaults for -mode bruker expansion:"
    puts stderr " -in     inName    \[ser\]           Bruker ser-format Input."
    puts stderr " -out    fullName  \[ser_full\]      Expanded Output."
    puts stderr " -vCount n         \[Auto\]          Number of Vector Pairs Actually Acquired."
    puts stderr " -acqus  acqusName \[acqus\]         File acqus, used only for -xN Auto option."
    puts stderr "Options and defaults for -mode pipe expansion:"
    puts stderr " -in     inName    \[test.fid\]      NMRPipe-format Input."
    puts stderr " -out    fullName  \[test_full.fid\] Output (Default is mask.fid for -mask)."
    puts stderr " -vCount n         \[Auto\]          Number of Samples to Process."
    puts stderr "Expansion of Random Phase Detection data:"
    puts stderr " -rpd                              Random Phase Detection Mode ON."
    puts stderr " -realKey rKey \[1\]                 Schedule Value for Real Measurement."
    puts stderr " -imagKey iKey \[2\]                 Schedule Value for Imaginary Measurement."
    puts stderr "Notes:"
    puts stderr " 1. The vdlist-format file has lines with text '3u' for increments that"
    puts stderr "    were measured, and '1u' for increments that were skipped."
    puts stderr " 2. Input and output file names must be different."
    puts stderr " 3. Use -in  stdin to read from standard input."
    puts stderr " 4. Use -out stdout to write to standard output."
    puts stderr " 5. (For -mode bruker): option -xN Auto will extract the TD parameter"
    puts stderr "    the bruker acqus file."
    puts stderr " 6. Input for Random Phase Detection Mode (-rpd) has lines with text '1'"
    puts stderr "    for increments that had only the real part measured, and text '2'"
    puts stderr "    for increments that had only the imaginary part measured."
    exit 0
   }

set inName      ser
set eMode       bruker
set outName     ser_full
set acqusName   acqus
set sampleName  vdlist
set vCount      Auto 
set xN          Auto
set inType      int 
set keepKey     3u
set realKey     1
set imagKey     2 
set rpdFlag     0
set maskFlag    0
set expandFlag  1
set verbose     1

if {[flagLoc $ARGV -mask]}      {set maskFlag 1}
if {[flagLoc $ARGV -nomask]}    {set maskFlag 0}

if {[flagLoc $ARGV -expand]}    {set expandFlag 1}
if {[flagLoc $ARGV -noexpand]}  {set expandFlag 0}

getArgD $ARGV -mode eMode 

if {![strcasecmp $eMode pipe]} \
   {
    set inName  test.fid
    set outName test_full.fid
    set inType  float

    if {$maskFlag} {set outName mask.fid}
   }

getArgD $ARGV -in          inName
getArgD $ARGV -out         outName
getArgD $ARGV -acqus       acqusName
getArgD $ARGV -sample      sampleName
getArgD $ARGV -vCount      vCount
getArgD $ARGV -xN          xN
getArgD $ARGV -keep        keepKey
getArgD $ARGV -realKey     realKey
getArgD $ARGV -imagKey     imagKey

if {[flagLoc $ARGV -rpd]}     {set rpdFlag 1}
if {[flagLoc $ARGV -norpd]}   {set rpdFlag 0}


if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {[strcasecmp $inName stdin]} \
   {
    if {![file exists $inName]} \
       {
        puts stderr "vdExpand Error Finding ${eMode}-format Input $inName"
        exit 1
       }
   }

if {![file exists $sampleName]} \
   {
    puts stderr "vdExpand Error Finding vdlist-format NUS Schedule Input $sampleName"
    exit 1
   }
  
set error [catch {set sUnit [open $sampleName r]}]

if {$error} \
   {
    puts stderr "vdExpand Error Opening vdlist-format NUS Schedule Input $sampleName"
    exit 1
   }

if {![strcasecmp $inName $outName] || ![strcasecmp [file normalize $inName] [file normalize $outName]]} \
   {
    puts stderr "vdExpand Error: Input and Output File Names Must Differ."
    exit 1
   }

if {![strcasecmp $inName stdin]} \
   {
    set inUnit 0
   } \
else \
   {
    set error [catch {set inUnit [sysOpen $inName r]}]

    if {$error} \
       {
        puts stderr "vdExpand Error Opening ${eMode}-format Input $inName"
        exit 1
       }
   }

if {![strcasecmp $outName stdout]} \
   {
    set outUnit 0
   } \
else \
   {
    set error [catch {set outUnit [sysOpen $outName w]}]

    if {$error} \
       {
        puts stderr "vdExpand Error Opening ser-format Output $outName"
        exit 1
       }
   }

if {![strcasecmp $eMode pipe]} \
   {
    set error [catch {set fdata [sysMalloc $FDATASIZE float]}]
    set error [catch "sysReadB $inUnit $fdata $FDATASIZE"]

    if {$error} \
       {
        puts stderr "vdExpand Error Reading NMRPipe header from $inName."
        exit 1
       }

    set xSize  [getParmI $fdata NDSIZE CUR_XDIM]
    set ySize  [getParmI $fdata NDSIZE CUR_YDIM]

    set qxSize [getQuadSize [getParmI $fdata NDQUADFLAG CUR_XDIM]]
    set qySize [getQuadSize [getParmI $fdata NDQUADFLAG CUR_YDIM]]
    set qSize  [getQuadSize [getParmI $fdata FDQUADFLAG NULL_DIM]]

    if {$expandFlag} \
       {
        setParm $fdata NDQUADFLAG 0.0 CUR_YDIM
        setParm $fdata FDQUADFLAG 0.0 NULL_DIM

        if {$qySize == 2} \
           {
            setParm $fdata NDSIZE [expr $ySize*2] CUR_YDIM
           }
       }

    set error [catch "sysWrite $outUnit $fdata $FDATASIZE"]

    if {$error} \
       {
        puts stderr "vdExpand Error Writing NMRPipe header to $outName."
        exit 1
       }

    if {![strcasecmp $xN Auto]} \
       {
        set xN [expr $qxSize*$xSize]
       }
   } \
else \
   {
    if {![strcasecmp $xN Auto]} \
       {
        if {![file exists $acqusName]} \
           {
            puts stderr "vdExpand Error Finding acqus Input $acqusName"
            exit 1
           }

        set tdVal 0
        set xN    0

        catch {set tdVal [exec jcampInfo.tcl -in $acqusName -parm TD -noverb]} 
        if {![info exists tdVal]} {set tdVal 0}

        if {[string is integer $tdVal]} \
           {
            set xN  $tdVal
            set pad [expr $xN % 256]
    
            if {$pad} {set xN [expr $xN + 256 - $pad]}
           }

        if {$xN < 1} \
           {
            puts stderr "vdExpand Error Extracting TD Parameter from File $acqusName"
            exit 1
           }
       }
   }

set n [expr 2*$xN]

set error [catch {set inBuffR  [sysMalloc $n $inType]}]
set error [catch {set inBuffI  [sysMalloc $n $inType]}]
set error [catch {set zeroBuff [sysMalloc $n $inType]}]
set error [catch {set oneBuff  [sysMalloc $n $inType]}]

if {$error} \
   {
    puts stderr "vdExpand Error Allocating Buffers."
    exit 1
   }

for {set i 0} {$i < $n} {incr i} \
   {
    setPtr $zeroBuff $i 0 $inType
    setPtr $oneBuff  $i 1 $inType
   }

set trueCount 0
set lineCount 0
set keepCount 0
set realCount 0
set imagCount 0

if {$verbose} \
   {
    if {$rpdFlag} \
       {
        set eStr "RPD"
       } \
    else \
       {
        set eStr "NUS"
       }

    puts stderr ""
    puts stderr "vdExpand. Version:         $thisVersion"
    puts stderr "vdExpand. Binary Type:     [getNMRBinType]"
    puts stderr "vdExpand. Input:           $inName"
    puts stderr "vdExpand. Input Mode:      $eMode"
    puts stderr "vdExpand. Input Schedule:  $sampleName"
    puts stderr "vdExpand. Schedule Type:   $eStr"
    puts stderr ""
   }

set thisList ""

if {$rpdFlag} \
   {
    while {[gets $sUnit str] >= 0} \
       {
        incr trueCount

        if {[strcasecmp $vCount Auto]} \
           {
            if {$lineCount > $vCount - 1} continue 
           }

        incr lineCount

        set error [catch "sysReadB $inUnit $inBuffR $xN"]

        if {$error} \
           {
            puts stderr "vdExpand Error $error Reading Data. Schedule: $lineCount."
            exit 1
           }

        set srcBuffR $inBuffR
        set srcBuffI $inBuffR

        if {!$expandFlag} \
           {
            set error [catch "sysReadB $inUnit $inBuffI $xN"]

            if {$error} \
               {
                puts stderr "vdExpand Error $error Reading Imaginary Data. Schedule: $lineCount."
                exit 1
               }

            set srcBuffI $inBuffI
           }

        set key [lindex $str 0]

        if {![strcasecmp $key $realKey]} \
           {
            incr realCount

            if {$maskFlag} \
               {
                set outBuffR $oneBuff
                set outBuffI $zeroBuff
               } \
            else \
               {
                set outBuffR $srcBuffR
                set outBuffI $zeroBuff
               }
           } \
        else \
           {
            incr imagCount

            if {$maskFlag} \
               {
                set outBuffR $zeroBuff
                set outBuffI $oneBuff
               } \
            else \
               {
                set outBuffR $zeroBuff
                set outBuffI $srcBuffI
               }
           }

         set error [catch "sysWrite $outUnit $outBuffR $xN"]

         if {$error} \
            {
             puts stderr "vdExpand Error $error Writing Real Data. Schedule: $lineCount"
             exit 1
            }

        set error [catch "sysWrite $outUnit $outBuffI $xN"]
           
        if {$error} \
           {
            puts stderr "vdExpand Error $error Writing Imaginary Data. Schedule: $lineCount"
            exit 1
           }
       }

    set thisList "$xN $lineCount 2"

    if {$verbose} \
       {
        set nS [expr $realCount + $imagCount]
        set b  0.0%

        if {$nS} {set b [format %11.1f%% [expr 100.0 - 100.0*double($realCount - $imagCount)/double($nS)]]}

        puts stderr "vdExpand. Vector Size:           [format %12d $xN]"
        puts stderr "vdExpand. Samples Found:         [format %12d $trueCount]"
        puts stderr "vdExpand. Samples Used:          [format %12d $lineCount]"
        puts stderr "vdExpand. Real Parts Used:       [format %12d $realCount]"
        puts stderr "vdExpand. Imaginary Parts Used:  [format %12d $imagCount]"
        puts stderr "vdExpand. Output Data Bytes:     [format %12s [getSafeDataSize $thisList]]"
        puts stderr "vdExpand. RPD Balance:           $b"
        puts stderr ""
       }
   } \
else \
   {
    set lineCount 0
    set keepCount 0

    while {[gets $sUnit str] >= 0} \
       {
        if {[strcasecmp $vCount Auto]} \
           {
            if {$lineCount > $vCount - 1} break
           }

        incr lineCount

        set key [lindex $str 0]

        if {![strcasecmp $key $keepKey]} \
           {
            incr keepCount

            set error [catch "sysReadB $inUnit $inBuffR $n"]
     
            if {$error} \
               {
                puts stderr "vdExpand Error $error Reading Data. Schedule: $lineCount Input: $keepCount."
                exit 1
               }

            if {$maskFlag} \
               {
                set error [catch "sysWrite $outUnit $oneBuff $n"]
               } \
            else \
               {
                set error [catch "sysWrite $outUnit $inBuffR  $n"]
               }

            if {$error} \
               {
                puts stderr "vdExpand Error $error Writing Data. Schedule: $lineCount Input: $keepCount."
                exit 1
               }
           } \
        else \
           {
            set error [catch "sysWrite $outUnit $zeroBuff $n"]
    
            if {$error} \
               {
                puts stderr "vdExpand Error $error Writing Zero Data. Schedule: $lineCount Input: $keepCount."
                exit 1
               }
           }
       }

    set thisList "$xN $lineCount 2"

    if {$verbose} \
       {
        if {$lineCount < 1} \
           {
            set pct 0.0
           } \
        else \
           {
            set pct [expr 100.0*double($keepCount)/double($lineCount)]
           }
   
 
        puts stderr "vdExpand. Vector Size:         [format %12d $xN]"
        puts stderr "vdExpand. Total Vector Pairs:  [format %12d $lineCount]"
        puts stderr "vdExpand. Vectors Pairs Kept:  [format %12d $keepCount]"
        puts stderr "vdExpand. Output Data Bytes:   [format %12s [getSafeDataSize $thisList]]"
        puts stderr "vdExpand. Sampling Density:    [format %11.1f%% $pct]"
        puts stderr ""
       }
   }

if {![dataSizeIsSafe $thisList]} \
   {
    puts stderr "vdExpand. *** Warning: Data Size Might Exceed 2GB."
    puts stderr "vdExpand. *** A 64-bit Version of NMRPipe Might Be Required."
   }

exit 0

