#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# TALOS: Torsion Angle Likeness Optimized by Shifts
# (C) Shen, Cornilescu, Delaglio, and Bax 1997-2010
#     NIH Lab of Chemical Physics NIDDK
#
# talos.tcl: finds chemical shift homology matches in database.
# vina.tcl:  classifies and summarizes predictions.
# rama.tcl:  interactive display and adjustment of predictions.

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set talosVersion 2010.160.15.01 

#
# Initialize variables:

proc vinaInitVars {} \
{
   global inName sumName mapName refName clipName predDir talosDir env
   global seqList r1 rN sumTab ramaTab refTab clipTab avgFunc devFunc
   global dPhi dPsi degLimit matchCount currentRes
   global autoFlag clipFlag

   if {[info exists env(TALOS_DIR)]} \
      {
       set talosDir $env(TALOS_DIR)
      } \
   else \
      {
       set talosDir .
      }

   set mapName     vina.dat 
   set clipName    sslimit.tab
   set inName      None
   set refName     None
   set sumName     pred.tab
   set predDir     pred
   set matchCount  10
   set autoFlag    3
   set clipFlag    0

   set avgFunc     getAvgAngle
   set devFunc     getStdDevAngle

   set dPhi        35.0
   set dPsi        35.0
   set degLimit    35.0
   set ramaTab     0
   set clipTab     0
   set refTab      0
   set sumTab      0

   set currentRes  0
   set r1          0
   set rN          0
   set seqList     ""
}

#
# Extract command line arguments.

proc vinaArgs {} \
{
   global inName sumName refName mapName predDir talosDir matchCount autoFlag
   global dPhi dPsi degLimit ARGC ARGV talosVersion

   puts "TALOS: Torsion Angle Likeness Optimized By Shifts."
   puts "Version $talosVersion Shen, Cornilescu, Delaglio, and Bax"
   puts ""

   if {[flagLoc $ARGV -help]} \
      {
       puts "vina.tcl: summarize TALOS or TALOS+ predictions in a single table."
       puts "VINA Options:"
       puts " -predDir  pDir   \[pred\]      Directory of existing predictions."
       puts " -talosDir mDir   \[.\]         TALOS Main Directory: TALOS_DIR."
       puts " -in       inName  \[None\]     Input shift table used previously."
       puts " -ref      refName \[None\]     Reference PDB for testing."
       puts " -sum      sumName \[pred.tab\] Prediction summary output name."
       puts " -map      mapName \[vina.dat\] Ramachandran Binary Map name."
       puts " -count    mc      \[10\]       Max match count per triplet."
       puts " -dPhi     phiDev  \[35.0\]     Max Allowable Phi Std Deviation."
       puts " -dPsi     psiDev  \[35.0\]     Max Allowable Psi Std Deviation."
       puts " -lim      limDeg  \[35.0\]     Limit for 'Near Test'."
       puts " -auto                        Automatic classification (Default)."
       puts " -noauto                      No automatic classification."
       puts " -notk                        Suppress Graphics Windows."
       puts "VINA Examples:"
       puts " vina.tcl -in ubiqCS.tab"
       puts " vina.tcl -in ubiqCS.tab -ref xray.pdb"
       puts "See also:"
       puts " talos.tcl                 Creates predictions by shift homology."
       puts " rama.tcl                  Graphical interface to inspect results."
       exit 0
      }

   getArgD $ARGV -predDir  predDir
   getArgD $ARGV -talosDir talosDir

   set mapName $talosDir/$mapName

   getArgD $ARGV -in       inName
   getArgD $ARGV -ref      refName
   getArgD $ARGV -sum      sumName
   getArgD $ARGV -out      sumName
   getArgD $ARGV -map      mapName

   getArgD $ARGV -mc       matchCount
   getArgD $ARGV -dPhi     dPhi
   getArgD $ARGV -dPsi     dPsi
   getArgD $ARGV -lim      degLimit

   if {[flagLoc $ARGV -auto]}   {set autoFlag 3}
   if {[flagLoc $ARGV -Auto]}   {set autoFlag 3}
   if {[flagLoc $ARGV -AUTO]}   {set autoFlag 3}
   if {[flagLoc $ARGV -noauto]} {set autoFlag 0}
}

#
# Create an empty table for the summary.

proc vinaInitTable {} \
{
   global sumTab

   gdbInit
   gdbCreate dBase

   set varInfo \
      {
       {RESID   int   %4d}
       {RESNAME text  %s}
       {PHI     float %8.3f}
       {PSI     float %8.3f}
       {DPHI    float %8.3f}
       {DPSI    float %8.3f}
       {DIST    float %8.3f}
       {COUNT   int   %2d}
       {CLASS   text  %s}
      }

   set sumTab [gdbCreate table -name "TALOS Prediction Summary"]

   foreach var $varInfo \
      {
       set name [lindex $var 0]
       set type [lindex $var 1]
       set fmt  [lindex $var 2]

       gdbCreate variable -parent $sumTab \
          -name $name -varType $type -varFmt $fmt
      }

   return $sumTab
}

#
# Read the reference table for testing.

proc vinaInitRef {} \
{
   global refName refTab

   if {![strcasecmp $refName None]} {return 0}

   set refTab [gdbRead table -pdb -in $refName -name "Reference PDB"]

   return $refTab
}

#
# Get the reference angles for testing.

proc vinaRef {} \
{
   global refTab currentRes refPhi refPsi

   if {!$refTab} {return -1}

   set aList   [getAngles [expr $currentRes - 1] $refTab]
   set refPhi  [lindex $aList 0]
   set refPsi  [lindex $aList 1]

   if {$refPhi > 999.0 || $refPsi > 999.0} {return -1}

   set id [vinaGetClassID $refPhi $refPsi]

   return $id
}

#
# Read the PHI/PSI classification map.

proc vinaInitMap {} \
{
   global alphaClass betaClass posClass neClass seClass sbClass nullClass
   global mapName mapPtr 

   readROI -ndim 2 -in $mapName -roi 1

   set roiInfo  [getROIInfo -roi 1]

   set mapPtr   [lindex $roiInfo 0]
   set sInfo    [lindex $roiInfo 2]

   set nx       [lindex $sInfo 0]
   set ny       [lindex $sInfo 1]

   set maxClass [format %.0f [vMax $mapPtr [expr $nx*$ny]]]

   set nullClass  [vinaGetSurfVal    0.0    0.0]
   set alphaClass [vinaGetSurfVal  -57.0  -46.0]
   set betaClass  [vinaGetSurfVal -102.0  118.0]
   set sbClass    [vinaGetSurfVal -102.0 -178.0]
   set posClass   [vinaGetSurfVal   90.0    2.0]

   set neClass    [format %.0f [expr $maxClass + 2]]
   set seClass    [format %.0f [expr $maxClass + 3]]

   return $mapPtr
}

#
# Get the PHI/PSI rama flag from the residue-specific ramachandran map.

proc vinaGetRamaVal { phi psi } \
{
    return 1
}

#
# Get the PHI/PSI class ID from the classification map.

proc vinaGetSurfVal { phi psi } \
{
    global mapPtr

    set x   [expr int(0.5 + $phi + 180.0)]
    set y   [expr int(0.5 + $psi + 180.0)]

    if {$x < 0} {set x 0}
    if {$y < 0} {set y 0}

    if {$x > 360} {set x 360}
    if {$y > 360} {set y 360}

    set loc [expr $x + 361*$y]

    set val [expr int([getPtrVal $mapPtr $loc])]

    return $val
}

#
# Get the PHI/PSI class ID from the classification map.

proc vinaGetClassID { phi psi } \
{
    global alphaClass betaClass sbClass posClass neClass seClass nullClass
    global mapPtr 

    set val [vinaGetSurfVal $phi $psi]

    if {$val == $sbClass}    {return $betaClass}
    if {$val == $alphaClass} {return $alphaClass}
    if {$val == $betaClass}  {return $betaClass}
    if {$val == $posClass}   {return $posClass}

    set x [expr int(0.5 + $phi + 180.0)]
    set y [expr int(0.5 + $psi + 180.0)]

    if {$x < 0} {set x 0} 
    if {$y < 0} {set y 0}
    
    if {$x > 360} {set x 360} 
    if {$y > 360} {set y 360}

    if {$val != $nullClass} {return $val}

    if {$x >= 0 && $x < 170 && $y >= 0 && $y < 40} {return $betaClass}
    if {$x >= 0 && $x < 170 && $y > 230} {return $betaClass}

    if {$x >= 0 && $x < 170 && $y > 50 && $y < 230} {return $alphaClass}
    if {$x > 180 && $y > 100 && $y < 260} {return $posClass}

    if {$x > 180 && $y > 260} {return $neClass}
    if {$x > 180 && $y < 100} {return $seClass}

    return $val
}

#
# Add an entry to the table.

proc vinaAddEntry { resID } \
{
   global seqList r1 rN sumTab

   set entry [gdbCreate entry -parent $sumTab]

   if {$resID < $r1 || $resID > $rN} \
      {
       set resName ?
      } \
   else \
      {
       set resName [lindex $seqList [expr $resID - $r1]]
      }

   gdbSet $entry RESID   $resID
   gdbSet $entry RESNAME $resName

   gdbSet $entry PHI     9999.0
   gdbSet $entry PSI     9999.0
   gdbSet $entry DPHI       0.0
   gdbSet $entry DPSI       0.0
   gdbSet $entry DIST       0.0

   gdbSet $entry COUNT        0
   gdbSet $entry CLASS     None 

   return $entry
}

#
# Save the summary.

proc vinaSave {} \
{
   global sumName sumTab

   if {![strcasecmp $sumName None]} \
      {
       gdbWrite $sumTab
      } \
   else \
      {
       gdbWrite $sumTab $sumName

       puts "Created prediction summary table $sumName"
      }
}

#
# Find the range of residues to analyze.

proc vinaResCount {} \
{
   global seqList r1 rN inName predDir

   set n 0

   if {![strcasecmp $inName None]} \
      {
       set fList [lsort -dictionary [glob -nocomplain $predDir/res*.tab]]

       foreach fName $fList \
          {
           set  rName  [file rootname [file tail $fName]]
           scan $rName "res%d" resID

           if {!$n} \
              {
               set r1 $resID
               set rN $resID
              }

           if {$resID < $r1} {set r1 $resID}
           if {$resID > $rN} {set rN $resID}

           incr n
          }
      } \
   else \
      {
       set inTab    [gdbRead table -in $inName]
       set seqList  [getSeqList $inTab r1 rN]
      }
}

#
# Extract and store statistics for a given residue.

proc vinaSetEntry { entry ramaTab } \
{
   global avgFunc devFunc refPhi refPsi

   set resID   [gdbGet $entry RESID]
   set resName [gdbGet $entry RESNAME]

   set var     [gdbFindByName $ramaTab PHI]
   set phiList [gdbGetInfo $var -parms values]

   set var     [gdbFindByName $ramaTab PSI]
   set psiList [gdbGetInfo $var -parms values]

   set class   [vinaClassify $resID $resName $ramaTab $phiList $psiList]

   set var     [gdbFindByName $ramaTab W]
   set wList   [gdbGetInfo $var -parms values]

   set var     [gdbFindByName $ramaTab DIST]
   set dList   [gdbGetInfo $var -parms values]

   set phiAvg  [format "%.2f" [$avgFunc [lselect $phiList $wList]]]
   set psiAvg  [format "%.2f" [$avgFunc [lselect $psiList $wList]]]

   set phiDev  [format "%.2f" [$devFunc [lselect $phiList $wList]]]
   set psiDev  [format "%.2f" [$devFunc [lselect $psiList $wList]]]

   set dAvg    [format "%.2f" [getAvg [lselect $dList $wList]]]
   set n       [llength [lselect $dList $wList]]

   if {![strcasecmp $class Bad]} \
      {
       if {[vinaNearRef $phiAvg $psiAvg $refPhi $refPsi]} \
          {
           set class Good
          }
      }

   gdbSet $entry PHI     $phiAvg 
   gdbSet $entry PSI     $psiAvg
   gdbSet $entry DPHI    $phiDev 
   gdbSet $entry DPSI    $psiDev 
   gdbSet $entry DIST    $dAvg 

   gdbSet $entry COUNT   $n 
   gdbSet $entry CLASS   $class
}

#
# Test if angles are reasonably near to reference.

proc vinaNearRef { phi psi refPhi refPsi } \
{
   global degLimit

   set phiDiff [expr $refPhi - $phi]
   set psiDiff [expr $refPsi - $psi]

   set dPhi2 [expr 180.0 - $phiDiff]
   set dPsi2 [expr 180.0 - $psiDiff]

   if {abs($dPhi2) < abs($phiDiff)} {set dPhi $dPhi2}
   if {abs($dPsi2) < abs($psiDiff)} {set dPsi $dPsi2}

   if {sqrt($phiDiff*$phiDiff + $psiDiff*$psiDiff) < $degLimit} {return 1}

   return 0
}

#
# Classify the individual and summarized predictions.

proc vinaClassify { resID resName ramaTab phiList psiList } \
{
   global alphaClass betaClass posClass neClass seClass nullClass
   global autoFlag matchCount avgFunc devFunc dPhi dPsi

#
# Initialize:

   set minMembers [expr $matchCount - 1]
   set n          [llength $phiList]

   set classList   ""
   set ramaList    ""
   set bestClass   -1

   set maxMembers   0
   set aCount       0
   set bCount       0
   set pCount       0
   set neCount      0
   set seCount      0
   set oCount       0

#
# Find the most populated region:

   for {set i 0} {$i < $n} {incr i} \
      {
       set phi   [lindex $phiList $i]
       set psi   [lindex $psiList $i]

       set class [vinaGetClassID $phi $psi]
       set rVal  [vinaGetRamaVal $phi $psi]

       if {$class == $alphaClass} \
          {
           incr aCount

           if {$aCount > $maxMembers} \
              {
               set maxMembers $aCount
               set bestClass $alphaClass
              }
          } \
       elseif {$class == $betaClass} \
          {
           incr bCount

           if {$bCount > $maxMembers} \
              {
               set maxMembers $bCount
               set bestClass $betaClass
              }
           } \
       elseif {$class == $posClass} \
          {
           incr pCount
 
           if {$pCount > $maxMembers} \
              {
               set maxMembers $pCount
               set bestClass $posClass
              }
          } \
       elseif {$class == $neClass} \
          {
           incr neCount
  
           if {$neCount > $maxMembers} \
              {
               set maxMembers $neCount
               set bestClass $neClass
              }
          } \
       elseif {$class == $seClass} \
          {
           incr seCount

           if {$seCount > $maxMembers} \
              {
               set maxMembers $seCount
               set bestClass $seClass
              }
          } \
       else \
          {
           set  class $nullClass
           incr oCount

           if {$oCount > $maxMembers} \
              {
               set maxMembers $oCount
               set bestClass  -1
              }
          }

       lappend classList $class
       lappend ramaList  $rVal
      }

#
# Set the weights on for only the most populated region:

   set entry [gdbFirst $ramaTab]
   set i     0

   while {$entry} \
      {
       set  thisClass [lindex $classList $i]
       incr i

       if {$thisClass == $bestClass || $bestClass < 0} \
          {
           gdbSet $entry W 1.0
          } \
       else \
          {
           gdbSet $entry W 0.0
          }       

       set entry [gdbNext $entry]
      }

#
# Adjust unclassified points if required:

   if {$autoFlag == 3} \
      {
       set var     [gdbFindByName $ramaTab W]
       set wList   [gdbGetInfo $var -parms values]

       set phiAvg  [format "%.2f" [$avgFunc [lselect $phiList $wList]]]
       set psiAvg  [format "%.2f" [$avgFunc [lselect $psiList $wList]]]

       set maxMembers   0
       set aCount       0
       set bCount       0
       set pCount       0
       set neCount      0
       set seCount      0
       set oCount       0
       set i            0

       set entry   [gdbFirst $ramaTab]

       while {$entry} \
          {
           set  oldClass [lindex $classList $i]
           set  class $oldClass
           incr i

           set phi [gdbGet $entry PHI]
           set psi [gdbGet $entry PSI]
           set w   [gdbGet $entry W]

           if {$class == $nullClass} \
              {
               if {[vinaNearRef $phi $psi $phiAvg $psiAvg]} \
                  {
                   gdbSet $entry W 1.0
                   set class  $bestClass
                  }
              }

           if {$class == $alphaClass} \
              {
               incr aCount
               if {$aCount > $maxMembers} {set maxMembers $aCount}
              } \
           elseif {$class == $betaClass} \
              {
               incr bCount
               if {$bCount > $maxMembers} {set maxMembers $bCount}
              } \
           elseif {$class == $posClass} \
              {
               incr pCount
               if {$pCount > $maxMembers} {set maxMembers $pCount}
              } \
           elseif {$class == $neClass} \
              {
               incr neCount
               if {$neCount > $maxMembers} {set maxMembers $neCount}
              } \
           elseif {$class == $seClass} \
              {
               incr seCount
               if {$seCount > $maxMembers} {set maxMembers $seCount}
              } \
           else \
              {
               incr oCount
               if {$oCount > $maxMembers} {set maxMembers $oCount}
              }

           set entry [gdbNext $entry]
          }
      }

#
# Classify the overall prediction:

   set var     [gdbFindByName $ramaTab W]
   set wList   [gdbGetInfo $var -parms values]

   set phiDev  [format "%.2f" [$devFunc [lselect $phiList $wList]]]
   set psiDev  [format "%.2f" [$devFunc [lselect $psiList $wList]]]

   switch $autoFlag \
      {
       0 {
          return New
         }

       1 {
          if {$n < $matchCount} {return Warn}
          if {$oCount > 0}      {return New}

          if {$pCount > 0 && $pCount < $minMembers} {return New}
          if {$maxMembers < $minMembers}            {return New}

          set refClass [vinaRef]

          if {$refClass >= 0 && $refClass != $bestClass} {return Bad}

          return Good
         }

       2 {
          if {$n < $matchCount} {return Warn}
          if {$oCount > 0}      {return New}

          if {$pCount > 0 && $pCount < $minMembers} {return Warn}
          if {$phiDev > $dPhi || $psiDev > $dPsi}   {return Warn}
          if {$maxMembers < $minMembers}            {return Warn}

          set refClass [vinaRef]

          if {$refClass >= 0 && $refClass != $bestClass} {return Bad}

          return Good
         }

       3 {
          if {$n < $matchCount} {return Warn}
          if {$oCount > 0}      {return Warn}

          if {$pCount > 0 && $pCount < $minMembers} {return Warn}
          if {$phiDev > $dPhi || $psiDev > $dPsi}   {return Warn}
          if {$maxMembers < $minMembers}            {return Warn}

          set refClass [vinaRef]

          if {$refClass >= 0 && $refClass != $bestClass} {return Bad}

          return Good
         }

       default {
                return New
               }
      }
}

#
# Display a message window for the summary computation.

proc vinaFrames {} \
{
    global bigfont wishFlag

    if {!$wishFlag} return

    label .l -bg red -fg white -relief raised -font $bigfont \
       -text "Summarizing Predictions, Please Wait ..."

    button .b -text Quit -command exit

    pack .l .b -side top -expand 1 -fill x
 
    update
}

#
# Update the summary for each residue.

proc vinaUpdate {} \
{
    global r1 rN predDir currentRes autoFlag wishFlag

    for {set currentRes $r1} {$currentRes <= $rN} {incr currentRes} \
       {
        if {$wishFlag} \
           {
            wm title . "Residue $currentRes of $r1 to $rN."
            update
           } \
        else \
           {
            puts "Residue $currentRes of $r1 to $rN."
           }

        set fName [format $predDir/res%03d.tab $currentRes]
        set entry [vinaAddEntry $currentRes]

        if {[file exists $fName]} \
           {
            set ramaTab [gdbRead table -in $fName]

            vinaSetEntry $entry $ramaTab

            gdbWrite   $ramaTab $fName
            gdbDestroy $ramaTab
           }
       } 
}

#
# Main application:

setOSType
setFonts

vinaInitVars
vinaArgs
vinaInitTable
vinaInitMap
vinaInitRef
vinaResCount
vinaFrames
vinaUpdate
vinaSave

exit
