#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@" -notk

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Prepare XPLOR Restraint Tables for MFR Refinement."
     puts " -pdb    \[frag.pdb\]         PDB Input Name."
     puts " -dc     \[dObs\[A-Z\].tab\]    DC Input Tables."
     puts "Optional Restraints:"
     puts " -noe    \[None\]          DYNAMO NOE Input." 
     puts " -tor    \[None\]          DYNAMO/TALOS Torsion Restraints." 
    exit 1
   }

set dcNameList [glob -nocomplain dObs\[A-Z\].tab]
set pdbName    frag.pdb
set noeInName  None
set torInName  None
set noeOutName noes.xdcn
set torOutName torsions.xdih 

getListArgD $ARGV -dc dcNameList

getArgD $ARGV -pdb    pdbName
getArgD $ARGV -noe    noeInName
getArgD $ARGV -tor    torInName

#
#----------------------------------------------------------------------

proc swapAtomPair { rIPtr aIPtr rJPtr aJPtr } \
{
   upvar $rIPtr resIDI
   upvar $aIPtr atomNameI

   upvar $rJPtr resIDJ
   upvar $aJPtr atomNameJ

   set resIDK    $resIDI
   set atomNameK $atomNameI

   set resIDI    $resIDJ
   set atomNameI $atomNameJ

   set resIDJ    $resIDK
   set atomNameJ $atomNameK
}

proc getDCPairName { rIPtr aIPtr rJPtr aJPtr } \
{
   upvar $rIPtr resIDI
   upvar $aIPtr atomNameI

   upvar $rJPtr resIDJ
   upvar $aJPtr atomNameJ

   if {$resIDI > $resIDJ} \
      {
       swapAtomPair resIDI atomNameI resIDJ atomNameJ
      }

   if {$resIDI == $resIDJ && [strcasecmp $atomNameI $atomNameJ] > 0} \
      {
       swapAtomPair resIDI atomNameI resIDJ atomNameJ
      }

   if {$resIDI == $resIDJ && \
        [string length $atomNameI] > [string length $atomNameJ]} \
      {
       swapAtomPair resIDI atomNameI resIDJ atomNameJ
      }


   set offset   [expr $resIDJ - $resIDI]
   set pairName [format "%s,%s:%+d" $atomNameI $atomNameJ $offset]

   return $pairName
}

proc groupDCTab { dcTab dInfoPtr r1 rN } \
{
   upvar $dInfoPtr dInfo

   set pairList ""

   if {!$dcTab} {return $pairList}

   set c ""

   append c "RESID_I >= $r1 && RESID_I <= $rN && "
   append c "RESID_J >= $r1 && RESID_J <= $rN && D < 999"

   set eList  [gdbSelect -from $dcTab -cond $c]
   set haList "HA1 HA2 HA3 HA#"

   foreach entry $eList \
      {
       set resIDI     [gdbGet $entry RESID_I]
       set atomNameI  [gdbGet $entry ATOMNAME_I]
       set resIDJ     [gdbGet $entry RESID_J]
       set atomNameJ  [gdbGet $entry ATOMNAME_J]
       set dObs       [gdbGet $entry D]

       if {$resIDI < $r1 || $resIDI > $rN} continue
       if {$resIDJ < $r1 || $resIDJ > $rN} continue

       if {$dObs > 999.0} continue

       if {[lsearch -exact $haList $atomNameI] >= 0} {set atomNameI HA}
       if {[lsearch -exact $haList $atomNameJ] >= 0} {set atomNameJ HA}

       set pairName [getDCPairName resIDI atomNameI resIDJ atomNameJ]

       if {[lsearch -exact $pairList $pairName] < 0} \
          {
           lappend pairList $pairName
          }

       lappend dInfo($pairName) $entry
      }

   return $pairList
}

#
#----------------------------------------------------------------------

dynInit -fast

set noeTab 0
set tTab   0

catch "file delete $noeOutName $torOutName"

if {[strcasecmp $noeInName None]} \
   {
    if {![file exists $noeInName]} \
       {
        puts stderr "$argv0 Error Finding NOE Input $noeInName"
        exit 1
       }

    set noeTab [gdbRead table -in $noeInName]
   }

if {[strcasecmp $torInName None]} \
   {
    if {![file exists $torInName]} \
       {
        puts stderr "$argv0 Error Finding NOE Input $torInName"
        exit 1
       }

    set tTab [gdbRead table -in $torInName]
   }

set dTabList ""

foreach dcName $dcNameList \
   {
    if {![file exists $dcName]} \
       {
        puts stderr "$argv0 Error Finding DC Input $dcName"
        exit 1
       }

    lappend dTabList [gdbRead table -in $dcName]
   }

set haList "HA1 HA2 HA3 HA#"

set pdbSrc  [dynRead -pdb -in $pdbName]
set pdbInfo [dynGetInfo -src $pdbSrc]
set r1      [lindex $pdbInfo 0]
set rN      [lindex $pdbInfo 1]

set t 500

foreach dTab $dTabList \
   {
    set pairList [groupDCTab $dTab dInfo $r1 $rN]

    foreach pair $pairList \
       {
        set atoms [split $pair ": ,"]
        set atomI [lindex $atoms 0]
        set atomJ [lindex $atoms 1]

        set thisName [format "dc-%03d-%s%s.xdip" $t $atomI $atomJ]

        set error [catch {set xUnit [open $thisName w]}]

        if {$error} \
           {
            puts "Error Opening Output File $thisName"
            exit 1
           }

        foreach entry $dInfo($pair) \
           {
            set residI    [gdbGet $entry RESID_I]
            set resNameI  [gdbGet $entry RESNAME_I]
            set atomNameI [gdbGet $entry ATOMNAME_I]
            set residJ    [gdbGet $entry RESID_J]
            set resNameJ  [gdbGet $entry RESNAME_J]
            set atomNameJ [gdbGet $entry ATOMNAME_J]
            set d         [gdbGet $entry D]
            set dd        [gdbGet $entry DD]
            set w         [gdbGet $entry W]

            if {![strcmp $resNameI GLY]} \
               {
                if {[lsearch -exact $haList $atomNameI] >= 0} \
                   {
                    set atomNameI HA#
                   }
               }

            if {![strcmp $resNameJ GLY]} \
               {
                if {[lsearch -exact $haList $atomNameJ] >= 0} \
                   {
                    set atomNameJ HA#
                   }
               }

            puts $xUnit ""

            puts $xUnit \
             "! $residI $resNameI $atomNameI - $residJ $resNameJ $atomNameJ"

            puts $xUnit "assign ( resid   $t and name OO )"
            puts $xUnit "       ( resid   $t and name  Z )"
            puts $xUnit "       ( resid   $t and name  X )"
            puts $xUnit "       ( resid   $t and name  Y )"
            puts $xUnit "       ( resid   $residI and name $atomNameI )"
            puts $xUnit "       ( resid   $residJ and name $atomNameJ )"
            puts $xUnit "       $d 0.0 0.0"
            puts $xUnit " "
           }

        close $xUnit
        set dInfo($pair) ""
       }

    incr t
   }

#
# Optional NOE restraints:

if {$noeTab} \
   {
    set error [catch {set noeUnit [open $noeOutName w]}]

    if {$error} \
       {
        puts "$argv0 Error Opening Output File $noeOutName"
        exit 1
       }

    set c "RESID_I >= $r1 && RESID_I <= $rN && RESID_J >= $r1 && RESID_J <= $rN"

    set noeList [gdbSelect -from $noeTab -cond $c]

    foreach entry $noeList \
       {
        set nameI [gdbGet $entry ATOMNAME_I]
        set resI  [gdbGet $entry RESID_I]
        set nameJ [gdbGet $entry ATOMNAME_J]
        set resJ  [gdbGet $entry RESID_J]
        set dHi   [gdbGet $entry D_HI]
        set dLo   [gdbGet $entry D_LO]

        set dTarget [expr ($dHi+$dLo)/2.0]
        set dWell   [expr ($dHi-$dLo)/2.0]

        puts $noeUnit " "
        puts $noeUnit "! $nameI $resI - $nameJ $resJ"
        puts $noeUnit "assign ( resid $resI and name $nameI )"
        puts $noeUnit "       ( resid $resJ and name $nameJ )"
        puts $noeUnit "       $dTarget $dWell $dWell"
       }

    close $noeUnit
   }

#
# Optional Torsion Restraints (e.g. TALOS):

if {$tTab} \
   {
    set error [catch {set tUnit [open $torOutName w]}]

    if {$error} \
       {
        puts "$argv0 Error Opening Output File $tOutName"
        exit 1
       }

    set c ""

    append c "RESID_I >= $r1 && RESID_I <= $rN && "
    append c "RESID_J >= $r1 && RESID_J <= $rN && "
    append c "RESID_K >= $r1 && RESID_K <= $rN && "
    append c "RESID_L >= $r1 && RESID_L <= $rN"

    set torList [gdbSelect -from $tTab -cond $c]

    foreach entry $torList \
       {
        set nameI [gdbGet $entry ATOMNAME_I]
        set resI  [gdbGet $entry RESID_I]
        set nameJ [gdbGet $entry ATOMNAME_J]
        set resJ  [gdbGet $entry RESID_J]
        set nameK [gdbGet $entry ATOMNAME_K]
        set resK  [gdbGet $entry RESID_K]
        set nameL [gdbGet $entry ATOMNAME_L]
        set resL  [gdbGet $entry RESID_L]

        set angleLo [gdbGet $entry ANGLE_LO]
        set angleHi [gdbGet $entry ANGLE_HI]

        set dAngle  [expr $angleHi - $angleLo]
        set tAngle  [expr 0.5*($angleHi + $angleLo)]

        puts $tUnit ""
        puts $tUnit " assign ( resid $resI and name $nameI )"
        puts $tUnit "        ( resid $resJ and name $nameJ )"
        puts $tUnit "        ( resid $resK and name $nameK )"
        puts $tUnit "        ( resid $resL and name $nameL )"
        puts $tUnit "        1.0 $tAngle $dAngle 2"
       }

    close $tUnit
   }

exit
