
/***/
/* bruk2pipe: conversions from Bruker format to nmrPipe stream.
/*            Frank Delaglio (c) 1995-2018.
/***/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define DSPLIST

#include "version.h"
#include "cmndargs.h"
#include "testsize.h"
#include "dataio.h"
#include "inquire.h"
#include "minmax.h"
#include "memory.h"
#include "fdatap.h"
#include "prec.h"

#include "bruker.h"
#include "bruk2pipe.h"
#include "namelist.h"
#include "lintfix.h"

#define FPR (void)fprintf
#define SPR (void)sprintf

#define NMRTXT       "NMRTXT"
#define BRUK_AMX     1
#define BRUK_DMX     2
#define BRUK_AM      3

static int getparams();

int main( argc, argv )

   int  argc;
   char **argv;
{
    int   xSize, ySize, zSize, aSize, xMid, yMid, zMid, aMid, planeCount, 
          zPlane, aPlane, xFreqSize, yFreqSize, zFreqSize, aFreqSize, xExtSize,
          megabytes, maxSize, maxSlices, slicesIn, slicesLeft, ptsIn, ptsOut, plane, 
          dimCount, dim, quadType, quadState, brukType, checkFlag, i2fFlag, xyzFlag, ovFlag, 
          swapFlag, thisSize, thisQuad, firstPlane, lastPlane, wordSize, byteCount, 
          byteOffset, pipeFlag, extFlag, yQuadFlag, zQuadFlag, aQuadFlag, closeFlag,
          xTD, yTD, zTD, aTD, decim, dspfvs, aqmod, zfFlag, dirFlag, convSize, skipSize,
          macCount, fmtCount, verbose, status, n, error;

    NMR_INT totalSlices64, slicesOut64;
 
    char  *brukMat, inName[NAMELEN+1], outName[NAMELEN+1], outFmt[NAMELEN+1],
          macName[NAMELEN+1], cmndName[NAMELEN+1], serverListName[NAMELEN+1],
          ctemp[NAMELEN+1];

    float *dataMat, xSW, ySW, zSW, aSW, xObs, yObs, zObs, aObs, 
          xCar, yCar, zCar, aCar, xOrig, yOrig, zOrig, aOrig, grpdly,
          badThresh, value, fc, fdata[FDATASIZE];

    UNIT  inUnit, outUnit;

/***/
/* Initialize:
/***/

    error      = 0;
    maxSize    = 0;
    wordSize   = 4;
    byteOffset = 0;
    fc         = 1.0;
    value      = 0.0;

    decim      = 0;
    dspfvs     = 0;
    grpdly     = 0.0;

    inUnit     = UNIT_NULL;
    outUnit    = UNIT_NULL;
 
    brukMat    = (char *) NULL;
    dataMat    = (float *) NULL;

    *macName   = '\0';
    *cmndName  = '\0';

    macCount   = 0;
    fmtCount   = 0;
    xyzFlag    = 0;
    i2fFlag    = 1;
    closeFlag  = 0;

    (void) initArgs( argc, argv, CHECK_ENV );
    (void) setHdrNull( fdata );
    (void) dmxNull();
    (void) initDataIO();

/***/
/* Extract command parameters:
/***/

    error = getparams( argc, argv,
                       inName, outFmt, serverListName,
                       fdata, &megabytes, &badThresh, &fc, &byteOffset,
                       &firstPlane, &lastPlane, &brukType, &wordSize, &i2fFlag, &xyzFlag,
                       &checkFlag, &ovFlag, &swapFlag, &pipeFlag, &extFlag,
                       &dirFlag, &decim, &dspfvs, &grpdly, &aqmod, &skipSize, 
                       &zfFlag, &verbose );

    if (error)
       {
        FPR( stderr, "Bruk2Pipe Error extracting arguments.\n" );
        goto shutdown;
       }

    (void) fixfdata( fdata );

    dimCount = getParm( fdata, FDDIMCOUNT, 0 );
    fmtCount = getFmtCount( outFmt );

    if (brukType == BRUK_AM) wordSize = 3;

/***/
/* Handle special indirect acquisition modes:
/***/

    yQuadFlag = getParm( fdata, NDQUADFLAG, CUR_YDIM );
    zQuadFlag = getParm( fdata, NDQUADFLAG, CUR_ZDIM );
    aQuadFlag = getParm( fdata, NDQUADFLAG, CUR_ADIM );

    switch( yQuadFlag )
       {
        case FD_SE:
           SPR( macName, "%s/%s -all", getenv( NMRTXT ), "bruk_seY.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_YDIM );
           macCount++;
           break;
        case FD_GRAD:
           SPR( macName, "%s/%s", getenv( NMRTXT ), "bruk_ranceY.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_YDIM );
           macCount++;
           break;
        default:
           break;
       }

    switch( zQuadFlag )
       {
        case FD_SE:
           SPR( macName, "%s/%s -all", getenv( NMRTXT ), "bruk_seZ.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ZDIM );
           macCount++;
           break;
        case FD_GRAD:
           SPR( macName, "%s/%s", getenv( NMRTXT ), "bruk_ranceZ.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ZDIM );
           macCount++;
           break;
        default:
           break;
       }

    switch( aQuadFlag )
       {
        case FD_SE:
           SPR( macName, "%s/%s -all", getenv( NMRTXT ), "bruk_seA.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ADIM );
           macCount++;
           break;
        case FD_GRAD:
           SPR( macName, "%s/%s", getenv( NMRTXT ), "bruk_ranceA.M" );
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ADIM );
           macCount++;
           break;
        default:
           break;
       }

    if (macCount > 1)
       {
        FPR( stderr, 
             "Bruk2Pipe Warning: %d special acquisition modes;\n",
             macCount );

        FPR( stderr, " Only one special mode is allowed.\n" );
       }

    if (*macName)
       {
        SPR( cmndName, "nmrPipe -fn MAC -macro %s -noRd -noWr", macName );

        if (*outFmt)
           {
            if (xyzFlag && dimCount > 2)
               {
                (void) sprintf( ctemp, " | pipe2xyz -out %s -to 0", outFmt );
                (void) strcat( cmndName, ctemp );
               }
            else
               {
                (void) sprintf( ctemp, " -out %s", outFmt );
                (void) strcat( cmndName, ctemp );
               }

            if (ovFlag) (void) strcat( cmndName, " -ov" );

            *outFmt = '\0';
           }

        xyzFlag = 0;
       }

/***/
/* Set redundant parameters if they have not been explicitly set:
/***/

    if (flagLoc( argc, argv, "-xMODE" ) && !flagLoc( argc, argv, "-aqMODE" ))
       {
        value = getParm( fdata, NDQUADFLAG, CUR_XDIM ); 
        (void) setParm( fdata, FDQUADFLAG, value, 0 );
       }

    if (flagLoc( argc, argv, "-xT" ) && !flagLoc( argc, argv, "-aqN" ))
       {
        value = getParm( fdata, NDAPOD, CUR_XDIM );
        (void) setParm( fdata, FDREALSIZE, value, 0 );
       }

    if (flagLoc( argc, argv, "-yMODE" ) && !flagLoc( argc, argv, "-aq2D" ))
       {
        thisQuad = dimCount < 2 ? 1 : getQuad( fdata, NDQUADFLAG, CUR_YDIM );

        if (thisQuad == 1)
           (void) setParm( fdata, FD2DPHASE, (float)FD_TPPI,   NULL_DIM );
        else
           (void) setParm( fdata, FD2DPHASE, (float)FD_STATES, NULL_DIM );
       }

/***/
/* Extract sizes and quad states; adjust sizes and states for output.
/*   XSIZE, YSIZE, etc begin as count of real+imag points for each axis.
/*   XSIZE is reduced to number of complex points.
/*   Pseudo-quad data is adjusted to be all-real on output.
/***/

    quadType = getParm( fdata, FDQUADFLAG, 0 );

    xSize    = dimCount < 1 ? 1 : getParm( fdata, NDSIZE, CUR_XDIM );
    ySize    = dimCount < 2 ? 1 : getParm( fdata, NDSIZE, CUR_YDIM );
    zSize    = dimCount < 3 ? 1 : getParm( fdata, NDSIZE, CUR_ZDIM ); 
    aSize    = dimCount < 4 ? 1 : getParm( fdata, NDSIZE, CUR_ADIM ); 

    xExtSize = getParm( fdata, NDAPOD, CUR_XDIM );

    switch( quadType )
       {
        case 0:
           quadState  = 2;
           xSize     /= quadState;
           xMid       = 1 + xSize/2;
           xFreqSize  = xSize;
           (void) setParm( fdata, NDSIZE, (float)xSize, CUR_XDIM );
           break;
        case 1:
           quadState = 1;
           xMid      = 1 + xSize/4;
           xFreqSize = xSize/2; 
           break;
        case 2:
           (void) setParm( fdata, NDQUADFLAG, (float)1.0,  CUR_XDIM );
           (void) setParm( fdata, FDQUADFLAG, (float)1.0,  0 );
           quadState = 1;
           xMid      = 1 + xSize/4;
           xFreqSize = xSize/2;
           break;
        default:
           FPR( stderr, "Bruk2Pipe Error:" );
           FPR( stderr, " bad quad type %d.\n", quadType );
           error = 1;
           goto shutdown;
           break;
       }

    planeCount = zSize*aSize;

    if (xyzFlag)
       {
        if (dimCount == 4 && fmtCount == 1)
           {
            (void) setParm( fdata, FDPIPEFLAG,  (float)0,     0 );
            (void) setParm( fdata, FDCUBEFLAG,  (float)1,     0 );
            (void) setParm( fdata, FDFILECOUNT, (float)aSize, 0 );
           }
        else
           {
            (void) setParm( fdata, FDPIPEFLAG,  (float)0,     0 );
            (void) setParm( fdata, FDCUBEFLAG,  (float)0,        0 );
            (void) setParm( fdata, FDFILECOUNT, (float)planeCount, 0 );
           }
       }
    else
       {
        if (dimCount > 2)
           (void) setParm( fdata, FDPIPEFLAG,  (float)1, 0 );
        else
           (void) setParm( fdata, FDPIPEFLAG,  (float)0, 0 );

        (void) setParm( fdata, FDCUBEFLAG,  (float)0,        0 );
        (void) setParm( fdata, FDFILECOUNT, (float)1,        0 );
       }

/***/
/* Center (Zero Freq) Locations are set according to complex data sizes.
/* Axis origins in hertz are set according to carrier at point freqSize/2 + 1.
/* Don't replace explicit definitions of Final Freq-Domain Size or Center:
/***/

    yFreqSize = ySize/2 ? ySize/2 : 1;
    zFreqSize = zSize/2 ? zSize/2 : 1;
    aFreqSize = aSize/2 ? aSize/2 : 1;

    if (flagLoc( argc, argv, "-xFD" ))
       {
        xFreqSize = intArg( argc, argv, "-xFD" );
        xMid      = xFreqSize/2 + 1;
       }

    if (flagLoc( argc, argv, "-yFD" )) yFreqSize = intArg( argc, argv, "-yFD" );
    if (flagLoc( argc, argv, "-zFD" )) zFreqSize = intArg( argc, argv, "-zFD" );
    if (flagLoc( argc, argv, "-aFD" )) aFreqSize = intArg( argc, argv, "-aFD" );

    yMid = 1 + yFreqSize/2;
    zMid = 1 + zFreqSize/2;
    aMid = 1 + aFreqSize/2; 

    if (flagLoc( argc, argv, "-xMID" )) xMid = intArg( argc, argv, "-xMID" );
    if (flagLoc( argc, argv, "-yMID" )) yMid = intArg( argc, argv, "-yMID" );
    if (flagLoc( argc, argv, "-zMID" )) zMid = intArg( argc, argv, "-zMID" );
    if (flagLoc( argc, argv, "-aMID" )) aMid = intArg( argc, argv, "-aMID" );

    (void) setParm( fdata, NDCENTER, (float)xMid, CUR_XDIM );
    (void) setParm( fdata, NDCENTER, (float)yMid, CUR_YDIM );
    (void) setParm( fdata, NDCENTER, (float)zMid, CUR_ZDIM );
    (void) setParm( fdata, NDCENTER, (float)aMid, CUR_ADIM );

    xObs  = getParm( fdata, NDOBS, CUR_XDIM );
    yObs  = getParm( fdata, NDOBS, CUR_YDIM );
    zObs  = getParm( fdata, NDOBS, CUR_ZDIM );
    aObs  = getParm( fdata, NDOBS, CUR_ADIM );

    xSW   = getParm( fdata, NDSW,  CUR_XDIM );
    ySW   = getParm( fdata, NDSW,  CUR_YDIM );
    zSW   = getParm( fdata, NDSW,  CUR_ZDIM );
    aSW   = getParm( fdata, NDSW,  CUR_ADIM );

    xCar  = getParm( fdata, NDCAR, CUR_XDIM );
    yCar  = getParm( fdata, NDCAR, CUR_YDIM );
    zCar  = getParm( fdata, NDCAR, CUR_ZDIM );
    aCar  = getParm( fdata, NDCAR, CUR_ADIM );

/***/
/* Accommodate DMX conversion and its change to the time-domain size.
/***/

    if (brukType == BRUK_DMX)
       {
        if (quadState != 2)
           {
            FPR( stderr, "Bruk2Pipe Error: DMX conversion must be complex.\n" );
            error = 1;
            goto shutdown;
           }

        error = dmxInit( xSize, xExtSize, skipSize, zfFlag,
                         decim, dspfvs, grpdly, aqmod, fc,
                         &convSize );

        if (error)
           {
            FPR( stderr, "Bruk2Pipe Error Initializing DMX conversion.\n" );
            goto shutdown;
           }

        xExtSize = convSize;

        (void) setParm( fdata, NDAPOD, (float)xExtSize, CUR_XDIM );
       }
    else
       {
        if (decim)
           {
            (void) getDMXVal( aqmod, decim, dspfvs, grpdly, &value );
            (void) setParm( fdata, FDDMXVAL, value, 0 );
            (void) setParm( fdata, FDDMXFLAG, (float)0.0, 0 );
           }
       }

    if (extFlag)
       {
        (void) setParm( fdata, NDSIZE, (float)xExtSize, CUR_XDIM );
       }

/***/
/* Don't replace explicit definitions of Axis Origin:
/***/

    if (flagLoc( argc, argv, "-xORIG" ))
       xOrig = fltArg( argc, argv, "-xORIG" );
    else
       xOrig = xObs*xCar - xSW*(xFreqSize - xMid)/xFreqSize;

    if (flagLoc( argc, argv, "-yORIG" ))
       yOrig = fltArg( argc, argv, "-yORIG" );
    else
       yOrig = yObs*yCar - ySW*(yFreqSize - yMid)/yFreqSize;

    if (flagLoc( argc, argv, "-zORIG" ))
       zOrig = fltArg( argc, argv, "-zORIG" );
    else
       zOrig = zObs*zCar - zSW*(zFreqSize - zMid)/zFreqSize;

    if (flagLoc( argc, argv, "-aORIG" ))
       aOrig = fltArg( argc, argv, "-aORIG" );
    else
       aOrig = aObs*aCar - aSW*(aFreqSize - aMid)/aFreqSize;

    (void) setParm( fdata, NDORIG, xOrig, CUR_XDIM );
    (void) setParm( fdata, NDORIG, yOrig, CUR_YDIM );
    (void) setParm( fdata, NDORIG, zOrig, CUR_ZDIM );
    (void) setParm( fdata, NDORIG, aOrig, CUR_ADIM );

/***/
/* Don't replace explicit definitions of Original Time-Domain Size:
/***/

    if (flagLoc( argc, argv, "-xTD" ))
       xTD = intArg( argc, argv, "-xTD" );
    else
       xTD = getParm( fdata, NDAPOD, CUR_XDIM );

    if (flagLoc( argc, argv, "-yTD" ))
       yTD = intArg( argc, argv, "-yTD" );
    else
       yTD = getParm( fdata, NDAPOD, CUR_YDIM );

    if (flagLoc( argc, argv, "-zTD" ))
       zTD = intArg( argc, argv, "-zTD" );
    else
       zTD = getParm( fdata, NDAPOD, CUR_ZDIM );

    if (flagLoc( argc, argv, "-aTD" ))
       aTD = intArg( argc, argv, "-aTD" );
    else
       aTD = getParm( fdata, NDAPOD, CUR_ADIM );

    (void) setParm( fdata, NDTDSIZE, (float)xTD, CUR_XDIM );
    (void) setParm( fdata, NDTDSIZE, (float)yTD, CUR_YDIM );
    (void) setParm( fdata, NDTDSIZE, (float)zTD, CUR_ZDIM );
    (void) setParm( fdata, NDTDSIZE, (float)aTD, CUR_ADIM );

/***/
/* Extract explicit user adjustments to header data.
/* Check use of command-line options.
/***/

    if ((error = parseHdr2( argc, argv, fdata )))
       {
        FPR( stderr, "Bruk2Pipe Error setting User Header.\n" );
        goto shutdown;
       }

    if (flagLoc( argc, argv, "-nocheckArgs" ))
       {
        if (verbose) FPR( stderr, "Bruk2Pipe: Skipping Unused/Redundant Argument Checking.\n" );
       } 
    else 
       {
        (void) checkArgs( argc, argv );
       }

/***/
/* Open a dummy output file for testing purposes.
/* Test for sufficient space as needed.
/* Delete the dummy file.
/* Open the bruker input file.
/***/

    if (*outFmt && checkFlag)
       {
        error = fidOpen( outFmt, outName, cmndName, &outUnit, 
                         1, 1, fdata, ovFlag, xyzFlag, fmtCount, dimCount, 
                         dirFlag, verbose );

        if (error)
           {
            FPR( stderr, "Bruk2Pipe Error Testing %s\n", outName );
            goto shutdown;
           }

        (void) dataClose( outUnit );

        outUnit = UNIT_NULL;

        switch( testNMRout( outName, fdata, xyzFlag, 1 ))
            {
             case FILE_SIZE_FAIL:
                FPR( stderr, "\n" );
                FPR( stderr, "Bruk2Pipe Warning: can't check size for file" );
                FPR( stderr, " %s\n", outName );
                break;
             case  FILE_SIZE_TOO_LARGE:
                FPR( stderr, "\n" );
                FPR( stderr, "Bruk2Pipe Error: insufficient space for file" );
                FPR( stderr, " %s\n", outName );
                error = 1;
                goto shutdown;
                break;
             default:
                break;
            }

        (void) delFile( outName );
       }

    if ((error = brukOpen( inName, &inUnit )))
       {
        FPR( stderr, "Bruk2Pipe Error Opening Input.\n" );
        goto shutdown;
       }

/***/
/* Adjust sizes and allocate memory:
/***/

    n = min( 4, planeCount );

    if (megabytes < 1)
       maxSize = xSize*ySize*quadState*n; /* n was planeCount */
    else
       maxSize = 1024 + (megabytes*1024*1024)/(sizeof(float)+wordSize);

    maxSlices = min( ySize, maxSize/(xSize*quadState) );
    maxSize   = maxSlices*xSize*quadState + 4*byteOffset;

    if (!(brukMat = charAlloc( "bruk", wordSize*maxSize )))
       {
        FPR( stderr, "Bruk2Pipe Error allocating input memory.\n" );
        error = 1;
        goto shutdown;
       }
 
    if (!(dataMat = fltAlloc( "bruk", maxSize )))
       {
        FPR( stderr, "Bruk2Pipe Error allocating output memory.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* Display initial status:
/***/

    if (verbose)
       {
        FPR( stderr, "\n" );

        switch( brukType )
           {
            case BRUK_AM:
               FPR( stderr, "Bruker AM --> NMRPipe Conversion.\n" );
               break;
            case BRUK_DMX:
               FPR( stderr, "Bruker DMX --> NMRPipe Conversion.\n" );
               break;
            default:
               FPR( stderr, "Bruker AMX --> NMRPipe Conversion.\n" );
               break;
           }

        if (*inName)
           FPR( stderr, "Input File: %s\n", inName );            
        else
           FPR( stderr, "Input File: Standard Input.\n" );

        if (*outFmt)
           {
            if (xyzFlag)
               FPR( stderr, "Output Template: %s\n", outFmt );
            else
               FPR( stderr, "Output File: %s\n", outFmt );
           }
        else
           {
            if (*cmndName)
               FPR( stderr, "Output Macro: %s\n", macName );
            else
               FPR( stderr, "Output File: Standard Output.\n" );
           }

        FPR( stderr, "%dD Sizes: ", dimCount );

        for( dim = 1; dim <= dimCount; dim++ )
           {
            thisSize = getParm( fdata, NDSIZE, dim );
            thisQuad = getParm( fdata, NDQUADFLAG, dim );

            if (!thisQuad && dim == 1) thisSize *= 2;

            FPR( stderr, 
                 "(%d %s)",
                 thisSize, thisQuad ? "Real" : "Real+Imag" );
           }

        FPR( stderr, "\n" );
        FPR( stderr, "Byte Swap Mode: %s\n", swapFlag ? "ON" : "OFF" );
        FPR( stderr, "\n" );

        (void) fflush( stderr );
       }

/*
    if (*outFmt && dimCount > 2 && !xyzFlag)
       {
        FPR( stderr, "Bruk2Pipe Notice: %dD output name", dimCount );
        FPR( stderr, " is not a template.\n" );
        FPR( stderr, "\n\n" );
        (void) fflush( stderr );
       }
*/

/***/
/* Find the subset of 2D planes to convert:
/***/

   /* Not implemented yet. */

/***/
/* Read past the specified byte offset of the input.
/*
/* For each plane:
/*   Open the output file associated with the plane.
/*   Write the file header to this plane.
/*
/*   For each group of slices in the plane:
/*     Read slices from the bruker format input.
/*     Convert bruker data to pipe format.
/*     Write the slices to the pipe format output.
/*     Display status.
/***/

    totalSlices64 = (NMR_INT)planeCount*ySize;
    slicesOut64   = (NMR_INT)0;

    if (byteOffset)
       {
        if (pipeFlag)
           {
            if ((error = dataReadCB( inUnit, brukMat, byteOffset, -1, 0 )))
               {
                FPR( stderr, "Bruk2Pipe Error Positioning Input (Pipe).\n" );
                goto shutdown;
               }
           }
        else
           {
            if ((error = dataReadC( inUnit, brukMat, byteOffset )))
               {
                FPR( stderr, "Bruk2Pipe Error Positioning Input.\n" );
                goto shutdown;
               }
           }
       }

    plane = 0;

    for( aPlane = 1; aPlane <= aSize; aPlane++ )
       {
        for( zPlane = 1; zPlane <= zSize; zPlane++ )
           {
            plane++;

            error = fidOpen( outFmt, outName, cmndName, &outUnit, 
                             zPlane, aPlane, fdata, ovFlag, xyzFlag, fmtCount, dimCount,
                             dirFlag, verbose );

            if (error)
               {
                FPR( stderr, "Bruk2Pipe Error Opening Output.\n" );
                goto shutdown;
               }

            slicesLeft = ySize;

            while( slicesLeft )
               {
                slicesIn  = min( slicesLeft, maxSlices );
                ptsIn     = slicesIn*xSize*quadState;
                byteCount = wordSize*ptsIn;

                if (extFlag)
                   ptsOut = slicesIn*xExtSize*quadState;
                else
                   ptsOut = ptsIn;
 
                if (pipeFlag)
                   {
                    if ((error = dataReadCB( inUnit, brukMat, byteCount, -1, 0 )))
                       {
                        FPR( stderr, "Bruk2Pipe Error Reading Input (Pipe).\n" );
                        goto shutdown; 
                       }
                   }
                else
                   { 
                    if ((error = dataReadC( inUnit, brukMat, byteCount )))
                       {
                        FPR( stderr, "Bruk2Pipe Error Reading Input.\n" );
                        goto shutdown; 
                       }
                   } 
    
                (void) ser2fid2D( brukMat, dataMat,
                                  xSize, slicesIn, quadState, quadType,
                                  wordSize, swapFlag, i2fFlag, badThresh, verbose );

                if (brukType == BRUK_DMX)
                   {   
                    (void) dmx2fid2D( dataMat, xSize, slicesIn );
                   }

                if (extFlag && xExtSize < xSize)
                   {
                    (void) xExt2D( dataMat, xSize, xExtSize, slicesIn*quadState );
                   }
 
                if (dataWrite( outUnit, dataMat, (int)(sizeof(float)*ptsOut) ))
                   {
                    FPR( stderr, "Bruk2Pipe Writing Output.\n" );
                    goto shutdown; 
                   }
 
                slicesLeft  -= slicesIn;
                slicesOut64 += slicesIn;

                if (verbose)
                   {
#ifdef NMR64
                    FPR( stderr, "Slice %ld of %ld\015", slicesOut64, totalSlices64 );
#else
                    FPR( stderr, "Slice %d of %d\015", slicesOut64, totalSlices64 );
#endif

                    (void) fflush( stderr );
                   }
               }

            closeFlag = 0;

            if (xyzFlag)
               {
                if (dimCount == 4 && fmtCount == 1)
                   {
                    if (zPlane == zSize) closeFlag = 1;
                   }
                else
                   {
                    closeFlag = 1;
                   }
               }
            else
               {
                if (zPlane == zSize && aPlane == aSize) closeFlag = 1;
               }

            if (closeFlag)
               {
                (void) dataClose( outUnit );
                outUnit = UNIT_NULL;
               }
           }
       }

    if (verbose)
       {
        FPR( stderr, "\n" );
        (void) fflush( stderr );
       }

    if (*cmndName)
       {
        (void) wait( &status );
       }

/***/
/* Exit point:
/***/

shutdown:

    if (brukType == BRUK_DMX) (void) dmxFree();

    if (inUnit  != FB_STDIN)  (void) dataClose( inUnit );
    if (outUnit != FB_STDOUT) (void) dataClose( outUnit );

    (void) deAlloc( "bruk", brukMat, wordSize*maxSize );
    (void) deAlloc( "bruk", dataMat, sizeof(float)*maxSize );

    (void) showDebugIO( argc, argv );

    return( error );
}

/***/
/* brukOpen: open bruker format file for input.
/***/

int brukOpen( inName, inUnitPtr )

   char *inName;
   UNIT *inUnitPtr;
{
    if (*inName)
       {
        if (dataOpen( inName, inUnitPtr, FB_READ )) return( 1 );
       }
    else
       {
        *inUnitPtr = FB_STDIN;
       }

    return( 0 );
}

/***/
/* fidOpen: open a pipe format output file, write an initial header.
/***/

int fidOpen( outFmt, outName, cmndName, outUnitPtr, 
             zPlane, aPlane, fdata, ovFlag, xyzFlag, fmtCount, dimCount,
             dirFlag, verbose )

   UNIT  *outUnitPtr;
   float fdata[FDATASIZE];
   char  *outFmt, *outName, *cmndName;
   int   zPlane, aPlane, ovFlag, xyzFlag, dimCount, dirFlag, verbose;
{
    char   dirName[NAMELEN+1];
    int    initFlag;

    static int dirCreated = 0;

    initFlag = 0;

    if (dirCreated) dirFlag = 0;

    if (dimCount == 4)
       {
        if (fmtCount == 1)
           (void) sprintf( outName, outFmt, aPlane );
        else
           (void) sprintf( outName, outFmt, aPlane, zPlane );
       }
    else
       {
        (void) sprintf( outName, outFmt, zPlane );
       }
 
    if (*outFmt)
       {
        if (zPlane == 1 && aPlane == 1)
           {
            if (dirFlag)
               {
                if (!dirExists( outName, dirName ))
                   {
                    if (verbose)
                       {
                        FPR( stderr,
                             "Bruk2Pipe Notice: Creating directory %s\n",
                             dirName );
                       }

                    if (dirCreate( dirName ))
                       {
                        FPR( stderr,
                             "Bruk2Pipe Warning: Error creating directory %s\n",
                             dirName );
                       }

                    dirCreated = 1;
                   }
               }

            if (!ovFlag && fileExists( outName ))
               {
                FPR( stderr, "Bruk2Pipe Error: output file %s exists.\n", outName );
                return( 1 );
               }
           }

        if (xyzFlag)
           {
            if (dimCount == 4 && fmtCount == 1)
               {
                if (zPlane == 1) initFlag = 1;
               }
            else
               {
                initFlag = 1;
               }
           }
        else
           {
            if (zPlane == 1 && aPlane == 1) initFlag = 1;
           }

        if (initFlag)
           {
            if (dataOpen( outName, outUnitPtr, FB_CREATE )) return( 1 );
           }
       }
    else if (*cmndName)
       {
        if (zPlane == 1 && aPlane == 1)
           {
            if (dataPOpen( cmndName, outUnitPtr, FB_WRITE )) return( 1 );
            initFlag = 1;
           }
       }
    else
       {
        if (zPlane == 1 && aPlane == 1) initFlag = 1;
        *outUnitPtr = FB_STDOUT;
       }

/***/
/* Write a header if this is a seperate plane:
/***/

    if (initFlag)
       {
        if (dataWrite( *outUnitPtr, fdata, (int)(sizeof(float)*FDATASIZE) ))
           {
            return( 1 );
           }
       }

    return( 0 );
}

/***/
/* ser2fid2D: converts a series of vectors from bruker to pipe format.
/***/

int ser2fid2D( bData, rData,
               xSize, ySize, quadState, quadType,
               wordSize, swapFlag, i2fFlag, badThresh, verbose )

   int   xSize, ySize, quadState, quadType, wordSize, swapFlag, i2fFlag, verbose;
   float *rData, badThresh;
   char  *bData;
{
    float *rPtr, *iPtr;
    int   length;

    length = xSize*ySize*quadState;

    rPtr = rData;
    iPtr = rData + xSize;
   
    switch( quadType )
       {
        case 0:
           while( ySize-- )
              {
               if (wordSize == 4)
                  {
                   (void) ser2quad4( bData, rPtr, iPtr, xSize, swapFlag, i2fFlag );
                  }
               else if (wordSize == 8)
                  {
                   (void) ser2quad8( bData, rPtr, iPtr, xSize, swapFlag, i2fFlag );
                  }
               else
                  {
                   (void) ser2quad3( bData, rPtr, iPtr, xSize, swapFlag );
                  }

               bData += wordSize*xSize*quadState;
               rPtr  += xSize*quadState;
               iPtr  += xSize*quadState;
              }
           break;

        default: 
           while( ySize-- )
              {
               if (wordSize == 4)
                  {
                   (void) ser2real4( bData, rPtr, xSize, swapFlag, i2fFlag );
                  }
               else if (wordSize == 8)
                  {
                   (void) ser2real8( bData, rPtr, xSize, swapFlag, i2fFlag );
                  }
               else
                  {
                   (void) ser2real3( bData, rPtr, xSize, swapFlag );
                  }

               bData += wordSize*xSize;
               rPtr  += xSize;
              }
           break;
       }

/***/
/* If required, clip points above the bad point threshold.
/***/

    if (badThresh != 0.0)
       {
        (void) badClip( rData, badThresh, length, verbose );
       }

    return( 0 );
}

/***/
/* dmx2fid2D: perform DMX conversion on a matrix of complex vectors.
/***/

int dmx2fid2D( dataMat, xSize, ySize )

   float *dataMat;
   int   xSize, ySize;
{
    float *rdata, *idata;

    rdata = dataMat;
    idata = rdata + xSize;

    while( ySize-- )
       {
        (void) dmx( rdata, idata );

        rdata += 2*xSize;
        idata += 2*xSize;
       }

    return( 0 );
}

/***/
/* xExt2D extract valid part of X-Axis in the given 2D matrix.
/***/

int xExt2D( dataMat, xSize, xExtSize, ySize )

    float *dataMat;
    int   xSize, xExtSize, ySize;
{
    float *src, *dest;
    int   ix, iy;

    dest = dataMat;

    for( iy = 0; iy < ySize; iy++ )
       {
        src = dataMat;

        for( ix = 0; ix < xExtSize; ix++ )
           {
            *dest++ = *src++;
           }

        dataMat += xSize; 
       }

    return( 0 );
}

/***/
/* badClip: zero values with a magnitude greater than the bad point threshold.
/***/

int badClip( rdata, badThresh, length, verbose )

   float *rdata, badThresh;
   int   length, verbose;
{
    static int count = 0;

    while( length-- )
       {
        if (*rdata > 0.0)
           {
            if (*rdata > badThresh)
               {
                if (verbose)
                   {
                    FPR( stderr,
                         "Bruk2Pipe Notice: Bad Point %d: % f\n",
                         count + 1, *rdata );
                   }

                *rdata = 0.0;
                count++;
               }
           }
        else
           {
            if (*rdata < -badThresh)
               {
                if (verbose)
                   {
                    FPR( stderr, "Bruk2Pipe Notice: Bad Point %d: % f\n", count + 1, *rdata );
                   }

                *rdata = 0.0;
                count++;
               }
           }

        rdata++;
       }

    return( 0 );
} 

/***/
/* ser2quad8: convert 8-byte complex bruker data to complex pipe data.
/***/

int ser2quad8( bdata, rdata, idata, xSize, swapFlag, i2fFlag )

   float *rdata, *idata;
   int   xSize, swapFlag, i2fFlag;
   char  *bdata;
{
#ifdef NMR64
    union r2i { long i; char s[sizeof(double)]; double f; } out;
#else
    union r2i { long long i; char s[sizeof(double)]; double f; } out;
#endif

    char  *c1, *c2, *c3, *c4, *c5, *c6, *c7, *c8;

/***/
/* Bruker AMX data are alternating R/I words.
/* Each word is an integer with bytes reversed relative to Sparc order.
/***/

    if (swapFlag)
       {
        c1 = &out.s[0];
        c2 = &out.s[1];
        c3 = &out.s[2];
        c4 = &out.s[3];
        c5 = &out.s[4];
        c6 = &out.s[5];
        c7 = &out.s[6];
        c8 = &out.s[7];
       }
    else
       {
        c8 = &out.s[0];
        c7 = &out.s[1];
        c6 = &out.s[2];
        c5 = &out.s[3];
        c4 = &out.s[4];
        c3 = &out.s[5];
        c2 = &out.s[6];
        c1 = &out.s[7];
       }

    while( xSize-- )
       {
#ifdef CRAY
        out.i = 0;
#endif
        *c8   = *bdata++;
        *c7   = *bdata++;
        *c6   = *bdata++;
        *c5   = *bdata++;
        *c4   = *bdata++;
        *c3   = *bdata++;
        *c2   = *bdata++;
        *c1   = *bdata++;

#ifdef CRAY
        *rdata = (float)(out.i/ISCALE);
#else
        *rdata = i2fFlag ? (float)out.i : (float)out.f;
#endif

#ifdef CRAY
        out.i = 0;
#endif
        *c8   = *bdata++;
        *c7   = *bdata++;
        *c6   = *bdata++;
        *c5   = *bdata++;
        *c4   = *bdata++;
        *c3   = *bdata++;
        *c2   = *bdata++;
        *c1   = *bdata++;

#ifdef CRAY
        *idata = (float)(out.i/ISCALE);
#else
        *idata = i2fFlag ? (float)out.i : (float)out.f;
#endif
 
        rdata++; idata++;
       }

    return( 0 );
}

/***/
/* ser2quad4: convert 4-byte complex bruker data to complex pipe data.
/*            this version adjusted for slight speed increase.
/***/

int ser2quad4( bdata, rdata, idata, xSize, swapFlag, i2fFlag )

   float *rdata, *idata;
   int   xSize, swapFlag, i2fFlag;
   char  *bdata;
{
    union r2i { int i; char s[sizeof(int)]; float f; } out;
    char  *c1, *c2, *c3, *c4;

/***/
/* Bruker AMX data are alternating R/I words.
/* Each word is an integer with bytes reversed relative to Sparc order.
/***/

    if (swapFlag)
       {
        c1 = &out.s[0];
        c2 = &out.s[1];
        c3 = &out.s[2];
        c4 = &out.s[3];
       }
    else
       {
        c4 = &out.s[0];
        c3 = &out.s[1];
        c2 = &out.s[2];
        c1 = &out.s[3];
       }

    while( xSize-- )
       {
#ifdef CRAY
        out.i = 0;
#endif
        *c4   = *bdata++;
        *c3   = *bdata++;
        *c2   = *bdata++;
        *c1   = *bdata++;

#ifdef CRAY
        *rdata = (float)(out.i/ISCALE);
#else
        *rdata = i2fFlag ? (float)out.i : out.f;
#endif

#ifdef CRAY
        out.i = 0;
#endif
        *c4   = *bdata++;
        *c3   = *bdata++;
        *c2   = *bdata++;
        *c1   = *bdata++;

#ifdef CRAY
        *idata = (float)(out.i/ISCALE);
#else
        *idata = i2fFlag ? (float)out.i : out.f;
#endif
 
        rdata++; idata++;
       }

    return( 0 );
}

/***/
/* ser2quad3: convert 3-byte complex bruker data to complex pipe data.
/***/

int ser2quad3( bdata, rdata, idata, xSize, swapFlag )

   float *rdata, *idata;
   int   xSize, swapFlag;
   char  *bdata;
{
    union r2i { int i; char s[sizeof(int)]; } out;
    char  *c1, *c2, *c3, *c4;

    if (swapFlag)
       {
        c1 = &out.s[1];
        c2 = &out.s[2];
        c3 = &out.s[3];
        c4 = &out.s[0];
       }
    else
       {
        c1 = &out.s[3];
        c2 = &out.s[2];
        c3 = &out.s[1];
        c4 = &out.s[0];
       }

/***/
/* Move 3-byte int into upper 3 bytes of 4-byte int.
/* Divide by 256 to reduce back to 3 bytes while retaining sign.
/***/

    while( xSize-- )
       {
#ifdef CRAY
        out.i = 0;
#endif
        *c4   = (char) 0;
        *c3   = *bdata++;
        *c2   = *bdata++;
        *c1   = *bdata++;

#ifdef CRAY
        *rdata   = (float)(out.i/ISCALE256);
#else
        *rdata   = (float)(out.i/256);
#endif

#ifdef CRAY
        out.i = 0;
#endif

        if (swapFlag)
           {
            *c4   = (char) 0;
            *c3   = *bdata++;
            *c2   = *bdata++;
            *c1   = *bdata++;
           }
        else
           {
            *c4   = *bdata++;
            *c3   = *bdata++;
            *c2   = *bdata++;
            *c1   = (char) 0;
           }

#ifdef CRAY
        *idata   = (float)(out.i/ISCALE256);
#else
        *idata   = (float)(out.i/256);
#endif

        rdata++; idata++;
       }

    return( 0 );
}

/***/
/* ser2real8: convert 8-byte real bruker data to real pipe data.
/***/

int ser2real8( bdata, rdata, xSize, swapFlag, i2fFlag )

   float *rdata;
   int   xSize, swapFlag, i2fFlag;
   char  *bdata;
{
#ifdef NMR64
    union r2i { long i; char s[sizeof(double)]; double f; } out;
#else
    union r2i { long long i; char s[sizeof(double)]; double f; } out;
#endif

    char  *c1, *c2, *c3, *c4, *c5, *c6, *c7, *c8;

    if (swapFlag)
       {
        c1 = &out.s[0];
        c2 = &out.s[1];
        c3 = &out.s[2];
        c4 = &out.s[3];
        c5 = &out.s[4];
        c6 = &out.s[5];
        c7 = &out.s[6];
        c8 = &out.s[7];
       }
    else
       {
        c8 = &out.s[0];
        c7 = &out.s[1];
        c6 = &out.s[2];
        c5 = &out.s[3];
        c4 = &out.s[4];
        c3 = &out.s[5];
        c2 = &out.s[6];
        c1 = &out.s[7];
       }

    while( xSize-- )
       {
#ifdef CRAY
        out.i = 0;
#endif
        *c8   = *bdata++;
        *c7   = *bdata++;
        *c6   = *bdata++;
        *c5   = *bdata++;
        *c4   = *bdata++;
        *c3   = *bdata++;
        *c2   = *bdata++;
        *c1   = *bdata++;

#ifdef CRAY
        *rdata = (float)(out.i/ISCALE);
#else
        *rdata = i2fFlag ? (float)out.i : (float)out.f;
#endif

        rdata++;
       }

    return( 0 );
}

/***/
/* ser2real4: convert 4-byte real bruker data to real pipe data.
/***/

int ser2real4( bdata, rdata, xSize, swapFlag, i2fFlag )

   float *rdata;
   int   xSize, swapFlag, i2fFlag;
   char  *bdata;
{
    union r2i { int i; char s[sizeof(int)]; float f; } out;
    char  *c1, *c2, *c3, *c4;

    if (swapFlag)
       {
        c1 = &out.s[0];
        c2 = &out.s[1];
        c3 = &out.s[2];
        c4 = &out.s[3];
       }
    else
       {
        c4 = &out.s[0];
        c3 = &out.s[1];
        c2 = &out.s[2];
        c1 = &out.s[3];
       }

    while( xSize-- )
       {
#ifdef CRAY
        out.i = 0;
#endif
        *c4   = *bdata++;
        *c3   = *bdata++;
        *c2   = *bdata++;
        *c1   = *bdata++;

#ifdef CRAY
        *rdata = (float)(out.i/ISCALE);
#else
        *rdata = i2fFlag ? (float)out.i : out.f;
#endif

        rdata++;
       }

    return( 0 );
}

/***/
/* ser2real3: convert 3-byte real bruker data to real pipe data.
/***/

int ser2real3( bdata, rdata, xSize, swapFlag )

   float *rdata;
   int   xSize, swapFlag;
   char  *bdata;
{
    union r2i { int i; char s[sizeof(int)]; } out;
    char  *c1, *c2, *c3, *c4;

    if (swapFlag)
       {
        c1 = &out.s[1];
        c2 = &out.s[2];
        c3 = &out.s[3];
        c4 = &out.s[0];
       }
    else
       {
        c1 = &out.s[3];
        c2 = &out.s[2];
        c3 = &out.s[1];
        c4 = &out.s[0];
       }

/***/
/* Move 3-byte int into upper 3 bytes of 4-byte int.
/* Divide by 256 to reduce back to 3 bytes while retaining sign.
/***/

    while( xSize-- )
       {
#ifdef CRAY
        out.i = 0;
#endif

        if (swapFlag)
           {
            *c4   = (char) 0;
            *c3   = *bdata++;
            *c2   = *bdata++;
            *c1   = *bdata++;
           }
        else 
           {
            *c4   = *bdata++;
            *c3   = *bdata++;
            *c2   = *bdata++;
            *c1   = (char) 0;
           }

#ifdef CRAY
        *rdata   = (float)(out.i/ISCALE256);
#else
        *rdata   = (float)(out.i/256);
#endif

        rdata++;
       }

    return( 0 );
}

/***/
/* getparams: extract command arguments for bruk2pipe.
/***/

static int getparams( argc, argv,
                      inName, outFmt, serverListName, fdata, megabytes, badThresh, fc,
                      byteOffset, firstPlane, lastPlane, brukType, wordSize, i2fFlag, xyzFlag, checkFlag,
                      ovFlag, swapFlag, pipeFlag, extFlag, dirFlag, decim, dspfvs, grpdly, 
                      aqmod, skipSize, zfFlag, verbose )

   int   argc, *megabytes, *checkFlag, *brukType, *firstPlane, *lastPlane,
         *byteOffset, *wordSize, *i2fFlag, *xyzFlag, *ovFlag, *swapFlag, *pipeFlag, *extFlag,
         *dirFlag, *decim, *dspfvs, *aqmod, *skipSize, *zfFlag, 
         *verbose;

   float fdata[FDATASIZE], *badThresh, *fc, *grpdly;
   char  **argv, *inName, *outFmt, *serverListName;
{

/***/
/* Initialize:
/***/

#ifdef SYS_BYTESWAP
    *swapFlag       = 0;
#else
    *swapFlag       = 1;
#endif

    *brukType       = BRUK_AMX;

    *badThresh      = 8000000.0; 
    *checkFlag      = 1;
    *dirFlag        = 1;
    *firstPlane     = 0;
    *lastPlane      = 0;
    *byteOffset     = 0;
    *verbose        = 0;
    *i2fFlag        = 1;
    *xyzFlag        = 0;
    *ovFlag         = 0;
    *pipeFlag       = 1;
    *extFlag        = 0;
    *zfFlag         = 1;
    *skipSize       = 4;
    *decim          = 0;
    *dspfvs         = 10;
    *grpdly         = 0.0;
    *aqmod          = AQ_MOD_QSIM;
    *fc             = 1.0;

    *serverListName = '\0';
    *inName         = '\0';
    *outFmt         = '\0';

    if (argc == 1 || flagLoc( argc, argv, "-help" ))
       {
        FPR_VERSION;
        FPR( stderr, "Bruk2Pipe: Bruker to NMRPipe Conversion.\n" );
        FPR( stderr, "%s hdrFlags < brukSER > pipeFile\n", argv[0] );
        FPR( stderr, "General Flags:\n" );
        FPR( stderr, " -in  inName       Bruker SER Input.\n" );
        FPR( stderr, " -out outTemplate  NMRPipe Output File or Template.\n" );
        FPR( stderr, " -mem memMB  [0]   MByte Memory Buffer (Use 0 for Auto).\n" );
        FPR( stderr, " -ov               Force Overwrite.\n" );
        FPR( stderr, " -noext            No Extract Performed (Default).\n" );
        FPR( stderr, " -ext              Extract Valid Points Only.\n" );
        FPR( stderr, " -nodir            Suppress Directory Creation.\n" );
        FPR( stderr, " -nofs             Suppress File Size Check.\n" );
        FPR( stderr, " -nocheckArgs      Suppress Unused/Redundant Argument Checking.\n" );
        FPR( stderr, " -verb             Verbose Mode On.\n" );
        FPR( stderr, "Bruker Format Flags (Use Only One):\n" );
        FPR( stderr, " -AMX              Input is AMX/DMX Format (Default).\n" );
        FPR( stderr, " -DMX              Input is DMX Format, Adjust for Digital Oversampling.\n" );
        FPR( stderr, " -AM               Input is 3-Byte Bruker AM Data.\n" );
        FPR( stderr, "Data Type Options:\n" );
        FPR( stderr, " -i2f              Perform Int-to-Float Conversion (Default).\n" );
        FPR( stderr, " -noi2f            No Int-to-Float Conversion (Not for 3-Byte Data).\n" );
        FPR( stderr, " -ws    wSize [4]  Input Word Size (Default is 3 for -AM Data).\n" );
        FPR( stderr, " -bo    bOff  [0]  Byte Offset for Input; Commonly 768 to skip AM header.\n" );
        FPR( stderr, " -bad   bPt [8e6]  Bad Point Threshold (Use 0.0 for None)\n" );
        FPR( stderr, "Byte-Swap Options:\n" );
#ifdef SYS_BYTESWAP
        FPR( stderr, " -swap             Do Four-Byte Swap.\n" );
        FPR( stderr, " -noswap           Suppress Four-Byte Swap (Default).\n");
        FPR( stderr, " -aswap            Suppress Four-Byte Swap.\n");
        FPR( stderr, "                   (Meaning is System-dependant)\n" );
        FPR( stderr, " -noaswap          Do Four-Byte Swap. (Meaning is System-dependant).\n" );
#else
        FPR( stderr, " -swap             Do Four-Byte Swap (Default).\n" );
        FPR( stderr, " -noswap           Suppress Four-Byte Swap.\n");
        FPR( stderr, " -aswap            Do Four-Byte Swap. (Meaning is System-dependant).\n" );
        FPR( stderr, " -noaswap          Suppress Four-Byte Swap. (Meaning is System-dependant).\n" );
#endif
        FPR( stderr, "Bruker Digital Filter Parameters from 'acqus':\n" );
        FPR( stderr, " -grpdly gV [0.0]  GRPDLY Value.\n" );
        FPR( stderr, " -decim  dV [0]    DECIM Value.\n" );
        FPR( stderr, " -dspfvs fV [10]   DSPFVS Value, 10-13,20-23 (Default = 11 for DQD mode).\n" );
        FPR( stderr, " -c      cV [1.0]  First/Last Point Scale.\n" );
        FPR( stderr, " -skip   sV [4]    Points to Skip in Head Correction (Use -1 for No Correction).\n" );
        FPR( stderr, " -zf               Temporary Zero Fill for Speed (Default).\n" );
        FPR( stderr, " -nozf             No Temporary Zero Fill.\n" );
/*
        FPR( stderr, "Parallel Processing:\n" );
        FPR( stderr, " -par   sListName   Server List File Name.\n" );
        FPR( stderr, " -first firstPlane  First Plane to process.\n");
        FPR( stderr, " -last  lastPlane   Last Plane to process.\n");
*/

        (void) parseHdr( argc, argv, fdata );

        FPR( stderr, "\n" );
        FPR( stderr, "Notes:\n" );
        FPR( stderr, " 1. Digital oversampling data requires -grpdly -decim and -dspfvs options.\n" );
        FPR( stderr, " 2. The -DMX flag will perform an adjustment for digital oversampling\n" );
        FPR( stderr, "    during conversion. It will reduce the size of the directly detected\n" );
        FPR( stderr, "    dimension by about ~70 points. It implies the -ext flag.\n" );
        FPR( stderr, " 3. When used on digital oversampling data, the -AMX flag will cause\n" );
        FPR( stderr, "    oversampling correction to be applied during processing instead of\n" );
        FPR( stderr, "    during conversion. This often gives better baselines.\n" ); 
        FPR( stderr, " 4. Output names for 3D data can be either single\n" );
        FPR( stderr, "    files (such as test.fid) or 2D series templates\n" );
        FPR( stderr, "    (such as fid/test%%03d.fid).\n" );
        FPR( stderr, " 5. Output names for 4D data can be either single\n" );
        FPR( stderr, "    files (such as test.fid) or 3D cube templates\n" );
        FPR( stderr, "    (such as fid/test%%03d.fid) or 2D series templates\n" );
        FPR( stderr, "    (such as fid/test%%03d%%03d.fid).\n" );
        FPR( stderr, "\n" );
        return( 1 );
       }

/***/
/* Extract command-specific parameters:
/***/

    if (flagLoc( argc, argv, "-AMX" ))  *brukType  = BRUK_AMX;
    if (flagLoc( argc, argv, "-DMX" ))  *brukType  = BRUK_DMX;
    if (flagLoc( argc, argv, "-AM" ))   *brukType  = BRUK_AM;

    if (flagLoc( argc, argv, "-i2f" ))    *i2fFlag   = 1;
    if (flagLoc( argc, argv, "-noi2f" ))  *i2fFlag   = 0;

    if (flagLoc( argc, argv, "-i2r" ))    *i2fFlag   = 1;
    if (flagLoc( argc, argv, "-noi2r" ))  *i2fFlag   = 0;

    if (*brukType == BRUK_AM) *wordSize = 3;

    (void) intArgD( argc, argv, "-ws",  wordSize );

#ifdef NO_DMX_SRC
    if (*brukType == BRUK_DMX)
       {
        FPR( stderr, "Bruk2Pipe Warning: no DMX conversion supported.\n" );
       }
#else
    if (*brukType == BRUK_DMX)
       {
        if (!flagLoc( argc, argv, "-decim" ))
           {
            FPR( stderr, "Bruk2Pipe Error: missing DMX -decim value.\n" );
            return( 1 );
           }

        *extFlag  = 1;
       }
#endif

    (void) strArgD( argc, argv, "-in",    inName );
    (void) strArgD( argc, argv, "-out",   outFmt );
    (void) strArgD( argc, argv, "-par",   serverListName );
    (void) intArgD( argc, argv, "-first", firstPlane );
    (void) intArgD( argc, argv, "-last",  lastPlane );

/***/
/* Redundant extraction of DSP aq mode for back-compatibility:
/***/

    (void) intStrArgD( argc, argv, "-aq_mod", aqmod, dspList );
    (void) intStrArgD( argc, argv, "-xMODE",  aqmod, dspList );

    *dspfvs = *aqmod == AQ_MOD_DQD ? 11 : 10;

    (void) fltArgD( argc, argv, "-grpdly",  grpdly );
    (void) intArgD( argc, argv, "-decim",   decim );
    (void) intArgD( argc, argv, "-dspfvs",  dspfvs );
    (void) intArgD( argc, argv, "-skip",    skipSize );

    if (flagLoc( argc, argv, "-zf" ))     *zfFlag    = 1;
    if (flagLoc( argc, argv, "-nozf" ))   *zfFlag    = 0;

#ifdef SYS_BYTESWAP
    if (flagLoc( argc, argv, "-aswap" ))   *swapFlag     = 0;
    if (flagLoc( argc, argv, "-noaswap" )) *swapFlag     = 1;
#else
    if (flagLoc( argc, argv, "-aswap" ))   *swapFlag     = 1;
    if (flagLoc( argc, argv, "-noaswap" )) *swapFlag     = 0;
#endif

    if (flagLoc( argc, argv, "-swap" ))    *swapFlag     = 1;
    if (flagLoc( argc, argv, "-noswap" ))  *swapFlag     = 0;

    if (flagLoc( argc, argv, "-in" ))     *pipeFlag  = 0;
    if (flagLoc( argc, argv, "-noext" ))  *extFlag   = 0;
    if (flagLoc( argc, argv, "-ext" ))    *extFlag   = 1;
    if (flagLoc( argc, argv, "-nodir" ))  *dirFlag   = 0;
    if (flagLoc( argc, argv, "-nofs" ))   *checkFlag = 0;
    if (flagLoc( argc, argv, "-ov" ))     *ovFlag    = 1;
    if (flagLoc( argc, argv, "-verb" ))   *verbose   = 1;

    (void) intArgD( argc, argv, "-bo",    byteOffset );
    (void) fltArgD( argc, argv, "-bad",   badThresh );
    (void) fltArgD( argc, argv, "-c",     fc );

    if (*badThresh < 0.0) *badThresh = -*badThresh;

/***/
/* Test for required output format with xyz style output:
/***/

    if (isZFmt( outFmt ) || isAZFmt( outFmt )) *xyzFlag = 1;

    if (*xyzFlag && !*outFmt)
       {
        FPR( stderr, "Bruk2Pipe Error: missing output format.\n" );
        FPR( stderr, " 2D Series Output requires -out template, e.g.:\n" );
        FPR( stderr, " %s -out test%%03d.fid < SER\n", argv[0] );
        return( 1 );
       }

/***/
/* Set default megabytes depending on output mode:
/***/

    if (*xyzFlag)
       *megabytes = 0;
    else
       *megabytes = 4;

    (void) intArgD( argc, argv, "-mem", megabytes );

/***/
/* Set some default parameters:
/***/

    (void) txt2Flt( "X", fdata + FDF2LABEL, SIZE_F2LABEL );
    (void) txt2Flt( "Y", fdata + FDF1LABEL, SIZE_F1LABEL );
    (void) txt2Flt( "Z", fdata + FDF3LABEL, SIZE_F3LABEL );
    (void) txt2Flt( "A", fdata + FDF4LABEL, SIZE_F4LABEL );

/***/
/* Parse header parameters:
/***/

    if (parseHdr( argc, argv, fdata )) return( 1 );

    return( 0 );
}

#ifdef NO_DMX_SRC

int getDMXVal( decim, dspfvs, grpdly, val )

   int   decim, dspfvs; 
   float grpdly, *val;
{
   *val = 0.0;

   return( 0 );
}

int dmxNull()
{
   return( 0 );
}

int dmxInit( inSize, validSize, zfFlag, decim, dspfvs, grpdly, outSize )

   int   inSize, validSize, zfFlag, decim, dspfvs, *outSize;
   float grpdly;
{
   *outSize = inSize;
   return( 0 );
}

int dmx( rdata, idata )

   float *rdata, *idata;
{
   return( 0 );
}

int dmxFree()
{
   return( 0 );
}

#endif

/***/
/* Bottom.
/***/
