/***/
/*  NMRPipe Copyright 1992-2018 Frank Delaglio 
/***/

/***/
/* pipe2xyz: converts nmrPipe stream of vectors back to an XYZ file list;
/*           option to output in NMRView format is also provided.
/***/

#define ETEST_PROG_NAME "Pipe2XYZ"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "version.h"
#include "cmndargs.h"
#include "testsize.h"
#include "minmax.h"
#include "memory.h"
#include "dataio.h"
#include "inquire.h"
#include "fdatap.h"
#include "vutil.h"
#include "etest.h"
#include "prec.h"
#include "atof.h"

#include "nvstuff.h"

#define FMT_NMRPIPE 1
#define FMT_NMRVIEW 2

#define VERBOSE_NO  0
#define VERBOSE_YES 1

#define FPR (void)fprintf

static int truncFlag  = 1;
static int creatFlag  = 1;
static int shufFlag   = 0;
static int oldFmtFlag = 0;

static int autoFlag  = 1;              /* Auto buffer size determination.    */
static int maxTries  = -1;             /* Pseudo-blocking I/O repeat count.  */
static int delayTime = DATAIO_TIMEOUT; /* Pseudo-blocking I/O delay, uSec.   */

static int getparams(), pipe2xyz(), pipeHead(), pipe2xyzFD();
static int x2xyz(), x2xyzN(), y2xyz(), y2xyzN(), z2xyz(), z2xyzN(), x2nv(), z2aListN();

static int c2riList2N(), c2riListN(), xy2zaShufN(), xy2zaShuf();

static int debugFlag = 0;
static int verbose   = 0;
static int vInc      = 16;

int main( argc, argv )

   int  argc;
   char **argv;

{
    int    xSize, ySize, zSize, aSize, /* Sizes of input stream data axes.   */
           quadState,                  /* Indicates X or Y Complex planes.   */
           xQuadState,                 /* Indicates X-Complex planes.        */
           yQuadState,                 /* Indicates Y-Complex planes.        */
           zQuadState,                 /* Indicates Z-Complex planes.        */
           aQuadState,                 /* Indicates A-Complex planes.        */
           dimCount,                   /* Number of dimensions in data.      */
           fmtFlag,                    /* Destination Format, Pipe or View.  */
           xyzFlag;                    /* Destination Axis: 1, 2 etc         */

    int   inPlace, firstPlane, lastPlane, maxPlane, maxPlanesIn, megabytes, multiFile, 
          fmtCount, nX, nY, nZ, nA, nQ, nDim, checkFlag, dirFlag, error;

    char  *outName, outTemplate[NAMELEN+1], inName[NAMELEN+1], dirName[NAMELEN+1];

    float fdata[FDATASIZE];
    UNIT  inUnit;

    struct ZListInfo **outListN;
 
    void  freeText();

/***/
/* Initialize:
/***/

    error       = 0;
    verbose     = 0;
    outListN    = (struct ZListInfo **)NULL;
    outName     = (char *)NULL;
    inUnit      = UNIT_NULL;
    autoFlag    = 1;
    maxTries    = -1;
    delayTime   = DATAIO_TIMEOUT;
    megabytes   = 0;
    maxPlanesIn = 0;
    nZ          = 0;
    nA          = 0;
    nDim        = 0;
    fmtCount    = 0;

    (void) initArgs( argc, argv, CHECK_ENV );
    (void) initDataIO();


/***/
/* Get command arguments.
/* Interpret file parameters from 2D input stream; silent abort on NULL header. 
/* Read or create Z-plane output list if needed.
/* Parse User suggestions for output header if any.
/* Perform final check or argument usage.
/***/

    error = getparams( argc, argv,
                       outTemplate, inName,
                       &firstPlane, &lastPlane, &xyzFlag, &fmtFlag,
                       &inPlace, &maxPlanesIn, &megabytes, &checkFlag, &dirFlag );

    if (error)
       {
        FPR( stderr, "Pipe2XYZ Error extracting arguments.\n" );
        error = 1;
        goto shutdown;
       }

    fmtCount = getFmtCount( outTemplate );

    error = pipe2xyzFD( inName, &inUnit, &dimCount,
                        &xQuadState, &yQuadState, &zQuadState, &aQuadState,
                        &quadState, &xSize, &ySize, &zSize, &aSize,
                        &firstPlane, &lastPlane, &maxPlane, &megabytes,
                        inPlace, xyzFlag, fmtFlag, fmtCount, fdata );

    if (error < 0)
       {
        error = 0;
        goto shutdown;
       }
    else if (error > 0)
       {
        FPR( stderr, "Pipe2XYZ Error setting file parameters.\n" );
        goto shutdown;
       }

    if (flagLoc( argc, argv, "-shuf" ))   shufFlag = 1;
    if (flagLoc( argc, argv, "-noshuf" )) shufFlag = 0;

    nDim = getParmI( fdata, FDDIMCOUNT, NULL_DIM );
    nX   = getParmI( fdata, NDSIZE,     CUR_XDIM );
    nY   = getParmI( fdata, NDSIZE,     CUR_YDIM );
    nZ   = getParmI( fdata, NDSIZE,     CUR_ZDIM );
    nA   = getParmI( fdata, NDSIZE,     CUR_ADIM );

    nQ   = getQuad( fdata, FDQUADFLAG, NULL_DIM );

    if (fmtFlag == FMT_NMRPIPE)
       { 
        if ((error = getZListN( outTemplate, nZ, nA, nDim, &outListN )))
           {
            FPR( stderr, "Pipe2XYZ Error getting file list;\n" );
            FPR( stderr, "Check for valid output name or template.\n" );
            goto shutdown;
           }

        outName   = (outListN[0])->name;
        multiFile = 1;
       }
    else
       {
        outName   = outTemplate;
        multiFile = 0;
       }

    if ((error = pipeHead( argc, argv, fdata )))
       {
        FPR( stderr, "Pipe2XYZ Error setting user header.\n" );
        goto shutdown;
       }

    (void) checkArgs( argc, argv );

/***/
/* Check for existance of output directory.
/* Check for sufficient output space.
/* Create the output stack.
/***/

    if ((fmtFlag == FMT_NMRPIPE) && dirFlag)
       {
        if (!dirExists( outName, dirName ))
           {
            if (verbose)
               {
                FPR( stderr,
                     "Pipe2XYZ Notice: Creating directory %s\n",
                     dirName );
               }

            if (dirCreate( dirName ))
               {
                FPR( stderr,
                     "Pipe2XYZ Warning: Error creating directory %s\n",
                     dirName );
               }
           }
       }

    if (checkFlag && firstPlane == 1 && lastPlane == maxPlane)
       {
        if (!fileExists( outName ))
           {
            if ((error = dataCreate( outName )))
               {
                FPR( stderr, "Pipe2XYZ Error creating test file " );
                FPR( stderr, " %s\n", outName );
                goto shutdown;
               }
           }

        switch( testNMRout( outName, fdata, multiFile, VERBOSE_YES ))
            {
             case FILE_SIZE_FAIL:
                FPR( stderr, "\n" );
                FPR( stderr, "Pipe2XYZ Warning:" );
                FPR( stderr, " can't check size for file" );
                FPR( stderr, " %s\n", outName );
                break;
             case  FILE_SIZE_TOO_LARGE:
                FPR( stderr, "\n" );
                FPR( stderr, "Pipe2XYZ Error:" );
                FPR( stderr, " insufficient space for file" );
                FPR( stderr, " %s\n", outName );
                error = 1;
                goto shutdown;
                break;
             default:
                break;
            }
       }

    if (fmtFlag == FMT_NMRVIEW)
       {
        error = x2nv( inUnit, outName,
                      xSize, ySize, zSize, aSize, dimCount,
                      xQuadState, yQuadState, zQuadState, aQuadState,
                      firstPlane, lastPlane, fdata, megabytes,
                      inPlace );
       }
    else
       {
        error = pipe2xyz( inUnit, 
                          outTemplate, outListN, 
                          fmtCount, dimCount, xyzFlag, 
                          xSize, ySize, zSize, aSize, nX, nY, nZ, nA, nQ,
                          xQuadState, yQuadState, zQuadState, aQuadState,
                          quadState, firstPlane, lastPlane, maxPlanesIn, megabytes,
                          fdata, inPlace );
      }

    if (error)
       {
        FPR( stderr, "Pipe2XYZ Error building stream.\n" );
        goto shutdown;
       }

/***/
/* Exit point:
/***/

shutdown:

    if (error) FPR( stderr, "Pipe2XYZ Status: %d\n", error );

    (void) dataClose( inUnit );
    (void) freeZListN( nZ, nA, nDim, outListN );
    (void) showDebugIO( argc, argv );

    return( error );
}

/***/
/* pipe2xyz: write a vector stream to the specified dimension of XYZ data.
/***/

static int pipe2xyz( inUnit, 
                     outTemplate, outListN, 
                     fmtCount, dimCount, xyzFlag, 
                     xSize, ySize, zSize, aSize, nX, nY, nZ, nA, nQ,
                     xQuadState, yQuadState, zQuadState, aQuadState, quadState, 
                     firstPlane, lastPlane, maxPlanesIn, megabytes, fdata, inPlace, verb )

   int   fmtCount, dimCount, xyzFlag, xSize, ySize, zSize, aSize, nX, nY, nZ, nA, nQ,
         xQuadState, yQuadState, zQuadState, aQuadState, quadState, firstPlane, lastPlane,
         maxPlanesIn, megabytes, inPlace, verb;

   char   *outTemplate;
   float  fdata[FDATASIZE];
   struct ZListInfo **outListN;
   UNIT   inUnit;
{
    int    error;

    error = 0;

    switch( xyzFlag )
       {
        case 1:

           if (oldFmtFlag)
              {
               error = x2xyz( inUnit, outListN,
                              dimCount, fmtCount,
                              xSize, ySize, zSize, aSize, xQuadState, yQuadState,
                              firstPlane, lastPlane, fdata, megabytes,
                              inPlace, verb );
              }
           else
              {
               error = x2xyzN( inUnit, outListN,
                               dimCount, fmtCount, 
                               xSize, ySize, zSize, aSize,
                               firstPlane, lastPlane, fdata, inPlace, verb );
              }

           break;

        case 2:

           if (oldFmtFlag)
              {
               error = y2xyz( inUnit, outListN,
                              dimCount, fmtCount,
                              xSize, ySize, zSize, aSize, xQuadState, yQuadState,
                              firstPlane, lastPlane, fdata, inPlace, verb );
              }
           else
              {
               error = y2xyzN( inUnit, outListN,
                               dimCount, fmtCount, 
                               xSize, ySize, zSize, aSize, xQuadState, yQuadState,
                               firstPlane, lastPlane, fdata, inPlace, verb );
              }

           break;

        case 3: case 4:

           if (oldFmtFlag)
              {
               error = z2xyz( inUnit, outListN,
                              dimCount, fmtCount, xSize, ySize, zSize, aSize, 
                              quadState, xQuadState, yQuadState, zQuadState, aQuadState,
                              firstPlane, lastPlane, fdata, megabytes, xyzFlag, inPlace, verb );
              }
           else
              {
               error = z2xyzN( inUnit,
                               outTemplate, outListN,
                               dimCount, fmtCount, xSize, ySize, zSize, aSize, 
                               nX, nY, nZ, nA, nQ, quadState, xQuadState, yQuadState, zQuadState, aQuadState,
                               firstPlane, lastPlane, maxPlanesIn, fdata, xyzFlag, inPlace, verb );
              }

           break;

        default:
           error = 1;
           break;
       }

    return( error );
}

/***/
/* x2xyz: write an XYZ-vector stream in XYZ order.
/***/

static int x2xyz( inUnit, outListN,
                  dimCount, fmtCount, 
                  xSize, ySize, zSize, aSize, xQuadState, yQuadState,
                  firstPlane, lastPlane, fdata, megabytes,
                  inPlace )

   int   dimCount, fmtCount, xSize, ySize, zSize, aSize, xQuadState, yQuadState,
         firstPlane, lastPlane, megabytes, inPlace;

   float  fdata[FDATASIZE];
   struct ZListInfo **outListN;
   UNIT   inUnit;
{
    NMR_INT maxSlices, slicesIn, slicesLeft, ptsIn, length2d;
    int     ip, ipp, error;
    float   *matrix2d;
    UNIT    outUnit;

/***/
/* Initialize:
/***/

    error     = 0;
    matrix2d  = (float *) NULL;
    outUnit   = UNIT_NULL;

/***/
/* Compute sizes, allocate memory.
/***/

    maxSlices  = (NMR_INT)megabytes*1024*1024/(xSize*sizeof(float));
    maxSlices  = min( ySize, maxSlices );
    length2d   = (NMR_INT)maxSlices*(NMR_INT)xSize;

    if (maxSlices < 1)
       {
        if (autoFlag)
           {
            maxSlices = ySize;
            length2d  = maxSlices*xSize;
            megabytes = 2 + (length2d*sizeof(float))/(1024*1024);
           }
        else
           {
            FPR( stderr, "X2XYZ Error: memory too small.\n" );
            error = 1;
            goto shutdown;
           }
       }

    if (!(matrix2d = fltAlloc( "x2xyz", length2d )))
       {
        FPR( stderr, "X2XYZ Error allocating memory.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* For each Z-plane in stream:
/*   While slices are left to read from this plane:
/*     Read XY stream.
/*     Open output file if it isn't already opened, write header.
/*     Write XY stream.
/***/

    for( ip = firstPlane - 1; ip < lastPlane; ip++ )
       {
        ipp = ip + 1;

        if (verbose && (ipp == firstPlane || !(ipp % vInc) || (ipp == lastPlane)))
           {
            FPR( stderr, "XY Plane: %d of %d\015", ip + 1, zSize*aSize );
            (void) fflush( stderr );
           }
   
        if (debugFlag)
           {
            FPR( stderr, "\nXY File %s\n", (outListN[ip])->name );
            (void) fflush( stderr );
           }
     
        slicesLeft = ySize;

        while( slicesLeft )
           {
            slicesIn = min( slicesLeft, maxSlices );
            ptsIn    = slicesIn*xSize;

            ETEST( 3, dataReadB( inUnit, matrix2d, sizeof(float)*ptsIn, maxTries, delayTime ));

            if (!outUnit)
               {
                ETEST( 1, dataOpen( (outListN[ip])->name, &outUnit, FB_CREATE ));
                ETEST( 2, dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ));
               }

            ETEST( 4, dataWrite( outUnit, matrix2d, sizeof(float)*ptsIn ));

            slicesLeft -= slicesIn;
           }

        ETEST( 5, dataClose( outUnit ));

        outUnit = UNIT_NULL;
       }

    if (verbose) FPR( stderr, "\n" );

/***/
/* Local exit point:
/***/

shutdown:

    (void) deAlloc( "x2xyz", matrix2d, sizeof(float)*length2d );

    return( error );
}

/***/
/* x2xyzN: write an XYZ-vector stream in XYZ order; variable template format version.
/***/

static int x2xyzN( inUnit, outListN,
                   dimCount, fmtCount, xSize, ySize, zSize, aSize,
                   firstPlane, lastPlane, fdata, inPlace )

   int    dimCount, fmtCount, xSize, ySize, zSize, aSize, firstPlane, lastPlane, inPlace;
   float  fdata[FDATASIZE];
   struct ZListInfo **outListN;
   UNIT   inUnit;
{
    int     iz, ia, thisPlane, openMode, hdrFlag, error;
    struct  ZListInfo *outPtr;
    float   *matrix2d;
    UNIT    outUnit;

    NMR_INT planeSize, planeJump, cubeJump, thisOffset, currentOffset;
    char    currentName[NAMELEN+1];

/***/
/* Initialize:
/***/

    error       = 0;
    matrix2d    = (float *) NULL;
    outUnit     = UNIT_NULL;
    thisPlane   = 0;

    currentName[0] = '\0';
    currentOffset  = (NMR_INT)0;
    thisOffset     = (NMR_INT)0;

   if (inPlace)
       openMode = FB_UNKNOWN;
    else
       openMode = FB_CREATE;

    if (!creatFlag) openMode = FB_UNKNOWN;

    planeSize = (NMR_INT)xSize*(NMR_INT)ySize;
    planeJump = (NMR_INT)xSize*(NMR_INT)ySize;
    cubeJump  = planeJump*zSize;

    if (!(matrix2d = fltAlloc( "x2xyz", planeSize )))
       {
        FPR( stderr, "X2XYZ Error allocating memory.\n" );
        error = 1;
        goto shutdown;
       }

    for( ia = 1; ia <= aSize; ia++ )
       {
        for( iz = 1; iz <= zSize; iz++ )
           {
            outPtr = outListN[thisPlane];
            thisPlane++;

            if (thisPlane < firstPlane || thisPlane > lastPlane) continue;

            if (verbose && (thisPlane == firstPlane || !(thisPlane % vInc) || (thisPlane == lastPlane)))
               {
                FPR( stderr, "XY Plane: %d of %d\015", thisPlane, zSize*aSize );
                (void) fflush( stderr );
               }
   
            ETEST( 1, dataReadB( inUnit, matrix2d, sizeof(float)*planeSize, maxTries, delayTime ));

            if (strcmp( currentName, outPtr->name ))
               {
                if (outUnit)
                   {
                    ETEST( 2, dataClose( outUnit ));

                    currentOffset  = (NMR_INT)0;
                    currentName[0] = '\0';
                    outUnit        = UNIT_NULL;
                   }

                ETEST( 3, dataOpen( outPtr->name, &outUnit, openMode ));

                (void) strcpy( currentName, outPtr->name );
                currentOffset = (NMR_INT)0;
               }

            hdrFlag = 0;

            if (fmtCount == 0)
               {
                thisOffset = (NMR_INT)FDATASIZE + planeJump*outPtr->zOff + cubeJump*outPtr->aOff;
                if (outPtr->zOff == 0 && outPtr->aOff == 0) hdrFlag = 1;
               }
            else if (fmtCount == 1 && dimCount == 4)
               {
                thisOffset = (NMR_INT)FDATASIZE + planeJump*outPtr->zOff;
                if (outPtr->zOff == 0) hdrFlag = 1;
               }
            else
               {
                thisOffset = (NMR_INT)FDATASIZE;
                hdrFlag = 1;
               }

            if (hdrFlag)
               {
                ETEST( 4, dataPos( outUnit, 0 ));
                ETEST( 5, dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ));
                currentOffset = (NMR_INT)FDATASIZE;
               }

            if (thisOffset != currentOffset)
               {
                ETEST( 6, dataPos( outUnit, sizeof(float)*thisOffset ));
                currentOffset = thisOffset; 
               }

            ETEST( 7, dataWrite( outUnit, matrix2d, sizeof(float)*planeSize ));

            currentOffset += planeSize;
           }
       }

    if (verbose) FPR( stderr, "\n" );

/***/
/* Local exit point:
/***/

shutdown:

    if (outUnit)
       {
        ETEST( 666, dataClose( outUnit ));

        currentOffset  = (NMR_INT)0;
        currentName[0] = '\0';
        outUnit        = UNIT_NULL;
       }

    (void) deAlloc( "x2xyz", matrix2d, sizeof(float)*planeSize );

    return( error );
}

/***/
/* y2xyz: write an YXZ-vector stream in XYZ order.
/***/

static int y2xyz( inUnit, outListN,
                  dimCount, fmtCount, 
                  xSize, ySize, zSize, aSize, xQuadState, yQuadState,
                  firstPlane, lastPlane, fdata, inPlace )

   int   dimCount, fmtCount, xSize, ySize, zSize, aSize, xQuadState, yQuadState,
         firstPlane, lastPlane, inPlace;

   float  fdata[FDATASIZE];
   struct ZListInfo **outListN;
   UNIT   inUnit;
{
    NMR_INT length2d;
    int     ip, ipp, error;
    float   *matrix;
    UNIT    outUnit;

    int    tp2DMMM();

/***/
/* Initialize:
/***/

    error    = 0;
    matrix   = (float *) NULL;
    outUnit  = UNIT_NULL;

/***/
/* Compute sizes, allocate half of memory (the rest by tp2dMMM).
/***/

    length2d  = (NMR_INT)xSize*ySize*xQuadState*yQuadState;

    if (!(matrix = fltAlloc( "y2xyz", length2d )))
       {
        FPR( stderr, "Y2XYZ Error allocating memory.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* For each Z-plane in stream:
/*   Read the YX stream.
/*   Transpose from YX to XY order.
/*   Write the header.
/*   Write the plane.
/***/

    for( ip = firstPlane - 1; ip < lastPlane; ip++ )
       {
        ipp = ip + 1;

        if (verbose && (ipp == firstPlane || !(ipp % vInc) || (ipp == lastPlane)))
           {
            FPR( stderr, "YX Plane: %d of %d\015", ip + 1, zSize*aSize );
            (void) fflush( stderr );
           }

        ETEST( 3, dataReadB( inUnit, matrix, sizeof(float)*length2d, maxTries, delayTime ));
        ETEST( 1, dataOpen( (outListN[ip])->name, &outUnit, FB_CREATE ));
        ETEST( 2, dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ));
        ETEST( 4, tp2DMMM( matrix, xSize, ySize, xQuadState*yQuadState ));
        ETEST( 5, dataWrite( outUnit, matrix, sizeof(float)*length2d ));
        ETEST( 6, dataClose( outUnit ));

        outUnit = UNIT_NULL;
       }

/***/
/* Local exit point:
/***/

shutdown:

    if (verbose) FPR( stderr, "\n" );

    (void) deAlloc( "y2xyz", matrix, sizeof(float)*length2d );

    return( error );
}

/***/
/* y2xyzN: write an XYZ-vector stream in YXZ order; variable template format version.
/***/

static int y2xyzN( inUnit, outListN,
                   dimCount, fmtCount, 
                   xSize, ySize, zSize, aSize, xQuadState, yQuadState,
                   firstPlane, lastPlane, fdata, inPlace )

   int    dimCount, fmtCount, xSize, ySize, zSize, aSize, xQuadState, yQuadState, firstPlane, lastPlane, inPlace;
   float  fdata[FDATASIZE];
   struct ZListInfo **outListN;
   UNIT   inUnit;
{
    int     iz, ia, thisPlane, openMode, hdrFlag, error;
    struct  ZListInfo *outPtr;
    float   *matrix2d;
    UNIT    outUnit;

    NMR_INT planeSize, planeJump, cubeJump, thisOffset, currentOffset;
    char    currentName[NAMELEN+1];

    int tp2DMMM();

/***/
/* Initialize:
/***/

    error       = 0;
    matrix2d    = (float *) NULL;
    outUnit     = UNIT_NULL;
    thisPlane   = 0;

    currentName[0] = '\0';
    currentOffset  = (NMR_INT)0;
    thisOffset     = (NMR_INT)0;

   if (inPlace)
       openMode = FB_UNKNOWN;
    else
       openMode = FB_CREATE;

    if (!creatFlag) openMode = FB_UNKNOWN;

    planeSize = (NMR_INT)xSize*ySize*xQuadState*yQuadState;
    planeJump = (NMR_INT)xSize*ySize*xQuadState*yQuadState;
    cubeJump  = planeJump*zSize;

    if (!(matrix2d = fltAlloc( "y2xyz", planeSize )))
       {
        FPR( stderr, "Y2XYZ Error allocating memory.\n" );
        error = 1;
        goto shutdown;
       }

    for( ia = 1; ia <= aSize; ia++ )
       {
        for( iz = 1; iz <= zSize; iz++ )
           {
            outPtr = outListN[thisPlane];
            thisPlane++;

            if (thisPlane < firstPlane || thisPlane > lastPlane) continue;

            if (verbose && (thisPlane == firstPlane || !(thisPlane % vInc) || (thisPlane == lastPlane))) 
               {
                FPR( stderr, "YX Plane: %d of %d\015", thisPlane, zSize*aSize );
                (void) fflush( stderr );
               }
   
            ETEST( 1, dataReadB( inUnit, matrix2d, sizeof(float)*planeSize, maxTries, delayTime ));

            if (strcmp( currentName, outPtr->name ))
               {
                if (outUnit)
                   {
                    ETEST( 2, dataClose( outUnit ));

                    currentOffset  = (NMR_INT)0;
                    currentName[0] = '\0';
                    outUnit        = UNIT_NULL;
                   }

                ETEST( 3, dataOpen( outPtr->name, &outUnit, openMode ));

                (void) strcpy( currentName, outPtr->name );
                currentOffset = (NMR_INT)0;
               }

            hdrFlag = 0;

            if (fmtCount == 0)
               {
                thisOffset = (NMR_INT)FDATASIZE + planeSize*outPtr->zOff + cubeJump*outPtr->aOff;
                if (outPtr->zOff == 0 && outPtr->aOff == 0) hdrFlag = 1;
               }
            else if (fmtCount == 1 && dimCount == 4)
               {
                thisOffset = (NMR_INT)FDATASIZE + planeSize*outPtr->zOff;
                if (outPtr->zOff == 0) hdrFlag = 1;
               }
            else
               {
                thisOffset = (NMR_INT)FDATASIZE;
                hdrFlag = 1;
               }

            if (hdrFlag)
               {
                ETEST( 4, dataPos( outUnit, 0 ));
                ETEST( 5, dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ));
                currentOffset = (NMR_INT)FDATASIZE;
               }

            if (thisOffset != currentOffset)
               {
                ETEST( 6, dataPos( outUnit, sizeof(float)*thisOffset ));
                currentOffset = thisOffset; 
               }

            ETEST( 7, tp2DMMM( matrix2d, xSize, ySize, xQuadState*yQuadState ));
            ETEST( 8, dataWrite( outUnit, matrix2d, sizeof(float)*planeSize ));

            currentOffset += planeSize;
           }
       }

    if (verbose) FPR( stderr, "\n" );

/***/
/* Local exit point:
/***/

shutdown:

    if (outUnit)
       {
        ETEST( 666, dataClose( outUnit ));

        currentOffset  = (NMR_INT)0;
        currentName[0] = '\0';
        outUnit        = UNIT_NULL;
       }

    (void) deAlloc( "y2xyz", matrix2d, sizeof(float)*planeSize );

    return( error );
}

/***/
/* z2xyz: write a ZXY-vector stream in XYZ order.
/***/

static int z2xyz( inUnit, outListN, 
                  dimCount, fmtCount,
                  xSize, ySize, zSize, aSize, quadState, xQuadState, yQuadState, zQuadState, aQuadState,
                  firstPlane, lastPlane, fdata, megabytes, xyzFlag, inPlace )

   int   dimCount, fmtCount, xSize, ySize, zSize, aSize, xQuadState, yQuadState, zQuadState,
         aQuadState, quadState, firstPlane, lastPlane, megabytes, xyzFlag, inPlace;

   float  fdata[FDATASIZE];
   struct ZListInfo **outListN;
   UNIT   inUnit;
{
    NMR_INT  predBytes, trueBytes, nz, nTmp, totalMem, planeSize, length2d, length3d, planesLeft, yPartSize, offset, ptsIn, ptsOut;
    int      ix, ia, hdrSize, openMode, maxPlanesIn, planesIn, thisPlane, workSize, shufCount, error;
    float    *zPtr, *ddPtr, *dddPtr, *matrix2d, *matrix3d, *workR, *workI;
    struct   FileSize predSizeInfo, trueSizeInfo;
    struct   ZListInfo **listPtr;
    UNIT     outUnit;

/***/
/* Initialize:
/***/

    error    = 0;
    matrix2d = (float *)NULL;
    matrix3d = (float *)NULL;
    ddPtr    = (float *)NULL;
    dddPtr   = (float *)NULL;
    workR    = (float *)NULL;
    workI    = (float *)NULL;
    workSize = 0;

    if (inPlace)
       openMode = FB_UNKNOWN;
    else
       openMode = FB_CREATE;

    if (!creatFlag) openMode = FB_UNKNOWN;

    (void) getNMRBytes( fdata, NMR_SINGLE_FILE, &predSizeInfo );

    predBytes = predSizeInfo.iTotalBytes;

/***/
/* Adjust interleaved complex name list if needed.
/* Swap Z and A file names for A-Axis output.
/***/

    if (xyzFlag == 4)
       {
        if ((error = z2aListN( outListN, xSize, aSize )))
           {
            FPR( stderr, "Z2XYZ Error adjusting Z/A list.\n" );
            return( error );
           }

        if (xQuadState == 2)
           {
            if ((error = c2riList2N( outListN, xSize, aSize )))
               {
                FPR( stderr, "Z2XYZ Error adjusting Z/A list.\n" );
                return( error );
               }
           }
       }
    else
       {
        if (xQuadState == 2)
           {
            if ((error = c2riList2N( outListN, xSize, aSize )))
               {
                FPR( stderr, "Z2XYZ Error adjusting name list.\n" );
                return( error );
               }
           }
       }

/***/
/* Allocate and partition memory:
/*  Nota Bene: 3D matrix must also be large enough for predicted output plane.
/***/

    nTmp = xSize > ySize ? xSize : ySize;
    nTmp = zSize > nTmp  ? zSize : nTmp;

    workR = fltAlloc( "z2xyz", nTmp );
    workI = fltAlloc( "z2xyz", nTmp );

    if (!workR || !workI)
       {
        FPR( stderr, "Z2XYZ Error allocating work memory.\n" );
        error = 1;
        goto shutdown;
       }

    workSize   = nTmp;
    totalMem   = (NMR_INT)megabytes*1024*1024/sizeof(float);
    yPartSize  = (NMR_INT)1 + lastPlane - firstPlane;
    planeSize  = (NMR_INT)xSize*zSize;
    planesLeft = yPartSize;

    maxPlanesIn  = totalMem/planeSize - 1;

    if (!(yPartSize % 2))
       {
        if (maxPlanesIn % 2)
           {
            maxPlanesIn++;
           }
       }

    maxPlanesIn = min( maxPlanesIn, yPartSize );
    length3d    = (NMR_INT)maxPlanesIn*planeSize;
    length2d    = (NMR_INT)zSize*yPartSize;
    nTmp        = (NMR_INT)(predBytes/sizeof(float));

    if (length3d < nTmp)
       {
        length3d = (NMR_INT)xSize + nTmp;
       }

    if (maxPlanesIn < 1)
       {
        if (autoFlag)
           {
            maxPlanesIn = 1;

            if (!(yPartSize % 2))
               {
                if (maxPlanesIn % 2)
                   {
                    maxPlanesIn++;
                   }
               }
        
            length3d  = (NMR_INT)maxPlanesIn*planeSize;
            length2d  = (NMR_INT)zSize*yPartSize;
           }
        else
           {
            FPR( stderr, "Z2XYZ Error: insufficient memory size.\n" );
            error = 1;
            goto shutdown;
           }
       }

    if (!(matrix3d = fltAlloc( "z2xyz", length3d )))
       {
        FPR( stderr, "Z2XYZ Error allocating 3D memory.\n" );
        error = 1;
        goto shutdown;
       }

    if (!(matrix2d = fltAlloc( "z2xyz", length2d )))
       {
        FPR( stderr, "Z2XYZ Error allocating 2D memory.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* For each 3D cube in input stream:
/*   When writing a subset of the data, avoid create/delete of existing data.
/*   Write the headers to each output plane.
/*
/*   For each group of ZX planes in stream:
/*     Read series of ZX vectors from stream.
/*     For each ZX plane in group, move Z vectors into 2D matrix in XZ order.
/*     Disperse Z vectors to 3D output files.
/*
/*   If in-place mode, for all 2D planes in 3D result cube:
/*     Truncate planes in cube if needed.
/***/

     listPtr = outListN;

     for( ia = 0; ia < aSize; ia++ )
        {
         for( ix = 0; ix < xSize; ix++ )
            {
             if (firstPlane == 1)
                {
                 ETEST( 1, dataOpen( (listPtr[ix])->name, &outUnit, openMode ));
                }
             else
                {
                 ETEST( 2, dataOpen( (listPtr[ix])->name, &outUnit, FB_UNKNOWN ));
                }

             ETEST( 3, dataPos( outUnit, 0 ));
             ETEST( 4, dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ));
             ETEST( 5, dataClose( outUnit ));

             outUnit = UNIT_NULL;
            }

         offset     = (NMR_INT)FDATASIZE + ((NMR_INT)firstPlane - 1)*zSize;
         planesLeft = yPartSize;
         thisPlane  = firstPlane;
  
         while( planesLeft )
            {
             planesIn = (planesLeft < maxPlanesIn) ? planesLeft : maxPlanesIn;
             ptsIn    = (NMR_INT)planesIn*xSize*zSize;
             ptsOut   = (NMR_INT)planesIn*zSize;
             dddPtr   = matrix3d;

             ETEST( 6, dataReadB( inUnit, dddPtr, sizeof(float)*ptsIn, maxTries, delayTime ));

             for( ix = 0; ix < xSize; ix++ )
                {
                 ddPtr = matrix2d;
                 zPtr = dddPtr++;
      
                 for( nz = 0; nz < (NMR_INT)zSize*planesIn; nz++ )
                    {
                     *ddPtr++  = *zPtr;
                     zPtr     += xSize;
                    }
   
                 if (shufFlag)
                    {
                     shufCount = ptsOut/(2*zSize);
                     (void) xy2zaShufN( matrix2d, zSize, shufCount, workR, workI );
                    }
 
                 ETEST(  7, dataOpen( (listPtr[ix])->name, &outUnit, FB_WRITE ));
                 ETEST(  8, dataPos( outUnit, sizeof(float)*offset ));
                 ETEST(  9, dataWrite( outUnit, matrix2d, sizeof(float)*ptsOut));
                 ETEST( 10, dataClose( outUnit ));

                 outUnit = UNIT_NULL;
                }

             if (verbose)
                {
                 if (xyzFlag == 3)
                    {
                     if (aSize > 1)
                        FPR( stderr, "XZ Planes: %3d %3d of %d A: %d\015", thisPlane, thisPlane + planesIn - 1, ySize, ia + 1 );
                     else
                        FPR( stderr, "XZ Planes: %3d %3d of %d\015", thisPlane, thisPlane + planesIn - 1, ySize );
                    }
                 else
                    {
                     FPR( stderr, "XA Planes: %3d %3d of %d A: %d\015", thisPlane, thisPlane + planesIn - 1, ySize, ia + 1 );
                    }

                 (void) fflush( stderr );
                }

             planesLeft -= planesIn;
             thisPlane  += planesIn;
             offset     += ptsOut;
            }

         if (truncFlag)
            {
             for( ix = 0; inPlace && ix < xSize; ix++ )
                {
                 (void) getFileBytes( (listPtr[ix])->name, &trueSizeInfo );

                 trueBytes = trueSizeInfo.iTotalBytes;
                 hdrSize   = (int)(sizeof(float)*FDATASIZE);

                 if (trueBytes > predBytes)
                    {
                     if (predBytes > (NMR_INT)sizeof(float)*length3d)
                        {
                         (void) deAlloc( "z2xyz", matrix3d, sizeof(float)*length3d );

                         length3d = 0;
                         nTmp     = 1 + predBytes/sizeof(float);

                         if (!(matrix3d = fltAlloc( "z2xyz", nTmp )))
                            {
                             FPR( stderr, "Z2XYZ Error allocating 3D memory.\n" );
                             error = 1;
                             goto shutdown;
                            }

                         length3d = nTmp;
                        }

                     ETEST( 11, dataOpen( (listPtr[ix])->name, &outUnit, FB_READ ));
                     ETEST( 12, dataRead( outUnit, matrix3d, predBytes ));
                     ETEST( 13, dataClose( outUnit ));

                     outUnit = UNIT_NULL;

                     ETEST( 14, dataOpen( (listPtr[ix])->name, &outUnit, FB_CREATE ));
                     ETEST( 15, dataWrite( outUnit, fdata, hdrSize ));
                     ETEST( 16, dataWrite( outUnit, matrix3d, predBytes ));
                     ETEST( 17, dataClose( outUnit ));

                     outUnit = UNIT_NULL;
                    }
                }
            }

         listPtr += xSize;
        }

/***/
/* Local Exit point:
/***/

shutdown:

    if (verbose) FPR( stderr, "\n" );

    (void) deAlloc( "z2xyz", workR,    sizeof(float)*workSize );
    (void) deAlloc( "z2xyz", workI,    sizeof(float)*workSize );
    (void) deAlloc( "z2xyz", matrix2d, sizeof(float)*length2d );
    (void) deAlloc( "z2xyz", matrix3d, sizeof(float)*length3d );

#ifdef S_SPLINT_S
    (void) free( (void *)dddPtr );
    (void) free( (void *)ddPtr );
#endif

    return( error );
}

/***/
/* z2xyzN: write a ZXY-vector stream in XYZ order; variable template format version.
/***/

static int z2xyzN( inUnit,
                   outTemplate, outListN, 
                   dimCount, fmtCount, xSize, ySize, zSize, aSize, 
                   nX, nY, nZ, nA, nQ, quadState, xQuadState, yQuadState, zQuadState, aQuadState,
                   firstPlane, lastPlane, maxPlanesIn, fdata, xyzFlag, inPlace )

   int   dimCount, fmtCount, xSize, ySize, zSize, aSize, nX, nY, nZ, nA, nQ,
         xQuadState, yQuadState, zQuadState, aQuadState, quadState, 
         firstPlane, lastPlane, maxPlanesIn, xyzFlag, inPlace;

   char   *outTemplate;
   float  fdata[FDATASIZE];
   struct ZListInfo **outListN;
   UNIT   inUnit;
{
    NMR_INT  predBytes, trueBytes, nz, nTmp, planeSize, planeJump, cubeJump, length2d, length3d, planesLeft, 
             yPartSize, vecOffset, ptsIn, ptsOut;

    int      ix, iz, ia, ip1, ipn, msgCount, hdrSize, hdrFlag, openMode, thisOpenMode, planesIn, thisPlane,
             workSize, shufCount, error;

    float    *zPtr, *ddPtr, *dddPtr, *matrix2d, *matrix3d, *workR, *workI;
    struct   FileSize predSizeInfo, trueSizeInfo;
    struct   ZListInfo **listPtr;
    UNIT     outUnit;

    char     currentName[NAMELEN+1];
    NMR_INT  currentOffset, thisOffset;

    struct ZListInfo *outPtr;

/***/
/* Initialize:
/***/

    error    = 0;
    outUnit  = UNIT_NULL;
    matrix2d = (float *)NULL;
    matrix3d = (float *)NULL;
    ddPtr    = (float *)NULL;
    dddPtr   = (float *)NULL;
    workR    = (float *)NULL;
    workI    = (float *)NULL;
    workSize = 0;
    msgCount = 0;

    currentName[0] = '\0';
    currentOffset  = (NMR_INT)0;
    thisOffset     = (NMR_INT)0;

    if (inPlace)
       openMode = FB_UNKNOWN;
    else
       openMode = FB_CREATE;

    if (!creatFlag) openMode = FB_UNKNOWN;

    (void) getNMRBytes( fdata, NMR_SINGLE_FILE, &predSizeInfo );

    predBytes = predSizeInfo.iTotalBytes;

/***/
/* Adjust interleaved complex name list if needed.
/* Swap Z and A file names for A-Axis output.
/***/

    if (xyzFlag == 4)
       {
        if ((error = z2aListN( outListN, xSize, aSize )))
           {
            FPR( stderr, "Z2XYZ Error adjusting Z/A list.\n" );
            return( error );
           }

        if (xQuadState == 2)
           {
            if ((error = c2riList2N( outListN, xSize, aSize )))
               {
                FPR( stderr, "Z2XYZ Error adjusting Z/A list.\n" );
                return( error );
               }
           }
       }
    else
       {
        if (xQuadState == 2)
           {
            if ((error = c2riList2N( outListN, xSize, aSize )))
               {
                FPR( stderr, "Z2XYZ Error adjusting name list.\n" );
                return( error );
               }
           }
       }

/***/
/* Allocate memory:
/***/

    nTmp = xSize > ySize ? xSize : ySize;
    nTmp = zSize > nTmp  ? zSize : nTmp;

    workR = fltAlloc( "z2xyz", nTmp );
    workI = fltAlloc( "z2xyz", nTmp );

    if (!workR || !workI)
       {
        FPR( stderr, "Z2XYZ Error allocating work memory.\n" );
        error = 1;
        goto shutdown;
       }

    workSize   = nTmp;
    yPartSize  = (NMR_INT)1 + lastPlane - firstPlane;
    planeSize  = (NMR_INT)xSize*zSize;
    planeJump  = (NMR_INT)nX*nY*nQ;
    cubeJump   = planeJump*nZ;
    planesLeft = yPartSize;

    if (maxPlanesIn == 0) maxPlanesIn  = 8;
    if (maxPlanesIn < 1)  maxPlanesIn  = yPartSize;

    if (!(yPartSize % 2))
       {
        if (maxPlanesIn % 2)
           {
            maxPlanesIn++;
           }
       }

    maxPlanesIn = min( maxPlanesIn, yPartSize );
    length3d    = (NMR_INT)maxPlanesIn*planeSize;
    length2d    = (NMR_INT)zSize*yPartSize;

    if (verbose) FPR( stderr, "XZ Buffer Size: %d Planes\n", maxPlanesIn );

    if (!(matrix3d = fltAlloc( "z2xyz", length3d )))
       {
        FPR( stderr, "Z2XYZ Error allocating 3D memory.\n" );
        error = 1;
        goto shutdown;
       }

    if (!(matrix2d = fltAlloc( "z2xyz", length2d )))
       {
        FPR( stderr, "Z2XYZ Error allocating 2D memory.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* Write headers to all files if needed:
/***/
 
     if (firstPlane == 1)
        {
         if (fmtCount == 0)
            {
             (void) strcpy( currentName, outTemplate );

             ETEST( 1, dataOpen( currentName, &outUnit, FB_UNKNOWN ));
             ETEST( 2, dataPos( outUnit, 0 ));
             ETEST( 3, dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ));
             ETEST( 4, dataClose( outUnit ));

             outUnit = UNIT_NULL;
             currentName[0] = '\0';
            }
         else if (fmtCount == 1 && dimCount == 3)
            {
             for( iz = 1; iz <= nZ; iz++ )
                {
                 (void) sprintf( currentName, outTemplate, iz );

                 ETEST( 5, dataOpen( currentName, &outUnit, FB_UNKNOWN ));
                 ETEST( 6, dataPos( outUnit, 0 ));
                 ETEST( 7, dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ));
                 ETEST( 8, dataClose( outUnit ));

                 outUnit = UNIT_NULL;
                 currentName[0] = '\0';
                }
            }
         else if (fmtCount == 1 && dimCount == 4)
            {
             for( ia = 1; ia <= nA; ia++ )
                {
                 (void) sprintf( currentName, outTemplate, ia );

                 ETEST(  9, dataOpen( currentName, &outUnit, FB_UNKNOWN ));
                 ETEST( 10, dataPos( outUnit, 0 ));
                 ETEST( 11, dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ));
                 ETEST( 12, dataClose( outUnit ));
            
                 outUnit = UNIT_NULL;
                 currentName[0] = '\0';
                }
            }
         else
            {
             for( ia = 1; ia <= nA; ia++ )
                {
                 for( iz = 1; iz <= nZ; iz++ )
                    {
                     (void) sprintf( currentName, outTemplate, ia, iz );
    
                     ETEST( 13, dataOpen( currentName, &outUnit, FB_UNKNOWN ));
                     ETEST( 14, dataPos( outUnit, 0 ));
                     ETEST( 15, dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE ));
                     ETEST( 16, dataClose( outUnit ));
    
                     outUnit = UNIT_NULL;
                     currentName[0] = '\0';
                    }
                }
            }
        }

/***/
/* Read and rearrange the data:
/***/
 
     listPtr        = outListN;
     currentName[0] = '\0';

     for( ia = 0; ia < aSize; ia++ )
        {
         vecOffset   = (NMR_INT)firstPlane - 1;
         planesLeft  = yPartSize;
         thisPlane   = firstPlane;
  
         while( planesLeft )
            {
             planesIn = (planesLeft < maxPlanesIn) ? planesLeft : maxPlanesIn;
             ptsIn    = (NMR_INT)planesIn*xSize*zSize;
             ptsOut   = (NMR_INT)planesIn*zSize;
             dddPtr   = matrix3d;

             ETEST( 17, dataReadB( inUnit, dddPtr, sizeof(float)*ptsIn, maxTries, delayTime ));

             for( ix = 0; ix < xSize; ix++ )
                {
                 outPtr = listPtr[ix];
                 ddPtr  = matrix2d;
                 zPtr   = dddPtr++;
      
                 for( nz = 0; nz < (NMR_INT)zSize*planesIn; nz++ )
                    {
                     *ddPtr++  = *zPtr;
                     zPtr     += xSize;
                    }
   
                 if (shufFlag)
                    {
                     shufCount = ptsOut/(2*zSize);
                     (void) xy2zaShufN( matrix2d, zSize, shufCount, workR, workI );
                    }

                 if (strcasecmp( currentName, outPtr->name ))
                    {
                     if (outUnit)
                        {
                         ETEST( 18, dataClose( outUnit ));

                         outUnit        = UNIT_NULL;
                         currentOffset  = (NMR_INT)0;
                         currentName[0] = '\0';
                        }

                     ETEST( 19, dataOpen( outPtr->name, &outUnit, FB_WRITE ));

                     (void) strcpy( currentName, outPtr->name );
                     currentOffset  = (NMR_INT)0;
                    }

                 if (fmtCount == 0)
                    {
                     if (xyzFlag == 3)
                        thisOffset = (NMR_INT)FDATASIZE + cubeJump*outPtr->aOff + planeJump*outPtr->zOff + vecOffset*zSize;
                     else
                        thisOffset = (NMR_INT)FDATASIZE + cubeJump*outPtr->aOff + planeJump*outPtr->zOff + vecOffset*zSize;
                    }
                 else if (fmtCount == 1 && dimCount == 4)
                    {
                     if (xyzFlag == 3)
                        thisOffset = (NMR_INT)FDATASIZE + planeJump*outPtr->zOff + vecOffset*zSize;
                     else
                        thisOffset = (NMR_INT)FDATASIZE + planeJump*outPtr->zOff + vecOffset*zSize;
                    }
                 else
                    {
                     thisOffset = (NMR_INT)FDATASIZE + vecOffset*zSize;
                    }

                 if (thisOffset != currentOffset)
                    {
                     ETEST( 20, dataPos( outUnit, sizeof(float)*thisOffset ));
                     currentOffset = thisOffset;
                    }

                 ETEST( 21, dataWrite( outUnit, matrix2d, sizeof(float)*ptsOut));

                 currentOffset += ptsOut;
                }

             if (verbose)
                {
                 if (xyzFlag == 3)
                    {
                     if (aSize > 1)
                        FPR( stderr, "XZ Planes: %3d %3d of %d A: %3d of %d\015", thisPlane, thisPlane + planesIn - 1, ySize, ia + 1, aSize );
                     else
                        FPR( stderr, "XZ Planes: %3d %3d of %d\015", thisPlane, thisPlane + planesIn - 1, ySize );
                    }
                 else
                    {
                     FPR( stderr, "XA Planes: %3d %3d of %d A: %3d of %d\015", thisPlane, thisPlane + planesIn - 1, ySize, ia + 1, aSize );
                    }
                   
                 (void) fflush( stderr );
                }

             planesLeft  -= planesIn;
             thisPlane   += planesIn;
             vecOffset += planesIn;
            }

         listPtr += xSize;
        }

/***/
/* Truncate files if needed:
/***/
 
     if (truncFlag)
        {
         if (fmtCount == 0)
            {
             (void) strcpy( currentName, outTemplate );
             (void) getFileBytes( currentName, &trueSizeInfo );

             trueBytes = trueSizeInfo.iTotalBytes;

             if (trueBytes > predBytes) (void) truncate( currentName, (off_t)predBytes );
            }
         else if (fmtCount == 1 && dimCount == 3)
            {
             for( iz = 1; iz <= nZ; iz++ )
                {
                 (void) sprintf( currentName, outTemplate, iz );
                 (void) getFileBytes( currentName, &trueSizeInfo );

                 trueBytes = trueSizeInfo.iTotalBytes;

                 if (trueBytes > predBytes) (void) truncate( currentName, (off_t)predBytes );
                }
            }
         else if (fmtCount == 1 && dimCount == 4)
            {
             for( ia = 1; ia <= nA; ia++ )
                {
                 (void) sprintf( currentName, outTemplate, ia );
                 (void) getFileBytes( currentName, &trueSizeInfo );

                 trueBytes = trueSizeInfo.iTotalBytes;

                 if (trueBytes > predBytes) (void) truncate( currentName, (off_t)predBytes );
                }
            }
         else
            {
             for( ia = 1; ia <= nA; ia++ )
                {
                 for( iz = 1; iz <= nZ; iz++ )
                    {
                     (void) sprintf( currentName, outTemplate, ia, iz );
                     (void) getFileBytes( currentName, &trueSizeInfo );

                     trueBytes = trueSizeInfo.iTotalBytes;

                     if (trueBytes > predBytes) (void) truncate( currentName, (off_t)predBytes );
                    }
                }
            }
        }

/***/
/* Local Exit point:
/***/

shutdown:

    if (verbose) FPR( stderr, "\n" );

    if (outUnit)
       {
        ETEST( 666, dataClose( outUnit ));

        outUnit        = UNIT_NULL;
        currentOffset  = (NMR_INT)0;
        currentName[0] = '\0';
       }

    (void) deAlloc( "z2xyz", workR,    sizeof(float)*workSize );
    (void) deAlloc( "z2xyz", workI,    sizeof(float)*workSize );
    (void) deAlloc( "z2xyz", matrix2d, sizeof(float)*length2d );
    (void) deAlloc( "z2xyz", matrix3d, sizeof(float)*length3d );

#ifdef S_SPLINT_S
    (void) free( (void *)dddPtr );
    (void) free( (void *)ddPtr );
#endif

    return( error );
}

/***/
/* 
/***/
/* x2nv: write an XYZ-vector stream in NMRView format.
/***/

static int x2nv( inUnit, outName,
                 xSize, ySize, zSize, aSize, dimCount,
                 xQuadState, yQuadState, zQuadState, aQuadState,
                 firstPlane, lastPlane, fdata, megabytes,
                 inPlace )

    int   xSize, ySize, zSize, aSize, dimCount, xQuadState, yQuadState,
          zQuadState, aQuadState, firstPlane, lastPlane, megabytes, inPlace;

    float fdata[FDATASIZE];
    char  *outName;
    UNIT  inUnit;
{
    NMR_INT maxSlices, slicesIn, slicesLeft, ptsIn, length2d;
    int     i, iz, isX, isY, isZ, isA, thisPlane, error;
    float   *matrix2d;
    DATASET *nvData;

/***/
/* Initialize:
/***/

    error    = 0;
    matrix2d = (float *) NULL;
    nvData   = (DATASET *)NULL;

/***/
/* Compute sizes, allocate memory.
/***/

    FPR( stderr,
         "%d %d %d %d %d %d %s\n",
         xSize, ySize, zSize, aSize, firstPlane, lastPlane, outName );

    maxSlices = (NMR_INT)megabytes*1024*1024/(xSize*sizeof(float));
    maxSlices = min( ySize, maxSlices );
    length2d  = maxSlices*xSize;

    if (dimCount >= 2 && (xQuadState > 1 || yQuadState > 1))
       {
        FPR( stderr, "X2NMRView Error: X-Axis and Y-Axis Should be Real-Only.\n" );
        error = 1;
        goto shutdown;
       }

    if (dimCount >= 3 && zQuadState > 1)
       {
        FPR( stderr, "X2NMRView Error: Z-Axis Should be Real-Only.\n" ); 
        error = 1;
        goto shutdown;
       }

    if (dimCount >= 4 && aQuadState > 1)
       {
        FPR( stderr, "X2NMRView Error: A-Axis Should be Real-Only.\n" ); 
        error = 1;
        goto shutdown;
       }

    if (maxSlices < 1)
       {
        FPR( stderr, "X2NMRView Error: memory too small.\n" );
        error = 1;
        goto shutdown;
       }

    if (!(matrix2d = fltAlloc( "x2nv", length2d )))
       {
        FPR( stderr, "X2NMRView Error allocating memory.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* Create and initialize the NMRView DATASET description:
/***/

    if (!(nvData = Make_Dataset( ".", outName )))
       {
        FPR( stderr, "X2NMRView Error creating DATASET structure.\n");
        error = 1;
        goto shutdown;
       }

    nvData->matfile.ndim    = dimCount;

    for( i = 0; i < 4; i++ )
       {
        nvData->matfile.size[i]  = getParm( fdata, NDSIZE,   i+1 );
        nvData->matfile.sw[i]    = getParm( fdata, NDSW,     i+1 );
        nvData->matfile.sf[i]    = getParm( fdata, NDOBS,    i+1 );
        nvData->matfile.ref[i]   = getParm( fdata, NDCAR,    i+1 );
        nvData->matfile.refpt[i] = getParm( fdata, NDCENTER, i+1 ) - 1.0;

        (void) strcpy( nvData->matfile.label[i], getParmStr( fdata, NDLABEL, i+1 ));
       }

    (void) SetBlockSize( nvData );

    if (open_file( nvData ) == ERROR)
       {
        FPR( stderr, "X2NMRView Error opening output file.\n" );
        error = 1;
        goto shutdown;
       }

    (void) nvSetup( nvData );

/***/
/* For each 2D plane in stream:
/*   While slices are left to read from this plane:
/*     Read XY stream.
/*     Write the NMRView format data.
/*     Update the NMRView output index locations.
/***/

    isY = 0;
    isX = 0;
    isZ = 0;
    isA = 0;

    for( iz = firstPlane - 1; iz < lastPlane; iz++ )
       {
        thisPlane = iz + 1;

        if (verbose && (thisPlane == firstPlane || !(thisPlane % vInc) || (thisPlane == lastPlane)))
           {
            FPR( stderr, "XY Plane: %d of %d\015", iz + 1, zSize*aSize );
            (void) fflush( stderr );
           }

        slicesLeft = ySize;

        isY = 0;
        isX = 0;

        while( slicesLeft > 0)
           {
            slicesIn = min( slicesLeft, maxSlices );
            ptsIn    = slicesIn*xSize;

            ETEST( 1, dataReadB( inUnit, matrix2d, sizeof(float)*ptsIn, maxTries, delayTime ));

            (void) nvWrite( nvData, matrix2d, isX, isY, isZ, isA, slicesIn );

            slicesLeft -= slicesIn;
            isY        += slicesIn;
           }

        isZ++;

        if (isZ >= zSize)
           {
            isZ = 0;
            isA++;
           }
       }

    if (verbose) FPR( stderr, "\n" );

    (void) nvWriteBuffers( nvData );
    (void) nvCloseFile( nvData );

/***/
/* Local exit point:
/***/

shutdown:

    (void) deAlloc( "x2nv", matrix2d, sizeof(float)*length2d );
    (void) deAlloc( "x2nv", nvData, 0 );

    return( error );
}

/***/
/* pipe2xyzFD: analyze input stream header for output series.
/*             Returns  0 on success.
/*             Returns -1 on NULL header.
/*             Returns  1 on other error.
/***/

static int pipe2xyzFD( inName, inUnitPtr, dimCount,
                       xQuadState, yQuadState, zQuadState, aQuadState, quadState,
                       xSize, ySize, zSize, aSize, firstPlane, lastPlane, maxPlane, 
                       megabytes, inPlace, xyzFlag, fmtFlag, fmtCount, fdata )

   int    *quadState, *xQuadState, *yQuadState, *zQuadState, *aQuadState,
          *xSize, *ySize, *zSize, *aSize, *dimCount, *firstPlane,
          *lastPlane, *maxPlane, *megabytes, xyzFlag, inPlace, fmtFlag, fmtCount;

   float  fdata[FDATASIZE];
   UNIT   *inUnitPtr;
   char   *inName;
{
    int   size, specnum, maxSize, xyQuadState, yaQuadState, sourceDim,
          oldZSize, oldASize, xOutSize, yOutSize, zOutSize, aOutSize,
          length2d, quad1, quad2, dim1, dim2, dim3, dim4, thisDim,
          yAlt, i, j, itemp, swapDone, outCount, error;

/***/
/* Convenient notation for complex state of 3D data:
/***/

#define XC (*xQuadState == 2)
#define YC (*yQuadState == 2)
#define ZC (*zQuadState == 2)
#define AC (*aQuadState == 2)

#define XR (*xQuadState == 1)
#define YR (*yQuadState == 1)
#define ZR (*zQuadState == 1)
#define AR (*aQuadState == 1)

#define ZCCR (XC && YC && ZR)
#define ZRCC (XR && YC && ZC)
#define ZRCR (XR && YC && ZR)
#define ZCCC (XC && YC && ZC)

#define ACCRC (XC && YC && ZR && AC)
#define ARCCC (XR && YC && ZC && AC)
#define ARCCR (XR && YC && ZC && AR)
#define ACCRR (XC && YC && ZR && AR)
#define ACCCR (XC && YC && ZC && AR)
#define ACCCC (XC && YC && ZC && AC)

/***/
/* Initialize:
/***/

    error      = 0;
    swapDone   = 0;
    *inUnitPtr = UNIT_NULL;

/***/
/* Open input stream as needed and read header:
/*  Error from fixfdata means NULL header.
/***/

    if (*inName)
       {
        ETEST( 1, dataOpen( inName, inUnitPtr, FB_READ ));
        maxTries  = 1;
        delayTime = 0;
       }
    else
       {
        *inUnitPtr = FB_STDIN;
       }

    ETEST( 2, readHdrB( *inUnitPtr, fdata, maxTries, delayTime, &swapDone ));

    if ((error = fixfdata( fdata ))) return( -1 );

/***/
/* Check for valid source dimension:
/***/

    *dimCount = getParmI( fdata, FDDIMCOUNT, NULL_DIM );
    sourceDim = getParmI( fdata, FDPIPEFLAG, 0 );

    switch( sourceDim )
       {
        case NULL_DIM:
           if (fmtFlag == FMT_NMRPIPE)
              {
               (void) FPR( stderr,
                           "Pipe2XYZ Notice: Bad Source Dimension %d.\n",
                           sourceDim );
              }
           break;
 
        case CUR_XDIM:
           if (inPlace && (xyzFlag != CUR_XDIM && xyzFlag != CUR_YDIM))
              {
               FPR( stderr, "%s%s%s",
                    "Pipe2XYZ Error: Bad In-Place Output.\n",
                    "Source \"-x\" must have destination",
                    " \"-x\" or \"-y\".\n" );

               return( 1 );
              }
           break;

        case CUR_YDIM:
           if (inPlace && (xyzFlag != CUR_XDIM && xyzFlag != CUR_YDIM))
              {
               FPR( stderr, "%s%s%s",
                    "Pipe2XYZ Error: Bad In-Place Output.\n", 
                    "Source \"-y\" must have destination\n", 
                    " \"-x\" or \"-y\".\n" );

               return( 1 );
              }
           break;

        case CUR_ZDIM:
           if (inPlace && xyzFlag != CUR_ZDIM)
              {
               FPR( stderr, "%s%s%s",
                    "Pipe2XYZ Error: Bad In-Place Output.\n", 
                    "Source \"-z\" must have destination", 
                    " \"-z\".\n" );

               return( 1 );
              }
           break;

        case CUR_ADIM:
           if (inPlace && xyzFlag != CUR_ADIM)
              {
               FPR( stderr, "%s%s%s",
                    "Pipe2XYZ Error: Bad In-Place Output.\n", 
                    "Source \"-a\" must have destination", 
                    " \"-a\".\n" );

               return( 1 );
              }
           break;

        default:
           (void) FPR( stderr,
                       "Pipe2XYZ Notice: Bad Source Dimension %d.\n",
                       sourceDim );

           break;
       }

/***/
/* Decode sizes and complex states:
/***/

    size        = getParmI( fdata, NDSIZE, CUR_XDIM );
    specnum     = getParmI( fdata, NDSIZE, CUR_YDIM );
    oldZSize    = getParmI( fdata, NDSIZE, CUR_ZDIM );
    oldASize    = *dimCount < 4 ? 1 : getParmI( fdata, NDSIZE, CUR_ADIM );

    *quadState  = getQuad( fdata, FDQUADFLAG, 0 );

    *xQuadState = getQuad( fdata, NDQUADFLAG, CUR_XDIM );
    *yQuadState = getQuad( fdata, NDQUADFLAG, CUR_YDIM );
    *zQuadState = getQuad( fdata, NDQUADFLAG, CUR_ZDIM );
    *aQuadState = *dimCount < 4 ? 1 : getQuad( fdata, NDQUADFLAG, CUR_ADIM );

    yAlt = 0;

    if ((0 == getParmI( fdata, NDFTFLAG, CUR_YDIM )))
       {
        itemp = getParmI( fdata, NDAQSIGN, CUR_YDIM );

        if (itemp == ALT_SEQUENTIAL || itemp == ALT_SEQUENTIAL_NEG)
           {
            yAlt = 1;
           } 
       }

    xyQuadState = (*xQuadState)*(*yQuadState)/(*quadState);
    yaQuadState = (*yQuadState)*(*aQuadState)/(*quadState);

    if (xyQuadState < 1) xyQuadState = 1;
    if (yaQuadState < 1) yaQuadState = 1;

/***/
/* Extract current dimension order and sizes:
/***/

    dim1     = getParmI( fdata, FDDIMORDER1, 0 );
    dim2     = getParmI( fdata, FDDIMORDER2, 0 );
    dim3     = getParmI( fdata, FDDIMORDER3, 0 );
    dim4     = getParmI( fdata, FDDIMORDER4, 0 );

    thisDim  = dim1;

    xOutSize = getParmI( fdata, NDSIZE, CUR_XDIM );
    yOutSize = getParmI( fdata, NDSIZE, CUR_YDIM );
    zOutSize = getParmI( fdata, NDSIZE, CUR_ZDIM );
    aOutSize = *dimCount < 4 ? 1 : getParmI( fdata, NDSIZE, CUR_ADIM );

    xOutSize = xOutSize < 1 ? 1 : xOutSize;
    yOutSize = yOutSize < 1 ? 1 : yOutSize;
    zOutSize = zOutSize < 1 ? 1 : zOutSize;
    aOutSize = aOutSize < 1 ? 1 : aOutSize;
 
/***/
/* Extract range of planes in data:
/***/

    i = getParmI( fdata, FDFIRSTPLANE, 0 ); 
    j = getParmI( fdata, FDLASTPLANE,  0 ); 

    if (debugFlag)
       {
        FPR( stderr,
             "\nHeader First Plane: %d Last Plane: %d\n",
             i, j );

        (void) fflush( stderr );
       }

    if (!*firstPlane && !*lastPlane)
       {
        *firstPlane = i;
        *lastPlane  = j;
       }
    else 
       {
        if ((1 + j - i) != (1 + *lastPlane - *firstPlane))
           {
            FPR( stderr, "Pipe2XYZ Error: Bad Plane Range.\n" );
            return( 1 );
           }

        creatFlag   = 0;
        *firstPlane = i;
        *lastPlane  = j;
       }

/***/
/* Special adjustments for NMRView format output:
/***/

    if (fmtFlag == FMT_NMRVIEW)
       {
        if (sourceDim == NULL_DIM)
           {
            *dimCount   = specnum > 1 ? 2 : 1;
            *firstPlane = 1;
            *lastPlane  = 1;

            zOutSize    = 1;
            aOutSize    = 1;
            oldZSize    = 1;
            oldASize    = 1;
           }
       }

/***/
/* Prepare sizes and header describing output stream:
/*  
/* Nota Bene: many special adjustments of sizes have been
/*            added to compensate for difficult history
/*            of size and quad state changes. 
/***/

    switch( xyzFlag )
       {
        case CUR_XDIM:
           *xSize      = size;
           *ySize      = specnum*(*quadState);
           *zSize      = oldZSize;
           *aSize      = oldASize;

           outCount    = oldZSize*oldASize;
           maxSize     = size*specnum*(*quadState);

           if (dim1 == 2)
              {
               (void) setParm( fdata, FDTRANSPOSED, (float)0.0, 0 );
              }

           thisDim  = dim1;
           break;

        case CUR_YDIM:
           *xSize           = size;
           *ySize           = specnum/xyQuadState;
           *zSize           = oldZSize;
           *aSize           = oldASize;

           outCount         = oldZSize*oldASize;
           xOutSize         = specnum/xyQuadState;
           yOutSize         = size*xyQuadState;

           maxSize          = 2*size*specnum*(*quadState);
           length2d         = (*xSize)*(*ySize)*(*xQuadState)*(*yQuadState);

           if (maxSize < 2*length2d) maxSize = 2*length2d;

           if (FD_MAGNITUDE == getParmI( fdata, FD2DPHASE, 0 ))
              {
               quad1 = getParmI( fdata, NDQUADFLAG, CUR_XDIM );
               quad2 = getParmI( fdata, NDQUADFLAG, CUR_YDIM );
  
               (void) setParm( fdata, NDQUADFLAG, (float)quad2, CUR_XDIM );
               (void) setParm( fdata, NDQUADFLAG, (float)quad1, CUR_YDIM );
              }

           if (*yQuadState == 1 && yAlt)
              {
               (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );
              }

           if (1 == getParmI( fdata, FDTRANSPOSED, 0 ))
              (void) setParm( fdata, FDTRANSPOSED, (float)0.0, 0 );
           else
              (void) setParm( fdata, FDTRANSPOSED, (float)1.0, 0 );

           (void) setParm( fdata, FDDIMORDER1, (float)dim2, 0 );
           (void) setParm( fdata, FDDIMORDER2, (float)dim1, 0 );

           if (dim2 == 2)
              {
               (void) setParm( fdata, FDTRANSPOSED, (float)0.0, 0 );
              }

           thisDim  = dim2;
           break;

        case CUR_ZDIM:
           *xSize      = size*(*xQuadState);
           *ySize      = oldZSize*(*quadState)/(*xQuadState);
           *zSize      = specnum;
           *aSize      = oldASize;

           outCount    = size*oldASize*(*xQuadState);
           xOutSize    = specnum;
           yOutSize    = oldZSize/(*zQuadState);
           zOutSize    = size*(*xQuadState);
           aOutSize    = oldASize;

           if (ZCCR) *ySize   *= 2;
           if (ZCCR) *zSize   /= 2;

           if (ZCCR) xOutSize /= 2;
           if (ZRCC) yOutSize *= 2;

           if (ZCCC) *ySize   *= 2;
           if (ZCCC) *zSize   /= 2;

           if (ZCCC) xOutSize /= 2;
           if (ZCCC) yOutSize *= 2;

           if (yAlt && *zQuadState == 2) yOutSize *= 2;

           if (*yQuadState == 2 && *zQuadState == 2) shufFlag = 1;
           if (*xQuadState == 1 && *yQuadState == 2 && *zQuadState == 1) shufFlag = 1;

           maxSize = 4*(*xSize)*(*ySize);
    
           if (*yQuadState == 2 || *zQuadState == 2)
              {
               if (yAlt && *zQuadState == 2)
                  (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );
               else
                  (void) setParm( fdata, FDQUADFLAG, (float)0.0, 0 );
              }
           else if (*yQuadState == 1 && *zQuadState == 1)
              {
               (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );
              }
 
           (void) setParm( fdata, FDDIMORDER1, (float)dim2, 0 );
           (void) setParm( fdata, FDDIMORDER2, (float)dim3, 0 );
           (void) setParm( fdata, FDDIMORDER3, (float)dim1, 0 );

           if (dim2 == 2)
              (void) setParm( fdata, FDTRANSPOSED, (float)0.0, 0 );
           else if (dim3 == 2)
              (void) setParm( fdata, FDTRANSPOSED, (float)1.0, 0 );

           thisDim  = dim2;
           break;

        case CUR_ADIM:
           *xSize      = size*(*xQuadState);
           *ySize      = oldZSize*(*quadState)/(*xQuadState);
           *zSize      = specnum;
           *aSize      = oldASize;

           outCount    = size*oldASize*(*xQuadState);
           xOutSize    = specnum;
           yOutSize    = oldZSize/(*zQuadState);
           zOutSize    = oldASize;
           aOutSize    = size*(*xQuadState);

           if (ACCCC) *ySize *= 2;
           if (ACCCC) *zSize /= 2;

           if (ACCRR) *ySize *= 2;
           if (ACCRR) *zSize /= 2;

           if (ACCCR) *ySize *= 2;
           if (ACCCR) *zSize /= 2;

           if (ACCRC) *ySize *= 2;
           if (ACCRC) *zSize /= 2;

           if (ACCRC) xOutSize /= 2;
           if (ACCRR) xOutSize /= 2;
           if (ARCCC) yOutSize *= 2;
           if (ARCCR) yOutSize *= 2;

           if (ACCCR) xOutSize /= 2;
           if (ACCCR) yOutSize *= 2;
           if (ACCCC) xOutSize /= 2;
           if (ACCCC) yOutSize *= 2;

           if (yAlt && *zQuadState == 2) yOutSize *= 2;

           if (*yQuadState == 2 && *aQuadState == 2) shufFlag = 1;
           if (*xQuadState == 1 && *yQuadState == 2 && *aQuadState == 1) shufFlag = 1;

           maxSize = 4*(*xSize)*(*ySize);

           if (*yQuadState == 2 || *zQuadState == 2)
              {
               if (yAlt && *zQuadState == 2)
                  (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );
               else
                  (void) setParm( fdata, FDQUADFLAG, (float)0.0, 0 );
              }
           else if (*yQuadState == 1 && *zQuadState == 1)
              {
               (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );
              }

           (void) setParm( fdata, FDDIMORDER1, (float)dim2, 0 );
           (void) setParm( fdata, FDDIMORDER2, (float)dim3, 0 );
           (void) setParm( fdata, FDDIMORDER3, (float)dim4, 0 );
           (void) setParm( fdata, FDDIMORDER4, (float)dim1, 0 );

           if (dim2 == 2)
              (void) setParm( fdata, FDTRANSPOSED, (float)0.0, 0 );
           else if (dim3 == 2)
              (void) setParm( fdata, FDTRANSPOSED, (float)1.0, 0 );

           thisDim  = dim2;
           break;

        default:
           return( 1 );
           break;
       }

/***/
/* Clip plane bounds if needed:
/***/

    if (xyzFlag == 1 || xyzFlag == 2)
       *maxPlane = outCount;
    else
       *maxPlane = *ySize;

    if (!*firstPlane) *firstPlane = 1;
    if (!*lastPlane)  *lastPlane  = *maxPlane;

    if (*firstPlane < 1) *firstPlane = 1;
    if (*firstPlane > *maxPlane) *firstPlane = *maxPlane;

    if (*lastPlane < 1) *lastPlane = 1;
    if (*lastPlane > *maxPlane) *lastPlane = *maxPlane;

     if (*firstPlane > *lastPlane)
        {
         i = *firstPlane; *firstPlane = *lastPlane; *lastPlane = i;
        }

    if (debugFlag)
       {
        FPR( stderr,
             "\nAdjusted First Plane: %d Last Plane: %d\n",
             *firstPlane, *lastPlane );

        (void) fflush( stderr );
       }

/***/
/* Save output sizes.
/* Remove restricted plane bounds.
/* Set parameters to indicate output format (single file, plane, cube).
/***/

    (void) setParm( fdata, NDSIZE,       (float)xOutSize, CUR_XDIM );
    (void) setParm( fdata, NDSIZE,       (float)yOutSize, CUR_YDIM );
    (void) setParm( fdata, NDSIZE,       (float)zOutSize, CUR_ZDIM );
    (void) setParm( fdata, NDSIZE,       (float)aOutSize, CUR_ADIM );

    (void) setParm( fdata, FDFIRSTPLANE, (float)0, 0 );
    (void) setParm( fdata, FDLASTPLANE,  (float)0, 0 );

    (void) setParm( fdata, FDPIPECOUNT,  (float)0,         0 );

    if (fmtCount == 0)
       {
        (void) setParm( fdata, FDFILECOUNT,  (float)1, 0 );
        (void) setParm( fdata, FDPIPEFLAG,   (float)1, 0 );
        (void) setParm( fdata, FDCUBEFLAG,   (float)0, 0 );
       }
    else if (fmtCount == 1 && *dimCount == 4)
       {
        (void) setParm( fdata, FDFILECOUNT,  (float)aOutSize,  0 );
        (void) setParm( fdata, FDPIPEFLAG,   (float)NULL_DIM,  0 );
        (void) setParm( fdata, FDCUBEFLAG,   (float)1,         0 );
       }
    else
       {
        (void) setParm( fdata, FDFILECOUNT,  (float)outCount, 0 );
        (void) setParm( fdata, FDPIPEFLAG,   (float)NULL_DIM, 0 );
        (void) setParm( fdata, FDCUBEFLAG,   (float)0,        0 );
       }

    (void) setFDATASliceCount( fdata, 0 );

/***/
/* If no memory size is specified, set a reasonable one:
/***/

    maxSize *= sizeof(float);

    if (!*megabytes)
       {
        *megabytes = 2 + maxSize/(1024*1024);
       }

    if (inPlace)
       {
        if (*megabytes < 2 + maxSize/(1024*1024))
           {
            *megabytes = 2 + maxSize/(1024*1024);
           }
       }

    if (*megabytes < 4) *megabytes = 4;

/***/
/* Local Exit point:
/***/

shutdown:

    return( error );
}

/***/
/* Special shuffling for X or Y output to Z/A when Y and Z/A are complex.
/***/

static int xy2zaShufN( mat, vecSize, vecCount, sR, sI )

   float *mat, *sR, *sI;
   int   vecSize, vecCount;
{
    float *ptrQ;
    int   iy, jump;

    if (!mat || vecSize < 1 || vecCount < 1) return( 0 );

    ptrQ = mat;
    jump = 2*vecSize;

    for( iy = 0; iy < vecCount; iy++ )
       {
        (void) xy2zaShuf( ptrQ, vecSize, sR, sI );
        ptrQ += jump;
       }

    return( 0 );
}

static int xy2zaShuf( qdata, size, sR, sI )

   float *qdata, *sR, *sI;
   int   size;
{
   float *rdata, *idata;
   int   i, k;

   rdata = qdata;
   idata = rdata + size;

   k = 0;

   for( i = 0; i < size; i++ )
      {
       sR[i] = qdata[k++];
       sI[i] = qdata[k++];
      }

   (void) vvCopy( rdata, sR, size );
   (void) vvCopy( idata, sI, size );

   return( 0 );
}

/***/
/* z2aList: rearrange 2D name list to swap Z and A dimensions.
/***/

static int z2aListN( nameListN, aSize, zSize )

   struct ZListInfo **nameListN;
   int    aSize, zSize;
{
    struct ZListInfo **tempList, **src, **dest;
    int    iz, ia;

    if (!(tempList = (struct ZListInfo **) voidAlloc( "z2a", sizeof(struct ZListInfo *)*zSize*aSize )))
       {
        return( 1 );
       }

    for( iz = 0; iz < zSize*aSize; iz++ ) tempList[iz] = nameListN[iz];

    dest = nameListN;

    for( iz = 0; iz < zSize; iz++ )
       {
        src = tempList + iz;

        for( ia = 0; ia < aSize; ia++ )
           {
            *dest++ = *src;
            src    += zSize;
           }
       }

    (void) deAlloc( "z2a", tempList, sizeof(struct ZListInfo *)*zSize*aSize );

    return( 0 );
}

/***/
/* c2riList2: adjust R/I alternating mixed Z/A Axis file list.
/***/

static int c2riList2N( nameListN, zSize, aSize )

   struct ZListInfo **nameListN;
   int    zSize, aSize;
{
    while( aSize-- )
       {
        if (c2riListN( nameListN, zSize )) return( 1 );
        nameListN += zSize;
       }

    return( 0 );
}

/***/
/* c2riList: adjust R/I alternating file list to continuous R, then I.
/***/

static int c2riListN( nameListN, nameCount )

   struct ZListInfo **nameListN;
   int    nameCount;
{
    struct ZListInfo **tempList;
    int    i, mid;

    if (!(tempList = (struct ZListInfo **) voidAlloc( "c2ri", sizeof(struct ZListInfo *)*nameCount )))
       {
        return( 1 );
       }

    mid = nameCount/2;

    for( i = 0; i < nameCount/2; i++ )
       {
        tempList[i]       = nameListN[2*i];
        tempList[i + mid] = nameListN[2*i + 1];
       }

    for( i = 0; i < nameCount; i++ )
       {
        nameListN[i] = tempList[i];
       }

    (void) deAlloc( "c2ri", tempList, sizeof(struct ZListInfo *)*nameCount );

    return( 0 );
}

/***/
/* pipeHead: parse for user-specified header parameters.
/***/

static int pipeHead( argc, argv, fdata )

   int   argc;
   char  **argv;
   float fdata[FDATASIZE];
{
    int   itemp, loc1, loc2;
    float rtemp;

    if ((loc1 = flagLoc( argc, argv, "-fdata" )))
       {
        loc2 = nextFlag( argc, argv, loc1 );

        while( loc1 < loc2 - 1 )
           {
            itemp = atoi( getNthArg( argc, argv, loc1+1 )) - 1;
            rtemp = ATOF( getNthArg( argc, argv, loc1+2 ));

            (void) setParm( fdata, itemp, rtemp, 0 );

            loc1 += 2;
           }
       }

    return( 0 );
}

/***/
/* getparams: extract command arguments for pipe2xyz.
/***/

static int getparams( argc, argv,
                      outTemplate, inName,
                      firstPlane, lastPlane, xyzFlag, fmtFlag,
                      inPlace, maxPlanesIn, megabytes, checkFlag, dirFlag )

   int  argc, *firstPlane, *lastPlane, *xyzFlag, *fmtFlag, *maxPlanesIn, *megabytes, *inPlace, *checkFlag, *dirFlag;
   char **argv, *outTemplate, *inName;
{
   int  dummyOvFlag;

/***/
/* Install defaults:
/***/

    verbose      = 0;
    vInc         = 16;

    *outTemplate = '\0';
    *inName      = '\0';

    *xyzFlag     = 1;
    *megabytes   = 0;
    *maxPlanesIn = 0;
    *firstPlane  = 0;
    *lastPlane   = 0;
    *inPlace     = 0;
    *checkFlag   = 1;
    *dirFlag     = 1;
    *fmtFlag     = FMT_NMRPIPE;

    autoFlag     = 1;
    delayTime    = getDefTimeout();
    maxTries     = -1;

/***/
/* Print a banner:
/***/

    if (argc == 1 || flagLoc( argc, argv, "-help" ))
       {
        FPR_VERSION;
        FPR( stderr, "Write an NMRPipe Pipeline Stream to an NMRPipe 3D-4D File Series.\n" );
        FPR( stderr, " ... nmrPipe ... | %s -out outTemplate\n", argv[0] );
        FPR( stderr, "Output Order Arguments (Choose Only One):\n" );
        FPR( stderr, " -x                   Output Vectors to X-Axis XYZA (Default).\n" );
        FPR( stderr, " -y                   Output Vectors to Y-Axis YXZA.\n" );
        FPR( stderr, " -z                   Output Vectors to Z-Axis YZXA.\n" );
        FPR( stderr, " -a                   Output Vectors to A-Axis YZAX.\n" );
        FPR( stderr, " -nv                  Output in NMRView Format.\n" );
        FPR( stderr, "Other Optional Arguments:\n" );
        FPR( stderr, " -in    inName        Input Name.\n" );
        FPR( stderr, " -fdata loc val ...   Adjust File Header.\n" );
        FPR( stderr, " -first firstPlane    First Plane to Process.\n");
        FPR( stderr, " -last  lastPlane     Last Plane to Process.\n");
        FPR( stderr, " -inPlace             In-Place Processing; Implies -nofs option.\n" );
        FPR( stderr, " -notrunc             Suppress Truncation of Files Larger than Header Shows.\n" ); 
        FPR( stderr, " -nocreat             Suppress Truncation of Files During Parallel Processing.\n" ); 
        FPR( stderr, " -nodir               Suppress Directory Creation.\n" );
        FPR( stderr, " -help                Prints This Text.\n" );
        FPR( stderr, " -shuf                Force X/Y to Z/A Shuffling ON.\n" );
        FPR( stderr, " -noshuf              Force X/Y to Z/A Shuffling OFF.\n" );
        FPR( stderr, " -verb                Verbose Mode On.\n" );
        FPR( stderr, " -inc   vInc [16]     Verbose Message Increment.\n" );
        FPR( stderr, "Suggested Memory Buffer Size:\n" );
        FPR( stderr, " -maxPlanes mp [0]    Max Input Planes (Use Zero for Auto, -1 for All).\n" );
        FPR( stderr, " -mem       mb [0]    Buffer Size, MB (Use Zero for Auto).\n" );
        FPR( stderr, "Stream I/O Control:\n" );
        FPR( stderr, " -nofs                No File Size Check;\n" );
        FPR( stderr, "                      Default for Processing a Sub-set of the Data.\n" ); 
#ifdef DATAIO_USE_MAX_TRIES
        FPR( stderr, " -mt maxTries [-1]    Maximum Retry Count for I/O.\n" );
#else
        FPR( stderr, " -mt maxTries [-1]    Currently Ignored.\n" );
#endif
        FPR( stderr, "Override NMR_IO_SELECT environment variable:\n" );
        FPR( stderr, " -ioSelect            Use select() for Blocking I/O.\n" );
        FPR( stderr, " -noioSelect          Suppress use of select().\n" );
        FPR( stderr, "Override NMR_IO_TIMEOUT environment variable:\n" );
        FPR( stderr, " -to uSecTimeOut [%d]  Delay Between I/O Retries.\n", DATAIO_TIMEOUT );
        FPR( stderr, "Output Template Examples:\n" );
        FPR( stderr, " test.dat             3D or 4D Stream                (Single File).\n" );
        FPR( stderr, " test%%03d.ft3         3D Z Format test001.ft3 ...    (2D Series).\n" );
        FPR( stderr, " test%%02d%%03d.ft4     4D AZ Format test01001.ft4 ... (2D Series).\n" );
        FPR( stderr, " test%%03d.ft4         4D Cube Format test001.ft4 ... (3D Series)\n" );
        FPR( stderr, "Notes:\n" );
        FPR( stderr, " 1. If the output template name begins with a directory, the\n" );
        FPR( stderr, "    directory will be created if it does not exist.\n" );
        FPR( stderr, " 2. Not all in-place or parallel processing schemes are compatible\n" );
        FPR( stderr, "    with single-file output or 4D cube format output.\n" );
        return( 1 );
       }

/***/
/* Extract the parameters:
/***/

    if (flagLoc( argc, argv, "-notrunc" )) truncFlag = 0; 
    if (flagLoc( argc, argv, "-nocreat" )) creatFlag = 0; 

    if (!flagLoc( argc, argv, "-out" )) return( 1 );

    (void) strArgD( argc, argv, "-out", outTemplate );
    (void) strArgD( argc, argv, "-in",  inName );

    (void) intArgD( argc, argv, "-mem",       megabytes );
    (void) intArgD( argc, argv, "-maxPlanes", maxPlanesIn );
    (void) intArgD( argc, argv, "-first",     firstPlane );
    (void) intArgD( argc, argv, "-last",      lastPlane );

    (void) intArgD( argc, argv, "-dbg", &debugFlag );
    (void) intArgD( argc, argv, "-to",  &delayTime );
    (void) intArgD( argc, argv, "-mt",  &maxTries );

    if (flagLoc( argc, argv, "-x" )) *xyzFlag = 1;
    if (flagLoc( argc, argv, "-y" )) *xyzFlag = 2;
    if (flagLoc( argc, argv, "-z" )) *xyzFlag = 3;
    if (flagLoc( argc, argv, "-a" )) *xyzFlag = 4;

    if (flagLoc( argc, argv, "-ov" ))   dummyOvFlag = 1;
    if (flagLoc( argc, argv, "-noov" )) dummyOvFlag = 0;

    if (flagLoc( argc, argv, "-nv" ))
       {
        *fmtFlag = FMT_NMRVIEW;
        *xyzFlag = 1;
       }

    if (flagLoc( argc, argv, "-inPlace" ))
       {
        *inPlace   = 1;
        *checkFlag = 0;
       }

    if (flagLoc( argc, argv, "-ioSelect" ))   (void) setIOSelect( 1 );
    if (flagLoc( argc, argv, "-noioSelect" )) (void) setIOSelect( 0 );

    if (flagLoc( argc, argv, "-nodir" ))   *dirFlag   = 0;
    if (flagLoc( argc, argv, "-nofs" ))    *checkFlag = 0;

    if (flagLoc( argc, argv, "-verb" ))    verbose   = 1;
    if (flagLoc( argc, argv, "-noverb" ))  verbose   = 0;

    (void) intArgD( argc, argv, "-inc",    &vInc );

    if (flagLoc( argc, argv, "-oldFmt" ))   oldFmtFlag = 1;
    if (flagLoc( argc, argv, "-newFmt" ))   oldFmtFlag = 0;

    if (flagLoc( argc, argv, "-nooldFmt" )) oldFmtFlag = 0;
    if (flagLoc( argc, argv, "-nonewFmt" )) oldFmtFlag = 1;

    if (*megabytes) autoFlag = 0;

    return( 0 );
}

/***/
/* Bottom.
/***/

