/***/
/*   NMRPipe Copyright 1992-2018 Frank Delaglio
/***/

/***/
/* xyz2Pipe: converts NMRPipe-format input to an NMRPipe stream of vectors.
/***/

#define ETEST_PROG_NAME "XYZ2Pipe"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "version.h"
#include "cmndargs.h"
#include "memory.h"
#include "testsize.h"
#include "dataio.h"
#include "fdatap.h"
#include "nmrserver.h"
#include "inquire.h"
#include "etest.h"
#include "prec.h"
#include "atof.h"
#include "vutil.h"

static int getparams(), getPipePlanes(), xyz2pipe(), xyzHead(), xyz2pipeFD();
static int c2riList2N(), c2riListN(), z2aListN();
static int xyzri2c(), ri2c();
static int xyz2x(), xyz2y(), xyz2z();
static int sendNullFD();

static int debugFlag = 0;
static int ri2cFlag  = 0;
static int verbose   = 0;
static int vInc      = 16;

#define FPR (void) fprintf

int main( argc, argv )

   int  argc;
   char **argv;

{
    int    xSize, ySize, zSize, aSize, xQuadState, yQuadState, zQuadState, aQuadState, planeCount, xyzFlag,
           fmtCount, megabytes, firstPlane, lastPlane, maxPlanesIn, checkFlag, wrFlag, hdrFlag, parFlag,
           nX, nY, nZ, nA, nQ, nDim, cpuID, error;

    char   inTemplate[NAMELEN+1], outName[NAMELEN+1], serverListName[NAMELEN+1];

    struct ZListInfo **inListN;

    float  fdata[FDATASIZE];
    UNIT   outUnit;


/***/
/* Initialize:
/***/

    error    = 0;
    inListN  = (struct ZListInfo **)NULL;
    outUnit  = UNIT_NULL;
    nDim     = 0;
    nZ       = 0;
    nA       = 0;

    fmtCount    = 0;
    megabytes   = 0;
    maxPlanesIn = 0;

    (void) initArgs( argc, argv, CHECK_ENV );
    (void) initDataIO();

/***/
/* Get command arguments.
/* Create 2D series plane input list.
/* Adjust file parameters for 2D output stream. 
/* Parse User suggestions for header if any.
/* Perform final argument checking.
/* Create the output stream.
/***/

    error = getparams( argc, argv,
                       inTemplate, outName, serverListName,
                       &cpuID, &firstPlane, &lastPlane, &maxPlanesIn, &megabytes, &xyzFlag,
                       &checkFlag, &wrFlag, &hdrFlag, &parFlag );

    if (error)
       {
        FPR( stderr, "XYZ2Pipe Error extracting arguments.\n" );

        (void) sendNullFD( outName, fdata );
        goto shutdown;
       }

    if ((error = readHdrN( inTemplate, fdata )))
       {
        FPR( stderr, "XYZ2Pipe Error getting file list;\n" );
        FPR( stderr, "Check for valid input name or template.\n" );
        FPR( stderr, "Check that input data exists.\n" );

        (void) sendNullFD( outName, fdata );
        goto shutdown;
       }

    (void) fixfdata( fdata );

    fmtCount = getFmtCount( inTemplate );    

    nDim = getParmI( fdata, FDDIMCOUNT, NULL_DIM );
    nX   = getParmI( fdata, NDSIZE,     CUR_XDIM );
    nY   = getParmI( fdata, NDSIZE,     CUR_YDIM );
    nZ   = getParmI( fdata, NDSIZE,     CUR_ZDIM );
    nA   = getParmI( fdata, NDSIZE,     CUR_ADIM );

    nQ   = getQuad( fdata, FDQUADFLAG, NULL_DIM );

    if ((error = getZListN( inTemplate, nZ, nA, nDim, &inListN )))
       {
        FPR( stderr, "XYZ2Pipe Error building file list.\n" );
        (void) sendNullFD( outName, fdata );
        goto shutdown;
       }

    planeCount = nZ*nA;

    error = xyz2pipeFD( inListN, planeCount, fmtCount,
                        outName, &outUnit, serverListName, cpuID,
                        &xQuadState, &yQuadState, &zQuadState, &aQuadState,
                        xyzFlag, &xSize, &ySize, &zSize, &aSize, &firstPlane, &lastPlane, &megabytes,
                        fdata, checkFlag, hdrFlag );

    if (error)
       {
        FPR( stderr, "XYZ2Pipe Error setting file parameters.\n" );
        (void) sendNullFD( outName, fdata );
        goto shutdown;
       }

    if ((error = xyzHead( argc, argv, fdata )))
       {
        FPR( stderr, "XYZ2Pipe Error setting user header.\n" );
        goto shutdown;
       }

    if ((error = dataWrite( outUnit, fdata, sizeof(float)*FDATASIZE )))
       {
        FPR( stderr, "XYZ2Pipe Error writing header.\n" );
        goto shutdown;
       }

    (void) checkArgs( argc, argv );

    if (wrFlag)
       {
        error = xyz2pipe( inTemplate, inListN, fmtCount, outUnit,
                          xyzFlag, nDim, xSize, ySize, zSize, aSize,
                          nX, nY, nZ, nA, nQ, xQuadState, yQuadState,
                          firstPlane, lastPlane, maxPlanesIn, megabytes );

        if (error)
           {
            FPR( stderr, "XYZ2Pipe Error building stream.\n" );
            goto shutdown;
           }
       }
    else 
       {
        if (verbose) FPR( stderr, "XYZ2Pipe: No-Write Mode.\n" );
       }

/***/
/* Exit point:
/***/

shutdown:

    if (error) FPR( stderr, "XYZ2Pipe Status: %d\n", error );

    if (*outName) (void) dataClose( outUnit );

    (void) freeZListN( nZ, nA, nDim, inListN );

    (void) showDebugIO( argc, argv );

    return( error );
}

/***/
/* xyz2pipe: write XYZ data as vector stream from specified dimension.
/***/

static int xyz2pipe( inTemplate, inListN, fmtCount, outUnit,
                     xyzFlag, dimCount, xSize, ySize, zSize, aSize,
                     nX, nY, nZ, nA, nQ, xQuadState, yQuadState,
                     firstPlane, lastPlane, maxPlanesIn, megabytes, verb )

   int    fmtCount, xSize, ySize, zSize, aSize, nX, nY, nZ, nA, nQ, xQuadState, yQuadState, megabytes, 
          firstPlane, lastPlane, maxPlanesIn, xyzFlag, verb;

   struct ZListInfo **inListN;
   char   *inTemplate;
   UNIT   outUnit;
{
    int    error;

    switch( xyzFlag )
       {
        case 1:
           error = xyz2x( inListN, outUnit, 
                          fmtCount, dimCount, xSize, ySize, zSize, aSize, 
                          firstPlane, lastPlane, verb );

           break;

        case 2:
           error = xyz2y( inListN, outUnit, 
                          fmtCount, dimCount, xSize, ySize, zSize, aSize, xQuadState, yQuadState,
                          firstPlane, lastPlane, verb );

           break;

        case 3: case 4:
           error = xyz2z( inListN, outUnit,
                          fmtCount, dimCount, xSize, ySize, zSize, aSize, 
                          nX, nY, nZ, nA, nQ, firstPlane, lastPlane, maxPlanesIn, megabytes, 
                          xyzFlag, verb );

           break;

        default:
           error = 1;
           break;
       }

    return( error );
}

/***/
/* xyz2x: write XYZ data as X-vector stream in XYZ order.
/***/

static int xyz2x( inListN, outUnit, fmtCount, dimCount, xSize, ySize, zSize, aSize, firstPlane, lastPlane )

   int    fmtCount, dimCount, xSize, ySize, zSize, aSize, firstPlane, lastPlane;
   struct ZListInfo **inListN;
   UNIT   outUnit;
{
    NMR_INT planeSize, planeJump, cubeJump;
    int     planeCount, ip, ipp, error;
    float   *matrix2d;
    UNIT    inUnit;
    struct  ZListInfo *inPtr;

    char    currentName[NAMELEN+1];
    NMR_INT thisOffset, currentOffset;

/***/
/* Initialize:
/***/

    error    = 0;
    matrix2d = (float *) NULL;
    inUnit   = UNIT_NULL;

    currentName[0] = '\0';
    currentOffset  = (NMR_INT)0;

/***/
/* Allocate memory:
/***/

    planeSize  = (NMR_INT)xSize*ySize;
    planeJump  = (NMR_INT)xSize*ySize;
    cubeJump   = planeJump*zSize;
    planeCount = zSize*aSize;

    if (!(matrix2d = fltAlloc( "xyz2x", planeSize )))
       {
        FPR( stderr, "XYZ2X Error allocating memory.\n" );
        error = 1;
        goto shutdown;
       }
 
/***/
/* For each plane:
/*   Read XY region.
/*   Write XY region.
/***/

    for( ip = firstPlane - 1; ip < lastPlane; ip++ )
       {
        inPtr = inListN[ip];
        ipp   = ip + 1;

        if (verbose && ((ipp == firstPlane) || !(ipp % vInc) || (ipp == lastPlane)))
           {
            FPR( stderr, "XY Plane: %d of %d\015", ip + 1, planeCount );
            (void) fflush( stderr );
           }

        if (fmtCount == 0)
           {
            thisOffset = (NMR_INT)FDATASIZE + planeJump*inPtr->zOff + cubeJump*inPtr->aOff;
           }
        else if (fmtCount == 1 && dimCount == 4)
           {
            thisOffset = (NMR_INT)FDATASIZE + planeJump*inPtr->zOff;
           }
        else
           {
            thisOffset = (NMR_INT)FDATASIZE;
           }
 
        if (strcmp( currentName, inPtr->name ))
           {
            if (inUnit)
               {
                ETEST( 1, dataClose( inUnit ));

                currentOffset  = (NMR_INT)0;
                currentName[0] = '\0';
                inUnit         = UNIT_NULL;
               }

            ETEST( 2, dataOpen( inPtr->name, &inUnit, FB_READ ));

            (void) strcpy( currentName, inPtr->name );
            currentOffset = (NMR_INT)0;
           }

        if (thisOffset != currentOffset)
           {
            ETEST( 3, dataPos( inUnit, sizeof(float)*thisOffset ));
            currentOffset = thisOffset;
           }

        ETEST( 4, dataRead( inUnit, matrix2d, sizeof(float)*planeSize ));

        currentOffset += planeSize;

        ETEST( 5, dataWrite( outUnit, matrix2d, sizeof(float)*planeSize ));
       }

    if (verbose) FPR( stderr, "\n" );

/***/
/* Local exit point:
/***/

shutdown:

    if (inUnit)
       {
        ETEST( 666, dataClose( inUnit ));

        currentOffset  = (NMR_INT)0;
        currentName[0] = '\0';
        inUnit         = UNIT_NULL;
       }

    (void) deAlloc( "xyz2x", matrix2d, sizeof(float)*planeSize );

    return( error );
}

/***/
/* xyz2y: write XYZ data as Y-vector stream in YXZ order.
/***/

static int xyz2y( inListN, outUnit, 
                  fmtCount, dimCount, xSize, ySize, zSize, aSize, xQuadState, yQuadState, 
                  firstPlane, lastPlane )

   int    fmtCount, dimCount, xSize, ySize, zSize, aSize, firstPlane, lastPlane, yQuadState;
   struct ZListInfo **inListN;
   UNIT   outUnit;
{
    NMR_INT planeSize, planeJump, cubeJump;
    int     ip, ipp, planeCount, error;
    float   *matrix2d;
    UNIT    inUnit;
    struct  ZListInfo *inPtr;

    int tp2DMMM();

    char    currentName[NAMELEN+1];
    NMR_INT thisOffset, currentOffset;

/***/
/* Initialize:
/***/

    error    = 0;
    matrix2d = (float *) NULL;
    inUnit   = UNIT_NULL;

    currentName[0] = '\0';
    currentOffset  = (NMR_INT)0;

/***/
/* Allocate input memory (TP function allocates its own work buffer):
/***/

    planeSize  = (NMR_INT)xSize*ySize*xQuadState*yQuadState;
    planeJump  = (NMR_INT)xSize*ySize*xQuadState*yQuadState;
    cubeJump   = planeJump*zSize;
    planeCount = zSize*aSize;

    if (!(matrix2d = fltAlloc( "xyz2y", planeSize )))
       {
        FPR( stderr, "XYZ2Y Error allocating memory.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* For each plane:
/*   Read the plane.
/*   Transpose from XY to YX order.
/*   Write the plane.
/***/

    for( ip = firstPlane - 1; ip < lastPlane; ip++ )
       {
        inPtr = inListN[ip];
        ipp   = ip + 1;

        if (verbose && ((ipp == firstPlane) || !(ipp % vInc) || (ipp == lastPlane)))
           {
            FPR( stderr, "YX Plane: %d of %d\015", ip + 1, planeCount );
            (void) fflush( stderr );
           }

        if (fmtCount == 0)
           {
            thisOffset = (NMR_INT)FDATASIZE + planeJump*inPtr->zOff + cubeJump*inPtr->aOff;
           }
        else if (fmtCount == 1 && dimCount == 4)
           {
            thisOffset = (NMR_INT)FDATASIZE + planeJump*inPtr->zOff;
           }
        else
           {
            thisOffset = (NMR_INT)FDATASIZE;
           }

        if (debugFlag)
           {
            FPR( stderr, "\nYX File %s Offset %ld\n", inPtr->name, (long)thisOffset );
            (void) fflush( stderr );
           }

        if (strcmp( currentName, inPtr->name ))
           {
            if (inUnit)
               {
                ETEST( 1, dataClose( inUnit ));

                currentOffset  = (NMR_INT)0;
                currentName[0] = '\0';
                inUnit         = UNIT_NULL;
               }

            ETEST( 2, dataOpen( inPtr->name, &inUnit, FB_READ ));

            (void) strcpy( currentName, inPtr->name );
            currentOffset = (NMR_INT)0;
           }

        if (thisOffset != currentOffset)
           {
            ETEST( 3, dataPos( inUnit, sizeof(float)*thisOffset ));
            currentOffset = thisOffset;
           }

        ETEST( 4, dataRead( inUnit, matrix2d, sizeof(float)*planeSize ));

        currentOffset += planeSize;

        ETEST( 5, tp2DMMM( matrix2d, xSize, ySize, xQuadState*yQuadState ));
        ETEST( 6, dataWrite( outUnit, matrix2d, sizeof(float)*planeSize ));
       }

/***/
/* Local exit point:
/***/

shutdown:

    if (verbose) FPR( stderr, "\n" );

    if (inUnit)
       {
        ETEST( 666, dataClose( inUnit ));

        currentOffset  = (NMR_INT)0;
        currentName[0] = '\0';
        inUnit         = UNIT_NULL;
       }

    (void) deAlloc( "xyz2y", matrix2d, sizeof(float)*planeSize );

    return( error );
}

/***/
/* xyz2z: write XYZ data as Z-vector stream in ZXY order.
/***/

static int xyz2z( inListN, outUnit,
                  fmtCount, dimCount, xSize, ySize, zSize, aSize, 
                  nX, nY, nZ, nA, nQ, firstPlane, lastPlane, maxPlanesIn, megabytes, 
                  xyzFlag )

   int    fmtCount, dimCount, xSize, ySize, zSize, aSize, nX, nY, nZ, nA, nQ, megabytes,
          xyzFlag, firstPlane, lastPlane;

   struct ZListInfo **inListN;
   UNIT   outUnit;
{
    NMR_INT totalMem, planeSize, planeJump, cubeJump, length3d, vecOffset, ptsIn;
    int     ix, iy, iz, ia, n, planesIn, planesOut, planesLeft, yPartSize, error;
    float   *zPtr, *ddPtr, *dddPtr, *matrix2d, *matrix3d, *riWork;
    struct  ZListInfo **listPtrN, *inPtr;
    UNIT    inUnit;

    char    currentName[NAMELEN+1];
    NMR_INT currentOffset, thisOffset;

/***/
/* Initialize:
/***/

    error    = 0;
    matrix2d = (float *)NULL;
    matrix3d = (float *)NULL;
    riWork   = (float *)NULL;
    ddPtr    = (float *)NULL;
    dddPtr   = (float *)NULL;

    currentOffset  = (NMR_INT)0;
    currentName[0] = '\0';
    inUnit         = UNIT_NULL;

/***/
/* Allocate and partition memory:
/***/

    yPartSize = 1 + lastPlane - firstPlane;
    totalMem  = (NMR_INT)megabytes*1024*1024/sizeof(float);
    planeSize = (NMR_INT)xSize*zSize;
    planeJump = (NMR_INT)nX*nY*nQ;
    cubeJump  = planeJump*nZ;

    n = totalMem/planeSize - 1;

    if (maxPlanesIn == 0) maxPlanesIn = 4;
    if (maxPlanesIn < 1)  maxPlanesIn = yPartSize;
    if (maxPlanesIn < 2)  maxPlanesIn = 2;
    if (maxPlanesIn < n)  maxPlanesIn = n;

    maxPlanesIn = maxPlanesIn < yPartSize ? maxPlanesIn : yPartSize; 
    length3d  = (NMR_INT)maxPlanesIn*planeSize;

    if (maxPlanesIn < 1)
       {
        FPR( stderr, "XYZ2Z Error: insufficient memory size.\n" );
        error = 1;
        goto shutdown;
       }

    if (!(matrix3d = fltAlloc( "xyz2z", length3d )))
       {
        FPR( stderr, "XYZ2Z Error allocating 3D memory.\n" );
        error = 1;
        goto shutdown;
       }

    if (!(matrix2d = fltAlloc( "xyz2z", planeSize )))
       {
        FPR( stderr, "XYZ2Z Error allocating 2D memory.\n" );
        error = 1;
        goto shutdown;
       }

    if (ri2cFlag)
       {
        if (!(riWork = fltAlloc( "xyz2z", xSize )))
           {
            FPR( stderr, "XYZ2Z Error allocating RI2C memory.\n" );
            error = 1;
            goto shutdown;
           }
       }

/***/
/* For each 3D cube:
/*   For each group of XZ planes:
/*     Read series of X vectors from each file.
/*     For each Z plane in group, move X vectors into 2D matrix in ZX order.
/*     Write 2D matrix to output stream.
/***/

     listPtrN = inListN;

     for( ia = 0; ia < aSize; ia++ )
        {
         vecOffset  = (NMR_INT)FDATASIZE + ((NMR_INT)firstPlane - 1)*xSize;
         vecOffset  = (NMR_INT)(firstPlane - 1);
         planesLeft = yPartSize;
         planesOut  = firstPlane;

         while( planesLeft )
            {
             planesIn = (planesLeft < maxPlanesIn) ? planesLeft : maxPlanesIn;
             ptsIn    = (NMR_INT)planesIn*xSize;
             dddPtr   = matrix3d;

             for( iz = 0; iz < zSize; iz++ )
                {
                 inPtr = listPtrN[iz];

                 if (strcmp( currentName, inPtr->name ))
                    {
                     if (inUnit)
                        {
                         ETEST( 1, dataClose( inUnit ));
         
                         currentOffset  = (NMR_INT)0;
                         currentName[0] = '\0';
                         inUnit         = UNIT_NULL;
                        }

                     ETEST( 2, dataOpen( inPtr->name, &inUnit, FB_READ ));

                     (void) strcpy( currentName, inPtr->name );
                     currentOffset = (NMR_INT)0;
                    }

                 if (fmtCount == 0)
                    {
                     if (xyzFlag == 3)
                        thisOffset = (NMR_INT)FDATASIZE + cubeJump*inPtr->aOff + planeJump*inPtr->zOff + vecOffset*xSize;
                     else
                        thisOffset = (NMR_INT)FDATASIZE + cubeJump*inPtr->aOff + planeJump*inPtr->zOff + vecOffset*xSize;
                    }
                 else if (fmtCount == 1 && dimCount == 4)
                    {
                     if (xyzFlag == 3)
                        thisOffset = (NMR_INT)FDATASIZE + planeJump*inPtr->zOff + vecOffset*xSize;
                     else
                        thisOffset = (NMR_INT)FDATASIZE + planeJump*inPtr->zOff + vecOffset*xSize;
                    }
                 else
                    {
                     thisOffset = (NMR_INT)FDATASIZE + vecOffset*xSize;
                    }

                 if (thisOffset != currentOffset)
                    {
                     ETEST( 3, dataPos( inUnit, sizeof(float)*thisOffset ));
                     currentOffset = thisOffset;
                    }

                 ETEST( 4, dataRead( inUnit, dddPtr, sizeof(float)*ptsIn ));

                 currentOffset += ptsIn;

                 if (ri2cFlag)
                    {
                     (void) xyzri2c( dddPtr, xSize/2, planesIn, riWork );
                    }

                 dddPtr += ptsIn;
                }

             dddPtr = matrix3d;
    
             for( iy = 0; iy < planesIn; iy++ )
                {
                 ddPtr = matrix2d;
      
                 for( ix = 0; ix < xSize; ix++ )
                    {
                     zPtr = dddPtr++;
    
                     for( iz = 0; iz < zSize; iz++ )
                        {
                         *ddPtr++  = *zPtr;
                         zPtr     += ptsIn;
                        }
                    }
    
                 if (verbose && ((planesOut == firstPlane) || !(planesOut % vInc) || (planesOut == lastPlane)))
                    {
                     if (xyzFlag == 3)
                        {
                         if (aSize > 1)
                            FPR( stderr, "ZX Plane: %3d of %d A: %3d of %d\015", planesOut, ySize, ia + 1, aSize );
                         else
                            FPR( stderr, "ZX Plane: %d of %d\015", planesOut, ySize );
                        }
                     else
                        {
                         FPR( stderr, "AX Plane: %3d of %d Z: %3d of %d\015", planesOut, ySize, ia + 1, aSize );
                        }
 
                     (void) fflush( stderr );
                    }
    
                 ETEST( 5, dataWrite( outUnit, matrix2d, sizeof(float)*planeSize ));

                 planesOut++;
                }

             planesLeft -= planesIn;
             vecOffset  += planesIn;
            }

         listPtrN += zSize;
        }

/***/
/* Local Exit point:
/***/

shutdown:

    if (verbose) FPR( stderr, "\n" );

    if (inUnit)
       {
        ETEST( 666, dataClose( inUnit ));

        currentOffset  = (NMR_INT)0;
        currentName[0] = '\0';
        inUnit         = UNIT_NULL;
       }

    (void) deAlloc( "xyz2z", matrix2d, planeSize );
    (void) deAlloc( "xyz2z", matrix3d, length3d );
    (void) deAlloc( "xyz2z", riWork,   xSize );

#ifdef S_SPLINT_S
    (void) free( (void *)ddPtr );
#endif

    return( error );
}

/***/
/* xyzri2c: interleave a group of seperated real/imaginary vectors.
/***/

static int xyzri2c( matrix, length, count, work )

   int   length, count;
   float *matrix, *work;
{
    float *rdata, *idata;

    rdata = matrix;
    idata = rdata + length;

    while( count-- )
       {
        (void) ri2c( rdata, idata, work, length );
        (void) vvCopy( rdata, work, 2*length );

        rdata += 2*length;
        idata += 2*length;
       }

    return( 0 );
}

/***/
/* ri2c: interleave elements from separate real and imag vectors.
/***/

static int ri2c( rdata, idata, cdata, length )

   float *rdata, *idata, *cdata;
   int   length;
{
    cdata += 2*length - 1;
    rdata += length   - 1;
    idata += length   - 1;

    while( length-- )
       {
        *cdata-- = *idata--;
        *cdata-- = *rdata--;
       }

    return( 0 );
}

/***/
/* xyz2pipeFD: analyze and prepare file unit and header for ND output stream.
/***/

static int xyz2pipeFD( inListN, planeCount, fmtCount,
                       outName, outUnitPtr, serverListName, cpuID,
                       xQuadState, yQuadState, zQuadState, aQuadState,
                       xyzFlag, xSize, ySize, zSize, aSize,
                       firstPlane, lastPlane, megabytes,
                       fdata, checkFlag, hdrFlag )

   int    *xQuadState, *yQuadState, *zQuadState, *aQuadState, *xSize, *ySize,
          *zSize, *aSize, *firstPlane, *lastPlane, *megabytes, planeCount, fmtCount,
          xyzFlag, checkFlag, hdrFlag, cpuID;

   struct ZListInfo **inListN;
   char   *outName, *serverListName;
   float  fdata[FDATASIZE];
   UNIT   *outUnitPtr;
{
    int   masterSize, masterSpecnum, masterQuadState, masterScaleFlag, 
          size, specnum, quadState, xyQuadState, trueZSize, trueASize, 
          dim1, dim2, dim3, dim4, xOutSize, yOutSize, zOutSize, aOutSize, 
          xAlt, yAlt, firstFile, lastFile, i, maxPlane, quad1, quad2,
          finalQuad, itemp, error;

    NMR_INT maxSize64, sliceCount64;

    float maxVal, minVal;
    UNIT  inUnit;


/***/
/* Convenient notation for complex state of 3D data:
/***/

#define ZCRC (*xQuadState == 2 && *yQuadState == 1 && *zQuadState == 2)
#define ZCCR (*xQuadState == 2 && *yQuadState == 2 && *zQuadState == 1)
#define ZCCC (*xQuadState == 2 && *yQuadState == 2 && *zQuadState == 2)

#define ACRC (*xQuadState == 2 && *yQuadState == 1 && *aQuadState == 2)
#define ACCR (*xQuadState == 2 && *yQuadState == 2 && *aQuadState == 1)
#define ACCC (*xQuadState == 2 && *yQuadState == 2 && *aQuadState == 2)

/***/
/* Initialize:
/***/

    error    = 0;
    inUnit   = UNIT_NULL;

/***/
/* Establish start and end files:
/*  For X and Y processing, these can be a limited subset of all files.
/*
/*  For Z or A processing, all files must exist; the plane limits for
/*  these cases will be recalculated later to indicate the subset of data
/*  within each file that will be processed.
/***/

    if (xyzFlag == 1 || xyzFlag == 2)
       {
        error = getPipePlanes( serverListName, cpuID, firstPlane, lastPlane, planeCount );

        if (error)
           {
            FPR( stderr, "XYZ2Pipe Error Extracting Limits.\n" );
            return( error );
           }

        firstFile = *firstPlane;
        lastFile  = *lastPlane;
        maxPlane  = planeCount;
       }
    else
       {
        firstFile = 1;
        lastFile  = planeCount;
       }

    maxVal          = getParm( fdata, FDMAX,       0 );
    minVal          = getParm( fdata, FDMIN,       0 );
    masterScaleFlag = getParm( fdata, FDSCALEFLAG, 0 );

    masterSize      = getParm( fdata, NDSIZE, CUR_XDIM );
    masterSpecnum   = getParm( fdata, NDSIZE, CUR_YDIM );
    trueZSize       = getParm( fdata, NDSIZE, CUR_ZDIM );
    trueASize       = getParm( fdata, NDSIZE, CUR_ADIM );

    if (1 == (int) getParm( fdata, FDQUADFLAG, 0 ))
       masterQuadState = 1;
    else
       masterQuadState = 2; 

/***/
/* Suppress all but first of header check if indicated:
/*  Extract and test remainder of file headers for consistancy:
/***/

    if (!hdrFlag) lastFile = firstFile;

    for( i = firstFile - 1; i < lastFile; i++ )
       {
        if ((error = readHdrN( (inListN[i])->name, fdata )))
           {
            FPR( stderr, "XYZ2Pipe Error Reading Header from file %s\n", (inListN[i])->name );
            return( 1 );
           }

        (void) fixfdata( fdata );

        if (checkFlag)
           {
            switch( testNMRin( (inListN[i])->name, fdata, 0, 1 ))
               {
                case FILE_SIZE_FAIL:
                   FPR( stderr, "XYZ2Pipe Warning:" );
                   FPR( stderr, " can't check size of file" );
                   FPR( stderr, " %s\n\n", (inListN[i])->name );
                   break;
                case  FILE_SIZE_TOO_SMALL:
                   FPR( stderr, "XYZ2Pipe Error:" );
                   FPR( stderr, " header/size mismatch for file" );
                   FPR( stderr, " %s (file too small)\n", (inListN[i])->name );
                   return( 1 );
                   break;
                case  FILE_SIZE_TOO_LARGE:
                   FPR( stderr, "XYZ2Pipe Warning:" );
                   FPR( stderr, " header/size mismatch for file" );
                   FPR( stderr, " %s (file too large)\n", (inListN[i])->name );
                   break;
                default:
                   break;
               }   
           } 

        maxVal    = getParm( fdata, FDMAX, 0 );
        minVal    = getParm( fdata, FDMIN, 0 );
 
        size      = getParm( fdata, NDSIZE, CUR_XDIM );
        specnum   = getParm( fdata, NDSIZE, CUR_YDIM );
        quadState = getQuad( fdata, FDQUADFLAG, 0 );

        if (size != masterSize || specnum != masterSpecnum || quadState != masterQuadState)
           {
            FPR( stderr, "XYZ2Pipe Error: bad sizes %s\n", (inListN[0])->name );
            return( 1 );
           }
       }

/***/
/* Set parameters to indicate output is a condensed 2D file of ND data.
/* Decode complex state of various dimensions.
/***/

    (void) setParm( fdata, FDPIPEFLAG,   (float)xyzFlag, 0 );
    (void) setParm( fdata, FDCUBEFLAG,   (float)0,       0 );
    (void) setParm( fdata, FDFILECOUNT,  (float)1,       0 );
    (void) setParm( fdata, FDPIPECOUNT,  (float)0,       0 );

    *xQuadState = getQuad( fdata, NDQUADFLAG, CUR_XDIM );
    *yQuadState = getQuad( fdata, NDQUADFLAG, CUR_YDIM );
    *zQuadState = getQuad( fdata, NDQUADFLAG, CUR_ZDIM );
    *aQuadState = getQuad( fdata, NDQUADFLAG, CUR_ADIM );

    xAlt = 0;
    yAlt = 0;

    if (0 == (int)getParm( fdata, NDFTFLAG, CUR_XDIM ))
       {
        itemp = getParm( fdata, NDAQSIGN, CUR_XDIM );

        if (itemp == ALT_SEQUENTIAL || itemp == ALT_SEQUENTIAL_NEG)
           {
            xAlt = 1;
           }
       }

    if (0 == (int)getParm( fdata, NDFTFLAG, CUR_YDIM ))
       {
        itemp = getParm( fdata, NDAQSIGN, CUR_YDIM );

        if (itemp == ALT_SEQUENTIAL || itemp == ALT_SEQUENTIAL_NEG)
           {
            yAlt = 1;
           }
       }

    if (*xQuadState == 1) ri2cFlag = 0;

/***/
/* If either X or Y imags are present, treat data as complex;
/* exception is sequential data.
/***/

    if (*xQuadState == 2)
       (void) setParm( fdata, FDQUADFLAG, (float)0.0, 0 );
    else if (*yQuadState == 2 && !xAlt)
       (void) setParm( fdata, FDQUADFLAG, (float)0.0, 0 );
    else
       (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );

    xyQuadState = (*xQuadState)*(*yQuadState)/quadState;
    xyQuadState = xyQuadState < 1 ? 1 : xyQuadState;

/***/
/* Extract current dimension order and axis sizes:
/***/

    dim1 = getParm( fdata, FDDIMORDER1, 0 );
    dim2 = getParm( fdata, FDDIMORDER2, 0 );
    dim3 = getParm( fdata, FDDIMORDER3, 0 );
    dim4 = getParm( fdata, FDDIMORDER4, 0 );

    xOutSize = getParm( fdata, NDSIZE, CUR_XDIM );
    yOutSize = getParm( fdata, NDSIZE, CUR_YDIM );
    zOutSize = getParm( fdata, NDSIZE, CUR_ZDIM );
    aOutSize = getParm( fdata, NDSIZE, CUR_ADIM );

/***/
/* Set file sizes and headers depending on axis to extract:
/*
/*   Nota Bene: these sizes are set up for specific and different output
/*              procedures, and so they are not consistant from case to case.
/***/

    switch( xyzFlag )
       {
        case 1:
           *xSize    = size;
           *ySize    = specnum*quadState;
           *zSize    = trueZSize;
           *aSize    = trueASize;

           xOutSize  = size;
           yOutSize  = specnum;
           maxSize64 = (NMR_INT)(*xSize)*(*ySize);

           break;

        case 2:
           *xSize    = size;
           *ySize    = specnum/xyQuadState;
           *zSize    = trueZSize;
           *aSize    = trueASize;

           xOutSize  = *ySize;
           yOutSize  = (*xSize)*xyQuadState;
           maxSize64 = (NMR_INT)2*(*xSize)*(*ySize)*(*xQuadState)*(*yQuadState);

           if (xAlt && *yQuadState == 2) yOutSize /= 2;

           if (FD_MAGNITUDE == (int)getParm( fdata, FD2DPHASE, 0 ))
              {
               quad1 = getParm( fdata, NDQUADFLAG, CUR_XDIM );
               quad2 = getParm( fdata, NDQUADFLAG, CUR_YDIM );

               (void) setParm( fdata, NDQUADFLAG, (float)quad2, CUR_XDIM );
               (void) setParm( fdata, NDQUADFLAG, (float)quad1, CUR_YDIM );
              }

           if (2 == *yQuadState)
              (void) setParm( fdata, FDQUADFLAG, (float)0.0, 0 );
           else if (yAlt == ALT_SEQUENTIAL && 2 == *xQuadState)
              (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );
           else if (yAlt == ALT_SEQUENTIAL_NEG && 2 == *xQuadState)
              (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );

           if (1 == (int)getParm( fdata, FDTRANSPOSED, 0 ))
              (void) setParm( fdata, FDTRANSPOSED, (float)0.0, 0 );
           else
              (void) setParm( fdata, FDTRANSPOSED, (float)1.0, 0 );

           (void) setParm( fdata, FDDIMORDER1, (float)dim2, 0 );
           (void) setParm( fdata, FDDIMORDER2, (float)dim1, 0 );

           break;

        case 3:
           *xSize   = size*quadState;
           *ySize   = specnum;
           *zSize   = trueZSize;
           *aSize   = trueASize;

           xOutSize = trueZSize/(*zQuadState);
           yOutSize = size;
           zOutSize = specnum*(*yQuadState);

           if (ZCRC) yOutSize *= 2;
           if (ZCCR) zOutSize /= 2;

           if (ZCCC) yOutSize *= 2;
           if (ZCCC) zOutSize /= 2;

           if (*xQuadState == 2 && *zQuadState == 2) ri2cFlag  = 1;
           if (*xQuadState == 2 && *yQuadState == 2 && *zQuadState == 1) ri2cFlag  = 1;

           if (xAlt && *yQuadState == 2) zOutSize /= 2;
 
           maxSize64 = (NMR_INT)4*(*xSize)*(*zSize)*quadState;

           if (*zQuadState == 2)
              {
               if ((error = c2riList2N( inListN, *zSize, *aSize ))) return( 1 );
               (void) setParm( fdata, FDQUADFLAG, (float)0.0, 0 );
              }
           else
              {
               if (*xQuadState == 1)
                  {
                   (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );
                  }
              }

           (void) setParm( fdata, FDDIMORDER1, (float)dim3, 0 );
           (void) setParm( fdata, FDDIMORDER2, (float)dim1, 0 );
           (void) setParm( fdata, FDDIMORDER3, (float)dim2, 0 );

           if (dim3 == 2)
              (void) setParm( fdata, FDTRANSPOSED, (float)0.0, 0 );
           else if (dim1 == 2)
              (void) setParm( fdata, FDTRANSPOSED, (float)1.0, 0 );

           break;

        case 4:
           if ((error = z2aListN( inListN, trueZSize, trueASize ))) return( 1 );

           *xSize   = size*quadState;
           *ySize   = specnum;
           *zSize   = trueASize;
           *aSize   = trueZSize;

           xOutSize = trueASize/(*aQuadState);
           yOutSize = size;
           zOutSize = specnum*(*yQuadState);
           aOutSize = trueZSize;

           if (ACRC) yOutSize *= 2;
           if (ACCR) zOutSize /= 2;

           if (ACCC) yOutSize *= 2;
           if (ACCC) zOutSize /= 2;

           if (*xQuadState == 2 && *aQuadState == 2) ri2cFlag  = 1;
           if (*xQuadState == 2 && *yQuadState == 2 && *aQuadState == 1) ri2cFlag  = 1;

           if (xAlt && *yQuadState == 2) zOutSize /= 2;

           maxSize64 = (NMR_INT)4*(*xSize)*(*zSize)*quadState;

           if (*aQuadState == 2)
              {
               if ((error = c2riList2N( inListN, *zSize, *aSize ))) return( 1 );
               (void) setParm( fdata, FDQUADFLAG, (float)0.0, 0 );
              }
           else
              {
               if (*xQuadState == 1)
                  {
                   (void) setParm( fdata, FDQUADFLAG, (float)1.0, 0 );
                  }
              }

           (void) setParm( fdata, FDDIMORDER1, (float)dim4, 0 );
           (void) setParm( fdata, FDDIMORDER2, (float)dim1, 0 );
           (void) setParm( fdata, FDDIMORDER3, (float)dim2, 0 );
           (void) setParm( fdata, FDDIMORDER4, (float)dim3, 0 );

           if (dim4 == 2)
              (void) setParm( fdata, FDTRANSPOSED, (float)0.0, 0 );
           else if (dim1 == 2)
              (void) setParm( fdata, FDTRANSPOSED, (float)1.0, 0 );

           break;

        default:
           return( 1 );
           break;
       }

/***/
/* Now that file sizes are calculated, find partition for Z and A axis.
/* Find total 1D slices to process.
/***/

    if (xyzFlag == 3 || xyzFlag == 4)
       {
        error = getPipePlanes( serverListName, cpuID, firstPlane, lastPlane, *ySize );

        if (error)
           {
            FPR( stderr, "XYZ2Pipe Error Extracting Limits.\n" );
            return( error );
           }

        finalQuad  = getQuad( fdata, FDQUADFLAG, 0 );
        maxPlane   = *ySize;

        if (*xQuadState == 1 && *yQuadState == 2)
           {
            firstFile  = 1 + (*firstPlane - 1)*2;
            lastFile   = 2 + (*lastPlane - 1)*2;
           }
        else
           {
            firstFile  = *firstPlane;
            lastFile   = *lastPlane;
           }
 
        sliceCount64  = (NMR_INT)(1 + *lastPlane - *firstPlane)*(*xSize)*(*zSize)*(*aSize);
        sliceCount64 /= finalQuad*xOutSize;
       }
    else
       {
        sliceCount64 = (NMR_INT)(1 + *lastPlane - *firstPlane)*yOutSize;
        firstFile    = *firstPlane;
        lastFile     = *lastPlane;
       }

/***/
/* Save final axis sizes.
/* Save locations of planes to process if they don't include all planes.
/***/

    (void) setParm( fdata, NDSIZE, (float)xOutSize, CUR_XDIM );
    (void) setParm( fdata, NDSIZE, (float)yOutSize, CUR_YDIM );
    (void) setParm( fdata, NDSIZE, (float)zOutSize, CUR_ZDIM );
    (void) setParm( fdata, NDSIZE, (float)aOutSize, CUR_ADIM );

    if (*firstPlane == 1 && *lastPlane == maxPlane)
       {
        (void) setParm( fdata, FDFIRSTPLANE, (float)0, 0 );
        (void) setParm( fdata, FDLASTPLANE,  (float)0, 0 );

        (void) setFDATASliceCount64( fdata, (NMR_INT)0 );
       }
     else
       {
        (void) setParm( fdata, FDFIRSTPLANE, (float)firstFile,   0 );
        (void) setParm( fdata, FDLASTPLANE,  (float)lastFile,    0 );

        (void) setFDATASliceCount64( fdata, sliceCount64 );
       }

/***/
/* If no memory size is specified, set a reasonable one:
/***/

    maxSize64 *= sizeof(float);

    if (!*megabytes)
       {
        *megabytes = 1 + maxSize64/(1024*1024);
        if (*megabytes < 2) *megabytes = 2;
       }

/***/
/* Adjust header to reflect global scaling of all planes:
/***/

    (void) setParm( fdata, FDMAX,       maxVal, 0 );
    (void) setParm( fdata, FDMIN,       minVal, 0 );
    (void) setParm( fdata, FDDISPMAX,   maxVal, 0 );
    (void) setParm( fdata, FDDISPMIN,   minVal, 0 );

    (void) setParm( fdata, FDSCALEFLAG, (float)masterScaleFlag, 0 );

/***/
/* Open the file before size check.
/* If possible, check consistancy of predicted and available output size:
/* Write the header to the output stream.
/***/

    if (*outName)
       {
        ETEST( 12, dataOpen( outName, outUnitPtr, FB_CREATE ));
       }
    else
       {
        *outUnitPtr = FB_STDOUT;
       }

    if (*outName && checkFlag)
       {
        switch( testNMRout( outName, fdata, 0, 1 ))
           {
            case FILE_SIZE_FAIL:
               FPR( stderr, "\n" );
               FPR( stderr, "XYZ2Pipe Warning:" );
               FPR( stderr, " can't check size for file" );
               FPR( stderr, " %s\n\n", outName );
               break;
            case  FILE_SIZE_TOO_LARGE:
               FPR( stderr, "\n" );
               FPR( stderr, "XYZ2Pipe Error:" );
               FPR( stderr, " insufficient space for file" );
               FPR( stderr, " %s\n\n", outName );
               return( 1 );
               break;
            default:
               break;
           }
       }

shutdown:

    return( error );
}

/***/
/* getPipePlanes: get locations of first and last plane to process.
/***/

static int getPipePlanes( serverListName, cpuID, firstPlane, lastPlane, planeCount )

    int  *firstPlane, *lastPlane, planeCount, cpuID;
    char *serverListName;
{
    Server serverList[MAXSOCK];
    int    i, serverCount, serverLoc, error;

/***/
/* If the server list exists, use it to decide partition:
/***/

    if (*serverListName)
       {
        error = rdServers( serverListName, serverList, MAXSOCK, &serverCount );

        if (error)
           {
            FPR( stderr, "XYX2Pipe Error reading server list.\n" );
            return( error );
           }

        if ((error = setPartition( serverList, serverCount, planeCount )))
           {
            FPR( stderr, "XYX2Pipe Error setting partitions.\n" );
            return( error );
           }

        error = getServerLoc( serverList, 
                              serverCount,
                              (char *)NULL, 
                              cpuID,
                              &serverLoc, 
                              debugFlag );

        if (error)
           {
            FPR( stderr, "XYX2Pipe Error finding server entry.\n" );
            return( error );
           }

        error = getServerRange( serverList, serverLoc, firstPlane, lastPlane );

        if (error)
           {
            FPR( stderr, "XYX2Pipe Error finding server range.\n" );
            return( error );
           }

        if (debugFlag)
           {
            (void) showServers( serverList, serverCount );
           }

        (void) freeServers( serverList, serverCount );
       }

/***/
/* Clip and adjust plane limits:
/***/

    if (!*firstPlane) *firstPlane = 1;
    if (!*lastPlane)  *lastPlane  = planeCount;

    if (*firstPlane < 1) *firstPlane = 1;
    if (*firstPlane > planeCount) *firstPlane = planeCount;

    if (*lastPlane < 1) *lastPlane = 1;
    if (*lastPlane > planeCount) *lastPlane = planeCount;

     if (*firstPlane > *lastPlane)
        {
         i = *firstPlane; *firstPlane = *lastPlane; *lastPlane = i;
        }

/***/
/* Show status:
/***/

    if (verbose)
       {
        FPR( stderr,
             "XYZ2Pipe Partition: Plane %d to %d of %d\n", 
             *firstPlane, *lastPlane, planeCount );
       }

    return( 0 );
}

/***/
/* sendNullFD: writes null header to indicate error.
/***/

static int sendNullFD( outName, fdata )

   char  *outName;
   float fdata[FDATASIZE];
{
    int i;

    (void) setHdrNull( fdata );

    if (!*outName)
       {
        FPR( stderr, "XYZ2Pipe Aborting with null header.\n" );
        (void) dataWrite( FB_STDOUT, fdata, sizeof(float)*FDATASIZE );
       }

    return( 0 );
}

/***/
/* c2riList2N: adjust R/I alternating mixed Z/A Axis file list.
/***/

static int c2riList2N( nameList, zSize, aSize )

   struct ZListInfo **nameList;
   int    zSize, aSize;
{
    while( aSize-- )
       {
        if (c2riListN( nameList, zSize )) return( 1 );
        nameList += zSize;
       }

    return( 0 );
}

/***/
/* c2riListN: adjust R/I alternating file list to continuous R, then I.
/***/

static int c2riListN( nameList, nameCount )

   struct ZListInfo **nameList;
   int    nameCount;
{
    struct ZListInfo **tempList;
    int  i;

    if (!(tempList = (struct ZListInfo **) voidAlloc( "c2ri", sizeof(struct ZListInfo *)*nameCount )))
       {
        return( 1 );
       }

    for( i = 0; i < nameCount/2; i++ )
       {
        tempList[i]               = nameList[2*i];
        tempList[i + nameCount/2] = nameList[2*i + 1];
       }

    for( i = 0; i < nameCount; i++ ) nameList[i] = tempList[i];

    (void) deAlloc( "c2ri", tempList, sizeof(struct ZListInfo *)*nameCount );

    return( 0 ); 
}

/***/
/* z2aListN: rearrange 2D name list to swap Z and A dimensions.
/***/

static int z2aListN( inListN, zSize, aSize )

   struct ZListInfo **inListN;
   int    zSize, aSize;
{
    struct ZListInfo **tempList, **src, **dest; 
    int    ip, iz, ia;

    if (!(tempList = (struct ZListInfo **) voidAlloc( "z2a", sizeof(struct ZListInfo *)*zSize*aSize )))
       {
        return( 1 );
       }

    for( ip = 0; ip < zSize*aSize; ip++ ) tempList[ip] = inListN[ip];

    dest = inListN;

    for( iz = 0; iz < zSize; iz++ )
       {
        src = tempList + iz;

        for( ia = 0; ia < aSize; ia++ )
           {
            *dest++ = *src;
            src    += zSize;
           }
       }

    (void) deAlloc( "z2a", tempList, sizeof(struct ZListInfo *)*zSize*aSize );

    return( 0 );
}

/***/
/* xyzHead: parse for user-specified header parameters.
/***/

static int xyzHead( argc, argv, fdata )

   int   argc;
   char  **argv;
   float fdata[FDATASIZE];
{
    int   itemp, loc1, loc2;
    float rtemp;

    if ((loc1 = flagLoc( argc, argv, "-fdata" )))
       {
        loc2 = nextFlag( argc, argv, loc1 );

        while( loc1 < loc2 - 1 )
           {
            itemp = atoi( getNthArg( argc, argv, loc1+1 )) - 1;
            rtemp = ATOF( getNthArg( argc, argv, loc1+2 ));

            (void) setParm( fdata, itemp, rtemp, 0 );

            loc1 += 2;
           }
       }

    return( 0 );
}

/***/
/* getparams: extract command arguments for xyz2pipe.
/***/

static int getparams( argc, argv,
                      inTemplate, outName, serverListName, cpuID,
                      firstPlane, lastPlane, maxPlanesIn, megabytes, xyzFlag,
                      checkFlag, wrFlag, hdrFlag, parFlag )

   int  argc, *xyzFlag, *megabytes, *firstPlane, *lastPlane, *maxPlanesIn,
        *checkFlag, *wrFlag, *hdrFlag, *parFlag, *cpuID;

   char **argv, *inTemplate, *outName, *serverListName;
{

/***/
/* Install defaults:
/***/

    verbose         = 0;
    vInc            = 16;

    *outName        = '\0';
    *serverListName = '\0';

    *firstPlane  = 0;
    *lastPlane   = 0;
    *xyzFlag     = 1;
    *megabytes   = 0;
    *maxPlanesIn = 0;
    *checkFlag   = 1;
    *wrFlag      = 1;
    *hdrFlag     = 0;
    *parFlag     = 0;
    *cpuID       = 0;

/***/
/* Print a banner:
/***/

    if (argc == 1 || flagLoc( argc, argv, "-help" ))
       {
        FPR_VERSION;
        FPR( stderr, "Output NMRPipe 3D-4D File Series to an NMRPipe Pipeline:\n" );
        FPR( stderr, " %s -in inTemplate | nmrPipe ...\n", argv[0] );
        FPR( stderr, "Organization of Output Stream:\n" );
        FPR( stderr, " -x                Output X-Vectors XYZA (Default).\n" );
        FPR( stderr, " -y                Output Y-Vectors YXZA.\n" );
        FPR( stderr, " -z                Output Z-Vectors ZXYA.\n" );
        FPR( stderr, " -a                Output A-Vectors AXYZ.\n" );
        FPR( stderr, "Other Options:\n" );
        FPR( stderr, " -out   outName    Output Name.\n" );
        FPR( stderr, " -fdata loc val... Adjust File Header of Output.\n" );
        FPR( stderr, " -ri2c             Interleave Real/Imag on Output.\n" );
        FPR( stderr, " -nofs             No File Size Check.\n" );
        FPR( stderr, " -noWr             Write Header Only, No Data.\n" );
        FPR( stderr, " -hdr              Check Headers.\n" );
        FPR( stderr, "Suggested Memory Buffer Size:\n" );
        FPR( stderr, " -maxPlanes mp [0] Max Input Planes (Use Zero for Auto, -1 for All).\n" );
        FPR( stderr, " -mem       mb [0] Buffer Size, MB (Use Zero for Auto).\n" );
        FPR( stderr, "Parallel Processing:\n" );
        FPR( stderr, " -par   sListName  [None]  Server List File Name.\n" );
        FPR( stderr, " -cpu   cpuNumber  [0]     CPU Number.\n" );
        FPR( stderr, " -first firstPlane [1]     First Z-Plane to Output.\n");
        FPR( stderr, " -last  lastPlane  [ZSIZE] Last Z-Plane to Output.\n");
        FPR( stderr, "Other:\n" );
        FPR( stderr, " -help                     Prints This Text.\n" );
        FPR( stderr, " -verb                     Verbose Mode On.\n" );
        FPR( stderr, " -noverb                   Verbose Mode Off (Default).\n" );
        FPR( stderr, " -inc   vInc       [16]    Verbose Message Increment.\n" );
        FPR( stderr, "Input Template Examples:\n" );
        FPR( stderr, " test.dat             3D or 4D Stream                (Single File).\n" );
        FPR( stderr, " test%%03d.ft3         3D Z Format test001.ft3 ...    (2D Series).\n" );
        FPR( stderr, " test%%02d%%03d.ft4     4D AZ Format test01001.ft4 ... (2D Series).\n" );
        FPR( stderr, " test%%03d.ft4         4D Cube Format test001.ft4 ... (3D Series)\n" );
        return( 1 );
       }

/***/
/* Extract the parameters:
/***/

    if (!flagLoc( argc, argv, "-in" )) return( 1 );

    (void) strArgD( argc, argv, "-in",  inTemplate );
    (void) strArgD( argc, argv, "-out", outName );
    (void) strArgD( argc, argv, "-par", serverListName );
    (void) intArgD( argc, argv, "-cpu", cpuID );

    (void) intArgD( argc, argv, "-maxPlanes", maxPlanesIn );
    (void) intArgD( argc, argv, "-mem",       megabytes );

    (void) intArgD( argc, argv, "-dbg", &debugFlag );

    if (flagLoc( argc, argv, "-ri2c" )) ri2cFlag = 1;

    if (flagLoc( argc, argv, "-x" )) *xyzFlag = 1;
    if (flagLoc( argc, argv, "-y" )) *xyzFlag = 2;
    if (flagLoc( argc, argv, "-z" )) *xyzFlag = 3;
    if (flagLoc( argc, argv, "-a" )) *xyzFlag = 4;

    if (flagLoc( argc, argv, "-wr" ))   *wrFlag      = 1;
    if (flagLoc( argc, argv, "-noWr" )) *wrFlag      = 0;

    if (flagLoc( argc, argv, "-hdr" ))  *hdrFlag     = 1;
    if (flagLoc( argc, argv, "-nofs" )) *checkFlag   = 0;

    if (flagLoc( argc, argv, "-verb" ))   verbose    = 1;
    if (flagLoc( argc, argv, "-noverb" )) verbose    = 0;

    (void) intArgD( argc, argv,  "-inc",  &vInc );
 
    (void) intArgD( argc, argv, "-first", firstPlane );
    (void) intArgD( argc, argv, "-last",  lastPlane );

/***/
/* Test for possiblity of any kind of parallel processing:
/***/

    if (*firstPlane || *lastPlane || *serverListName) *parFlag = 1;

    return( 0 );
}

/***/
/* Bottom.
/***/
