#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Determine Shifts Which Align VnmrJ-Format 1D Array Data:"
   echo "Regions Specified via Command Line (Default):"
   echo " -x1      x1List  [0%]      List of Region Start Locations."
   echo " -xn      xnList  [100%]    Corresponding List of Region End Locations."
   echo " -refLoc  rLoc    [Max]     Loc of Reference Spectra in Array, or Keyword Max."
   echo " -exclLoc locList [None]    Array Locs to Skip in -refLoc Max Determination."
   echo "Regions Specified via Two-Column File:"
   echo " -in     inName   [None]    Optional Region Limits Input Name."
   echo "Other Options:"
   echo " -sMin   sMin     [-20%]    Minimum Shift to Try, Hz, Pts, PPM, %"
   echo " -sMax   sMax     [20%]     Maximum Shift to Try, Hz, Pts, PPM, %"
   echo " -xUnits xU       [Hz]      Default Region Units, Hz, Pts, PPM, %, or None."
   echo " -sUnits sU       [Hz]      Units for Reporting Shift, Hz, Pts, PPM, %."
   echo " -cosW   cW       [10%]     Cosine Roll-Off Width, Hz, Pts, PPM, %."
   echo " -save                      Save All Results."
   echo " -nosave                    Save Only Last Region Results (Default)."
   echo " -progress                  Graphical Progress Bar ON (Default)."
   echo " -noprogress                No Graphical Progress Bar."
   echo " -verb                      Verbose Mode ON."
   echo " -noverb                    Verbose Mode OFF (Default)."
   echo "Options for vj2pipe.com (Should be Last on Command Line):"
   echo " -vj2pipe vArgs   [None]    See vj2pipe.com -help for More."
   echo " -cd      cdDir   [align]   Output Directory (Also used by vj2pipe.com)."
   echo " -vjfid   vjfid   [fid]     VnmrJ Binary fid Input."
   echo " -procpar pName   [procpar] VnmrJ procpar Parameter File."
   echo "Notes:"
   echo " 1. Input will be converted and processed via usual options"
   echo "    in the vj2pipe.com scheme."
   echo " 2. Locations for -refLoc and -exclLoc start at 1."
   echo " 3. Shift Range sMin to sMax should be smaller than region." 
   echo " 4. Cosine Roll-Off width cW should be smaller than region." 
   echo " 5. If the -in option is given, -x1 and -xn are ignored."
   echo " 6. Input for the -in option must contain two columns of values;"
   echo "    blank lines, # and // comments are allowed."
   echo " 7. Shifts are reported as left shifts for positive values."
   echo " 8. Shifts reported for empty regions will not be meaningful."
   echo " 9. This version only applies shifts in integer numbers of points,"
   echo "    relative to the processed spectrum."
   exit 0
endif

set alignDir    = (`getArgD $argv -cd     align`) 
set regTextName = (`getArgD $argv -in     None`) 

set refLoc   = (`getArgD $argv -refLoc Max`)
set sMin     = (`getArgD $argv -sMin  -20%`)
set sMax     = (`getArgD $argv -sMax   20%`)
set cosW     = (`getArgD $argv -cosW   10%`)
set xUnits   = (`getArgD $argv -xUnits Hz`)
set sUnits   = (`getArgD $argv -sUnits Hz`)

set x1List   = (`getListArgD $argv -x1 -- 0%`)
set xnList   = (`getListArgD $argv -xn -- 100%`)
set exclList = (`getListArgD $argv -exclLoc -- None`)

set argList  = (`getListArgDN $argv -vj2pipe -- None`)

set origDir    = (`pwd`)
set zeroHzFlag = 1
set hdrFlag    = 1
set saveFlag   = 0
set verbFlag   = 0

set ft1Name    = spec.ft1
set roiName    = roi.ft1
set zfName     = zf.ft1
set shiftName  = shift.ft1
set refName    = ref.ft1

if (`flagLoc $argv -zeroHz`) then
   set zeroHzFlag = 1
endif

if (`flagLoc $argv -nozeroHz`) then
   set zeroHzFlag = 0
endif

if (`flagLoc $argv -hdr`) then
   set hdrFlag = 1
endif

if (`flagLoc $argv -nohdr`) then
   set hdrFlag = 0
endif

if (`flagLoc $argv -save`) then
   set saveFlag = 1
endif

if (`flagLoc $argv -nosave`) then
   set saveFlag = 0
endif

if (`flagLoc $argv -verb`) then
   set verbFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set verbFlag = 0
endif

if (`string csame $xUnits None`) then
   set xUnits = ""
endif

if (`string csame $regTextName None` == 0) then
   if (!(-e $regTextName)) then
      nmrPrintf -stderr "VJAlign Error Finding Input $regTextName\n"
      exit 1
   endif

   set x1List = (`getCols.tcl -in $regTextName -col 0 -out stdout -skip`)
   set xnList = (`getCols.tcl -in $regTextName -col 1 -out stdout -skip`)
endif

if ($#x1List != $#xnList) then
   nmrPrintf -stderr "VJAlign Error: -x1/-xn Coordinate List Mismatch.\n"
   exit 1
endif

set thisPID      = $$
set progressFlag = 1
set sendName     = nmrProgress.tcl

if (`flagLoc $argv -progress`) then
   set progressFlag = 1
   set sendName     = nmrProgress.tcl
endif

if (`flagLoc $argv -noprogress`) then
   set progressFlag = 0
   set sendName     = None
endif

#
#-------------------------------------------------------------------------------

if (!(-d $alignDir)) then
   mkdir $alignDir
endif

if (`string csame $argList None`) then
   set argList = ""
endif

vj2pipe.com -cd $alignDir $argList >& $alignDir/align.out

cd $alignDir

foreach d (split1D)
   if (-e $d) then
      /bin/rm -rf $d 
   endif

   mkdir $d
end

if ($progressFlag != 0) then
   if ($verbFlag != 0) then
      echo "Stopping any existing progress bar displays ..."
   endif

   nmrProgress.com -stop -noprogress -noverb 

   if ($verbFlag != 0) then
      echo "Starting new progress bar display ..."
   endif

   (nmrProgress.tcl -silent &)

   sleep 3

   if ($verbFlag != 0) then
      echo "Done starting new progress bar display."
   endif
endif

if ($zeroHzFlag != 0) then
   sethdr $ft1Name -xORIG 0.0
endif

if (!(-e $ft1Name)) then
   nmrPrintf -stderr "VJAlign Error Finding Spectrum $alignDir/$ft1Name\n"
   exit 1
endif

split2D.com -in $ft1Name -outDir split1D -outName test%05d.ft1 -title >& split.out

set fList = (split1D/test*.ft1)
set nReg  = $#x1List
set nSpec = $#fList

set keepList = ""
set j = 1

while ($j <= $nSpec)
   set skipFlag = 0

   foreach ii ($exclList)
      if (`string csame $ii $j`) then
         set skipFlag = 1
         break
      endif
   end

   if ($skipFlag == 0) then
      set keepList = ($keepList $j)
   endif

   @ j++
end

set keepList = ($keepList 1)

if ($verbFlag != 0) then
   echo "VJAlign: $nReg Regions $nSpec Spectra"
else
   if ($hdrFlag != 0) then
      echo "VARS   REGION SPECTRUM REF X1 X1_UNITS XN XN_UNITS SHIFT_PTS SHIFT SHIFT_UNITS"
      echo "FORMAT %4d %4d %4d %.3f %s %.3f %s %4d %11.6f %s"
      echo ""
   endif
endif

set nn = (`IMATH "$nReg*$nSpec"`)

if ($progressFlag != 0) then
   set thisMsg = "nmrProgress -loc 0 -n $nn -pct 0.0 -fromID 1 -pid $thisPID -title CRAFT Alignment -axis None"
   nmrSend -norecv -send $sendName -msg "$thisMsg"
endif

set n = 1
set i = 1
set j = 1

while ($i <= $nReg)
   if ($verbFlag != 0) then
      echo " Region: $i Limits: $x1List[$i]$xUnits $xnList[$i]$xUnits"
   endif

   if ($saveFlag == 0) then
      set iStr = ""
   else
      set iStr = (`nmrPrintf _%05d $i`)
   endif

   foreach d (ext1D$iStr zf1D$iStr shift1D$iStr)
      if (-e $d) then
         /bin/rm -rf $d
      endif

      mkdir $d
   end

   if (`string csame $refLoc Max`) then
      set thisRefLoc = $keepList[1] 
   else
      set thisRefLoc = $refLoc
   endif

   set maxRMS = 0.0
   set j      = 1

   while ($j <= $nSpec)
      set skipFlag = 0

      foreach ii ($exclList)
         if (`string csame $ii $j`) then
            set skipFlag = 1
            break
         endif
      end

      set inName     = (`nmrPrintf split1D/test%05d.ft1 $j`)

      set roiName    = (`nmrPrintf ext1D$iStr/test%05d.ft1   $j`)
      set zfName     = (`nmrPrintf zf1D$iStr/test%05d.ft1    $j`)
      set shiftName  = (`nmrPrintf shift1D$iStr/test%05d.ft1 $j`)

      readROI -in $inName -ndim 1 -x X_AXIS $x1List[$i]$xUnits $xnList[$i]$xUnits -out $roiName -ov -noverb -silent

      if (!(-e $roiName)) then
         nmrPrintf -stderr "VJAlign Error Finding Region Extract $alignDir/$roiName\n"
         exit 1
      endif

      nmrPipe -in $roiName -fn TM -cos -t1 $cosW -t2 $cosW | nmrPipe -fn ZF | nmrPipe -fn CS -rs 25% -out $zfName -ov

      if (!(-e $zfName)) then
         nmrPrintf -stderr "VJAlign Error Finding Processed Extract $alignDir/$zfName\n"
         exit 1
      endif

      if ($j == 1) then
         if (`string isint $sMin` == 0) then
            set sW = (`spec2pntW $roiName X_AXIS $sMin`)
            set sMinPts = (`IMATH $sW`)
         else
            set sMinPts = $sMin
         endif

         if (`string isint $sMax` == 0) then
            set sW = (`spec2pntW $roiName X_AXIS $sMax`)
            set sMaxPts = (`IMATH $sW`)
         else
            set sMaxPts = $sMax
         endif

         if ($verbFlag != 0) then
            echo " Shift: $sMin to $sMax / $sMinPts Pts to $sMaxPts Pts"
         endif
      endif

      if ($skipFlag == 0 && `string csame $refLoc Max`) then
         set thisRMS = (`readROI -ndim 1 -in $zfName -out None -x X_AXIS 0% 100% -silent -vec vRMS`)

         if (`IMATH "$thisRMS > $maxRMS"`) then
            set maxRMS = $thisRMS
            set thisRefLoc = $j
         endif
      endif

      @ j++
   end

   set j = 1

   while ($j <= $nSpec)
      set inName     = (`nmrPrintf split1D/test%05d.ft1 $j`)

      set roiName    = (`nmrPrintf ext1D$iStr/test%05d.ft1   $j`)
      set zfName     = (`nmrPrintf zf1D$iStr/test%05d.ft1    $j`)
      set shiftName  = (`nmrPrintf shift1D$iStr/test%05d.ft1 $j`)
      set refName    = (`nmrPrintf zf1D$iStr/test%05d.ft1    $thisRefLoc`)

      if (!(-e $refName)) then
         nmrPrintf -stderr "VJAlign Error Finding Reference $thisRefLoc file $refName\n"
         exit 1
      endif

      if ($progressFlag != 0) then
         set progPct = (`MATH "100.0*div($n,$nn)"`)
         set thisMsg = "nmrProgress -loc $i -pct `nmrPrintf %.1f $progPct` -n $nn -fromID 1 -pid $thisPID -title CRAFT Alignment -axis None"
         nmrSend -norecv -send $sendName -msg "$thisMsg"
      endif

      @ n++

      if ($verbFlag == 0) then
         set aStr = (`autoShift.com -in $zfName -ref $refName -out $shiftName -x1 0% -xn 100% -sMin $sMinPts -sMax $sMaxPts -noverb` 0)
         set sStr = (`pnt2specW $roiName X_AXIS $aStr[1] $sUnits` 0)
         nmrPrintf " %4s %4s %4s %9s %s %9s %s  %5s %s %10s %s\n" $i $j $thisRefLoc $x1List[$i] $xUnits $xnList[$i] $xUnits $aStr[1] Pts $sStr[1] $sUnits
      else
         set aStr = (`autoShift.com -in $zfName -ref $refName -out $shiftName -x1 0% -xn 100% -sMin $sMinPts -sMax $sMaxPts`)
         echo " $aStr"
      endif

      set sStr = (`pnt2specW $roiName X_AXIS $aStr[1] $sUnits` 0)

      if (!(-e $shiftName)) then
         nmrPrintf -stderr "VJAlign Error Finding Shifted Data $alignDir/$shiftName\n"
         exit 1
      endif

      set title = (`getParm -in $inName -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)

      if ($saveFlag) then
         sethdr $zfName    -title orig-$i-$title
         sethdr $shiftName -title shift-$i-$title
      else
         sethdr $zfName    -title orig-$title
         sethdr $shiftName -title shift-$title
      endif

      @ j++      
   end

   if ($verbFlag != 0) then
      echo ""
   endif

   @ i++
end

if ($progressFlag != 0) then
   set thisMsg = "nmrProgress -loc $nn -n $nn -pct 100.0 -fromID 1 -pid $thisPID -title CRAFT Alignment -axis None"

   nmrSend -norecv -send $sendName -msg "$thisMsg"
endif

exit 0
